# CData
#
# READ var [CData file ...] [APPEND] [DEFAULT_FILE filename] [ALL]
#
# GENERATE [VERBOSE] [WORKING_DIRECTORY dir (for ADD_TEST/TARGET)]
# 	EXECUTE command
# 	TARGET name [DEPENDENTS dep ...] [ALWAYS_RUN]
# 	ADD_TEST name
# 	
# DOWNLOAD [VERBOSE]
#


include(Map)
include(CMakeParseArguments)

function(CData)
	set(options APPEND ALL VERBOSE ALWAYS_RUN)
	set(oneValueArgs URL DIR GENERATE ID TARGET WORKING_DIRECTORY ADD_TEST)
	set(multiValueArgs READ DOWNLOAD EXECUTE DEPENDENTS)
	cmake_parse_arguments(CDATA "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN})
	
	if (NOT CDATA_ID AND CDATA_DEFAULT_ID)
		set(CDATA_ID ${CDATA_DEFAULT_ID})
	endif ()
	if (CDATA_ALL)
		set(CDATA_ID "")
	endif ()
	
	set(read_result "")
	if (CDATA_READ)
		list(GET CDATA_READ 0 ret)
		if (CDATA_APPEND)
			set(read_result ${${ret}})
		endif ()
		list(REMOVE_AT CDATA_READ 0)
		
		if (NOT CDATA_READ)
			if (CDATA_DEFAULT_FILE)
				list(APPEND CDATA_READ ${CDATA_DEFAULT_FILE})
			else ()
				list(APPEND CDATA_READ "CMakeData.txt")
			endif ()
		endif ()
		
		foreach(list ${CDATA_READ})
			if (EXISTS ${list})
				file(READ ${list} list_contents)
				string(REGEX REPLACE ";" "\\\\;" list_contents "${list_contents}")
				string(REGEX REPLACE "\n" ";" list_contents "${list_contents}")
				
				foreach(line ${list_contents})
					if ("${line}" MATCHES "^([+-=]) (.+)$")
						if (CMAKE_MATCH_1 STREQUAL "=")
							if (id STREQUAL CDATA_ID)
								break()
							else ()
								set(id "${CMAKE_MATCH_2}")
								if (id MATCHES "[ \t]")
									string(REGEX REPLACE "[ \t]+.*$" "" "${id}" stripped_id)
								else ()
									set(stripped_id "${id}")
								endif ()
							endif ()
						elseif (CMAKE_MATCH_1 STREQUAL "-")
							map(REMOVE read_result ${CMAKE_MATCH_2})
						elseif ("${line}" MATCHES "^\\+ (.+) (.+) (.+)$")
							map(SET read_result ${CMAKE_MATCH_1} "${stripped_id}/${CMAKE_MATCH_2} ${CMAKE_MATCH_3}")
						else ()
							message(FATAL_ERROR "Invalid data file line: ${line}")
						endif ()
					endif ()
				endforeach(line)
			endif (EXISTS ${list})
		endforeach(list)
		
	    set(${ret} ${read_result} PARENT_SCOPE)
    endif ()
    
    if (CDATA_DOWNLOAD OR CDATA_GENERATE)
		if (NOT CDATA_URL)
			set(CDATA_URL ${CDATA_DEFAULT_URL})
		endif ()
		if (NOT CDATA_URL)
			message(FATAL_ERROR "No URL given to download data files")
		endif ()
		if (NOT CDATA_DIR)
			set(CDATA_DIR ${CMAKE_CURRENT_BINARY_DIR})
		endif ()
    endif ()
    
	if (CDATA_GENERATE)
		file(WRITE ${CDATA_GENERATE} "# CMake file generated to download any required data files\n\n")
		foreach(map ${CDATA_DOWNLOAD} read_result)
			foreach(path ${${map}})
				if (path MATCHES "^([^=]+)=(.+) (.+)$")
					if (CDATA_VERBOSE)
						file(APPEND ${CDATA_GENERATE} "message(STATUS \"Downloading ${CDATA_URL}${CMAKE_MATCH_2} => ${CDATA_DIR}/${CMAKE_MATCH_1}\")\n")
					endif ()
					file(APPEND ${CDATA_GENERATE} "file(DOWNLOAD \"${CDATA_URL}${CMAKE_MATCH_2}\" \"${CDATA_DIR}/${CMAKE_MATCH_1}\" SHOW_PROGRESS EXPECTED_HASH SHA1=${CMAKE_MATCH_3})\n")
				else ()
					message(FATAL_ERROR "Malformed file map")
				endif()
			endforeach(path)
		endforeach(map)
		if (CDATA_EXECUTE)
			file(APPEND ${CDATA_GENERATE} "execute_process(COMMAND ${CDATA_EXECUTE} RESULT_VARIABLE res)\n")
			file(APPEND ${CDATA_GENERATE} "if (\${res})\n")
			file(APPEND ${CDATA_GENERATE} "\tmessage(FATAL_ERROR \"execution failed: \${res}\")\n")
			file(APPEND ${CDATA_GENERATE} "else ()\n")
			file(APPEND ${CDATA_GENERATE} "\tmessage(\"execution succeeded: \${res}\")\n")
			file(APPEND ${CDATA_GENERATE} "endif ()\n")
		endif ()
		
		if (NOT CDATA_WORKING_DIRECTORY)
			get_filename_component(CDATA_WORKING_DIRECTORY ${CDATA_GENERATE} DIRECTORY)
		endif ()
		if (CDATA_TARGET)
			if (CDATA_ALWAYS_RUN)
				add_custom_target(${CDATA_TARGET} ALL COMMAND "${CMAKE_COMMAND}" ARGS -P ${CDATA_GENERATE} WORKING_DIRECTORY ${CDATA_WORKING_DIRECTORY})
			else ()
				add_custom_target(${CDATA_TARGET} COMMAND "${CMAKE_COMMAND}" ARGS -P ${CDATA_GENERATE} WORKING_DIRECTORY ${CDATA_WORKING_DIRECTORY})
			endif ()
			if (CDATA_DEPENDENTS)
				foreach(dependent ${CDATA_DEPENDENTS})
					add_dependencies(${dependent} ${CDATA_TARGET})
				endforeach(dependent)
			endif ()
		endif ()
		if (CDATA_ADD_TEST)
			add_test(NAME ${CDATA_ADD_TEST} COMMAND "${CMAKE_COMMAND}" ARGS -P ${CDATA_GENERATE} WORKING_DIRECTORY ${CDATA_WORKING_DIRECTORY})
		endif ()
	elseif (CDATA_DOWNLOAD)
		foreach(map ${CDATA_DOWNLOAD} read_result)
			foreach(path ${${map}})
				if (path MATCHES "^([^=]+)=(.+) (.+)$")
					if (CDATA_VERBOSE)
						message(STATUS "Downloading ${CDATA_URL}${CMAKE_MATCH_2} => ${CDATA_DIR}/${CMAKE_MATCH_1}")
					endif ()
					file(DOWNLOAD "${CDATA_URL}${CMAKE_MATCH_2}" "${CDATA_DIR}/${CMAKE_MATCH_1}" SHOW_PROGRESS EXPECTED_HASH SHA1=${CMAKE_MATCH_3})
				else ()
					message(FATAL_ERROR "Malformed file map")
				endif()
			endforeach(path)
		endforeach(map)
    endif ()
endfunction(CData)
