#!/bin/csh -f

if ( $#argv == 0) then
    echo " "
    echo "get_ancillary_files.csh L1Afilename"
    exit
endif

set file=$1

echo file: $file

# STARTTIME
set starttime=`h5_dumpattr.csh $file "Start Time" | cut -d "=" -f2`
echo "starttime: " $starttime

set yrdoy=`echo $starttime | cut -c 1-7`
set hr=`echo $starttime | cut -c 8-9`
echo hr: $hr

@ hr6 = $hr / 6

if ( $hr6 == 0) then
    set hr6 = 00
endif

if ( $hr6 == 1) then
    set hr6 = 06
endif

if ( $hr6 == 2) then
    set hr6 = 12
endif

if ( $hr6 == 3) then
    set hr6 = 18
endif

set first06=${yrdoy}${hr6}
set hr06_1 = $hr6


# ENDTIME
set endtime=`h5_dumpattr.csh $file "End Time" | cut -d "=" -f2`
echo "endtime:   " $endtime

set yrdoy=`echo $endtime | cut -c 1-7`
set hr=`echo $endtime | cut -c 8-9`
echo hr: $hr

# Get YYYYMMDD
set mndy=`doy2mndy.csh $yrdoy`
set dte_yyyymmdd=`echo $mndy | cut -c 1-4``echo $mndy | cut -c 6-7``echo $mndy | cut -c 9-10`


@ hr6 = $hr / 6

if ( $hr6 == 0) then
    set hr6 = 06
endif

if ( $hr6 == 1) then
    set hr6 = 12
endif

if ( $hr6 == 2) then
    set hr6 = 18
endif

if ( $hr6 == 3) then
    set hr6 = 00

    set sec=`date -u -d $dte_yyyymmdd +%s`
    @ sec2 = $sec + 86400

    set yrdoy=`date -u -d "1970-01-01 $sec2 seconds" +%Y%j`
endif

set last06=${yrdoy}${hr6}
set hr06_2 = $hr6

# Test whether the ancillary files exist
set yfile1=y`doy2mndy.csh $first06 | sed -e 's/-//g'`${hr06_1}.h5
set yfile2=y`doy2mndy.csh $last06 | sed -e 's/-//g'`${hr06_2}.h5
if( -e $yfile1  && -e $yfile2) then 
    exit
endif


# Get bracketing OISST files
set day = `echo $dte_yyyymmdd | cut -c 1-8`
set yr = `echo $day | cut -c 1-4`

echo Retrieving oisst/N${yrdoy}00_SST_Reynolds_168h
ncftpget samoa.gsfc.nasa.gov . "/pub/joel/oisst/N${yrdoy}00_SST_Reynolds_168h" >& /dev/null
set stat=$status     

set oisstfile=N${yrdoy}00_SST_Reynolds_168h

# Check backward for oisst file
@ dayfrombeg = 0
while ($stat != 0) 
    set sec=`date -u -d $day +%s`
    @ sec1 = $sec - 86400
    set day=`date -u -d "1970-01-01 $sec1 seconds" +%Y%m%d`
    set yrdoy=`date -u -d "1970-01-01 $sec1 seconds" +%Y%j`
#echo "/pub/joel/oisst/N${yrdoy}00_SST_Reynolds_168h"
    ncftpget samoa.gsfc.nasa.gov . "/pub/joel/oisst/N${yrdoy}00_SST_Reynolds_168h" >& /dev/null
    set stat=$status     
    set oisstfile=N${yrdoy}00_SST_Reynolds_168h
    @ dayfrombeg++
end
set beg_sstfile=$oisstfile

# Check forward for next oisst file
set stat=-1
while ($stat != 0) 
    set sec=`date -u -d $day +%s`
    @ sec2 = $sec + 86400
    set day=`date -u -d "1970-01-01 $sec2 seconds" +%Y%m%d`
    set yrdoy=`date -u -d "1970-01-01 $sec2 seconds" +%Y%j`
#echo "/pub/joel/oisst/N${yrdoy}00_SST_Reynolds_168h"
    ncftpget samoa.gsfc.nasa.gov . "/pub/joel/oisst/N${yrdoy}00_SST_Reynolds_168h" >& /dev/null
    set stat=$status     
    set oisstfile=N${yrdoy}00_SST_Reynolds_168h
end
set end_sstfile=$oisstfile

#echo $beg_sstfile $end_sstfile

echo first06/last06: $first06 $last06


# Get ATMOS/TB files
@ i = $first06
while ( $i <= $last06)
    echo Retrieving /pub/joel/tb_atm/N${i}_QATM_NCEP_6h.h5.bz2
    ncftpget samoa.gsfc.nasa.gov . "/pub/joel/tb_atm/N${i}_QATM_NCEP_6h.h5.bz2"

    bunzip2 N${i}_QATM_NCEP_6h.h5.bz2
    if ($status != 0) then
	exit (1)
    endif

    echo Retrieving /pub/joel/aquarius-met/N${i}_QMET_NCEP_6h.bz2
    ncftpget samoa.gsfc.nasa.gov . "/pub/joel/aquarius-met/N${i}_QMET_NCEP_6h.bz2"

    bunzip2 N${i}_QMET_NCEP_6h.bz2
    if ($status != 0) then
	exit (1)
    endif

    set mndy=`doy2mndy.csh $i`
    set yr=`echo $mndy | cut -d "-" -f1`
    set mn=`echo $mndy | cut -d "-" -f2`
    set dy=`echo $mndy | cut -d "-" -f3`
    set hr = `echo $i | cut -c 8-9`
    set jul=`date -u -d $mndy +%j`

    echo mk_aquarius_ancillary_data $yr $mn $dy $hr $jul $beg_sstfile $end_sstfile \
    $dayfrombeg N${i}_QATM_NCEP_6h.h5 N${i}_QMET_NCEP_6h

    mk_aquarius_ancillary_data $yr $mn $dy $hr $jul $beg_sstfile $end_sstfile \
    $dayfrombeg N${i}_QATM_NCEP_6h.h5 N${i}_QMET_NCEP_6h

    rm N${i}_QATM_NCEP_6h.h5 N${i}_QMET_NCEP_6h


    set fmtdte = `doy2mndy.csh $i`
    set sec = `date -u -d "$fmtdte $hr" +%s`
    @ sec2 = $sec + 21600

    set i=`date -u -d "1970-01-01 $sec2 seconds" +%Y%j%H`
    echo i: $i
end

rm $beg_sstfile $end_sstfile

exit




