#include <stdio.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <stdint.h>
#include <iostream>
#include <fstream>

#define VERSION "1.02"
#define LAUNCHTIME 991750828

//    Modification history:
//          Programmer     Organization      Date      Description of change
//        --------------   ------------    --------    ---------------------
//        Joel Gales       FutureTech      09/21/10    Original development
//        Joel Gales       FutureTech      06/17/11    Make Start/Stop times
//                                                     more robust (reliable)
//        Joel Gales       FutureTech      07/27/11    Use LAUNCHTIME as lower
//                                                     time limit
//        Joel Gales       FutureTech      08/10/11    Update to V 1.00
//        Joel Gales       FutureTech      08/29/11    Change time from OBT
//                                                     to CDH
//        Joel Gales       FutureTech      09/19/11    Change to CDH time for
//                                                     max_time



using namespace std;

double gpstai2unix( double gpstai);

int main (int argc, char* argv[])
{
  cerr << "get_acs_times " << VERSION << " (" 
       <<  __DATE__ << " " << __TIME__ << ")" << endl;

  if ( argc == 1) {
    cerr << endl << "get_acs_times hkt_file"
	 << endl;
    return 0;
  }

  uint32_t gps_hkt;
  char *cptr = (char *) &gps_hkt;

  uint8_t hktrec[4000];

  ifstream hktfile;

  char timebuf[80];
  time_t rawtime;
  tm *ptm;

  // Open housekeeping file andget_acs_times get # of records
  hktfile.open(argv[1], fstream::in | fstream::binary);
  hktfile.seekg (0, ios::end);
  uint32_t totalbytes = hktfile.tellg();
  hktfile.seekg (0, ios::beg);

  // Find max time
  uint32_t max_time=0;
  for (size_t i=0; i<totalbytes/4000; i++) {
    hktfile.read( (char *) hktrec, 4000);

    memcpy( cptr+0, &hktrec[8+92+3], 1);
    memcpy( cptr+1, &hktrec[8+92+2], 1);
    memcpy( cptr+2, &hktrec[8+92+1], 1);
    memcpy( cptr+3, &hktrec[8+92+0], 1);

    // Convert from mixed endian
    //    memcpy( cptr+0, &hktrec[580+816+2], 1);
    //memcpy( cptr+1, &hktrec[580+816+3], 1);
    //memcpy( cptr+2, &hktrec[580+816+0], 1);
    //memcpy( cptr+3, &hktrec[580+816+1], 1);

    if ( gps_hkt > max_time) max_time = gps_hkt;
  }

  //  cout << gps_hkt << endl;

  // Find start time
  hktfile.seekg (0, ios::beg);
  uint32_t min_time = max_time;
  for (size_t i=0; i<totalbytes/4000; i++) {
    hktfile.read( (char *) hktrec, 4000);

    // CDH
    memcpy( cptr+0, &hktrec[8+92+3], 1);
    memcpy( cptr+1, &hktrec[8+92+2], 1);
    memcpy( cptr+2, &hktrec[8+92+1], 1);
    memcpy( cptr+3, &hktrec[8+92+0], 1);

    // Convert from mixed endian (OBT)
    //    memcpy( cptr+0, &hktrec[580+816+2], 1);
    //memcpy( cptr+1, &hktrec[580+816+3], 1);
    //memcpy( cptr+2, &hktrec[580+816+0], 1);
    //memcpy( cptr+3, &hktrec[580+816+1], 1);

    if ( gps_hkt > LAUNCHTIME && gps_hkt < min_time)
      min_time = gps_hkt;
  }

  // Print start time
  rawtime = (time_t) gpstai2unix( min_time);
  ptm = gmtime ( &rawtime );
  strftime (timebuf, 80, "%Y-%m-%dT%H:%M:%S", ptm);
  cout << "start_time=" << timebuf << endl;

  // Print stop time
  rawtime = (time_t) gpstai2unix( max_time);
  ptm = gmtime ( &rawtime );
  strftime (timebuf, 80, "%Y-%m-%dT%H:%M:%S", ptm);
  cout << "stop_time=" << timebuf << endl;

  hktfile.close();

  return 0;
}

