#include <stdio.h>
#include <stdint.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>

#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_interp.h>
#include <gsl/gsl_linalg.h>

#define PI      3.141592653589793

#define VERSION "1.05"

//    Modification history:
//          Programmer     Organization      Date      Description of change
//        --------------   ------------    --------    ---------------------
//        Joel Gales       FutureTech      09/20/10    Original development
//        Joel Gales       FutureTech      07/28/11    Rename gps_hkt to
//                                                     obt_hkt
//        Joel Gales       FutureTech      08/10/11    Update to V 1.00
//        Joel Gales       FutureTech      08/10/11    Only output good att
//                                                     records where both
//                                                     HKT & EPH exist.
//        Joel Gales       FutureTech      09/27/11    Modify to handle yaw
//                                                     angles greater than 90
//        Joel Gales       FutureTech      12/20/11    Fix AOCS time jumps
//        Joel Gales       FutureTech      06/10/13    Exit if out-of-order
//                                                     times
//        Joel Gales       FutureTech      06/11/15    Generate a truncted
//                                                     attitude file if
//                                                     out-of-order times


using namespace std;

int get_tme_z( string *, double *, double *);
double unix2gpstai( double unixtime);
double gpstai2unix( double gpstai);
extern "C" void invert_3by3_( double (*)[3], double (*)[3], double *);

int main (int argc, char* argv[])
{
  cout << "gen_aquarius_att " << VERSION << " (" 
       <<  __DATE__ << " " << __TIME__ << ")" << endl;

  if ( argc == 1) {
    cout << endl << "gen_aquarius_att ephemfile hktfile attfile"
	 << endl;
    return 0;
  }

  uint32_t obt_hkt;

  double gps_tme;
  double dummy_z;

  char *cptr = (char *) &obt_hkt;
  char *qptr;

  uint32_t nrec_eph = 0;

  string ydhmsf_str;
  string line;

  uint8_t hktrec[4000];
  const double oneth = 1000.0000000000;

  std::ostringstream ostr;

  int32_t quat_offset = 650;

  ifstream ephfile;
  ifstream hktfile;


  char *ephname = argv[1];
  ephfile.open( ephname, ifstream::in);
  if (ephfile.is_open()) {
    getline( ephfile, line);
    istringstream iStream(line);
    iStream >> nrec_eph;
  } else {
    cout << ephname << " not found." << endl;
    exit(1);
  }
  double *orb_time = (double *) calloc(nrec_eph, sizeof(double));
  double *orb_pos = (double *)  calloc(nrec_eph, 3*sizeof(double));
  double *orb_vel = (double *)  calloc(nrec_eph, 3*sizeof(double));

  for (size_t j=0; j<nrec_eph; j++) {
    getline( ephfile, line);
    get_tme_z( &line, &gps_tme, &dummy_z);

    orb_time[j] = gps_tme;

    orb_pos[3*j]   = atof(line.substr( 28, 15).c_str()) * oneth;
    orb_pos[3*j+1] = atof(line.substr( 43, 15).c_str()) * oneth;
    orb_pos[3*j+2] = atof(line.substr( 58, 15).c_str()) * oneth;
    
    orb_vel[3*j]   = atof(line.substr( 73, 15).c_str()) * oneth;
    orb_vel[3*j+1] = atof(line.substr( 88, 15).c_str()) * oneth;
    orb_vel[3*j+2] = atof(line.substr(103, 15).c_str()) * oneth;
  }


  hktfile.open(argv[2], fstream::in | fstream::binary);
  hktfile.seekg (0, ios::end);
  int32_t totalbytes = hktfile.tellg();
  hktfile.seekg (0, ios::beg);
  uint32_t nrec_hkts = totalbytes / 4000;


  gsl_matrix *time_mat = gsl_matrix_alloc(4,4);
  gsl_vector *b = gsl_vector_alloc(4);

  gsl_permutation *p = gsl_permutation_alloc(4);
  int s;
  double C, D;

  uint32_t *Time = (uint32_t *) calloc(nrec_hkts, sizeof(uint32_t));
  double *Pos = (double *) calloc(3*nrec_hkts, sizeof(double));
  double *Vel = (double *) calloc(3*nrec_hkts, sizeof(double));
  float *quaternion = (float *) calloc(4*nrec_hkts, sizeof(float));

  uint32_t ngood=0;
  uint32_t prev_obt_hkt;
  uint32_t prev_Time=0;
  for (size_t i=0; i<nrec_hkts; i++) {
    hktfile.read( (char *) hktrec, 4000);
    //t = byte([obt[2], obt[3], obt[0], obt[1]])
    memcpy( cptr+0, &hktrec[580+816+2], 1);
    memcpy( cptr+1, &hktrec[580+816+3], 1);
    memcpy( cptr+2, &hktrec[580+816+0], 1);
    memcpy( cptr+3, &hktrec[580+816+1], 1);

    if ( i > 0) {
      if ( (obt_hkt-prev_obt_hkt) > 9 && (obt_hkt-prev_obt_hkt) < 15) {
	cout << "Bad AOCS time:  " << obt_hkt << " in " << argv[2] << endl;
	obt_hkt = prev_obt_hkt + 8;
	cout << "Corrected time: " << obt_hkt << endl;
      }
    }
    Time[i] = obt_hkt;
    prev_obt_hkt = obt_hkt;
    //    cout << obt_hkt << endl;

    if ( prev_Time > Time[i]) {
      cout << "Out of order Time: " << prev_Time << " " << Time[i];
      cout << " for HKT record: " << i << endl;
      nrec_hkts = i;
      break;
      //      exit(1);
    }

    qptr = (char *) &quaternion[4*i+0];
    memcpy( qptr+0, &hktrec[580+quat_offset+2], 1);
    memcpy( qptr+1, &hktrec[580+quat_offset+3], 1);
    memcpy( qptr+2, &hktrec[580+quat_offset+0], 1);
    memcpy( qptr+3, &hktrec[580+quat_offset+1], 1);

    qptr = (char *) &quaternion[4*i+1];
    memcpy( qptr+0, &hktrec[580+quat_offset+4+2], 1);
    memcpy( qptr+1, &hktrec[580+quat_offset+4+3], 1);
    memcpy( qptr+2, &hktrec[580+quat_offset+4+0], 1);
    memcpy( qptr+3, &hktrec[580+quat_offset+4+1], 1);

    qptr = (char *) &quaternion[4*i+2];
    memcpy( qptr+0, &hktrec[580+quat_offset+8+2], 1);
    memcpy( qptr+1, &hktrec[580+quat_offset+8+3], 1);
    memcpy( qptr+2, &hktrec[580+quat_offset+8+0], 1);
    memcpy( qptr+3, &hktrec[580+quat_offset+8+1], 1);

    qptr = (char *) &quaternion[4*i+3];
    memcpy( qptr+0, &hktrec[580+quat_offset+12+2], 1);
    memcpy( qptr+1, &hktrec[580+quat_offset+12+3], 1);
    memcpy( qptr+2, &hktrec[580+quat_offset+12+0], 1);
    memcpy( qptr+3, &hktrec[580+quat_offset+12+1], 1);

    for (size_t j=0; j<nrec_eph-1; j++) {

      if ( orb_time[j] <= obt_hkt && orb_time[j+1] > obt_hkt) {
	double time1_1 = orb_time[j] - obt_hkt;
	double time2_1 = orb_time[j+1] - obt_hkt;
	double time1_2 = time1_1 * time1_1;
	double time2_2 = time2_1 * time2_1;
	double time1_3 = time1_2 * time1_1;
	double time2_3 = time2_2 * time2_1;
	
	gsl_matrix_set( time_mat, 0, 0, time1_3);
	gsl_matrix_set( time_mat, 0, 1, time1_2);
	gsl_matrix_set( time_mat, 0, 2, time1_1);
	gsl_matrix_set( time_mat, 0, 3, 1.0);

	gsl_matrix_set( time_mat, 1, 0, time2_3);
	gsl_matrix_set( time_mat, 1, 1, time2_2);
	gsl_matrix_set( time_mat, 1, 2, time2_1);
	gsl_matrix_set( time_mat, 1, 3, 1.0);

	gsl_matrix_set( time_mat, 2, 0, 3*time1_2);
	gsl_matrix_set( time_mat, 2, 1, 2*time1_1);
	gsl_matrix_set( time_mat, 2, 2, 1.0);
	gsl_matrix_set( time_mat, 2, 3, 0.0);

	gsl_matrix_set( time_mat, 3, 0, 3*time2_2);
	gsl_matrix_set( time_mat, 3, 1, 2*time2_1);
	gsl_matrix_set( time_mat, 3, 2, 1.0);
	gsl_matrix_set( time_mat, 3, 3, 0.0);

	gsl_linalg_LU_decomp( time_mat, p, &s);


	// X, Vx
	gsl_vector_set( b, 0, orb_pos[3*j]);
	gsl_vector_set( b, 1, orb_pos[3*(j+1)]);
	gsl_vector_set( b, 2, orb_vel[3*j]);
	gsl_vector_set( b, 3, orb_vel[3*(j+1)]);
	gsl_linalg_LU_svx( time_mat, p, b);

	C = gsl_vector_get( b, 2);
	D = gsl_vector_get( b, 3);

	Pos[3*i] = D;
	Vel[3*i] = C;

	// Y, Vy
	gsl_vector_set( b, 0, orb_pos[3*j+1]);
	gsl_vector_set( b, 1, orb_pos[3*(j+1)+1]);
	gsl_vector_set( b, 2, orb_vel[3*j+1]);
	gsl_vector_set( b, 3, orb_vel[3*(j+1)+1]);
	gsl_linalg_LU_svx( time_mat, p, b);

	C = gsl_vector_get( b, 2);
	D = gsl_vector_get( b, 3);

	Pos[3*i+1] = D;
	Vel[3*i+1] = C;

	// Z, Vz
	gsl_vector_set( b, 0, orb_pos[3*j+2]);
	gsl_vector_set( b, 1, orb_pos[3*(j+1)+2]);
	gsl_vector_set( b, 2, orb_vel[3*j+2]);
	gsl_vector_set( b, 3, orb_vel[3*(j+1)+2]);
	gsl_linalg_LU_svx( time_mat, p, b);

	C = gsl_vector_get( b, 2);
	D = gsl_vector_get( b, 3);

	Pos[3*i+2] = D;
	Vel[3*i+2] = C;

	if ( quaternion[4*i+0] != 0) ngood++;
      }
    }
    prev_Time = Time[i];
  } // for (size_t i=0; i<nrec_hkts; i++)


  ofstream fout;
  fout.open( argv[3]);
  fout << ngood << endl;

  double Mrpy[3][3];
  double Morb[3][3];
  float Msp[3][3];

  //  fout << nrec_hkts << endl;
  float adj = 0;
  double prev_pitch;
  for (size_t i=0; i<nrec_hkts; i++) {
    /*
    // Get GPS time
    memcpy( cptr+0, &hktrec[580+816+2], 1);
    memcpy( cptr+1, &hktrec[580+816+3], 1);
    memcpy( cptr+2, &hktrec[580+816+0], 1);
    memcpy( cptr+3, &hktrec[580+816+1], 1);
    //    cout << obt_hkt << endl;
    */
    // Print start time
    char timebuf[80];
    time_t rawtime;
    tm *ptm;

    rawtime = (time_t) gpstai2unix( (double) Time[i]);
    ptm = gmtime ( &rawtime );
    strftime (timebuf, 80, "%Y/%m/%d %H:%M:%S.00000000", ptm);

    // Get quaternion values
    float q1 = quaternion[4*i];
    float q2 = quaternion[4*i+1];
    float q3 = quaternion[4*i+2];
    float q4 = quaternion[4*i+3];

    // Generate Msp
    Msp[0][0] = q1*q1 - q2*q2 - q3*q3 + q4*q4;
    Msp[0][1] = 2 * (q1*q2 + q3*q4);
    Msp[0][2] = 2 * (q1*q3 - q2*q4);

    Msp[1][0] = 2 * (q1*q2 - q3*q4);
    Msp[1][1] = -q1*q1 + q2*q2 - q3*q3 + q4*q4;
    Msp[1][2] = 2 * (q2*q3 + q1*q4);

    Msp[2][0] = 2 * (q1*q3 + q2*q4);
    Msp[2][1] = 2 * (q2*q3 - q1*q4);
    Msp[2][2] = -q1*q1 - q2*q2 + q3*q3 + q4*q4;


    double p[3];
    p[0] = Pos[3*i];
    p[1] = Pos[3*i+1];
    p[2] = Pos[3*i+2];

    double pm = sqrt(p[0]*p[0] + p[1]*p[1] + p[2]*p[2]);
    Morb[2][0] = -p[0] / pm;
    Morb[2][1] = -p[1] / pm;
    Morb[2][2] = -p[2] / pm;

    double v[3];
    v[0] = Vel[3*i];
    v[1] = Vel[3*i+1];
    v[2] = Vel[3*i+2];

    double on[3];
    on[0] = v[1]*Morb[2][2] - v[2]*Morb[2][1];
    on[1] = v[2]*Morb[2][0] - v[0]*Morb[2][2];
    on[2] = v[0]*Morb[2][1] - v[1]*Morb[2][0];
    double onm = sqrt(on[0]*on[0] + on[1]*on[1] + on[2]*on[2]);
    Morb[1][0] = -on[0] / onm;
    Morb[1][1] = -on[1] / onm;
    Morb[1][2] = -on[2] / onm;

    Morb[0][0] = Morb[1][1]*Morb[2][2] - Morb[1][2]*Morb[2][1];
    Morb[0][1] = Morb[1][2]*Morb[2][0] - Morb[1][0]*Morb[2][2];
    Morb[0][2] = Morb[1][0]*Morb[2][1] - Morb[1][1]*Morb[2][0];

    double Morb_inv[3][3], det;
    invert_3by3_( Morb, Morb_inv, &det);

    Mrpy[0][0] = Msp[0][0]*Morb_inv[0][0] +
                 Msp[0][1]*Morb_inv[1][0] +
                 Msp[0][2]*Morb_inv[2][0];

    Mrpy[2][2] = Msp[2][0]*Morb_inv[0][2] +
                 Msp[2][1]*Morb_inv[1][2] +
                 Msp[2][2]*Morb_inv[2][2];

    Mrpy[1][0] = Msp[1][0]*Morb_inv[0][0] +
                 Msp[1][1]*Morb_inv[1][0] +
                 Msp[1][2]*Morb_inv[2][0];

    Mrpy[2][0] = Msp[2][0]*Morb_inv[0][0] +
                 Msp[2][1]*Morb_inv[1][0] +
                 Msp[2][2]*Morb_inv[2][0];

    Mrpy[2][1] = Msp[2][0]*Morb_inv[0][1] +
                 Msp[2][1]*Morb_inv[1][1] +
                 Msp[2][2]*Morb_inv[2][1];

    double roll = atan(-Mrpy[2][1] / Mrpy[2][2]) * 180 / PI;
    //    double pitch = asin(Mrpy[2][0]) * 180 / PI;
    double pitch = atan2(Mrpy[2][0]*cos(roll*PI/180), Mrpy[2][2]) * 180 / PI;

    // Handle yaw angles greater than 90 deg
    double yaw = 
      atan2(-Mrpy[1][0]/cos(pitch*PI/180), Mrpy[0][0]/cos(pitch*PI/180)) * 
      180 / PI;

    if ( isnan(roll)) {
      continue;
    }

    // Adjustment for pitch < -180
    if ( i > 0) {
      if ( (pitch-prev_pitch) > +180) adj = 360;
      if ( (pitch-prev_pitch) < -180) adj = 0;
      prev_pitch = pitch;
      pitch = pitch - adj;
    }

    fout << timebuf;

    fout << setprecision(6) << setw(15) << scientific << roll;
    fout << setprecision(6) << setw(15) << scientific << pitch;
    fout << setprecision(6) << setw(15) << scientific << yaw;

    fout << setprecision(10) << setw(15) << fixed << q1;
    fout << setprecision(10) << setw(15) << fixed << q2;
    fout << setprecision(10) << setw(15) << fixed << q3;
    fout << setprecision(10) << setw(15) << fixed << q4;

    fout << endl;
  }

  free( orb_time);
  free( orb_pos);
  free( orb_vel);
  free( Pos);
  free( Vel);
  free( quaternion);

  hktfile.close();
  ephfile.close();
  fout.close();

  return 0;
}

int get_tme_z( string *line, double *tme, double *z)
{
  // tme is seconds from 01/06/1980

  if (z != NULL) *z = atof(line->substr(58, 15).c_str());

  struct tm tt;
  tt.tm_sec = atoi(line->substr(17, 2).c_str());
  tt.tm_min = atoi(line->substr(14, 2).c_str());
  tt.tm_hour = atoi(line->substr(11, 2).c_str());
  tt.tm_mday = atoi(line->substr(8, 2).c_str());
  tt.tm_mon = atoi(line->substr(5, 2).c_str()) - 1;
  tt.tm_year = atoi(line->substr(0, 4).c_str()) - 1900;
  tt.tm_isdst = 0;

  time_t sec = (time_t) unix2gpstai(timegm( &tt) + 
				    atof(line->substr(19, 9).c_str()));
  *tme = sec;

  return 0;
}
