C     10/16/99 VERSION CHANGED ON 12/5/01 TO SAVE ALLOCATABLE VARIABLES.  NEED FOR COMPAQ COMPILER

C     THIS IS THE SAME AS ESTIMA1X.FOR EXCEPT SUBROUTINE HAVE DIFFERENT NAMES
C     SO AS NOT TO CONFLICT WITH OTHER ESTIMA ROUTINES

      SUBROUTINE ESTIMA1X(ICASE,NPAR,XMEA,YMEA,WT,TOL,  IFLAG,XPAR,ERR)     
C
C     ICASE=1 DENOTES INITIALIZATION, MUST BE CALLED BEFORE PROCESSING  
C     ICASE=2 DENOTES ACCUMULATION OF MEASUREMENTS                      
C     ICASE=3 DENOTES COMPUTING FINAL STATS AFTER PROCESSING MEAS.      
C                  
C     N IS THE NUMBER OF PARAMETERS TO BE FOUND. 
C                                                                       
C     XMEA ARE THE MEASUREMENTS ON THE LHS                              
C     YMEA IS THE MEASUREMENT ON THE RHS                                
C                                                                       
C     IFLAG IS TO BE CHECKED FOR THE ICASE=3.  IFLAG=1 DENOTES SINGULAR 
C                                                                       
C     XPAR IS THE RETURNED PARAMETERS                                   
C                                                                       
C     THE MEASUREMENT EQUATIONS ARE OF THE FORM:                        
C     XPAR(1)*XMEA(1)+XPAR(2)*XMEA(2)+...=YMEA                          
C
      REAL*8 XMEA(NPAR),YMEA,WT,TOL,XPAR(NPAR),ERR(NPAR)
C
      NTOT=(NPAR*(NPAR+1))/2
      CALL EST_1(ICASE,NPAR,NTOT,XMEA,YMEA,WT,TOL,  IFLAG,XPAR,ERR)
      RETURN
      END
C
      SUBROUTINE EST_1(ICASE,N,NTOT,XMEA,YMEA,WT,TOL,  IFLAG,XPAR,ERR)
C
      REAL*8 XMEA(N),YMEA,WT,TOL,XPAR(N),ERR(N),SUM,XFAC
        REAL(8), ALLOCATABLE:: A(:),Y(:)
        SAVE A,Y
C
      IF(ICASE.EQ.1) THEN
        ALLOCATE(A(NTOT),Y(N))
      DO 100 K=1,NTOT                                                   
      IF(K.LE.N) Y(K)=0.D0                                              
      A(K)=0.D0                                                         
  100 CONTINUE                                                          
      RETURN
      ENDIF
C
      IF(ICASE.EQ.2) THEN
      K=0                                                               
      DO 300 I=1,N                                                      
      XFAC=WT*XMEA(I)                                                  
      Y(I)=Y(I)+XFAC*YMEA                                               
      DO 299 J=1,I                                                      
      K=K+1                                                             
      A(K)=A(K)+XFAC*XMEA(J)                                            
  299 CONTINUE                                                          
  300 CONTINUE                                                          
      RETURN
      ENDIF
C
      IF(ICASE.EQ.3) THEN
      CALL SINV_1(N,NTOT,TOL, A, IFLAG)                                        
      DO 500 I=1,N                                                      
      SUM=0.D0                                                          
      DO 480 J=1,N                                                      
      IF(J.GE.I) K=I+((J-1)*J)/2                                        
      IF(J.LT.I) K=J+((I-1)*I)/2                                        
      SUM=SUM+A(K)*Y(J)                                                 
      IF(I.EQ.J) ERR(I)=DSQRT(A(K))                                     
  480 CONTINUE                                                          
      XPAR(I)=SUM                                                       
  500 CONTINUE 
        DEALLOCATE(A,Y)                                                         
      RETURN
      ENDIF
C                                                            
      END                                                               
CFF                                                                    
      SUBROUTINE SINV_1(N,NTOT,TOL, A, IFLAG)                                  
C
C     IBM SUBROUTINE SINV, PP 119-120                                   
C
      REAL*8 A(NTOT),TOL,SUM,DIN                                               
C
      CALL MFSD_1(N,NTOT,TOL, A, IFLAG)                                        
      IF(IFLAG.NE.0) RETURN                                             
      IPIV=NTOT                                                    
      IND=IPIV                                                          
      DO 300 I=1,N                                                      
      DIN=1.D0/A(IPIV)                                                  
      A(IPIV)=DIN                                                       
      MIN=N                                                             
      KEND=I-1                                                          
      LANF=N-KEND                                                       
      IF(KEND.EQ.0) GO TO 210                                           
      J=IND                                                             
      DO 200 K=1,KEND                                                   
      SUM=0.D0                                                          
      MIN=MIN-1                                                         
      LHOR=IPIV                                                         
      LVER=J                                                            
      DO 100 L=LANF,MIN                                                 
      LVER=LVER+1                                                       
      LHOR=LHOR+L                                                       
      SUM=SUM+A(LVER)*A(LHOR)                                           
  100 CONTINUE                                                          
      A(J)=-SUM*DIN                                                     
      J=J-MIN                                                           
  200 CONTINUE                                                          
  210 CONTINUE                                                          
      IPIV=IPIV-MIN                                                     
      IND=IND-1                                                         
  300 CONTINUE                                                          
C                                                                       
      DO 400 I=1,N                                                      
      IPIV=IPIV+I                                                       
      J=IPIV                                                            
      DO 399 K=I,N                                                      
      SUM=0.D0                                                          
      LHOR=J                                                            
      DO 390 L=K,N                                                      
      LVER=LHOR+K-I                                                     
      SUM=SUM+A(LHOR)*A(LVER)                                           
      LHOR=LHOR+L                                                       
  390 CONTINUE                                                          
      A(J)=SUM                                                          
      J=J+K                                                             
  399 CONTINUE                                                          
  400 CONTINUE                                                          
      RETURN                                                            
      END                                                               
CFF                                                                    
      SUBROUTINE MFSD_1(N,NTOT,TOL, A, IFLAG)
C                                  
C     IBM SUBROUTINE MFSD, PP 158-159                                   
C
      REAL*8 A(NTOT),TOL,SUM,DPIV,XTOL                                         
C
      IFLAG=0                                                           
      KPIV=0                                                            
      DO 300 K=1,N                                                      
      KPIV=KPIV+K                                                       
      IND=KPIV                                                          
      LEND=K-1                                                          
      XTOL=DABS(TOL*A(KPIV))                                            
      DO 200 I=K,N                                                      
      SUM=0.D0                                                          
      IF(LEND.EQ.0) GO TO 110                                           
      DO 100 L=1,LEND                                                   
      SUM=SUM+A(KPIV-L)*A(IND-L)                                        
  100 CONTINUE                                                          
  110 CONTINUE                                                          
      SUM=A(IND)-SUM                                                    
      IF(I.NE.K) GO TO 150                                              
      IF(SUM.LE.XTOL) GO TO 910                                         
      DPIV=DSQRT(SUM)                                                   
      A(KPIV)=DPIV                                                      
      DPIV=1.D0/DPIV                                                    
      GO TO 190                                                         
  150 CONTINUE                                                          
      A(IND)=SUM*DPIV                                                   
  190 CONTINUE                                                          
      IND=IND+I                                                         
  200 CONTINUE                                                          
  300 CONTINUE                                                          
      RETURN                                                            
  910 CONTINUE                                                          
      IFLAG=1                                                           
      RETURN                                                            
      END                                                               
