c     july 30 2015 change august 31 2015.  beta changed from 35.47 to 35.57 

c     july 9 2015 version changed july 30 2015.  kflag_sun removed.  it was redundant with iflag_sun
        module  l2_module
        use smap_id_module
        implicit none
        
c     ==================================================================================================
c     ===============================   math and  astronomical constants ====================================
c     ==================================================================================================
        real(8), parameter :: pi=3.14159265358979323846d0
        real(8), parameter :: rad=pi/180.d0

        real(8), parameter :: sidereal_year= 365.25636d0  ! astronomical almanac (days)
        real(8), parameter :: tropical_year= 365.24219d0  ! wikipedia (days)
        
      real(4), parameter :: tb_cos=3.0  !floor value for galaxy radiation, treated like uniform cosmic background, subtract out of galaxy tables
        real(4), parameter :: solar_flux_bak= 264.  !nominal solar flux for solar backscatter tables
        
c     ==================================================================================================
c     ==================================== geolocation constants =======================================
c     ==================================================================================================
        real(8), parameter :: re=6378.137d3  !earth equatorial radius (meters)
      real(8), parameter :: rp=6356.752d3  !earth      polar radius (meters)
      real(8), parameter :: ffac=(rp/re)*(rp/re)

c     ==================================================================================================
c     ==================================== faraday rotation constants ==================================
c     ==================================================================================================
        real(4), parameter :: freq=1.413
        real(4), parameter :: faraday_factor=135.493e-7/(freq*freq)
        real(4), parameter :: xtec_scale = 0.75
      
c     ==================================================================================================
c     ==================================== smap sensor constants =======================================
c     ==================================================================================================
      integer(4), parameter :: iorbit0=46  !orbit with longitude of 111.205 at the begin of the orbit (zang=0), make sure this is less than any orbit being processed
        real(8), parameter :: orbit_phase_dif=188.3    ! looking at node data for orbits 864-983 shows smap ascending node time is 188.3 seconds after 6:00:00 pm 
      real(8), parameter :: orbit_period=5907.69d0   !'O:\smap\orbits\memo1.txt'
      real(8), parameter :: dlon_tot=-360.d0*(orbit_period/86400.d0)  !total change in longitude for an orbit just due to rotating earth
      real(8), parameter :: cos_inclination=-0.14061d0  !       cosalp=cosd(98.083) !smap inclination for sim files, for windsat I verified using inclination gives best fit

c     real(4), parameter    :: beta=35.6 d0 !smap cone angle  used for prelaunch simulation
c       real(4), parameter    :: beta=35.5 d0  !smap cone angle in goddards simulated ata
c       real(4), parameter    :: beta=35.47   !smap mean cone angle for real data, used until aug 31 2015 
        real(4), parameter    :: beta=35.57   !thomas value implement aug 31 2015
        
        real(4), parameter :: rpy_error=2.
        
c     min and max ta for ocean
c       real(4), parameter :: tamin(4)=(/ 80.,  50.,-25.,-25./)
c       real(4), parameter :: tamax(4)=(/180., 140., 25., 25./)
        
c     min and max ta for ocean and land
        real(4), parameter :: tamin(4)=(/ 80.,  50.,-100.,-100./)
        real(4), parameter :: tamax(4)=(/330., 330., 100., 100./)
        
c     ==================================================================================================
c     =============================== dimension sizes for arrays =======================================
c     ==================================================================================================
      integer(4), parameter :: ncel=180, nomega=1441, nzang=1441  
        integer(4), parameter :: nzangtot1=117*2880 + 1
        integer(4), parameter :: nzangtot2=117*1440 + 1 

        integer(4), parameter :: nobs_max=  189  !for resampling arrays
        integer(4), parameter :: ncel_trg=  720  !for resampling arrays
        integer(4), parameter :: ntarget_max=7*ncel_trg  !for resampling arrays
        integer(4), parameter :: idscan_max=7  !max range of scans for resampling
        integer(4), parameter :: nlat= 720  !for resampling arrays
        integer(4), parameter :: nlon=1440  !for resampling arrays
        integer(4), parameter :: nlon1= -840  !for resampling arrays
        integer(4), parameter :: nlon2= 2380  !for resampling arrays

        
        integer(4) ksat
        integer(4) iflag_l0(maxscan)

!     ==========================================================================================================
!     ======================================== input l1a arrays ================================================
!     ==========================================================================================================
!     alpha_in and time_obs only needed by
!     1,  'o:\smap_real\routines\read_rss_l1a.f'
!     2.  'o:\smap_real\routines\resample_along_scan.f'
!     3.  'o:\smap_real\routines\smap_geolocation_cells.f' 

      integer(4) iorbit,numscan
        integer(4) vernum

      integer(1), dimension(       maxscan)   :: iasc_scan
      integer(4), dimension(       maxscan)   :: numcel
      real(8),    dimension(       maxscan)   :: utc_2000
      real(4),    dimension(     3,maxscan)   :: scpos,scvel,scrpy,scloc_jpl
      real(8),    dimension(maxcel,maxscan)   :: time_obs
      real(4),    dimension(maxcel,maxscan)   :: alpha_in,xlat_jpl,xlon_jpl
      real(4),    dimension(maxcel,maxscan,4) :: ta,tf
      real(4),    dimension(     2,maxscan)   :: ant_loss,ant_temp
      integer(4), dimension(maxcel,maxscan,4) :: iflag_qc
      
!     ==========================================================================================================
!     ====================================== calculated by read routine ========================================
!     ==========================================================================================================
      real(8) scan_period(maxscan)
      real(4) alpha_step(maxscan)

!     ==========================================================================================================
!     ========================== values read in by readin_data_files.f =========================================
!     ==========================================================================================================

        integer(4) nbad,iorbit_bad(10000)
        
!     read in from sun_file
        real(8) time_sun(nzang)
        real(4) eia_sun(nzang,ncel)
        integer(2), allocatable :: tasun_tab(:,:,:,:,:)


!     read in from galaxy_file
        real(8) time_galaxy(nzang)
        real(4) eia_galaxy(nzang,ncel)
        integer(2), allocatable :: tagal_tab(:,:,:,:,:)
        
!     read in from sun_bak_file
        real(4) tasun_bak_tab(161,26,3,ncel)

!     read in from land_file
        integer(2) ifpt_lnd(ncel,nzangtot1),ifrc_lnd(ncel,nzangtot1)

        integer(2) nobs_tab(ncel_trg),icel_tab(nobs_max,ncel_trg)
        integer(1)                    iscn_tab(nobs_max,ncel_trg)
        
        real(4) rsp_weight_scan(8,11) 
        real(4) rsp_weight_field(nobs_max,ntarget_max)
        

!     ==========================================================================================================
!     =================================== geolocation arrays ===================================================
!     ==========================================================================================================

!     iflag_sun  is 1 if sun  is visible from spacecraft (usual case), otherwise it = 0
!     iflag_moon is 1 if moon is visible from spacecraft (usual case), otherwise it = 0
!     zang is the intra-orbit angle. it is zero at the south  pole, 90 deg at the equator ascending node, etc.
!     three elements in scloc are:
!       geodetic latitude of the spacecraft earth nadir point (deg)
!       east longitude    of the spacecraft earth nadir point (deg)
!       the spacecraft altitude, nadir point to spacecraft  (meters)
!     sund  is the earth-to-sun unit vector in eci coordinates 
!     sunr  is the unit vector from spacecraft to sun reflection point on earth in eci coordinates
!     moond is the earth-to-moon unit vector in eci coordinates 
!     moonr is the unit vector from spacecraft to moon reflection point on earth in eci coordinates

!     cellat is geodetic latitude (deg)
!       cellon is    east longitude (deg)
!     celtht is boresight earth incidence angle (deg)
!     celphi is boresight  earth-projection relative to clockwise from north (deg), boresight points towards earth
!     suntht is sun vector earth incidence angle (deg)
!     sunphi is sun vector earth_projection relative to clockwise from north (deg), sun vector points away from earth
!     sunglt   is the sun  glint angle: angle between the specular reflected boresight vector and the  sun vector (deg)
!     moonglt is the moon glint angle: angle between the specular reflected boresight vector and the moon vector (deg)
!     glxlat is j2k declination latitude from where the specular galactic radiation originated (deg)
!     glxlon is j2k ascension from where the specular galactic radiation originated (deg)

      integer(1), dimension(         maxscan) :: iflag_sun,iflag_moon
      real(8),    dimension(         maxscan) :: orbit,zang,frcrev,dlon,sun_zenith,sunlat,sunlon,sundis,beta_sun,alpha_sun 
      real(8),    dimension(       3,maxscan) :: y0,sund,moond,scloc,sunr,moonr
      real(4),    dimension(  maxcel,maxscan) :: cellat,cellon,celtht,celphi,celsun,suntht,sunphi,celmoon,glxlat,glxlon !these are only used by 'o:\smap_real\routines\smap_geolocation_cells.f'
        
!     ==========================================================================================================
!     ====================================== resampled arrays ==================================================
!     ==========================================================================================================
        
      integer(1) iflag_rsp(360,maxscan)
      real(8) time_rsp(360,maxscan)
      real(4) ta_rsp(8,360,maxscan)
        real(4) rcell(3,360,maxscan),buf_rsp(28,360,maxscan)
        
      real(4) cel1_earth(nlon,nlat),cel2_earth(nlon,nlat),cel3_earth(nlon,nlat)
 
        integer(2) iscn_target(nlon1:nlon2,nlat,2), itrg_target(  nlon1:nlon2,nlat,2)
        real(4) dissq_target(nlon1:nlon2,nlat,2),xlat_target(nlon1:nlon2,nlat,2),xlon_target(nlon1:nlon2,nlat,2)
        real(4) rcell_target(3,nlon1:nlon2,nlat,2)

        end module  l2_module


