c     modeled after the amsr2 routine with faraday variables added
c     routine get_gm_angle want ut1, but here i give it utc.  this is not a problem
c     because of the circular method of converting earth fixed scpos,scpos to j2000 values
c     that is done in read_l1a and then using the same utc time to convert back here.
c     there will be a extremely small error in sunlon due to this that is completely negligible.

      subroutine smap_geolocation_cells
      use l2_module
      implicit none
        
      real(4), parameter :: alphax=0 !dummy value for alpha
      real(4), parameter :: betax =0 !dummy value for beta

      real(8) x_eci(3),y_eci(3),z_eci(3),scpos_plumb(3)
      real(8) scpos_j2000(3),scpos_eci(3),rpy(3)
      real(8) scpos0(3),scvel0(3),sunvec(3),moonvec(3)
      real(8) observation_time,time_since_scan_start
      real(8) rotangle,days,np(3,3),np_inv(3,3),det 
        
      real(8) b(3),refl(3),xlat,xlon_rot,range,rearth,thtinc,azim,pra,suninc,sunazm,sunglt,moonglt
      real(4) dsdh,xlat_ion,xlon_ion,k_earth(3)

      integer(4) iscan,icel,ierror,jflag

      real(4) dellat,dellon
      
      real(4) sind, cosd, asind, atan2d

      do iscan=1,numscan
        
c     for smap, 10 is missing data, 8 and 9 are problem related to scan (ie icel)
c     since this routine consider scans, i only need to check for 8, 9 10
         if(iflag_l0(iscan).ge.8) then
            cellat(:,iscan)=0
            cellon(:,iscan)=0
            celtht(:,iscan)=0
            celphi(:,iscan)=0
            celsun(:,iscan)=0
            suntht(:,iscan)=0
            sunphi(:,iscan)=0
            celmoon(:,iscan)=0
            glxlat(:,iscan)=0
            glxlon(:,iscan)=0
            cycle
         endif

         scpos0=scpos(:,iscan)
         scvel0=scvel(:,iscan)
         rpy=   scrpy(:,iscan)

c     days essentially constant over scan and hence days, np, and np_inv need only be found once per scan
         days=utc_2000(iscan)/86400.d0 - 0.5d0
         call fd_precession(days, np) !i do not bother doing the nutation.  
         call invert_3by3(np, np_inv,det)
         if(abs(det-1).gt.1.e-6) stop 'error in np_inv,pgm stopped'
        
        
c     ============================================================================================================================
c     ===============================================  loop through icel =========================================================      
c     ============================================================================================================================
        jflag=0

        do icel=1,numcel(iscan)
        
           time_since_scan_start=time_obs(icel,iscan) - utc_2000(iscan)
           observation_time = utc_2000(iscan) + time_since_scan_start 
           call get_gm_angle(observation_time,days, rotangle)
           scpos_j2000 = scpos0 + time_since_scan_start*scvel0
           scpos_eci=matmul(np,scpos_j2000)
 
c     for most sats rpy are relative to geocentric and the s/c axes are found there and then converted to eci
c     for smap, rpy are relative to geoid, and scpos is first converted to eci and then s/c axis are found
c       call get_sc_axes_from_rpy(scpos_j2000,y0,rpy(1),rpy(2),rpy(3) ,x_j2000,y_j2000,z_j2000)
c     this is where you do pitch and roll correction, smap does not do this for now
c       x_eci    =matmul(np,x_j2000)
c       y_eci    =matmul(np,y_j2000)
c       z_eci    =matmul(np,z_j2000)

           call fd_scpos_plumb(re,rp,ffac,scpos_eci, scpos_plumb)
           call get_sc_axes_from_rpy(scpos_plumb,y0(:,iscan),rpy(1),rpy(2),rpy(3) ,x_eci,y_eci,z_eci)
        
           call fd_cell_parameters(3,re,rp,ffac,
     &          rotangle,scpos_eci,x_eci,y_eci,z_eci,beta,alpha_in(icel,iscan),sunvec,moonvec,
     &          b,refl,xlat,xlon_rot,range,rearth,thtinc,azim,pra,suninc,sunazm,sunglt,moonglt,
     &          dsdh, xlat_ion, xlon_ion, k_earth,ierror)

           if(ierror.ne.0) jflag=1

           cellat(icel,iscan)=xlat
           cellon(icel,iscan)=xlon_rot
           call fixang4( cellon(icel,iscan)) !needed because of real(8) to real(4) conversion
           celtht(icel,iscan)=thtinc                       
           celphi(icel,iscan)=azim
           celsun(icel,iscan)=sunglt
           suntht(icel,iscan)=suninc
           sunphi(icel,iscan)=sunazm       
           celmoon(icel,iscan)=moonglt

           refl=matmul(np_inv,refl) !convert reflection vector back to j2000
           glxlat(icel,iscan)=asind(refl(3)) !geocentric latitude
           glxlon(icel,iscan)=atan2d(refl(2),refl(1))
           call fixang4(glxlon(icel,iscan))  
      
        
c       if(iscan.eq.400 .and. icel.eq.numcel(iscan)) write(*,*) cellat(icel,iscan),cellon(icel,iscan)
c       if(iscan.eq.401 .and. icel.eq.1)             write(*,*) cellat(icel,iscan),cellon(icel,iscan)
        
c       if(icel.eq.100) write(*,*) scloc(3,iscan)
        
c       if(iscan.ne.150) cycle
c       if(icel .ne.100) cycle

c      cycle
        
           dellat=xlat_jpl(icel,iscan)-cellat(icel,iscan)
           dellon=xlon_jpl(icel,iscan)-cellon(icel,iscan)
           if(dellon.lt.-180) dellon=dellon+360
           if(dellon.gt. 180) dellon=dellon-360
           dellon=cosd(cellat(icel,iscan))*dellon
        
           if(abs(dellat).gt.0.006 .or. abs(dellon).gt.0.006) then 
c       write(*,'(2i5,11f9.4)') iscan,icel,xlat_jpl(icel,iscan),cellat(icel,iscan),xlon_jpl(icel,iscan),cellon(icel,iscan),
c     * dellat,dellon,rpy,alpha_in(icel,iscan),thtinc
           endif

        enddo                   !icel
      

        if(jflag.eq.1 .and. iflag_l0(iscan).ne.5 .and. iflag_l0(iscan).ne.7) then
           if(iflag_l0(iscan).eq.6) then
              iflag_l0(iscan)=7 !rpy error plus no earth intersection error
           else
              iflag_l0(iscan)=5 !rpy ok but no earth intersection error occured
           endif
        endif

      enddo                     !iscan
 
      return
      end subroutine smap_geolocation_cells
