c     march 10 2015 version update march 26 2015
c     safeguard normalizaiton applied to refl and safeguard to the asind function for r_ion(3) applied


c     x, y, z  s/c vectors
c     x vector points in general direction of s/c velocity
c     z vector points is s/c nadir and points towards earth for normal attitude
c     y vector is given by z cross x
 

c     icase=1 means compute satellite subpoint location
c     icase=2 means compute just cell locations given alpha and beta
c     icase=3 means compute all  cell parameters given alpha and beta
c     icase=4 means compute cell locations given boresight

c     alpha is defined the same way as it is for most other sats. this is also how jpl defines it
c     this usual convention is the negative of the mathematical alpha

      subroutine fd_cell_parameters(icase,re,rp,ffac,
     &     rotangle,scpos,x,y,z,beta,alpha,sunvec,moonvec, 
     &     b,refl,xlat,xlon_rot,range,rearth,thtinc,azim,pra,suninc,sunazm,sunglt,moonglt,
     &     dsdh, xlat_ion, xlon_ion, k_earth,ierror)
      implicit none

      real(8), parameter :: alt_ion=420.e3 !effective height of ionosphere in meters

      integer(4), intent(in) :: icase
      real(8),    intent(in)    :: re,rp,ffac,rotangle,scpos(3),x(3),y(3),z(3),sunvec(3),moonvec(3)
      real(4),    intent(in)    :: beta,alpha
      real(8),    intent(out)   :: refl(3),xlat,xlon_rot,range,rearth,thtinc,azim,pra,suninc,sunazm,sunglt,moonglt
      real(4),    intent(out)   :: dsdh,xlat_ion,xlon_ion,k_earth(3)
      integer(4), intent(out):: ierror        
      real(8), intent(inout) :: b(3)
                                                                                                  
      real(8) dot_product_unit8

      real(8) r,ru(3),sinlat,coslat,costht,xlon
      real(8) sinbeta,cosbeta
      real(8) bx,by,bz,cel1,cel2,cel3
      real(8) coslon,sinlon
      real(8) geoid(3),cosazim,cosang
      real(8) deltau
      real(8) ha(3),he(3),ve(3),dot1,dot2
        
!     following are additional arrays needed to compute pra,dsdh,xlat_ion,xlon_ion,k_earth(3)
      real(8) rearth_ion,k(3),r_ion(3),h_ion,cosbetax,arg,cosrot,sinrot
        
      real(4) sind, cosd, asind, acosd, atand, atan2d
      real(8) dsind, dcosd, dasind, dacosd, datand, datan2d

      r=sqrt(dot_product(scpos,scpos))
      ru=scpos/r

      if(icase.eq.1) then       !find pointing vector b(3) to geoid
         sinlat=ru(3)
         coslat=sqrt(ru(1)*ru(1)+ru(2)*ru(2))
         rearth=rp/sqrt(ffac*coslat**2+sinlat**2)  
         deltau=((r-rearth)/re)*sqrt(1 - (1-ffac)*sinlat**2/((ffac*coslat)**2+sinlat**2))
         b(1)=-ru(1)
         b(2)=-ru(2)
         b(3)=-ru(3)*(1+deltau)/(ffac+deltau)
         b=b/sqrt(dot_product(b,b))      
      endif

      if(icase.eq.2 .or. icase.eq.3) then !compute b(3) from alpha and beta
         cosbeta=cosd(beta)                                                                      
         sinbeta=sind(beta)                                                                      
         bx= sinbeta*cosd(alpha) !input alpha defined relative to the x axes,  alpha=-90 denotes bx=0, by=1
         by=-sinbeta*sind(alpha) !input alpha defined relative to the x axes , alpha=-90 denotes bx=0, by=1
         bz=cosbeta                                 
         b=bx*x+by*y+bz*z                            
      endif


c     for icase=4, b(3) is an input

      call loccel_range(r,ru(1),ru(2),ru(3),b(1),b(2),b(3),re,rp, cel1,cel2,cel3,rearth,range,ierror)

      if(ierror.ne.0) then
         refl=0
         xlat=0
         xlon_rot=0
         range=0
         rearth=0
         thtinc=0
         azim=0
         pra=0
         suninc=0
         sunazm=0
         sunglt=0
         moonglt=0
         dsdh=0
         xlat_ion=0
         xlon_ion=0
         k_earth=0
         return
      endif

      if(icase.eq.4) return     !case 4 is to see if s/c can see sun, so there is nothing else to computed

      coslat=sqrt(cel1*cel1+cel2*cel2) !cosine of geocentric lat                                 
      xlat=datand(cel3/(ffac*coslat)) !geodetic cell latitude
      xlon=datan2d(cel2,cel1)                                              
      call fixang8( xlon)

      xlon_rot=xlon - rotangle  !convert from inertial to earth
      call fixang8( xlon_rot)

      if(icase.ne.3) return     !nothing else to do for case 1,2 and 4
 
 
c     =================================================================================================================
c     ======= compute earth inc. ang. (thtinc) and earth azimuth angle wrt. north (azim) ==============================
c     =================================================================================================================

      coslon=dcosd(xlon)
      sinlon=dsind(xlon)
      coslat=dcosd(xlat)         
      geoid(1)=coslat*coslon                                
      geoid(2)=coslat*sinlon                                
      geoid(3)=dsind(xlat)

      costht=-dot_product_unit8(b,geoid)
      thtinc=dacosd(costht)
      
      if(abs(costht).gt.0.9999999999d0) then !too close to nadir to get azim
         azim=0 
      else                         
         cosazim=(-sinlon*b(1)+coslon*b(2))/sqrt(1.-costht*costht)
         if(cosazim.lt.-1.) cosazim=-1.
         if(cosazim.gt. 1.) cosazim= 1.
         azim=dacosd(cosazim)
         if(b(3)+costht*geoid(3).lt.0.) azim=-azim
         azim=90.-azim          ! convert to relative to clockwise from north
         call fixang8(azim)
      endif

c     =================================================================================================================
c     ======= compute sun glint angle (sunglt), sun inc. ang. (suninc) and sun azimuth angle (sunazm) =================
c     =================================================================================================================
 
      refl=b + 2*costht*geoid   !specular reflected boresight ray
      refl=refl/sqrt(dot_product(refl,refl)) !round error can cause slight normalization error.  this fixes it

c     compute sun glitter angle
      cosang=dot_product_unit8(refl,sunvec)
      sunglt=dacosd(cosang)

c     compute moon glitter angle
      cosang=dot_product_unit8(refl,moonvec)
      moonglt=dacosd(cosang)

c     compute sun tht and azimuth angles
      costht=dot_product_unit8(sunvec,geoid)
      suninc=dacosd(costht)

      if(abs(costht).gt.0.9999999999d0) then !too close to nadir to get sunazm
         sunazm=0 
      else                         
         cosazim=(-sinlon*sunvec(1)+coslon*sunvec(2))/sqrt(1.-costht*costht)
         if(cosazim.lt.-1.) cosazim=-1.
         if(cosazim.gt. 1.) cosazim= 1.
         sunazm=dacosd(cosazim)
         if(sunvec(3)-costht*geoid(3).lt.0.) sunazm=-sunazm !minus sign used here becuase sun is pointing away from surface
         sunazm=90-sunazm       ! convert to relative to clockwise from north
         call fixang8(sunazm)
      endif

c     =================================================================================================================
c     ============================= compute polarization rotation angle (pra) =========================================
c     =================================================================================================================
 
 
      call cross_norm(b,z, ha)  !ha=kx(-z)=(-b)x(-z)=bxz
 
      if(thtinc.gt.0.01) then
         call cross_norm(geoid,b, he) !he=kxg=(-b)xg=gxb
      else
         he=ha
      endif
 
      call cross_norm(b,he, ve) !ve=hexk=bxhe
 
      dot1=dot_product(ve,ha)
      dot2=dot_product(he,ha)
      pra=datan2d(dot1,dot2)

c     =================================================================================================================
c     ======================================= compute faraday rotation parmeters =========================================
c     =================================================================================================================

      k=-b                      !k is propagation vector
        
      sinlat=ru(3)
      coslat=sqrt(ru(1)*ru(1)+ru(2)*ru(2))
      rearth_ion=rp/sqrt(ffac*coslat**2+sinlat**2)  
 
      h_ion=rearth_ion + alt_ion !this is an approximation because rearth_ion varies over ionoshpere field of view by about 2 km
 
      cosbetax=-dot_product_unit8(ru,b)
 
      arg=h_ion**2 - r*r*(1-cosbetax**2)
      if(arg.le.0) stop 'program stopped, arg oob'
      arg=sqrt(arg)
      dsdh= h_ion/arg
 
      r_ion=scpos + (r*cosbetax-arg)*b
      r_ion=r_ion/sqrt(dot_product(r_ion,r_ion))
        
c     following probably not needed.  it just protects asind
      if(r_ion(3).lt.-1) r_ion(3)=-1
      if(r_ion(3).gt. 1) r_ion(3)= 1
 
      xlat_ion= dasind(r_ion(3)) !ion table uses geocentric lat
      xlon_ion=datan2d(r_ion(2),r_ion(1))
      xlon_ion=xlon_ion - rotangle !convert from inertial to earth
      xlon_ion=xlon_ion-360.d0*floor(xlon_ion/360.d0) !very small negative xlon_ion result in 360, thus i do the following to make sure
      if(xlon_ion.lt.0 .or. xlon_ion.gt.359.9999) xlon_ion=0
        
      sinrot=dsind(rotangle)
      cosrot=dcosd(rotangle)
      k_earth(1)=k(1)*cosrot + k(2)*sinrot !convert from inertial to earth
      k_earth(2)=k(2)*cosrot - k(1)*sinrot !convert from inertial to earth
      k_earth(3)=k(3)

      return
      end


      subroutine loccel_range(r,r1,r2,r3,s1,s2,s3,re,rp, cel1,cel2,cel3,rearth,range,ierror)     
      implicit none
        
      real(8), intent(in)  :: r,r1,r2,r3,s1,s2,s3
      real(8), intent(out) :: cel1,cel2,cel3,rearth,range
      real(8) delta,a,cosx,b,c,celmag,arg
      real(8) re,rp
      integer(4) ierror

      delta=(re/rp)*(re/rp)-1.                                               
      a=1.+delta*s3*s3                                                  
      cosx=-r1*s1-r2*s2-r3*s3
      b=cosx-delta*r3*s3
      c=1.-(re/r)*(re/r)+delta*r3*r3

      arg=b*b-a*c
      if(arg.lt.0) then         !vector does not intercept earth
         ierror=1
         return
      else
         ierror=0
      endif

      range=(b-sqrt(arg))/a                                         
      cel1=r1+range*s1                                                  
      cel2=r2+range*s2                                                  
      cel3=r3+range*s3                                                  
      celmag=sqrt(cel1*cel1+cel2*cel2+cel3*cel3)                              
      cel1=cel1/celmag                                                  
      cel2=cel2/celmag                                                  
      cel3=cel3/celmag                                                  
      rearth=r*celmag                                                   
      range=range*r
      if(range.le.0) ierror=1
      return                                                            
      end                                                               



        
      subroutine get_sc_axes_from_rpy(scpos,y0,roll,pitch,yaw, x,y,z)
      implicit none
 
!arguments
      real(8), intent(in):: scpos(3), y0(3), roll,pitch,yaw
      real(8), intent(out)::  x(3), y(3), z(3)
        
!local variables
      real(8) x0(3), z0(3)
      real(8) att_mat(3,3),a0(3,3), a(3,3)
      real(8) cos_roll,sin_roll,cos_pitch,sin_pitch,cos_yaw,sin_yaw
 
      real(4) sind, cosd
      real(8) dsind, dcosd
                        
c     x0,y0,z0 are s/c axis for 0 roll,pitch,yaw
      z0=-scpos/sqrt(dot_product(scpos,scpos))
      call cross_norm(y0,z0, x0)
 
      a0(1,:)=x0
      a0(2,:)=y0
      a0(3,:)=z0
 
      cos_roll= dcosd(roll)
      sin_roll= dsind(roll)
      cos_pitch=dcosd(pitch)
      sin_pitch=dsind(pitch)
      cos_yaw=  dcosd(yaw)
      sin_yaw=  dsind(yaw)
 
c     following is the 'rpy' or  '123' convention
 
      att_mat(1,1)= cos_pitch*cos_yaw 
      att_mat(1,2)= sin_roll*sin_pitch*cos_yaw + cos_roll*sin_yaw     
      att_mat(1,3)=-cos_roll*sin_pitch*cos_yaw + sin_roll*sin_yaw     
      att_mat(2,1)=-cos_pitch*sin_yaw 
      att_mat(2,2)=-sin_roll*sin_pitch*sin_yaw + cos_roll*cos_yaw     
      att_mat(2,3)= cos_roll*sin_pitch*sin_yaw + sin_roll*cos_yaw     
      att_mat(3,1)= sin_pitch 
      att_mat(3,2)=-sin_roll*cos_pitch        
      att_mat(3,3)= cos_roll*cos_pitch
 
      a=matmul(att_mat,a0)
 
      x=a(1,:)
      y=a(2,:)
      z=a(3,:)
      
      return
      end
 
 
c     magnitude of scpos_plumb.  it is only used by get_sc_axes_from_rpy, which first normalized it
      subroutine fd_scpos_plumb(re,rp,ffac,scpos, scpos_plumb)
      implicit none
      
      real(8) re,rp,ffac,scpos(3),scpos_plumb(3)
      real(8) r,ru(3),sinlat,coslat,rearth,deltau
      
      r=sqrt(dot_product(scpos,scpos))
      ru=scpos/r

      sinlat=ru(3)
      coslat=sqrt(ru(1)*ru(1)+ru(2)*ru(2))
      rearth=rp/sqrt(ffac*coslat**2+sinlat**2)  
      deltau=((r-rearth)/re)*sqrt(1 - (1-ffac)*sinlat**2/((ffac*coslat)**2+sinlat**2))
      scpos_plumb(1)=scpos(1)
      scpos_plumb(2)=scpos(2)
      scpos_plumb(3)=scpos(3)*(1+deltau)/(ffac+deltau)
      return
      end  

        
      subroutine fd_sunr(scpos_eci,scalt,sun, iflag_sun,sunr)
      implicit none
 
      integer(4), parameter :: nstep=9000
      real(8),    parameter ::  step=90./nstep
 
      real(8) scpos_eci(3),scalt,sun(3)
      integer(1) iflag_sun
      real(8) sunr(3)
        
      real(8) geoid(3),rearth_geoid
      real(8) rs,rg,gs,aa,bb,fov_ang
      real(8) cos_alpha,alpha,beta_glint,ratio,z,y,y1,y2,e
      real(8) sinz(0:nstep),cosz(0:nstep),sin2z(0:nstep),sina,cosa
      integer(4) istart,i,i_sv,n

      real(8) dsind, dcosd, dasind, dacosd

      data istart/1/
 
      if(istart.eq.1) then
         istart=0
         do i=0,nstep
            z=step*i
            sinz(i) =dsind(z)
            cosz(i) =dcosd(z)
            sin2z(i)=dsind(2*z)
         enddo
      endif

      call find_geoid(scpos_eci, geoid,rearth_geoid) !geoid point away from earth like surface normal vector
 
      ratio=(scalt+rearth_geoid)/rearth_geoid
 
      fov_ang=dasind(1./ratio)
      cos_alpha=dot_product(sun,geoid)
      alpha=dacosd(cos_alpha)
 
      if(alpha.gt.180-fov_ang-0.1) then !0.1 is a tolerance to handle uncertainties at limb
         iflag_sun=0.
         return
      endif
 
      iflag_sun=1
 
      if(alpha.le.1.0) then
         e=ratio-1
         beta_glint=alpha*(1+e)/(1+2*e)
         goto 100
      endif
 
      sina=dsind(0.5*alpha)
      cosa=dcosd(0.5*alpha)
        
      if(alpha.le.90) then
         n=int((0.5*alpha)/step) + 2 !n is the max limit for z when alpha<90
      else
         n=int((90-0.5*alpha)/step) + 2 !when alpha>90, max limit set by beta_glint must be 90 or less
      endif
 
      i_sv=0
      do i=1,n
         y=(sinz(i)*cosa + cosz(i)*sina)/sin2z(i)
         if(y.lt.ratio) then
            i_sv=i
            y2=y
            exit
         endif
         y1=y
      enddo
 
      if(i_sv.le.2) stop 'error1 in fd_sun'
 
      z=(i_sv-1 + (ratio-y1)/(y2-y1))*step
 
      beta_glint=z + 0.5*alpha
        
 100  continue
        
      rs=-dcosd(2*beta_glint)
      rg=-dcosd(2*beta_glint-alpha)
      gs= cos_alpha
        
      aa=(1-rg**2)/(rs-gs*rg)
      bb=rg-aa*gs
        
      sunr=aa*sun + bb*geoid          
        
      return
      end
                                                            
  
  
      subroutine find_geoid(scpos_eci, geoid,rearth) !geoid points away from earth like the surface normal vector
      use l2_module
      implicit none

      real(8), intent(in)  :: scpos_eci(3)
      real(8), intent(out) :: geoid(3),rearth

      real(8) r,ru(3),sinlat,coslat
      real(8) sindel,cosdel,xsinlat,xcoslat,ratio

      r=sqrt(dot_product(scpos_eci,scpos_eci))
      ru=scpos_eci/r

      sinlat=ru(3)                                                        
      coslat=sqrt(ru(1)*ru(1)+ru(2)*ru(2))    
      rearth=re*rp/sqrt((rp*coslat)*(rp*coslat)+(re*sinlat)*(re*sinlat))
      sindel=sin((rearth/r)*(1.003*(1-ffac)*coslat*sinlat))
                                    
      cosdel=sqrt(1.-sindel*sindel)                                         
      xsinlat=sinlat*cosdel+coslat*sindel                               
      xcoslat=coslat*cosdel-sinlat*sindel                               
      ratio=xcoslat/coslat                                              
      geoid(1)=ratio*ru(1)                                      
      geoid(2)=ratio*ru(2)                                                    
      geoid(3)=xsinlat
      return
      end


      subroutine fd_cell_vector(re,rp,nlat,nlon,ilat,ilon,xlat,xlon, cell0,cell)
      implicit none
 
      integer(4) nlat,nlon,ilat,ilon
      real(8) re,rp,xlat,xlon,rearth
      real(8) cell0(3,nlon,nlat),cell(3,nlon,nlat)
      real(8) coslat,sinlat,coslon,sinlon
 
      real(8) dsind, dcosd

      coslat=dcosd(xlat)
      sinlat=dsind(xlat)
      coslon=dcosd(xlon)
      sinlon=dsind(xlon)
 
      cell0(1,ilon,ilat)=coslat*coslon
      cell0(2,ilon,ilat)=coslat*sinlon
      cell0(3,ilon,ilat)=sinlat
 
      rearth=re*rp/sqrt((rp*coslat)*(rp*coslat)+(re*sinlat)*(re*sinlat))
 
      cell(:,ilon,ilat)= rearth*cell0(:,ilon,ilat)
 
      return
      end
 
