c     modeled after the amsr2 routine with faraday variables added
c     routine get_gm_angle want ut1, but here i give it utc.  this is not a problem
c     because of the circular method of converting earth fixed scpos,scpos to j2000 values
c     that is done in read_l1a and then using the same utc time to convert back here.
c     there will be a extremely small error in sunlon due to this that is completely negligible.

      subroutine smap_geolocation_scans
      use l2_module
      implicit none
        
      real(4), parameter :: alphax=0 !dummy value for alpha
      real(4), parameter :: betax =0 !dummy value for beta
     
      real(8) dot_product_unit8

      real(8) ru(3),zangx,zxx
cq      real(8) x_j2000(3),y_j2000(3),z_j2000(3)
      real(8) x_eci(3),y_eci(3),z_eci(3),scpos_plumb(3),r3,v3
      real(8) scpos_eci(3),scvel_eci(3),rpy(3)
      real(8) scpos0(3),scvel0(3),sunvec_j2000(3),sunvec(3),sundis_km,moonvec_j2000(3),moonvec(3),moondis_km
      real(8) x0(3),z0(3)
      real(8) rotangle,days,np(3,3),np_inv(3,3),det 
        
      real(8) b(3),refl(3),xlat,xlon_rot,range,rearth,thtinc,azim,pra,suninc,sunazm,sunglt,moonglt
      real(4) dsdh,xlat_ion,xlon_ion,k_earth(3)

      real(8) sc_to_sun(3),dot1,dot2,cossunang

      integer(4) iscan,ierror

      real(4) sind, cosd, asind, acosd, atan2d
      real(8) dsind, dcosd, dasind, dacosd, datan2d
      
      do iscan=1,numscan
        
c     for smap, 10 is missing data, 8 and 9 are problem related to scan (ie icel)
c     since this routine does not consider scan, i only need to check for 10
         if(iflag_l0(iscan).ge.10) then
            orbit(iscan)=0
            zang(iscan)=0
            y0(:,iscan)=0
            scloc(:,iscan)=0
            beta_sun(iscan)=0
            alpha_sun(iscan)=0
            iflag_sun(iscan)=0
            iflag_moon(iscan)=0
            sun_zenith(iscan)=0
            sunlat(iscan)=0
            sunlon(iscan)=0
            sundis(iscan)=0
            sund(:,iscan)=0
            moond(:,iscan)=0
            sunr(:,iscan)=0
            moonr(:,iscan)=0
            cycle
         endif
        
         scpos0=scpos(:,iscan)
         scvel0=scvel(:,iscan)
         rpy=   scrpy(:,iscan)
      
c     x0,y0,z0 are the reference axes for rpy.  for smap ref axes are the geoid axes and are found below
c       z0= -scpos0
c       x0=  scvel0
c       call cross_norm(z0,x0, y0)
        
c     this is where the spin rate (omega) is usually performed
c     currently no doing this for smap  

         if(maxval(abs(rpy)).gt.rpy_error) iflag_l0(iscan)=6

c     ============================================================================================================================
c     =================================  find parameters that only depend on iscan, not icel =====================================      
c     ============================================================================================================================

c     days essentially constant over scan and hence days, np, and np_inv need only be found once per scan
         days=utc_2000(iscan)/86400.d0 - 0.5d0
         call fd_precession(days, np) !i do not bother doing the nutation.  
         call invert_3by3(np, np_inv,det)
         if(abs(det-1).gt.1.e-6) stop 'error in np_inv,pgm stopped'
        
         call get_gm_angle(utc_2000(iscan),days, rotangle)
        
         scpos_eci=matmul(np,scpos0) 
         scvel_eci=matmul(np,scvel0)
        
c     i use to compute zang in the j2000 system (windsat still does i think).  but its better to compute it in the eci system
         r3=scpos_eci(3)/sqrt(dot_product(scpos_eci,scpos_eci))
         v3=scvel_eci(3)/sqrt(dot_product(scvel_eci,scvel_eci))

         zang(iscan)=datan2d(r3,v3) + 90.
         call fixang8(zang(iscan))
        
         zangx=zang(iscan)
!         write(*,*) iscan, zangx !, r3, v3,scvel_eci,scvel(:,iscan)

         if(iasc_scan(iscan).ne.1 .and. iasc_scan(iscan).ne.2) stop 'iasc_scan oob, pgm stopped'
         if(iasc_scan(iscan).eq.1 .and. zangx.gt.270) zangx=zangx-360.d0
         if(iasc_scan(iscan).eq.2 .and. zangx.lt. 90) zangx=zangx+360.d0
        
         orbit(iscan)=iorbit + zangx/360.d0
        
         frcrev(iscan)=orbit(iscan)-iorbit

c     dlon is the change in s/c longitude relative to the longitude at orbit start (zang=0) 
c     atan2d(cos_inclination*sind(z),cosd(z))  - 90 is the dlon for non-rotating earth
c     frcrev*dlon_tot adds the effect of rotating earth
         zxx=360*frcrev(iscan)-90
         dlon(iscan)=datan2d(cos_inclination*dsind(zxx),dcosd(zxx)) - 90 + frcrev(iscan)*dlon_tot 
         if(frcrev(iscan).gt.0.5 .and. dlon(iscan).ge.-180)  dlon(iscan)=dlon(iscan)-360

         call sun_vector(days,  sunvec_j2000,sundis_km) !returns vector in j2000 system, dis is km
         sunvec=matmul(np,sunvec_j2000)
         sund(:,iscan)=sunvec
        
         call moon_vector(days, moonvec_j2000,moondis_km) !returns vector in j2000 system, dis is km
         moonvec=matmul(np,moonvec_j2000) 
         moond(:,iscan)=moonvec           

         ru=scpos_eci/sqrt(dot_product(scpos_eci,scpos_eci))
         sun_zenith(iscan)=dacosd(dot_product_unit8(ru,sunvec))

         sunlat(iscan)=dasind(sunvec(3))
         sunlon(iscan)=datan2d(sunvec(2),sunvec(1)) - rotangle
         call fixang8( sunlon(iscan))
         sundis(iscan)=sundis_km/149.619d6
 
         sc_to_sun=sundis_km*sunvec - 1.d-3*scpos_eci
         sc_to_sun=sc_to_sun/sqrt(dot_product(sc_to_sun,sc_to_sun))
        
         x_eci=0; y_eci=0; z_eci=0 !these are not need for icase=1 and 4 calls to fd_cell_parameters

         call fd_cell_parameters(1,re,rp,ffac,
     &        rotangle,scpos_eci,x_eci,y_eci,z_eci,betax,alphax,sunvec,moonvec,
     &        b,refl,xlat,xlon_rot,range,rearth,thtinc,azim,pra,suninc,sunazm,sunglt,moonglt,
     &        dsdh, xlat_ion, xlon_ion, k_earth,ierror)

         if(ierror.ne.0) then
            if(iflag_l0(iscan).eq.6) then
               iflag_l0(iscan)=7 !rpy error plus no earth intersection error
            else
               iflag_l0(iscan)=5 !rpy ok but no earth intersection error occured
            endif
         endif
        
c     x0,y0,z0 are the reference axes for rpy.  for smap ref axes are the geoid axes and are found here
         z0=b                   !geoid nadir vector pointing down towards earth
         x0=scvel_eci
         call cross_norm(z0,x0, y0(:,iscan))
        
         scloc(1,iscan)=xlat
         scloc(2,iscan)=xlon_rot
         scloc(3,iscan)=range
        
         call fd_scpos_plumb(re,rp,ffac,scpos_eci, scpos_plumb)
         call get_sc_axes_from_rpy(scpos_plumb,y0(:,iscan),rpy(1),rpy(2),rpy(3) ,x_eci,y_eci,z_eci)
        
         cossunang=-dot_product_unit8(z_eci,sc_to_sun) !minus sign accounts for dif between windsat x,y,z and amsr x,y,z
         beta_sun(iscan)=dacosd(cossunang)
 
         dot1= dot_product(x_eci,sc_to_sun)
         dot2=-dot_product(y_eci,sc_to_sun) !minus sign accounts for dif between windsat x,y,z and amsr x,y,z
         alpha_sun(iscan)=datan2d(dot2,dot1)
        
c     sunr(3,iscan) is from unit vector from spacecraft to sun reflection point on earth
c     iflag_sun(iscan)=1 if sun is visible from spacecraft, otherwise it = 0
         call fd_sunr(scpos_eci,range,sunvec, iflag_sun(iscan),sunr(:,iscan)) 

c     moonr(3,iscan) is from unit vector from spacecraft to moon reflection point on earth
c     iflag_moon(iscan)=1 if moon is visible from spacecraft, otherwise it = 0
         call fd_sunr(scpos_eci,range,moonvec, iflag_moon(iscan),moonr(:,iscan)) 
        

      enddo                     !iscan
 
      return
      end subroutine smap_geolocation_scans
