#include <stdio.h>
#include <math.h>

#include <timeutils.h>

#include <iostream>
#include <sstream>
#include <fstream>
#include <list>
#include <stack>

#include "hdf_bin.h"
#include "hdf5utils.h"
//#include "passthebuck.h"
//#include "l2bin_uncertainty.h"

#define PI      3.141592653589793
#define VERSION "1.00"

//////////////////////////////////////////////////////////////////////////////
//  Revision 1.00  11/29/17
//  Initial version
//  J. Gales
//////////////////////////////////////////////////////////////////////////////

using namespace std;
using namespace Hdf;

int main (int argc, char* argv[])
{
  cout << "fixmeta " << VERSION << " (" 
       <<  __DATE__ << " " << __TIME__ << ")" << endl;

  if ( argc == 1) {
    cout << endl << "fixmeta L3B_mapfile"
	 << endl;
    cout << "where L3B_mapfile is the file to modify" << endl;
    cout << " " << endl;
    return 0;
  }

  hid_t h5fid = H5Fopen(argv[1], H5F_ACC_RDWR, H5P_DEFAULT);
  hid_t gid = H5Gopen1(h5fid,"/");

  // SSS
  hid_t dsid = H5Dopen1( gid, "SSS");

  H5Adelete_by_name( dsid, ".", "Intercept", H5P_DEFAULT);
  H5Adelete_by_name( dsid, ".", "Scaling", H5P_DEFAULT);
  H5Adelete_by_name( dsid, ".", "Scaling Equation", H5P_DEFAULT);
  H5Adelete_by_name( dsid, ".", "Slope", H5P_DEFAULT);
  H5Adelete_by_name( dsid, ".", "add_offset", H5P_DEFAULT);
  H5Adelete_by_name( dsid, ".", "scale_factor", H5P_DEFAULT);

  hid_t aid  = H5Screate(H5S_SCALAR);
  hid_t atype = H5Tcopy(H5T_C_S1);
  
  H5Tset_size(atype, 21);
  H5Tset_strpad(atype, H5T_STR_NULLTERM);
  hid_t attr = H5Acreate2( dsid, "long_name", atype, aid,
                           H5P_DEFAULT, H5P_DEFAULT);
  H5Awrite(attr, atype, "Sea Surface Salinity");
  H5Dclose(dsid);

  
  // SSS_ran_unc
  dsid = H5Dopen1( gid, "SSS_ran_unc");
  H5Tset_size(atype, 42);
  H5Tset_strpad(atype, H5T_STR_NULLTERM);
  attr = H5Acreate2( dsid, "long_name", atype, aid, H5P_DEFAULT, H5P_DEFAULT);
  H5Awrite(attr, atype, "Sea Surface Salinity Uncertainty (random)");

  float flagval = -32767.0;
  attr = H5Acreate2(dsid, "_FillValue", H5T_NATIVE_FLOAT, aid,
                    H5P_DEFAULT, H5P_DEFAULT);
  H5Awrite(attr, H5T_NATIVE_FLOAT, &flagval);
  H5Dclose(dsid);


  // SSS_sys_unc
  dsid = H5Dopen1( gid, "SSS_sys_unc");
  H5Tset_size(atype, 46);
  H5Tset_strpad(atype, H5T_STR_NULLTERM);
  attr = H5Acreate2( dsid, "long_name", atype, aid, H5P_DEFAULT, H5P_DEFAULT);
  H5Awrite(attr, atype, "Sea Surface Salinity Uncertainty (systematic)");

  attr = H5Acreate2(dsid, "_FillValue", H5T_NATIVE_FLOAT, aid,
                    H5P_DEFAULT, H5P_DEFAULT);
  H5Awrite(attr, H5T_NATIVE_FLOAT, &flagval);
  H5Dclose(dsid);

  
  H5Gclose(gid);
  H5Fclose(h5fid);
  
  return 0;
}


