#include <stdio.h>
#include <math.h>

#include <timeutils.h>

#include <iostream>
#include <sstream>
#include <fstream>
#include <list>
#include <stack>

#include "hdf_bin.h"
#include "hdf5utils.h"
#include "passthebuck.h"
#include "l2bin_uncertainty.h"

#define PI      3.141592653589793
#define VERSION "1.00"

//////////////////////////////////////////////////////////////////////////////
//  Revision 1.00  10/13/17
//  Change "l3m_data" to "SSS"
//  J. Gales
//
//  Revision 0.30  06/08/17
//  Change fill value from -999 to -32767
//  J. Gales
//
//  Revision 0.20  06/08/17
//  Check for existance of uncertainty fields in L2 granule
//  J. Gales

//  Revision 0.10  03/26/17
//  Initial version
//  J. Gales
//////////////////////////////////////////////////////////////////////////////

using namespace std;
using namespace Hdf;

int main (int argc, char* argv[])
{
  double blkSec;

  int32_t ibeam, iblk;
  uint32_t ifile;
  uint32_t nfiles;

  instr l2binInput;
  int32_t numBlks[MAXNFILES];
  double granstart[MAXNFILES], granstop[MAXNFILES];

  int32 i32;

  int32 *bin_flag;

  char **flagname;

  char buf[2048];

  FILE *fp;

  cout << "l2bin_uncertainty " << VERSION << " (" 
       <<  __DATE__ << " " << __TIME__ << ")" << endl;

  if ( argc == 1) {
    cout << endl << "l2bin_uncertainty infile=input_filelist ofile=L3B_output_file flaguse=flaguse pversion=pversion"
	 << endl;
    cout << "where input_filelist is the list of L2 files to bin" << endl;
    cout << "      L3B_output_filelist is the L3B output filename" << endl;
    cout << "      pversion is processing version" << endl;

    cout << " " << endl;
    return 0;
  }

  string processControl;
  parseInput(argc, argv, &l2binInput, &processControl);
  static Hdf::hdf5_Aquarius l2file[MAXNFILES];

  int32_t s_orb, e_orb;

  // Filelist input - Determine number of input files
  // ------------------------------------------------
  nfiles = 0;
  fp = fopen(l2binInput.infile, "r");
  if (fp == NULL) {
    printf("Input listing file: \"%s\" not found.\n", l2binInput.infile);
    return -1;
  }
  while(fgets(buf, 256, fp) != NULL) nfiles++;
  fclose(fp);
  cout << nfiles << " input files" << endl;

  // Open L2 input files
  // -------------------
  fp = fopen(l2binInput.infile, "r");
  for (ifile=0; ifile<nfiles; ifile++) {

    fgets(buf, 256, fp);
    buf[strlen(buf)-1] = 0;
    if ( ifile == 0) {
      l2file[ifile].openl2(buf, H5F_ACC_RDONLY, &numBlks[ifile],
                           &s_orb, &granstart[ifile], &granstop[ifile]);
    } else if ( ifile == (nfiles-1)) {
      l2file[ifile].openl2(buf, H5F_ACC_RDONLY, &numBlks[ifile],
                           &e_orb, &granstart[ifile], &granstop[ifile]);

    } else {
      l2file[ifile].openl2(buf, H5F_ACC_RDONLY, &numBlks[ifile],
                           NULL, &granstart[ifile], &granstop[ifile]);
    }

  } // ifile loop
  fclose(fp);

  
  /* Parse FLAGUSE list */
  /* ------------------ */
  size_t len_flag = strlen(l2binInput.flaguse);
  int32_t nFlag = 0;
  for (size_t i=1; i<len_flag; i++) 
    if (l2binInput.flaguse[i] == ',') nFlag++;

  flagname = (char **) calloc(nFlag+1, sizeof(char *));

  int j = 0;
  for (size_t i=0; i<len_flag; i++) {
    if (l2binInput.flaguse[i] == ',') {
      flagname[j] = l2binInput.flaguse + i + 1; 
      l2binInput.flaguse[i] = 0;
      j++;
    }
  }

  int32 flagtest[4] = {0,0,0,0};
  for (int32_t i=0; i<nFlag; i++) {
    if ( strcmp( flagname[i], "LANDYELLOW") == 0) 
      flagtest[0] |= (uint32_t) pow( 2, LAND); 
    if ( strcmp( flagname[i], "LANDRED") == 0) 
      flagtest[1] |= (uint32_t) pow( 2, LAND); 
    if ( strcmp( flagname[i], "LAND") == 0) { 
      flagtest[0] |= (uint32_t) pow( 2, LAND); 
      flagtest[1] |= (uint32_t) pow( 2, LAND); 
    }

    if ( strcmp( flagname[i], "ICEYELLOW") == 0) 
      flagtest[0] |= (uint32_t) pow( 2, ICE); 
    if ( strcmp( flagname[i], "ICERED") == 0) 
      flagtest[1] |= (uint32_t) pow( 2, ICE); 
    if ( strcmp( flagname[i], "ICE") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, ICE); 
      flagtest[1] |= (uint32_t) pow( 2, ICE); 
    }

    if ( strcmp( flagname[i], "WINDYELLOW") == 0) 
      flagtest[0] |= (uint32_t) pow( 2, WIND); 
    if ( strcmp( flagname[i], "WINDRED") == 0) 
      flagtest[1] |= (uint32_t) pow( 2, WIND); 
    if ( strcmp( flagname[i], "WIND") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, WIND); 
      flagtest[1] |= (uint32_t) pow( 2, WIND); 
    }
    if ( strcmp( flagname[i], "WINDCONVERGE") == 0) 
      flagtest[2] |= (uint32_t) pow( 2, WIND); 
    if ( strcmp( flagname[i], "SCATRFI") == 0) 
      flagtest[3] |= (uint32_t) pow( 2, WIND); 


    if ( strcmp( flagname[i], "FLAREYELLOW") == 0) 
      flagtest[0] |= (uint32_t) pow( 2, FLARE); 
    if ( strcmp( flagname[i], "FLARERED") == 0) 
      flagtest[1] |= (uint32_t) pow( 2, FLARE); 
    if ( strcmp( flagname[i], "FLARE") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, FLARE); 
      flagtest[1] |= (uint32_t) pow( 2, FLARE); 
    }

    if ( strcmp( flagname[i], "RFIYELLOW") == 0) 
      flagtest[0] |= (uint32_t) pow( 2, RFI); 
    if ( strcmp( flagname[i], "RFIRED") == 0) 
      flagtest[1] |= (uint32_t) pow( 2, RFI); 
    if ( strcmp( flagname[i], "RFI") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, RFI); 
      flagtest[1] |= (uint32_t) pow( 2, RFI); 
    }


    if ( strcmp( flagname[i], "TEMPYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, TEMP); 
      flagtest[2] |= (uint32_t) pow( 2, TEMP); 
    }

    if ( strcmp( flagname[i], "TEMPRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, TEMP); 
      flagtest[3] |= (uint32_t) pow( 2, TEMP); 
    }
    if ( strcmp( flagname[i], "TEMPVYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, TEMP); 
    }
    if ( strcmp( flagname[i], "TEMPHYELLOW") == 0) {
      flagtest[2] |= (uint32_t) pow( 2, TEMP); 
    }
    if ( strcmp( flagname[i], "TEMPVRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, TEMP); 
    }
    if ( strcmp( flagname[i], "TEMPHRED") == 0) {
      flagtest[3] |= (uint32_t) pow( 2, TEMP); 
    }
    if ( strcmp( flagname[i], "TEMP") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, TEMP); 
      flagtest[1] |= (uint32_t) pow( 2, TEMP); 
      flagtest[2] |= (uint32_t) pow( 2, TEMP); 
      flagtest[3] |= (uint32_t) pow( 2, TEMP); 
    }


    if ( strcmp( flagname[i], "MOONYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, MOON); 
      flagtest[2] |= (uint32_t) pow( 2, MOON); 
    }

    if ( strcmp( flagname[i], "MOONRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, MOON); 
      flagtest[3] |= (uint32_t) pow( 2, MOON); 
    }
    if ( strcmp( flagname[i], "MOONVYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, MOON); 
    }
    if ( strcmp( flagname[i], "MOONHYELLOW") == 0) {
      flagtest[2] |= (uint32_t) pow( 2, MOON); 
    }
    if ( strcmp( flagname[i], "MOONVRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, MOON); 
    }
    if ( strcmp( flagname[i], "MOONHRED") == 0) {
      flagtest[3] |= (uint32_t) pow( 2, MOON); 
    }
    if ( strcmp( flagname[i], "MOON") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, MOON); 
      flagtest[1] |= (uint32_t) pow( 2, MOON); 
      flagtest[2] |= (uint32_t) pow( 2, MOON); 
      flagtest[3] |= (uint32_t) pow( 2, MOON); 
    }

    if ( strcmp( flagname[i], "SUNGLINTYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, SUNGLINT); 
      flagtest[2] |= (uint32_t) pow( 2, SUNGLINT); 
    }

    if ( strcmp( flagname[i], "SUNGLINTRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, SUNGLINT); 
      flagtest[3] |= (uint32_t) pow( 2, SUNGLINT); 
    }
    if ( strcmp( flagname[i], "SUNGLINTVYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, SUNGLINT); 
    }
    if ( strcmp( flagname[i], "SUNGLINTHYELLOW") == 0) {
      flagtest[2] |= (uint32_t) pow( 2, SUNGLINT); 
    }
    if ( strcmp( flagname[i], "SUNGLINTVRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, SUNGLINT); 
    }
    if ( strcmp( flagname[i], "SUNGLINTHRED") == 0) {
      flagtest[3] |= (uint32_t) pow( 2, SUNGLINT); 
    }
    if ( strcmp( flagname[i], "SUNGLINT") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, SUNGLINT); 
      flagtest[1] |= (uint32_t) pow( 2, SUNGLINT); 
      flagtest[2] |= (uint32_t) pow( 2, SUNGLINT); 
      flagtest[3] |= (uint32_t) pow( 2, SUNGLINT); 
    }

    if ( strcmp( flagname[i], "FLUXDYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, FLUXD); 
      flagtest[2] |= (uint32_t) pow( 2, FLUXD); 
    }

    if ( strcmp( flagname[i], "FLUXDRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, FLUXD); 
      flagtest[3] |= (uint32_t) pow( 2, FLUXD); 
    }
    if ( strcmp( flagname[i], "FLUXDVYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, FLUXD); 
    }
    if ( strcmp( flagname[i], "FLUXDHYELLOW") == 0) {
      flagtest[2] |= (uint32_t) pow( 2, FLUXD); 
    }
    if ( strcmp( flagname[i], "FLUXDVRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, FLUXD); 
    }
    if ( strcmp( flagname[i], "FLUXDHRED") == 0) {
      flagtest[3] |= (uint32_t) pow( 2, FLUXD); 
    }
    if ( strcmp( flagname[i], "FLUXD") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, FLUXD); 
      flagtest[1] |= (uint32_t) pow( 2, FLUXD); 
      flagtest[2] |= (uint32_t) pow( 2, FLUXD); 
      flagtest[3] |= (uint32_t) pow( 2, FLUXD); 
    }

    if ( strcmp( flagname[i], "FLUXRYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, FLUXR); 
      flagtest[2] |= (uint32_t) pow( 2, FLUXR); 
    }

    if ( strcmp( flagname[i], "FLUXRRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, FLUXR); 
      flagtest[3] |= (uint32_t) pow( 2, FLUXR); 
    }
    if ( strcmp( flagname[i], "FLUXRVYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, FLUXR); 
    }
    if ( strcmp( flagname[i], "FLUXRHYELLOW") == 0) {
      flagtest[2] |= (uint32_t) pow( 2, FLUXR); 
    }
    if ( strcmp( flagname[i], "FLUXRVRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, FLUXR); 
    }
    if ( strcmp( flagname[i], "FLUXRHRED") == 0) {
      flagtest[3] |= (uint32_t) pow( 2, FLUXR); 
    }
    if ( strcmp( flagname[i], "FLUXR") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, FLUXR); 
      flagtest[1] |= (uint32_t) pow( 2, FLUXR); 
      flagtest[2] |= (uint32_t) pow( 2, FLUXR); 
      flagtest[3] |= (uint32_t) pow( 2, FLUXR); 
    }

    if ( strcmp( flagname[i], "GALACTICYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, GALACTIC); 
      flagtest[2] |= (uint32_t) pow( 2, GALACTIC); 
    }

    if ( strcmp( flagname[i], "GALACTICRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, GALACTIC); 
      flagtest[3] |= (uint32_t) pow( 2, GALACTIC); 
    }
    if ( strcmp( flagname[i], "GALACTICVYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, GALACTIC); 
    }
    if ( strcmp( flagname[i], "GALACTICHYELLOW") == 0) {
      flagtest[2] |= (uint32_t) pow( 2, GALACTIC); 
    }
    if ( strcmp( flagname[i], "GALACTICVRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, GALACTIC); 
    }
    if ( strcmp( flagname[i], "GALACTICHRED") == 0) {
      flagtest[3] |= (uint32_t) pow( 2, GALACTIC); 
    }
    if ( strcmp( flagname[i], "GALACTIC") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, GALACTIC); 
      flagtest[1] |= (uint32_t) pow( 2, GALACTIC); 
      flagtest[2] |= (uint32_t) pow( 2, GALACTIC); 
      flagtest[3] |= (uint32_t) pow( 2, GALACTIC); 
    }


    if ( strcmp( flagname[i], "NAVROLL") == 0) 
      flagtest[0] |= (uint32_t) pow( 2, NAV); 
    if ( strcmp( flagname[i], "NAVPITCH") == 0) 
      flagtest[1] |= (uint32_t) pow( 2, NAV); 
    if ( strcmp( flagname[i], "NAVYAW") == 0) 
      flagtest[2] |= (uint32_t) pow( 2, NAV); 
    if ( strcmp( flagname[i], "NAVOFF") == 0) 
      flagtest[3] |= (uint32_t) pow( 2, NAV); 
    if ( strcmp( flagname[i], "NAV") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, NAV); 
      flagtest[1] |= (uint32_t) pow( 2, NAV); 
      flagtest[2] |= (uint32_t) pow( 2, NAV); 
      flagtest[3] |= (uint32_t) pow( 2, NAV);
    }


    if ( strcmp( flagname[i], "SAOVERFLOW") == 0) 
      flagtest[0] |= (uint32_t) pow( 2, SAOVERFLOW);  

    if ( strcmp( flagname[i], "ROUGHOOB") == 0) 
      flagtest[0] |= (uint32_t) pow( 2, ROUGH);  
    if ( strcmp( flagname[i], "SWH") == 0) 
      flagtest[1] |= (uint32_t) pow( 2, ROUGH);  

    if ( strcmp( flagname[i], "POINTING") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, POINTING); 
      flagtest[1] |= (uint32_t) pow( 2, POINTING);  
    }

    if ( strcmp( flagname[i], "TBCONS") == 0)
      flagtest[0] |= (uint32_t) pow( 2, TBCONS); 
    if ( strcmp( flagname[i], "TBCONS") == 0) 
      flagtest[1] |= (uint32_t) pow( 2, TBCONS); 

    if ( strcmp( flagname[i], "COLDWATERYELLOW") == 0) 
      flagtest[0] |= (uint32_t) pow( 2, COLDWATER); 
    if ( strcmp( flagname[i], "COLDWATERRED") == 0) 
      flagtest[1] |= (uint32_t) pow( 2, COLDWATER); 
    if ( strcmp( flagname[i], "COLDWATER") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, COLDWATER); 
      flagtest[1] |= (uint32_t) pow( 2, COLDWATER); 
    }

    if ( strcmp( flagname[i], "TFTADIFFYELLOW") == 0) 
      flagtest[0] |= (uint32_t) pow( 2, TFTADIFF); 
    if ( strcmp( flagname[i], "TFTADIFFRED") == 0) 
      flagtest[1] |= (uint32_t) pow( 2, TFTADIFF); 
    if ( strcmp( flagname[i], "TFTADIFF") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, TFTADIFF); 
      flagtest[1] |= (uint32_t) pow( 2, TFTADIFF); 
    }

    if ( strcmp( flagname[i], "REFL_1STOKESMOONYELLOW") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, REFL_1STOKES);
    }
    if ( strcmp( flagname[i], "REFL_1STOKESMOONRED") == 0) {
      flagtest[1] |= (uint32_t) pow( 2, REFL_1STOKES);
    }
    if ( strcmp( flagname[i], "REFL_1STOKESGAL") == 0) {
      flagtest[2] |= (uint32_t) pow( 2, REFL_1STOKES);
    }
    if ( strcmp( flagname[i], "REFL_1STOKES") == 0) {
      flagtest[0] |= (uint32_t) pow( 2, REFL_1STOKES);
      flagtest[1] |= (uint32_t) pow( 2, REFL_1STOKES);
      flagtest[2] |= (uint32_t) pow( 2, REFL_1STOKES);
    }

    if ( strcmp( flagname[i], "RFI_REGION") == 0)
      flagtest[0] |= (uint32_t) pow( 2, RFI_REGION); 
  }

  float *cellon, *cellat;
  double *zang;

  double minTime=1e30, maxTime=-1e30;

  float wgt_map[180][360], ran_map[180][360], sys_map[180][360];
  for (size_t i=0; i<180; i++) {
    for (size_t j=0; j<360; j++) {
      wgt_map[i][j] = 0.0;
      ran_map[i][j] = 0.0;
      sys_map[i][j] = 0.0;
    }
  }
  
  // ============ Main loop =========
  for (ifile=0; ifile<nfiles; ifile++) {

    hid_t gid[6];
    l2file[ifile].getH5gid( l2file[ifile].getH5fid(), gid);

    
    /* Save old error handler */
    H5E_auto_t old_func;
    void *old_client_data;
    H5Eget_auto(H5E_DEFAULT, &old_func, &old_client_data);
    H5Eset_auto( H5E_DEFAULT, NULL, NULL); // Turn off error handling
    
    hid_t dsid1 = H5Dopen1( gid[3], "SSS_unc_ran");
    hid_t dsid2 = H5Dopen1( gid[3], "SSS_unc_sys");

    // Restore previous error handler
    H5Eset_auto( H5E_DEFAULT, old_func, old_client_data);

    if ( dsid1 < 0)
      cout << "SSS_unc_ran field does not exist in file: " << ifile << endl;
    if ( dsid2 < 0)
      cout << "SSS_unc_sys field does not exist in file: " << ifile << endl;
    if ( dsid1 < 0 || dsid2 < 0) continue;

    
    cout << "Binning Uncertainties for file: " << ifile << endl;

    cellon = new float[numBlks[ifile]*NUMBER_OF_BEAMS];
    cellat = new float[numBlks[ifile]*NUMBER_OF_BEAMS];
    zang = new double[numBlks[ifile]];

    l2file[ifile].readl2_nav(cellat, cellon, zang);

    // Loop over blocks
    for (iblk=0; iblk<numBlks[ifile]; iblk++) {

      // Skip if in orbit overlap region
      l2file[ifile].readl2sec( iblk, gid[1], &blkSec);
      if (blkSec <= granstart[ifile] || blkSec > granstop[ifile]) {
      	continue;
      }

      int32_t rad_flags[NUMBER_OF_BEAMS][4];
      l2file[ifile].readl2radflag( iblk, gid[2], &rad_flags[0]);

      float SSS_unc_ran[NUMBER_OF_BEAMS];
      float SSS_unc_sys[NUMBER_OF_BEAMS];
      float wgt[NUMBER_OF_BEAMS];;
      
      for (ibeam=0; ibeam<NUMBER_OF_BEAMS; ibeam++) {

        // Skip if flagged
        if ((rad_flags[ibeam][0] & flagtest[0]) != 0) continue;
        if ((rad_flags[ibeam][1] & flagtest[1]) != 0) continue;
        if ((rad_flags[ibeam][2] & flagtest[2]) != 0) continue;
        if ((rad_flags[ibeam][3] & flagtest[3]) != 0) continue;

        if (cellon[NUMBER_OF_BEAMS*iblk+ibeam] == -999) continue;
        if (cellat[NUMBER_OF_BEAMS*iblk+ibeam] == -999) continue;

        int irow = 90 - (int) (cellat[NUMBER_OF_BEAMS*iblk+ibeam]);
        int icol = (int) (cellon[NUMBER_OF_BEAMS*iblk+ibeam]) + 180;

        l2file[ifile].readl2( iblk, gid[3], "SSS_unc_ran", SSS_unc_ran);
        l2file[ifile].readl2( iblk, gid[3], "SSS_unc_sys", SSS_unc_sys);

        for (size_t i=0; i<NUMBER_OF_BEAMS; i++) {
          if ( SSS_unc_ran[i] == -9999) continue;
          if ( SSS_unc_sys[i] == -9999) continue;
          wgt[i] = sqrt(pow(SSS_unc_ran[i],2) + pow(SSS_unc_sys[i],2));
          wgt_map[irow][icol] += wgt[i];
          ran_map[irow][icol] += pow(wgt[i]*SSS_unc_ran[i],2);
          sys_map[irow][icol] += wgt[i]*abs(SSS_unc_sys[i]);
        }
        
      } // ibeam

    } // iblk

    delete[] cellon;
    delete[] cellat;
    delete[] zang;
  } // ifile

  for (size_t i=0; i<180; i++) {
    for (size_t j=0; j<360; j++) {
      if ( wgt_map[i][j] != 0.0) {
        ran_map[i][j] = sqrt(ran_map[i][j]) / wgt_map[i][j];
        sys_map[i][j] = sys_map[i][j] / wgt_map[i][j];
      } else {
        ran_map[i][j] = -32767;
        sys_map[i][j] = -32767;
      }
    }
  }
  
  hsize_t dims[2] = {180,360};
  hid_t h5fid = H5Fopen(l2binInput.ofile, H5F_ACC_RDWR, H5P_DEFAULT);
  hid_t gid = H5Gopen1(h5fid,"/");
  H5LTmake_dataset( gid, "SSS_ran_unc", 2, dims, H5T_NATIVE_FLOAT, ran_map);
  H5LTmake_dataset( gid, "SSS_sys_unc", 2, dims, H5T_NATIVE_FLOAT, sys_map);

  H5Lmove( gid, "l3m_data", gid, "SSS", H5P_DEFAULT, H5P_DEFAULT);

  H5Gclose(gid);
  H5Fclose(h5fid);
  
  return 0;
}


int parseInput( int argc, char* argv[], instr *l2binInput, string *procControl)
{
  using namespace std;

  string::size_type ipos=0;
  string            sLine, sValue;
  string            sParmWord;
  istringstream istr;
  stack<string> parmStack, procCntlStack;

  strcpy(l2binInput->pversion, "");
  strcpy(l2binInput->orbit_type, "");

  // Push regular command line parameters
  for (int i=1; i<argc; i++) {
    extParmWordValue( sLine.assign(argv[i]), &sParmWord, &sValue);

    if (sParmWord.compare("PAR") == 0) continue;
    if (sParmWord.compare("SUITE") == 0) continue;

    parmStack.push( sLine);
  }

  // Push SUITE parameters
  for (int i=1; i<argc; i++) {
    extParmWordValue( sLine.assign(argv[i]), &sParmWord, &sValue);
    if (sParmWord.compare("SUITE") == 0) parmStack.push( sLine);
  }

  // Push PAR parameters
  for (int i=1; i<argc; i++) {
    extParmWordValue( sLine.assign(argv[i]), &sParmWord, &sValue);
    if (sParmWord.compare("PAR") == 0) parmStack.push( sLine);
  }

  // Push default parameters
  sValue = "$OCDATAROOT/aquarius/l2bin_uncertainty_defaults.par";
  expandEnvVar( &sValue);
  sLine = "par=" + sValue;
  parmStack.push( sLine);

  // Pop stack loop
  while (!parmStack.empty()) {

    sLine = parmStack.top();
    parmStack.pop();
    procCntlStack.push( sLine);

    extParmWordValue( sLine, &sParmWord, &sValue);

    if (sParmWord.compare("INFILE") == 0) {
      int len = sValue.copy(l2binInput->infile, sValue.length());
      l2binInput->infile[len] = 0;
      continue;
    }

    if (sParmWord.compare("OFILE") == 0) {
      int len = sValue.copy(l2binInput->ofile, sValue.length());
      l2binInput->ofile[len] = 0;
      continue;
    }

    if (sParmWord.compare("FLAGUSE") == 0) {
      int len;
      l2binInput->flaguse[0] = ',';
      len = sValue.copy(&l2binInput->flaguse[1], sValue.length());
      l2binInput->flaguse[len+1] = ',';
      l2binInput->flaguse[len+2] = 0;
      continue;
    }


    if (sParmWord.compare("PVERSION") == 0) {
      int len = sValue.copy(l2binInput->pversion, sValue.length());
      l2binInput->pversion[len] = 0;
      continue;
    }

    if (sParmWord.compare("PAR") == 0 ||
	sParmWord.compare("SUITE") == 0) {
      ifstream parfile;
      if ( sParmWord.compare("SUITE") == 0)
	sValue = 
	  "$OCDATAROOT/aquarius/l2bin_aquarius_defaults_" + sValue + ".par";
      if ( sValue.substr( 0, 1).compare("$") == 0) expandEnvVar( &sValue);
      parfile.open ( sValue.c_str(), ifstream::in);
      if ( parfile.fail() == true) {
	cout << "Cannot open: "  <<  sValue.c_str() << endl;
	exit(1);
      }

      while( !parfile.eof()) {
	getline( parfile, sLine);
	if ( sLine.size() != 0 && sLine.substr( 0, 1).compare("#") != 0)
	  parmStack.push( sLine);
      }
      parfile.close();
      continue;
    }

    cout << endl << 
      "Parameter '" << sParmWord.c_str() << "' not defined." << endl;
    exit(1);
  }

  // Write to process control string
  while (!procCntlStack.empty()) {
    sLine = procCntlStack.top();
    procCntlStack.pop();

    extParmWordValue( sLine, &sParmWord, &sValue);
    if (sParmWord.compare("PAR") != 0 && sParmWord.compare("SUITE") != 0) {
      //      cout << sLine.c_str() << endl;
      for (size_t j=0; j<sParmWord.length(); j++)
	sParmWord[j] = tolower(sParmWord[j]);
      sParmWord = " " + sParmWord + "=";
      if ( procControl->find( sParmWord) == string::npos)
	procControl->append( " " + sLine);
    }
  }
  *procControl = procControl->substr(1);

  return 0;
}

