C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L1A_to_L2.f,v 1.33 2009/10/30 19:38:51 fore Exp $
c*****************************************************************      
      program l2gen_scatter
c*****************************************************************      
c**   
c**   FILE NAME: l2gen_scatter
c**   
c**   DATE WRITTEN: 
c**   
c**   PROGRAMMER: 
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**   11/30/2010         V3                     0.92  
c**   01/02/2011         Change NUM_BLOCKS      0.921
c**                      to last_block in
c**                      call to L2_populate_output_buffer
c**                      to fix write bug of extracts.
c**   01/04/2011         If extract then        0.922
c**                      check against first record not 
c**                      first extract block.
c**   03/18/2011         Fix NaN's due to bad   0.93
c**                      itemp decode (int hex constants)
c**   03/25/2011         Return 1 for bad cmd   0.94
c**                      line arguments
c**   04/21/2011         Update to Version 2.10 JPL code 0.95
c**   06/06/2011         Add support for non-optimal 0.96
c**                      ancillary files/Ancillary
c**                      filename output text file
c**   06/21/2011         Add process control text file 0.97
c**   08/10/2011         Update to V 1.00
c**   08/15/2011         Fix problem with all TEC files not found V 1.01
c**   09/16/2011         Implement RFI flag fix V 1.02
c**                      Update call to L2_load_RFI_flags
c**   10/19/2011         Implement new RFI code 
c**                      (JPL_scat_sw_delivery_10.17.2011)
c**                      V 1.03
c**   05/02/2012         Implement new scatterometer code
c**                      (Aquarius_scat_sw_1.3.1)
c**   09/10/2012         Implement new model functions (v1.3.2)
c**   04/02/2015         Add exponential sigma0 bias adjustment
c*****************************************************************   
      use hdf5

      implicit none
      include 'L0.inc'
      include 'L1B_instrument_module_params.inc'
      include 'L1A.inc'
      include 'L1B.inc'
      include 'L2.inc'
      include 'L1B_attitude_bias.inc'

c     PARAMETER STATEMENTS
c     INPUT VARIABLES
            
      character*320 L1A_HDF_file
      character*320 L1B_limits_file
      character*320 parameter_file
      character*320 read_sci_prefix
      character*320 emph_XYZ_file
      character*320 emph_RPY_file
      character*320 telemetry_out_directory
      character*320 L2_APC_file

      character*320 blk_limit_file

      logical flag_l1a_file_entered        /.false./
      logical flag_l1b_limits_file_entered /.false./
      logical flag_use_scale_model_patts   /.false./
      logical flag_write_l1b_output        /.false./
      logical flag_cal_level_entered       /.false./
      logical flag_param_file_entered      /.false./
      logical flag_read_sci_prefix_entered /.false./
      logical flag_ephem_XYZ_entered       /.false./
      logical flag_ephem_RPY_entered       /.false./
      logical flag_write_telem_entered     /.false./
      logical flag_suppress_tlm_warnings   /.false./
      logical flag_L2_allow_bad_entered    /.false./
      logical flag_L2_filter_rfi_entered   /.false./
      logical flag_aq_dir_dat_entered      /.false./
      logical flag_aq_dir_out_entered      /.false./
      logical flag_aq_dir_scratch_entered  /.false./
      logical flag_L2_apc_file_entered     /.false./
      logical flag_L2_no_far_corr_entered  /.false./
      
      logical flag_blk_limit_file_entered  /.false./
      logical flag_no_landice_entered      /.false./

      integer calibration_level
      
c     OUTPUT VARIABLES
c     LOCAL VARIABLES
      
      integer i_block, i_seq, i_beam, i_chan, ii ! loop iterators
      integer block_start  ! 1st valid block in blk_sec array from L1A input file
      integer ierr         ! used to trap return values
      integer i_len             ! typically used to hold the length of a string
      integer lnblnk
      integer first_block
      integer last_block

C--Variables used to parse command line arguments
      integer optind
      character*320 str_argv
      character*320 string

c--L1A Attributes
      integer NUM_XYZ_EMPH
      integer NUM_ATT_EMPH
      integer NUM_BLOCKS
      character*4 L1A_data_type

C--L1A telemetry.  We read the L0 telem from L1A file, 
C--convert to EU, and populate these arrays
      real*4    external_temperatures_eu(      38, MAX_NUM_BLOCKS )
      real*4    scat_telemetry_eu(             38, MAX_NUM_BLOCKS )
      real*4    icds_engineering_telemetry_eu( 24, MAX_NUM_BLOCKS )
      integer*4 icds_processing_status_eu(      4, MAX_NUM_BLOCKS )
      
      integer*4 icds_blknum

      real*8    ext_temps( 38 )
      real*8    scat_tlm( 38 ) 
      
C--L1A Block time tags      
      real*8  blk_sec(         MAX_NUM_BLOCKS )

C--L0 time tags; read in from L1A file to check blk_sec time tags.
      integer gps_time_tag(    MAX_NUM_BLOCKS )
      integer time_tag_offset( MAX_NUM_BLOCKS )
      
c--L1A times
      integer start_year
      integer start_day
      integer start_millisec
      integer end_year
      integer end_day
      integer end_millisec
      integer orbit_number
      real*4  orbit_node_longitude

C--Read in from input file(s) ( L1A or read_sci )
      real*8 RX_power( 6, 3, 8, MAX_NUM_BLOCKS ) ! pol, beam, seq, block
      real*8 LB_power( 6, 3,    MAX_NUM_BLOCKS ) ! pol, beam,      block
      real*8 DC_offset(      2, MAX_NUM_BLOCKS ) ! pol,            block
      integer*1 L0_scat_headers(8, MAX_NUM_BLOCKS )

      integer*1 RFI_FLAGS_VPOL(3,8,MAX_NUM_BLOCKS) ! set to 1 if RFI, 0 if no RFI
      integer*1 RFI_FLAGS_HPOL(3,8,MAX_NUM_BLOCKS) ! set to 1 if RFI, 0 if no RFI

C--Filtered Noise Power
      real*8    HRX_Noise_Power( 3, 8, MAX_NUM_BLOCKS )
      real*8    VRX_Noise_Power( 3, 8, MAX_NUM_BLOCKS )

C--Populated block by block for L1A -> L1B processing
      real*8    L1B_time                  ! time in julian days [days] (units of days)
      real*8    L1B_xyz_mj2k(3)           ! spacecraft position, in mj2k coordinates [m]
      real*8    L1B_vxyz_mj2k(3)          ! spacecraft velocity, in mj2k coordinates [m/s]
      real*8    L1B_roll                  ! sc attitude, roll [deg]
      real*8    L1B_pitch                 ! sc attitude, pitch [deg]
      real*8    L1B_yaw                   ! sc attitude, yaw [deg]
      real*8    L1B_KPC(4,3,8)            ! KPC estimates per measurement [linear]
      real*8    L1B_SNR(4,3,8)            ! Signal noise ratio estimates per measurement [dB]
      real*8    L1B_Noise(2,3,8)          ! Noise-equivlent sigma0 ( {V,H}, beam, seq ) [dB]
      real*8    L1B_Noise_Pwr(2,3,8) ! Noise RX Power ( {V,H}, beam, seq ) [dBm]
      real*8    L1B_sig0(4,3,8)           ! Sigma 0  ({HV,VV,VH,HH},beam,seq) [dB]
      real*8    L1B_lat_echo_center(3,8)  ! beam center latitude [deg]
      real*8    L1B_lon_echo_center(3,8)  ! beam center longitude [deg]
      real*8    L1B_lat_beam_edge(4,3)    ! latitude of beam edge points (4 for each beam) [deg]
      real*8    L1B_lon_beam_edge(4,3)    ! longitude of beam edge points [deg]
      real*8    L1B_ant_azi_rel_north(3)  ! antenna azimuth angle; measured clockwise from north [deg]
      real*8    L1B_pol_roll_angle(3)     ! polarization roll angle [deg]
      real*8    L1B_inc_angle(3)          ! beam incidence angle [deg]
      real*8    L1B_land_fraction(3)      ! land fraction (0-no land, 1-all land) for each beam
      real*8    L1B_ice_fraction(3)       ! ice fraction (0-no ice, 1-all ice) for each beam
      real*8    L2_polarization_roll(3)     ! polarization roll angle for each beam [deg]
      real*4    L1B_housekeeping(35)
      integer*2 L1B_quality_flag(4,3,8)   ! Quality flag at L1B level (see documentation)

C--L2 Quantites; computed from L1B quantities at the end of i_block loop
      real*8    L2_time                     ! time in julian days [days] (units of days)
      real*8    L2_scat_wind_speed(3)       ! retrived wind speed [m/s]  (beam index)
      real*8    L2_scat_wind_speed_err(3)   ! one-sigma wind speed error estimates [m/s] (beam index)
      real*8    L2_faraday_rotation_angle(3)! Estimated faraday rotation angle used for faraday rotation corrections [deg]
      real*8    L2_tec_scale_factor         ! VTEC scale factor 
      real*8    L2_excess_surf_emiss(2,3) ! Excess surface emissivity due to wind; (order V,H; i_beam)
      real*8    L2_excess_surf_emiss_err(2,3) ! error Excess surface emissivity due to wind; (order V,H; i_beam)

      real*8    L2_sigma_ANT(4,3)           ! antenna sigma 0
      real*8    L2_var_sigma_ANT(4,3)       ! variance of antenna sigma 0
      real*8    L2_KPC_ANT(4,3)             ! KPC for antenna sigma 0
      
      real*8    L2_sigma_TOI(4,3)           ! top of ionosphere sigma 0 
      real*8    L2_var_sigma_TOI(4,3)       ! variance of top of ionosphere sigma 0

      real*8    L2_sigma_TOA(4,3)           ! top of atmosphere sigma 0 
      real*8    L2_var_sigma_TOA(4,3)       ! variance of top of atmosphere sigma 0      
      real*8    L2_KPC_TOA(4,3)             ! KPC for top of atmosphere sigma 0

      real*8    L2_sigma_Far(4,3)           ! Correction to Sigma0 due to faraday rotation
      real*8    L2_sigma_exp(4,3)           ! Expected sigma0

      real*8    L2_tot_sigma_TOA(3)         ! top of atmosphere total sigma 0
      real*8    L2_tot_KPC_TOA(3)           ! KPC for top of atmosphere total sigma 0
      real*8    L2_var_tot_sigma_TOA(3)     ! variance of top of atmosphere total sigma 0

      real*8    L2_beam_clat(3)             ! beam center latitude [deg]
      real*8    L2_beam_clon(3)             ! beam center longitude [deg]
      real*8    L2_beam_elat(4,3)           ! latitude of beam edge points (4 for each beam) [deg]
      real*8    L2_beam_elon(4,3)           ! longitude of beam edge points [deg]
      real*8    L2_land_fraction(3)         ! land fraction (0-no land, 1-all land) for each beam
      real*8    L2_ice_fraction(3)          ! ice fraction (0-no ice, 1-all ice) for each beam
      integer*4 L2_scatterometer_flags(3)   ! L2 Quality flags
      integer*4 L2_scat_samples(3)          ! Number of scat obs used per block and beam
      integer*1 L2_scat_rfi_flags(3,2)      ! beam, RX-polarization
      logical   L2_good_far_corr(3)         ! logical flag for goodness of faraday rotation correction
      logical   L2_neg_sig0_ANT(4,3)        ! logical for neg sig0 ANT (chan,beam)
      logical   L2_neg_sig0_TOA(4,3)        ! logical for neg sig0 TOA (chan,beam)
      logical   L2_off_nom_no_echo(3)       ! logical for pointing off-nominal without echo (beam)
      
C--Temporaries need to compute faraday rotation angle
      real*8 sc_position_block_start(3)      ! x,y,z ECF components
      real*8 sc_lon_block_start              ! spacecraft longitude at block start
      real*8 sc_lat_block_start              ! spacecraft latitude at block start
      real*8 sc_alt_block_start              ! spacecraft altitude at block start
      real*8 earth_position_block_start(3,3) ! 1st index x,y,z component; 2nd index beam (1,2,3)
      real*8 date(6)                         ! year,month,day,hour,minute,seconds
      integer i_year, i_month, i_day, i_hour, i_minute, i_second
      real*8 r_frac_seconds

      real*8 julian_time   ! julian_time (updated every sequence 0.18 sec) [day]
      real*8 julian_time_0 ! julian_time at 0 UTC [day]
      real*8 start_JD      ! used for read_sci inputs; controls temporal interval of cubic splines
      real*8 end_JD        ! used for read_sci inputs; controls temporal interval of cubic splines
      real*8 time_start
      real*8 time_end
      real*8 time_ref
      real*8 block_time    ! UTC second-of-day at start of current block [sec]
      real*8 sequence_time ! UTC second-of-day at start of current sequence [sec]


C--All of the below variables are used for processing the data; their values 
C--will change throughout the orbit.
      real*8 current_spline(10) ! contains (t,x,y,z,vx,vy,vz,y,p,r) @ sequence_time
      real*8 yaw_deg
      real*8 pitch_deg
      real*8 roll_deg
      
      real*8 sc_position(3)
      real*8 sc_velocity(3)
      real*8 sc_position_mj2k(3)
      real*8 sc_velocity_mj2k(3)
      real*8 spacecraft_nadir_vector(3)
      real*8 spacecraft_lon
      real*8 spacecraft_lat
      real*8 prev_spacecraft_lat
      real*8 spacecraft_altitude
            
      real*8 r_KPC(4)
      real*8 r_SNR(4)
      real*8 r_sig0(4)
      real*8 r_VN_sig0, r_HN_sig0
      real*8 r_Hnoise, r_Vnoise  

      real*8 antenna_pointing_vector_rect(3)
      real*8 cell_azimuth_angle_deg(3)
      real*8 incidence_angle(3)
      real*8 polarization_roll_angle(3) 
      
      real*8 xnormal(3), h_vector(3), v_vector(3)
      real*8 epsilon1(3), rot_epsilon1(3)
      real*8 antlook(3), antazi
      real*8 beamw1V, beamw2V, beamw1H, beamw2H
      real*8 areaV(3), afootV(3), bfootV(3), diffV(3)
      real*8 areaH(3), afootH(3), bfootH(3), diffH(3)
      real*8 Rslant, ainc      
      real*8 vector_in(3)
      real*8 slant_range(3)
      real*8 earth_position(3)
      real*8 latitude_geocentric_deg
      real*8 longitude_deg
      real*8 latitude_deg
      
      real*8 land_fraction(3), ice_fraction(3)
      
      real*8 latitude_points(4), longitude_points(4)
      
      real*8 dotprod, dotprod2
      
      real*8 r_DCoff_power
      real*8 r_rx_power(10)
      real*8 r_sgnl_power(4)
      real*8 r_xint_inv(4)              ! geometric and beam calibration factors
      real*8 r_Xcal(6,3), r_xcal_inv(4) ! instrument calibration factors
      real*8 r_TBd
      
      real*8 r_total_sig0_dB
      real*8 wind_speed_sig0_plus_kpc
      real*8 wind_speed_sig0_minus_kpc
      real*8 tot_sig0_plus_kpc
      real*8 tot_sig0_minus_kpc
      real*8 sig0_vv_plus_kpc
      real*8 sig0_hh_plus_kpc
      real*8 sig0_vv_minus_kpc
      real*8 sig0_hh_minus_kpc

      real*8 wind_dir_ancilliary(3)
      real*8 wind_speed_ancilliary(3)
      real*8 relazang
      integer i_sigwave
      
      real*8 sum_sin_lat, sum_cos_lat ! used to average L1B geolocation to L2
      real*8 sum_sin_lon, sum_cos_lon ! used to average L1B geolocation to L2
    
      real*4 bias_adj(3,2)      ! beam:(1,2,3), polarization: (HH,VV)  

C--Logicals used to track conditions during L1B processing; some may affect L1B flagging
      logical good_solution
      logical bad_time_tag_flag
      logical bad_block_id_flag
      logical unable_to_geolocate_flag
      logical beam_off_earth_flag(3)
      logical bad_kfactor_flag(3)
      logical ascending

      integer*2 L2_bitmask ! Used to exclude observations from L2 averages
      integer*2 L2_bitmask_temp
      logical L2_all_rfi
      logical L2_include_sequence

C--KFactor related variables...
      integer dir
      real*8 kfactors(3)
      
C--Antenna pattern related variables.
      character*200 antenna_file_name_in, antenna_dir
      real*8 r_max_gain(2) ! for compute_xcal
      real*8 max_gain(3,2)
      real*8 antenna_pattern_relative_1h(360,721,6)
      real*8 antenna_pattern_relative_1v(360,721,6)
      real*8 antenna_pattern_relative_2h(360,721,6)
      real*8 antenna_pattern_relative_2v(360,721,6)
      real*8 antenna_pattern_relative_3h(360,721,6)
      real*8 antenna_pattern_relative_3v(360,721,6)


C--Expanded/Diminished landmask related variables.
      character*200 landmask_expand_diminish_file

      character*320 L2_prefix

c     DATA STATEMENTS
      real*8 rho_HHVV(3), f_HVHH(3), f_HVVV(3)
!      data rho_HHVV/0.997,0.991,0.988/
!      data f_HVHH/0.004102041029866,0.007888601176186,0.011376272858234/
!      data f_HVVV/0.002275097430772,0.003090295432514,0.003483373150360/

! From real sigma ANT L2 data--10/6/2011
      data rho_HHVV/0.986137,0.960105,0.951029/
      data f_HVHH/0.011128,0.037563,0.090092/
      data f_HVVV/0.006899,0.015065,0.020984/

      real*8 half_beam_width_ideal_1V(3), half_beam_width_ideal_2V(3)
      real*8 half_beam_width_ideal_1H(3), half_beam_width_ideal_2H(3)

      data half_beam_width_ideal_1V/3.38d0,3.55d0,3.79d0/ ! From scale_model_data_report_V4.ppt
      data half_beam_width_ideal_2V/3.09d0,3.20d0,3.29d0/ ! From scale_model_data_report_V4.ppt
      data half_beam_width_ideal_1H/3.41d0,3.58d0,3.64d0/ ! From scale_model_data_report_V4.ppt
      data half_beam_width_ideal_2H/3.11d0,3.18d0,3.27d0/ ! From scale_model_data_report_V4.ppt
      
      real*8 half_beam_width_meas_1V(3), half_beam_width_meas_2V(3)
      real*8 half_beam_width_meas_1H(3), half_beam_width_meas_2H(3)
      
      data half_beam_width_meas_1V/3.4096d0,3.5419d0,3.6605d0/ ! From scale_model_data_report_V4.ppt
      data half_beam_width_meas_2V/3.0898d0,3.1705d0,3.2711d0/ ! From scale_model_data_report_V4.ppt
      data half_beam_width_meas_1H/3.4670d0,3.5748d0,3.6312d0/ ! From scale_model_data_report_V4.ppt
      data half_beam_width_meas_2H/3.0619d0,3.1686d0,3.3128d0/ ! From scale_model_data_report_V4.ppt
      
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level

      real*8 r_LB_expected(6)
      real*8 r_tx_pow_expected(6)
      real*8 r_tx_loss_expected(6)
      real*8 r_rx_loss_expected(6)
      real*8 r_rx_chain_expected(6)
      real*8 r_Tsys_expected(6)
      real*8 r_CND_chain(6)
      common /other_instrument/ r_LB_expected, r_tx_pow_expected, r_tx_loss_expected,
     &   r_rx_loss_expected, r_rx_chain_expected, r_Tsys_expected, r_CND_chain


      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT,
     $ AQ_SCAT_SIM_DIR_SCRATCH
     
      real*8 earth_radius_eq, earth_radius_pole
      real*8 earth_eccentricity, earth_eccentricity_sq, one_minus_earth_eccentricity_sq
      real*8 one, two, one_eighty, rad_to_deg, deg_to_rad, pi, R_SOL, wavelength
      common /block/earth_radius_eq, earth_radius_pole,
     &     earth_eccentricity, earth_eccentricity_sq, one_minus_earth_eccentricity_sq,
     &     one, two, one_eighty, rad_to_deg, deg_to_rad, pi, R_SOL, wavelength      
          
c     FUNCTION STATEMENTS
      real*8 compute_dot_product
      integer iargc
      integer get_L1A_splines     
      integer get_READ_SCI_spline
      integer L1A_check_time_tags

c     PROCESSING STEPS

      write(*,*) 'l2gen_scatter 1.06 ('//__DATE__//' '//__TIME__//')'
      write(*,*) ''

C--Defaults----      
      i_dbg_level         = 0
      ascending           = .true.
      prev_spacecraft_lat = -999
      calibration_level   = 0
      
      call initialize_constants

C--Parse command-line arguments      
      optind = 1
      call getarg(optind,str_argv)
      do while( optind .le. iargc() .and. str_argv(1:1) .eq. '-')
        i_len = lnblnk(str_argv)

        if( str_argv(1:i_len) .eq. '-debug' )then
          optind = optind + 1
          call getarg(optind, string )
          read(string,*) i_dbg_level
        
        elseif( str_argv(1:i_len) .eq. '-param_file' )then
          flag_param_file_entered = .true.
          optind = optind + 1
          call getarg(optind, parameter_file )
          
        elseif( str_argv(1:i_len) .eq. '-limits' )then
          flag_l1b_limits_file_entered = .true.
          optind = optind + 1
          call getarg(optind, L1B_limits_file )

        elseif( str_argv(1:i_len) .eq. '-i' )then
          flag_l1a_file_entered = .true.
          optind = optind + 1
          call getarg(optind, L1A_HDF_file )

        elseif( str_argv(1:i_len) .eq. '-blk_lim_file' )then
           flag_blk_limit_file_entered = .true.
          optind = optind + 1
          call getarg(optind, blk_limit_file )
!          write(*,*) 'flag_blk_limit'
 
        elseif( str_argv(1:i_len) .eq. '-sm_patts' )then
          flag_use_scale_model_patts = .true.
        
        elseif( str_argv(1:i_len) .eq. '-l1b' )then
          flag_write_l1b_output = .true.
        
        elseif( str_argv(1:i_len) .eq. '-cal_level' )then
          flag_cal_level_entered = .true.
          optind = optind + 1
          call getarg(optind, string )
          read(string,*) calibration_level
        
        elseif( str_argv(1:i_len) .eq. '-read_sci_prefix' )then
          flag_read_sci_prefix_entered = .true.
          optind = optind + 1
          call getarg(optind, read_sci_prefix )
          
        elseif( str_argv(1:i_len) .eq. '-ephem_XYZ' .or. 
     $          str_argv(1:i_len) .eq. '-emph_XYZ' )then
          flag_ephem_XYZ_entered = .true.
          optind = optind + 1
          call getarg(optind, emph_XYZ_file )          

        elseif( str_argv(1:i_len) .eq. '-ephem_RPY' .or.
     $          str_argv(1:i_len) .eq. '-emph_RPY' )then
          flag_ephem_RPY_entered = .true.
          optind = optind + 1
          call getarg(optind, emph_RPY_file )          

        elseif( str_argv(1:i_len) .eq. '-write_telem' )then
          flag_write_telem_entered = .true.
          optind = optind + 1
          call getarg(optind, telemetry_out_directory )          

        elseif( str_argv(1:i_len) .eq. '-suppress_tlm_warnings' )then
          flag_suppress_tlm_warnings = .true.

        elseif( str_argv(1:i_len) .eq. '-L2_allow_bad' )then
          flag_L2_allow_bad_entered = .true.

        elseif( str_argv(1:i_len) .eq. '-L2_filter_rfi' )then
          flag_L2_filter_rfi_entered  = .true.

        elseif( str_argv(1:i_len) .eq. '-no_landice' )then
          flag_no_landice_entered  = .true.
        
        elseif( str_argv(1:i_len) .eq. '-dir_dat' )then
          flag_aq_dir_dat_entered = .true.
          optind = optind + 1
          call getarg(optind, AQ_SCAT_SIM_DIR_DAT )          
        
        elseif( str_argv(1:i_len) .eq. '-dir_out' )then
          flag_aq_dir_out_entered = .true.
          optind = optind + 1
          call getarg(optind, AQ_SCAT_SIM_DIR_OUT )          

        elseif( str_argv(1:i_len) .eq. '-dir_scratch' )then
          flag_aq_dir_scratch_entered = .true.
          optind = optind + 1
          call getarg(optind, AQ_SCAT_SIM_DIR_SCRATCH )          

       elseif( str_argv(1:i_len) .eq. '-apc_file' )then
          flag_L2_apc_file_entered = .true.
          optind = optind + 1
          call getarg(optind, L2_APC_file )          

        elseif( str_argv(1:i_len) .eq. '-no_far_corr' )then
          flag_L2_no_far_corr_entered = .true.     
   
        endif
        optind = optind + 1
        call getarg(optind,str_argv)
      enddo      

C--If these were not entered on the command-line, try to get them
C--from the enviornment variables. 
C--This is implemented for backward compatability with some scripts.
      if( .not.flag_aq_dir_dat_entered .and.
     $    .not.flag_aq_dir_out_entered .and.
     $    .not.flag_aq_dir_scratch_entered ) then

        call getenv( 'AQ_SCAT_SIM_DIR_DAT', AQ_SCAT_SIM_DIR_DAT )
        call getenv( 'AQ_SCAT_SIM_DIR_OUT', AQ_SCAT_SIM_DIR_OUT )
        call getenv( 'AQ_SCAT_SIM_DIR_SCRATCH', AQ_SCAT_SIM_DIR_SCRATCH )

        if( lnblnk(AQ_SCAT_SIM_DIR_DAT).eq.0 .or.
     $      lnblnk(AQ_SCAT_SIM_DIR_OUT).eq.0 .or.
     $      lnblnk(AQ_SCAT_SIM_DIR_SCRATCH).eq.0 ) then
          call L1A_to_L2_usage
          stop
        else
          write(6,*) 'Next time please use the -dir_dat, -dir_out, and -dir_scratch options.'
        endif
      endif
      
      if( .not.( flag_l1a_file_entered                        .or. 
     $         ( flag_ephem_RPY_entered                        .and. 
     $           flag_ephem_XYZ_entered                        .and. 
     $           flag_read_sci_prefix_entered ) )             .or. 
     $    .not.flag_param_file_entered                        .or. 
     $     calibration_level.lt.1 .or. calibration_level.gt.3 .or.
     $    .not.flag_l1b_limits_file_entered  )then
        call L1A_to_L2_usage
        stop
      endif

C--Check L2 filter command line arguments and write to stdout for log files
C--(please redirect stdout to log file when running!!!)
      if( flag_L2_allow_bad_entered ) then
        write(6,*) 'Not filtering data flagged as BAD_DATA for L2 averaging.'
      else
        write(6,*) 'Filtering data flagged as BAD_DATA for L2 averaging.'
      endif
      if( flag_L2_filter_rfi_entered ) then
        write(6,*) 'Filtering data flagged as RFI for L2 averaging.'
      endif

C--Construct bitmask for L2 filtering.
      L2_bitmask = z"0000"
      call L1B_set_flag( L2_bitmask, L1B_QF_GEOLOCATION_ERROR, L1B_QF_GEOLOCATION_ERROR )
      call L1B_set_flag( L2_bitmask, L1B_QF_BEAM_OFF_EARTH, L1B_QF_BEAM_OFF_EARTH )
      call L1B_set_flag( L2_bitmask, L1B_QF_POINTING_ERROR, L1B_QF_POINTING_ERROR )
      if( .not.flag_L2_allow_bad_entered ) 
     $  call L1B_set_flag( L2_bitmask, L1B_QF_BAD_DATA, L1B_QF_BAD_DATA )
      if( flag_L2_filter_rfi_entered )     
     $  call L1B_set_flag( L2_bitmask, L1B_QF_RFI, L1B_QF_RFI )
       
      write(6,*) 'Using L2_bitmask: ', L2_bitmask

C--Read the Limits file and parameters file.
      call L1B_read_limits_file( L1B_limits_file )     
      call L1B_read_params(      parameter_file  )

C-----We expect the antenna patterns to be in the directory antenna_dir   
      write(6,*) 'READING ANTENNA FILE PATTERNS'
      i_len = lnblnk(AQ_SCAT_SIM_DIR_DAT)
      if( .not.flag_use_scale_model_patts )then
        antenna_dir=AQ_SCAT_SIM_DIR_DAT(1:i_len) // '/beam_patterns/theory'
      
        i_len = lnblnk(antenna_dir)
        antenna_file_name_in=antenna_dir(1:i_len)//'/cuts_n1h_1260MHz_big.cut'
        call read_jpl_cut_antenna_pattern_relative(antenna_file_name_in, 
     $   antenna_pattern_relative_1h,max_gain(1,1))
     
        antenna_file_name_in=antenna_dir(1:i_len)//'/cuts_n1v_1260MHz_big.cut'
        call read_jpl_cut_antenna_pattern_relative(antenna_file_name_in, 
     $   antenna_pattern_relative_1v,max_gain(1,2))
     
        antenna_file_name_in=antenna_dir(1:i_len)//'/cuts_p0h_1260MHz_big.cut'
        call read_jpl_cut_antenna_pattern_relative(antenna_file_name_in, 
     $   antenna_pattern_relative_2h,max_gain(2,1))
     
        antenna_file_name_in=antenna_dir(1:i_len)//'/cuts_p0v_1260MHz_big.cut'
        call read_jpl_cut_antenna_pattern_relative(antenna_file_name_in, 
     $   antenna_pattern_relative_2v,max_gain(2,2))

        antenna_file_name_in=antenna_dir(1:i_len)//'/cuts_p1h_1260MHz_big.cut'
        call read_jpl_cut_antenna_pattern_relative(antenna_file_name_in, 
     $   antenna_pattern_relative_3h,max_gain(3,1))

        antenna_file_name_in=antenna_dir(1:i_len)//'/cuts_p1v_1260MHz_big.cut'
        call read_jpl_cut_antenna_pattern_relative(antenna_file_name_in, 
     $   antenna_pattern_relative_3v,max_gain(3,2))
      else
        antenna_dir=AQ_SCAT_SIM_DIR_DAT(1:i_len) // '/beam_patterns/scale_model'
        i_len = lnblnk(antenna_dir)
        antenna_file_name_in=antenna_dir(1:i_len)//'/n1h_12431_Mhz.txt'
        call read_alex_groupD_antenna_pattern_relative(antenna_file_name_in, 
     $   antenna_pattern_relative_1h,max_gain(1,1))
     
        antenna_file_name_in=antenna_dir(1:i_len)//'/n1v_12431_Mhz.txt'
        call read_alex_groupD_antenna_pattern_relative(antenna_file_name_in, 
     $   antenna_pattern_relative_1v,max_gain(1,2))
     
        antenna_file_name_in=antenna_dir(1:i_len)//'/p0h_12431_Mhz.txt'
        call read_alex_groupD_antenna_pattern_relative(antenna_file_name_in, 
     $   antenna_pattern_relative_2h,max_gain(2,1))
     
        antenna_file_name_in=antenna_dir(1:i_len)//'/p0v_12431_Mhz.txt'
        call read_alex_groupD_antenna_pattern_relative(antenna_file_name_in, 
     $   antenna_pattern_relative_2v,max_gain(2,2))

        antenna_file_name_in=antenna_dir(1:i_len)//'/p1h_12431_Mhz.txt'
        call read_alex_groupD_antenna_pattern_relative(antenna_file_name_in, 
     $   antenna_pattern_relative_3h,max_gain(3,1))

        antenna_file_name_in=antenna_dir(1:i_len)//'/p1v_12431_Mhz.txt'
        call read_alex_groupD_antenna_pattern_relative(antenna_file_name_in, 
     $   antenna_pattern_relative_3v,max_gain(3,2))
      endif      

C--Find the edge points in the beam coordinate system. (this
C--subroutine just sets up a common block that the 
C--subroutine points_edge has access to.
      call find_edge_points( antenna_pattern_relative_1h,
     $ antenna_pattern_relative_1v, antenna_pattern_relative_2h,
     $ antenna_pattern_relative_2v, antenna_pattern_relative_3h,
     $ antenna_pattern_relative_3v )
      
      write(6,*) 'Reading K Factors.'
      call read_k_factors(flag_use_scale_model_patts)

      write(6,*) 'Generating cubic splines.'
      if( flag_l1a_file_entered ) then
        call gen_L1A_splines( L1A_HDF_file, time_start, time_end )

C--Read data from the L1A HDF file...
C--Null terminate the hdf file name for the call to the C HDF5 reader

C--Check the L1A data type attribute
        ierr = L1A_hdf5_get_data_type( L1A_data_type,
     $             L1A_HDF_file( 1:lnblnk(L1A_HDF_file) ) // char(0) )
        if(ierr.lt.0) write(6,*) 'ERROR: L1A_hdf5_get_data_type returned: ', ierr
      
        if( L1A_data_type(1:3) .ne. 'SCI' .and.
     $      L1A_data_type(1:3) .ne. 'SIM' )then
          write(6,*) 'ERROR: L1A data not of type SCI or SIM, quitting: ', L1A_data_type(1:3)
          call exit(1)
        endif
      
        ierr = L1A_HDF5_get_sizes( NUM_BLOCKS, 
     $                             NUM_XYZ_EMPH, 
     $                             NUM_ATT_EMPH, 
     $                             L1A_HDF_file( 1:lnblnk(L1A_HDF_file) ) // char(0) )
        if(ierr.lt.0) write(6,*) 'ERROR: L1A_HDF5_get_sizes returned: ', ierr
      
        ierr = L1A_HDF5_read_gps_time_tag( gps_time_tag, 
     $                                     time_tag_offset,
     $                                     L1A_HDF_file( 1:lnblnk(L1A_HDF_file) ) // char(0) )
        if(ierr.lt.0) write(6,*) 'ERROR: L1A_HDF5_read_gps_time_tag returned: ', ierr
      
        ierr = L1A_HDF5_get_times( start_year, start_day, start_millisec,
     $   end_year, end_day, end_millisec, orbit_node_longitude,
     $   L1A_HDF_file( 1:lnblnk(L1A_HDF_file) ) // char(0) )
        if(ierr.lt.0) then
           write(6,*) 'ERROR: L1A_HDF5_get_times returned: ', ierr
           call exit(1)
        endif

        ierr = L1A_HDF5_read_block_times( blk_sec, 
     $   L1A_HDF_file( 1:lnblnk(L1A_HDF_file) ) // char(0) )
        if(ierr.lt.0) then 
           write(6,*) 'ERROR: L1A_HDF5_read_block_times returned: ', ierr
           call exit(1)
        endif

        call L1A_load_scat_sci( L1A_HDF_file, 
     $                          RX_power, 
     $                          LB_power,
     $                          DC_offset,
     $                          L0_scat_headers )
      
        call L1A_load_telemetry( L1A_HDF_file,
     $                           external_temperatures_eu,
     $                           scat_telemetry_eu,
     $                           icds_engineering_telemetry_eu,
     $                           icds_processing_status_eu )
      
        call L1A_load_RFI_flags( L1A_HDF_file, RFI_FLAGS_VPOL,
     $         RFI_FLAGS_HPOL )

      else if( flag_read_sci_prefix_entered) then
        
        call READ_SCI_read_block_times( read_sci_prefix,
     $                                  NUM_BLOCKS,
     $                                  start_year,
     $                                  start_day,
     $                                  start_JD,
     $                                  end_JD,
     $                                  blk_sec          )

        call gen_READ_SCI_splines( emph_XYZ_file,
     $                             emph_RPY_file,
     $                             start_JD,
     $                             end_JD,
     $                             time_ref       )
        
        call READ_SCI_load_all( read_sci_prefix,
     $                          RX_power, 
     $                          LB_power,
     $                          DC_offset,
     $                          L0_scat_headers,              
     $                          external_temperatures_eu,
     $                          scat_telemetry_eu,
     $                          icds_engineering_telemetry_eu,
     $                          icds_processing_status_eu,
     $                          RFI_FLAGS_VPOL,
     $                          RFI_FLAGS_HPOL )
        
      endif ! if( flag_l1a_file_entered ) then
      
      call L1A_Filter_RFI( RX_power,
     $                     RFI_FLAGS_VPOL,
     $                     RFI_FLAGS_HPOL,
     $                     NUM_BLOCKS,
     $                     VRX_Noise_Power,
     $                     HRX_Noise_Power )

      
C--Compute the exponential sigma0 bias adjustment
      call sigma0_bias_adj(start_year, start_day, bias_adj)

C--Find Julian time at 0 UTC on start_day in start_year
      write(*,*) 'start_year: ', start_year
      call find_julian_day( start_year, 1, 1, julian_time_0 )
      julian_time_0 = julian_time_0 + 1.0d0 * (start_day-1)
      
C--Find the index (i_block) of the first orbit position...
      i_block = 1      
      do while( blk_sec(i_block) < 0 )
        i_block = i_block + 1
      enddo
      block_start = i_block
      
      write(6,*) 'First orbit position: ', block_start, blk_sec(block_start)

      if( flag_write_telem_entered )then ! write converted L0 data.
        call write_telem_eu( telemetry_out_directory,
     $                       external_temperatures_eu,
     $                       scat_telemetry_eu,
     $                       icds_engineering_telemetry_eu,
     $                       icds_processing_status_eu,     
     $                       blk_sec,     
     $                       RX_power,     
     $                       LB_power,          
     $                       DC_offset,      
     $                       julian_time_0,
     $                       block_start,
     $                       NUM_BLOCKS      )
      endif

C--Generate expanded/diminished landmask


C--Set block loop limits
      first_block = 1
      last_block = NUM_BLOCKS
      if ( flag_blk_limit_file_entered) then
         open(25,file=blk_limit_file,status='old')
         read(25, 250) first_block
         read(25, 250) last_block
 250     format(i5)
         close(25)
         block_start = first_block
      endif

      write(*,*) 'first_block: ', first_block
      write(*,*) 'last_block:  ', last_block

      L2_prefix = L1A_HDF_file(scan(L1A_HDF_file, '/', back=.true.)+1:
     $                         len(L1A_HDF_file))
      L2_prefix = L2_prefix(1:scan(L2_prefix, '.', back=.true.)-1)
      L2_prefix = trim(L2_prefix)
      write(*,*) L2_prefix(1:lnblnk(L2_prefix))

      open(35,file=L2_prefix(1:lnblnk(L2_prefix))//'_L2_ancillary_files.txt',status='unknown')

c--Write process control string file
      optind = 1
      open(36,file=L2_prefix(1:lnblnk(L2_prefix))//'_L2_process_control.txt',status='unknown')
      do while( optind .le. iargc())
         call getarg(optind,str_argv)
         i_len = lnblnk(str_argv)
         write(36,'(a,$)') str_argv(1:i_len+1)
         optind = optind + 1
      enddo
      close(36)


C -- Main Loop --
C--Loop over the number of blocks.
      do i_block = first_block, last_block

        if ( MOD(i_block, 100).eq.0) then
           write(*,*) 'i_block: ', i_block, ' out of ', NUM_BLOCKS
        endif
!        write(*,*) 'i_block: ', i_block, ' out of ', NUM_BLOCKS

        bad_time_tag_flag   = .false.
        bad_block_id_flag   = .false.
C--Initialize L1B buffers at start of block----------------------
        L1B_time  = 0.0d0
        L1B_roll  = 0.0d0
        L1B_pitch = 0.0d0
        L1B_yaw   = 0.0d0
        
        do ii = 1, 35
          L1B_housekeeping(ii) = 0.0
        enddo
        
        do ii = 1, 3
          L1B_xyz_mj2k(ii)          = 0.0d0
          L1B_vxyz_mj2k(ii)         = 0.0d0
          L1B_ant_azi_rel_north(ii) = 0.0d0
          L1B_pol_roll_angle(ii)    = 0.0d0
          L1B_inc_angle(ii)         = 0.0d0
          L1B_land_fraction(ii)     = 0.0d0
          L1B_ice_fraction(ii)      = 0.0d0
        enddo
        
        do i_seq = 1, 8
          do i_beam = 1, 3
            L1B_lat_echo_center(i_beam,i_seq) = 0.0d0
            L1B_lon_echo_center(i_beam,i_seq) = 0.0d0
            
            do i_chan = 1, 4
              L1B_KPC(i_chan,i_beam,i_seq)          = -999.0 ! dB
              L1B_SNR(i_chan,i_beam,i_seq)          = -999.0 ! dB
              L1B_sig0(i_chan,i_beam,i_seq)         = -999.0 ! dB
              L1B_quality_flag(i_chan,i_beam,i_seq) = 0
              
              if( i_seq.eq.1)then
                L1B_lat_beam_edge(i_chan,i_beam) = 0.0d0
                L1B_lon_beam_edge(i_chan,i_beam) = 0.0d0
              endif
            enddo
            
            do ii = 1, 2
              L1B_Noise(ii,i_beam,i_seq) = -999.0 ! dB
              L1B_Noise_Pwr(ii,i_beam,i_seq) = -999.0 ! dB
            enddo
          enddo
        enddo
C--DONE Initializing L1B buffers----------------------------------
        
C--Check blk_sec block time tags
        if( flag_l1a_file_entered ) then
          call L1A_check_block_timetags( blk_sec,
     $                                   gps_time_tag(    i_block ),
     $                                   time_tag_offset( i_block ),
     $                                   icds_processing_status_eu,
     $                                   i_block,
     $                                   block_start,
     $                                   NUM_BLOCKS,
     $                                   bad_block_id_flag,
     $                                   bad_time_tag_flag )
        endif
        
        if( blk_sec(i_block) .lt. 0 ) then   ! Bogus time tag
          bad_time_tag_flag        = .true.  ! set bad_time_tag_flag
          unable_to_geolocate_flag = .true.  ! can't geolocate with a bogus time tag
        endif
        
C--Load L1B housekeeping temperatures from converted telemetry.
        call L1B_load_housekeeping( i_block,
     $                              external_temperatures_eu,
     $                              scat_telemetry_eu,
     $                              L1B_housekeeping )
                
C--Copy the scat and external temperatures telemetry to a R*8 (they are in a R*4 type)
C--type for L1B_populate_temps (this could be done inside L1B_populate_temps...)
        do ii = 1, 38
          ext_temps( ii ) = external_temperatures_eu( ii, i_block )
          scat_tlm(  ii ) = scat_telemetry_eu(        ii, i_block )
        enddo


C--Populate the instrument_module_temps common block with instrument temperatures.
C--This must be done before computing the parameters, since they may depend on 
C--these temperatures (depending on calibration_level option).
        call L1B_populate_temps( ext_temps, scat_tlm )

C--Compute instrument parameters depending on cal_level option
        call L1B_compute_params( calibration_level )

C--Call compute_xcal once per block.  This computes the instrument calibration.
        call L1B_compute_xcal( LB_power(1,1,i_block), max_gain, r_Xcal )
        
        if( i_dbg_level.ge.1 ) then
          write(6,*) ' '
          write(6,'((a50),I5)') '--L1B xCal dbg output, i_blk, i_seq, i_beam: ',i_block
          write(6,'((a50),6f12.6)') 'Xcal beam 1 [dB]: ', (r_Xcal(ii,1),ii=1,6)
          write(6,'((a50),6f12.6)') 'Xcal beam 2 [dB]: ', (r_Xcal(ii,2),ii=1,6)
          write(6,'((a50),6f12.6)') 'Xcal beam 3 [dB]: ', (r_Xcal(ii,3),ii=1,6)
          write(6,'((a50),6f12.6)') 'LB_expected [dB]: ', (r_LB_expected(ii),ii=1,6)
          write(6,'((a50),6f12.6)') 'TX Pow expected [dB]: ', (r_tx_pow_expected(ii),ii=1,6)
          write(6,'((a50),6f12.6)') 'TX Loss expected [dB]: ', (r_tx_loss_expected(ii),ii=1,6)
          write(6,'((a50),6f12.6)') 'RX Loss expected [dB]: ', (r_rx_loss_expected(ii),ii=1,6)
          write(6,'((a50),6f12.6)') 'RX Chain expected [dB]: ', (r_rx_chain_expected(ii),ii=1,6)
          write(6,'((a50),6f12.6)') 'T sys expected [K]: ', (r_Tsys_expected(ii),ii=1,6)
          write(6,'((a50),6f12.6)') 'CND Chain [dB]: ', (r_CND_chain(ii),ii=1,6)
        endif
        
        
        block_time = blk_sec(i_block) ! units of seconds
C--Check if we passed over a day boundary and add 86400 seconds if so.
!       If extract then check against first record not first extract block
        if ( flag_blk_limit_file_entered) then
           if( block_time .lt. blk_sec(1) )then
              block_time = block_time + 86400.0d0
           endif
        else
           if( block_time .lt. blk_sec(block_start) )then
              block_time = block_time + 86400.0d0
           endif
        endif

        L2_time = julian_time_0

cccccccccccccccccccccccccccccccccccccccccccc i_seq loop ccccccccccccccccccccccccccccccccccccc
C--Loop over sequence    
        do i_seq=1,8
          unable_to_geolocate_flag  = .false.
        
          sequence_time = (i_seq-1) * 0.18d0 + block_time + 0.09d0 ! units of seconds
          sequence_time = sequence_time / ( 24.0d0*60.0d0*60.0d0 ) ! units of days
          julian_time   = sequence_time + julian_time_0            ! units of days
          
                                ! L2_time is block mid-point time.
                                ! L1B_time is time at start of block
          if( i_seq .eq. 1 ) then
            L1B_time = julian_time
            L2_time  = ( block_time + 0.72d0 ) / ( 24.0d0*60.0d0*60.0d0 ) + julian_time_0
          endif
          
C--Check for power accumulation overflows in this sequence
          if( iand( L0_scat_headers(i_seq,i_block), 
     $              L0_SCAT_SEQ_HDR_PWR_ACCUM_OVERFLOW ) .gt. 0 )then
             write(6,*) 'WARNING: Power Accumulation overflow flag set!!'
          endif
          
C--Skip inversion if we can't get splines for this orbit position.
C--get_L1A_splines returns a negative number if it cannot find either the 
C--XYZ spline or YPR spline.
          if( flag_l1a_file_entered ) then
            if( get_L1A_splines(sequence_time,current_spline) .lt. 0 )then
               write(6,*) 'WARNING: Unable to get cubic spline.'
               unable_to_geolocate_flag = .true.
              goto 2000
            endif
            
C--Similarly, get_READ_SCI_spline returns a negative number if it cannot find either the 
C--XYZ spline or YPR spline.
          else
            if( get_READ_SCI_spline(julian_time-time_ref,current_spline) .lt. 0 )then
               write(6,*) 'WARNING: Unable to get cubic spline.'
              unable_to_geolocate_flag = .true.
              goto 2000
            endif
          endif
          
          if( i_dbg_level .gt. -1 )then
            write(6,*) ' '
            write(6,*) ' '
            write(6,'((a),I6,(a),I1,(a),f16.10)') 
     $       'Block #: ', i_block, ' seq #: ', i_seq, 
     $       ' sequence_time: ',sequence_time
            
          endif

          sc_position_mj2k(1) = current_spline(2) ! x position m
          sc_position_mj2k(2) = current_spline(3) ! y position m
          sc_position_mj2k(3) = current_spline(4) ! z position m

          sc_velocity_mj2k(1) = current_spline(5) ! x velocity (m/s)
          sc_velocity_mj2k(2) = current_spline(6) ! y velocity (m/s)
          sc_velocity_mj2k(3) = current_spline(7) ! z velocity (m/s)

          yaw_deg             = current_spline(8) ! yaw in deg
          pitch_deg           = current_spline(9) ! pitch in deg
          roll_deg            = current_spline(10)! roll in deg

          yaw_deg   = yaw_deg   + yaw_bias        ! add attitude bias from params file
          pitch_deg = pitch_deg + pitch_bias      ! 4/23/2012 AGF
          roll_deg  = roll_deg  + roll_bias       ! 

          if( i_dbg_level.ge.2)then
            write(*,*) ' '
            write(*,*) "mj2k from splines..."
            write(*,'((a10),1x,f19.7)') "SC pos x: ", sc_position_mj2k(1)
            write(*,'((a10),1x,f19.7)') "SC pos y: ", sc_position_mj2k(2)
            write(*,'((a10),1x,f19.7)') "SC pos z: ", sc_position_mj2k(3)
            write(*,'((a10),1x,f19.7)') " "
            write(*,'((a10),1x,f19.7)') "SC vel x: ", sc_velocity_mj2k(1)
            write(*,'((a10),1x,f19.7)') "SC vel y: ", sc_velocity_mj2k(2)
            write(*,'((a10),1x,f19.7)') "SC vel z: ", sc_velocity_mj2k(3)
          endif
          
          call mj2k_to_ecf( sc_position_mj2k, 
     $                      sc_velocity_mj2k,
     $                      julian_time, 
     $                      sc_position, 
     $                      sc_velocity )
          
          call compute_sc_nadir_components( sc_position, 
     $                                      spacecraft_lon,
     $                                      spacecraft_lat,
     $                                      spacecraft_altitude, 
     $                                      spacecraft_nadir_vector)  
          
C--Save sc_position when i_seq == 1 for getting a predicted farady rotation angle
C--From TEC maps.  GSFC won't need this since we plan to use predicted faraday
C--rotation angle from radiometer processing...
          if( i_seq .eq. 1 ) then
            sc_position_block_start(1) = sc_position(1)
            sc_position_block_start(2) = sc_position(2)
            sc_position_block_start(3) = sc_position(3)
            
            sc_lon_block_start         = spacecraft_lon
            sc_lat_block_start         = spacecraft_lat
            sc_alt_block_start         = spacecraft_altitude
          endif

          if( ascending .and. prev_spacecraft_lat > spacecraft_lat )
     $      ascending = .false.
          
          if( .not.ascending .and. prev_spacecraft_lat < spacecraft_lat )
     $      ascending = .true.
          
          if( i_dbg_level .ge. 2 ) then
            write(6,*) "Ascending:", ascending
            write(6,*) "spacecraft_lat: ", spacecraft_lat
            write(6,*) "prev_spacecraft_lat: ", prev_spacecraft_lat
          endif
          
          if( i_dbg_level .ge. 1 ) then
            write(*,'((a10),1x,f19.7)') "Jul Time: ", julian_time          
            write(6,*) "Ascending:", ascending 
            write(*,'((a10),1x,f19.7)') "SC pos x: ", sc_position(1)
            write(*,'((a10),1x,f19.7)') "SC pos y: ", sc_position(2)
            write(*,'((a10),1x,f19.7)') "SC pos z: ", sc_position(3)
            write(*,'((a10),1x,f19.7)') " "
            write(*,'((a10),1x,f19.7)') "SC vel x: ", sc_velocity(1)
            write(*,'((a10),1x,f19.7)') "SC vel y: ", sc_velocity(2)
            write(*,'((a10),1x,f19.7)') "SC vel z: ", sc_velocity(3)
                
            write(*,'((a10),1x,f19.7)') "SC NADIR x: ", spacecraft_nadir_vector(1)
            write(*,'((a10),1x,f19.7)') "SC NADIR y: ", spacecraft_nadir_vector(2)
            write(*,'((a10),1x,f19.7)') "SC NADIR z: ", spacecraft_nadir_vector(3)
                
            write(*,'((a10),1x,f19.7)') "yaw     : ", yaw_deg
            write(*,'((a10),1x,f19.7)') "pitch   : ", pitch_deg
            write(*,'((a10),1x,f19.7)') "roll    : ", roll_deg
          endif     

C--Compute the gain-weighted land and ice fractions for each of the 
C--three beams at this orbit position.
          if( i_seq .eq. 1 .and. .not. flag_no_landice_entered)then ! only compute this once per block...
            call L1B_compute_landice_fractions( julian_time, 
     $                                          sc_position,
     $                                          sc_velocity,
     $                                          yaw_deg, pitch_deg, roll_deg,
     $                                          antenna_pattern_relative_1H,
     $                                          antenna_pattern_relative_1V,
     $                                          antenna_pattern_relative_2H,
     $                                          antenna_pattern_relative_2V,
     $                                          antenna_pattern_relative_3H,
     $                                          antenna_pattern_relative_3V,     
     $                                          land_fraction,
     $                                          ice_fraction  )   
          endif

C--Check for possible range ambiguities based on ephemeris and attitude.
C--Right now, we will not flag based on the output of this subroutine,
C--only let it print WARNINGs to stdout.
          call L1B_check_range_ambig( sc_position,
     $                                sc_velocity,
     $                                yaw_deg, pitch_deg, roll_deg,
     $                                L2_off_nom_no_echo            )


          do i_beam = 1,3
          
             beam_off_earth_flag(i_beam) = .true.
             bad_kfactor_flag(i_beam)    = .true.

             if( i_dbg_level .ge. 1 )then
                write(6,*) ' ' 
                write(6,*) ' i_beam: ', i_beam
             endif
            
C--------------------------------------------------------------------
C             Begin geometry computations
C--------------------------------------------------------------------
            
C--Solve for the antenna azimuth and look angle
            call find_ant_look_and_ant_azi_angle( sc_position, 
     $                                            sc_velocity, 
     $                                            yaw_deg, 
     $                                            pitch_deg, 
     $                                            roll_deg, 
     $                                            i_beam, 
     $                                            antlook(i_beam), 
     $                                            antazi )

C--Set the beam widths used in the cell area computation below
C--depending on i_beam and if user specifies to use scale-model patterns.
            if( flag_use_scale_model_patts ) then
              beamw1V = half_beam_width_meas_1V(i_beam) * sqrt( 2.0d0 )
              beamw2V = half_beam_width_meas_2V(i_beam) * sqrt( 2.0d0 )
              beamw1H = half_beam_width_meas_1H(i_beam) * sqrt( 2.0d0 )
              beamw2H = half_beam_width_meas_2H(i_beam) * sqrt( 2.0d0 )
            else
              beamw1V = half_beam_width_ideal_1V(i_beam) * sqrt( 2.0d0 )
              beamw2V = half_beam_width_ideal_2V(i_beam) * sqrt( 2.0d0 )
              beamw1H = half_beam_width_ideal_1H(i_beam) * sqrt( 2.0d0 )
              beamw2H = half_beam_width_ideal_2H(i_beam) * sqrt( 2.0d0 )
            endif

C--Solve for the V-TX V-RX cell area
            call cellarea( spacecraft_altitude, 
     $                     spacecraft_lat, 
     $                     antlook(i_beam), 
     $                     antazi, 
     $                     beamw1V, 
     $                     beamw2V, 
     $                     areaV(i_beam), 
     $                     afootV(i_beam),
     $                     bfootV(i_beam), 
     $                     diffV(i_beam), 
     $                     Rslant, 
     $                     ainc )

C--Solve for the H-TX H-RX cell area
            call cellarea( spacecraft_altitude, 
     $                     spacecraft_lat, 
     $                     antlook(i_beam),
     $                     antazi, 
     $                     beamw1H,
     $                     beamw2H, 
     $                     areaH(i_beam), 
     $                     afootH(i_beam),
     $                     bfootH(i_beam), 
     $                     diffH(i_beam), 
     $                     Rslant, 
     $                     ainc )
            
            
C--Pointing vector in local beam HTX vector basis is always given by
C--the following. (+ \hat{z}_{i_beam,HTX} )
            vector_in(1) = 0.0d0
            vector_in(2) = 0.0d0
            vector_in(3) = 1.0d0
            
C--Locate center of beam on earth.
            call geolocate( vector_in, 
     $                      sc_position, 
     $                      sc_velocity, 
     $                      yaw_deg, pitch_deg, roll_deg, 
     $                      i_beam, 
     $                      earth_position, 
     $                      slant_range(i_beam), 
     $                      good_solution )

C--Save sc_position when i_seq == 1 for getting a predicted farady rotation angle
C--From TEC maps.  GSFC won't need this since we plan to use predicted faraday
C--rotation angle from radiometer processing...
            if( i_seq .eq. 1 ) then
              earth_position_block_start(1,i_beam) = earth_position(1)
              earth_position_block_start(2,i_beam) = earth_position(2)              
              earth_position_block_start(3,i_beam) = earth_position(3)
            endif

C--If imaginary solution => beam not on Earth surface, if so then
C--skip the rest of the inversion and set beam_off_earth_flag = .true.
            if( .not.good_solution ) then
               write(6,*) 'WARNING: Center of beam off-earth.', i_beam
               call flush(6)
               beam_off_earth_flag(i_beam) = .true.
               goto 1000
            endif
            beam_off_earth_flag(i_beam) = .false.

C--Compute the (lon,lat) coordinates for the center of beam on earth.                
            call compute_lat_long( earth_position, 
     $                             latitude_geocentric_deg, 
     $                             latitude_deg, 
     $                             longitude_deg  )
            
C--Change basis from HTX beam vector basis to ECF vector basis.
            call convert_to_rect_basis( vector_in, 
     $                                  sc_position, 
     $                                  sc_velocity, 
     $                                  yaw_deg, pitch_deg, roll_deg, 
     $                                  i_beam, 
     $                                  antenna_pointing_vector_rect )

C--Computes the incidence angle as well as the local unit normal on the surface
C--of Earth.
            call compute_incidence_angle( earth_position,
     $                                    antenna_pointing_vector_rect, 
     $                                    incidence_angle(i_beam), 
     $                                    xnormal )

C--Computes the cell azimuth angle relative to north.
            call compute_cell_azimuth( earth_position, 
     $                                 antenna_pointing_vector_rect, 
     $                                 cell_azimuth_angle_deg(i_beam) )

C--Computation of polarization roll angle...
C---epsilon1 is beam h-pol in local beam coordinate system
            epsilon1(1)=1.0d0
            epsilon1(2)=0.0d0
            epsilon1(3)=0.0d0

C--Change basis from HTX beam vector basis to ECF vector basis.
            call convert_to_rect_basis( epsilon1, 
     $                                  sc_position, 
     $                                  sc_velocity, 
     $                                  yaw_deg, pitch_deg, roll_deg, 
     $                                  i_beam, 
     $                                  rot_epsilon1 )


C--Compute the ground h-pol and v-pol directions from unit normal and
C--antenna pointing vector
            call cross_product( xnormal,antenna_pointing_vector_rect,h_vector)
            call convert_to_unit_vector(h_vector)
              
            call cross_product(antenna_pointing_vector_rect,h_vector,v_vector)
            call convert_to_unit_vector(v_vector)
            
C--Compute the angle between the beam h-pol and ground h-pol (polarization roll angle)
            dotprod  = compute_dot_product( rot_epsilon1, h_vector )
            dotprod2 = compute_dot_product( rot_epsilon1, v_vector )
            polarization_roll_angle(i_beam) = - atan2(dotprod2,dotprod) * rad_to_deg


C--Get four points that are on the two-way -3dB contour
            call points_edge( sc_position, 
     $                        sc_velocity, 
     $                        yaw_deg, pitch_deg, roll_deg, 
     $                        i_beam, 
     $                        latitude_points, 
     $                        longitude_points,
     $                        good_solution )

C--Check that all 4 edge points are on-Earth...
            if( .not.good_solution )then
             write(6,*) 'WARNING: At least one edge point off-earth', i_beam
              call flush(6)
              beam_off_earth_flag(i_beam) = .true.
            endif            
            
            if( i_dbg_level .ge. 1 )then
              write(6,*) ' '
              write(6,'((a50),3I5)') '--L1B Geom dbg output, i_blk, i_seq, i_beam: ',i_block, i_seq, i_beam
              write(6,'((a50),3f12.6)') 'antlook, antazi, cell_azi: ',antlook(i_beam), antazi,cell_azimuth_angle_deg(i_beam) 
              write(6,'((a50),3f12.6)') 'pointing_vector_rect: ',antenna_pointing_vector_rect       
              write(6,'((a50),3f16.6)') 'earth_position: ',earth_position
              write(6,'((a50),1f16.6)') 'slant_range: ',slant_range(i_beam)
              write(6,'((a50),L1)') 'good_solution: ',good_solution
              write(6,'((a50),2f12.6)') 'areaH(i_beam),areaV(i_beam): ', areaH(i_beam), areaV(i_beam)
              write(6,'((a50),3f12.6)') 'epsilon_1 rect: ',rot_epsilon1
              write(6,'((a50),2f12.6)') '(lon,lat) beam center [deg]: ',longitude_deg, latitude_deg
              write(6,'((a50),2f12.6)') 'inc angle, polarization roll angle [deg]: ',
     1                                  incidence_angle(i_beam), polarization_roll_angle(i_beam)
              write(6,'((a50),4f12.6)') 'lon_points [deg]: ', longitude_points
              write(6,'((a50),4f12.6)') 'lat_points [deg]: ', latitude_points
            endif

C--------------------------------------------------------------------
C--                Begin computation of Sigma0                      
C--------------------------------------------------------------------

            do i_chan = 1,6   ! Convert back to linear units
              r_rx_power( i_chan ) = 10.d0**( 
     $                        RX_power(i_chan,i_beam,i_seq,i_block) / 10.d0 )
            enddo
            
            r_max_gain(1) = max_gain(i_beam,1)
            r_max_gain(2) = max_gain(i_beam,2)

C--Compute the noise to be used for the noise subtraction...
C--r_{H,V}noise is the noise-only power (must be careful since the CND is firing...)
            if( ( mod(i_seq,2).eq.1 .and. i_beam.eq.2 ) .or. ! CND fires on beam 2 nV for odd n_seq
     $          ( mod(i_seq,2).eq.0 .and. 
     $          (i_beam.eq.1 .or. i_beam.eq.3) ) ) then      ! CND fires on beams 1,3 nV for even n_seq
     
               if( i_seq.eq.1 ) then ! if 1st seq, use next seq for noise
                  r_Vnoise = VRX_Noise_Power(i_beam,i_seq+1,i_block)
               else             ! else use previous one
                  r_Vnoise = VRX_Noise_Power(i_beam,i_seq-1,i_block)
               endif
            else                ! No CND firing, can use nV channel.
               r_Vnoise = VRX_Noise_Power(i_beam,i_seq,i_block)
            endif
           
            r_Hnoise = HRX_Noise_Power(i_beam,i_seq,i_block)
           
C--Compute DC offset power
            r_DCoff_power = ( max( abs( DC_offset(1,i_block) ), 
     $                             abs( DC_offset(2,i_block) ) ) ** 2 ) * 1.d-3 / 50.0d0 ! power in dBm

C--Noise subtraction
            r_sgnl_power(1) = r_rx_power(1) - r_Vnoise ! HV - nV
            r_sgnl_power(2) = r_rx_power(3) - r_Vnoise ! VV - nV
            r_sgnl_power(3) = r_rx_power(4) - r_Hnoise ! VH - nH
            r_sgnl_power(4) = r_rx_power(6) - r_Hnoise ! HH - nH

            do i_chan = 1, 4
              if( r_sgnl_power(i_chan)/r_Vnoise .lt. -0.01 .and. i_chan .le. 2 )then
                 write(6,*) 'WARNING: Signal level too negative: ', i_chan, r_sgnl_power(i_chan), r_Vnoise
              elseif( r_sgnl_power(i_chan)/r_Hnoise .lt. -0.01 .and. i_chan .gt. 2 )then
                 write(6,*) 'WARNING: Signal level too negative: ', i_chan, r_sgnl_power(i_chan), r_Hnoise
              endif
            enddo
            
            if( r_DCoff_power/min( r_Vnoise, r_Hnoise ) .gt. 0.02 ) then
               write(6,*) 'WARNING: DC offset power becoming significant: ', r_DCoff_power, 10.0*log10(r_DCoff_power)
            endif
            
C--convert (echo - noise) power back to dB
            do i_chan = 1, 4
C--If echo - noise power is negative, set the quality flag bit for negative sigma0...            
              if( r_sgnl_power(i_chan) .lt. 0.0d0 ) then
                call L1B_set_flag( L1B_quality_flag(i_chan,i_beam,i_seq),
     $                             L1B_QF_NEGATIVE_SIGMA0,
     $                             L1B_QF_NEGATIVE_SIGMA0 )
              endif
C--Use abs(.) below to make sure it is positive before we take log10...            
              r_sgnl_power(i_chan) = 10.0d0*log10(abs(r_sgnl_power(i_chan)))
            enddo

C--Compute SNR and KPC            
            call L1B_compute_kpc( r_rx_power, ! r_rx_power is in linear units at this point...
     $                            r_Vnoise,   ! linear units
     $                            r_Hnoise,   ! linear units
     $                            i_beam, 
     $                            sc_position, 
     $                            sc_velocity, 
     $                            yaw_deg, pitch_deg, roll_deg, 
     $                            r_KPC, r_TBd, r_SNR )

C--Get the Kfactors from the tables.
C--kfactors(1) is for HH
C--kfactors(2) is for VH
C--kfactors(3) is for VV
C--get_kfactor wants an int not a logical for the asc/dec flag
            dir = 0
            if( .not.ascending ) dir = 1
            call get_kfactor( spacecraft_lat, incidence_angle(i_beam), i_beam, dir, kfactors )
    
            ! Check for bad kfactor (out of range) AGF 11/21/2011
            ! returns -999 when lat or inc angle is out of range
            if( kfactors(1).le.0.0d0 .or. kfactors(2).le.0.0d0 .or. kfactors(3).le.0.0d0 ) then
              bad_kfactor_flag(i_beam)  = .true.
              goto 1000 ! skip sigma0 computation for this beam
            else
              bad_kfactor_flag(i_beam)  = .false.
            endif  
                    
            
            r_xint_inv(1) = 10.0d0*log10( kfactors(2) / (slant_range(i_beam))**4 ! cx-pol VH
     $       * sqrt( areaH(i_beam)*areaV(i_beam) )*1000.d0**2 ) 

            r_xint_inv(2) = 10.0d0*log10( kfactors(3) / (slant_range(i_beam))**4 ! co-pol VV
     $       * areaV(i_beam) *1000.d0 **2 ) 

            r_xint_inv(3) = r_xint_inv(1)                                        ! cx-pol HV

            r_xint_inv(4) = 10.0d0*log10( kfactors(1) / (slant_range(i_beam))**4 ! co-pol HH 
     $       * areaH(i_beam) *1000.d0**2 ) 
                        
            r_xcal_inv(1) = r_Xcal(1,i_beam)
            r_xcal_inv(2) = r_Xcal(3,i_beam)
            r_xcal_inv(3) = r_Xcal(4,i_beam)
            r_xcal_inv(4) = r_Xcal(6,i_beam)

C--Compute sigma-0            
            do i_chan = 1, 4 ! Module SB6
              r_sig0(i_chan) = r_sgnl_power(i_chan) - r_xint_inv(i_chan)  - r_xcal_inv(i_chan)
     $                       + 10.0d0 * log10( r_rx_length(i_beam)  ! AGF added 9/9/2009
     $                                       / r_tx_pulse_length  ) ! AGF added 9/9/2009
            enddo

C--Apply sigma0 bias corrections (use geometric average for X-pol)
            r_sig0(1) = r_sig0(1) - 0.5d0*(bias_adj(i_beam, 1) + bias_adj(i_beam, 2))
            r_sig0(2) = r_sig0(2) - bias_adj(i_beam, 2)
            r_sig0(3) = r_sig0(3) - 0.5d0*(bias_adj(i_beam, 1) + bias_adj(i_beam, 2))
            r_sig0(4) = r_sig0(4) - bias_adj(i_beam, 1)

C--Compute noise equivalent sigma-0
            r_VN_sig0 = 10.0d0*log10(r_Vnoise) - r_xint_inv(2) - r_xcal_inv(2) ! use V-RX co-pol xint,xcal
            r_HN_sig0 = 10.0d0*log10(r_Hnoise) - r_xint_inv(4) - r_xcal_inv(4) ! use H-RX co-pol xint,xcal
            
            if( i_dbg_level .ge. 1 )then
              write(6,*) ' '
              write(6,'((a50),3I5)') '--L1B Sigma0 dbg output, i_blk, i_seq, i_beam: ',i_block, i_seq, i_beam
              write(6,'((a50),6f12.6)') 'r_rx_power [dB]: ',(10.0*log10(r_rx_power(ii)),ii=1,6)
              write(6,'((a50),6f12.6)') 'LB_power [dB]: ',(LB_power(i_chan,i_beam,i_block), i_chan=1,6)
              write(6,'((a50),2f12.6)') 'DC_offsets [mV]: ', DC_offset(1,i_block), DC_offset(2,i_block)
              write(6,'((a50),f12.6)') 'r_DCoff_power [dB]: ',10.0*log10(r_DCoff_power)
              write(6,'((a50),2f12.6)') 'r_Vnoise, r_Hnoise [dB]: ',10.0*log10(r_Vnoise),10.0*log10(r_Hnoise)
              write(6,'((a50),4f12.6)') 'r_sgnl_power [dB]: ',(r_sgnl_power(ii),ii=1,4)
              write(6,'((a50),3f12.6)') 'kfactors (HH,VH,VV) [-]: ',kfactors
              write(6,'((a50),4f12.6)') 'r_xint_inv [dB]: ', r_xint_inv
              write(6,'((a50),4f12.6)') 'r_xcal_inv [dB]: ', r_xcal_inv
              write(6,'((a50),4F12.6)') 'Sigma0 [dB]: ', r_sig0
              write(6,'((a50),4F12.6)') 'Sigma0 + r_xint_inv [dB]: ', (r_xint_inv(ii)+r_sig0(ii),ii=1,4)
              write(6,'((a50),2F12.6)') 'Noise Eqv. Sigma0 (V,H) [dB]: ',r_VN_sig0, r_HN_sig0
              write(6,'((a50),4F12.6)') 'SNR [dB]: ', r_SNR
              write(6,'((a50),4F12.6)') 'Kpc [linear]: ', r_KPC
              write(6,'((a50),f12.6)') 'Time Bandwidth Product [-]: ',r_TBd
            endif
            
1000        continue ! We jumped here if beam not on Earth for this 
                     ! orbit position and beam #.
      
C--Populate L1B buffers for this block------------------------            
            if( i_seq .eq. 1 .and. .not.unable_to_geolocate_flag )then
              if( i_beam .eq. 1 )then
                
                L1B_roll  = roll_deg
                L1B_pitch = pitch_deg
                L1B_yaw   = yaw_deg
              
                do ii=1,3
                  L1B_land_fraction(ii) = land_fraction(ii)
                  L1B_ice_fraction(ii)  = ice_fraction(ii)
                  
                  L1B_xyz_mj2k(ii)      = sc_position_mj2k(ii)
                  L1B_vxyz_mj2k(ii)     = sc_velocity_mj2k(ii)
                enddo
              endif ! if( i_beam .eq. 1)
            endif ! if( i_seq .eq. 1 .and. .not.unable_to_geolocate_flag )
            
            L1B_Noise_Pwr(1,i_beam,i_seq) = RX_power(2,i_beam,i_seq,i_block)
            L1B_Noise_Pwr(2,i_beam,i_seq) = RX_power(5,i_beam,i_seq,i_block)
            
            if( .not.beam_off_earth_flag(i_beam) .and. .not.unable_to_geolocate_flag )then

              ! If kfactor not computable, skip assignment of sigma0 quantities AGF 11/21/2011
              if( .not.bad_kfactor_flag(i_beam) ) then
                 do i_chan = 1, 4
                    L1B_KPC(i_chan,i_beam,i_seq)  = r_KPC(i_chan)
                    L1B_SNR(i_chan,i_beam,i_seq)  = r_SNR(i_chan)
                    L1B_sig0(i_chan,i_beam,i_seq) = r_sig0(i_chan)
                 enddo
            
                 L1B_Noise(1,i_beam,i_seq) = r_VN_sig0
                 L1B_Noise(2,i_beam,i_seq) = r_HN_sig0
              endif             ! if( .not.bad_kfactor_flag(i_beam) ) then

              L1B_lat_echo_center(i_beam,i_seq) = latitude_deg
              L1B_lon_echo_center(i_beam,i_seq) = longitude_deg
            
              if( i_seq.eq.1 )then
                L1B_inc_angle(i_beam)         = incidence_angle(i_beam)
                L1B_ant_azi_rel_north(i_beam) = cell_azimuth_angle_deg(i_beam)
                L1B_pol_roll_angle(i_beam)    = polarization_roll_angle(i_beam)
              
                do ii = 1, 4
                  L1B_lat_beam_edge(ii,i_beam) = latitude_points(ii)
                  L1B_lon_beam_edge(ii,i_beam) = longitude_points(ii)
                enddo
              endif ! if( i_seq .eq. 1 )
            endif ! if( .not.beam_off_earth_flag .and. .not.unable_to_geolocate_flag)
            
C------DONE populating L1B buffers for this block-----
          enddo ! do i_beam = 1,3
          
C--Set Quality flags for this orbit position          
2000      continue  ! We jumped here if cannot get spline for this orbit position.
                    ! or if bad time tag for this block.
C--Computes the quality flag for beams 1-3 and channels {HV, VV, VH, HH}
          call L1B_gen_quality_flag( i_seq,
     $                               calibration_level, 
     $                               L1B_land_fraction,
     $                               L1B_ice_fraction,
     $                               antlook,
     $                               bad_time_tag_flag,
     $                               bad_block_id_flag,
     $                               unable_to_geolocate_flag,
     $                               beam_off_earth_flag,
     $                               bad_kfactor_flag,
     $                               RFI_FLAGS_VPOL(1,1,i_block),
     $                               RFI_FLAGS_HPOL(1,1,i_block),
     $                               RX_power(1,1,1,i_block),
     $                               LB_power(1,1,i_block),
     $                               scat_telemetry_eu(1,i_block),
     $                               external_temperatures_eu(1,i_block),
     $                               icds_engineering_telemetry_eu(1,i_block),
     $                               flag_suppress_tlm_warnings,
     $                               L1B_quality_flag(1,1,i_seq)                   )
          
        enddo   ! do i_seq = 1, 8
        
cccccccccccccccccccccccccccccccccccccccccccc i_seq loop ccccccccccccccccccccccccccccccccccccc
        call L1B_check_CND_gain( RX_power(1,1,1,i_block),
     $                           L1B_quality_flag )
        
C------------------------------------------------------------------------------
C
C-----------------   Done with L1B processing          ------------------------
C
C------------------------------------------------------------------------------

        
C--Call L1B_populate_output_buffers to append L1B data for this block
C--to the L1B buffers for the whole orbit (so we can write out a L1B
C--data product). Writes out data if( i_block .eq. NUM_BLOCKS )
        if( flag_write_l1b_output )then
           icds_blknum = icds_processing_status_eu(      4, i_block )
          call L1B_populate_output_buffer( i_block,
     $                                     block_start,
     $                                     last_block, !NUM_BLOCKS,
     $                                     L1B_time, 
     $                                     L1B_xyz_mj2k, 
     $                                     L1B_vxyz_mj2k,
     $                                     L1B_roll,
     $                                     L1B_pitch,
     $                                     L1B_yaw,
     $                                     L1B_KPC,
     $                                     L1B_SNR,
     $                                     L1B_Noise,
     $                                     L1B_Noise_Pwr,
     $                                     L1B_sig0,
     $                                     L1B_lat_echo_center,
     $                                     L1B_lon_echo_center,
     $                                     L1B_lat_beam_edge,
     $                                     L1B_lon_beam_edge,
     $                                     L1B_ant_azi_rel_north,
     $                                     L1B_pol_roll_angle,
     $                                     L1B_inc_angle,
     $                                     L1B_land_fraction,
     $                                     L1B_ice_fraction,
     $                                     L1B_housekeeping,
     $                                     L1B_quality_flag,
     $                                     L2_prefix )
        endif
        prev_spacecraft_lat = spacecraft_lat

! icds_processing_status_eu


C------------------------------------------------------------------------------
C
C----------------- Level 2 Processing (block-by-block) ------------------------
C
C------------------------------------------------------------------------------

C--Initialize L2 quantities        
        do i_beam = 1, 3
          L2_scat_wind_speed( i_beam )     = 0.0
          L2_scat_wind_speed_err( i_beam ) = 0.0
          L2_beam_clat( i_beam )           = 0.0
          L2_beam_clon( i_beam )           = 0.0
          L2_polarization_roll(i_beam)     = L1B_pol_roll_angle(i_beam)
          L2_faraday_rotation_angle(i_beam)= 0.0
          L2_scatterometer_flags(i_beam)   = 0 ! int*4          

          do ii = 1, 2
            L2_scat_rfi_flags( i_beam, ii ) = 0
            L2_excess_surf_emiss( ii, i_beam )       = 0
            L2_excess_surf_emiss_err( ii, i_beam )   = 0
          enddo
          
          do ii = 1, 4
            L2_beam_elat( ii, i_beam )     = L1B_lat_beam_edge(ii,i_beam)
            L2_beam_elon( ii, i_beam )     = L1B_lon_beam_edge(ii,i_beam)
          enddo

          L2_scat_samples( i_beam )        = 0 

          do i_chan = 1, 4
            L2_neg_sig0_ANT( i_chan, i_beam )  = .false.
            L2_neg_sig0_TOA( i_chan, i_beam )  = .false.
            
            L2_sigma_ANT( i_chan, i_beam )     = 0.0d0 ! top-of-ionosphere sigma0            
            L2_KPC_ANT( i_chan, i_beam )       = 0.0d0 ! kpc for top-of-ionosphere sigma0
            L2_var_sigma_ANT( i_chan, i_beam ) = 0.0d0 ! variance for top-of-ionosphere sigma0

            L2_sigma_TOI( i_chan, i_beam )     = 0.0d0 ! top-of-atmosphere sigma0
            L2_var_sigma_TOI( i_chan, i_beam ) = 0.0d0 ! variance for top-of-ionosphere sigma0

            L2_sigma_TOA( i_chan, i_beam )     = 0.0d0 ! top-of-atmosphere sigma0
            L2_var_sigma_TOA( i_chan, i_beam ) = 0.0d0 ! variance for top-of-ionosphere sigma0
            L2_KPC_TOA( i_chan, i_beam )       = 0.0d0 ! kpc for top-of-ionosphere sigma0
            
            L2_sigma_Far( i_chan, i_beam )     = 0.0d0 ! faraday correction to sigma0            
            
          enddo
          
        enddo
C--Done initializing L2 quantities


C--Copy L0 RFI into the L2 output array
!        call L2_load_RFI_flags( RFI_FLAGS_VPOL, 
!     $                          RFI_FLAGS_HPOL,
!     $                          L2_scat_rfi_flags )

C--Copy L0 RFI into the L2 output array
!  Replace JMG 09/14/11 from A. Fore
        call L2_load_RFI_flags( RFI_FLAGS_VPOL(1,1,i_block), 
     $       RFI_FLAGS_HPOL(1,1,i_block),
     $       L2_scat_rfi_flags )

C--Compute L2 quantities----
C--Average Quantities over the block
        do i_beam = 1, 3
          L2_land_fraction(i_beam) = L1B_land_fraction(i_beam)
          L2_ice_fraction(i_beam)  = L1B_ice_fraction(i_beam)
          
          sum_sin_lat = 0.0d0
          sum_cos_lat = 0.0d0
          sum_sin_lon = 0.0d0
          sum_cos_lon = 0.0d0
          
          !--check if all RFI here; do special processing if so.--- 9/26/2011--
          L2_all_rfi = .true.
          do i_seq = 1, 8
            if( iand(L1B_quality_flag(2,i_beam,i_seq),L1B_QF_RFI).eq.0 .and.
     $          iand(L1B_quality_flag(4,i_beam,i_seq),L1B_QF_RFI).eq.0 ) L2_all_rfi = .false.
          enddo
          
          do i_seq = 1, 8
            L2_include_sequence = .false.
            if( L2_all_rfi ) then ! if all RFI, unset RFI bits in test mask (L2_bitmask)
              L2_bitmask_temp = L2_bitmask
              call L1B_clear_flag( L2_bitmask_temp, L1B_QF_RFI )              
            else
              L2_bitmask_temp = L2_bitmask
            endif
            
            ! if any of the 4 channels don't meet the criteria, exclude
            ! these measurements from L2 averaging. 
            if( iand(L1B_quality_flag(1,i_beam,i_seq),L2_bitmask_temp).eq.0 .and.
     $          iand(L1B_quality_flag(2,i_beam,i_seq),L2_bitmask_temp).eq.0 .and.
     $          iand(L1B_quality_flag(3,i_beam,i_seq),L2_bitmask_temp).eq.0 .and.
     $          iand(L1B_quality_flag(4,i_beam,i_seq),L2_bitmask_temp).eq.0 ) L2_include_sequence = .true.
             
            if( L2_include_sequence ) then
     
              L2_scat_samples( i_beam ) = L2_scat_samples( i_beam ) + 1
              
              ! Averaging of geolocation...
              sum_sin_lat = sum_sin_lat + sin( L1B_lat_echo_center(i_beam,i_seq) * PI / 180.0d0 )
              sum_cos_lat = sum_cos_lat + cos( L1B_lat_echo_center(i_beam,i_seq) * PI / 180.0d0 )
              sum_sin_lon = sum_sin_lon + sin( L1B_lon_echo_center(i_beam,i_seq) * PI / 180.0d0 )
              sum_cos_lon = sum_cos_lon + cos( L1B_lon_echo_center(i_beam,i_seq) * PI / 180.0d0 )

              do i_chan = 1, 4 ! Add in linear
                if( iand( L1B_quality_flag(i_chan,i_beam,i_seq), ! check if negative sigma0 flag set
     $                    L1B_QF_NEGATIVE_SIGMA0 ) .ne. 0 ) then
     
                  L2_sigma_ANT( i_chan, i_beam ) = L2_sigma_ANT( i_chan, i_beam ) - ! subtract if set
     $                                             10.0d0 ** ( 0.1d0 * 
     $                                             L1B_sig0(i_chan,i_beam,i_seq) )
                else
                  L2_sigma_ANT( i_chan, i_beam ) = L2_sigma_ANT( i_chan, i_beam ) + ! add if not set
     $                                             10.0d0 ** ( 0.1d0 * 
     $                                             L1B_sig0(i_chan,i_beam,i_seq) )
                endif
                
                ! Computation of L2 KPC from L1B KPC + L1B sig0
                if( 10.0d0 **( 0.1d0 * L1B_sig0(i_chan,i_beam,i_seq) ) .eq. 0 )then
                   ! If Sigma0 == 0; compute variance using # of looks + noise equiv.
                   ! Sigma0.  Can't use KPC since 0 * Inf == NaN; then NaNs will
                   ! corrupt all the L2 quantities that use this quantity.
                   
                  L2_var_sigma_ANT(i_chan,i_beam) = L2_var_sigma_ANT(i_chan,i_beam) + 
     $              (10.0d0**( 0.1d0 * L1B_Noise(i_chan/2+1,i_beam,i_seq)) *  ! ( NESZ in linear units
     $              (r_rx_length(i_beam) / r_tx_pulse_length) )**2 /          ! * ratio of window times) ** 2
     $              ( r_rx_BW * r_rx_length(i_beam) )                         ! / # of looks for noise integration.

                else ! use KPC to get variance contribution
                  L2_var_sigma_ANT(i_chan,i_beam) = L2_var_sigma_ANT(i_chan,i_beam) + 
     $              (L1B_KPC(i_chan,i_beam,i_seq) * ( 10.0d0**(0.1d0*L1B_sig0(i_chan,i_beam,i_seq))))**2
                endif
                
              enddo
            endif ! check if data meets criteria for L2 averaging.
            
            
            if( i_seq .eq. 8 )then ! Last time through i_seq loop
                                   ! Divide by L2_scat_samples

              ! compute average (lon,lat) from components
              L2_beam_clat(i_beam) = atan2( sum_sin_lat, sum_cos_lat ) * 180.0d0 / PI
              L2_beam_clon(i_beam) = atan2( sum_sin_lon, sum_cos_lon ) * 180.0d0 / PI
              
              do i_chan = 1, 4
                if( L2_sigma_ANT(i_chan,i_beam) .lt. 0 ) then 
                  write(6,'((a),3I6)') 'WARNING: L2_sigma_ANT negative', i_chan, i_beam, i_block                  
                  L2_neg_sig0_ANT( i_chan, i_beam ) = .true.
                endif

                ! Compute L2_sigma_ANT
                L2_sigma_ANT( i_chan, i_beam ) = L2_sigma_ANT( i_chan, i_beam ) / 
     $                                           real(L2_scat_samples(i_beam)) 
     
                ! divide by # of samples squared to get variance of L2_sigma_ANT
                L2_var_sigma_ANT(i_chan,i_beam) = L2_var_sigma_ANT(i_chan,i_beam) /
     $                                            ((real(L2_scat_samples(i_beam)))**2)

                ! compute KPC for sigma ANT from variance of sigma ANT and sigma ANT.
                ! Fixed  JMG  10/20/11
                L2_KPC_ANT(i_chan,i_beam) = sqrt( L2_var_sigma_ANT(i_chan,i_beam) / 
     $                                      L2_sigma_ANT(i_chan,i_beam)**2)
                
              enddo ! i_chan = 1, 4
            endif ! if( i_seq .eq. 8 )
          enddo ! i_seq = 1, 8
        enddo ! i_beam = 1, 3

C--Apply Faraday rotation corrections (this may need to be changed...)
C------------------------------------------------------------------------------
C --- Antenna pattern corrrection and faraday rotation correction.
C------------------------------------------------------------------------------
C--Correct for antenna pattern; ( ANT -> TOI ); also propagate variance through the APC
C--correction.
         call L2_APC_correction( L2_sigma_ANT, ! still in linear units
     $        L2_var_sigma_ANT,
     $        L2_APC_file,
     $        flag_L2_apc_file_entered,
     $        L2_sigma_TOI,     ! returned in linear units
     $        L2_var_sigma_TOI )
C--After APC, hv and vh are now symmetrized; thus L2_sigma_TOI(1,i_beam) = L2_sigma_TOI(3,i_beam)
        

C--Compute and apply Faraday rotation corrections
        do i_beam = 1, 3
        
C--This stuff may need to be replaced with LeVine's predicted faraday rotation angle,
C--or optionaly with the radiometer's estimate (use command line argument ??)

C--Estimate faraday rotation angle from TEC maps
          call JD2CAL( L1B_time, i_year, i_month, i_day, i_hour, 
     $ i_minute, i_second, r_frac_seconds )
          date(1) = real( i_year )
          date(2) = real( i_month )
          date(3) = real( i_day )
          date(4) = real( i_hour )
          date(5) = real( i_minute )
          date(6) = real( i_second ) + r_frac_seconds
        
          call faraday_rotation_IONEX( sc_position_block_start, 
     $                                 sc_lon_block_start,
     $                                 sc_lat_block_start,
     $                                 sc_alt_block_start,
     $                                 earth_position_block_start(1,i_beam),
     $                                 date,
     $                                 L2_faraday_rotation_angle(i_beam) )   ! returns in radians
C--End of code that should be replaced with radiometer's faraday angle
C--estimates.

!  Skip if no TEC file          
          if ( L2_faraday_rotation_angle(i_beam) .eq. -999)
     $    flag_L2_no_far_corr_entered = .true.

          L2_faraday_rotation_angle(i_beam) = L2_faraday_rotation_angle(i_beam) * 180.0d0/PI
          
          call iri2007_VTEC_Scale_Factor( sc_position_block_start,
     $                                    earth_position_block_start(1,i_beam),
     $                                    date,
     $                                    L2_tec_scale_factor )

          
          ! Approximate scale factors from comparing radiometer derived angle to 
          ! our prediction and scaling radiometer estimate by rad f^2 / scat f^2
          !if( i_beam .eq. 1 ) then
          !  L2_tec_scale_factor = 0.79d0
          !elseif( i_beam .eq. 2 ) then
          !  L2_tec_scale_factor = 0.78d0
          !elseif( i_beam .eq. 3 ) then
          !  L2_tec_scale_factor = 0.77d0
          !endif
       
          L2_tec_scale_factor = 0.75d0

! Can scale angle instead of VTEC (angle is linearly prop. to VTEC)
          L2_faraday_rotation_angle(i_beam) = L2_faraday_rotation_angle(i_beam) * L2_tec_scale_factor

C--Correct for faraday rotation ( TOI -> TOA ); also propagate variance through.
          call L2_Faraday_rotation_correction( L2_sigma_TOI(1,i_beam),
     $                                         L2_var_sigma_TOI(1,i_beam),
     $                                         L2_faraday_rotation_angle(i_beam),
     $                                         rho_HHVV(i_beam),
     $                                         f_HVHH(i_beam),
     $                                         f_HVVV(i_beam),
     $                                         flag_L2_no_far_corr_entered,
     $                                         L2_sigma_TOA(1,i_beam),
     $                                         L2_var_sigma_TOA(1,i_beam),
     $                                         L2_good_far_corr(i_beam)   )
          
        enddo

        if( i_dbg_level.ge.1) then
          write(6,*) ' '
          write(6,'(a50,4f12.6)') 'Sig0 Before APC beam 1:', (10*log10(L2_sigma_ANT(i_chan,1)),i_chan=1,4)
          write(6,'(a50,4f12.6)') 'Sig0 After APC beam 1:', (10*log10(L2_sigma_TOI(i_chan,1)),i_chan=1,4)
          write(6,'(a50,4f12.6)') 'Sig0 After Faraday Corr. beam 1:', (10*log10(L2_sigma_TOA(i_chan,1)),i_chan=1,4)
          write(6,*) ' '
          write(6,'(a50,4f12.6)') 'Sig0 Before APC beam 2:', (10*log10(L2_sigma_ANT(i_chan,2)),i_chan=1,4)
          write(6,'(a50,4f12.6)') 'Sig0 After APC beam 2:', (10*log10(L2_sigma_TOI(i_chan,2)),i_chan=1,4)
          write(6,'(a50,4f12.6)') 'Sig0 After Faraday Corr. beam 2:', (10*log10(L2_sigma_TOA(i_chan,2)),i_chan=1,4)
          write(6,*) ' '
          write(6,'(a50,4f12.6)') 'Sig0 Before APC beam 3:', (10*log10(L2_sigma_ANT(i_chan,3)),i_chan=1,4)
          write(6,'(a50,4f12.6)') 'Sig0 After APC beam 3:', (10*log10(L2_sigma_TOI(i_chan,3)),i_chan=1,4)
          write(6,'(a50,4f12.6)') 'Sig0 After Faraday Corr. beam 3:', (10*log10(L2_sigma_TOA(i_chan,3)),i_chan=1,4)
          
        endif
C--End of APC and faraday rotation corrections.
        
C--Compute total sigma0 + variance and KPC and wind speed + delta TB retrival.        
        do i_beam = 1, 3

          L2_tot_sigma_TOA(i_beam)     = 0
          L2_tot_KPC_TOA(i_beam)       = 0
          L2_var_tot_sigma_TOA(i_beam) = 0
          
          do i_chan = 1, 4
            
            if( L2_sigma_TOA(i_chan,i_beam) .lt. 0 ) then
              write(6,'((a),3I6)') 'WARNING: L2_sigma_TOA negative', i_chan, i_beam, i_block                  
              L2_neg_sig0_TOA( i_chan, i_beam ) = .true.
            endif
            
            L2_KPC_TOA(i_chan,i_beam) = sqrt( L2_var_sigma_TOA(i_chan,i_beam) / 
     $                                        L2_sigma_TOA(i_chan,i_beam)**2    )
            
            
            
            L2_tot_sigma_TOA(i_beam) = L2_tot_sigma_TOA(i_beam) + 
     $                                 L2_sigma_TOA( i_chan, i_beam )
            

            ! Compute variance of total TOA sigma0
            ! variance of total TOA sigma0 == variance of ANT total sigma0
            L2_var_tot_sigma_TOA(i_beam) = L2_var_tot_sigma_TOA(i_beam) + 
     $                                     L2_var_sigma_ANT( i_chan, i_beam )

          enddo
          
          ! Debugging checks
          if( L2_KPC_TOA(1,i_beam) .ne. L2_KPC_TOA(1,i_beam) .or.
     $        L2_KPC_TOA(2,i_beam) .ne. L2_KPC_TOA(2,i_beam) .or.
     $        L2_KPC_TOA(3,i_beam) .ne. L2_KPC_TOA(3,i_beam) .or.
     $        L2_KPC_TOA(4,i_beam) .ne. L2_KPC_TOA(4,i_beam) ) then
            
             if( i_dbg_level.gt.-10) then
                write(6,'(a,I6)') 'ERROR: NaN in L2_KPC_TOA!, i_beam', i_beam
                write(6,'(a50,4d16.4)') 'L2_var_sigma_ANT: ',(L2_var_sigma_ANT(i_chan,i_beam),i_chan=1,4)
                write(6,'(a50,4d16.4)') 'L2_var_sigma_TOI: ',(L2_var_sigma_TOI(i_chan,i_beam),i_chan=1,4)
                write(6,'(a50,4d16.4)') 'L2_var_sigma_TOA: ',(L2_var_sigma_TOA(i_chan,i_beam),i_chan=1,4)
                write(6,'(a50,4f16.8)') 'L2_sigma_TOA [lin]: ',(L2_sigma_TOA(i_chan,i_beam),i_chan=1,4)
                write(6,'(a50,4f16.8)') 'L2_sigma_TOA [dB]: ',(10*log10(L2_sigma_TOA(i_chan,i_beam)),i_chan=1,4)
                write(6,'(a50,4f16.8)') 'L2_KPC_TOA [lin]: ',(L2_KPC_TOA(i_chan,i_beam),i_chan=1,4)
             endif            
          endif
          
          if( L2_tot_sigma_TOA(i_beam) .lt. 0 ) then
             write(6,*) 'WARNING: L2_tot_sigma_TOA negative', i_beam, i_block
          endif

          ! compute KPC of L2 total sigma0 (note L2_tot_sigma_TOA still in linear units...)
          L2_tot_KPC_TOA(i_beam)   = sqrt( L2_var_tot_sigma_TOA(i_beam) /
     $                                     L2_tot_sigma_TOA(i_beam)**2    )

C--Wind speed retrival using ancilliary wind direction information...              
          call L2_get_wind_ancilliary( L2_time,
     $                                 L2_beam_clat(i_beam), 
     $                                 L2_beam_clon(i_beam), 
     $                                 wind_dir_ancilliary(i_beam),
     $                                 wind_speed_ancilliary(i_beam) )

! Scale NCEP speed by 1.03 to agree with RSS 3/29/2011
          wind_speed_ancilliary(i_beam) = wind_speed_ancilliary(i_beam) * 1.03
          
          relazang = L1B_ant_azi_rel_north(i_beam) - wind_dir_ancilliary(i_beam)
          if(relazang.gt. 180.) relazang = relazang - 360.
          if(relazang.le.-180.) relazang = relazang + 360.
!          relazang  = abs(relazang)
          i_sigwave = 2
C--Retrive the wind speed from the total sigma-0, assuming the
C--wind direction and using our model function.
          call L2_retrive_wind_dualpol( L1B_inc_angle(i_beam), relazang, 
     $         i_sigwave, L2_sigma_TOA( 2, i_beam ), L2_sigma_TOA( 4, i_beam ),
     $         L2_KPC_ANT( 2, i_beam ), L2_KPC_ANT( 4, i_beam ),
     $         i_beam, L2_scat_wind_speed(i_beam), wind_speed_ancilliary(i_beam) )   

C--Use the KPC estimates to estimate 1-sigma wind speed error
          tot_sig0_plus_kpc  = L2_tot_sigma_TOA(i_beam) * ( 1 + L2_tot_KPC_TOA(i_beam) )
          tot_sig0_minus_kpc = L2_tot_sigma_TOA(i_beam) * ( 1 - L2_tot_KPC_TOA(i_beam) )
          
          sig0_vv_plus_kpc   = L2_sigma_TOA(2,i_beam ) * ( 1 + L2_KPC_TOA(2,i_beam) )
          sig0_vv_minus_kpc  = L2_sigma_TOA(2,i_beam ) * ( 1 - L2_KPC_TOA(2,i_beam) )
          sig0_hh_plus_kpc   = L2_sigma_TOA(4,i_beam ) * ( 1 + L2_KPC_TOA(4,i_beam) )
          sig0_hh_minus_kpc  = L2_sigma_TOA(4,i_beam ) * ( 1 - L2_KPC_TOA(4,i_beam) )

          call L2_retrive_wind_dualpol( L1B_inc_angle(i_beam), relazang, 
     $         i_sigwave, sig0_vv_plus_kpc, sig0_hh_plus_kpc,
     $         L2_KPC_ANT( 2, i_beam ), L2_KPC_ANT( 4, i_beam ),
     $         i_beam, wind_speed_sig0_plus_kpc, wind_speed_ancilliary(i_beam) )  

          call L2_retrive_wind_dualpol( L1B_inc_angle(i_beam), relazang, 
     $         i_sigwave, sig0_vv_minus_kpc, sig0_hh_minus_kpc,
     $         L2_KPC_ANT( 2, i_beam ), L2_KPC_ANT( 4, i_beam ),
     $         i_beam, wind_speed_sig0_minus_kpc, wind_speed_ancilliary(i_beam) )  

          if( L2_scat_wind_speed( i_beam ) .ge. 0.0d0 .and.   ! set 
     $        wind_speed_sig0_plus_kpc     .ge. 0.0d0 .and.
     $        wind_speed_sig0_minus_kpc    .ge. 0.0d0 ) then
            L2_scat_wind_speed_err( i_beam ) = 0.5 * ( 
     $         abs( wind_speed_sig0_plus_kpc  - L2_scat_wind_speed( i_beam ) ) + 
     $         abs( wind_speed_sig0_minus_kpc - L2_scat_wind_speed( i_beam ) ) )
          else
            L2_scat_wind_speed_err( i_beam ) = 0.0
          endif
          
C--Use the wind speed + wind speed errors to estimate delta Tb + errors
c          call L2_PALS_delta_TB_model( L2_scat_wind_speed( i_beam ),
c     $                                 L2_scat_wind_speed_err( i_beam ),
c     $                                 L1B_inc_angle( i_beam ),
c     $                                 L2_delta_Tb( 1, i_beam ),
c     $                                 L2_delta_Tb_err( 1, i_beam ) )
          
          call L2_delta_emissivity_surface_model( L2_scat_wind_speed( i_beam ),
     $                                            L2_scat_wind_speed_err( i_beam ),
     $                                            relazang,
     $                                            L1B_inc_angle( i_beam ),
     $                                            L2_excess_surf_emiss( 1, i_beam ),
     $                                            L2_excess_surf_emiss_err( 1, i_beam ) )

          call L2_compute_sigma_exp( wind_speed_ancilliary(i_beam),
     $                               relazang,
     $                               L1B_inc_angle( i_beam ),
     $                               L2_land_fraction( i_beam ),
     $                               L2_ice_fraction( i_beam ),
     $                               L2_sigma_exp( 1, i_beam ) )

C--Convert back to dB; set L2 quality flags for negative sigma0... (which sigma-0 to flag!?)

! convert L2_tot_sigma_TOA to dB
          L2_tot_sigma_TOA(i_beam) = 10.0d0*log10( abs( L2_tot_sigma_TOA(i_beam) ) ) 

          do i_chan = 1, 4
            
            L2_sigma_FAR(i_chan,i_beam) = L2_sigma_TOA(i_chan,i_beam) - 
     $                                    L2_sigma_ANT(i_chan,i_beam)
            
            L2_sigma_ANT(i_chan,i_beam) = 10.0d0*log10(abs(L2_sigma_ANT(i_chan,i_beam)))
            L2_sigma_TOI(i_chan,i_beam) = 10.0d0*log10(abs(L2_sigma_TOI(i_chan,i_beam)))
            L2_sigma_TOA(i_chan,i_beam) = 10.0d0*log10(abs(L2_sigma_TOA(i_chan,i_beam)))
          enddo
          
          if( i_dbg_level .ge. 1 ) then
            write(6,*) ' '
            write(6,'((a50),2I5)') '----------L2 Debugging output, i_blk, i_beam: ', i_block, i_beam
            write(6,'((a50),4f12.6)') 'L2_sigma_ANT (HV,VV,VH,HH) [dB]: ', (L2_sigma_ANT(ii,i_beam),ii=1,4)
            write(6,'((a50),4f12.6)') 'L2_KPC_ANT   (HV,VV,VH,HH) [--]: ', (L2_KPC_ANT(ii,i_beam),ii=1,4)
            write(6,'((a50),4f12.6)') 'L2_sigma_TOA (HV,VV,VH,HH) [dB]: ', (L2_sigma_TOA(ii,i_beam),ii=1,4)
            write(6,'((a50),4f12.6)') 'L2_KPC_TOA   (HV,VV,VH,HH) [--]: ', (L2_KPC_TOA(ii,i_beam),ii=1,4)
            write(6,'((a50),f12.6)')  'Faraday Rotation angle [deg]: ', L2_faraday_rotation_angle(i_beam)
            write(6,'((a50),2f12.6)') 'L2_tot_sigma_TOA, L2_tot_KPC_TOA [dB,--]: ', L2_tot_sigma_TOA(i_beam), L2_tot_KPC_TOA(i_beam)
            write(6,'((a50),2f12.6)') 'Inc angle, relazang : ', L1B_inc_angle(i_beam), relazang
            write(6,'((a50),3f12.6)') 'QMET speed, direction, cell azi: ', 
     1                                 wind_speed_ancilliary(i_beam), wind_dir_ancilliary(i_beam),L1B_ant_azi_rel_north(i_beam)
            write(6,'((a50),2f12.6)') 'land fraction, ice fraction: ', L1B_land_fraction(i_beam), L1B_ice_fraction(i_beam)
            write(6,'((a50),2f12.6)') 'Ret. speed, speed error [m/s]: ', 
     1                                 L2_scat_wind_speed(i_beam), L2_scat_wind_speed_err( i_beam )
            write(6,'((a50),4f12.6)') 'Excess Emiss (H,V), Excess Em. error (H,V) [-]: ', 
     1           (L2_excess_surf_emiss(ii,i_beam),ii=1,2), (L2_excess_surf_emiss_err(ii,i_beam),ii=1,2)
            
          endif
        enddo ! do i_beam = 1, 3
        
        call L2_gen_quality_flag( L1B_quality_flag, 
     $                            L2_neg_sig0_ANT, 
     $                            L2_neg_sig0_TOA, 
     $                            L2_scat_rfi_flags, 
     $                            L2_good_far_corr, 
     $                            L2_scatterometer_flags,
     $                            L2_off_nom_no_echo,
     $                            L2_scat_samples )      
                
C--Check for cases where no data was included in L2 averages; replace L2 data with
C--missing data values. 
        do i_beam = 1, 3
          if( L2_scat_samples(i_beam).eq.0) then
            
            !L2_scatterometer_flags( i_beam ) = z"FFFFFFFF"
            L2_land_fraction( i_beam )       = -999
            L2_ice_fraction( i_beam )        = -999
            L2_scat_wind_speed( i_beam )     = -999
            L2_scat_wind_speed_err( i_beam ) = -999
            L2_beam_clat( i_beam )           = -999
            L2_beam_clon( i_beam )           = -999
            L2_polarization_roll( i_beam )   = -999
            L2_faraday_rotation_angle(i_beam)= -999
            L2_tot_KPC_TOA( i_beam )         = -999
            L2_tot_sigma_TOA( i_beam )       = -999
            
            do ii = 1, 4
              L2_beam_elat( ii, i_beam )     = -999
              L2_beam_elon( ii, i_beam )     = -999
              
              if (ii.le.2) then
                L2_excess_surf_emiss(ii,i_beam)     = -999
                L2_excess_surf_emiss_err(ii,i_beam) = -999
              endif
              
              L2_sigma_ANT( ii, i_beam )     = -999
              L2_KPC_ANT( ii, i_beam )       = -999
              L2_sigma_TOA( ii, i_beam )     = -999
              L2_KPC_TOA( ii, i_beam )       = -999
              L2_sigma_Far( ii, i_beam )     = -999      
              L2_sigma_exp( ii, i_beam )     = -999
            enddo
            
          endif
        enddo
        
        
C--Populate an L2 output buffer and write if( i_block .eq. NUM_BLOCKS )
        icds_blknum = icds_processing_status_eu(      4, i_block )
        call L2_populate_output_buffer( i_block,
     $                                  block_start,
     $                                  last_block, !NUM_BLOCKS,
     $                                  icds_blknum,
     $                                  L2_time,
     $                                  L2_scat_wind_speed,
     $                                  L2_scat_wind_speed_err,
     $                                  L2_excess_surf_emiss,
     $                                  L2_excess_surf_emiss_err,     
     $                                  wind_speed_ancilliary,
     $                                  wind_dir_ancilliary,
     $                                  L2_faraday_rotation_angle,
     $                                  L2_beam_clat,
     $                                  L2_beam_clon,
     $                                  L2_beam_elat,
     $                                  L2_beam_elon,
     $                                  L2_sigma_ANT,
     $                                  L2_sigma_TOA,
     $                                  L2_sigma_Far,
     $                                  L2_tot_sigma_TOA,
     $                                  L2_KPC_ANT,
     $                                  L2_KPC_TOA,
     $                                  L2_tot_KPC_TOA,
     $                                  L2_scat_samples,
     $                                  L2_scatterometer_flags,
     $                                  L2_scat_rfi_flags,
     $                                  L2_land_fraction,
     $                                  L2_ice_fraction,
     $                                  L2_polarization_roll,
     $                                  L2_sigma_exp,
     $                                  L2_prefix    )
        

        if( i_dbg_level.ge.1 ) then
!          if( flag_l1a_file_entered .and. i_block.ge.44 ) pause         
!          if( flag_read_sci_prefix_entered ) pause
C          if( flag_read_sci_prefix_entered .and. sequence_time.ge.0.0430562500 ) pause
C           if( flag_read_sci_prefix_entered .and. 
C     $         L1B_land_fraction(1)+L1B_land_fraction(2)+L1B_land_fraction(3) .le. 0.01 .and.
C     $         L1B_ice_fraction(1)+L1B_ice_fraction(2)+L1B_ice_fraction(3) .le. 0.01 ) pause
        endif
        
      enddo     ! do i_block = block_start, NUM_BLOCKS
      end
      
      subroutine L1A_to_L2_usage
      implicit none

      write(6,'(a)') ' '
      write(6,'(a)') 'Required arguments: '
      write(6,'(a)') '  -limits           l1b_limits_file'
      write(6,'(a)') '  -param_file       parameters_file'
      write(6,'(a)') '  -dir_dat          Aquarius_data_directory'
      write(6,'(a)') '  -dir_out          output_directory'
      write(6,'(a)') '  -dir_scratch      scratch_directory'
      write(6,'(a)') ' '
      write(6,'(a)') 'Additional required arguments for L1A input processing: '
      write(6,'(a)') '  -i                input_L1A_file'
      write(6,'(a)') ' '
      write(6,'(a)') 'Additional required arguments for read_sci input processing: '
      write(6,'(a)') '  -read_sci_prefix  read_sci_prefix'
      write(6,'(a)') '  -ephem_XYZ        XYZ_ephemeris_file'
      write(6,'(a)') '  -ephem_RPY        RPY_ephemeris_file'
      write(6,'(a)') ' '
      write(6,'(a)') 'Optional arguments: '
      write(6,'(a)') '  -apc_file     apc_filename   Uses antenna pattern correction in apc_filename.'
      write(6,'(a)') '  -cal_level    {1,2,3}        Sets the temperature calibration level.'
      write(6,'(a)') '  -write_telem  telem_out_dir  Write converted L0 telemetry to ASCII files in telem_out_dir.'
      write(6,'(a)') '  -debug        dbg_level      Turns on various debugging output.'
      write(6,'(a)') '  -l1b                         Turns on L1B HDF-4 output (default is off).'
      write(6,'(a)') '  -suppress_tlm_warnings       Suppresses out-of-range warnings for telemetry.'
      write(6,'(a)') '  -L2_allow_bad                Allows data flagged as bad into the L2 averages.'
      write(6,'(a)') '  -L2_filter_rfi               Rejects data flagged as having RFI from the L2 averages.'
      write(6,'(a)') '  -no_far_corr                 Turns off faraday rotation correction.'
      write(6,'(a)') ' '

      call exit(1)

      return
      end

