C--$Header: /u/caretr2/scatsim/CVSROOT/L1A_to_L2/L1A_Filter_RFI.f,v 1.1 2011/10/17 12:38:42 fore Exp $
c*****************************************************************      
      subroutine L1A_Filter_RFI( RX_power,
     $                           RFI_FLAGS_VPOL,
     $                           RFI_FLAGS_HPOL,
     $                           NUM_BLOCKS,
     $                           VRX_Noise_Power,
     $                           HRX_Noise_Power )
c*****************************************************************      
c**   
c**   FILE NAME: L1A_Filter_RFI.f
c**   
c**   DATE WRITTEN: 10/1/2011
c**   
c**   PROGRAMMER: Alex Fore and Adam Freedman
c**   
c**   FUNCTIONAL DESCRIPTION: Filters noise power and flags for RFI
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1A.inc'   
      include 'L1A_RFI_params.inc'
c     PARAMETER STATEMENTS
      integer HALF_WINDOW_MAX
      parameter( HALF_WINDOW_MAX = 20 )
c     INPUT VARIABLES
      real*8    RX_power( 6, 3, 8, MAX_NUM_BLOCKS ) ! pol, beam, seq, block [dBm]
      integer*4 NUM_BLOCKS

c     Input/Output variables (rfi flags are modified by this subroutine)
      integer*1 RFI_FLAGS_VPOL(3,8,MAX_NUM_BLOCKS)  ! set to 1 if RFI, 0 if no RFI
      integer*1 RFI_FLAGS_HPOL(3,8,MAX_NUM_BLOCKS)  ! set to 1 if RFI, 0 if no RFI

c     OUTPUT VARIABLES
      real*8    HRX_Noise_Power( 3, 8, MAX_NUM_BLOCKS ) ! linear
      real*8    VRX_Noise_Power( 3, 8, MAX_NUM_BLOCKS ) ! linear

c     LOCAL VARIABLES
      integer i_beam
      integer i_seq
      integer seqidx, seqidx_nV
      integer ii
      integer i_block
      integer ierr
      logical cnd_firing(3,8)

      real*8  nH_pwr(3,8*MAX_NUM_BLOCKS) ! copies of RX_power data; in linear; used locally
      real*8  VH_pwr(3,8*MAX_NUM_BLOCKS)
      real*8  HH_pwr(3,8*MAX_NUM_BLOCKS)
      real*8  nV_pwr(3,8*MAX_NUM_BLOCKS)
      real*8  VV_pwr(3,8*MAX_NUM_BLOCKS)
      real*8  HV_pwr(3,8*MAX_NUM_BLOCKS)

      real*8  data_buffer_nH(2*HALF_WINDOW_MAX) ! buffer for median value calculator
      real*8  data_buffer_VH(2*HALF_WINDOW_MAX)
      real*8  data_buffer_HH(2*HALF_WINDOW_MAX)
      real*8  data_buffer_nV(2*HALF_WINDOW_MAX)
      real*8  data_buffer_VV(2*HALF_WINDOW_MAX)
      real*8  data_buffer_HV(2*HALF_WINDOW_MAX)   
      
      real*8  nV_mean, nV_median, nV_std !output from median, mean, std calculator
      real*8  HV_mean, HV_median, HV_std
      real*8  VV_mean, VV_median, VV_std
      real*8  nH_mean, nH_median, nH_std
      real*8  HH_mean, HH_median, HH_std
      real*8  VH_mean, VH_median, VH_std
      
      integer*1 rfi_flag_hpol( 3, 8 * MAX_NUM_BLOCKS )
      integer*1 rfi_flag_vpol( 3, 8 * MAX_NUM_BLOCKS )      
      
c     DATA STATEMENTS
c     COMMON BLOCKS
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      
      if( i_rfi_half_window_size_hrx .gt. HALF_WINDOW_MAX .or.
     $    i_rfi_half_window_size_vrx .gt. HALF_WINDOW_MAX ) then
       write(6,*) 'In L1A_Filter_RFI: Increase HALF_WINDOW_MAX!!'
       stop
      endif
      
      do i_beam=1,3 ! Set cnd_firing array of logicals for later use
        do i_seq=1,8
          cnd_firing(i_beam,i_seq) = .false.
          if( mod(i_seq,2).eq.1 .and. mod(i_beam,2).eq.0 ) cnd_firing(i_beam,i_seq) = .true.
          if( mod(i_seq,2).eq.0 .and. mod(i_beam,2).eq.1 ) cnd_firing(i_beam,i_seq) = .true.
        enddo
      enddo
      
      ! copy noise power and flags into local arrays
      do i_block=1,NUM_BLOCKS
        do i_beam=1,3
          do i_seq=1,8
            seqidx = i_seq + (i_block-1)*8 

            HV_pwr(i_beam,seqidx) = 10.0d0**(0.1d0*RX_power(1,i_beam,i_seq,i_block))
            nV_pwr(i_beam,seqidx) = 10.0d0**(0.1d0*RX_power(2,i_beam,i_seq,i_block))
            VV_pwr(i_beam,seqidx) = 10.0d0**(0.1d0*RX_power(3,i_beam,i_seq,i_block))
            
            VH_pwr(i_beam,seqidx) = 10.0d0**(0.1d0*RX_power(4,i_beam,i_seq,i_block))
            nH_pwr(i_beam,seqidx) = 10.0d0**(0.1d0*RX_power(5,i_beam,i_seq,i_block))
            HH_pwr(i_beam,seqidx) = 10.0d0**(0.1d0*RX_power(6,i_beam,i_seq,i_block))
            
            ! copy RFI flags; ignore V flag when CND is firing
            rfi_flag_hpol(i_beam,seqidx) = z"00"
            rfi_flag_vpol(i_beam,seqidx) = z"00"
            
            if( RFI_FLAGS_HPOL(i_beam,i_seq,i_block).eq.1)
     $        rfi_flag_hpol(i_beam,seqidx) = ibset(rfi_flag_hpol(i_beam,seqidx),0)
            
            if(.not.cnd_firing(i_beam,i_seq).and.RFI_FLAGS_VPOL(i_beam,i_seq,i_block).eq.1 )
     $        rfi_flag_vpol(i_beam,seqidx) = ibset(rfi_flag_vpol(i_beam,seqidx),0)
            
          enddo
        enddo
      enddo
      
      
      do i_block=1,NUM_BLOCKS
        do i_beam=1,3
          do i_seq=1,8
            seqidx = i_seq + (i_block-1)*8
            !------------------------------------------------------------------            
            !--Check noise power thresholds------------------------------------
            !--Set bit 1 if threshold is exceeded.-----------------------------
            !------------------------------------------------------------------            
            if( 10*log10(nH_pwr(i_beam,seqidx)).gt.r_hrx_noise_threshold ) then
              rfi_flag_hpol(i_beam,seqidx) = ibset(rfi_flag_hpol(i_beam,seqidx),1)
            endif
            ! Check noise VRX Power Thresholds
            if( cnd_firing(i_beam,i_seq) ) then
              if( 10*log10(nV_pwr(i_beam,seqidx)).gt.r_vrx_noise_cnd_threshold ) then
                rfi_flag_vpol(i_beam,seqidx) = ibset(rfi_flag_vpol(i_beam,seqidx),1)
              endif
            else
              if( 10*log10(nV_pwr(i_beam,seqidx)).gt.r_vrx_noise_threshold ) then
                rfi_flag_vpol(i_beam,seqidx) = ibset(rfi_flag_vpol(i_beam,seqidx),1)
              endif            
            endif
            !------------------------------------------------------------------            
            !--END Check noise power thresholds--------------------------------
            !------------------------------------------------------------------


            !------------------------------------------------------------------            
            !--Compute mean, median, std for each polarization about 
            !--this sequence.
            !------------------------------------------------------------------            
            ! Fill in data buffers for median value computer
            do ii=1,i_rfi_half_window_size_hrx

              seqidx = max( i_seq + (i_block-1)*8 - ii, 1 )
              data_buffer_VH(ii) = VH_pwr(i_beam,seqidx)
              data_buffer_nH(ii) = nH_pwr(i_beam,seqidx)
              data_buffer_HH(ii) = HH_pwr(i_beam,seqidx)
              
              seqidx = min( i_seq + (i_block-1)*8 + ii, NUM_BLOCKS*8 )
              data_buffer_VH(ii+i_rfi_half_window_size_hrx) = VH_pwr(i_beam,seqidx)
              data_buffer_nH(ii+i_rfi_half_window_size_hrx) = nH_pwr(i_beam,seqidx)
              data_buffer_HH(ii+i_rfi_half_window_size_hrx) = HH_pwr(i_beam,seqidx)
            enddo
            
            do ii=1,i_rfi_half_window_size_vrx
              ! For VRX every other sequence will be of the same type (CND or not CND)
              seqidx_nV = max( i_seq + (i_block-1)*8 - 2*ii, 1 )
              seqidx    = max( i_seq + (i_block-1)*8 -   ii, 1 )
              data_buffer_HV(ii) = HV_pwr(i_beam,seqidx)
              data_buffer_nV(ii) = nV_pwr(i_beam,seqidx_nV)
              data_buffer_VV(ii) = VV_pwr(i_beam,seqidx)
              
              seqidx_nV = min( i_seq + (i_block-1)*8 + 2*ii, NUM_BLOCKS*8 )
              seqidx    = min( i_seq + (i_block-1)*8 +   ii, NUM_BLOCKS*8 )
              data_buffer_HV(ii+i_rfi_half_window_size_vrx) = HV_pwr(i_beam,seqidx)
              data_buffer_nV(ii+i_rfi_half_window_size_vrx) = nV_pwr(i_beam,seqidx_nV)
              data_buffer_VV(ii+i_rfi_half_window_size_vrx) = VV_pwr(i_beam,seqidx)
            enddo
            
            call rfi_stats_(data_buffer_HV,2*i_rfi_half_window_size_vrx,HV_mean,HV_median,HV_std)
            call rfi_stats_(data_buffer_nV,2*i_rfi_half_window_size_vrx,nV_mean,nV_median,nV_std)
            call rfi_stats_(data_buffer_VV,2*i_rfi_half_window_size_vrx,VV_mean,VV_median,VV_std)

            call rfi_stats_(data_buffer_VH,2*i_rfi_half_window_size_hrx,VH_mean,VH_median,VH_std)
            call rfi_stats_(data_buffer_nH,2*i_rfi_half_window_size_hrx,nH_mean,nH_median,nH_std)
            call rfi_stats_(data_buffer_HH,2*i_rfi_half_window_size_hrx,HH_mean,HH_median,HH_std)
            
            HV_std = min( HV_std, r_rfi_filter_max_std )
            nV_std = min( nV_std, r_rfi_filter_max_std )
            VV_std = min( VV_std, r_rfi_filter_max_std )
            VH_std = min( VH_std, r_rfi_filter_max_std )
            nH_std = min( nH_std, r_rfi_filter_max_std )
            HH_std = min( HH_std, r_rfi_filter_max_std )
            
            !------------------------------------------------------------------
            !--END Compute mean, median, std for each polarization about 
            !--this sequence.
            !------------------------------------------------------------------


            !------------------------------------------------------------------
            !--Threshold check of abs(value-median)/std------------------------
            !--Set bit 2 for noise threshold exceeded--------------------------            
            !--Set bit 3 for co-pol threshold exceeded-------------------------
            !--Set bit 4 for cx-pol threshold exceeded-------------------------
            !------------------------------------------------------------------            
            seqidx = i_seq + (i_block-1)*8
            !----V-pol threshold tests
            if( cnd_firing(i_beam,i_seq) ) then
              if( abs(nV_pwr(i_beam,seqidx)-nV_median).gt.r_vrx_noise_cnd_pwr_std_threshold*nV_std )
     $          rfi_flag_vpol(i_beam,seqidx) = ibset(rfi_flag_vpol(i_beam,seqidx),2)
            else
              if( abs(nV_pwr(i_beam,seqidx)-nV_median).gt.r_vrx_noise_pwr_std_threshold*nV_std )
     $          rfi_flag_vpol(i_beam,seqidx) = ibset(rfi_flag_vpol(i_beam,seqidx),2)       
            endif
            
            if( abs(VV_pwr(i_beam,seqidx)-VV_median).gt.r_vrx_echo_pwr_std_threshold*VV_std )
     $        rfi_flag_vpol(i_beam,seqidx) = ibset(rfi_flag_vpol(i_beam,seqidx),3)

            if( abs(HV_pwr(i_beam,seqidx)-HV_median).gt.r_vrx_echo_pwr_std_threshold*HV_std )
     $        rfi_flag_vpol(i_beam,seqidx) = ibset(rfi_flag_vpol(i_beam,seqidx),4)
            
            !----H-pol threshold tests
            if( abs(nH_pwr(i_beam,seqidx)-nH_median).gt.r_hrx_noise_pwr_std_threshold*nH_std )
     $        rfi_flag_hpol(i_beam,seqidx) = ibset(rfi_flag_hpol(i_beam,seqidx),2)

            if( abs(HH_pwr(i_beam,seqidx)-HH_median).gt.r_hrx_echo_pwr_std_threshold*HH_std )
     $        rfi_flag_hpol(i_beam,seqidx) = ibset(rfi_flag_hpol(i_beam,seqidx),3)

            if( abs(VH_pwr(i_beam,seqidx)-VH_median).gt.r_hrx_echo_pwr_std_threshold*VH_std )
     $        rfi_flag_hpol(i_beam,seqidx) = ibset(rfi_flag_hpol(i_beam,seqidx),4)
            !------------------------------------------------------------------            
            !--END Threshold check of abs(value-median)/std--------------------
            !------------------------------------------------------------------

            
            !------------------------------------------------------------------
            !--Replace flagged sequences with surrounding medians, and recompute
            !--means, medians, and stds of the channels.-----------------------
            !------------------------------------------------------------------ 
            ! Fill in data buffers for median value computer
            do ii=1,i_rfi_half_window_size_hrx

              seqidx = max( i_seq + (i_block-1)*8 - ii, 1 )
              if( rfi_flag_hpol(i_beam,seqidx).ne.0 ) then
                data_buffer_VH(ii) = VH_median
                data_buffer_nH(ii) = nH_median
                data_buffer_HH(ii) = HH_median
              else
                data_buffer_VH(ii) = VH_pwr(i_beam,seqidx)
                data_buffer_nH(ii) = nH_pwr(i_beam,seqidx)
                data_buffer_HH(ii) = HH_pwr(i_beam,seqidx)
              endif
              
              seqidx = min( i_seq + (i_block-1)*8 + ii, NUM_BLOCKS*8 )
              if( rfi_flag_hpol(i_beam,seqidx).ne.0 ) then
                data_buffer_VH(ii+i_rfi_half_window_size_hrx) = VH_median
                data_buffer_nH(ii+i_rfi_half_window_size_hrx) = nH_median
                data_buffer_HH(ii+i_rfi_half_window_size_hrx) = HH_median
              else
                data_buffer_VH(ii+i_rfi_half_window_size_hrx) = VH_pwr(i_beam,seqidx)
                data_buffer_nH(ii+i_rfi_half_window_size_hrx) = nH_pwr(i_beam,seqidx)
                data_buffer_HH(ii+i_rfi_half_window_size_hrx) = HH_pwr(i_beam,seqidx)
              endif
            enddo
            
            do ii=1,i_rfi_half_window_size_vrx
              ! For VRX every other sequence will be of the same type (CND or not CND)
              seqidx_nV = max( i_seq + (i_block-1)*8 - 2*ii, 1 )
              seqidx    = max( i_seq + (i_block-1)*8 -   ii, 1 )
              
              if( rfi_flag_vpol(i_beam,seqidx_nV).ne.0 ) then
                data_buffer_nV(ii) = nV_median
              else
                data_buffer_nV(ii) = nV_pwr(i_beam,seqidx_nV)
              endif
              
              if( rfi_flag_vpol(i_beam,seqidx).ne.0 ) then
                data_buffer_HV(ii) = HV_median
                data_buffer_VV(ii) = VV_median
              else
                data_buffer_HV(ii) = HV_pwr(i_beam,seqidx)
                data_buffer_VV(ii) = VV_pwr(i_beam,seqidx)
              endif
              
              seqidx_nV = min( i_seq + (i_block-1)*8 + 2*ii, NUM_BLOCKS*8 )
              seqidx    = min( i_seq + (i_block-1)*8 +   ii, NUM_BLOCKS*8 )

              if( rfi_flag_vpol(i_beam,seqidx_nV).ne.0 ) then
                data_buffer_nV(ii) = nV_median
              else
                data_buffer_nV(ii) = nV_pwr(i_beam,seqidx_nV)
              endif
              
              if( rfi_flag_vpol(i_beam,seqidx).ne.0 ) then
                data_buffer_HV(ii+i_rfi_half_window_size_vrx) = HV_median
                data_buffer_VV(ii+i_rfi_half_window_size_vrx) = VV_median
              else
                data_buffer_HV(ii+i_rfi_half_window_size_vrx) = HV_pwr(i_beam,seqidx)
                data_buffer_VV(ii+i_rfi_half_window_size_vrx) = VV_pwr(i_beam,seqidx)
              endif
            enddo
            
            call rfi_stats_(data_buffer_HV,2*i_rfi_half_window_size_vrx,HV_mean,HV_median,HV_std)
            call rfi_stats_(data_buffer_nV,2*i_rfi_half_window_size_vrx,nV_mean,nV_median,nV_std)
            call rfi_stats_(data_buffer_VV,2*i_rfi_half_window_size_vrx,VV_mean,VV_median,VV_std)

            call rfi_stats_(data_buffer_VH,2*i_rfi_half_window_size_hrx,VH_mean,VH_median,VH_std)
            call rfi_stats_(data_buffer_nH,2*i_rfi_half_window_size_hrx,nH_mean,nH_median,nH_std)
            call rfi_stats_(data_buffer_HH,2*i_rfi_half_window_size_hrx,HH_mean,HH_median,HH_std)

            HV_std = min( HV_std, r_rfi_filter_max_std )
            nV_std = min( nV_std, r_rfi_filter_max_std )
            VV_std = min( VV_std, r_rfi_filter_max_std )
            VH_std = min( VH_std, r_rfi_filter_max_std )
            nH_std = min( nH_std, r_rfi_filter_max_std )
            HH_std = min( HH_std, r_rfi_filter_max_std )
            
            !------------------------------------------------------------------
            !--Threshold check of abs(value-median)/std with recomputed medians, stds
            !------------------------------------------------------------------            
            seqidx = i_seq + (i_block-1)*8
            !----V-pol threshold tests 
            if( cnd_firing(i_beam,i_seq) ) then
              if( abs(nV_pwr(i_beam,seqidx)-nV_median).gt.r_vrx_noise_cnd_pwr_std_threshold*nV_std )
     $          rfi_flag_vpol(i_beam,seqidx) = ibset(rfi_flag_vpol(i_beam,seqidx),2)
            else
              if( abs(nV_pwr(i_beam,seqidx)-nV_median).gt.r_vrx_noise_pwr_std_threshold*nV_std )
     $          rfi_flag_vpol(i_beam,seqidx) = ibset(rfi_flag_vpol(i_beam,seqidx),2)       
            endif
            
            if( abs(VV_pwr(i_beam,seqidx)-VV_median).gt.r_vrx_echo_pwr_std_threshold*VV_std )
     $        rfi_flag_vpol(i_beam,seqidx) = ibset(rfi_flag_vpol(i_beam,seqidx),3)

            if( abs(HV_pwr(i_beam,seqidx)-HV_median).gt.r_vrx_echo_pwr_std_threshold*HV_std )
     $        rfi_flag_vpol(i_beam,seqidx) = ibset(rfi_flag_vpol(i_beam,seqidx),4)
            
            !----H-pol threshold tests
            if( abs(nH_pwr(i_beam,seqidx)-nH_median).gt.r_hrx_noise_pwr_std_threshold*nH_std )
     $        rfi_flag_hpol(i_beam,seqidx) = ibset(rfi_flag_hpol(i_beam,seqidx),2)

            if( abs(HH_pwr(i_beam,seqidx)-HH_median).gt.r_hrx_echo_pwr_std_threshold*HH_std )
     $        rfi_flag_hpol(i_beam,seqidx) = ibset(rfi_flag_hpol(i_beam,seqidx),3)

            if( abs(VH_pwr(i_beam,seqidx)-VH_median).gt.r_hrx_echo_pwr_std_threshold*VH_std )
     $        rfi_flag_hpol(i_beam,seqidx) = ibset(rfi_flag_hpol(i_beam,seqidx),4)
            !------------------------------------------------------------------            
            !--END Threshold check of abs(value-median)/std--------------------
            !------------------------------------------------------------------            
            
            !------------------------------------------------------------------            
            !--Replace Noise RX Power data with median when flagged as RFI.----
            !------------------------------------------------------------------            
            seqidx = i_seq + (i_block-1)*8
            
            if( rfi_flag_hpol(i_beam,seqidx).ne.0 ) then
              HRX_Noise_Power(i_beam,i_seq,i_block) = nH_median
            else
              HRX_Noise_Power(i_beam,i_seq,i_block) = nH_pwr(i_beam,seqidx)            
            endif
            
            if( rfi_flag_vpol(i_beam,seqidx).ne.0 ) then
              VRX_Noise_Power(i_beam,i_seq,i_block) = nV_median
            else
              VRX_Noise_Power(i_beam,i_seq,i_block) = nV_pwr(i_beam,seqidx)            
            endif
            !------------------------------------------------------------------            
            !--END Replace Noise RX Power data with median when flagged as RFI.
            !------------------------------------------------------------------            

            !------------------------------------------------------------------
            !--Copy new RFI flags into the main program memory space-----------
            !------------------------------------------------------------------
            RFI_FLAGS_VPOL(i_beam,i_seq,i_block) = rfi_flag_vpol(i_beam,seqidx)
            RFI_FLAGS_HPOL(i_beam,i_seq,i_block) = rfi_flag_hpol(i_beam,seqidx)
            !------------------------------------------------------------------
            !--Done!-----------------------------------------------------------
            !------------------------------------------------------------------
          enddo
        enddo
      enddo
      
      end
      
      
      
      
      
      
      
