//--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L1A_HDF5_reader.c,v 1.7 2009/10/26 18:47:37 fore Exp $
//==============================================================//
// Copyright (C) 2009, California Institute of Technology.      //
// U.S. Government sponsorship acknowledged.                    //
//==============================================================//

#include <stdio.h>
#include <stdlib.h>
#include "hdf5.h"
#include "hdf5_hl.h"

//--------------All reader functions have these return types-----------------//
// returns  0  if sucessful                                                  //
// returns -1 if can't open file                                             //
// returns -2 if can't read one of the datasets                              //
// returns -3 if can't close the file.                                       //
//---------------------------------------------------------------------------//
int l1a_hdf5_get_data_type_( char *data_type,
                              char *filename, 
                              long int filename_len )
{
  hid_t       file_id;
  int         return_val = 0;
  
  file_id = H5Fopen (filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if( file_id < 0 ) return -1;  
  if( H5LTget_attribute_string(file_id, "/", "Data Type"
      , data_type              ) < 0 ) return_val = -2;
  if( H5Fclose (file_id) < 0 ) return -3;  
  return return_val;
}

int l1a_hdf5_get_rfi_( char* scatter_headers,
                        char* scatter_rfi,
                        char *filename, 
                        long int filename_len )
{
  hid_t       file_id;
  int         return_val = 0;
  
  file_id = H5Fopen (filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if( file_id < 0 ) return -1;  
  
  if( H5LTread_dataset(file_id,"/Raw Aquarius Data/scatter_headers",H5T_NATIVE_UCHAR
                       ,scatter_headers ) < 0 ) return_val = -2;

  if( H5LTread_dataset(file_id,"/Raw Aquarius Data/scatter_rfi",H5T_NATIVE_UCHAR
                       ,scatter_rfi ) < 0 ) return_val = -2;                     
  
  if( H5Fclose (file_id) < 0 ) return -3;  
  return return_val;
}

int l1a_hdf5_get_times_( int* start_year, 
                          int* start_day, 
                          int* start_millisec,
                          int* end_year,   
                          int* end_day,   
                          int* end_millisec,
                          float* orbit_node_longitude,
                          char *filename, 
                          long int filename_len )
{
  hid_t       file_id;
  int         return_val = 0;
  file_id = H5Fopen (filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if( file_id < 0 ) return -1;  
  
  if( H5LTget_attribute_int(file_id, "/", "Start Year"
                            , start_year           ) < 0 ) return_val = -2;
  if( H5LTget_attribute_int(file_id, "/", "Start Day"
                            , start_day            ) < 0 ) return_val = -2;
  if( H5LTget_attribute_int(file_id, "/", "Start Millisec"
                            , start_millisec       ) < 0 ) return_val = -2;
  if( H5LTget_attribute_int(file_id, "/", "End Year"
                            , end_year             ) < 0 ) return_val = -2;
  if( H5LTget_attribute_int(file_id, "/", "End Day"
                            , end_day              ) < 0 ) return_val = -2;
  if( H5LTget_attribute_int(file_id, "/", "End Millisec"
                            , end_millisec         ) < 0 ) return_val = -2; 

  //  Removed 03/31/11 JMG
  //  if( H5LTget_attribute_float(file_id, "/", "Orbit Node Longitude"
  //    , orbit_node_longitude ) < 0 ) return_val = -2;
  
  if( H5Fclose (file_id) < 0 ) return -3;
  return return_val;    
}

//int l1a_hdf5_get_sizes__( int* num_blocks,  // JMG remove double __
int l1a_hdf5_get_sizes_( int* num_blocks, 
                          int* num_xyz_emph, 
                          int* num_att_emph,
                          char *filename, 
                          long int filename_len )
{
  hid_t       file_id;
  int         return_val = 0;
  
  file_id = H5Fopen (filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if( file_id < 0 ) return -1;
  
  if( H5LTget_attribute_int(file_id, "/", "Number of Orbit Vectors"
      , num_xyz_emph ) < 0 ) return_val = -2;
  if( H5LTget_attribute_int(file_id, "/", "Number of Attitude Samples"
      , num_att_emph ) < 0 ) return_val = -2;
  if( H5LTget_attribute_int(file_id, "/", "Number of Blocks"
      , num_blocks ) < 0 ) return_val = -2;
  if( H5Fclose (file_id) < 0 ) return -3;
  return return_val;                 
}



int l1a_hdf5_read_nav_data_( double* orb_time, 
                              double* orb_pos, 
                              double* orb_vel,
                              double* att_time, 
                              double* att_ang, 
                              double* quaternion,
                              int* att_flags, 
                              char *filename, 
                              long int filename_len )
{
  hid_t       file_id;
  int         return_val = 0;
  printf("%s\n",filename);
  
  file_id = H5Fopen (filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if( file_id < 0 ) return -1;
  
  if( H5LTread_dataset_double(file_id,"/Navigation/orb_time"
                               ,orb_time)    < 0 ) return_val = -2;
  if( H5LTread_dataset_double(file_id,"/Navigation/orb_pos"
                               ,orb_pos )    < 0 ) return_val = -2;
  if( H5LTread_dataset_double(file_id,"/Navigation/orb_vel"
                               ,orb_vel )    < 0 ) return_val = -2;
  if( H5LTread_dataset_double(file_id,"/Navigation/att_time"
                               ,att_time )   < 0 ) return_val = -2;
  if( H5LTread_dataset_double(file_id,"/Navigation/att_ang"
                               ,att_ang )    < 0 ) return_val = -2;
  if( H5LTread_dataset_double(file_id,"/Navigation/quaternion"
                               ,quaternion ) < 0 ) return_val = -2;
  if( H5LTread_dataset_int   (file_id,"/Navigation/att_flags"
                               ,att_flags )  < 0 ) return_val = -2;
  if( H5Fclose (file_id) < 0 ) return -3;
  return return_val;    
}

int l1a_hdf5_read_raw_scat_sci_data_( char *scatter_headers,
                                       unsigned short *scatter_pwr,
                                       unsigned short *scatter_loop,
                                       unsigned short *scatter_dc,
                                       char *filename, 
                                       long int filename_len )
{
  hid_t          file_id;
  int         return_val = 0;
  file_id = H5Fopen (filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if( file_id < 0 ) return -1;

  if( H5LTread_dataset(file_id,"/Raw Aquarius Data/scatter_headers",H5T_NATIVE_UCHAR
                       ,scatter_headers ) < 0 ) return_val = -2;
  if( H5LTread_dataset(file_id,"/Raw Aquarius Data/scatter_pwr",H5T_STD_U16BE
                       ,scatter_pwr )     < 0 ) return_val = -2;
  if( H5LTread_dataset(file_id,"/Raw Aquarius Data/scatter_loop",H5T_STD_U16BE
                       ,scatter_loop )    < 0 ) return_val = -2;
  if( H5LTread_dataset(file_id,"/Raw Aquarius Data/scatter_dc",H5T_STD_U16BE
                       ,scatter_dc )      < 0 ) return_val = -2;
  return return_val;  
}

int l1a_hdf5_read_gps_time_tag_( int* gps_time_tag, 
                                  int* time_tag_offset,
                                  char *filename, 
                                  long int filename_len )
{
  hid_t     file_id;
  int         return_val = 0;
  file_id = H5Fopen (filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if( file_id < 0 ) return -1;
  if( H5LTread_dataset(file_id,"/Raw Aquarius Data/gps_time_tag",H5T_NATIVE_INT
                       ,gps_time_tag )     < 0 ) return_val = -2;
  if( H5LTread_dataset(file_id,"/Raw Aquarius Data/time_tag_offset",H5T_NATIVE_INT
                       ,time_tag_offset )  < 0 ) return_val = -2;
  if( H5Fclose (file_id) < 0 ) return -3;
  return return_val;  
}

int l1a_hdf5_read_raw_telemetry_( char *icds_tlm, 
                                   char *temp_tlm, 
                                   char *scatter_tlm,
                                   char *icds_status,
                                   char *filename, 
                                   long int filename_len )
{
  hid_t     file_id;
  int         return_val = 0;
  file_id = H5Fopen (filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if( file_id < 0 ) return -1;

  if( H5LTread_dataset(file_id,"/Raw Aquarius Data/icds_tlm",H5T_NATIVE_UCHAR
                          ,icds_tlm )     < 0 ) return_val = -2;
  if( H5LTread_dataset(file_id,"/Raw Aquarius Data/temp_tlm",H5T_NATIVE_UCHAR
                          ,temp_tlm )     < 0 ) return_val = -2;
  if( H5LTread_dataset(file_id,"/Raw Aquarius Data/scatter_tlm",H5T_NATIVE_UCHAR
                          ,scatter_tlm )  < 0 ) return_val = -2;
  if( H5LTread_dataset(file_id,"/Raw Aquarius Data/icds_status",H5T_NATIVE_UCHAR
                          ,icds_status )  < 0 ) return_val = -2;
  if( H5Fclose (file_id) < 0 ) return -3;
  return return_val;  
}

int l1a_hdf5_read_block_times_( double *blk_sec,
                                 char *filename, 
                                 long int filename_len )
{
  hid_t     file_id;
  int         return_val = 0;
  file_id = H5Fopen (filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if( file_id < 0 ) return -1;

  if( H5LTread_dataset_double(file_id,"/Block Attributes/blk_sec"
                             ,blk_sec )     < 0 )return_val = -2;
  if( H5Fclose (file_id) < 0 ) return -3;
  return return_val;  
}
