C--$Header: /u/caretr2/scatsim/CVSROOT/L1A_to_L2/L1A_check_block_timetags.f,v 1.2 2012/03/07 16:51:39 fore Exp $
c*****************************************************************      
      subroutine L1A_check_block_timetags( blk_sec,
     $                                     gps_time_tag,
     $                                     time_tag_offset,
     $                                     icds_processing_status_eu,
     $                                     i_block,
     $                                     block_start,
     $                                     NUM_BLOCKS,
     $                                     bad_block_id_flag,
     $                                     bad_time_tag_flag )
c*****************************************************************      
c**   
c**   FILE NAME: L1A_check_block_timetags.f
c**   
c**   DATE WRITTEN: 12/9/2011
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  bad_block_id_flag should
c**     set L1B_QF_HARDWARE_PROBLEM_OTHER in 
c**     L1B_gen_quality_flag.f
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: Checks for continous last block IDs, checks
c**     for continuity in time-tags, checks for correctness
c**     of L1A converted time-tag from GPS L0 time-tags.
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     3/6/2012         Bug Fix        
c*****************************************************************   
      implicit none
      include 'L1A.inc'
      include 'L1B.inc'
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8    blk_sec( MAX_NUM_BLOCKS )
      real*8    prev_block_time
      real*8    curr_block_time
      real*8    next_block_time
      integer*4 gps_time_tag     ! L0 gps time tag integer (units of seconds).
      integer*4 time_tag_offset  ! L0 gps time tag offset (units of -62.5 nanosec).
      integer*4 icds_processing_status_eu(4, MAX_NUM_BLOCKS )
      integer*4 i_block          ! which block to test
      integer*4 block_start      ! 1st valid block index
      integer*4 NUM_BLOCKS       ! number of valid blocks in L1A data
c     OUTPUT VARIABLES
      logical   bad_block_id_flag
      logical   bad_time_tag_flag
c     LOCAL VARIABLES
      real*8    r_gpssec
      real*8    r_juldate
      real*8    r_juldatelps
      integer   i_yyyy, i_mm, i_dd
      integer   i_hh, i_mn 
      real*8    r_ss
      integer   i_doy
      real*8    r_sod      
      
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      
      bad_block_id_flag = .false.
      bad_time_tag_flag = .false.
      
      if( i_block .gt. 1 ) then
        if( icds_processing_status_eu(4,i_block) .ne.
     $      icds_processing_status_eu(4,i_block-1)+1 ) then
          bad_block_id_flag = .true.
        endif
        
        ! Check if the time-tags have rolled over back to zero from 86400
        prev_block_time = blk_sec(i_block-1)
        if(prev_block_time.lt.blk_sec(block_start)) prev_block_time = prev_block_time + 86400.0d0
        
        curr_block_time = blk_sec(i_block)
        if(curr_block_time.lt.blk_sec(block_start)) curr_block_time = curr_block_time + 86400.0d0
        
        if( i_block.ne.NUM_BLOCKS ) then
          next_block_time = blk_sec(i_block+1)
          if(next_block_time.lt.blk_sec(block_start)) next_block_time = next_block_time + 86400.0d0
        endif    
        
        if(abs(curr_block_time-prev_block_time-1.44d0).gt.TIME_TAG_ERROR_limits(2))then
          ! check if just one out of order, if so fix it and
          if( i_block.ne.NUM_BLOCKS .and. 
     $     abs(next_block_time-prev_block_time-2.88d0).lt.TIME_TAG_ERROR_limits(2) )then
            blk_sec(i_block) = blk_sec(i_block-1) + 1.44d0
          else
            bad_time_tag_flag = .true.
          endif
        endif
      endif

C--Compute GPS time tag from L0 data      
      r_gpssec = dfloat( gps_time_tag ) - .0000000625d0 * dfloat( time_tag_offset )

C--Compute second-of-day in UTC time (r_sod)      
      call gps2time(r_gpssec,r_juldate,r_juldatelps,
     &   i_yyyy,i_mm,i_dd,i_hh,i_mn,r_ss,i_doy,r_sod)

C--Compare r_sod to the blk_sec time tag given in the L1A HDF file.      
      if( abs( blk_sec(i_block) - r_sod ) .gt. TIME_TAG_ERROR_limits(2) )then
        bad_time_tag_flag = .true.
      endif
      
      return
      end