C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L1A_check_time_tags.f,v 1.6 2009/10/05 14:30:08 fore Exp $
c*****************************************************************      
      function L1A_check_time_tags( blk_sec,
     $                              gps_time_tag,
     $                              time_tag_offset )
c*****************************************************************      
c**   
c**   FILE NAME: L1A_check_time_tags.f
c**   
c**   DATE WRITTEN: 9/1/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  Compares the blk_sec UTC time tag
c**     to the GPS->UTC conversion of L0 instrument gps time tag
c**     and time tag offset integers.
c**    
c**     Returns 0 if difference is less than the threshold in 
c**     L1B_limits.txt, and 1 if difference is larger.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1B.inc'
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8  blk_sec          ! L1A UTC block time tag
      integer gps_time_tag     ! L0 gps time tag integer (units of seconds).
      integer time_tag_offset  ! L0 gps time tag offset (units of -62.5 nanosec).

c     OUTPUT VARIABLES
      integer L1A_check_time_tags
      
c     LOCAL VARIABLES
      real*8 r_gpssec
      
      real*8 r_juldate
      real*8 r_juldatelps
      integer i_yyyy, i_mm, i_dd
      integer i_hh, i_mn 
      real*8 r_ss
      integer i_doy
      real*8 r_sod      
      
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS


      L1A_check_time_tags = 0  ! Default return value of function

C--Compute GPS time tag from L0 data      
      r_gpssec = dfloat( gps_time_tag ) - .0000000625d0 * dfloat( time_tag_offset )

C--Compute second-of-day in UTC time (r_sod)      
      call gps2time(r_gpssec,r_juldate,r_juldatelps,
     &   i_yyyy,i_mm,i_dd,i_hh,i_mn,r_ss,i_doy,r_sod)

C--Compare r_sod to the blk_sec time tag given in the L1A HDF file.      
      if( abs( blk_sec - r_sod ) .gt. TIME_TAG_ERROR_limits(2) )then
        L1A_check_time_tags = 1
      endif
      
      if( i_dbg_level .ge. 1 ) then
        write(6,'((a50),3f12.6)') 'L1A_check_time_tags: blk_sec, r_sod, diff: ',
     $              blk_sec, r_sod, blk_sec - r_sod
      endif    
      return
      end