C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L1A_load_QMET_winds_from_HDF.f,v 1.6 2010/07/13 14:09:44 fore Exp $
c*****************************************************************      
      subroutine L1A_load_QMET_winds_from_HDF( 
     $ QMET_file, u_wind, v_wind )
c*****************************************************************      
c**   
c**   FILE NAME: L1A_load_QMET_winds_from_HDF.f
c**   
c**   DATE WRITTEN: 8/12/2009
c**   
c**   PROGRAMMER:
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG: Updated 7/13/2010 to be more flexible and
c**    query for the sds_index based on the known sds name.
c**    Will issue stop if an error occurs during reading of data.
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'hdf.inc'
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      character*320 QMET_file
c     OUTPUT VARIABLES
      real*4 u_wind(0:359,-90:90), v_wind(0:359,-90:90)
c     LOCAL VARIABLES
      integer*4    sd_id
      integer*4    sds_id
      integer      ierr
      integer      start(2)
      integer      stride(2)
      integer      edge(2)
      integer      num_datasets
      integer      num_global_attrs
!      integer      i_lat, i_lon
      integer      i_len
      integer      ii
      integer      rank
      integer      data_type
      integer      num_attrs
      integer      dimsizes( 32 )
      character*64 sds_name
      integer      sds_index
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level

c     FUNCTION STATEMENTS
      integer sfstart
      integer sffinfo
      integer sfselect
      integer sfginfo
      integer sfrdata
      integer sfendacc
      integer sfend
      integer sfn2index
      integer lnblnk
c     PROCESSING STEPS
      
      start(1) = 0
      start(2) = 0
      
      stride(1) = 1
      stride(2) = 1
      
      edge(1) = 360
      edge(2) = 181
      
      sd_id  = sfstart( QMET_file, DFACC_RDONLY )
      if( sd_id .eq. -1 ) then
        write(6,*) 'ERROR: Unable to open HDF4 file: ', QMET_file(1:lnblnk(QMET_file))
        call exit(1)
      endif
      
      ierr      = sffinfo( sd_id, num_datasets, num_global_attrs )
      sds_index = sfn2index( sd_id, 'u_wind' )
      if( sds_index .eq. -1 ) then
        write(6,*) 'ERROR: Unable to find u_wind in HDF4 file: ', 
     $   QMET_file(1:lnblnk(QMET_file))
        call exit(1)
      endif
      
      sds_id    = sfselect(sd_id, sds_index)
      if( sds_id .eq. -1 ) then
        write(6,*) 'ERROR: Unable to select u_wind in HDF4 file: ', 
     $   QMET_file(1:lnblnk(QMET_file))
        call exit(1)
      endif
      
      ierr      = sfginfo( sds_id, sds_name, rank, dimsizes, data_type, num_attrs )
      i_len     = lnblnk( sds_name )
      
      if( i_dbg_level.ge.2 ) then
        write(6,*) 'L1A_load_QMET_winds_from_HDF: sds # ',sds_index,' info: '
        write(6,*) 'sds_name, rank, dimsizes, data_type, num_attrs: ',
     $              sds_name(1:i_len), rank, (dimsizes(ii),ii=1,rank), 
     $              data_type, num_attrs
      endif
      
      if( sds_name(1:i_len) .ne. 'u_wind' ) then
        write(6,*) 'ERROR: L1A_load_QMET_winds_from_HDF: sds_id ',sds_index,' sds_name != u_wind!'
        call exit(1)
      endif

      ierr   = sfrdata( sds_id, start, stride, edge, u_wind )
      if( ierr .eq. -1 ) then
        write(6,*) 'ERROR: Unable to read u_wind in HDF4 file: ', 
     $   QMET_file(1:lnblnk(QMET_file))
        call exit(1)
      endif
      
      ierr   = sfendacc(sds_id)
      if( ierr .eq. -1 ) then
        write(6,*) 'ERROR: Unable to close sds u_wind in HDF4 file: ', 
     $   QMET_file(1:lnblnk(QMET_file))
        call exit(1)
      endif 
      
      sds_index = sfn2index( sd_id, 'v_wind' )
      if( sds_index .eq. -1 ) then
        write(6,*) 'ERROR: Unable to find v_wind in HDF4 file: ', 
     $   QMET_file(1:lnblnk(QMET_file))
        call exit(1)
      endif
      
      sds_id    = sfselect(sd_id, sds_index)
      if( sds_id .eq. -1 ) then
        write(6,*) 'ERROR: Unable to select v_wind in HDF4 file: ', 
     $   QMET_file(1:lnblnk(QMET_file))
        call exit(1)
      endif  

      ierr      = sfginfo( sds_id, sds_name, rank, dimsizes, data_type, num_attrs )
      i_len     = lnblnk( sds_name )

      if( i_dbg_level.ge.2 ) then
        write(6,*) 'L1A_load_QMET_winds_from_HDF: sds # ',sds_index,' info: '
        write(6,*) 'sds_name, rank, dimsizes, data_type, num_attrs: ',
     $              sds_name(1:i_len), rank, (dimsizes(ii),ii=1,rank), 
     $              data_type, num_attrs
      endif

      if( sds_name(1:i_len) .ne. 'v_wind' ) then
        write(6,*) 'ERROR: L1A_load_QMET_winds_from_HDF: sds_id ',sds_index,' sds_name != v_wind!'
        call exit(1)
      endif

      ierr   = sfrdata( sds_id, start, stride, edge, v_wind )
      if( ierr .eq. -1 ) then
        write(6,*) 'ERROR: Unable to read v_wind in HDF4 file: ', 
     $   QMET_file(1:lnblnk(QMET_file))
        call exit(1)
      endif
      ierr   = sfendacc(sds_id)
      if( ierr .eq. -1 ) then
        write(6,*) 'ERROR: Unable to close sds v_wind in HDF4 file: ', 
     $   QMET_file(1:lnblnk(QMET_file))
        call exit(1)
      endif 
      
      ierr   = sfend(sd_id)
      if( ierr .eq. -1 ) then
        write(6,*) 'ERROR: Unable to close HDF4 file: ', QMET_file(1:lnblnk(QMET_file))
        call exit(1)
      endif
      
      
C      write(6,*) 'HDF file: ', QMET_file(1:lnblnk(QMET_file))
C      
C      do i_lat=-90,90
C        do i_lon=0,359
C          write(70,*) i_lat, i_lon, u_wind(i_lon,i_lat),v_wind(i_lon,i_lat)
C        enddo
C      enddo
C      pause
      end
      
