C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L1A_load_scat_sci.f,v 1.5 2009/10/05 14:30:08 fore Exp $
c*****************************************************************      
      subroutine L1A_load_scat_sci( L1A_HDF_file, RX_power, LB_power,
     $ DC_offset, L0_scat_headers )
c*****************************************************************      
c**   
c**   FILE NAME: L1A_load_scat_sci.f
c**   
c**   DATE WRITTEN: 8/11/2009
c**   
c**   PROGRAMMER:
c**   
c**   FUNCTIONAL DESCRIPTION:  Loads in the scat sci data and does
c**     the dn -> eu conversions. Reads in (RX_power, LB_power,
c**     DC_offset) arrays.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1A.inc'
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      character*320 L1A_HDF_file
c     OUTPUT VARIABLES

      real*8    RX_power( 6, 3, 8, MAX_NUM_BLOCKS ) ! pol, beam, seq, block
      real*8    LB_power( 6, 3,    MAX_NUM_BLOCKS ) ! pol, beam,      block
      real*8    DC_offset(      2, MAX_NUM_BLOCKS ) ! pol,            block
      integer*1 L0_scat_headers(8, MAX_NUM_BLOCKS ) !            seq, block
      
c     LOCAL VARIABLES
      
      integer ierr
      
      integer i_block
      integer i_seq
      integer i_beam
      integer i_pol

      integer lnblnk
      
      integer*4 itemp
      real*4 r4_value
      
c     L1A Attributes
      integer NUM_XYZ_EMPH
      integer NUM_ATT_EMPH
      integer NUM_BLOCKS      

c     Raw Scat Sci Data
      integer*1 scatter_headers(       8, MAX_NUM_BLOCKS )
      integer*1 scatter_pwr(  2, 6, 3, 8, MAX_NUM_BLOCKS )
      integer*1 scatter_loop( 2, 6, 3,    MAX_NUM_BLOCKS )
      integer*1 scatter_dc(   2, 2,       MAX_NUM_BLOCKS )

c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level
      
c     FUNCTION STATEMENTS
      real*4 convert_to_milliwatts
      real*4 convert_to_dcv
      real*4 decode_scat_science
      real*4 convert_to_db

c     PROCESSING STEPS

      ierr = L1A_HDF5_get_sizes( NUM_BLOCKS, NUM_XYZ_EMPH, NUM_ATT_EMPH, 
     $ L1A_HDF_file( 1:lnblnk(L1A_HDF_file) ) // char(0) )
      if(ierr.lt.0) write(6,*) 'L1A_load_scat_sci: ERROR: L1A_HDF5_get_sizes returned: ', ierr
      
      ierr = L1A_HDF5_read_raw_scat_sci_data( scatter_headers, scatter_pwr,
     $ scatter_loop, scatter_dc, 
     $ L1A_HDF_file( 1:lnblnk(L1A_HDF_file) ) // char(0) )
      if(ierr.lt.0) write(6,*) 'L1A_load_scat_sci: ERROR: L1A_HDF5_read_raw_scat_sci_data returned: ', ierr
      
      do i_block=1,NUM_BLOCKS
        do i_seq=1,8
          
          L0_scat_headers( i_seq, i_block ) = scatter_headers( i_seq, i_block )
          
          do i_beam=1,3
            do i_pol=1,6

C--Normal channels              
              call return_2_z( scatter_pwr(1,i_pol,i_beam,i_seq,i_block), itemp )
              r4_value = 
     $         convert_to_db( 
     $           convert_to_milliwatts( 
     $             decode_scat_science( itemp) ), (i_beam-1)*3+i_pol )
              RX_power(i_pol,i_beam,i_seq,i_block) = r4_value

C--Loopback
              if( i_seq .eq. 1 ) then
                call return_2_z( scatter_loop(1,i_pol,i_beam,i_block), itemp )
                r4_value = 
     $           convert_to_db( 
     $             convert_to_milliwatts( 
     $               decode_scat_science( itemp) ), (i_beam-1)*3+i_pol )
                LB_power(i_pol,i_beam,i_block) = r4_value
              endif
            enddo
          enddo
        enddo

C--DC offsets
        call return_2_z( scatter_dc(1,1,i_block), itemp )
        r4_value = convert_to_dcv( decode_scat_science( itemp) )
        DC_offset( 1, i_block ) = r4_value
      
        call return_2_z( scatter_dc(1,2,i_block), itemp )
        r4_value = convert_to_dcv( decode_scat_science( itemp) )
        DC_offset( 2, i_block ) = r4_value

      enddo      
      end
