C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L1A_load_telemetry.f,v 1.8 2009/10/27 20:27:24 fore Exp $
c*****************************************************************      
      subroutine L1A_load_telemetry( L1A_HDF_file,
     $                               external_temperatures_eu,
     $                               scat_telemetry_eu,
     $                               icds_engineering_telemetry_eu,
     $                               icds_processing_status_eu )
c*****************************************************************      
c**   
c**   FILE NAME:  L1A_load_telemetry.f
c**   
c**   DATE WRITTEN: 8/11/2009
c**   
c**   PROGRAMMER:
c**   
c**   FUNCTIONAL DESCRIPTION:  Reads in the temp_telem and
c**     scatter_tlm headers from the L1A HDF file and does
c**     the dn -> eu conversions.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG: Updated 8/20/2009 to read icds_status L0 data.
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1A.inc'
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      character*320 L1A_HDF_file
c     OUTPUT VARIABLES
      real*4    external_temperatures_eu(      38, MAX_NUM_BLOCKS )
      real*4    scat_telemetry_eu(             38, MAX_NUM_BLOCKS )
      real*4    icds_engineering_telemetry_eu( 24, MAX_NUM_BLOCKS )
      integer*4 icds_processing_status_eu(      4, MAX_NUM_BLOCKS )
      
c     LOCAL VARIABLES
      integer ierr
      integer i_block
      integer itemp

      integer lnblnk
      
c     L1A Attributes
      integer NUM_XYZ_EMPH
      integer NUM_ATT_EMPH
      integer NUM_BLOCKS      
      
C     Raw Telemetry Fields 
      integer*1    icds_tlm( 24, MAX_NUM_BLOCKS )
      integer*1    temp_tlm( 70, MAX_NUM_BLOCKS )
      integer*1 scatter_tlm( 37, MAX_NUM_BLOCKS )
      integer*1 icds_status( 12, MAX_NUM_BLOCKS )
      
      

c     DATA STATEMENTS
c     COMMON BLOCKS
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      
      ierr = L1A_HDF5_get_sizes( NUM_BLOCKS, NUM_XYZ_EMPH, NUM_ATT_EMPH, 
     $ L1A_HDF_file( 1:lnblnk(L1A_HDF_file) ) // char(0) )
      if(ierr.lt.0) write(6,*) 'L1A_load_telemetry: ERROR: L1A_HDF5_get_sizes returned: ', ierr      
      
      ierr = L1A_HDF5_read_raw_telemetry( icds_tlm, temp_tlm, scatter_tlm, 
     $ icds_status,
     $ L1A_HDF_file( 1:lnblnk(L1A_HDF_file) ) // char(0) )      
      if(ierr.lt.0) write(6,*) 'L1A_load_telemetry: ERROR: read_raw_telemetry returned: ', ierr
      
      do i_block=1, NUM_BLOCKS
        
        call L1A_convert_icds_tlm_dn_to_eu( icds_tlm(1,i_block), 
     $   icds_engineering_telemetry_eu(1,i_block) )
        
        call L1A_convert_scat_telem_dn_to_eu( scatter_tlm(1,i_block),
     $   icds_engineering_telemetry_eu(1,i_block),
     $   scat_telemetry_eu(1,i_block) )
        
        call L1A_convert_external_temperatures_dn_to_eu(
     $   temp_tlm(1,i_block),
     $   icds_engineering_telemetry_eu(1,i_block),
     $   external_temperatures_eu(1,i_block) )
     
C--Convert icds_status to EU    
        call return_4_z( icds_status(1,i_block), itemp ) ! gps time @ last boot
        icds_processing_status_eu(1,i_block) = itemp

        call return_2_z( icds_status(5,i_block), itemp ) ! cmd RX counter
        icds_processing_status_eu(2,i_block) = itemp
        
        call return_2_z( icds_status(7,i_block), itemp ) ! last cmd
        icds_processing_status_eu(3,i_block) = itemp
        
        call return_4_z( icds_status(9,i_block), itemp ) ! last block ID
        icds_processing_status_eu(4,i_block) = itemp
        
      enddo
      end

c*****************************************************************      
      subroutine L1A_convert_external_temperatures_dn_to_eu(
     $ temp_tlm,
     $ icds_engineering_telemetry_eu,
     $ external_temperatures_eu)
c*****************************************************************      
c**   
c**   FILE NAME: L1A_load_telemetry.f
c**   
c**   DATE WRITTEN: 8/11/2009
c**   
c**   PROGRAMMER:
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES:  Based on "load_ext_temperatures" and 
c**    "convert_external_temperatures" in read_sci_v75.f
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1A.inc'
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      integer*1 temp_tlm( 70 )
      real*4    icds_engineering_telemetry_eu( 24 )
      
c     OUTPUT VARIABLES
      real*4    external_temperatures_eu( 38 )
      
c     LOCAL VARIABLES
      integer*4 external_temperatures_dn( 38 )

      integer*4 itemp
      integer*4 ii
      
      real*4 ext_prt_constants(60,3)
      real*8 r0, a, b
      real*4 resistance2, temperature, gain, offset, resistance3, resistance
      
      logical prt_table_loaded /.false./
      
c     DATA STATEMENTS
c     COMMON BLOCKS
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      save prt_table_loaded, ext_prt_constants
      
      do ii=1,6  ! OMT probe temp (1H,1V,2H,2V,3H,3V)
        call return_2_z( temp_tlm(2*ii-1), itemp ) ! 1st -- 12th byte of temp_tlm
        external_temperatures_dn( ii ) = itemp
      enddo

      do ii=1,6 ! diplexer to SFE coax (H1,H2,H3,V1,V2,V3)
        call return_1_z( temp_tlm(12+ii), itemp ) ! 13th -- 18th byte of temp_tlm
        external_temperatures_dn( 6+ii ) = itemp    
      enddo
            
      do ii=1,6 ! diplexer (1H,1V,2H,2V,3H,3V)
        call return_2_z( temp_tlm(18+2*ii-1), itemp )
        external_temperatures_dn( 12+ii ) = itemp
      enddo

      do ii=1,6 ! coupler (1H,1V,2H,2V,3H,3V)
        call return_2_z( temp_tlm(30+2*ii-1), itemp )
        external_temperatures_dn( 18+ii ) = itemp
      enddo
      
      do ii=1,3 ! CND (1,2,3)
        call return_2_z( temp_tlm(42+2*ii-1), itemp )
        external_temperatures_dn( 24+ii ) = itemp
      enddo
      
      do ii=1,8 ! reflector temperatures (1,2,3,4,5,6,7,8)
        call return_2_z( temp_tlm(48+2*ii-1), itemp )
        external_temperatures_dn( 27+ii ) = itemp
      enddo
      
      do ii=1,3 
        call return_2_z( temp_tlm(64+2*ii-1), itemp )
        external_temperatures_dn( 35+ii ) = itemp
      enddo
C--Done loading external_temperatures into integer*4 containers.      

C--Load the prt_constants table
      if( .not.prt_table_loaded )then
        call read_ext_prt_constants(ext_prt_constants)
        prt_table_loaded = .true.
      endif
      
      do ii=1,6 ! OMT probe temp (1H,1V,2H,2V,3H,3V)
        r0          = ext_prt_constants(ii,1)
        a           = ext_prt_constants(ii,2)
        b           = ext_prt_constants(ii,3)
        gain        = icds_engineering_telemetry_eu(21)
        offset      = icds_engineering_telemetry_eu(22)
        resistance2 = external_temperatures_dn(ii) * gain + offset
        
        call convert_resistance_to_temperature_ext(resistance2, r0, a, b, temperature)
        external_temperatures_eu(ii) = temperature
      enddo
      
      do ii=7,12 ! diplexer to SFE coax (H1,H2,H3,V1,V2,V3)
        call convert_dn_to_temperature_8_bit_norm(
     $   external_temperatures_dn(ii), resistance, temperature)
        
        r0 = ext_prt_constants(ii,1)
        a  = ext_prt_constants(ii,2)
        b  = ext_prt_constants(ii,3)
        
        call convert_resistance_to_temperature_ext(resistance, r0, a, b, temperature)
        external_temperatures_eu(ii) = temperature
      enddo
      
      do ii=13,27 ! diplexer (1H,1V,2H,2V,3H,3V), and
                  ! coupler (1H,1V,2H,2V,3H,3V), CND (1,2,3)
        r0          = ext_prt_constants(ii,1)
        a           = ext_prt_constants(ii,2)
        b           = ext_prt_constants(ii,3)
        gain        = icds_engineering_telemetry_eu(21)
        offset      = icds_engineering_telemetry_eu(22)    
        resistance2 = external_temperatures_dn(ii) * gain + offset
        
        call convert_resistance_to_temperature_ext(resistance2, r0, a, b, temperature)
        external_temperatures_eu(ii) = temperature        
      enddo
      
      do ii=28,35 ! reflector temps
        r0          = ext_prt_constants(ii,1)
        a           = ext_prt_constants(ii,2)
        b           = ext_prt_constants(ii,3)
        gain        = icds_engineering_telemetry_eu(21)
        offset      = icds_engineering_telemetry_eu(22)    
        resistance2 = external_temperatures_dn(ii) * gain + offset
        resistance3 = resistance2 - ext_prt_constants(39 - 28 + ii, 1)
        
        call convert_resistance_to_temperature_ext(resistance3, r0, a, b, temperature)
        external_temperatures_eu(ii) = temperature
      enddo
      
      do ii=36,38 ! feeds (1,2,3)
        r0          = ext_prt_constants(ii,1)
        a           = ext_prt_constants(ii,2)
        b           = ext_prt_constants(ii,3)
        gain        = icds_engineering_telemetry_eu(21)
        offset      = icds_engineering_telemetry_eu(22)    
        resistance2 = external_temperatures_dn(ii) * gain + offset
        
        call convert_resistance_to_temperature_ext(resistance2, r0, a, b, temperature)
        external_temperatures_eu(ii) = temperature    
      enddo
      
      end
      

c*****************************************************************      
      subroutine L1A_convert_scat_telem_dn_to_eu( 
     $  scatter_tlm,
     $  icds_engineering_telemetry_eu,
     $  scat_telemetry_eu )
c*****************************************************************      
c**   
c**   FILE NAME: L1A_load_telemetry.f
c**   
c**   DATE WRITTEN:  8/11/2009
c**   
c**   PROGRAMMER:
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: Based on subroutine "load_scat_telemetry" and
c**    "convert_scat_telemetry" in read_sci_v75.f
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      integer*1 scatter_tlm( 37 )
      real*4    icds_engineering_telemetry_eu( 24 )
      
c     OUTPUT VARIABLES
      real*4    scat_telemetry_eu( 38 )

c     LOCAL VARIABLES
      integer*4 scat_telemetry( 38 )
      integer*4 itemp
      
      integer*4 i_dn
      real*4    resistance, temperature, offset, gain, resistance2
      real*8    r0, a, b
      
      integer*4 ii, jk, jkk, jkj, jjk

      real*4    scat_prt_constants(20,3)
      
      logical   prt_table_loaded /.false./
      
c     DATA STATEMENTS
c     COMMON BLOCKS
c     FUNCTION STATEMENTS
      real*4    decode_scat_science
      real*4    convert_to_dcv
      real*4    convert_to_db
      real*4    convert_to_milliwatts
      real*4    ie_volt
      
c     PROCESSING STEPS
      save scat_prt_constants, prt_table_loaded
      
      
C--1st load the scat telem DNs into the integer*4 array scat_telemetry
      call return_2_z( scatter_tlm(1), itemp ) ! loopback B1HV DC
      scat_telemetry(1) = itemp

      call return_2_z( scatter_tlm(3), itemp ) ! loopback B1HV power
      scat_telemetry(2) = itemp

      call return_1_z( scatter_tlm(5), itemp ) ! status flags

      do ii=0,7                                ! initialize all flags to zero
        scat_telemetry(3+ii) = 0
      enddo

      if( iand(itemp,z"00000001") .gt. 0)  scat_telemetry( 3  ) = 1 ! CG latchup redundant
      if( iand(itemp,z"00000002") .gt. 0)  scat_telemetry( 4  ) = 1 ! CG latchup prime
      if( iand(itemp,z"00000004") .gt. 0)  scat_telemetry( 5  ) = 1 ! PLM 2 lock
      if( iand(itemp,z"00000008") .gt. 0)  scat_telemetry( 6  ) = 1 ! PLM 1 lock
      if( iand(itemp,z"00000010") .gt. 0)  scat_telemetry( 7  ) = 1 ! scat LVPS bus voltage flag
      if( iand(itemp,z"00000020") .gt. 0)  scat_telemetry( 8  ) = 1 ! scat LVPS overcurrent flag
      if( iand(itemp,z"00000040") .gt. 0)  scat_telemetry( 9  ) = 1 ! FIFO error underflow
      if( iand(itemp,z"00000080") .gt. 0)  scat_telemetry( 10 ) = 1 ! FIFO error
      
      do ii=1,7 ! inside LVPS box temp -- SFE TX load temp
        call return_1_z( scatter_tlm(5+ii), itemp )
        scat_telemetry(10+ii) = itemp
      enddo
      
      call return_2_z( scatter_tlm(13), itemp ) ! SBE Step attenuator temp
      scat_telemetry(18) = itemp
      
      call return_2_z( scatter_tlm(15), itemp ) ! SFE loopback attenuator temp
      scat_telemetry(19) = itemp

      call return_2_z( scatter_tlm(17), itemp ) ! SFE loopback switch temp
      scat_telemetry(20) = itemp

      call return_2_z( scatter_tlm(19), itemp ) ! SFE beam switch temp
      scat_telemetry(21) = itemp

      do ii=1,17 ! SFE TX power monitor -- SBE +12V monitor (non-switched)
        call return_1_z( scatter_tlm(20+ii), itemp)
        scat_telemetry(21+ii) = itemp
      enddo
      
C--done loading scat_telemetry into integer*4 containers    

C-------------------------------------------------------------------------------------
C--Remainer of this subroutine was based on "convert_scat_telemetry" in read_sci_v75.f
C--Read in the scat prt constants      
      if( .not.prt_table_loaded )then
        call read_scat_prt_constants(scat_prt_constants)
        prt_table_loaded = .true.
      endif

      scat_telemetry_eu( 1 ) = convert_to_dcv( 
     $  decode_scat_science( scat_telemetry( 1 ) ) )
      
      
      scat_telemetry_eu( 2 ) = convert_to_db( 
     $ convert_to_milliwatts( decode_scat_science( scat_telemetry( 2 ) ) ), 0 )
     
C-- scat_telemetry(3) through scat_telemetry(10) are status flags not temperatures...

      do jk = 1, 7
        jkk = jk + 10               ! jkk=11,18 in (scat_telemetry, scat_telemetry_eu) vectors
        i_dn = scat_telemetry(jkk)  ! inside LVPS box temp -- SFE TX load temp
        r0   = scat_prt_constants(jk,1)
        a    = scat_prt_constants(jk,2)
        b    = scat_prt_constants(jk,3)
        call convert_dn_to_temperature_8_bit_norm(i_dn, resistance, temperature)
        call convert_resistance_to_temperature_ext(resistance, r0, a, b, temperature)
        scat_telemetry_eu( jkk ) = temperature
      enddo      
      
      do jk = 1, 4
        jkk = jk + 17
        jkj = jk + 7                      ! jkk = 18,21 in (scat_telemetry, scat_telemetry_eu)
        i_dn        = scat_telemetry(jkk) ! SBE step attenuator temp -- SFE beam switch temp
        r0          = scat_prt_constants(jkj,1)
        a           = scat_prt_constants(jkj,2)
        b           = scat_prt_constants(jkj,3)
        gain        = icds_engineering_telemetry_eu(21)
        offset      = icds_engineering_telemetry_eu(22)
        resistance2 = i_dn * gain + offset
        call convert_resistance_to_temperature_ext(resistance2, r0, a, b, temperature)
        scat_telemetry_eu( jkk ) = temperature
      enddo      
      
      scat_telemetry_eu( 22 ) = ie_volt( scat_telemetry( 22 ) )        ! SFE Tx power monitor
      scat_telemetry_eu( 23 ) = ie_volt( scat_telemetry( 23 ) ) * (-6.0) ! SFE 15N voltage monitor
      
      do jjk = 24,31 !SBE PLM 1 tuning voltage (960) -- LVPS Converter Current
        scat_telemetry_eu( jjk ) = ie_volt( scat_telemetry( jjk ) ) 
      enddo

      scat_telemetry_eu( 32 ) = ie_volt( scat_telemetry( 32 ) ) * 10. !SSPA output stage voltage
      scat_telemetry_eu( 33 ) = ie_volt( scat_telemetry( 33 ) ) * 10. !SSPA intermediate stage voltage
      scat_telemetry_eu( 34 ) = ie_volt( scat_telemetry( 34 ) ) * 5.  !SSPA input stage voltage
      scat_telemetry_eu( 35 ) = ie_volt( scat_telemetry( 35 ) ) * 2.  !SFE +5V monitor
      scat_telemetry_eu( 36 ) = ie_volt( scat_telemetry( 36 ) ) * 2.  !SCG +5V monitor
      scat_telemetry_eu( 37 ) = ie_volt( scat_telemetry( 37 ) ) * 2.  !SBE +5V monitor
      scat_telemetry_eu( 38 ) = ie_volt( scat_telemetry( 38 ) ) * 4.8 !SBE +12V monitor (non switched)
      
      end
      
c*****************************************************************      
      subroutine L1A_convert_icds_tlm_dn_to_eu( 
     $   icds_tlm, 
     $   icds_engineering_telemetry_eu )
c*****************************************************************      
c**   
c**   FILE NAME: L1A_load_telemetry.f
c**   
c**   DATE WRITTEN: 8/11/2009
c**   
c**   PROGRAMMER:
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: Based on subroutine "load_icds_telem" in read_sci_v75.f
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      integer*1    icds_tlm( 24 )
      
c     OUTPUT VARIABLES
      real*4    icds_engineering_telemetry_eu( 24 )

c     LOCAL VARIABLES
      integer*4 itemp
      integer j

      integer*4 icds_engineering_telemetry( 24 )
      
      real*8 r0, a, b      
      real*8 cnst_sci_adc_r0, cnst_icds_chass_r0, cnst_icds_atc1_r0, cnst_icds_atc2_r0
      real*8 cnst_icds_a, cnst_icds_b
      real*4 constant_voltage_gain, constant_voltage_offset
      real*4 v1, voltage
      integer*4 i5v, i15v, i_dn
      real*4 constant_cal_resistor_1_nom, constant_cal_resistor_2_nom, constant_cal_resistor_3_nom, constant_cal_resistor_4_nom
      integer*4 i_dn_1, i_dn_2, i_dn_3, i_dn_4
      real*4 gain_norm_cmp, offset_norm_cmp, gain_ext_cmp, offset_ext_cmp
      real*4 temperature, resistance
c     DATA STATEMENTS
c     COMMON BLOCKS
c     FUNCTION STATEMENTS
c     PROCESSING STEPS

C--Copy bytes and convert to int*4 for the rest of the dn->eu code.
      do j=1,10 ! ICDS+5v to ICDS temp ATC 2
        call return_1_z( icds_tlm(j), itemp )
        icds_engineering_telemetry(j) = itemp
      enddo
      
      do j=1,4 ! ICDS telem board cal resistor 1 - ICDS telem board cal resistor 4
        call return_2_z( icds_tlm(11+2*(j-1) ), itemp )
        icds_engineering_telemetry(10+j) = itemp
      enddo

C--KLUDGE to fix ICDS cal resistor errors in simulated L0 DATA
      if( icds_engineering_telemetry(13).eq.32768 )then !KLUDGE LINE
        icds_engineering_telemetry(13) = 45277!KLUDGE LINE
        write(6,*) 'kludge applied!'!KLUDGE LINE
      endif!KLUDGE LINE
C--END KLUDGE --- (REMOVE ME when joel sends us updated L1A data 9/1/2009 AGF)

      call return_1_z( icds_tlm(19), itemp ) ! rotating register ID
      icds_engineering_telemetry(15) = itemp
      
      call return_2_z( icds_tlm(20), itemp ) ! rotating register value
      icds_engineering_telemetry(16) = itemp

      call return_1_z( icds_tlm(22), itemp ) ! selected register ID
      icds_engineering_telemetry(17) = itemp

      call return_2_z( icds_tlm(23), itemp ) ! selected register value
      icds_engineering_telemetry(18) = itemp
C--Done copying and converting to int*4

      constant_voltage_gain = 8.28e-5
      constant_voltage_offset = -0.24785

c     IE_ICDS_P5V
      i5v = icds_engineering_telemetry(1)
      v1 = i5v*256.*constant_voltage_gain + constant_voltage_offset
      voltage = 2.0*v1
      icds_engineering_telemetry_eu(1) = voltage

c     IE_ICDS_P15V
      i15v = icds_engineering_telemetry(2)
      v1 = i15v*256.*constant_voltage_gain + constant_voltage_offset
      voltage = 6.0*v1
      icds_engineering_telemetry_eu(2) = voltage
      
c     IE_ICDS_N15V
      i15v = icds_engineering_telemetry(3)
      v1 = i15v*256*constant_voltage_gain + constant_voltage_offset
      voltage = -5.0*v1
      icds_engineering_telemetry_eu(3) = voltage
      
c     IE_SCI_ADC_5VA
      i5v = icds_engineering_telemetry(4)
      v1 = i5v*256*constant_voltage_gain + constant_voltage_offset
      voltage = 2.0*v1
      icds_engineering_telemetry_eu(4) = voltage
      
c     IE_SCI_ADC_VREF
      i5v = icds_engineering_telemetry(5)
      v1 = float(i5v*256)*constant_voltage_gain + constant_voltage_offset
      voltage = 2.0*v1
      icds_engineering_telemetry_eu(5) = voltage      
      
C--Constants used below-----
      cnst_sci_adc_r0    = 501.36743d0
      cnst_icds_chass_r0 = 497.4265d0
      cnst_icds_atc1_r0  = 497.15944d0
      cnst_icds_atc2_r0  = 500.25343d0
 
      cnst_icds_a        = 0.003937794d0
      cnst_icds_b        = -0.00000052171d0

      a                  = cnst_icds_a
      b                  = cnst_icds_b      
C---End constants-----------

      i_dn = icds_engineering_telemetry(6)
      call convert_dn_to_temperature_8_bit_ext(i_dn, resistance, temperature)  ! uncalibrated temperature, use the resistance
      r0 = cnst_sci_adc_r0
      call convert_resistance_to_temperature_ext(resistance, r0, a, b, temperature)
      icds_engineering_telemetry_eu(6) = temperature
      
      i_dn = icds_engineering_telemetry(7)
      call convert_dn_to_temperature_8_bit_ext(i_dn, resistance, temperature)
      r0 = cnst_icds_chass_r0
      call convert_resistance_to_temperature_ext(resistance, r0, a, b, temperature)
      icds_engineering_telemetry_eu(7) = temperature      
      
      i_dn = icds_engineering_telemetry(8)
      call convert_dn_to_temperature_8_bit_rad6k(i_dn, resistance, temperature)
      icds_engineering_telemetry_eu(8) = temperature
      
      i_dn = icds_engineering_telemetry(9)
      call convert_dn_to_temperature_8_bit_norm(i_dn, resistance, temperature)
      r0 = cnst_icds_atc1_r0
      call convert_resistance_to_temperature_ext(resistance, r0, a, b, temperature)
      icds_engineering_telemetry_eu(9) = temperature
      
      i_dn = icds_engineering_telemetry(10)
      call convert_dn_to_temperature_8_bit_norm(i_dn, resistance, temperature)
      r0 = cnst_icds_atc2_r0
      call convert_resistance_to_temperature_ext(resistance, r0, a, b, temperature)
      icds_engineering_telemetry_eu(10) = temperature      
      
      
      constant_cal_resistor_1_nom = 361.
      constant_cal_resistor_2_nom = 459.
      constant_cal_resistor_3_nom = 642.
      constant_cal_resistor_4_nom = 549.


      i_dn_1 = icds_engineering_telemetry(11)
      i_dn_2 = icds_engineering_telemetry(12)
      i_dn_3 = icds_engineering_telemetry(13)
      i_dn_4 = icds_engineering_telemetry(14)      
      
      gain_norm_cmp = (constant_cal_resistor_2_nom-constant_cal_resistor_3_nom)/(i_dn_2-i_dn_3)
      offset_norm_cmp = constant_cal_resistor_2_nom-i_dn_2*gain_norm_cmp
c          write(*,*) " riow", gain_norm_cmp, offset_norm_cmp      

      gain_ext_cmp = (constant_cal_resistor_1_nom-constant_cal_resistor_4_nom)/(i_dn_1-i_dn_4)
      offset_ext_cmp = constant_cal_resistor_1_nom-i_dn_1*gain_ext_cmp      
      
c     cal resistor 1
      i_dn = icds_engineering_telemetry(11)
      call convert_dn_to_temperature_16_bit_ext_disp(i_dn, resistance, temperature)
          
      icds_engineering_telemetry_eu(11) = resistance

c     cal resistor 2
      i_dn = icds_engineering_telemetry(12)
      call convert_dn_to_temperature_16_bit_norm_disp(i_dn, resistance, temperature)
      icds_engineering_telemetry_eu(12) = resistance

c     cal resistor 3
      i_dn = icds_engineering_telemetry(13)
      call convert_dn_to_temperature_16_bit_norm_disp(i_dn, resistance, temperature)
      icds_engineering_telemetry_eu(13) = resistance

c     cal resistor 4
      i_dn = icds_engineering_telemetry(14)
      call convert_dn_to_temperature_16_bit_ext_disp(i_dn, resistance, temperature)
      icds_engineering_telemetry_eu(14) = resistance


      icds_engineering_telemetry_eu(21) = gain_norm_cmp
      icds_engineering_telemetry_eu(22) = offset_norm_cmp
      icds_engineering_telemetry_eu(23) = gain_ext_cmp
      icds_engineering_telemetry_eu(24) = offset_ext_cmp
      
      
      end      
