C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L1A_splines.f,v 1.5 2009/10/05 14:30:08 fore Exp $
c*****************************************************************      
      function get_L1A_splines( time, filled_spline )
c*****************************************************************      
c**   
c**   FILE NAME: L1A_splines.f
c**   
c**   DATE WRITTEN:
c**   
c**   PROGRAMMER:
c**   
c**   FUNCTIONAL DESCRIPTION:   Returns 0 if good, -1 if cant
c**     find XYZ spline, -2 if cant find YPR spline, and -3
c**     if it can't find either.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1A.inc'
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 time
c     OUTPUT VARIABLES
      integer get_L1A_splines
      real*8 filled_spline(10)
c     LOCAL VARIABLES
      integer m
      integer m_XYZ_last /0/
      integer m_YPR_last /0/

      integer np
      real*8 time0, time1
      real*8 xdelta
      real*8 aa, bb, cc, dd
      logical XYZ_spline_found
      logical YPR_spline_found      
c     DATA STATEMENTS
c     COMMON BLOCKS
      real*8 XYZ_spline_c(7,6,0:MAX_NUM_XYZ_EMPH)
      real*8 YPR_spline_c(4,6,0:MAX_NUM_ATT_EMPH)
      integer n_XYZ_splines, n_YPR_splines
      common /L1A_ephemeris_spline/ XYZ_spline_c, YPR_spline_c, 
     $ n_XYZ_splines, n_YPR_splines

      integer i_dbg_level
      common /debug/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      save m_XYZ_last, m_YPR_last
      
      XYZ_spline_found = .false.
      YPR_spline_found = .false.      
      
      get_L1A_splines = 0

      do m=m_XYZ_last,n_XYZ_splines
        time0 = XYZ_spline_c(2,1,m)
        time1 = XYZ_spline_c(2,6,m)
        if ( time0 .le. time .and. time .lt. time1 ) then
          if( i_dbg_level .ge. 1)then
            
          endif
          m_XYZ_last = m
          
          XYZ_spline_found = .true.
          xdelta = ( time - time0 )
          filled_spline(1) = time

          if( i_dbg_level .ge. 1)then
            write(6,'((a),I7)') ' in get_L1A_splines, m_XYZ: ',m
          endif
          
          do np = 2, 7
            aa = XYZ_spline_c(np,2,m)
            bb = XYZ_spline_c(np,3,m)
            cc = XYZ_spline_c(np,4,m)
            dd = XYZ_spline_c(np,5,m)
            filled_spline(np) = aa + bb*xdelta +  cc*xdelta**2 + dd*xdelta**3
          enddo
        endif
      enddo
      
      if( .not.XYZ_spline_found )then
        write(6,*) 'get_L1A_splines: ERROR: XYZ Spline not found in get_L1A_splines!!!'
        get_L1A_splines = get_L1A_splines - 1
      endif
      
      do m=m_YPR_last,n_YPR_splines
        time0 = YPR_spline_c(2,1,m)
        time1 = YPR_spline_c(2,6,m)
        if ( time0 .le. time .and. time .lt. time1 ) then
          if( i_dbg_level .ge. 1)then
            write(6,'((a),I7)') ' in get_L1A_splines, m_YPR: ',m
          endif
          m_YPR_last = m
          
          YPR_spline_found = .true.
          xdelta = ( time - time0 )
          do np = 2, 4
            aa = YPR_spline_c(np,2,m)
            bb = YPR_spline_c(np,3,m)
            cc = YPR_spline_c(np,4,m)
            dd = YPR_spline_c(np,5,m)
            filled_spline(np+6) = aa + bb*xdelta +  cc*xdelta**2 + dd*xdelta**3
          enddo
        endif
      enddo
      
      if( .not.YPR_spline_found )then
        write(6,*) 'get_L1A_splines: ERROR: Spline not found in get_L1A_splines!!!'
        get_L1A_splines = get_L1A_splines - 2
      endif   
      end

c*****************************************************************      
      subroutine gen_L1A_splines( L1A_HDF_file, time_start, time_end )
c*****************************************************************      
c**   
c**   FILE NAME: L1A_splines.f
c**   
c**   DATE WRITTEN:
c**   
c**   PROGRAMMER:
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1A.inc'      
c     PARAMETER STATEMENTS      
c     INPUT VARIABLES
      character*320 L1A_HDF_file
c     OUTPUT VARIABLES
      real*8 time_start
      real*8 time_end
c     LOCAL VARIABLES
      
c     Navigation data
      real*8    orb_time(      0:MAX_NUM_XYZ_EMPH )
      real*8    orb_pos(    3, 0:MAX_NUM_XYZ_EMPH )
      real*8    orb_vel(    3, 0:MAX_NUM_XYZ_EMPH )
      real*8    att_time(      0:MAX_NUM_ATT_EMPH )
      real*8    att_ang(    3, 0:MAX_NUM_ATT_EMPH )
      real*8    quaternion( 4, 0:MAX_NUM_ATT_EMPH )
      integer*4 att_flags(     0:MAX_NUM_ATT_EMPH )       

c     L1A Attributes
      integer NUM_XYZ_EMPH
      integer NUM_ATT_EMPH
      integer NUM_BLOCKS
      
      integer ierr
      integer i_len
      
      integer m
      integer i,j,jj
      integer nparameter
      integer isize, isize1
      
      real*8 time
      
      real*8 store(7,0:MAX_NUM_ATT_EMPH)
      real*8 x(0:MAX_NUM_ATT_EMPH), a(0:MAX_NUM_ATT_EMPH), h(0:MAX_NUM_ATT_EMPH)
      real*8 alpha(0:MAX_NUM_ATT_EMPH)
      real*8 mu(0:MAX_NUM_ATT_EMPH), l(0:MAX_NUM_ATT_EMPH), z(0:MAX_NUM_ATT_EMPH)
      real*8 b(0:MAX_NUM_ATT_EMPH), c(0:MAX_NUM_ATT_EMPH), d(0:MAX_NUM_ATT_EMPH)
      real*8 fp0, fpn


      
c     DATA STATEMENTS
c     COMMON BLOCKS
      real*8 XYZ_spline_c(7,6,0:MAX_NUM_XYZ_EMPH)
      real*8 YPR_spline_c(4,6,0:MAX_NUM_ATT_EMPH)
      real*8 time_ref_XYZ, time_ref_YPR
      integer n_XYZ_splines, n_YPR_splines
      common /L1A_ephemeris_spline/ XYZ_spline_c, YPR_spline_c, 
     $ n_XYZ_splines, n_YPR_splines

      integer lnblnk

      integer i_dbg_level
      common /debug_level/ i_dbg_level 
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      
C--Null terminate the hdf file name for C's benefit
      ierr = L1A_HDF5_get_sizes( NUM_BLOCKS, NUM_XYZ_EMPH, NUM_ATT_EMPH, 
     $ L1A_HDF_file( 1:lnblnk(L1A_HDF_file) ) // char(0) )
      if(ierr.lt.0) write(6,*) 'gen_L1A_splines: ERROR: L1A_HDF5_get_sizes returned: ', ierr
      
      if(   NUM_BLOCKS   .gt. MAX_NUM_BLOCKS   .or.
     $      NUM_XYZ_EMPH .gt. MAX_NUM_XYZ_EMPH .or.
     $      NUM_ATT_EMPH .gt. MAX_NUM_ATT_EMPH )then
        write(6,*) 'gen_L1A_splines: ERROR: Need to increase sizes in gen_L1A_splines...'
        stop
      endif
      
      ierr = L1A_HDF5_read_nav_data( orb_time, orb_pos, orb_vel, att_time,
     $ att_ang, quaternion, att_flags, 
     $ L1A_HDF_file( 1:lnblnk(L1A_HDF_file) ) // char(0) )
      
      if(ierr.lt.0) write(6,*) 'gen_L1A_splines: ERROR: L1A_HDF5_read_nav_data returned: ', ierr
      
      do m=0,NUM_XYZ_EMPH-1
        time = orb_time(m)/(24.0d0*60.0d0*60.0d0)
        
        if( m.eq.0 ) time_ref_XYZ = time
        
C--If time < 1st time we must have passed over a day boundary... 
        if( time .lt. time_ref_XYZ .and. m.ne.0 ) time = time + 1.0d0
        
        store(1,m) = time
        store(2,m) = orb_pos(1,m)
        store(3,m) = orb_pos(2,m)
        store(4,m) = orb_pos(3,m)
        store(5,m) = orb_vel(1,m)
        store(6,m) = orb_vel(2,m)
        store(7,m) = orb_vel(3,m)
      enddo
      
      write(6,*) "for XYZ splines ", m, " samples read "
      
      isize  = m-1
      isize1 = isize-1
       
      do jj = 0,isize
            x(jj) = store(1,jj)
      enddo
              
      do nparameter = 2,7          
        if( i_dbg_level .ge. 2) write(6,*) " nparameter", nparameter
        do jj = 0,isize
          a(jj) = store(nparameter,jj)
        enddo
          

c     step 1
        do i = 0, isize1
          h(i) = x(i+1) - x(i)
        enddo
            
        fp0 = 0.0d0
        fpn = 0.0d0
        
C--Use velocity as the boundary conditions on df/dt for the position spline generation.      
C--The time in the spline generation is in units of days, thus we must convert
C--the initial velocity (which is in meters / sec) to meters / day (thus the factor of 
C-- 86400.)
        if( nparameter.gt.1 .and. nparameter.lt.5 )then
          fp0 = store(nparameter+3,0)     * 86400.0d0 ! order t,x,y,z,vx,vy,vz,y,p,r
          fpn = store(nparameter+3,isize) * 86400.0d0
        endif
C--finite difference approximation to time derivative of velocity at 
C--start and end time.
        if( nparameter.gt.4 .and. nparameter.lt.8 )then
          fp0 = ( store(nparameter,1) - store(nparameter,0) ) / h(0)
          fpn = ( store(nparameter,isize) - store(nparameter,isize-1) ) / h(isize1)
        endif
       
c     step 2
        alpha(0)     = 3.0d0*(a(1) - a(0))/h(0) - 3.0d0 * fp0
        alpha(isize) = 3.0d0 * fpn - 3.0d0*(a(isize) - a(isize1))/h(isize1)
          
c     step 3
        do i = 1, isize1
          alpha(i) = 3.d0*(a(i+1)*h(i-1) - a(i)*(x(i+1) - x(i-1))  + 
     $     a(i-1)*h(i) ) / ( h(i-1) * h(i) )
        enddo
          
c     step 4
        l(0)  = 2.d0 * h(0)
        mu(0) = 0.5d0
        z(0)  = alpha(0) / l(0)
          
c     step 5
        do i = 1, isize1
          l(i)  = 2.d0*( x(i+1) - x(i-1) ) - ( h(i) * mu(i-1) )
          mu(i) = h(i) / l(i)
          z(i)  = ( alpha(i) - h(i) * z(i-1)) / l(i)
        enddo
          
c     step 6
        l(isize) = h(isize1) * ( 2.d0 - mu(isize1) )
        z(isize) = ( alpha(isize) - h(isize1) * z(isize1) )/ l(isize)
        c(isize) = z(isize)
          
c     step 7
        do j = isize1, 0, -1
          c(j) = z(j) - mu(j) * c(j+1)
          b(j) = ( a(j+1) - a(j) )/h(j) - h(j)*(c(j+1) + 2.d0*c(j))/3.d0
          d(j) = ( c(j+1) - c(j) )/(3.d0*h(j))
        enddo
          
          
c     step 8
        do j = 0, isize1
          XYZ_spline_c(nparameter,1,j) = x(j)
          XYZ_spline_c(nparameter,6,j) = x(j+1)          
          XYZ_spline_c(nparameter,2,j) = a(j)
          XYZ_spline_c(nparameter,3,j) = b(j)
          XYZ_spline_c(nparameter,4,j) = c(j)
          XYZ_spline_c(nparameter,5,j) = d(j)
c     write(33,334) x(j), a(j), b(j), c(j), d(j)
c334       format(1x, 5e25.15)
        enddo       
      enddo                    ! with nparameter
      
      n_XYZ_splines = isize1
      
      time_start = XYZ_spline_c(2,1,0)
      time_end   = XYZ_spline_c(2,1,isize1)
      
      
      do m=0,NUM_ATT_EMPH-1
        time = att_time(m)/(24.0d0*60.0d0*60.0d0)
        
        if( m.eq.0 ) time_ref_YPR = time
        
C--If time0 < 0 we must have passed over a day boundary...        
        if( time .lt. time_ref_YPR .and. m.ne.0 ) time = time + 1.0d0
        
        store(1,m) = time
        store(2,m) = att_ang(3,m)
        store(3,m) = att_ang(2,m)
        store(4,m) = att_ang(1,m)
      enddo
      
      write(6,*) "for YPR splines ", m, " samples read "

      isize  = m-1
      isize1 = isize-1
       
      do jj = 0,isize
            x(jj) = store(1,jj)
      enddo
              
      do nparameter = 2,4          
        if( i_dbg_level .ge. 2) write(6,*) " nparameter", nparameter
        do jj = 0,isize
          a(jj) = store(nparameter,jj)
        enddo
          

c     step 1
        do i = 0, isize1
          h(i) = x(i+1) - x(i)
        enddo
            
        fp0 = 0.0d0
        fpn = 0.0d0
       
c     step 2
        alpha(0)     = 3.0d0*(a(1) - a(0))/h(0) - 3.0d0 * fp0
        alpha(isize) = 3.0d0 * fpn - 3.0d0*(a(isize) - a(isize1))/h(isize1)
          
c     step 3
        do i = 1, isize1
          alpha(i) = 3.d0*(a(i+1)*h(i-1) - a(i)*(x(i+1) - x(i-1))  + 
     $     a(i-1)*h(i) ) / ( h(i-1) * h(i) )
        enddo
          
c     step 4
        l(0)  = 2.d0 * h(0)
        mu(0) = 0.5d0
        z(0)  = alpha(0) / l(0)
          
c     step 5
        do i = 1, isize1
          l(i)  = 2.d0*( x(i+1) - x(i-1) ) - ( h(i) * mu(i-1) )
          mu(i) = h(i) / l(i)
          z(i)  = ( alpha(i) - h(i) * z(i-1)) / l(i)
        enddo
          
c     step 6
        l(isize) = h(isize1) * ( 2.d0 - mu(isize1) )
        z(isize) = ( alpha(isize) - h(isize1) * z(isize1) )/ l(isize)
        c(isize) = z(isize)
          
c     step 7
        do j = isize1, 0, -1
          c(j) = z(j) - mu(j) * c(j+1)
          b(j) = ( a(j+1) - a(j) )/h(j) - h(j)*(c(j+1) + 2.d0*c(j))/3.d0
          d(j) = ( c(j+1) - c(j) )/(3.d0*h(j))
        enddo
          
          
c     step 8
        do j = 0, isize1
          YPR_spline_c(nparameter,1,j) = x(j)
          YPR_spline_c(nparameter,6,j) = x(j+1)          
          YPR_spline_c(nparameter,2,j) = a(j)
          YPR_spline_c(nparameter,3,j) = b(j)
          YPR_spline_c(nparameter,4,j) = c(j)
          YPR_spline_c(nparameter,5,j) = d(j)
c     write(33,334) x(j), a(j), b(j), c(j), d(j)
c334       format(1x, 5e25.15)
        enddo       
      enddo                    ! with nparameter

      
      n_YPR_splines = isize1

      end
