C--$Header: /u/caretr2/scatsim/CVSROOT/L1A_to_L2/L1B_check_CND_gain.f,v 1.6 2012/04/27 16:31:03 fore Exp $
c*****************************************************************      
      subroutine L1B_check_CND_gain( RX_power, L1B_quality_flag )
c*****************************************************************      
c**   
c**   FILE NAME: L1B_check_CND_gain.f
c**   
c**   DATE WRITTEN: 10/5/2009
c**   
c**   PROGRAMMER: APF and AGF
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1B.inc' ! contains the flagging thresholds, read in 
                        ! already by main software (L1A_to_L2 calls 
                        ! subroutine L1B_read_limits) from 
                        ! text file "L1B_limits_YYYY-MM-DD.txt".

      include 'L1B_instrument_module_params.inc'
      
c     PARAMETER STATEMENTS
      integer stdout, stderr, stdin
      parameter( stdout = 6, stderr = 0, stdin = 5 )
c     INPUT VARIABLES
      real*8    RX_power(6,3,8)          ! [dB] indicies are: i_chan, i_beam, i_seq 

c     OUTPUT VARIABLES
      integer*2 L1B_quality_flag(4,3,8)  ! May be modified by this subroutine if 
                                         ! CND gain out of range triggers a hdware status == 1 flag
c     LOCAL VARIABLES
      real*8 r_kBoltz
      real*8 r_kB_BW
      
      real*8 r_VCND(3,3)
      real*8 r_VnoCND(3,3)
      real*8 r_CND(3)
      real*8 r_gain_comp(3)
      
      logical cnd_gain_out_of_range(3)
      
      integer i_beam
      integer i_seq
      integer i_chan
      integer ii
      logical has_rfi_vrx
      
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      
      r_kBoltz = 1.38d-23
      r_kB_BW  = r_kBoltz * r_rx_BW ! r_rx_BW from params table
      
      do i_beam = 1, 3

       cnd_gain_out_of_range(i_beam) = .false.
       
       do ii = 1, 3
         r_VCND(i_beam,ii)   = 0.0d0
         r_VnoCND(i_beam,ii) = 0.0d0
       enddo
      enddo
      
      do i_seq = 1, 8
        do i_beam = 1, 3
          
          ! Don't compute CND gain using RFI nV measurments
          if( iand(L1B_quality_flag(1,i_beam,i_seq),L1B_QF_RFI).gt.0) cycle
          
          if( ( mod(i_seq,2).eq.1 .and. i_beam.eq.2 ) .or. ! CND fires on beam 2 nV for odd n_seq
     $        ( mod(i_seq,2).eq.0 .and. 
     $        (i_beam.eq.1 .or. i_beam.eq.3) ) ) then      ! CND fires on beams 1,3 nV for even n_seq
            
            if( RX_power(2,i_beam,i_seq) .le. CND_PWR_limits(2,i_beam) ) then
              r_VCND(i_beam,1) = r_VCND(i_beam,1) + 1.0d0
              r_VCND(i_beam,2) = r_VCND(i_beam,2) + 
     $                           10.0d0**(0.1d0 * RX_power(2,i_beam,i_seq) ) ! Add nV channel to r_VCND
            endif
            
          else ! No CND firing... 
               ! NOISE_PWR_limits(2,1,i_beam) is the upper nV limit for beam i_beam
            if( RX_power(2,i_beam,i_seq) .le. NOISE_PWR_limits(2,1,i_beam) ) then
              r_VnoCND(i_beam,1) = r_VnoCND(i_beam,1) + 1.0d0
              r_VnoCND(i_beam,2) = r_VnoCND(i_beam,2) + 
     $                             10.0d0**(0.1d0 * RX_power(2,i_beam,i_seq) ) ! Add nV channel to r_VCND
            endif

          endif
        enddo
      enddo
      
      do i_beam = 1, 3
        
        ! Don't check CND gain if none of nV measurments were free of RFI
        if( r_VCND(i_beam,1).lt.1 .or. r_VnoCND(i_beam,1).lt.1 ) cycle
        
        r_VCND(i_beam,3)    = r_VCND(i_beam,2)   / r_VCND(i_beam,1)   ! average over block
        r_VnoCND(i_beam,3)  = r_VnoCND(i_beam,2) / r_VnoCND(i_beam,1) ! average over block
        
        r_CND(i_beam)       = r_VCND(i_beam,3) - r_VnoCND(i_beam,3)
        
        r_gain_comp(i_beam) = r_CND(i_beam) / ( r_kB_BW * r_T_cnd(i_beam) * 1000.0d0 )
        r_gain_comp(i_beam) = 10.0d0 * log10( r_gain_comp(i_beam) )
        
        
C--Check gain against limit tables
        if( .not. (r_gain_comp(i_beam).ge.Gain_limits(1,i_beam) .and.
     $             r_gain_comp(i_beam).le.Gain_limits(2,i_beam)) ) then
          cnd_gain_out_of_range(i_beam) = .true.
          
          if( i_dbg_level.ge.0 ) then
            write(6,'((a50),I4,3f12.6)') 'L1B_check_CND_gain: WARNING: CND gain out of range: ', 
     $       i_beam, r_gain_comp(i_beam), Gain_limits(1,i_beam), Gain_limits(2,i_beam)
          endif
          
        endif
      enddo
      
      do i_seq = 1, 8
        do i_beam = 1, 3
          do i_chan = 1, 4            
            if( cnd_gain_out_of_range(i_beam) ) then
            
            ! Set HDW problem other flag
              call L1B_set_flag( L1B_quality_flag(i_chan,i_beam,i_seq), 
     $                           L1B_QF_HARDWARE_PROBLEM_OTHER, L1B_QF_HARDWARE_PROBLEM_OTHER )
            
            endif
          enddo
        enddo
      enddo
      
      if( i_dbg_level.ge.1 )then
        write(stdout,*) ' '
        write(stdout,'(a50)') 'L1B_check_CND_gain dbg output: '
        write(stdout,'((a50),3f12.6)') 'mean nV CND Power [dB]: ', (10.0*log10(r_VCND(i_beam,3)),i_beam=1,3)
        write(stdout,'((a50),3f12.6)') 'mean nV no CND Power [dB]: ', (10.0*log10(r_VnoCND(i_beam,3)),i_beam=1,3)
        write(stdout,'((a50),3f12.6)') 'number of obs in CND mean: ', (r_VCND(i_beam,1),i_beam=1,3)
        write(stdout,'((a50),3f12.6)') 'number of obs in no CND mean: ', (r_VnoCND(i_beam,1),i_beam=1,3)
        write(stdout,'((a50),3f12.6)') 'computed CND gain [dB]: ', (r_gain_comp(i_beam),i_beam=1,3)
        write(stdout,'((a50),3l2)') 'CND gain out of range [T/F]: ', (cnd_gain_out_of_range(i_beam),i_beam=1,3)
      endif
      end