C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L1B_check_range_ambig.f,v 1.4 2010/11/22 15:41:36 fore Exp $
c*****************************************************************      
      subroutine L1B_check_range_ambig( sc_position,
     $                                  sc_velocity,
     $                                  yaw_deg, pitch_deg, roll_deg,
     $                                  L2_off_nom_no_echo            )
c*****************************************************************      
c**   
c**   FILE NAME: L1B_check_range_ambig.f
c**   
c**   DATE WRITTEN: 10/6/2009
c**   
c**   PROGRAMMER: APF and AGF
c**   
c**   FUNCTIONAL DESCRIPTION: Checks for range ambiguities.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1B_instrument_module_params.inc'
c     PARAMETER STATEMENTS
      integer stdout, stderr, stdin
      parameter( stdout = 6, stderr = 0, stdin = 5 )
c     INPUT VARIABLES
      real*8 sc_position(3)     ! spacecraft position
      real*8 sc_velocity(3)     ! spacecraft veloctiy
      real*8 yaw_deg                    ! must be in degrees
      real*8 pitch_deg                  ! must be in degrees
      real*8 roll_deg                   ! must be in degrees
c     OUTPUT VARIABLES
      logical L2_off_nom_no_echo(3) ! logical flags to be passed to 
                                    ! L2_gen_quality_flag subroutine
c     LOCAL VARIABLES
      integer i_beam
      integer i_pol
      integer ii
      integer jj

c  i_time_stat(ibeam,i,j) represents whether beam vector is within receive window or not
c    i_time_stat = 1 => beam within Rx window, 0 => not within window
c    where i=1,2,3,4 are 3 dB footprint points, i=5,6,7,8 are 3 times 3-dB footprint points,
c    and i=9 is center of footprint,
c    and j = 1 is primary receive window, and j = 2 is first ambiguity at 10 ms RX window
      
      integer i_time_stat(3,9,2) ! indicies are (beam,??,??)
      integer i_main_window_stat
      integer i_ambig_window_stat
      
      real*8 slant_range
      
      real*8 r_wind_open
      real*8 r_wind_close
      real*8 r_A1wind_open
      real*8 r_A1wind_close
      
      real*8 r_theta_B(4)
      real*8 r_phi_B(4)
      real*8 r_3theta_B(4)
      
      real*8 r_vector_beam(3)
      real*8 r_bm_slant_range(3) ! slant range to beam center
      
      real*8 r_2way_time(4)  ! two-way time to 4 edge points             
      real*8 r_2way_time3(4) ! two-way time to 4 exaggerated edge points
      
      real*8 r_2way_cntr(3)  ! two-way time to beam center position
      
      real*8 earth_position(3)
      real*8 r_min_2way_time 
      real*8 r_L2_off_boresight_no_echo_angle  /15.0d0/
      logical good_solution
      
c     DATA STATEMENTS
      real*8 R_SOL              ! speed of light (put in constants.f file with include statement?)
      data R_SOL/299792458.0d0/ ! m/s
      real*8 PI
      data PI /3.141592653589793238d0/
c     COMMON BLOCKS
      real*8 theta_B_edge(3,3,4)  ! first index i_beam, second index i_pol, third index 4 corner pts
      real*8 phi_B_edge(3,3,4)
      common /beam_edge_points/ theta_B_edge, phi_B_edge

c     FUNCTION STATEMENTS
c     PROCESSING STEPS


      do i_beam = 1, 3

c   Compute window times
        r_wind_open    = r_rx_pos( i_beam )                     ! r_rx_pos from L1B_instrument_module_params.inc
        r_wind_close   = r_rx_pos(i_beam) + r_rx_length(i_beam) ! So is r_rx_length
        r_A1wind_open  = r_wind_open + 0.010d0 
        r_A1wind_close = r_wind_close + 0.010d0

C   Compute slant range to beam center position
        r_vector_beam(1) = 0.0d0
        r_vector_beam(2) = 0.0d0
        r_vector_beam(3) = 1.0d0
        
        call geolocate( r_vector_beam, sc_position, sc_velocity,
     $   yaw_deg, pitch_deg, roll_deg, i_beam, earth_position,
     $   slant_range, good_solution )
        
        if( good_solution ) then
          r_bm_slant_range(i_beam) = slant_range
        else
          write(6,*) 'L1B_check_range_ambig: WARNING: Imaginary solution found for beam center!'
        endif


c       Compute fill time      
        do ii = 1, 4
          r_theta_B(ii)  = 0.0d0
          r_phi_B(ii)    = 0.0d0
          r_3theta_B(ii) = 0.0d0
        end do ! do ii = 1, 4
      
        do i_pol=1,3    ! 3 polarizations: HV, VV, HH
          do ii = 1, 4  ! 4 corner points: phi = 0, +90, +180, +270
            r_theta_B(ii) = r_theta_B(ii) + theta_B_edge(i_beam,i_pol,ii) / 3.0d0 
            r_phi_B(ii)   = r_phi_B(ii)   + phi_B_edge(i_beam,i_pol,ii)   / 3.0d0
            r_3theta_B(ii) = 2.0d0 * r_theta_B(ii) ! can change constant if 3 times 3-dB beamwidth not ideal
          enddo ! ii = 1, 4
          
        enddo ! i_pol=1,3

        do ii = 1, 4      
          r_vector_beam(1) = sin( r_theta_B(ii) * PI / 180.0d0 ) * cos( r_phi_B(ii) * PI / 180.0d0 )
          r_vector_beam(2) = sin( r_theta_B(ii) * PI / 180.0d0 ) * sin( r_phi_B(ii) * PI / 180.0d0 )
          r_vector_beam(3) = cos( r_theta_B(ii) * PI / 180.0d0 )
        
          call geolocate( r_vector_beam, sc_position, sc_velocity,
     $   yaw_deg, pitch_deg, roll_deg, i_beam, earth_position,
     $   slant_range, good_solution )
     
          if (good_solution) then
            r_2way_time(ii) = 2.0d0 * slant_range / R_SOL
          else
            r_2way_time(ii) = -1.0d0
            write(6,*) 'L1B_check_range_ambig: WARNING: Imaginary solution found for -3db points!'
          end if
        
          r_vector_beam(1) = sin( r_3theta_B(ii) * PI / 180.0d0 ) * cos( r_phi_B(ii) * PI / 180.0d0 )
          r_vector_beam(2) = sin( r_3theta_B(ii) * PI / 180.0d0 ) * sin( r_phi_B(ii) * PI / 180.0d0 )
          r_vector_beam(3) = cos( r_3theta_B(ii) * PI / 180.0d0 )
        
          call geolocate( r_vector_beam, sc_position, sc_velocity,
     $   yaw_deg, pitch_deg, roll_deg, i_beam, earth_position,
     $   slant_range, good_solution )
     
          if (good_solution) then
            r_2way_time3(ii) = 2.0d0 * slant_range / R_SOL
          else
            r_2way_time3(ii) = -1.0d0
            write(6,*) 'L1B_check_range_ambig: WARNING: Imaginary solution found for 3x -3dB points!'
          end if       
        enddo ! ii = 1, 4  
        
        r_2way_cntr(i_beam) = 2.0d0 * r_bm_slant_range(i_beam) / R_SOL
        
        do ii = 1, 9
          do jj = 1, 2
            i_time_stat(i_beam,ii,jj) = 0
          enddo
        enddo   

c  i_time_stat(ibeam,i,j) represents whether beam vector is within receive window or not
c    i_time_stat = 1 => beam within Rx window, 0 => not within window
c    where i=1,2,3,4 are 3 dB footprint points, i=5,6,7,8 are 3 times 3-dB footprint points,
c    and i=9 is center of footprint,
c    and j = 1 is primary receive window, and j = 2 is first ambiguity at 10 ms RX window
        
        
        if( r_2way_cntr(i_beam).gt.r_wind_open .and. 
     $      r_2way_cntr(i_beam).lt.r_wind_close ) then
          i_time_stat(i_beam,9,1) = 1 ! 1 = OK, 0 = not OK
        else
          i_time_stat(i_beam,9,1) = 0
        endif
        
        if( r_2way_cntr(i_beam).gt.r_A1wind_open .and. 
     $      r_2way_cntr(i_beam).lt.r_A1wind_close ) then
          i_time_stat(i_beam,9,2) = 1  ! 1 = not OK, 0 = OK
        else
          i_time_stat(i_beam,9,2) = 0
        endif
     
        do ii = 1, 4
          
          if( r_2way_time(ii).gt.r_wind_open .and. r_2way_time(ii).lt.r_wind_close ) then
            i_time_stat(i_beam,ii,1) = 1  ! 1 = OK, 0 = not OK
          else
            i_time_stat(i_beam,ii,1) = 0
          endif
          
          if( r_2way_time(ii).gt.r_A1wind_open .and. r_2way_time(ii).lt.r_A1wind_close ) then
            i_time_stat(i_beam,ii,2) = 1  ! 1 = not OK, 0 = OK    
          else
            i_time_stat(i_beam,ii,2) = 0
          endif
          
          if( r_2way_time3(ii).gt.r_wind_open .and. r_2way_time3(ii).lt.r_wind_close ) then
            i_time_stat(i_beam,ii+4,1) = 1 ! 1 = OK, 0 = not OK
          else
            i_time_stat(i_beam,ii+4,1) = 0
          endif
          
          if( r_2way_time3(ii).gt.r_A1wind_open .and. r_2way_time3(ii).lt.r_A1wind_close ) then
            i_time_stat(i_beam,ii+4,2) = 1 ! 1 = not OK, 0 = OK       
          else
            i_time_stat(i_beam,ii+4,2) = 0
          endif
        enddo ! ii = 1, 4
      
        i_main_window_stat  = 0
        i_ambig_window_stat = 0
        
        do ii = 1,9
          i_main_window_stat = i_main_window_stat + i_time_stat(i_beam,ii,1)
          i_ambig_window_stat = i_ambig_window_stat + i_time_stat(i_beam,ii,2)
        enddo ! ii = 1,9
      
        if (i_main_window_stat .ne. 9) then
          write(6,*) 'L1B_check_range_ambig: WARNING: Portion of footprint outside of receive window'
          write(6,*) ' Beam number ', i_beam
          write(6,*) (i_time_stat(i_beam,ii,1),ii=1,9)
        endif
       
        if (i_ambig_window_stat .gt. 0) then
          write(6,*) 'L1B_check_range_ambig: WARNING: Portion of footprint echo in first ambiguity window'
          write(6,*) ' Beam number ', i_beam
          write(6,*) (i_time_stat(i_beam,ii,2),ii=1,9)
        endif
      enddo ! do i_beam = 1, 3
      
      
      
C--Check for majority of beam outside receive-window 4/8/2010
      do i_beam = 1, 3 
c   Compute window times
        r_wind_open    = r_rx_pos( i_beam )                     ! r_rx_pos from L1B_instrument_module_params.inc
        r_wind_close   = r_rx_pos(i_beam) + r_rx_length(i_beam) ! So is r_rx_length

C--Initialize the 4 15 deg off-boresight points.      
        r_theta_B(1) =  r_L2_off_boresight_no_echo_angle
        r_theta_B(2) =  r_L2_off_boresight_no_echo_angle
        r_theta_B(3) = -r_L2_off_boresight_no_echo_angle
        r_theta_B(4) = -r_L2_off_boresight_no_echo_angle
        
        r_phi_B(1)   =  0.0d0
        r_phi_B(2)   = 90.0d0
        r_phi_B(3)   =  0.0d0
        r_phi_B(4)   = 90.0d0
        
        do ii = 1, 4
          r_vector_beam(1) = sin( r_theta_B(ii) * PI / 180.0d0 ) * cos( r_phi_B(ii) * PI / 180.0d0 )
          r_vector_beam(2) = sin( r_theta_B(ii) * PI / 180.0d0 ) * sin( r_phi_B(ii) * PI / 180.0d0 )
          r_vector_beam(3) = cos( r_theta_B(ii) * PI / 180.0d0 )
          
          call geolocate( r_vector_beam, sc_position, sc_velocity,
     $   yaw_deg, pitch_deg, roll_deg, i_beam, earth_position,
     $   slant_range, good_solution )
     
          if (good_solution) then
            r_2way_time(ii) = 2.0d0 * slant_range / R_SOL
          else
            r_2way_time(ii) = -1.0d0
            write(6,*) 'L1B_check_range_ambig: WARNING: Imaginary solution found for 15 deg off-boresight points!'
          end if
        enddo  
        ! We want to check the inner-most point's two-way travel time and see if it
        ! is larger than the window close time.  If so, we want to set the
        ! L2_QF_ATT_OFF_NOM_NO_ECHO (see L2.inc include file).          
        ! Note that the inner-most point will be the one with the smallest two-way time (>0).
          
        r_min_2way_time = 10.0d8 ! big number
        
        do ii=1,4
          if( r_2way_time(ii) .gt. 0.0d0 .and. ! make sure to not use one of the off-earth points!
     $        r_2way_time(ii) .lt. r_min_2way_time ) r_min_2way_time = r_2way_time(ii)
        enddo
        
        if( r_min_2way_time .gt. r_wind_close ) then
          write(6,*) 'L1B_check_range_ambig: WARNING: Pointing off-nominal without echo!'
          L2_off_nom_no_echo(i_beam) = .true.
        else
          L2_off_nom_no_echo(i_beam) = .false.
        endif
        
      enddo
      
      end