C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L1B_compute_landice_fractions.f,v 1.11 2011/01/07 16:54:24 fore Exp $
c*****************************************************************      
      subroutine L1B_compute_landice_fractions( 
     $  julian_time, 
     $  sc_position,
     $  sc_velocity,
     $  yaw_deg, pitch_deg, roll_deg,
     $  antenna_pattern_relative_1H,
     $  antenna_pattern_relative_1V,
     $  antenna_pattern_relative_2H,
     $  antenna_pattern_relative_2V,
     $  antenna_pattern_relative_3H,
     $  antenna_pattern_relative_3V,     
     $  land_fraction,
     $  ice_fraction )
c*****************************************************************      
c**   
c**   FILE NAME: L1B_compute_land_fraction.f
c**   
c**   DATE WRITTEN: 8/12/2009
c**   
c**   PROGRAMMER:
c**   
c**   FUNCTIONAL DESCRIPTION:  Computes the gain and area weighted
c**     land and ice fractions in the beam.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1B_instrument_module_params.inc'
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 julian_time
      real*8 sc_position(3)
      real*8 sc_velocity(3)
      real*8 yaw_deg, pitch_deg, roll_deg
      real*8 antenna_pattern_relative_1H(360,721,6)
      real*8 antenna_pattern_relative_1V(360,721,6)      
      real*8 antenna_pattern_relative_2H(360,721,6)
      real*8 antenna_pattern_relative_2V(360,721,6)      
      real*8 antenna_pattern_relative_3H(360,721,6)
      real*8 antenna_pattern_relative_3V(360,721,6)      

c     OUTPUT VARIABLES
      real*8 land_fraction(3) ! dimensionless, i_beam = {1,2,3}
      real*8 ice_fraction(3)  ! dimensionless, i_beam = {1,2,3}
      
c     LOCAL VARIABLES
      integer i_beam
      integer i_dtheta_B
      integer i_dphi_B
      integer i_theta_B
      integer i_phi_B
      integer i_theta_patt
      integer i_phi_patt
      
      real*8 PI
      real*8 theta_B
      real*8 phi_B
      real*8 theta_B_deg
      real*8 phi_B_deg
      real*8 vector(3)
      real*8 vector_1(3), vector_2(3), vector_3(3), vector_4(3)
      real*8 earth_position_center(3)
      real*8 earth_position_1(3), earth_position_2(3), earth_position_3(3), earth_position_4(3)
      real*8 slant_range_center
      real*8 slant_range_1, slant_range_2, slant_range_3, slant_range_4
      real*8 latitude_geocentric_deg
      real*8 latitude_deg
      real*8 longitude_deg
      real*8 spherical_excess_1, spherical_excess_2, cell_area
      real*8 sum_two_way_gain(3)
      real*8 sum_two_way_gain_land_mask(3)
      real*8 sum_two_way_gain_ice_mask(3)
      real*8 two_way_gain
      real*8 delta_theta
      real*8 delta_phi      
      
      integer*1 land_mask
      real*4    ice_mask

      real*4 r4_lat
      real*4 r4_lon
      logical good_solution, good_solution_all 
      
      real*8 two_way_time_center
      real*8 in_window
c     DATA STATEMENTS
      real*8 R_SOL              ! speed of light 
      data R_SOL/299792458.0d0/ ! m/s
c     COMMON BLOCKS
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      
      PI = 4.0d0 * atan( 1.0d0 )
      
      do i_beam = 1,3
        sum_two_way_gain(i_beam)           = 0.0d0
        sum_two_way_gain_land_mask(i_beam) = 0.0d0
        sum_two_way_gain_ice_mask(i_beam)  = 0.0d0
      enddo
      
      i_dtheta_B = 1
      i_dphi_B   = 5
      
      do i_theta_B = -10, 10, i_dtheta_B
         do i_phi_B = 0, 180, i_dphi_B
          
            delta_theta = real( i_dtheta_B ) * PI / 180.0d0
            delta_phi   = real( i_dphi_B   ) * PI / 180.0d0
          
            theta_B     = real(i_theta_B)    * PI / 180.0d0
            phi_B       = real(i_phi_B)      * PI / 180.0d0

            theta_B_deg = theta_B * 180.0d0 / PI
            phi_B_deg   = phi_B   * 180.0d0 / PI
          
            vector(1)   = sin( theta_B ) * cos( phi_B )
            vector(2)   = sin( theta_B ) * sin( phi_B )
            vector(3)   = cos( theta_B ) 
          
            vector_1(1) = sin( theta_B - delta_theta/2.0 ) * cos( phi_B - delta_phi/2.0 )
            vector_1(2) = sin( theta_B - delta_theta/2.0 ) * sin( phi_B - delta_phi/2.0 )
            vector_1(3) = cos( theta_B - delta_theta/2.0 )

            vector_2(1) = sin( theta_B - delta_theta/2.0 ) * cos( phi_B + delta_phi/2.0 )
            vector_2(2) = sin( theta_B - delta_theta/2.0 ) * sin( phi_B + delta_phi/2.0 )
            vector_2(3) = cos( theta_B - delta_theta/2.0 )

            vector_3(1) = sin( theta_B + delta_theta/2.0 ) * cos( phi_B - delta_phi/2.0 )
            vector_3(2) = sin( theta_B + delta_theta/2.0 ) * sin( phi_B - delta_phi/2.0 )
            vector_3(3) = cos( theta_B + delta_theta/2.0 )

            vector_4(1) = sin( theta_B + delta_theta/2.0 ) * cos( phi_B + delta_phi/2.0 )
            vector_4(2) = sin( theta_B + delta_theta/2.0 ) * sin( phi_B + delta_phi/2.0 )
            vector_4(3) = cos( theta_B + delta_theta/2.0 )
            
            i_phi_patt   = 1 + nint( 359.0d0 * phi_B_deg / 180.0d0 )
            i_theta_patt = 1 + nint( 720.0d0 * (theta_B_deg+180.0d0) / 360.0d0 )    
          
            do i_beam = 1,3
               good_solution_all = .true.
C--   Geolocate the center of the cell 
               call geolocate( vector, sc_position, sc_velocity, yaw_deg,
     $              pitch_deg, roll_deg, i_beam, earth_position_center, 
     $              slant_range_center, good_solution )
               good_solution_all = good_solution_all.and.good_solution
     
C--   Geolocate the four corners of the current cell
               call geolocate( vector_1, sc_position, sc_velocity, yaw_deg,
     $              pitch_deg, roll_deg, i_beam, earth_position_1, 
     $              slant_range_1, good_solution )
               good_solution_all = good_solution_all.and.good_solution
          
               call geolocate( vector_2, sc_position, sc_velocity, yaw_deg,
     $              pitch_deg, roll_deg, i_beam, earth_position_2, 
     $              slant_range_2, good_solution )
               good_solution_all = good_solution_all.and.good_solution

               call geolocate( vector_3, sc_position, sc_velocity, yaw_deg,
     $              pitch_deg, roll_deg, i_beam, earth_position_3, 
     $              slant_range_3, good_solution )
               good_solution_all = good_solution_all.and.good_solution

               call geolocate( vector_4, sc_position, sc_velocity, yaw_deg,
     $              pitch_deg, roll_deg, i_beam, earth_position_4, 
     $              slant_range_4, good_solution )

               if( .not.good_solution_all )then
                  write(6,*) 'L1B_compute_landice_fractions: WARNING: imaginary solution found!'
                  call flush(6)
                  continue
               endif
            
C--   Compute the spherical excess contained in the cell.
               if( i_theta_B .eq. 0 )then ! Different geometery if theta_B == 0
                  call return_spherical_triangle_excess( earth_position_1,
     $                 earth_position_2,
     $                 earth_position_center,
     $                 spherical_excess_1 )
     
                  call return_spherical_triangle_excess( earth_position_3,
     $                 earth_position_4,
     $                 earth_position_center,
     $                 spherical_excess_2 )
               else
                  call return_spherical_triangle_excess( earth_position_1,
     $                 earth_position_2,
     $                 earth_position_4,
     $                 spherical_excess_1 )
                  
                  call return_spherical_triangle_excess( earth_position_1,
     $                 earth_position_3,
     $                 earth_position_4,
     $                 spherical_excess_2 )
               endif

C--Computation of the cell area from spherical excess.            
               cell_area = (spherical_excess_1+spherical_excess_2) *
     $              ( earth_position_center(1)**2 + 
     $              earth_position_center(2)**2 + 
     $              earth_position_center(3)**2 )

            
C--   Compute lat,lon coordinates of pointing vector
               call compute_lat_long( earth_position_center, 
     $              latitude_geocentric_deg, 
     $              latitude_deg,
     $              longitude_deg )
            
               r4_lat = latitude_deg
               r4_lon = longitude_deg
            
               if( r4_lon .gt. 180) r4_lon = r4_lon - 360
            
!call get_mask(                     r4_lat, r4_lon, 1, land_mask )
! Modified to call faster version of land mask subroutine. 1/7/2011
               call get_mask2(                    r4_lat, r4_lon, land_mask )
            
               call get_ice_mask_Op( julian_time, r4_lat, r4_lon, 1, ice_mask  )
!               write(*,*) julian_time,  i_dtheta_B,  i_phi_B, r4_lat, r4_lon, land_mask 
C--   Compute 2 way travel time; check if in range window            
               in_window = 1
               two_way_time_center = 2 * slant_range_center / R_SOL
               if( two_way_time_center.lt.r_rx_pos(i_beam) .or.
     $              two_way_time_center.gt.r_rx_pos(i_beam)+r_rx_length(i_beam) ) then
                  in_window = 0
               endif
                   
C--   Use VV two-way gain to weight the integrated land / ice fractions
               if( i_beam .eq. 1) then
                  two_way_gain = 10.0d0**( antenna_pattern_relative_1V(i_phi_patt,i_theta_patt,1)
     $                 / 5.0d0 )
               else if( i_beam .eq. 2 )then
                  two_way_gain = 10.0d0**( antenna_pattern_relative_2V(i_phi_patt,i_theta_patt,1)
     $                 / 5.0d0 )
               else
                  two_way_gain = 10.0d0**( antenna_pattern_relative_3V(i_phi_patt,i_theta_patt,1)
     $                 / 5.0d0 )
               endif
            
               two_way_gain = two_way_gain * in_window

               sum_two_way_gain(i_beam) = sum_two_way_gain(i_beam) 
     $              + two_way_gain * cell_area / slant_range_center**4
            
               sum_two_way_gain_land_mask(i_beam) = sum_two_way_gain_land_mask(i_beam)
     $              + two_way_gain * real(land_mask) * cell_area / slant_range_center**4

               sum_two_way_gain_ice_mask(i_beam)  = sum_two_way_gain_ice_mask(i_beam)
     $              + two_way_gain * ice_mask * cell_area / slant_range_center**4
             
            enddo
         enddo
      enddo
      
      do i_beam = 1,3
        land_fraction(i_beam) = sum_two_way_gain_land_mask(i_beam) / sum_two_way_gain(i_beam)
        ice_fraction(i_beam)  = sum_two_way_gain_ice_mask(i_beam)  / sum_two_way_gain(i_beam)
      enddo
      end
