C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L1B_compute_params.f,v 1.3 2009/10/05 20:25:52 fore Exp $
c*****************************************************************      
      subroutine L1B_compute_params( calibration_level )
c*****************************************************************      
c**   
c**   FILE NAME: L1B_compute_params.f
c**   
c**   DATE WRITTEN: 10/1/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: Code to compute params based on i_cal_option moved
c**     here from compute_xcal.  
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1B_compute_params.inc'
      include 'L1B_instrument_module_temps.inc'
      include 'L1B_instrument_module_params.inc'
c     PARAMETER STATEMENTS
      integer stdout, stderr, stdin
      parameter( stdout = 6, stderr = 0, stdin = 5 )
c     INPUT VARIABLES
          integer calibration_level  ! calibration option: 
                                 ! 1 = no temp correction, 
                                     ! 2 = primary temp correction, 
                                     ! 3 = primary and secondary temp correction

c     OUTPUT VARIABLES
c     LOCAL VARIABLES
      integer i_pol
      integer i_beam
      integer ii, jj
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
            
C--First set all parameters to nominal values
      r_SSPA_PowerW = r_SSPA_PowerW_nom
      
      do ii = 1, 6
        r_SFE_tx_loss(ii) = r_SFE_tx_loss_nom(ii)
        r_SFE_rx_loss(ii) = r_SFE_rx_loss_nom(ii)
        r_SFE_LB_loss(ii) = r_SFE_LB_loss_nom(ii)
      enddo
      
      do ii = 1, 2
        r_SBE_rx_loss(ii) = r_SBE_rx_loss_nom(ii)
      enddo
      
      r_SBE_gain = r_SBE_gain_nom
      
      do ii = 1, 6
        do jj = 1, 2
          r_OMT_loss(jj,ii) = r_OMT_loss_nom(jj,ii)
        enddo
      enddo
      
      r_tx_reflect    = r_tx_reflect_nom
      r_SBE_noise_fig = r_SBE_noise_fig_nom
      r_rx_BW         = r_rx_BW_nom
      r_tx_BW         = r_tx_BW_nom
      r_tx_frequency  = r_tx_frequency_nom
      
      do ii = 1, 3
        r_rx_length(ii) = r_rx_length_nom(ii)
      enddo
      
      r_chirp_rate      = r_chirp_rate_nom
      r_tx_pulse_length = r_tx_pulse_length_nom
      r_B1HV            = r_B1HV_nom
      r_B1NV            = r_B1NV_nom
      
      do ii = 1, 3
        r_T_cnd(ii) = r_T_cnd_nom(ii)
      enddo
      
      do ii = 0, 2
        do jj = 1,3
          r_gain_bias(jj,ii,1) = r_gain_bias_nom(jj,ii,1)
          r_gain_bias(jj,ii,2) = r_gain_bias_nom(jj,ii,2)
        enddo
      enddo
      
      do ii = 1, 2
        do jj = 1, 3
          r_averaging_time(jj,ii) = r_averaging_time_nom(jj,ii)
        enddo
      enddo
      
      r_averaging_time_LB = r_averaging_time_LB_nom
      
      do ii = 1, 3
        r_rx_pos(ii) = r_rx_pos_nom(ii)
      enddo
      
C--Conditionally perturb parameters with temperature information
C--Apply primary temperature calibration if calibration_level equals 2 or 3.
      if( calibration_level .eq. 2 .or. calibration_level .eq. 3 ) then 

         do i_pol=1,6
            r_SFE_tx_loss(i_pol) = r_SFE_tx_loss_nom(i_pol) + r_SFE_tx_loss_var(i_pol)
     $         * (r_SC_SFE_BM_SW_TMP - r_nominal_temp)
     
            r_SFE_rx_loss(i_pol) = r_SFE_rx_loss_nom(i_pol) + r_SFE_rx_loss_var(i_pol)
     $         * (r_SC_SFE_LBK_SW_TMP - r_nominal_temp)
     
            r_SFE_LB_loss(i_pol) = r_SFE_LB_loss_nom(i_pol) + r_SFE_LB_loss_var(i_pol)
     $         * (r_SC_SFE_LBK_ATTEN_TMP - r_nominal_temp)
         enddo
      
         do i_pol=1,2
            do i_beam=1,3
         
               ii = i_beam+(i_pol-1)*3 ! 2nd index in r_OMT_loss
          
               r_OMT_loss(1,ii) = r_OMT_loss_nom(1,ii) + r_OMT_loss_var(1,ii) *
     $            (r_ET_DPLX_TMP(i_beam,i_pol) - r_nominal_temp)

C--Alternate sourece of temperature information (leave commented out for now).
C          r_OMT_loss(1,ii) = r_OMT_loss_nom(1,ii)+r_OMT_loss_var(1,ii)*
C     $     (r_ET_SFE_COAX_TMP(i_beam,i_pol)-r_nominal_temp)
     
               r_OMT_loss(2,ii) = r_OMT_loss_nom(2,ii) + r_OMT_loss_var(2,ii) *
     $            (r_ET_OMT_PROBE_TMP(i_beam,i_pol) - r_nominal_temp)
            enddo
         enddo
            
         r_SBE_rx_loss(1) = r_SBE_rx_loss_nom(1) + r_SBE_rx_loss_var(1)
     $      * (r_SC_SBE_STP_ATTEN_TMP - r_nominal_temp)
     
         r_SBE_rx_loss(2) = r_SBE_rx_loss_nom(2) + r_SBE_rx_loss_var(2)
     $      * (r_SC_SBE_STP_ATTEN_TMP - r_nominal_temp)

      endif
      
      
      
      if( calibration_level .eq. 3 ) then ! apply secondary temperature calibration

         r_SSPA_PowerW = r_SSPA_PowerW_nom + r_SSPA_PowerW_var
     $      * (r_SC_SSPA_RF_DCK_TMP - r_nominal_temp)

         r_SBE_gain = r_SBE_gain_nom + r_SBE_gain_var
     $      * (r_SC_SBE_STP_ATTEN_TMP - r_nominal_temp)

      endif

C--This should never be true since the main software also performs this check and stops
C--if we were not given a valid value for calibration_level
      if( calibration_level .lt. 1 .or. calibration_level .gt. 3 ) then      
        write(6,*) 'L1B_compute_params: ERROR: Invalid value for calibration_level!'
      endif
      
      
      end