C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L1B_compute_xcal.f,v 1.7 2009/10/05 14:30:08 fore Exp $
c*******************************************************************

      subroutine L1B_compute_xcal( r_LB_pow, r_peak_gain, r_Xcal)
      !(r_time,r_latlon,r_temperatures,r_peak_gain,r_LB_power,i_beam,r_Xcal) ! Module SB3 (inputs in dBm power and dB output)

c*******************************************************************
c**   
c**    FILE NAME:  L1B_compute_xcal.f (based on instrument_module.f)
c**
c**    DATE WRITTEN:   12-MARCH-2007, 27-AUG-2009
c**
c**    PROGRAMMER:  Adam P. Freedman
c**
c**    FUNCTIONAL DESCRIPTION: This routine computes the calibration correction using the 
c**   loopback measurement for the scatterometer measurement. It corresponds to Greg's
c**   SB3 module.
c**
c**    ROUTINES CALLED: 
c**
c**    INCLUDE FILES:  
c**
c**    NOTES: 
c**
c**    UPDATE LOG: 
c**
c**    Date Changed        Reason Changed                  CR # and Version #
c**    ------------       ----------------                 -----------------  
c**
c*******************************************************************

      implicit none

c     PARAMETER AND STRUCTURE DEFINITION FILES: 

c     INPUT VARIABLES:    

          real*8 r_LB_pow(6,3)   ! Loopback power, dBm
          real*8 r_peak_gain(3,2) ! Max antenna gain, db

c     OUTPUT VARIABLES:

      real*8 r_Xcal(6,3)        ! Calibration corrections for each channel and beam value
                                                        ! (HV,nV,VV,VH,nH,HH), (B1,B2,B3) dB

c     LOCAL VARIABLES:

      real*8 r_SSPA_power
      !real*8 r_eff_ant_aperture(6) ! effective antenna aperture (G*wvlngth^2/4*pi)
      real*8 r_mult_cnstnt   
      real*8 r_wavelength
      integer i_chan, i_chan2
      integer i_pol, i_poltx, i_polrx
      integer i_beam
      integer ii, ij
      integer i_tx
      integer i_rx
      logical l_noise
      real*8 r_G_const     

c     COMMON BLOCKS: 

      include 'L1B_instrument_module_temps.inc'
      include 'L1B_instrument_module_params.inc'
      
      integer i_dbg_level
      common /debug/ i_dbg_level
      
      real*8 r_LB_expected(6)
      real*8 r_tx_pow_expected(6)
      real*8 r_tx_loss_expected(6)
      real*8 r_rx_loss_expected(6)
      real*8 r_rx_chain_expected(6)
      real*8 r_Tsys_expected(6)
      real*8 r_CND_chain(6)
      common /other_instrument/ r_LB_expected, r_tx_pow_expected, r_tx_loss_expected,
     &   r_rx_loss_expected, r_rx_chain_expected, r_Tsys_expected, r_CND_chain

c     EQUIVALENCE STATEMENTS: none

c     DATA STATEMENTS:

      real*8 r_pi
      real*8 R_SOL              ! speed of light
      data r_pi/3.14159265359d0/
      data R_SOL/299792458.0d0/ ! m/s
      
c     SAVE STATEMENTS:

c     FORMAT STATEMENTS:

 1001 format (/1X,A/)    ! skip line before and after
 1002 format (/1X,A)     ! skip line before
 1003 format (1X,A/)     ! skip line after

c     FUNCTION STATEMENTS:

c     PROCESSING STEPS:

      r_wavelength = R_SOL / r_tx_frequency   ! r_tx_frequency from parameters file.
      r_G_const = r_wavelength * r_wavelength / (4.d0 * r_pi)
      r_G_const = r_G_const / (16.d0 * r_pi * r_pi)
      r_G_const = 10.0d0 * log10( r_G_const )


      r_SSPA_power = 10.d0*log10(r_SSPA_powerW) + 30.d0 ! +30 to convert dB_Watts to dB_milliWatts

      if( i_dbg_level.ge.2)then
        write(6,*) 'Compute_Xcal r_G_const: ', r_G_const
        write(6,*) 'Compute_Xcal SFE tx, rx, lb losses: ', r_SFE_tx_loss, r_SFE_rx_loss,
     &       r_SFE_LB_loss
        write(6,*) 'Compute_Xcal OMT losses: ', r_OMT_loss
        write(6,*) 'Compute_Xcal r_SBE_rx_loss: ', r_SBE_rx_loss
        write(6,*) 'Compute_Xcal Tx power, gain: ', r_SSPA_power, r_SBE_gain
      endif      
      
c       Compute Xcal
c               make sure to get indices correct: **** DOUBLE CHECK ***

      do i_chan = 1, 6 ! HV, nV, VV, VH, nH, HH
         do i_beam = 1, 3
         
            l_noise = .false.
            if (i_chan.gt.3) then
               i_rx = 1 ! H receive
            else
               i_rx = 2 ! V receive
            endif
            if ((i_chan.eq.1).or.(i_chan.eq.6)) then
               i_tx = 1 ! H transmit
            elseif ((i_chan.eq.3).or.(i_chan.eq.4)) then
               i_tx = 2 ! V transmit
            else
               i_tx = 0 ! noise only
               l_noise = .true.
            endif
            
            r_mult_cnstnt = r_G_const + r_peak_gain(i_beam,i_rx)
            if (i_tx.gt.0) r_mult_cnstnt = r_mult_cnstnt + r_peak_gain(i_beam,i_tx)
            
            
            
            !!ii = i_beam+(i_pol-1)*3 ! 2nd index in r_OMT_loss
            if (.not.l_noise) then
            
               i_poltx = i_beam + (i_tx-1)*3
               i_polrx = i_beam + (i_rx-1)*3
               
               r_Xcal(i_chan,i_beam) = r_mult_cnstnt + r_LB_pow(i_chan,i_beam)
     &            + r_SFE_tx_loss(i_poltx) + r_SFE_rx_loss(i_polrx)
     &                    + r_OMT_loss(1,i_poltx) + r_OMT_loss(2,i_poltx)
     &            + r_OMT_loss(1,i_polrx) + r_OMT_loss(2,i_polrx)
     &            - r_SFE_LB_loss(i_poltx) 
     &            + r_SBE_rx_loss(1) - r_SBE_rx_loss(2)
     &            + r_gain_bias(i_beam,i_tx,i_rx)
            else
               r_Xcal(i_chan,i_beam) = 0.0d0 ! noise only channel
            endif
            
         enddo  ! i_beam
      enddo  ! i_chan
      
      if (i_dbg_level.ge.2) then
         write(6,*) 'Compute_Xcal r_Xcal: ', r_Xcal
      endif

c    Compute other values for debugging or monitoring

c       Expected loopback power level
      do i_chan=1,6
        r_LB_expected(i_chan) = r_SSPA_power + r_SFE_LB_loss(i_chan) + r_SBE_rx_loss(2) + r_SBE_gain
      enddo

c       Expected TX power out of antenna and losses
      do i_chan=1,6
        r_tx_pow_expected(i_chan) = r_SSPA_power + r_SFE_tx_loss(i_chan) + r_OMT_loss(1,i_chan) + 
     $    r_OMT_loss(2,i_chan) + r_peak_gain(1+mod(i_chan-1,3),(i_chan-1)/3+1)
        r_tx_loss_expected(i_chan) = r_SFE_tx_loss(i_chan) + r_OMT_loss(1,i_chan) + r_OMT_loss(2,i_chan)
      enddo
      
c       Expected RX gain and losses
      
      do i_chan = 1,6
         r_rx_loss_expected(i_chan) =  r_SFE_rx_loss(i_chan) + r_OMT_loss(1,i_chan) +   
     $      r_OMT_loss(2,i_chan)
     
         r_rx_chain_expected(i_chan) = r_SBE_gain + r_SBE_rx_loss(1) + r_rx_loss_expected(i_chan)
        
         r_Tsys_expected(i_chan) = ( r_SC_SBE_STP_ATTEN_TMP + 273.15 ) * 
     &      (10.0d0**(0.1*abs(r_rx_loss_expected(i_chan) + r_SBE_noise_fig)) - 1.0d0)
     
         r_CND_chain(i_chan) = r_SFE_rx_loss(i_chan) + r_OMT_loss(1,i_chan) + r_SBE_rx_loss(1)

      enddo
      
      return

      end
