C--$Header: /u/caretr2/scatsim/CVSROOT/L1A_to_L2/L1B_read_params.f,v 1.8 2012/04/27 16:32:59 fore Exp $
c*****************************************************************      
      subroutine L1B_read_params( parameter_file )
c*****************************************************************      
c**   
c**   FILE NAME: L1B_read_params.f
c**   
c**   DATE WRITTEN: 6/12/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  Reads in instrument parameters
c**    from a plain text file.  Populates a common block with
c**    the parameters (nominal, d_param_dT).  The include file
c**    'compute_params.inc' contains the variable declarations 
c**    and the common block.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:  Modified 9/2/2009 for use in L1A_to_L2
c**     project.
c**     Updated 9/26/2011 to add RFI parameters.
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1B_compute_params.inc'
      include 'L1A_RFI_params.inc'
      include 'L1B_attitude_bias.inc'
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      character*(*) parameter_file
c     OUTPUT VARIABLES
c     LOCAL VARIABLES
      character*80 junk, name_tag
      real*8 param_nom, param_var
      integer*4 i_len
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level
c     FUNCTION STATEMENTS
      integer*4 lnblnk
c     PROCESSING STEPS
      
      !--Initialize attitude bias to zero--4/23/2012 AGF
      roll_bias  = 0.0d0
      pitch_bias = 0.0d0
      yaw_bias   = 0.0d0
      
      write(6,*) 'Reading parameters from ',parameter_file(1:lnblnk(parameter_file))
      
      open(50,file=parameter_file,status='old')
      read(50,*) junk ! 1st line is comments/info about file      
123   continue
      read(50,*,END=321) param_nom, param_var, name_tag
      
      i_len = lnblnk(name_tag)
      
      if( name_tag(1:i_len).eq.'SSPA_PowerW')then
        r_SSPA_PowerW_nom = param_nom
        r_SSPA_PowerW_var = param_var
      elseif( name_tag(1:i_len).eq.'SFE_tx_loss_1')then
        r_SFE_tx_loss_nom(1) = param_nom
        r_SFE_tx_loss_var(1) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_tx_loss_2')then
        r_SFE_tx_loss_nom(2) = param_nom
        r_SFE_tx_loss_var(2) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_tx_loss_3')then
        r_SFE_tx_loss_nom(3) = param_nom
        r_SFE_tx_loss_var(3) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_tx_loss_4')then
        r_SFE_tx_loss_nom(4) = param_nom
        r_SFE_tx_loss_var(4) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_tx_loss_5')then
        r_SFE_tx_loss_nom(5) = param_nom
        r_SFE_tx_loss_var(5) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_tx_loss_6')then
        r_SFE_tx_loss_nom(6) = param_nom
        r_SFE_tx_loss_var(6) = param_var        
      elseif( name_tag(1:i_len).eq.'SFE_rx_loss_1')then
        r_SFE_rx_loss_nom(1) = param_nom
        r_SFE_rx_loss_var(1) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_rx_loss_2')then
        r_SFE_rx_loss_nom(2) = param_nom
        r_SFE_rx_loss_var(2) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_rx_loss_3')then
        r_SFE_rx_loss_nom(3) = param_nom
        r_SFE_rx_loss_var(3) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_rx_loss_4')then
        r_SFE_rx_loss_nom(4) = param_nom
        r_SFE_rx_loss_var(4) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_rx_loss_5')then
        r_SFE_rx_loss_nom(5) = param_nom
        r_SFE_rx_loss_var(5) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_rx_loss_6')then
        r_SFE_rx_loss_nom(6) = param_nom
        r_SFE_rx_loss_var(6) = param_var        
      elseif( name_tag(1:i_len).eq.'SFE_LB_loss_1')then
        r_SFE_LB_loss_nom(1) = param_nom
        r_SFE_LB_loss_var(1) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_LB_loss_2')then
        r_SFE_LB_loss_nom(2) = param_nom
        r_SFE_LB_loss_var(2) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_LB_loss_3')then
        r_SFE_LB_loss_nom(3) = param_nom
        r_SFE_LB_loss_var(3) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_LB_loss_4')then
        r_SFE_LB_loss_nom(4) = param_nom
        r_SFE_LB_loss_var(4) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_LB_loss_5')then
        r_SFE_LB_loss_nom(5) = param_nom
        r_SFE_LB_loss_var(5) = param_var
      elseif( name_tag(1:i_len).eq.'SFE_LB_loss_6')then
        r_SFE_LB_loss_nom(6) = param_nom
        r_SFE_LB_loss_var(6) = param_var
      elseif( name_tag(1:i_len).eq.'SBE_rx_loss_1')then
        r_SBE_rx_loss_nom(1) = param_nom
        r_SBE_rx_loss_var(1) = param_var      
      elseif( name_tag(1:i_len).eq.'SBE_rx_loss_2')then
        r_SBE_rx_loss_nom(2) = param_nom
        r_SBE_rx_loss_var(2) = param_var           
      elseif( name_tag(1:i_len).eq.'SBE_gain')then
        r_SBE_gain_nom = param_nom
        r_SBE_gain_var = param_var
      elseif( name_tag(1:i_len).eq.'OMT_loss_11')then
        r_OMT_loss_nom(1,1) = param_nom
        r_OMT_loss_var(1,1) = param_var
      elseif( name_tag(1:i_len).eq.'OMT_loss_21')then
        r_OMT_loss_nom(2,1) = param_nom
        r_OMT_loss_var(2,1) = param_var
      elseif( name_tag(1:i_len).eq.'OMT_loss_12')then
        r_OMT_loss_nom(1,2) = param_nom
        r_OMT_loss_var(1,2) = param_var
      elseif( name_tag(1:i_len).eq.'OMT_loss_22')then
        r_OMT_loss_nom(2,2) = param_nom
        r_OMT_loss_var(2,2) = param_var
      elseif( name_tag(1:i_len).eq.'OMT_loss_13')then
        r_OMT_loss_nom(1,3) = param_nom
        r_OMT_loss_var(1,3) = param_var
      elseif( name_tag(1:i_len).eq.'OMT_loss_23')then
        r_OMT_loss_nom(2,3) = param_nom
        r_OMT_loss_var(2,3) = param_var  
      elseif( name_tag(1:i_len).eq.'OMT_loss_14')then
        r_OMT_loss_nom(1,4) = param_nom
        r_OMT_loss_var(1,4) = param_var  
      elseif( name_tag(1:i_len).eq.'OMT_loss_24')then
        r_OMT_loss_nom(2,4) = param_nom
        r_OMT_loss_var(2,4) = param_var  
      elseif( name_tag(1:i_len).eq.'OMT_loss_15')then
        r_OMT_loss_nom(1,5) = param_nom
        r_OMT_loss_var(1,5) = param_var  
      elseif( name_tag(1:i_len).eq.'OMT_loss_25')then
        r_OMT_loss_nom(2,5) = param_nom
        r_OMT_loss_var(2,5) = param_var  
      elseif( name_tag(1:i_len).eq.'OMT_loss_16')then
        r_OMT_loss_nom(1,6) = param_nom
        r_OMT_loss_var(1,6) = param_var  
      elseif( name_tag(1:i_len).eq.'OMT_loss_26')then
        r_OMT_loss_nom(2,6) = param_nom
        r_OMT_loss_var(2,6) = param_var  
      elseif( name_tag(1:i_len).eq.'tx_reflect')then
        r_tx_reflect_nom = param_nom
        r_tx_reflect_var = param_var
      elseif( name_tag(1:i_len).eq.'SBE_noise_fig')then
        r_SBE_noise_fig_nom = param_nom
        r_SBE_noise_fig_var = param_var      
      elseif( name_tag(1:i_len).eq.'rx_BW')then
        r_rx_BW_nom = param_nom
        r_rx_BW_var = param_var             
      elseif( name_tag(1:i_len).eq.'tx_BW')then
        r_tx_BW_nom = param_nom
        r_tx_BW_var = param_var 
      elseif( name_tag(1:i_len).eq.'tx_frequency')then
        r_tx_frequency_nom = param_nom
        r_tx_frequency_var = param_var
      elseif( name_tag(1:i_len).eq.'B1_rx_length')then
        r_rx_length_nom(1) = param_nom
        r_rx_length_var(1) = param_var
      elseif( name_tag(1:i_len).eq.'B2_rx_length')then
        r_rx_length_nom(2) = param_nom
        r_rx_length_var(2) = param_var         
      elseif( name_tag(1:i_len).eq.'B3_rx_length')then
        r_rx_length_nom(3) = param_nom
        r_rx_length_var(3) = param_var         
      elseif( name_tag(1:i_len).eq.'chirp_rate')then
        r_chirp_rate_nom = param_nom
        r_chirp_rate_var = param_var          
      elseif( name_tag(1:i_len).eq.'tx_pulse_length')then
        r_tx_pulse_length_nom = param_nom
        r_tx_pulse_length_var = param_var             
      elseif( name_tag(1:i_len).eq.'B1HV')then
        r_B1HV_nom = param_nom
        r_B1HV_var = param_var
      elseif( name_tag(1:i_len).eq.'B1NV')then
        r_B1NV_nom = param_nom
        r_B1NV_var = param_var
      elseif( name_tag(1:i_len).eq.'nominal_temp')then
        r_nominal_temp = param_nom
      elseif( name_tag(1:i_len).eq.'T_cnd_B1')then
        r_T_cnd_nom(1) = param_nom
        r_T_cnd_var(1) = param_var
      elseif( name_tag(1:i_len).eq.'T_cnd_B2')then
        r_T_cnd_nom(2) = param_nom
        r_T_cnd_var(2) = param_var
      elseif( name_tag(1:i_len).eq.'T_cnd_B3')then
        r_T_cnd_nom(3) = param_nom
        r_T_cnd_var(3) = param_var

      elseif( name_tag(1:i_len).eq.'Bias_B1HV')then
        r_gain_bias_nom(1,1,2) = param_nom
        r_gain_bias_var(1,1,2) = param_var      
      elseif( name_tag(1:i_len).eq.'Bias_B1nV')then
        r_gain_bias_nom(1,0,2) = param_nom
        r_gain_bias_var(1,0,2) = param_var      
      elseif( name_tag(1:i_len).eq.'Bias_B1VV')then
        r_gain_bias_nom(1,2,2) = param_nom
        r_gain_bias_var(1,2,2) = param_var      
      elseif( name_tag(1:i_len).eq.'Bias_B1VH')then
        r_gain_bias_nom(1,2,1) = param_nom
        r_gain_bias_var(1,2,1) = param_var    
      elseif( name_tag(1:i_len).eq.'Bias_B1nH')then
        r_gain_bias_nom(1,0,1) = param_nom
        r_gain_bias_var(1,0,1) = param_var    
      elseif( name_tag(1:i_len).eq.'Bias_B1HH')then
        r_gain_bias_nom(1,1,1) = param_nom
        r_gain_bias_var(1,1,1) = param_var    


      elseif( name_tag(1:i_len).eq.'Bias_B2HV')then
        r_gain_bias_nom(2,1,2) = param_nom
        r_gain_bias_var(2,1,2) = param_var      
      elseif( name_tag(1:i_len).eq.'Bias_B2nV')then
        r_gain_bias_nom(2,0,2) = param_nom
        r_gain_bias_var(2,0,2) = param_var      
      elseif( name_tag(1:i_len).eq.'Bias_B2VV')then
        r_gain_bias_nom(2,2,2) = param_nom
        r_gain_bias_var(2,2,2) = param_var      
      elseif( name_tag(1:i_len).eq.'Bias_B2VH')then
        r_gain_bias_nom(2,2,1) = param_nom
        r_gain_bias_var(2,2,1) = param_var    
      elseif( name_tag(1:i_len).eq.'Bias_B2nH')then
        r_gain_bias_nom(2,0,1) = param_nom
        r_gain_bias_var(2,0,1) = param_var    
      elseif( name_tag(1:i_len).eq.'Bias_B2HH')then
        r_gain_bias_nom(2,1,1) = param_nom
        r_gain_bias_var(2,1,1) = param_var    
        
        
      elseif( name_tag(1:i_len).eq.'Bias_B3HV')then
        r_gain_bias_nom(3,1,2) = param_nom
        r_gain_bias_var(3,1,2) = param_var      
      elseif( name_tag(1:i_len).eq.'Bias_B3nV')then
        r_gain_bias_nom(3,0,2) = param_nom
        r_gain_bias_var(3,0,2) = param_var      
      elseif( name_tag(1:i_len).eq.'Bias_B3VV')then
        r_gain_bias_nom(3,2,2) = param_nom
        r_gain_bias_var(3,2,2) = param_var      
      elseif( name_tag(1:i_len).eq.'Bias_B3VH')then
        r_gain_bias_nom(3,2,1) = param_nom
        r_gain_bias_var(3,2,1) = param_var    
      elseif( name_tag(1:i_len).eq.'Bias_B3nH')then
        r_gain_bias_nom(3,0,1) = param_nom
        r_gain_bias_var(3,0,1) = param_var    
      elseif( name_tag(1:i_len).eq.'Bias_B3HH')then
        r_gain_bias_nom(3,1,1) = param_nom
        r_gain_bias_var(3,1,1) = param_var            



      elseif( name_tag(1:i_len).eq.'Averaging_time_B1nH')then
        r_averaging_time_nom(1,1) = param_nom
        r_averaging_time_var(1,1) = param_var            

      elseif( name_tag(1:i_len).eq.'Averaging_time_B1nV')then
        r_averaging_time_nom(1,2) = param_nom
        r_averaging_time_var(1,2) = param_var            

      elseif( name_tag(1:i_len).eq.'Averaging_time_B2nH')then
        r_averaging_time_nom(2,1) = param_nom
        r_averaging_time_var(2,1) = param_var            

      elseif( name_tag(1:i_len).eq.'Averaging_time_B2nV')then
        r_averaging_time_nom(2,2) = param_nom
        r_averaging_time_var(2,2) = param_var            

      elseif( name_tag(1:i_len).eq.'Averaging_time_B3nH')then
        r_averaging_time_nom(3,1) = param_nom
        r_averaging_time_var(3,1) = param_var            

      elseif( name_tag(1:i_len).eq.'Averaging_time_B3nV')then
        r_averaging_time_nom(3,2) = param_nom
        r_averaging_time_var(3,2) = param_var            

      elseif( name_tag(1:i_len).eq.'Averaging_time_LB')then
        r_averaging_time_LB_nom = param_nom
        r_averaging_time_LB_var = param_var     

      elseif( name_tag(1:i_len).eq.'RX_POS_B1')then
        r_rx_pos_nom(1) = param_nom
        r_rx_pos_var(1) = param_var     

      elseif( name_tag(1:i_len).eq.'RX_POS_B2')then
        r_rx_pos_nom(2) = param_nom
        r_rx_pos_var(2) = param_var     

      elseif( name_tag(1:i_len).eq.'RX_POS_B3')then
        r_rx_pos_nom(3) = param_nom
        r_rx_pos_var(3) = param_var     
      
      elseif( name_tag(1:i_len).eq.'RFI_HALF_WINDOW_SIZE_HRX')then
        i_rfi_half_window_size_hrx = param_nom

      elseif( name_tag(1:i_len).eq.'RFI_HALF_WINDOW_SIZE_VRX')then
        i_rfi_half_window_size_vrx = param_nom

      elseif( name_tag(1:i_len).eq.'VRX_NOISE_THRESHOLD')then
        r_vrx_noise_threshold = param_nom

      elseif( name_tag(1:i_len).eq.'VRX_NOISE_CND_THRESHOLD')then
        r_vrx_noise_cnd_threshold = param_nom

      elseif( name_tag(1:i_len).eq.'HRX_NOISE_THRESHOLD')then
        r_hrx_noise_threshold = param_nom

      elseif( name_tag(1:i_len).eq.'VRX_ECHO_PWR_STD_THRESHOLD')then
        r_vrx_echo_pwr_std_threshold = param_nom

      elseif( name_tag(1:i_len).eq.'VRX_NOISE_PWR_STD_THRESHOLD')then
        r_vrx_noise_pwr_std_threshold = param_nom

      elseif( name_tag(1:i_len).eq.'VRX_NOISE_CND_PWR_STD_THRESHOLD')then
        r_vrx_noise_cnd_pwr_std_threshold = param_nom

      elseif( name_tag(1:i_len).eq.'HRX_ECHO_PWR_STD_THRESHOLD')then
        r_hrx_echo_pwr_std_threshold = param_nom

      elseif( name_tag(1:i_len).eq.'HRX_NOISE_PWR_STD_THRESHOLD')then
        r_hrx_noise_pwr_std_threshold = param_nom

      elseif( name_tag(1:i_len).eq.'RFI_FILTER_MAX_STD')then
        r_rfi_filter_max_std = param_nom
      
      elseif( name_tag(1:i_len).eq.'ROLL_BIAS')then
        roll_bias = param_nom

      elseif( name_tag(1:i_len).eq.'PITCH_BIAS')then
        pitch_bias = param_nom

      elseif( name_tag(1:i_len).eq.'YAW_BIAS')then
        yaw_bias = param_nom
      
      else
        write(6,*) 'L1B_read_params: WARNING: name_tag:',name_tag(1:i_len),' not found!!'
      endif
      goto 123
321   continue
      close(50)

      if( i_dbg_level .ge. 4 )then      
        write(6,*) 'r_SSPA_PowerW: ', r_SSPA_PowerW_nom, r_SSPA_PowerW_var
        
        write(6,*) 'r_SFE_tx_loss_1: ',r_SFE_tx_loss_nom(1),r_SFE_tx_loss_var(1)
        write(6,*) 'r_SFE_tx_loss_2: ',r_SFE_tx_loss_nom(2),r_SFE_tx_loss_var(2)
        write(6,*) 'r_SFE_tx_loss_3: ',r_SFE_tx_loss_nom(3),r_SFE_tx_loss_var(3)
        write(6,*) 'r_SFE_tx_loss_4: ',r_SFE_tx_loss_nom(4),r_SFE_tx_loss_var(4)
        write(6,*) 'r_SFE_tx_loss_5: ',r_SFE_tx_loss_nom(5),r_SFE_tx_loss_var(5)
        write(6,*) 'r_SFE_tx_loss_6: ',r_SFE_tx_loss_nom(6),r_SFE_tx_loss_var(6)
        
        write(6,*) 'r_SFE_rx_loss_1: ',r_SFE_rx_loss_nom(1),r_SFE_rx_loss_var(1)
        write(6,*) 'r_SFE_rx_loss_2: ',r_SFE_rx_loss_nom(2),r_SFE_rx_loss_var(2)
        write(6,*) 'r_SFE_rx_loss_3: ',r_SFE_rx_loss_nom(3),r_SFE_rx_loss_var(3)
        write(6,*) 'r_SFE_rx_loss_4: ',r_SFE_rx_loss_nom(4),r_SFE_rx_loss_var(4)
        write(6,*) 'r_SFE_rx_loss_5: ',r_SFE_rx_loss_nom(5),r_SFE_rx_loss_var(5)
        write(6,*) 'r_SFE_rx_loss_6: ',r_SFE_rx_loss_nom(6),r_SFE_rx_loss_var(6)
        
        write(6,*) 'r_SFE_LB_loss_1: ',r_SFE_LB_loss_nom(1),r_SFE_LB_loss_var(1)
        write(6,*) 'r_SFE_LB_loss_2: ',r_SFE_LB_loss_nom(2),r_SFE_LB_loss_var(2)
        write(6,*) 'r_SFE_LB_loss_3: ',r_SFE_LB_loss_nom(3),r_SFE_LB_loss_var(3)
        write(6,*) 'r_SFE_LB_loss_4: ',r_SFE_LB_loss_nom(4),r_SFE_LB_loss_var(4)
        write(6,*) 'r_SFE_LB_loss_5: ',r_SFE_LB_loss_nom(5),r_SFE_LB_loss_var(5)
        write(6,*) 'r_SFE_LB_loss_6: ',r_SFE_LB_loss_nom(6),r_SFE_LB_loss_var(6)
        
        write(6,*) 'r_SBE_rx_loss_1: ',r_SBE_rx_loss_nom(1),r_SBE_rx_loss_var(1)
        write(6,*) 'r_SBE_rx_loss_2: ',r_SBE_rx_loss_nom(2),r_SBE_rx_loss_var(2)
        
        write(6,*) 'r_SBE_gain: ',r_SBE_gain_nom,r_SBE_gain_var
        
        write(6,*) 'r_OMT_loss_11: ',r_OMT_loss_nom(1,1),r_OMT_loss_var(1,1)
        write(6,*) 'r_OMT_loss_21: ',r_OMT_loss_nom(2,1),r_OMT_loss_var(2,1)
        write(6,*) 'r_OMT_loss_12: ',r_OMT_loss_nom(1,2),r_OMT_loss_var(1,2)
        write(6,*) 'r_OMT_loss_22: ',r_OMT_loss_nom(2,2),r_OMT_loss_var(2,2)
        write(6,*) 'r_OMT_loss_13: ',r_OMT_loss_nom(1,3),r_OMT_loss_var(1,3)
        write(6,*) 'r_OMT_loss_23: ',r_OMT_loss_nom(2,3),r_OMT_loss_var(2,3)
        write(6,*) 'r_OMT_loss_14: ',r_OMT_loss_nom(1,4),r_OMT_loss_var(1,4)
        write(6,*) 'r_OMT_loss_24: ',r_OMT_loss_nom(2,4),r_OMT_loss_var(2,4)
        write(6,*) 'r_OMT_loss_15: ',r_OMT_loss_nom(1,5),r_OMT_loss_var(1,5)
        write(6,*) 'r_OMT_loss_25: ',r_OMT_loss_nom(2,5),r_OMT_loss_var(2,5)
        write(6,*) 'r_OMT_loss_16: ',r_OMT_loss_nom(1,6),r_OMT_loss_var(1,6)
        write(6,*) 'r_OMT_loss_26: ',r_OMT_loss_nom(2,6),r_OMT_loss_var(2,6)
        
        write(6,*) 'r_tx_reflect: ',r_tx_reflect_nom,r_tx_reflect_var
        write(6,*) 'r_SBE_noise_fig: ',r_SBE_noise_fig_nom,r_SBE_noise_fig_var
        
        write(6,*) 'r_rx_BW: ',r_rx_BW_nom,r_rx_BW_var
        write(6,*) 'r_tx_BW: ',r_tx_BW_nom,r_tx_BW_var
        
        write(6,*) 'r_tx_frequency: ',r_tx_frequency_nom,r_tx_frequency_var

        write(6,*) 'r_B1_rx_length: ',r_rx_length_nom(1),r_rx_length_var(1)
        write(6,*) 'r_B2_rx_length: ',r_rx_length_nom(2),r_rx_length_var(2)
        write(6,*) 'r_B3_rx_length: ',r_rx_length_nom(3),r_rx_length_var(3)

        write(6,*) 'r_chirp_rate: ',r_chirp_rate_nom,r_chirp_rate_var
        write(6,*) 'r_tx_pulse_length: ',r_tx_pulse_length_nom,r_tx_pulse_length_var

        write(6,*) 'r_B1HV: ',r_B1HV_nom,r_B1HV_var
        write(6,*) 'r_B1NV: ',r_B1NV_nom,r_B1NV_var

        write(6,*) 'r_nominal_temp: ',r_nominal_temp

        write(6,*) 'r_T_cnd_B1: ',r_T_cnd_nom(1),r_T_cnd_var(1)
        write(6,*) 'r_T_cnd_B2: ',r_T_cnd_nom(2),r_T_cnd_var(2)
        write(6,*) 'r_T_cnd_B3: ',r_T_cnd_nom(3),r_T_cnd_var(3)
        
        write(6,*) 'r_gain_bias_B1HV: ',r_gain_bias_nom(1,1,2),r_gain_bias_var(1,1,2)
        write(6,*) 'r_gain_bias_B1nV: ',r_gain_bias_nom(1,0,2),r_gain_bias_var(1,0,2)
        write(6,*) 'r_gain_bias_B1VV: ',r_gain_bias_nom(1,2,2),r_gain_bias_var(1,2,2)

        write(6,*) 'r_gain_bias_B2HV: ',r_gain_bias_nom(2,1,2),r_gain_bias_var(2,1,2)
        write(6,*) 'r_gain_bias_B2nV: ',r_gain_bias_nom(2,0,2),r_gain_bias_var(2,0,2)
        write(6,*) 'r_gain_bias_B2VV: ',r_gain_bias_nom(2,2,2),r_gain_bias_var(2,2,2)

        write(6,*) 'r_gain_bias_B3HV: ',r_gain_bias_nom(3,1,2),r_gain_bias_var(3,1,2)
        write(6,*) 'r_gain_bias_B3nV: ',r_gain_bias_nom(3,0,2),r_gain_bias_var(3,0,2)
        write(6,*) 'r_gain_bias_B3VV: ',r_gain_bias_nom(3,2,2),r_gain_bias_var(3,2,2)

        write(6,*) 'r_averaging_time_B1nH: ',r_averaging_time_nom(1,1),r_averaging_time_var(1,1)
        write(6,*) 'r_averaging_time_B1nV: ',r_averaging_time_nom(1,2),r_averaging_time_var(1,2)

        write(6,*) 'r_averaging_time_B2nH: ',r_averaging_time_nom(2,1),r_averaging_time_var(2,1)
        write(6,*) 'r_averaging_time_B2nV: ',r_averaging_time_nom(2,2),r_averaging_time_var(2,2)

        write(6,*) 'r_averaging_time_B3nH: ',r_averaging_time_nom(3,1),r_averaging_time_var(3,1)
        write(6,*) 'r_averaging_time_B3nV: ',r_averaging_time_nom(3,2),r_averaging_time_var(3,2)

        write(6,*) 'r_averaging_time_LB: ',r_averaging_time_LB_nom,r_averaging_time_LB_var
        
        write(6,*) 'r_rx_pos_B1: ', r_rx_pos_nom(1), r_rx_pos_var(1)
        write(6,*) 'r_rx_pos_B2: ', r_rx_pos_nom(2), r_rx_pos_var(2)
        write(6,*) 'r_rx_pos_B3: ', r_rx_pos_nom(3), r_rx_pos_var(3)
        
        
        write(6,*) 'i_rfi_half_window_size_hrx:        ', i_rfi_half_window_size_hrx
        write(6,*) 'i_rfi_half_window_size_vrx:        ', i_rfi_half_window_size_vrx
        write(6,*) 'r_vrx_noise_threshold:             ', r_vrx_noise_threshold
        write(6,*) 'r_vrx_noise_cnd_threshold:         ', r_vrx_noise_cnd_threshold
        write(6,*) 'r_hrx_noise_threshold:             ', r_hrx_noise_threshold
        write(6,*) 'r_vrx_echo_pwr_std_threshold:      ', r_vrx_echo_pwr_std_threshold
        write(6,*) 'r_vrx_noise_pwr_std_threshold:     ', r_vrx_noise_pwr_std_threshold
        write(6,*) 'r_vrx_noise_cnd_pwr_std_threshold: ', r_vrx_noise_cnd_pwr_std_threshold
        write(6,*) 'r_hrx_echo_pwr_std_threshold:      ', r_hrx_echo_pwr_std_threshold
        write(6,*) 'r_hrx_noise_pwr_std_threshold:     ', r_hrx_noise_pwr_std_threshold
        write(6,*) 'r_rfi_filter_max_std:              ', r_rfi_filter_max_std
        write(6,*) 'roll_bias:                         ', roll_bias
        write(6,*) 'pitch_bias:                        ', pitch_bias
        write(6,*) 'yaw_bias:                          ', yaw_bias
        pause
      endif
      end
