C--$Header: /u/caretr2/scatsim/CVSROOT/L1A_to_L2/L1B_write_HDF4.f,v 1.9 2011/10/17 12:38:43 fore Exp $
c*****************************************************************
      subroutine L1B_write_HDF4( 
     $                           r_time, 
     $                           r_xyz, 
     $                           r_vxyz,
     $                           r_roll,
     $                           r_yaw,
     $                           r_pitch,
     $                           r_ant_azi_rel_north,
     $                           r_pol_roll,
     $                           r_inc_angle,
     $                           r_lat_edge,
     $                           r_lon_edge, 
     $                           r_sig0, 
     $                           r_Kpc, 
     $                           r_SNR, 
     $                           r_Noise,
     $                           r_Noise_Pwr,
     $                           r_lat_echo_center,
     $                           r_lon_echo_center, 
     $                           i_quality_flag,
     $                           r_land_fraction,
     $                           r_ice_fraction,
     $                           housekeeping,
     $                           NUM_BLOCKS, 
     $                           block_start,
     $                           L2_prefix  )
c*****************************************************************
c**   
c**   FILE NAME:  write_level_1.f
c**   
c**   DATE WRITTEN:  8/28/2008
c**   
c**   PROGRAMMER:  Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: Updated and integrated with L1A_to_L2 software 
c**    8/13/2009
c**    
c**
c*****************************************************************
      implicit none
      
      include 'hdf.inc'
      include 'L1A.inc'
      
c     PARAMETER STATEMENTS     
c     INPUT VARIABLES
      real*8 r_time(MAX_NUM_BLOCKS)
      real*8 r_xyz(3,MAX_NUM_BLOCKS)
      real*8 r_vxyz(3,MAX_NUM_BLOCKS)
      real*8 r_roll(MAX_NUM_BLOCKS)
      real*8 r_yaw(MAX_NUM_BLOCKS)
      real*8 r_pitch(MAX_NUM_BLOCKS)
      real*8 r_ant_azi_rel_north(3,MAX_NUM_BLOCKS)
      real*8 r_pol_roll(3,MAX_NUM_BLOCKS)
      real*8 r_inc_angle(3,MAX_NUM_BLOCKS)
      real*8 r_lat_edge(4,3,8,MAX_NUM_BLOCKS)
      real*8 r_lon_edge(4,3,8,MAX_NUM_BLOCKS)
      real*8 r_sig0(4,3,8,MAX_NUM_BLOCKS)
      real*8 r_Kpc(4,3,8,MAX_NUM_BLOCKS)
      real*8 r_SNR(4,3,8,MAX_NUM_BLOCKS)
      real*8 r_Noise(2,3,8,MAX_NUM_BLOCKS)
      real*8 r_Noise_Pwr(2,3,8,MAX_NUM_BLOCKS)
      real*8 r_lat_echo_center(3,8,MAX_NUM_BLOCKS)
      real*8 r_lon_echo_center(3,8,MAX_NUM_BLOCKS)
      real*8 r_land_fraction(3,MAX_NUM_BLOCKS)
      real*8 r_ice_fraction(3,MAX_NUM_BLOCKS)
      real*4 housekeeping(35,MAX_NUM_BLOCKS)
      integer*2 i_quality_flag(4,3,8,MAX_NUM_BLOCKS)

      character*320 L2_prefix

      integer NUM_BLOCKS
      integer block_start
      
c     OUTPUT VARIABLES
c     LOCAL VARIABLES
      integer i_len, i_sds_id, i_sd_id, i_stat, i_rank, i_dims(4), 
     $ i_start(4), i_stride(4), i_edges(4)
      integer i_year, i_month, i_day, i_hour, i_minute, i_second,
     $ i_day2
      real*8 r_frac_seconds
      character*320 str_hdf_fullpath2file_out, str_hdf_file_out
      
      integer i_block
      integer i_num_blocks
      integer lnblnk

c     DATA STATEMENTS
c     COMMON BLOCKS
      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
      integer i_dbg_level
      common /debug_level/ i_dbg_level     
      
c     FUNCTION STATEMENTS
      integer sfstart, sfcreate, sfwdata, sfendacc, sfend, heprnt
      
c     PROCESSING STEPS
      i_num_blocks = NUM_BLOCKS
C  Generate output file name.

C--Find the 1st valid time tag (using quality flags)...
      call JD2CAL(r_time(block_start), i_year, i_month, i_day, i_hour, 
     $ i_minute, i_second, r_frac_seconds)
      
      call convert_day_and_month_to_dayofyear(i_year, i_day, 
     $ i_month, i_day2)
     
900   format('Q',I4.4,I3.3,I2.2,I2.2,I2.2,'_L1B.hdf') 

!      write(str_hdf_file_out,900) i_year, i_day2, i_hour, i_minute, i_second

       str_hdf_file_out = L2_prefix(1:lnblnk(L2_prefix)) // '_L1B.hdf'
     
C  Generate full path to output file.
      i_len = lnblnk(AQ_SCAT_SIM_DIR_OUT)
      str_hdf_fullpath2file_out = AQ_SCAT_SIM_DIR_OUT(1:i_len) // '/' //
     $ str_hdf_file_out

      write(6,*) str_hdf_fullpath2file_out(1:lnblnk(str_hdf_fullpath2file_out))

C  Create HDF File.      
      i_sd_id = sfstart(str_hdf_fullpath2file_out,DFACC_CREATE)
      if(i_sd_id.eq.FAIL)then
        write(6,*) 'sfstart failed.'
        i_stat=heprnt(0)
      else
         i_len=lnblnk(str_hdf_file_out)
         write(6,*) '... ',str_hdf_file_out(1:i_len),' created'
      end if        

C--Write out r_time---------------------------------------------
      i_rank=2                          ! rank of data set and size of each dim.
      i_dims(1) = 1
      i_dims(2) = i_num_blocks
      
      i_start(1) = 0            ! where to put data
      i_start(2) = 0
      
      i_edges(1) = 1
      i_edges(2) = i_num_blocks
      
      i_stride(1) = 1
      i_stride(2) = 1

      i_sds_id = sfcreate(i_sd_id, 'Julian_Time', DFNT_NFLOAT64, i_rank, 
     $ i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Julian_Time created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_time)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if


C--Write out r_xyz, r_vxyz---------------------------------------------
      i_rank=2                          ! rank of data set and size of each dim.
      i_dims(1) = 3
      i_dims(2) = i_num_blocks
      
      i_start(1) = 0            ! where to put data
      i_start(2) = 0
      
      i_edges(1) = 3
      i_edges(2) = i_num_blocks
      
      i_stride(1) = 1
      i_stride(2) = 1

C------Write out r_xyz------------------------------------------------  
      i_sds_id = sfcreate(i_sd_id, 'SC_Position', DFNT_NFLOAT64, i_rank, 
     $ i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... SC_Position created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_xyz)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if
C------Write out r_vxyz------------------------------------------------  
      i_sds_id = sfcreate(i_sd_id, 'SC_Velocity', DFNT_NFLOAT64, i_rank, 
     $ i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... SC_velocity created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_vxyz)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if      

C--Write out r_roll,r_yaw,r_pitch---------------------------------------
      i_rank=2                          ! rank of data set and size of each dim.
      i_dims(1) = 1
      i_dims(2) = i_num_blocks
      
      i_start(1) = 0            ! where to put data
      i_start(2) = 0
      
      i_edges(1) = 1
      i_edges(2) = i_num_blocks
      
      i_stride(1) = 1
      i_stride(2) = 1

      i_sds_id = sfcreate(i_sd_id, 'SC_Roll', DFNT_NFLOAT64, i_rank, 
     $ i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... SC_roll created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_roll)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if
      
      i_sds_id = sfcreate(i_sd_id, 'SC_Yaw', DFNT_NFLOAT64, i_rank, 
     $ i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... SC_yaw created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_yaw)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if

      i_sds_id = sfcreate(i_sd_id, 'SC_Pitch', DFNT_NFLOAT64, i_rank, 
     $ i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... SC_Pitch created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_pitch)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if

C--Write out r_dop_f_shift,r_ant_azi_rel_north,r_pol_roll,r_inc_angle--
      i_rank=2                          ! rank of data set and size of each dim.
      i_dims(1) = 3
      i_dims(2) = i_num_blocks
      
      i_start(1) = 0            ! where to put data
      i_start(2) = 0
      
      i_edges(1) = 3
      i_edges(2) = i_num_blocks
      
      i_stride(1) = 1
      i_stride(2) = 1

      i_sds_id = sfcreate(i_sd_id, 'Antenna_Azimuth_Relative_North', 
     $ DFNT_NFLOAT64, i_rank, i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Antenna_Azimuth_Relative_North created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges,
     $ r_ant_azi_rel_north)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if

      i_sds_id = sfcreate(i_sd_id, 'Polarization_Roll_Angle', 
     $ DFNT_NFLOAT64, i_rank, i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Polarization_Roll_Angle created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges,
     $ r_pol_roll)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if

      i_sds_id = sfcreate(i_sd_id, 'Incidence_Angle', 
     $ DFNT_NFLOAT64, i_rank, i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Incidence_Angle created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges,
     $ r_inc_angle)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if


C--Write out r_lat_edge, r_lon_edge----------------------------------
      i_rank=4                          ! rank of data set and size of each dim.
      i_dims(1) = 4
      i_dims(2) = 3
      i_dims(3) = 8
      i_dims(4) = i_num_blocks
      
      i_start(1) = 0            ! where to put data
      i_start(2) = 0
      i_start(3) = 0
      i_start(4) = 0
      
      i_edges(1) = 4
      i_edges(2) = 3
      i_edges(3) = 8
      i_edges(4) = i_num_blocks
      
      i_stride(1) = 1
      i_stride(2) = 1
      i_stride(3) = 1
      i_stride(4) = 1      
      
      i_sds_id = sfcreate(i_sd_id, 'Latitude_Beam_Edges', 
     $ DFNT_NFLOAT64, i_rank, i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Latitude_Beam_Edges created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges,
     $ r_lat_edge)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if
      i_sds_id = sfcreate(i_sd_id, 'Longitude_Beam_Edges', 
     $ DFNT_NFLOAT64, i_rank, i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Longitude_Beam_Edges created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges,
     $ r_lon_edge)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if



C--Write out r_sig0, r_Kpc, r_SNR
      i_rank=4                          ! rank of data set and size of each dim.
      i_dims(1) = 4
      i_dims(2) = 3
      i_dims(3) = 8
      i_dims(4) = i_num_blocks
      
      i_start(1) = 0            ! where to put data
      i_start(2) = 0
      i_start(3) = 0
      i_start(4) = 0
      
      i_edges(1) = 4
      i_edges(2) = 3
      i_edges(3) = 8
      i_edges(4) = i_num_blocks
      
      i_stride(1) = 1
      i_stride(2) = 1
      i_stride(3) = 1
      i_stride(4) = 1

C------Write out r_sig0------------------------------------------------  
      i_sds_id = sfcreate(i_sd_id, 'Sigma0', DFNT_NFLOAT64, i_rank, 
     $ i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Sigma0 created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_sig0)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if
C------Write out r_Kpc------------------------------------------------  
      i_sds_id = sfcreate(i_sd_id, 'Kpc', DFNT_NFLOAT64, i_rank, 
     $ i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Kpc created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_Kpc)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if     
C------Write out r_SNR------------------------------------------------  
      i_sds_id = sfcreate(i_sd_id, 'SNR', DFNT_NFLOAT64, i_rank, 
     $ i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... SNR created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_SNR)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if        
C------Write out r_Noise, r_Noise_Pwr------------------------------------      
      i_dims(1)=2
      i_edges(1)=2

      i_sds_id = sfcreate(i_sd_id, 'Sigma0_Noise', DFNT_NFLOAT64, 
     $ i_rank, i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Sigma0_Noise created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_Noise)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      endif
 
      i_sds_id = sfcreate(i_sd_id, 'Noise_Power', DFNT_NFLOAT64, 
     $ i_rank, i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Noise_Power created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_Noise_Pwr)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      endif
 
 
C--Write out r_lat_echo_center and r_lon_echo_center
      i_rank=3                          ! rank of data set and size of each dim.
      i_dims(1) = 3
      i_dims(2) = 8
      i_dims(3) = i_num_blocks
      
      i_start(1) = 0            ! where to put data
      i_start(2) = 0
      i_start(3) = 0
      
      i_edges(1) = 3
      i_edges(2) = 8
      i_edges(3) = i_num_blocks
      
      i_stride(1) = 1
      i_stride(2) = 1
      i_stride(3) = 1
      
C------Write out r_lat_echo_center------------------------------------
      i_sds_id = sfcreate(i_sd_id, 'Lat_Echo_Center', DFNT_NFLOAT64,
     $ i_rank, i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Lat_Echo_Center created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, 
     $ r_lat_echo_center)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if          
      
C------Write out r_lon_echo_center------------------------------------
      i_sds_id = sfcreate(i_sd_id, 'Lon_Echo_Center', DFNT_NFLOAT64,
     $ i_rank, i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Lon_Echo_Center created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, 
     $ r_lon_echo_center)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if          

C--Write out Housekeeping---------------------------------------
      i_rank=2                          ! rank of data set and size of each dim.
      i_dims(1) = 35
      i_dims(2) = i_num_blocks
      
      i_start(1) = 0            ! where to put data
      i_start(2) = 0
      
      i_edges(1) = 35
      i_edges(2) = i_num_blocks
      
      i_stride(1) = 1
      i_stride(2) = 1

      i_sds_id = sfcreate(i_sd_id, 'Housekeeping', DFNT_NFLOAT32,
     $ i_rank, i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Housekeeping created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, 
     $ housekeeping )
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if 
      
C--Write out i_quality_flag
      i_rank=4                          ! rank of data set and size of each dim.
      i_dims(1) = 4
      i_dims(2) = 3
      i_dims(3) = 8
      i_dims(4) = i_num_blocks
      
      i_start(1) = 0            ! where to put data
      i_start(2) = 0
      i_start(3) = 0
      i_start(4) = 0
      
      i_edges(1) = 4
      i_edges(2) = 3
      i_edges(3) = 8
      i_edges(4) = i_num_blocks
      
      i_stride(1) = 1
      i_stride(2) = 1
      i_stride(3) = 1
      i_stride(4) = 1

      i_sds_id = sfcreate(i_sd_id, 'Quality_Flag', DFNT_UINT16, i_rank, 
     $ i_dims )
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Quality_Flag created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, i_quality_flag)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if      
      

C--Write out r_land_fraction, r_ice_fraction---------------------------------------------
      i_rank=2                          ! rank of data set and size of each dim.
      i_dims(1) = 3
      i_dims(2) = i_num_blocks
      
      i_start(1) = 0            ! where to put data
      i_start(2) = 0
      
      i_edges(1) = 3
      i_edges(2) = i_num_blocks
      
      i_stride(1) = 1
      i_stride(2) = 1

C------Write out r_land_fraction------------------------------------------------  
      i_sds_id = sfcreate(i_sd_id, 'Land_Fraction', DFNT_NFLOAT64, i_rank, 
     $ i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Land_Fraction created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_land_fraction)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if
C------Write out r_ice_fraction------------------------------------------------  
      i_sds_id = sfcreate(i_sd_id, 'Ice_Fraction', DFNT_NFLOAT64, i_rank, 
     $ i_dims)
      if(i_sds_id.eq.FAIL)then
        print*,'ERROR: sfcreate failed.'
        i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... Ice_Fraction created'
      end if        

C     Write the stored data to the data set.
      i_stat =  sfwdata(i_sds_id, i_start, i_stride, i_edges, r_ice_fraction)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfwdata failed.'
         i_stat=heprnt(0)
      else
         if( i_dbg_level.ge.1) print*,'... data written out'
      end if      
      
C     Terminate access to the array.
      i_stat =  sfendacc(i_sds_id)
      if(i_stat.eq.FAIL) then
         print*,'ERROR: sfendacc failed.'
         i_stat=heprnt(0)
      end if      



      
C----Terminate access to the SD interface and close the file.==========
      i_stat =  sfend(i_sd_id)
      if (i_stat .eq. FAIL) then
         print*,'ERROR: sfend failed.'
         i_stat=heprnt(0)
      else
         print*,'... file closed'
         print*
      endif
      
      end
