C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L2_APC_correction.f,v 1.1 2010/03/25 17:36:12 fore Exp $
c*****************************************************************      
      subroutine L2_APC_correction( L2_sigma_ANT,
     $                              L2_var_sigma_ANT,
     $                              L2_APC_file,
     $                              flag_L2_apc_file_entered,
     $                              L2_sigma_TOI,
     $                              L2_var_sigma_TOI )
c*****************************************************************      
c**   
c**   FILE NAME: L2_APC_correction.f
c**   
c**   DATE WRITTEN: 3/18/2010
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  Applies a antenna pattern correction
c**     to the ANT sigma-0s.  Produces a symmetrized cross-pol
c**     channel.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
      integer stdout, stderr, stdin
      parameter( stdout = 6, stderr = 0, stdin = 5 )
c     INPUT VARIABLES
      real*8        L2_sigma_ANT(4,3)       ! antenna sigma-0 [linear units]
      real*8        L2_var_sigma_ANT(4,3)   ! variance of antenna sigma 0
      character*(*) L2_APC_file
      logical       flag_L2_apc_file_entered
c     OUTPUT VARIABLES
      real*8        L2_sigma_TOI(4,3)       ! top of ionosphere sigma-0 [linear units]
      real*8        L2_var_sigma_TOI(4,3)   ! variance of TOI sigma-0   [linear units]
c     LOCAL VARIABLES
      logical       l_apc_loaded   /.false./
      real*8        L2_APC_mat(3,4,3)       ! TOI pol (hh,hv,vv); ANT pol (hh hv vh vv ); beam (1,2,3)
      
      real*8        sig0_TOI_HH
      real*8        sig0_TOI_HV
      real*8        sig0_TOI_VV

      real*8        var_sig0_TOI_HH
      real*8        var_sig0_TOI_HV
      real*8        var_sig0_TOI_VV
      
      integer*4     i_beam
      integer*4     i_chan
      character*80  junk
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
c     SAVE STATEMENTS
      save l_apc_loaded, L2_APC_mat
      
      
      if( .not.l_apc_loaded ) then
        if( flag_L2_apc_file_entered ) then
          open(50,file=L2_APC_file)
          do i_beam = 1, 3
            read(50,'(a)') junk
            read(50,*) (L2_APC_mat(1,i_chan,i_beam),i_chan=1,4)
            read(50,*) (L2_APC_mat(2,i_chan,i_beam),i_chan=1,4)
            read(50,*) (L2_APC_mat(3,i_chan,i_beam),i_chan=1,4)
          enddo
          close(50)
        else
          do i_beam = 1, 3
            L2_APC_mat(1,1,i_beam) = 1.0d0
            L2_APC_mat(1,2,i_beam) = 0.0d0
            L2_APC_mat(1,3,i_beam) = 0.0d0
            L2_APC_mat(1,4,i_beam) = 0.0d0
            
            L2_APC_mat(2,1,i_beam) = 0.0d0
            L2_APC_mat(2,2,i_beam) = 0.5d0
            L2_APC_mat(2,3,i_beam) = 0.5d0
            L2_APC_mat(2,4,i_beam) = 0.0d0
            
            L2_APC_mat(3,1,i_beam) = 0.0d0
            L2_APC_mat(3,2,i_beam) = 0.0d0
            L2_APC_mat(3,3,i_beam) = 0.0d0
            L2_APC_mat(3,4,i_beam) = 1.0d0
          enddo
        endif
        
        if( i_dbg_level.ge.2 ) then
          do i_beam = 1, 3
            write(6,*) 'L2_APC_mat, beam: ', i_beam
            write(6,*) (L2_APC_mat(1,i_chan,i_beam),i_chan=1,4)
            write(6,*) (L2_APC_mat(2,i_chan,i_beam),i_chan=1,4)
            write(6,*) (L2_APC_mat(3,i_chan,i_beam),i_chan=1,4)
          enddo
        endif
        
        l_apc_loaded = .true.
      endif
      
      do i_beam = 1, 3
        ! Compute Sigma TOI
        ! hh polarization
        sig0_TOI_HH = L2_APC_mat(1,1,i_beam) * L2_sigma_ANT(4,i_beam) + ! hh ANT
     $                L2_APC_mat(1,2,i_beam) * L2_sigma_ANT(1,i_beam) + ! hv ANT
     $                L2_APC_mat(1,3,i_beam) * L2_sigma_ANT(3,i_beam) + ! vh ANT
     $                L2_APC_mat(1,4,i_beam) * L2_sigma_ANT(2,i_beam)   ! vv ANT

        ! hv polarization
        sig0_TOI_HV = L2_APC_mat(2,1,i_beam) * L2_sigma_ANT(4,i_beam) + ! hh ANT
     $                L2_APC_mat(2,2,i_beam) * L2_sigma_ANT(1,i_beam) + ! hv ANT
     $                L2_APC_mat(2,3,i_beam) * L2_sigma_ANT(3,i_beam) + ! vh ANT
     $                L2_APC_mat(2,4,i_beam) * L2_sigma_ANT(2,i_beam)   ! vv ANT

        ! vv polarization
        sig0_TOI_VV = L2_APC_mat(3,1,i_beam) * L2_sigma_ANT(4,i_beam) + ! hh ANT
     $                L2_APC_mat(3,2,i_beam) * L2_sigma_ANT(1,i_beam) + ! hv ANT
     $                L2_APC_mat(3,3,i_beam) * L2_sigma_ANT(3,i_beam) + ! vh ANT
     $                L2_APC_mat(3,4,i_beam) * L2_sigma_ANT(2,i_beam)   ! vv ANT


        ! The variance propagation is straight-forward
        ! Compute Var Sigma TOI (hh)
        var_sig0_TOI_HH = L2_APC_mat(1,1,i_beam)**2 * L2_var_sigma_ANT(4,i_beam) + ! hh ANT
     $                    L2_APC_mat(1,2,i_beam)**2 * L2_var_sigma_ANT(1,i_beam) + ! hv ANT
     $                    L2_APC_mat(1,3,i_beam)**2 * L2_var_sigma_ANT(3,i_beam) + ! vh ANT
     $                    L2_APC_mat(1,4,i_beam)**2 * L2_var_sigma_ANT(2,i_beam)   ! vv ANT

        ! Compute Var Sigma TOI (hv)
        var_sig0_TOI_HV = L2_APC_mat(2,1,i_beam)**2 * L2_var_sigma_ANT(4,i_beam) + ! hh ANT
     $                    L2_APC_mat(2,2,i_beam)**2 * L2_var_sigma_ANT(1,i_beam) + ! hv ANT
     $                    L2_APC_mat(2,3,i_beam)**2 * L2_var_sigma_ANT(3,i_beam) + ! vh ANT
     $                    L2_APC_mat(2,4,i_beam)**2 * L2_var_sigma_ANT(2,i_beam)   ! vv ANT

        ! Compute Var Sigma TOI (vv)
        var_sig0_TOI_VV = L2_APC_mat(3,1,i_beam)**2 * L2_var_sigma_ANT(4,i_beam) + ! hh ANT
     $                    L2_APC_mat(3,2,i_beam)**2 * L2_var_sigma_ANT(1,i_beam) + ! hv ANT
     $                    L2_APC_mat(3,3,i_beam)**2 * L2_var_sigma_ANT(3,i_beam) + ! vh ANT
     $                    L2_APC_mat(3,4,i_beam)**2 * L2_var_sigma_ANT(2,i_beam)   ! vv ANT
        
        ! copy to output variables.
        L2_sigma_TOI(1,i_beam)     = sig0_TOI_HV
        L2_sigma_TOI(2,i_beam)     = sig0_TOI_VV
        L2_sigma_TOI(3,i_beam)     = sig0_TOI_HV
        L2_sigma_TOI(4,i_beam)     = sig0_TOI_HH
        
        L2_var_sigma_TOI(1,i_beam) = var_sig0_TOI_HV
        L2_var_sigma_TOI(2,i_beam) = var_sig0_TOI_VV
        L2_var_sigma_TOI(3,i_beam) = var_sig0_TOI_HV
        L2_var_sigma_TOI(4,i_beam) = var_sig0_TOI_HH
      enddo
      
      end
