C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L2_PALS_delta_TB_model.f,v 1.1 2009/12/10 22:08:03 fore Exp $
c*****************************************************************      
      subroutine L2_PALS_delta_TB_model( wind_speed,
     $                                   wind_speed_err,
     $                                   inc_angle,
     $                                   delta_TB,
     $                                   delta_TB_err )
c*****************************************************************      
c**   
c**   FILE NAME: L2_PALS_delta_TB_model.f
c**   
c**   DATE WRITTEN: 12/2/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  Estimates delta TB from wind speed;
c**    Also estimates 1-sigma error given 1-sigma error in wind
c**    speed.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
      integer stdout, stderr, stdin
      parameter( stdout = 6, stderr = 0, stdin = 5 )
c     INPUT VARIABLES
      real*8 wind_speed      ! estimated wind speed [m/s]
      real*8 wind_speed_err  ! estimated wind speed error [m/s]
      real*8 inc_angle       ! incidence angle [deg]
c     OUTPUT VARIABLES
      real*8 delta_TB(2)     ! order (V,H) [K]
      real*8 delta_TB_err(2) ! order (V,H) [K]
c     LOCAL VARIABLES
      real*8 dTv_dW
      real*8 dTh_dW
c     DATA STATEMENTS
c     COMMON BLOCKS
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      
      ! I got this model from paper draft of march 2009 highwinds campaign:
      ! "Passive and Active L-Band Microwave Observations and Modeling of Ocean Surface Winds"
      
      ! dTv/dW = 0.275 - 0.0024153 * theta_inc 
      !        + 0.00014026  * theta_inc**2     
      !        - 2.3326×10-6 * theta_inc**3
      
      ! dTh/dW = 0.275 + 0.003001  * theta_inc 
      !        - 2.5181 × 10**-6 * theta_inc**2        
      !        - 6.9763 × 10**-7 * theta_inc**3
      
      dTv_dW = 0.275 - 0.0024153 * inc_angle + 0.00014026    * inc_angle**2 - 2.3326*10**(-6) * inc_angle**3
      dTh_dW = 0.275 + 0.003001  * inc_angle - 2.5181*10**(-6) * inc_angle**2 - 6.9763*10**(-7) * inc_angle**3
      
      if( wind_speed .ge. 0.0d0 ) then
        delta_TB(1)     = dTv_dW * wind_speed
        delta_TB(2)     = dTh_dW * wind_speed
      
        delta_TB_err(1) = dTv_dW * wind_speed_err
        delta_TB_err(2) = dTh_dW * wind_speed_err
      else
        delta_TB(1)     = 0.0d0
        delta_TB(2)     = 0.0d0
      
        delta_TB_err(1) = 0.0d0
        delta_TB_err(2) = 0.0d0
      endif
      end
