C--$Header: /u/caretr2/scatsim/CVSROOT/L1A_to_L2/L2_gen_quality_flag.f,v 1.9 2012/01/23 21:03:34 fore Exp $
c*****************************************************************      
      subroutine L2_gen_quality_flag( L1B_quality_flag, 
     $                                L2_neg_sig0_ANT, 
     $                                L2_neg_sig0_TOA, 
     $                                L2_scat_rfi_flags, 
     $                                L2_good_far_corr, 
     $                                L2_scatterometer_flags,
     $                                L2_off_nom_no_echo,
     $                                L2_scat_samples ) 
c*****************************************************************      
c**   
c**   FILE NAME: L2_gen_quality_flag.f
c**   
c**   DATE WRITTEN: 3/25/2010
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION: Sets the L2 quality flag.  Uses
c**    the L1B flags as well as some logicals and the RFI flags
c**    from the main s/w.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES:
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1B.inc'
      include 'L2.inc'
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      integer*2 L1B_quality_flag(4,3,8)     ! L1B flags
      logical   L2_neg_sig0_ANT(4,3)        ! neg sig0 flag; ANT; (chan,beam)
      logical   L2_neg_sig0_TOA(4,3)        ! neg sig0 flag; TOA; (chan,beam)
      integer*1 L2_scat_rfi_flags(3,2)      ! beam, RX-polarization
      logical   L2_good_far_corr(3)         ! logical flag for goodness of faraday rotation correction
      logical   L2_off_nom_no_echo(3)       ! logical for pointing off-nominal without echo (beam)
      integer*4 L2_scat_samples(3)       
      
c     OUTPUT VARIABLES
      integer*4 L2_scatterometer_flags(3)
c     LOCAL VARIABLES
      integer i_chan
      integer i_beam
      integer i_seq
      
      integer num_rfi_set_H
      integer num_rfi_set_V
      integer num_rfi_set
      
      logical L1B_pointing_error
      logical L1B_beam_off_earth
      
c     DATA STATEMENTS
c     COMMON BLOCKS
c     FUNCTION STATEMENTS
      logical L1B_test_flag
      logical L2_test_flag
c     PROCESSING STEPS
      
      do i_beam = 1, 3
        L2_scatterometer_flags(i_beam) = 0

C--Check for RFI flags----------------------------------------------------
        num_rfi_set_H = 0
        num_rfi_set_V = 0
        num_rfi_set   = 0
        ! Compute # of RFI flags set at the sequence level in this block
        do i_seq = 1, 8
          ! check VV L1B quality flag for RFI
          if( .not.L1B_test_flag( L1B_quality_flag(2,i_beam,i_seq), 
     $      L1B_QF_RFI, L1B_QF_RFI_none ) )
     $      num_rfi_set_V = num_rfi_set_V + 1
     
          ! check HH L1B quality flag for RFI
          if( .not.L1B_test_flag( L1B_quality_flag(4,i_beam,i_seq), 
     $      L1B_QF_RFI, L1B_QF_RFI_none ) )
     $      num_rfi_set_H = num_rfi_set_H + 1
          
          ! is either is set, add one to num_rfi_set (ugly)
          ! since we use all 8 sequences if all of them have either HRX OR VRX flagged,
          ! must catch this and report the L2 flag correctly. AGF 11/21/2011
          if( 
     $ (.not.L1B_test_flag(L1B_quality_flag(4,i_beam,i_seq),L1B_QF_RFI,L1B_QF_RFI_none)) .or. 
     $ (.not.L1B_test_flag(L1B_quality_flag(2,i_beam,i_seq),L1B_QF_RFI,L1B_QF_RFI_none)) )
     $      num_rfi_set = num_rfi_set + 1
          
        enddo
        
        if( num_rfi_set .eq. 8 ) then 
          call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_RFI_V, L2_QF_RFI_V_ALL_SEQUENCES )
        elseif( num_rfi_set_V .ge. L2_rfi_V_severe_threshold ) then
          call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_RFI_V, L2_QF_RFI_V_SEVERE )
        elseif( num_rfi_set_V .ge. L2_rfi_V_moderate_threshold ) then
          call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_RFI_V, L2_QF_RFI_V_MODERATE )
        endif

        ! looks redundant, but it really is not!
        if( num_rfi_set .eq. 8 ) then
          call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_RFI_H, L2_QF_RFI_H_ALL_SEQUENCES )
        elseif( num_rfi_set_H .ge. L2_rfi_H_severe_threshold ) then
          call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_RFI_H, L2_QF_RFI_H_SEVERE )
        elseif( num_rfi_set_H .ge. L2_rfi_H_moderate_threshold ) then
          call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_RFI_H, L2_QF_RFI_H_MODERATE )
        endif
C--End check for RFI flagging---------------------------------------------


C------Rain flagging section-----------------------------------------------
C--Leave rain flagging empty for now; will need some kind of ancillary data
C--- 3/25/2010
C------End of rain flaggging section--------------------------------------


C--Check for neg sig0 TOA for all 4 channels------------------------------
        if( L2_neg_sig0_TOA(1,i_beam) )
     $    call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_NEG_SIG0_TOA_HV, L2_QF_NEG_SIG0_TOA_HV )
     
        if( L2_neg_sig0_TOA(2,i_beam) )
     $    call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_NEG_SIG0_TOA_VV, L2_QF_NEG_SIG0_TOA_VV )

        if( L2_neg_sig0_TOA(3,i_beam) )
     $    call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_NEG_SIG0_TOA_VH, L2_QF_NEG_SIG0_TOA_VH )

        if( L2_neg_sig0_TOA(4,i_beam) )
     $    call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_NEG_SIG0_TOA_HH, L2_QF_NEG_SIG0_TOA_HH )
C--Done with neg sig0 TOA flag--------------------------------------------

C--Check for attitude flagging--------------------------------------------
        L1B_pointing_error = .false.
        L1B_beam_off_earth = .false.
        do i_seq = 1, 8
          do i_chan = 1, 4
          ! test to see if any of the seq * chan pointing flags 
          ! or off-earth flags are set at L1B level.
            if( L1B_test_flag( L1B_quality_flag(i_chan,i_beam,i_seq),
     $          L1B_QF_POINTING_ERROR, L1B_QF_POINTING_ERROR ) )
     $        L1B_pointing_error = .true.
            if( L1B_test_flag( L1B_quality_flag(i_chan,i_beam,i_seq),
     $          L1B_QF_BEAM_OFF_EARTH, L1B_QF_BEAM_OFF_EARTH ) )
     $        L1B_beam_off_earth = .true.
          enddo
        enddo
        
        ! 1st test for off-earth state
        ! 2nd for off-nom w/out echo state
        ! 3rd for off-nom with echo state
        if( L1B_beam_off_earth ) then
          call L2_set_flag( L2_scatterometer_flags(i_beam),
     $                      L2_QF_ATT, L2_QF_ATT_OFF_EARTH )
        elseif( L2_off_nom_no_echo(i_beam) ) then
          call L2_set_flag( L2_scatterometer_flags(i_beam),
     $                      L2_QF_ATT, L2_QF_ATT_OFF_NOM_NO_ECHO )
        elseif( L1B_pointing_error ) then
          call L2_set_flag( L2_scatterometer_flags(i_beam),
     $                      L2_QF_ATT, L2_QF_ATT_OFF_NOM_WITH_ECHO )
        endif
        
C--Done with check for attitude-------------------------------------------


C--Check for good faraday rotation correction-----------------------------     
        if( .not.L2_good_far_corr( i_beam ) ) then
          call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_BAD_FAR_CORR_HV, L2_QF_BAD_FAR_CORR_HV )
          call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_BAD_FAR_CORR_VV, L2_QF_BAD_FAR_CORR_VV )
          call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_BAD_FAR_CORR_VH, L2_QF_BAD_FAR_CORR_VH )
          call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_BAD_FAR_CORR_HH, L2_QF_BAD_FAR_CORR_HH )
        endif
C--Done with faraday rotation correction flag-----------------------------


C--Check for neg sig0 ANT for all 4 channels------------------------------
        if( L2_neg_sig0_ANT(1,i_beam) )
     $    call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_NEG_SIG0_ANT_HV, L2_QF_NEG_SIG0_ANT_HV )
     
        if( L2_neg_sig0_ANT(2,i_beam) )
     $    call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_NEG_SIG0_ANT_VV, L2_QF_NEG_SIG0_ANT_VV )

        if( L2_neg_sig0_ANT(3,i_beam) )
     $    call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_NEG_SIG0_ANT_VH, L2_QF_NEG_SIG0_ANT_VH )

        if( L2_neg_sig0_ANT(4,i_beam) )
     $    call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_NEG_SIG0_ANT_HH, L2_QF_NEG_SIG0_ANT_HH )
C--Done with neg sig0 ANT flag--------------------------------------------
        
C--Set overall quality flag-----------------------------------------------
        if( L2_scat_samples(i_beam).eq.0                                                     .or.
     $    L2_test_flag(L2_scatterometer_flags(i_beam),L2_QF_RFI_V,L2_QF_RFI_V_SEVERE       ) .or.
     $    L2_test_flag(L2_scatterometer_flags(i_beam),L2_QF_RFI_V,L2_QF_RFI_V_ALL_SEQUENCES) .or.
     $    L2_test_flag(L2_scatterometer_flags(i_beam),L2_QF_RFI_H,L2_QF_RFI_H_SEVERE       ) .or.
     $    L2_test_flag(L2_scatterometer_flags(i_beam),L2_QF_RFI_H,L2_QF_RFI_H_ALL_SEQUENCES) .or.
     $    L2_test_flag(L2_scatterometer_flags(i_beam),L2_QF_ATT,L2_QF_ATT_OFF_NOM_WITH_ECHO) .or.
     $    L2_test_flag(L2_scatterometer_flags(i_beam),L2_QF_ATT,L2_QF_ATT_OFF_NOM_NO_ECHO  ) .or.
     $    L2_test_flag(L2_scatterometer_flags(i_beam),L2_QF_ATT,L2_QF_ATT_OFF_EARTH        ) ) then
          
          call L2_set_flag( L2_scatterometer_flags(i_beam), 
     $                      L2_QF_OVERALL_QUALITY, L2_QF_OVERALL_QUALITY )
          
        endif
C--Done with overall quality flag-----------------------------------------

      enddo
      
      
      end
