C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L2_get_wind_ancilliary.f,v 1.4 2009/10/05 14:30:08 fore Exp $
c*****************************************************************      
      subroutine L2_get_wind_ancilliary( julian_time,
     $  lat, lon, wind_dir_ancilliary, wind_speed_ancilliary )
c*****************************************************************      
c**   
c**   FILE NAME: L2_get_wind_ancilliary.f
c**   
c**   DATE WRITTEN: 8/12/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: L1A_load_QMET_winds_from_HDF,
c**     r4_bilinear_interp, r4_lininterp1d
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**   05/31/11           Add non-optimal processing support  
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 julian_time
      real*8 lat
      real*8 lon
 
c     OUTPUT VARIABLES
      real*8 wind_dir_ancilliary
      real*8 wind_speed_ancilliary
c     LOCAL VARIABLES
      character*320 QMET_file_1 ! /path/to/QMET_file_1
      character*320 QMET_file_2 ! /path/to/QMET_file_2

      character*320 QMET_file_1_last /'not_a_file'/
      character*320 QMET_file_2_last /'not_a_file'/
      
      character*320 QMET_file_nonopt
      logical       file_exists1, file_exists2, file_exists
      real*8        jd_tmp

      character*2   QMET_hour_str_1
      character*2   QMET_hour_str_2
      
      integer i_year, i_month, i_day, i_hour, i_minute, i_second
      integer i_day_of_year
      
      integer i_year_2
      integer i_day_of_year_2
      
      
      integer i_lat_1, i_lat_2
      integer i_lon_1, i_lon_2
      
      integer is_leap_year
      integer counter

      real*8 r_frac
      
      real*4 u_wind_1(0:359,-90:90), v_wind_1(0:359,-90:90)
      real*4 u_wind_2(0:359,-90:90), v_wind_2(0:359,-90:90)
      
      real*4 u_1, u_2
      real*4 v_1, v_2
      real*4 r4_lon, r4_lat
      real*4 u_wind, v_wind
      
      real*4 t_1, t_2, t_now
      real*8 PI
      
c     DATA STATEMENTS
      data counter/1/

c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level

      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT,
     $ AQ_SCAT_SIM_DIR_SCRATCH

c     FUNCTION STATEMENTS
      integer lnblnk
c     PROCESSING STEPS
      save QMET_file_1_last, QMET_file_2_last
      save u_wind_1, v_wind_1, u_wind_2, v_wind_2
      save counter

      PI = 4.0d0 * atan(1.0d0)
      
C--Construct the two filenames of the QMET files which bound the julian_time      
C--Convert the julian_time to YYYY MM DD ..etc integers.
      call JD2CAL(julian_time,i_year,i_month,i_day,i_hour,i_minute,
     $ i_second,r_frac)
      
      call convert_day_and_month_to_dayofyear(i_year, i_day, 
     $ i_month, i_day_of_year )
      
      i_year_2        = i_year
      i_day_of_year_2 = i_day_of_year
      
      t_now = real(i_hour) + real(i_minute)/60.0
      
      if( i_hour .lt. 6)then
        QMET_hour_str_1 = '00'
        QMET_hour_str_2 = '06'
        t_1 = 0.0
        t_2 = 6.0
      elseif( i_hour .lt. 12)then
        QMET_hour_str_1 = '06'
        QMET_hour_str_2 = '12'      
        t_1 = 6.0
        t_2 = 12.0
      elseif( i_hour .lt. 18)then
        QMET_hour_str_1 = '12'
        QMET_hour_str_2 = '18'      
        t_1 = 12.0
        t_2 = 18.0
      elseif( i_hour .lt. 24)then
        QMET_hour_str_1 = '18' 
        QMET_hour_str_2 = '00'
        t_1 = 18.0
        t_2 = 24.0

        if( mod(i_year,400).eq.0 ) then
           is_leap_year = 1
        elseif( mod(i_year,100).eq.0) then
           is_leap_year = 0
        elseif( mod(i_year,4).eq.0) then
           is_leap_year = 1
        else
           is_leap_year = 0
        endif
        

        if( ( i_day_of_year .eq. 365 .and. is_leap_year .eq. 0 ) .or.
     $       ( i_day_of_year .eq. 366 .and. is_leap_year .eq. 1 ) ) then
          
           i_day_of_year_2 = 1
           i_year_2        = i_year + 1
        else
           i_day_of_year_2 = i_day_of_year + 1
           i_year_2        = i_year
        endif     
      endif
     
      write(QMET_file_1,'((a),I4.4,I3.3,(a),(a),(a))')
     $  './N',
     $  i_year,
     $  i_day_of_year,
     $  QMET_hour_str_1,
     $  '_QMET_NCEP_6h'
      
      write(QMET_file_2,'((a),I4.4,I3.3,(a),(a),(a))')
     $  './N',
     $  i_year_2,
     $  i_day_of_year_2,
     $  QMET_hour_str_2,
     $  '_QMET_NCEP_6h'
      
      if( i_dbg_level .ge. 2)then
        write(6,*) QMET_file_1(1:lnblnk(QMET_file_1))
        write(6,*) QMET_file_2(1:lnblnk(QMET_file_2))      
      endif     
C--Finished generating QMET filenames...

C--Load in the data
      if( QMET_file_1 .ne. QMET_file_1_last .or.
     $    QMET_file_2 .ne. QMET_file_2_last )then

         inquire( file=QMET_file_1, exist=file_exists1 )
         inquire( file=QMET_file_2, exist=file_exists2 )

         if ( .not.file_exists1 .or. .not.file_exists2) then
            jd_tmp          = julian_time
            file_exists     = .false.

            do while( .not.file_exists)  
               call JD2CAL(jd_tmp,i_year,i_month,i_day,i_hour,i_minute,i_second,r_frac)
               call convert_day_and_month_to_dayofyear(i_year, i_day, i_month, i_day_of_year )
        
               if( i_hour .lt. 6 ) then
                  QMET_hour_str_1 = '00'
               elseif( i_hour .lt. 12)then
                  QMET_hour_str_1 = '06'
               elseif( i_hour .lt. 18)then
                  QMET_hour_str_1 = '12'
               elseif( i_hour .lt. 24)then
                  QMET_hour_str_1 = '18' 
               endif
    
               write(QMET_file_nonopt,'((a),I4.4,I3.3,(a),(a),(a))')
     $              './N',
     $              i_year,
     $              i_day_of_year,
     $              QMET_hour_str_1,
     $              '_QMET_NCEP_6h'      

               QMET_file_nonopt =  QMET_file_nonopt(1:lnblnk(QMET_file_nonopt))
        
               inquire( file=QMET_file_nonopt, exist=file_exists )
        
               jd_tmp = jd_tmp - 0.25d0 ! Check previous 6h interval        
            enddo

            QMET_file_1_last = QMET_file_1
            QMET_file_2_last = QMET_file_2
            QMET_file_1 = QMET_file_nonopt
            QMET_file_2 = QMET_file_nonopt
         endif
!        End nonoptimal processing!

C--Need to load in new data
        write(6,*) 'Loading QMET winds from: ',QMET_file_1(1:lnblnk(QMET_file_1))
        if (counter .eq. 1)
     $       write(35,'((a),(a))') 
     $       'QMETFILE1=', QMET_file_1(scan(QMET_file_1, '/', back=.true.)+1:lnblnk(QMET_file_1))
        call L1A_load_QMET_winds_from_HDF( QMET_file_1, u_wind_1, v_wind_1 )
        write(6,*) 'Loading QMET winds from: ',QMET_file_2(1:lnblnk(QMET_file_2))
        if (counter .eq. 1)
     $       write(35,'((a),(a))') 
     $       'QMETFILE2=', QMET_file_2(scan(QMET_file_2, '/', back=.true.)+1:lnblnk(QMET_file_2))
        if (counter .eq. 2)
     $       write(35,'((a),(a))') 
     $       'QMETFILE3=', QMET_file_2(scan(QMET_file_2, '/', back=.true.)+1:lnblnk(QMET_file_2))
        call L1A_load_QMET_winds_from_HDF( QMET_file_2, u_wind_2, v_wind_2 )     
        
        !write(6,*) ' '
        !write(6,*) 'julian_time: ', julian_time
        !write(6,*) i_year, i_month, i_day, i_day_of_year, i_hour
        !write(6,*) ' '

        counter = counter + 1
      endif
!     End data load


C--Compute indices to use for spatial interpolation of the wind field.
C--I assume [-90:1:90] x [0:1:359] for lat,lon index -> value conversion
C--Since there is not any (lat,lon) specified in QMET file I can't be
C--sure this is right!
      r4_lon = lon
      r4_lat = lat

      if( r4_lon < 0 ) r4_lon = r4_lon + 360.0 ! wrap to interval 0,360

C--I want the floor operation but g77 doesn't implement it!
C--int() intrinsic removes fractional part and preserves sign.  Thus it rounds
C--down positive reals and rounds up negative reals to integers.
      i_lat_1 = int(r4_lat) 
      if( i_lat_1 .gt. r4_lat ) i_lat_1 = i_lat_1 - 1
      
C--Keep i_lat_1 contained in [-90:89]
      if( i_lat_1 .ge. 89.0  ) i_lat_1 = 89
      if( i_lat_1 .lt. -90.0 ) i_lat_1 = -90
      i_lat_2 = i_lat_1 + 1
      
      i_lon_1 = int(r4_lon) ! Again I want the floor operation, but lon always > 0
C--Keep i_lon_1 contained in [0:358]      
      if( i_lon_1 .ge. 358.0 ) i_lon_1 = 358
      if( i_lon_1 .lt. 0.0   ) i_lon_1 = 0
      i_lon_2 = i_lon_1 + 1
      
C--Bilinear interpolation of (u_1,v_1) @ time t_1 and (u_2,v_2) @ time t_2
C--at coordinate (lat,lon).
      call r4_bilinear_interp( 
     $ real(i_lon_1), real(i_lon_2), ! x1, x2
     $ real(i_lat_1), real(i_lat_2), ! y1, y2
     $ u_wind_1(i_lon_1,i_lat_1),    ! f(x1,y1)
     $ u_wind_1(i_lon_1,i_lat_2),    ! f(x1,y2)
     $ u_wind_1(i_lon_2,i_lat_1),    ! f(x2,y1)
     $ u_wind_1(i_lon_2,i_lat_2),    ! f(x2,y2)
     $ r4_lon, r4_lat, u_1 )         ! x, y, f(x,y)
     
      call r4_bilinear_interp(
     $ real(i_lon_1), real(i_lon_2), 
     $ real(i_lat_1), real(i_lat_2), 
     $ v_wind_1(i_lon_1,i_lat_1),
     $ v_wind_1(i_lon_1,i_lat_2),
     $ v_wind_1(i_lon_2,i_lat_1),
     $ v_wind_1(i_lon_2,i_lat_2), 
     $ r4_lon, r4_lat, v_1 )     

      call r4_bilinear_interp(
     $ real(i_lon_1), real(i_lon_2), 
     $ real(i_lat_1), real(i_lat_2), 
     $ u_wind_2(i_lon_1,i_lat_1),
     $ u_wind_2(i_lon_1,i_lat_2),
     $ u_wind_2(i_lon_2,i_lat_1),
     $ u_wind_2(i_lon_2,i_lat_2), 
     $ r4_lon, r4_lat, u_2 )
     
      call r4_bilinear_interp(
     $ real(i_lon_1), real(i_lon_2), 
     $ real(i_lat_1), real(i_lat_2), 
     $ v_wind_2(i_lon_1,i_lat_1),
     $ v_wind_2(i_lon_1,i_lat_2),
     $ v_wind_2(i_lon_2,i_lat_1),
     $ v_wind_2(i_lon_2,i_lat_2), 
     $ r4_lon, r4_lat, v_2 )
      
C--Temporal interpolation of (u_1,u_2) and (v_1,v_2) to (u,v) at julian_time.
      call r4_lininterp1d( t_1, t_2, u_1, u_2, t_now, u_wind ) ! x1,x2,f(x1),f(x2),x,f(x)
      call r4_lininterp1d( t_1, t_2, v_1, v_2, t_now, v_wind )
      
C--Wind direction.
      wind_dir_ancilliary   = atan2( u_wind, v_wind ) * 180.0d0 / PI
      wind_speed_ancilliary = sqrt( u_wind**2 + v_wind**2 )
      
      if( i_dbg_level .ge. 4)then
            write(6,'((a),3f12.5)') 't_1,     t_2,   t_now: ',t_1, t_2, t_now
        write(6,'((a),2I5,f12.5)') 'i_lat_1, i_lat_2, lat: ',i_lat_1, i_lat_2, lat
        write(6,'((a),2I5,f12.5)') 'i_lon_1, i_lon_2, lon: ',i_lon_1, i_lon_2, lon
        write(6,*) 'u_wind: ', u_wind,' v_wind: ',v_wind,' wind_dir: ',wind_dir_ancilliary
      endif

C--Save the file names so we know what is already loaded into
C--memory.
      if ( file_exists1 .and. file_exists2) then
         QMET_file_1_last = QMET_file_1
         QMET_file_2_last = QMET_file_2
      endif

      end

      subroutine L2_get_wind_ancilliary_DEBUG( julian_time )
      implicit none
      
      integer i_lon, i_lat
      real*8  julian_time
      real*8  lon, lat
      real*8  wind_dir, wind_spd
      real*8  u, v
      real*8 PI
      integer i_dbg_level
      common /debug/ i_dbg_level
      
      PI = 4.0d0 * atan( 1.0d0 )
      
      do i_lon = 0,359
        do i_lat = -720,720
          
          lon = real(i_lon)
          lat = real(i_lat) * 0.125
          
          call L2_get_wind_ancilliary( julian_time, lat, lon, wind_dir, wind_spd )
          
          v = wind_spd * cos( wind_dir * PI / 180.0d0 )
          u = wind_spd * sin( wind_dir * PI / 180.0d0 )
          
          write(70,'(6f12.6)') lon, lat, u, v, wind_dir, wind_spd
          
        enddo
      enddo
      write(6,*) 'L2_get_wind_ancilliary_DEBUG: Written winds out to fort.70...'
      end

