C--$Header: /u/caretr2/scatsim/CVSROOT/L1A_to_L2/L2_load_RFI_flags.f,v 1.5 2011/10/17 12:38:43 fore Exp $
c*****************************************************************      
      subroutine L2_load_RFI_flags( RFI_FLAGS_VPOL, 
     $                              RFI_FLAGS_HPOL,
     $                              L2_scat_rfi_flags )
c*****************************************************************      
c**   
c**   FILE NAME: L2_load_RFI_flags.f
c**   
c**   DATE WRITTEN: 10/16/2009
c**   
c**   PROGRAMMER:
c**   
c**   FUNCTIONAL DESCRIPTION: Copies the RFI flags into the L2
c**     scat_rfi flags array for L2 output.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
      integer stdout, stderr, stdin
      parameter( stdout = 6, stderr = 0, stdin = 5 )
c     INPUT VARIABLES
      integer*1 RFI_FLAGS_VPOL(3,8) ! set to 1 if RFI, 0 if no RFI
      integer*1 RFI_FLAGS_HPOL(3,8) ! set to 1 if RFI, 0 if no RFI
      
c     OUTPUT VARIABLES
      integer*1 L2_scat_rfi_flags(3,2) ! beam, RX-polarization (order 1==V,2==H)

c     LOCAL VARIABLES
      integer i_beam
      integer i_seq
c     DATA STATEMENTS
c     COMMON BLOCKS
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      
      do i_beam = 1, 3
        L2_scat_rfi_flags(i_beam,1) = 0 ! all bits unset
        L2_scat_rfi_flags(i_beam,2) = 0 ! all bits unset

        ! Bit 0 (right-most, or least-significant bit) is for sequence 1
        ! Bit 1 is for sequence 2
        ! Bit 2 is for sequence 3 ...etc
        do i_seq = 1, 8
        
          if( RFI_FLAGS_VPOL(i_beam,i_seq) .ge. 1 ) then
            L2_scat_rfi_flags(i_beam,1) = ibset( L2_scat_rfi_flags(i_beam,1), i_seq-1 )
          else
            L2_scat_rfi_flags(i_beam,1) = ibclr( L2_scat_rfi_flags(i_beam,1), i_seq-1 )
          endif
          
          if( RFI_FLAGS_HPOL(i_beam,i_seq) .ge. 1 ) then
            L2_scat_rfi_flags(i_beam,2) = ibset( L2_scat_rfi_flags(i_beam,2), i_seq-1 )
          else
            L2_scat_rfi_flags(i_beam,2) = ibclr( L2_scat_rfi_flags(i_beam,2), i_seq-1 )
          endif
          
        enddo
      enddo

      !write(6,*) 'B1 RFI_FLAGS_VPOL:   ', (RFI_FLAGS_VPOL(1,i_seq),i_seq=1,8)
      !write(6,*) 'B2 RFI_FLAGS_VPOL:   ', (RFI_FLAGS_VPOL(2,i_seq),i_seq=1,8)
      !write(6,*) 'B3 RFI_FLAGS_VPOL:   ', (RFI_FLAGS_VPOL(3,i_seq),i_seq=1,8)
      !write(6,*) 'B1 RFI_FLAGS_HPOL:   ', (RFI_FLAGS_HPOL(1,i_seq),i_seq=1,8)
      !write(6,*) 'B2 RFI_FLAGS_HPOL:   ', (RFI_FLAGS_HPOL(2,i_seq),i_seq=1,8)
      !write(6,*) 'B3 RFI_FLAGS_HPOL:   ', (RFI_FLAGS_HPOL(3,i_seq),i_seq=1,8)
      !write(6,*) 'L2_scat_rfi_flags V: ', (L2_scat_rfi_flags(i_beam,1),i_beam=1,3)
      !write(6,*) 'L2_scat_rfi_flags H: ', (L2_scat_rfi_flags(i_beam,2),i_beam=1,3)
      
      end
