C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/L2_retrive_wind_speed.f,v 1.7 2009/10/20 14:29:34 fore Exp $
c***************************************************************** 
      subroutine L2_retrive_wind_speed( incang, 
     $                                  relazang,
     $                                  sigwave,
     $                                  measure_sig0,
     $                                  i_beam,
     $                                  windspd  )
c*****************************************************************      
c**   
c**   FILE NAME:  L2_retrive_wind_speed.f
c**   
c**   DATE WRITTEN:  5/11/2009
c**   
c**   PROGRAMMER:  Julian Chaubell
c**   
c**   FUNCTIONAL DESCRIPTION:  Retrive the wind speed from the input 
c**                            sigma_0 = sigma_vv + sigma_hh + sigma_vh + sigma_hv
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c***************************************************************** 
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 incang     ! incidence angle in degrees
      real*8 relazang   ! relative azimuth angle (cell azimuth - wind direction) in degrees
      real*8 measure_sig0       ! linear_to_db( sum( db_to_linear( sig0_ipol ) ) ) 
                        !  summed over ipol = { HH, HV, VH, VV } polarizations
      integer sigwave   ! significant wave height flag
      integer i_beam    ! which beam is this? (1,2,3)
c     OUTPUT VARIABLES
      real*8 windspd    ! wind speed in m/s
      
c     LOCAL VARIABLES
      real*8 sig0
      real*8 initial_wind
      real*8 locazang

      real*8 r_sig0_plus(9)
      real*8 r_sig0_minus(9)
      real*8 r_sig0(9)

      real*8 linear_sig0          ! input sig0 in linear units
      real*8 linear_r_sig0        ! trial sig0 in linear units
      real*8 linear_r_sig0_plus   ! sig0 of windspd + h
      real*8 linear_r_sig0_minus  ! sig0 of windspd - h

      real*8 dsig0
      real*8 corr   ! incremental corretion to wind speed
      real*8 h      ! wind retrival step size
      real*8 eps    ! quit when corr less than this value

      integer i_iter
      
      real*8 regress_coeff1(3), regress_coeff2(3)

c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug_level/ i_dbg_level
      
c     FUNCTION STATEMENTS
c     PROCESSING STEPS

C--Supposed to be a linear regression of the beam-integrated sigma0
C--to the beam center sigma0; Due to range^4, the beam integrated
C--Sigma0 is "weighted" more towards smaller incidence angles
C--than the inc angle at beam center.
      regress_coeff1(1) = -0.152857684747261
      regress_coeff1(2) =  0.109803819564509
      regress_coeff1(3) =  0.224250879486013
      regress_coeff2(1) =  1.0347867745314
      regress_coeff2(2) =  1.02655999456674
      regress_coeff2(3) =  1.02933330900326
      
C--Sig0 would be the "beam-center" sigma0;       
      sig0 = regress_coeff1(i_beam) + 10.0*log10(measure_sig0)*regress_coeff2(i_beam)

      initial_wind = 6                ! initial guess
      h            = 0.0001
      eps          = 0.00001
      corr         = 100000.          ! big number so that abs(corr) > eps 1st time
      windspd      = initial_wind
      linear_sig0  = 10.**(sig0/10.)
           
      if( sigwave.ge.2.and.sigwave.le.5 .and. 
     $    relazang .ge.0  .and. relazang.le.360 .and. 
     $    incang   .ge.10 .and. incang  .le.60 ) then
        i_iter = 0 
        
        do while( abs(corr) .gt. eps  .and. i_iter  .lt. 20   .and. 
     $             windspd  .ge. 1.0  .and. windspd .le. 50.0 )
     
          call model_function(incang,windspd,relazang,sigwave,r_sig0)

          linear_r_sig0 = 10.**(r_sig0(1)/10.) + 2.* 10.**(r_sig0(2)/10.)
     $                  + 10.**(r_sig0(3)/10.)
          
          call model_function(incang,windspd + h,relazang,sigwave,r_sig0)
          
          linear_r_sig0_plus = 10.**(r_sig0(1)/10.) + 2.* 10.**(r_sig0(2)/10.) 
     $                       + 10.**(r_sig0(3)/10.)
          
          call model_function(incang,windspd - h,relazang,sigwave,r_sig0)
          
          linear_r_sig0_minus = 10.**(r_sig0(1)/10.) + 2.* 10.**(r_sig0(2)/10.) 
     $                        + 10.**(r_sig0(3)/10.)
          
          dsig0   = (linear_r_sig0_plus - linear_r_sig0_minus) / ( 2.0d0 * h )
          corr    = (linear_sig0 - linear_r_sig0) / dsig0
          windspd = windspd + corr
          i_iter  = i_iter + 1
        end do
        if( i_dbg_level .ge. 2) write(6,*) "Retrieved Wind: ",windspd
      else
        write(6,*) "L2_retrive_wind_speed: WARNING: Input values out of model boundaries."
            windspd = -999.
            return
      endif

      if( windspd.lt.1.0 .or. windspd.gt.50.0 ) then
            windspd = -999.
!        write(6,*) "L2_retrive_wind_speed: WARNING: not converging inside the model."
        return
      endif
      
      return
      end
