C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/READ_SCI_load_all.f,v 1.5 2009/10/20 14:20:26 fore Exp $
c*****************************************************************      
      subroutine READ_SCI_load_all( read_sci_prefix,
     $                              RX_power, 
     $                              LB_power,
     $                              DC_offset,
     $                              L0_scat_headers,              
     $                              external_temperatures_eu,
     $                              scat_telemetry_eu,
     $                              icds_engineering_telemetry_eu,
     $                              icds_processing_status_eu,
     $                              RFI_FLAGS_VPOL,
     $                              RFI_FLAGS_HPOL )
c*****************************************************************      
c**   
c**   FILE NAME: READ_SCI_load_all.f
c**   
c**   DATE WRITTEN: 9/18/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**    --Doesn't load all of the ICDS ENG telemetry since not all are written out in read_sci_v76
c**    --Only loads some of the L0_scat_headers ( pwr_accum overflow, i_tx_pol, i_rx_pol )
c**    --Doesn't load any of the icds_processing_status_eu right now.

c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1A.inc'
      include 'L0.inc'
c     PARAMETER STATEMENTS
      integer stdout, stderr, stdin
      parameter( stdout = 6, stderr = 0, stdin = 5 )
c     INPUT VARIABLES
      character*(*)    read_sci_prefix           ! path+prefix of read_sci output files
c     OUTPUT VARIABLES

      real*8    RX_power( 6, 3, 8, MAX_NUM_BLOCKS ) ! pol, beam, seq, block
      real*8    LB_power( 6, 3,    MAX_NUM_BLOCKS ) ! pol, beam,      block
      real*8    DC_offset(      2, MAX_NUM_BLOCKS ) ! pol,            block
      integer*1 L0_scat_headers(8, MAX_NUM_BLOCKS ) !            seq, block
      
      real*4    external_temperatures_eu(      38, MAX_NUM_BLOCKS )
      real*4    scat_telemetry_eu(             38, MAX_NUM_BLOCKS )
      real*4    icds_engineering_telemetry_eu( 24, MAX_NUM_BLOCKS )
      integer*4 icds_processing_status_eu(      4, MAX_NUM_BLOCKS )

      integer*1 RFI_FLAGS_VPOL(3,8,MAX_NUM_BLOCKS) ! set to 1 if RFI, 0 if no RFI
      integer*1 RFI_FLAGS_HPOL(3,8,MAX_NUM_BLOCKS) ! set to 1 if RFI, 0 if no RFI

c     LOCAL VARIABLES
      real*8 tmp_buffer(18)
      character*320    filename
      character*23     time_str
      integer ii
      integer i_seq
      integer i_block
      integer i_beam
      integer i_chan
      integer i_len
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level
c     FUNCTION STATEMENTS
      integer lnblnk
c     PROCESSING STEPS

C--Load RX_power arrays
      i_len = lnblnk(read_sci_prefix)
      filename = read_sci_prefix(1:i_len) // '.scat_Beams_j'
      
      write(stdout,*) 'Reading RX_power from ',filename(1:lnblnk(filename))
      
      open(25,file=filename,status='old')
      do i_block = 1, MAX_NUM_BLOCKS
        do i_seq = 1, 8
          read(25,*,END=1001) time_str, (tmp_buffer(ii),ii=1,18)
          
          do i_beam=1,3
            do i_chan = 1, 6
              RX_power(i_chan,i_beam,i_seq,i_block) = tmp_buffer(i_chan+(i_beam-1)*6)
            enddo ! i_chan
          enddo ! i_beam
        enddo ! i_seq
      enddo ! i_bock
1001  continue
      if( .not.( i_seq.eq.8 .and. i_block.eq.MAX_NUM_BLOCKS 
     $          .or. i_seq.eq.1) ) then ! # of lines of text file should be 
                                        ! divisible by 8...
        write(stderr,*) 'READ_SCI_load_all: ERROR: incomplete block!'
        call exit(1)
      endif
      close(25)
      
      if( i_dbg_level.ge.2 ) then
        write(6,*) 'Read ',i_block,' blocks of scat echo powers.'
      endif

C--Load LB_power arrays
      filename = read_sci_prefix(1:i_len) // '.scat_lbc_j'
      open(25,file=filename,status='old')
      write(stdout,*) 'Reading LB_power from ',filename(1:lnblnk(filename))

      filename = read_sci_prefix(1:i_len) // '.scat_lbc_noise_j'
      open(26,file=filename,status='old')
      write(stdout,*) 'Reading LB_noise_power from ',filename(1:lnblnk(filename))

      do i_block = 1, MAX_NUM_BLOCKS
        read(25,*,END=1002) time_str, (tmp_buffer(ii),ii=1,12)
        do i_beam = 1, 3
          LB_Power(1,i_beam,i_block) = tmp_buffer(1+(i_beam-1)*4) ! HV
          LB_Power(3,i_beam,i_block) = tmp_buffer(2+(i_beam-1)*4) ! VV
          LB_Power(4,i_beam,i_block) = tmp_buffer(3+(i_beam-1)*4) ! VH
          LB_Power(6,i_beam,i_block) = tmp_buffer(4+(i_beam-1)*4) ! HH
        enddo
        
        !write(6,*) i_block, time_str, (tmp_buffer(ii),ii=1,12)
        
        read(26,*,END=1002) time_str, (tmp_buffer(ii),ii=1,6)
        do i_beam = 1, 3
          LB_Power(2,i_beam,i_block) = tmp_buffer(1+(i_beam-1)*2) ! nV
          LB_Power(5,i_beam,i_block) = tmp_buffer(2+(i_beam-1)*2) ! nH          
        enddo
        
        !write(6,*) i_block, time_str, (tmp_buffer(ii),ii=1,6)
        
      enddo
1002  continue
      close(25)
      close(26)

      if( i_dbg_level.ge.2) then
        write(6,*) 'Read ',i_block,' blocks of scat Loopback powers.'
      endif

C--Load DC offset arrays
      filename = read_sci_prefix(1:i_len) // '.scat_dc_offset_j'
      open(25,file=filename,status='old')
      do i_block = 1, MAX_NUM_BLOCKS
        read(25,*,END=1003) time_str, (tmp_buffer(ii),ii=1,2)
        
        DC_offset(1,i_block) = tmp_buffer(1)
        DC_offset(2,i_block) = tmp_buffer(2)
      enddo
1003  continue
      close(25)

      if( i_dbg_level.ge.2) then
        write(6,*) 'Read ',i_block,' blocks of scat DC offsets.'
      endif
      
C--Load external temperature telemetry
      filename = read_sci_prefix(1:i_len) // '.ext_temp_omt_eu_j' ! contains external_temperatures_eu(1:6,:)
      open(25,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.ext_temp_diplexer_cables_eu_j' ! contains external_temperatures_eu(7:12,:)
      open(26,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.ext_temp_diplexers_eu_j' ! contains external_temperatures_eu(13:18,:)
      open(27,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.ext_temp_couplers_eu_j' ! contains external_temperatures_eu(19:24,:)
      open(28,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.ext_temp_cnd_feed_eu_j' ! contains external_temperatures_eu(25:27),(36:38,:)
      open(29,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.ext_temp_reflector_eu_j' ! contains external_temperatures_eu(28:35,:)
      open(30,file=filename,status='old')
      
      do i_block = 1, MAX_NUM_BLOCKS
      
        read(25,*,END=1004) time_str, (tmp_buffer(ii),ii=1,6) ! OMT probe temp (1H,1V,2H,2V,3H,3V)
        do ii = 1, 6
          external_temperatures_eu(ii,i_block) = tmp_buffer(ii)
        enddo

        read(26,*,END=1004) time_str, (tmp_buffer(ii),ii=1,6) ! diplexer to SFE coax (H1,H2,H3,V1,V2,V3)
        do ii = 1, 6
          external_temperatures_eu(ii+6,i_block) = tmp_buffer(ii)
        enddo
        
        read(27,*,END=1004) time_str, (tmp_buffer(ii),ii=1,6) ! diplexer (1H,1V,2H,2V,3H,3V)
        do ii = 1, 6
          external_temperatures_eu(ii+12,i_block) = tmp_buffer(ii)
        enddo

        read(28,*,END=1004) time_str, (tmp_buffer(ii),ii=1,6) ! coupler (1H,1V,2H,2V,3H,3V)
        do ii = 1, 6
          external_temperatures_eu(ii+18,i_block) = tmp_buffer(ii)
        enddo
        
        read(29,*,END=1004) time_str, (tmp_buffer(ii),ii=1,6) ! CND (1,2,3); (?,?,?) 
        do ii = 1, 3
          external_temperatures_eu(ii+24,i_block) = tmp_buffer(ii)    ! CND (1,2,3)
          external_temperatures_eu(ii+35,i_block) = tmp_buffer(ii+3)  ! What temp is this (?,?,?)
        enddo
        
        read(30,*,END=1004) time_str, (tmp_buffer(ii),ii=1,8) ! reflector temperatures (1,2,3,4,5,6,7,8)
        do ii = 1, 8
          external_temperatures_eu(ii+27,i_block) = tmp_buffer(ii)
        enddo
      enddo
1004  continue
      
      close(25)
      close(26)
      close(27)
      close(28)
      close(29)
      close(30)

      if( i_dbg_level.ge.2) then
        write(6,*) 'Read ',i_block,' blocks of external temperature telemetry.'
      endif
      
C--Load scat_telemetry_eu telemetry array from txt files.
      filename = read_sci_prefix(1:i_len) // '.scat_loopback_eu_j'      ! contains scat_telemetry_eu(1:2,:)
      open(25,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.scat_status_flags_dn_j'  ! contains scat_telemetry_dn(3:10,:)
      open(26,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.scat_temperatures_eu_j'  ! contains scat_telemetry_eu(11:17,:)
      open(27,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.scat_temperatures4_eu_j' ! contains scat_telemetry_eu(18:21,:)
      open(28,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.scat_telem_eu_j'         ! contains scat_telemetry_eu(22:31,:)
      open(29,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.scat_volt_eu_j'          ! contains scat_telemetry_eu(32:38,:)
      open(30,file=filename,status='old')
      
      do i_block = 1, MAX_NUM_BLOCKS
        
        read(25,*,END=1005) time_str, (tmp_buffer(ii),ii=1,2)
        scat_telemetry_eu(1,i_block) = tmp_buffer(1) ! loopback B1HV DC
        scat_telemetry_eu(2,i_block) = tmp_buffer(2) ! loopback B1HV power
        
        read(26,*,END=1005) time_str, (tmp_buffer(ii),ii=1,6)  ! contains scat telem status flags
        if( tmp_buffer(1) .gt. 0.5 ) scat_telemetry_eu(3,i_block)  = 1 ! CG latchup red
        if( tmp_buffer(2) .gt. 0.5 ) scat_telemetry_eu(4,i_block)  = 1 ! CG latchup prime
                                     scat_telemetry_eu(5,i_block)  = 0 ! not used (PLM 1 Lock)
                                     scat_telemetry_eu(6,i_block)  = 0 ! not used (PLM 2 Lock)
        if( tmp_buffer(3) .gt. 0.5 ) scat_telemetry_eu(7,i_block)  = 1 ! scat LVPS bus voltage
        if( tmp_buffer(4) .gt. 0.5 ) scat_telemetry_eu(8,i_block)  = 1 ! scat LVPS overcurrent
        if( tmp_buffer(5) .gt. 0.5 ) scat_telemetry_eu(9,i_block)  = 1 ! FIFO error underflow/overflow type
        if( tmp_buffer(6) .gt. 0.5 ) scat_telemetry_eu(10,i_block) = 1 ! FIFO error
        
        read(27,*,END=1005) time_str, (tmp_buffer(ii),ii=1,7)
        do ii = 1, 7
          scat_telemetry_eu(10+ii,i_block) = tmp_buffer(ii) ! inside LVPS box temp -- SFE TX load temp
        enddo
        
        read(28,*,END=1005) time_str, (tmp_buffer(ii),ii=1,4)
        do ii = 1, 4
          scat_telemetry_eu(17+ii,i_block) = tmp_buffer(ii) ! SBE Step attenuator temp -- SFE beam switch temp
        enddo

        read(29,*,END=1005) time_str, (tmp_buffer(ii),ii=1,10)
        do ii = 1, 10
          scat_telemetry_eu(21+ii,i_block) = tmp_buffer(ii) ! SFE TX power monitor -- LVPS Converter Current
        enddo

        read(30,*,END=1005) time_str, (tmp_buffer(ii),ii=1,7)
        do ii = 1, 7
          scat_telemetry_eu(31+ii,i_block) = tmp_buffer(ii) ! SSPA output stage voltage -- SBE +12V monitor (non switched)
        enddo

      enddo
1005  continue
      close(25)
      close(26)
      close(27)
      close(28)
      close(29)
      close(30)
      
      if( i_dbg_level.ge.2) then
        write(6,*) 'Read ',i_block,' blocks of scat telemetry.'
      endif
      
C--Load ICDS eng telemetry
      filename = read_sci_prefix(1:i_len) // '.icds_voltage_eu_j' ! contains icds_engineering_telemetry_eu(1:5,:)
      open(25,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.icds_temperature_eu_j' ! contains icds_engineering_telemetry_eu(6:10,:)
      open(26,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.icds_cal_resistance_eu_j' ! contains icds_engineering_telemetry_eu(11:14,:)
      open(27,file=filename,status='old')
      
      do i_block = 1, MAX_NUM_BLOCKS
        read(25,*,END=1006) time_str, (tmp_buffer(ii),ii=1,5)
        do ii = 1, 5
          icds_engineering_telemetry_eu(ii,i_block) = tmp_buffer(ii) ! ICDS +5v -- sci ADC VREF
        enddo

        read(26,*,END=1006) time_str, (tmp_buffer(ii),ii=1,5)
        do ii = 1, 5
          icds_engineering_telemetry_eu(ii+5,i_block) = tmp_buffer(ii) ! sci ADC temperature -- ICDS temp ATC 2
        enddo

        read(27,*,END=1006) time_str, (tmp_buffer(ii),ii=1,4)
        do ii = 1, 4
          icds_engineering_telemetry_eu(ii+10,i_block) = tmp_buffer(ii) ! ICDS telem board cal resistor 1 -- 4
        enddo
        
        do ii = 15, 24
          icds_engineering_telemetry_eu(ii,i_block) = 0.0 ! dummy values for rotating register ID -- selected register value
        enddo
        
      enddo
1006  continue      
      close(25)
      close(26)
      close(27)
      
      if( i_dbg_level.ge.2) then
        write(6,*) 'Read ',i_block,' blocks of icds engineering telemetry.'
      endif
      
C--Load ICDS processing status
      ! nothing written out in read_sci (we only check the block IDs in main software)

C--Load RFI flags && L0_scat_headers
      filename = read_sci_prefix(1:i_len) // '.scat_rfi_j'     ! contains: patch_flag, 
      open(25,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.scat_rfi_err_j' ! 
      open(26,file=filename,status='old')
      filename = read_sci_prefix(1:i_len) // '.scat_sci_st_j'  ! contains i_tx, i_rx (0 for h, 1 for v)
      open(27,file=filename,status='old')
      
      do i_block = 1, MAX_NUM_BLOCKS
        do i_seq = 1, 8
C--Initilizations        
          L0_scat_headers(i_seq,i_block)  = 0 
          RFI_FLAGS_VPOL(1,i_seq,i_block) = 0
          RFI_FLAGS_HPOL(1,i_seq,i_block) = 0
          RFI_FLAGS_VPOL(2,i_seq,i_block) = 0
          RFI_FLAGS_HPOL(2,i_seq,i_block) = 0
          RFI_FLAGS_VPOL(3,i_seq,i_block) = 0
          RFI_FLAGS_HPOL(3,i_seq,i_block) = 0

          read(25,*,END=1007) time_str, (tmp_buffer(ii),ii=1,8)
          ! file contains time_tag, fsw_patch, 1v, 2v, 3v, 1h, 2h, 3h, hdr_pwr_accum_overflow
          if( tmp_buffer(8) .gt. 0.5 ) L0_scat_headers(i_seq,i_block) = L0_scat_headers(i_seq,i_block)
     $                                                                + L0_SCAT_SEQ_HDR_PWR_ACCUM_OVERFLOW
     
          if( tmp_buffer(2).gt.0.5 ) RFI_FLAGS_VPOL(1,i_seq,i_block) = 1
          if( tmp_buffer(3).gt.0.5 ) RFI_FLAGS_VPOL(2,i_seq,i_block) = 1
          if( tmp_buffer(4).gt.0.5 ) RFI_FLAGS_VPOL(3,i_seq,i_block) = 1

          if( tmp_buffer(1) .gt. 0.5 ) then
            if( tmp_buffer(5).gt.0.5 ) RFI_FLAGS_HPOL(1,i_seq,i_block) = 1
            if( tmp_buffer(6).gt.0.5 ) RFI_FLAGS_HPOL(2,i_seq,i_block) = 1
            if( tmp_buffer(7).gt.0.5 ) RFI_FLAGS_HPOL(3,i_seq,i_block) = 1
          else
          ! No FSW patch, use V-pol RX RFI flags for H-pol as well.
            RFI_FLAGS_HPOL(1,i_seq,i_block) = RFI_FLAGS_VPOL(1,i_seq,i_block)
            RFI_FLAGS_HPOL(2,i_seq,i_block) = RFI_FLAGS_VPOL(2,i_seq,i_block)
            RFI_FLAGS_HPOL(3,i_seq,i_block) = RFI_FLAGS_VPOL(3,i_seq,i_block)
          endif
          
          
          read(27,*,END=1007) time_str, (tmp_buffer(ii),ii=1,2) ! tx,rx polarization flag
                                                                ! if unset => H pol, if set => V pol...
          if( tmp_buffer(1) .ge. 0.5 ) L0_scat_headers(i_seq,i_block) = L0_scat_headers(i_seq,i_block)
     $                                                                + L0_SCAT_SEQ_HDR_TX_FLAG ! TX V
          if( tmp_buffer(2) .ge. 0.5 ) L0_scat_headers(i_seq,i_block) = L0_scat_headers(i_seq,i_block)
     $                                                                + L0_SCAT_SEQ_HDR_RX_FLAG ! RX V

        enddo  
      enddo
1007  continue      
      if( .not.( i_seq.eq.8 .and. i_block.eq.MAX_NUM_BLOCKS 
     $          .or. i_seq.eq.1) ) then ! # of lines of text file should be 
                                        ! divisible by 8...
        write(stderr,*) 'READ_SCI_load_all: ERROR: incomplete block!'
        call exit(1)
      endif
      if( i_dbg_level.ge.2) then
        write(6,*) 'Read ',i_block,' blocks of RFI flags and scat sequence headers.'
      endif
      
      
      end
