C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/READ_SCI_read_block_times.f,v 1.3 2009/10/07 15:46:18 fore Exp $
c*****************************************************************      
      subroutine READ_SCI_read_block_times( read_sci_prefix,
     $                                      NUM_BLOCKS,
     $                                      start_year,
     $                                      start_day, ! day of year, not day of month
     $                                      start_JD,
     $                                      end_JD,
     $                                      blk_sec          )
c*****************************************************************      
c**   
c**   FILE NAME: READ_SCI_read_block_times.f
c**   
c**   DATE WRITTEN: 9/18/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  Read block time tags from output
c**    of 'read_sci' software.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'L1A.inc'
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      character*(*)    read_sci_prefix           ! path+prefix of 
c     OUTPUT VARIABLES
      real*8           blk_sec( MAX_NUM_BLOCKS ) ! UTC block time tags (second-of-day)
      integer          NUM_BLOCKS                ! number of blocks found
      integer          start_year                ! integer year for start
      integer          start_day                 ! integer (day-of-yaer) at start
      real*8           start_JD                  ! UTC julian date of 1st block
      real*8           end_JD                    ! UTC julian date of last block
c     LOCAL VARIABLES
      character*320    dc_offset_file
      character*23     time_str
      integer          i_block
      integer          i_len
      integer          i_year, i_month, i_day, i_hour, i_minute
      integer          i_yyyy, i_mn, i_dd, i_hh, i_mm, i_doy
      real*8 r_juldate
      real*8 r_juldatelps
      real*8           r_ss
      real*8           r_sec
      real*8           r_sod_utc
      real*8           r_sod_gps
      real*8           r_gps_sec
      real*8           r_JD
      real*8           dummy_1, dummy_2
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level
c     FUNCTION STATEMENTS
      integer lnblnk
      real*8  JD2SECGPS
c     PROCESSING STEPS
      
      i_len          = lnblnk( read_sci_prefix )
      dc_offset_file = read_sci_prefix(1:i_len) // '.scat_dc_offset_j'

      i_block = 0
      open(30,file=dc_offset_file,status='old')

123   continue      
      read(30,*,END=321) time_str, dummy_1, dummy_2 ! read a line from the dc_offset_j file
      i_block = i_block + 1                         ! increment if read was successful
      
      read( time_str(1:4),*) i_year
      read( time_str(6:7),*) i_month
      read( time_str(9:10),*) i_day
      read( time_str(12:13),*) i_hour
      read( time_str(15:16),*) i_minute
      read( time_str(18:23),*) r_sec
      
      if( i_block.eq.1 ) then
        start_year  = i_year
        call convert_day_and_month_to_dayofyear( i_year, i_day, 
     $                                           i_month, start_day )
      endif
      
C--Compute (GPS) seconds of day  
      r_sod_gps = 60.0d0*60.0d0*i_hour + 60.0d0*i_minute + r_sec
      call find_julian_day( i_year, i_month, i_day, r_JD )

      r_gps_sec = JD2SECGPS( r_JD ) + r_sod_gps
      
      call gps2time(r_gps_sec,r_juldate,r_juldatelps,
     &   i_yyyy,i_mm,i_dd,i_hh,i_mn,r_ss,i_doy,r_sod_utc)
      
      if( i_block.eq.1) start_JD = r_juldatelps
      end_JD = r_juldatelps  ! continue overwriting and it will contain the last one at the end.
      
C--We KNOW that (r_sod_utc - r_sod_gps) MUST be an integer (by definition), so let it be so.
C--(Kill the microsecond errors in the conversions that use julian date as
C--an intermediate step.)
      blk_sec( i_block ) = r_sod_gps + nint( r_sod_utc - r_sod_gps )
      
      if( i_dbg_level.ge.5 ) then
        write(6,*) 'time_str: ',time_str
        write(6,*) 'year:', time_str(1:4) 
        write(6,*) 'month:', time_str(6:7)
        write(6,*) 'day:', time_str(9:10)
        write(6,*) 'hour:', time_str(12:13)
        write(6,*) 'minute:', time_str(15:16)
        write(6,*) 'second: ', time_str(18:23)
        
        write(6,'((a50),2f20.8)') 'r_JD, r_gps_sec: ',r_JD, r_gps_sec
        write(6,'((a50),3f20.8)') 'r_sod_gps, r_sod_utc: ', r_sod_gps, r_sod_utc, blk_sec( i_block )
        write(6,'((a50),2f20.8)') 'r_juldate, r_juldatelps: ', r_juldate,r_juldatelps
      endif

      goto 123      
321   continue
      close(30)
      
      NUM_BLOCKS = i_block
      
      end
