C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/READ_SCI_splines.f,v 1.4 2009/10/07 15:46:18 fore Exp $
c*****************************************************************      
      function get_READ_SCI_spline( time, filled_spline )
c*****************************************************************      
c**   
c**   FILE NAME: READ_SCI_splines.f
c**   
c**   DATE WRITTEN: 9/18/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
      integer stdout, stderr, stdin
      parameter( stdout = 6, stderr = 0, stdin = 5 )
      integer N_SPLINES_MAX
      parameter( N_SPLINES_MAX = 10000 )      ! Enough for one orbit at 1sec per spline
c     INPUT VARIABLES
      real*8 time
c     OUTPUT VARIABLES
      integer get_READ_SCI_spline
      real*8 filled_spline(10)
c     LOCAL VARIABLES
      integer m
      integer m_XYZ_last /0/
      integer m_YPR_last /0/

      integer np
      real*8 time0, time1
      real*8 xdelta
      real*8 aa, bb, cc, dd
      logical XYZ_spline_found
      logical YPR_spline_found   
c     DATA STATEMENTS
c     COMMON BLOCKS
      real*8 XYZ_spline_c(7,6,0:N_SPLINES_MAX)
      real*8 YPR_spline_c(4,6,0:N_SPLINES_MAX)
      real*8 time_ref_XYZ, time_ref_YPR
      integer n_XYZ_splines, n_YPR_splines
      common /READ_SCI_ephemeris_spline/ XYZ_spline_c, YPR_spline_c, 
     $ time_ref_XYZ, time_ref_YPR, n_XYZ_splines, n_YPR_splines

      integer i_dbg_level
      common /debug/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      save m_XYZ_last, m_YPR_last

      XYZ_spline_found    = .false.
      YPR_spline_found    = .false.      
      get_READ_SCI_spline = 0

      do m=m_XYZ_last,n_XYZ_splines
        time0 = XYZ_spline_c(2,1,m)
        time1 = XYZ_spline_c(2,6,m)
        if ( time0 .le. time .and. time .lt. time1 ) then
          if( i_dbg_level .ge. 1)then
            
          endif
          m_XYZ_last = m
          
          XYZ_spline_found = .true.
          xdelta = ( time - time0 )
          filled_spline(1) = time

          if( i_dbg_level .ge. 1)then
            write(6,'((a),I7)') ' in get_READ_SCI_spline, m_XYZ: ',m
          endif
          
          do np = 2, 7
            aa = XYZ_spline_c(np,2,m)
            bb = XYZ_spline_c(np,3,m)
            cc = XYZ_spline_c(np,4,m)
            dd = XYZ_spline_c(np,5,m)
            filled_spline(np) = aa + bb*xdelta +  cc*xdelta**2 + dd*xdelta**3
          enddo
        endif
      enddo
      
      if( .not.XYZ_spline_found )then
        write(6,*) 'get_READ_SCI_spline: ERROR: XYZ Spline not found!'
        get_READ_SCI_spline = get_READ_SCI_spline - 1
      endif
      
      do m=m_YPR_last,n_YPR_splines
        time0 = YPR_spline_c(2,1,m)
        time1 = YPR_spline_c(2,6,m)
        if ( time0 .le. time .and. time .lt. time1 ) then
          if( i_dbg_level .ge. 1)then
            write(6,'((a),I7)') ' in get_READ_SCI_spline, m_YPR: ',m
          endif
          m_YPR_last = m
          
          YPR_spline_found = .true.
          xdelta = ( time - time0 )
          do np = 2, 4
            aa = YPR_spline_c(np,2,m)
            bb = YPR_spline_c(np,3,m)
            cc = YPR_spline_c(np,4,m)
            dd = YPR_spline_c(np,5,m)
            filled_spline(np+6) = aa + bb*xdelta +  cc*xdelta**2 + dd*xdelta**3
          enddo
        endif
      enddo
      
      if( .not.YPR_spline_found )then
        write(6,*) 'get_READ_SCI_spline: ERROR: YPR Spline not found!'
        get_READ_SCI_spline = get_READ_SCI_spline - 2
      endif   
      end


c*****************************************************************      
      subroutine gen_READ_SCI_splines(  emph_XYZ_file,
     $                                  emph_RPY_file,
     $                                  start_JD,
     $                                  end_JD,
     $                                  time_ref   )
c*****************************************************************      
c**   
c**   FILE NAME: READ_SCI_splines.f
c**   
c**   DATE WRITTEN: 9/18/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
      integer stdout, stderr, stdin
      parameter( stdout = 6, stderr = 0, stdin = 5 )
      integer N_SPLINES_MAX
      parameter( N_SPLINES_MAX = 10000 )      ! Enough for one orbit at 1sec per spline
c     INPUT VARIABLES
      character*(*) emph_XYZ_file             ! binary Wentz orbit XYZ filename
      character*(*) emph_RPY_file             ! binary Wentz orbit RPY filename
c     OUTPUT VARIABLES
      real*8        start_JD                  ! Julian date of 1st block
      real*8        end_JD                    ! Julian date of last block
      real*8 time_ref
c     LOCAL VARIABLES
      real*8 wentz_t_sec_since_2000, time, time0
      real*8 x_pos, y_pos, z_pos
      real*8 x_vel, y_vel, z_vel
      real*8 ypr_time, yaw_deg, pitch_deg, roll_deg
      
      real*8 store(7,0:N_SPLINES_MAX)
      real*8 x(0:N_SPLINES_MAX), a(0:N_SPLINES_MAX), h(0:N_SPLINES_MAX)
      real*8 alpha(0:N_SPLINES_MAX)
      real*8 mu(0:N_SPLINES_MAX), l(0:N_SPLINES_MAX), z(0:N_SPLINES_MAX)
      real*8 b(0:N_SPLINES_MAX), c(0:N_SPLINES_MAX), d(0:N_SPLINES_MAX)
      real*8 fp0, fpn
      
      logical done_reading_emph
      
      integer xyz_rec, rpy_rec
      integer m, isize, isize1, jj, i, j, nparameter
c     DATA STATEMENTS
c     COMMON BLOCKS
      real*8 XYZ_spline_c(7,6,0:N_SPLINES_MAX)
      real*8 YPR_spline_c(4,6,0:N_SPLINES_MAX)
      real*8 time_ref_XYZ, time_ref_YPR
      integer n_XYZ_splines, n_YPR_splines
      common /READ_SCI_ephemeris_spline/ XYZ_spline_c, YPR_spline_c, 
     $ time_ref_XYZ, time_ref_YPR, n_XYZ_splines, n_YPR_splines

      integer i_dbg_level
      common /debug/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS

C--Generate the XYZ splines
      open(25,file=emph_XYZ_file,form='unformatted',access='direct',recl=7*8,status='old')
      
      m                 = 0
      xyz_rec           = 0
      done_reading_emph = .false.
      
      do while ( .not.done_reading_emph )
        xyz_rec = xyz_rec + 1
        read(25,rec=xyz_rec,err=101) wentz_t_sec_since_2000, x_pos, y_pos, z_pos, x_vel, y_vel, z_vel
C--He gives time in sec since start of year 2000. (00:00:00 Z 1/1/2000)
C--2451544.5d0 is julian time of start of year 2000 (00:00:00 Z 1/1/2000)
        time = (wentz_t_sec_since_2000) / (24.d0*60.d0*60.d0) + 2451544.5d0

C--If time tag is within one minute of the interval [start_JD,end_JD] then
C--start populating the arrays used to generate the splines.
        if( time .gt. start_JD - 1.0d0 / (24.0d0 * 60.0d0 ) .and.
     $      time .lt.   end_JD + 1.0d0 / (24.0d0 * 60.0d0 )       ) then
          
          if( m.eq.0 ) time_ref_XYZ = time
          
          time0      = time - time_ref_XYZ
          store(1,m) = time0
          store(2,m) = x_pos
          store(3,m) = y_pos
          store(4,m) = z_pos
          store(5,m) = x_vel
          store(6,m) = y_vel
          store(7,m) = z_vel 
          
          m = m + 1

          if( m .ge. N_SPLINES_MAX )then
            write(6,*) 'gen_spline: ERROR: increase parameter N_SPLINES_MAX!'
            goto 101
          endif
        elseif ( time .gt. end_JD + 1.0d0 / (24.0d0 * 60.0d0 ) ) then
          done_reading_emph = .true. ! we can quit reading the orbit file now.
        endif
      enddo

101   continue      
      close(25)
      
      write(6,*) "for XYZ splines ", m, " samples read "

      isize  = m-1
      isize1 = isize-1
       
      do jj = 0,isize
            x(jj) = store(1,jj)
      enddo
              
      do nparameter = 2,7          
        if( i_dbg_level .ge. 2) write(6,*) " nparameter", nparameter
        do jj = 0,isize
          a(jj) = store(nparameter,jj)
        enddo
          

c     step 1
        do i = 0, isize1
          h(i) = x(i+1) - x(i)
        enddo
            
        fp0 = 0.0d0
        fpn = 0.0d0
        
C--Use velocity as the boundary conditions on df/dt for the position spline generation.      
C--The time in the spline generation is in units of days, thus we must convert
C--the initial velocity (which is in meters / sec) to meters / day (thus the factor of 
C-- 86400.)
        if( nparameter.gt.1 .and. nparameter.lt.5 )then
          fp0 = store(nparameter+3,0)     * 86400.0d0 ! order t,x,y,z,vx,vy,vz,y,p,r
          fpn = store(nparameter+3,isize) * 86400.0d0
        endif
C--finite difference approximation to time derivative of velocity at 
C--start and end time.
        if( nparameter.gt.4 .and. nparameter.lt.8 )then
          fp0 = ( store(nparameter,1) - store(nparameter,0) ) / h(0)
          fpn = ( store(nparameter,isize) - store(nparameter,isize-1) ) / h(isize1)
        endif
       
c     step 2
        alpha(0)     = 3.0d0*(a(1) - a(0))/h(0) - 3.0d0 * fp0
        alpha(isize) = 3.0d0 * fpn - 3.0d0*(a(isize) - a(isize1))/h(isize1)
          
c     step 3
        do i = 1, isize1
          alpha(i) = 3.d0*(a(i+1)*h(i-1) - a(i)*(x(i+1) - x(i-1))  + 
     $     a(i-1)*h(i) ) / ( h(i-1) * h(i) )
        enddo
          
c     step 4
        l(0)  = 2.d0 * h(0)
        mu(0) = 0.5d0
        z(0)  = alpha(0) / l(0)
          
c     step 5
        do i = 1, isize1
          l(i)  = 2.d0*( x(i+1) - x(i-1) ) - ( h(i) * mu(i-1) )
          mu(i) = h(i) / l(i)
          z(i)  = ( alpha(i) - h(i) * z(i-1)) / l(i)
        enddo
          
c     step 6
        l(isize) = h(isize1) * ( 2.d0 - mu(isize1) )
        z(isize) = ( alpha(isize) - h(isize1) * z(isize1) )/ l(isize)
        c(isize) = z(isize)
          
c     step 7
        do j = isize1, 0, -1
          c(j) = z(j) - mu(j) * c(j+1)
          b(j) = ( a(j+1) - a(j) )/h(j) - h(j)*(c(j+1) + 2.d0*c(j))/3.d0
          d(j) = ( c(j+1) - c(j) )/(3.d0*h(j))
        enddo
          
          
c     step 8
        do j = 0, isize1
          XYZ_spline_c(nparameter,1,j) = x(j)
          XYZ_spline_c(nparameter,6,j) = x(j+1)
          XYZ_spline_c(nparameter,2,j) = a(j)
          XYZ_spline_c(nparameter,3,j) = b(j)
          XYZ_spline_c(nparameter,4,j) = c(j)
          XYZ_spline_c(nparameter,5,j) = d(j)
c     write(33,334) x(j), a(j), b(j), c(j), d(j)
c334       format(1x, 5e25.15)
        enddo       
      enddo                    ! with nparameter
      
      n_XYZ_splines = isize

      m                 = 0
      rpy_rec           = 0
      done_reading_emph = .false.
      
      open(25,file=emph_RPY_file,form='unformatted',access='direct',recl=4*8,status='old')
      
      do while ( .not.done_reading_emph )
        rpy_rec = rpy_rec + 1
        read(25,rec=rpy_rec,err=201) wentz_t_sec_since_2000, roll_deg, pitch_deg, yaw_deg
C--He gives time in sec since start of year 2000. (00:00:00 Z 1/1/2000)
C--2451544.5d0 is julian time of start of year 2000 (00:00:00 Z 1/1/2000)
        time = (wentz_t_sec_since_2000) / (24.d0*60.d0*60.d0) + 2451544.5d0

C--If time tag is within one minute of the interval [start_JD,end_JD] then
C--start populating the arrays used to generate the splines.
        if( time .gt. start_JD - 1.0d0 / (24.0d0 * 60.0d0 ) .and.
     $      time .lt.   end_JD + 1.0d0 / (24.0d0 * 60.0d0 )       ) then
          
          time0      = time - time_ref_XYZ
          store(1,m) = time0
          store(2,m) = yaw_deg
          store(3,m) = pitch_deg
          store(4,m) = roll_deg 
          
          m = m + 1

          if( m .ge. N_SPLINES_MAX )then
            write(6,*) 'gen_spline: ERROR: increase parameter N_SPLINES_MAX!'
            goto 201
          endif
        elseif ( time .gt. end_JD + 1.0d0 / (24.0d0 * 60.0d0 ) ) then
          done_reading_emph = .true. ! we can quit reading the orbit file now.
        endif
      enddo

201   continue      
      close(25)

      time_ref_YPR = time_ref_XYZ
      
      write(6,*) "for YPR splines ", m, " samples read "

      isize  = m-1
      isize1 = isize-1
       
      do jj = 0,isize
            x(jj) = store(1,jj)
      enddo
              
      do nparameter = 2,4          
        if( i_dbg_level .ge. 2) write(6,*) " nparameter", nparameter
        do jj = 0,isize
          a(jj) = store(nparameter,jj)
        enddo
          

c     step 1
        do i = 0, isize1
          h(i) = x(i+1) - x(i)
        enddo
            
        fp0 = 0.0d0
        fpn = 0.0d0
       
c     step 2
        alpha(0)     = 3.0d0*(a(1) - a(0))/h(0) - 3.0d0 * fp0
        alpha(isize) = 3.0d0 * fpn - 3.0d0*(a(isize) - a(isize1))/h(isize1)
          
c     step 3
        do i = 1, isize1
          alpha(i) = 3.d0*(a(i+1)*h(i-1) - a(i)*(x(i+1) - x(i-1))  + 
     $     a(i-1)*h(i) ) / ( h(i-1) * h(i) )
        enddo
          
c     step 4
        l(0)  = 2.d0 * h(0)
        mu(0) = 0.5d0
        z(0)  = alpha(0) / l(0)
          
c     step 5
        do i = 1, isize1
          l(i)  = 2.d0*( x(i+1) - x(i-1) ) - ( h(i) * mu(i-1) )
          mu(i) = h(i) / l(i)
          z(i)  = ( alpha(i) - h(i) * z(i-1)) / l(i)
        enddo
          
c     step 6
        l(isize) = h(isize1) * ( 2.d0 - mu(isize1) )
        z(isize) = ( alpha(isize) - h(isize1) * z(isize1) )/ l(isize)
        c(isize) = z(isize)
          
c     step 7
        do j = isize1, 0, -1
          c(j) = z(j) - mu(j) * c(j+1)
          b(j) = ( a(j+1) - a(j) )/h(j) - h(j)*(c(j+1) + 2.d0*c(j))/3.d0
          d(j) = ( c(j+1) - c(j) )/(3.d0*h(j))
        enddo
          
          
c     step 8
        do j = 0, isize1
          YPR_spline_c(nparameter,1,j) = x(j)
          YPR_spline_c(nparameter,6,j) = x(j+1)          
          YPR_spline_c(nparameter,2,j) = a(j)
          YPR_spline_c(nparameter,3,j) = b(j)
          YPR_spline_c(nparameter,4,j) = c(j)
          YPR_spline_c(nparameter,5,j) = d(j)
c     write(33,334) x(j), a(j), b(j), c(j), d(j)
c334       format(1x, 5e25.15)
        enddo       
      enddo                    ! with nparameter
      
      n_YPR_splines = isize
      time_ref      = time_ref_XYZ
      
      end