C--$Header: /u/caretr2/scatsim/CVSROOT+/L1A_to_L2/aqradarmod.f,v 1.3 2012/04/27 16:33:20 fore Exp $
c*****************************************************************      
      subroutine aqradargmf_op( ainc, wspd, wdir, sigma0 )
c*****************************************************************      
c modifed version of simon's aqradar_gmf subroutine (see below)
c*****************************************************************      
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 ainc, wspd, wdir
c     OUTPUT VARIABLES
      real*8 sigma0(3)
c     LOCAL VARIABLES
      real*8  dtheta, b1, b2
      real*4  r4_wspd, r4_wdir
      real*4  an(3,0:2)
      real*4  slope_left, slope_right
      real*4  tmp_sigma0(3)
      integer i_pol, i_beam, i_spd, n, ib1, ib2
      
      
      logical l_first          /.true./
      real*4  spdvec(0:50)
      real*4  a( 0:50,0:2,3,3) ! (ispd, n, ipol, ibeam)
      real*4  a2(0:50,0:2,3,3) ! (ispd, n, ipol, ibeam)
      real*4  an1, an2
      
c     DATA STATEMENTS
      real beamincidence(3)
!      data beamincidence/29.3568, 38.4501, 46.2918/
      data beamincidence/29.358,38.487,46.286/

c     COMMON BLOCKS
c     FUNCTION STATEMENTS
      real*4 cosd
c     PROCESSING STEPS
      save l_first, a, a2, spdvec
      
      if( l_first ) then ! populate spline 2nd derivative tables
        do i_beam=1,3
          do i_spd=0,50
            spdvec(i_spd)=real(i_spd)
            call aqradarmod(i_beam,spdvec(i_spd),r4_wdir,tmp_sigma0,an)
            do i_pol=1,3 ! VV, HH, HV
              do n=0,2
                a(i_spd,n,i_pol,i_beam) = an(i_pol,n)
              enddo
            enddo
          enddo
          
          do i_pol=1,3  
            ! Fix slope at end points for spline boundary conditions.
            ! Only A0 has non-zero slope at boundaries. (use finite-difference)
            slope_left  = ( a( 1,n,i_pol,i_beam) - a( 0,n,i_pol,i_beam) ) / 1.0
            slope_right = ( a(50,n,i_pol,i_beam) - a(49,n,i_pol,i_beam) ) / 1.0
            
            call spline( spdvec, a(0,0,i_pol,i_beam), 51, slope_left, slope_right, a2(0,0,i_pol,i_beam) )
            call spline( spdvec, a(0,1,i_pol,i_beam), 51,        0.0,         0.0, a2(0,1,i_pol,i_beam) )
            call spline( spdvec, a(0,2,i_pol,i_beam), 51,        0.0,         0.0, a2(0,2,i_pol,i_beam) )
          enddo
        enddo
        l_first = .false.
      endif

      if( ainc .le. beamincidence(2) ) then
        ib1=1
        ib2=2
      else
        ib1=2
        ib2=3
      endif

      dtheta=beamincidence(ib2)-beamincidence(ib1)
      b1=(beamincidence(ib2)-ainc)/dtheta
      b2=(ainc-beamincidence(ib1))/dtheta
      
      r4_wspd = wspd
      r4_wdir = wdir

      do i_pol=1,3
        do n=0,2
          call splint( spdvec, a(0,n,i_pol,ib1), a2(0,n,i_pol,ib1), 51, r4_wspd, an1 )
          call splint( spdvec, a(0,n,i_pol,ib2), a2(0,n,i_pol,ib2), 51, r4_wspd, an2 )
          an(i_pol,n)=b1*an1+b2*an2  
        enddo
        sigma0(i_pol) = an(i_pol,0)*(1+an(i_pol,1)*cosd(r4_wdir)+an(i_pol,2)*cosd(2*r4_wdir) )        
      enddo

      return
      end

c*****************************************************************      
      subroutine aqradargmf_op_bak( ainc, wspd, wdir, sigma0 )
c*****************************************************************      
c modifed version of simon's aqradar_gmf subroutine (see below)
c*****************************************************************      
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 ainc, wspd, wdir
c     OUTPUT VARIABLES
      real*8 sigma0(3), an(3,0:2)
c     LOCAL VARIABLES
      real*8 dtheta, b1, b2
      real*4 r4_wspd, r4_wdir
      real*4 sigma01(3),an1(3,0:2)
      real*4 sigma02(3),an2(3,0:2)
      integer ipol, n, ib1, ib2
c     DATA STATEMENTS
      real beamincidence(3)
!      data beamincidence/29.3568, 38.4501, 46.2918/
      data beamincidence/29.358,38.487,46.286/

c     COMMON BLOCKS
c     FUNCTION STATEMENTS
c     PROCESSING STEPS

      r4_wspd = wspd
      r4_wdir = wdir
      
      if( ainc .le. beamincidence(2) ) then
        ib1=1
        ib2=2
      else
        ib1=2
        ib2=3
      endif

      dtheta=beamincidence(ib2)-beamincidence(ib1)
      b1=(beamincidence(ib2)-ainc)/dtheta
      b2=(ainc-beamincidence(ib1))/dtheta
        
      call aqradarmod(ib1,r4_wspd,r4_wdir,sigma01,an1)
      call aqradarmod(ib2,r4_wspd,r4_wdir,sigma02,an2)
        
      do ipol=1,3
        sigma0(ipol)=b1*sigma01(ipol)+b2*sigma02(ipol)
        do n=0,2
          an(ipol,n)=b1*an1(ipol,n)+b2*an2(ipol,n)
        enddo
      enddo

      return
      end

c*********************************************************
        subroutine aqradargmf(ainc,wspd,wdir,sigma0,an)
c author: S Yueh
c v1: Nov 26, 2011
c This routine calculates the wind induced backscatter 
c using the Aquarius roughness model for each beam and interpolation over 
c incidence angles.
c input:
c ainc: incidence angle in degree
c wspd: wind speed in m/s
c wdir: wind direction in deg
c output:
c sigma0(3): 1 for vv, 2 for hh, and 3 vh in real number
c an(3,0:2): the first index for  polarization with 1 for vv, 2 for hh, and 3 for vh and the second array index for nth harmonics

        real beamincidence(3)
!        data beamincidence/29.3568, 38.4501, 46.2918/
        data beamincidence/29.358,38.487,46.286/
        real sigma0(3),an(3,0:2)
        real sigma01(3),an1(3,0:2)
        real sigma02(3),an2(3,0:2)

        if( ainc .le. beamincidence(2) ) then
        ib1=1
        ib2=2
        else
        ib1=2
        ib2=3
        endif

        dtheta=beamincidence(ib2)-beamincidence(ib1)
        b1=(beamincidence(ib2)-ainc)/dtheta
        b2=(ainc-beamincidence(ib1))/dtheta

        call aqradarmod(ib1,wspd,wdir,sigma01,an1)
        call aqradarmod(ib2,wspd,wdir,sigma02,an2)

        do ipol=1,3
        sigma0(ipol)=b1*sigma01(ipol)+b2*sigma02(ipol)
                do n=0,2
        an(ipol,n)=b1*an1(ipol,n)+b2*an2(ipol,n)
                enddo
        enddo

        return
        end

c*********************************************************
        subroutine aqradarmod(jbeam,wspd,wdir,sigma0,an)
c input:
c jbeam: beam number
c wspd: wind speed in m/s
c wdir: wind direction in deg
c output:
c sigma0(3): 1 for vv, 2 for hh, and 3 vh in real number
c an(3,0:2): the first index for  polarization with 1 for vv, 2 for hh, and 3 for vh and the second array index for nth harmonics

        integer jbeam
        real wspd,w
        real table

!      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
!      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT,
!     $ AQ_SCAT_SIM_DIR_SCRATCH   
        
        common /aqradartable/table(0:30,27)
        common /initradartable/switch
        data switch/0/
        character*200 infile
        include 'aqradarmod.inc'

        real sigma0(3),an(3,0:2)
        
        if(switch .eq. 0)then

        do i=0,30
        do j=1,27
                table(i,j)=0
        enddo
        enddo

c       infile='/home/simon/proj/seamod/aqmod/radar-ssmi5/scat_ssmif17a240_299fit012_L2.VV'
        infile=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT))//infileVV

        open(unit=50,file=infile)
        read(50,*)
        do i=0,24
        read(50,*)wn,(table(i,j),j=1,9)
        enddo
        close(unit=50)
c       infile='/home/simon/proj/seamod/aqmod/radar-ssmi5/scat_ssmif17a240_299fit012_L2.HH'
        infile=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT))//infileHH

        open(unit=50,file=infile)
        read(50,*)
        do i=0,24
           read(50,*)wn,(table(i,j+9),j=1,9)
        enddo
        close(unit=50)
c       infile='/home/simon/proj/seamod/aqmod/radar-ssmi5/scat_ssmif17a240_299fit012_L2.VH'
        infile=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT))//infileVH
        open(unit=50,file=infile)
        read(50,*)
        do i=0,24
        read(50,*)wn,(table(i,j+18),j=1,9)
        enddo
        close(unit=50)

        switch=1
        endif

        w=wspd
        wc=21
        wd=12

        if( w .le. 19 )then
        iw1=w
                if( iw1 .lt. 1 ) iw1=1
        iw2=iw1+1
        b1=iw2-w
        b2=w-iw1
                do ipol=1,3
                do n=0,2
        d1=table(iw1,jbeam+(ipol-1)*9+n*3)
        d2=table(iw2,jbeam+(ipol-1)*9+n*3)
        an(ipol,n)=b1*d1+b2*d2
                enddo
                enddo
        else if( w .ge. 18 .and. w. lt. 20 )then
        iw1=18
        iw2=20
        dw=iw2-iw1
        b1=(iw2-w)/dw
        b2=(w-iw1)/dw
                do ipol=1,3
                do n=0,2
        d1=table(iw1,jbeam+(ipol-1)*9+n*3)
        d2=table(iw2,jbeam+(ipol-1)*9+n*3)
        an(ipol,n)=b1*d1+b2*d2
                enddo
                enddo
        else 
        iw1=18
        iw2=20
        dw=iw2-iw1
        b1=(iw2-w)/dw
        b2=(w-iw1)/dw
                do ipol=1,3
                do n=0,0
        d1=table(iw1,jbeam+(ipol-1)*9+n*3)
        d2=table(iw2,jbeam+(ipol-1)*9+n*3)
        an(ipol,n)=b1*d1+b2*d2
                enddo
                enddo
                do ipol=1,3
                do n=1,2
        d2=table(iw2,jbeam+(ipol-1)*9+n*3)
        an(ipol,n)=d2
                enddo
                enddo
        do ipol=1,3
        do n=1,2
        if(w .ne. wc)then
        an(ipol,n)=an(ipol,n)*(1-exp(-(wd/abs(w-wc))**1))
        endif
        enddo
        enddo
        endif


        do ipol=1,3
        sigma0(ipol)=an(ipol,0)*(1+an(ipol,1)*cosd(wdir)
     *                  +an(ipol,2)*cosd(2*wdir) )
        enddo

        return 
        end
