C--$Header: /u/caretr2/scatsim/CVSROOT/L1A_to_L2/aqradmod.f,v 1.2 2012/04/27 16:29:53 fore Exp $
c*********************************************************
      subroutine L2_delta_emissivity_surface_model( spd, 
     $                                              serr, 
     $                                              relazi_in, 
     $                                              ainc, 
     $                                              delta_emiss, 
     $                                              delta_emiss_err )
c*********************************************************
c-Computes the excess surface emissivity due to wind
c*********************************************************
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 spd       ! scat wind speed  [m/s]
      real*8 serr      ! scat speed error [m/s]
      real*8 relazi_in ! relative azimuth angle [deg]
      real*8 ainc      ! incidence angle [deg]
c     OUTPUT VARIABLES
      real*8 delta_emiss(2)     ! excess emissivity (V,H)
      real*8 delta_emiss_err(2) ! excess emissivity error (V,H)
c     LOCAL VARIABLES
      real*8 relazi
      real*8 delta_emiss_plus(2)
      real*8 delta_emiss_minus(2)
      real*8 spd_plus
      real*8 spd_minus
c     DATA STATEMENTS
c     COMMON BLOCKS
c     FUNCTION STATEMENTS
c     PROCESSING STEPS      
      
      ! switch wind direction conventions to use S. Yueh's subroutines
      relazi = relazi_in + 180.0
      if( relazi .ge. 180 ) relazi = relazi - 360
      if( relazi .lt.-180 ) relazi = relazi + 360
      
      spd_plus  = spd + serr
      spd_minus = spd - serr
      
      call aqdtbwgmf_op( ainc, spd,       relazi, delta_emiss(1),       delta_emiss(2)       )
      call aqdtbwgmf_op( ainc, spd_plus,  relazi, delta_emiss_plus(1),  delta_emiss_plus(2)  )
      call aqdtbwgmf_op( ainc, spd_minus, relazi, delta_emiss_minus(1), delta_emiss_minus(2) )
      
      delta_emiss_err(1) = abs((delta_emiss_plus(1)-delta_emiss_minus(1))/2.0)
      delta_emiss_err(2) = abs((delta_emiss_plus(2)-delta_emiss_minus(2))/2.0)
      
      return
      end

c*********************************************************
      subroutine aqdtbwgmf_op(ainc,w,phi,dev,deh)
c*********************************************************
c modifed version of simon's aqdtbwgmf subroutine (see below)
c*********************************************************
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 ainc, w, phi ! inc, spd, relazi
c     OUTPUT VARIABLES
      real*8 dev, deh     ! Excess surface emissivity for V, H pol
c     LOCAL VARIABLES
      real*4  dtheta, b1, b2
      real*4  r4_w, r4_phi
      real*4  surface_temp
      real*4  dtv1, dtv2, dth1, dth2
      integer i_pol, i_beam, i_spd, ib1, ib2
c     DATA STATEMENTS
      real beamincidence(3)
!      data beamincidence/29.3568, 38.4501, 46.2918/
      data beamincidence/29.358,38.487,46.286/

c     COMMON BLOCKS
c     FUNCTION STATEMENTS
c     PROCESSING STEPS      

      if( ainc .le. beamincidence(2) ) then
        ib1=1
        ib2=2
      else
        ib1=2
        ib2=3
      endif

      dtheta=beamincidence(ib2)-beamincidence(ib1)
      b1=(beamincidence(ib2)-ainc)/dtheta
      b2=(ainc-beamincidence(ib1))/dtheta

      surface_temp = 5.0 ! dummy value [deg C]
      r4_w   = w
      r4_phi = phi
      call aqdtbw(ib1,r4_w,r4_phi,surface_temp,dtv1,dth1)
      call aqdtbw(ib2,r4_w,r4_phi,surface_temp,dtv2,dth2)

      dev=(b1*dtv1+b2*dtv2)/(273.16+surface_temp)
      deh=(b1*dth1+b2*dth2)/(273.16+surface_temp)
      
      end
      
c*********************************************************
        subroutine aqdtbwgmf(ainc,w,phi,Ts,dtv,dth)
c author: S Yueh
c v1: Nov 26, 2011
c This routine calculates the wind induced brightness temperatures
c using the Aquarius roughness model for each beam and interpolation over 
c incidence angle.
c input:
c ainc: incidence angle in degree
c w: wind speed in m/s
c phi: wind direction in deg 
c Ts: surface temperature in deg C
c output:
c dTV: delta TBv due to wind in K
c dTH: delta TBh due to wind in K
        real beamincidence(3)
!        data beamincidence/29.3568, 38.4501, 46.2918/
        data beamincidence/29.358,38.487,46.286/

        if( ainc .le. beamincidence(2) ) then
        ib1=1
        ib2=2
        else
        ib1=2
        ib2=3
        endif

        dtheta=beamincidence(ib2)-beamincidence(ib1)
        b1=(beamincidence(ib2)-ainc)/dtheta
        b2=(ainc-beamincidence(ib1))/dtheta

        call aqdtbw(ib1,w,phi,Ts,dtv1,dth1)
        call aqdtbw(ib2,w,phi,Ts,dtv2,dth2)

        dtv=b1*dtv1+b2*dtv2
        dth=b1*dth1+b2*dth2

        return
        end
c*********************************************************
        subroutine aqdtbw(jbeam,w,phi,Ts,dtv,dth)
c This routine calculates the wind induced brightness temperatures
c input:
c jbeam: beam number
c w: wind speed in m/s
c phi: wind direction in deg 
c Ts: surface temperature in deg C
c output:
c dTV: delta TBv due to wind in K
c dTH: delta TBh due to wind in K

        integer jbeam
        real dtv,dth,w
        real table

!      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
!      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT,
!     $ AQ_SCAT_SIM_DIR_SCRATCH   
        
        common /aqdtdwtable/table(0:30,27)
        real tbbias
        common /aqradan/an
        common /initrad/switch
        data switch/0/
        character*200 infile
        include 'aqradmod.inc'

        real dtb(3),an(3,0:2)
        
        if(switch .eq. 0)then

        do i=0,30
        do j=1,27
                table(i,j)=0
        enddo
        enddo

        infile=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT))//infile_Tvgmf
        open(unit=50,file=infile)
        read(50,*)
        do i=0,25
        read(50,*)wn,(table(i,j),j=1,9)
c       write(*,*)wn,(table(i,j),j=1,9)
        enddo
        close(unit=50)

        infile=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT))//infile_Thgmf

        open(unit=50,file=infile)
        read(50,*)
        do i=0,25
        read(50,*)wn,(table(i,j+9),j=1,9)
c       write(*,*)wn,(table(i,j+9),j=1,9)
        enddo
        close(unit=50)

c carry out 3-pt moving window average
        do i=16,23
                do j=1,27
        table(i,j)=table(i-1,j)+table(i,j)+table(i+1,j)
        table(i,j)=table(i,j)/3
                enddo
        enddo

c adjust for zero offset using linear extrapolation
c       do j=2,7
c       tbbias(j-1)=table(1,j)-(table(2,j)-table(1,j))
c       do i=1,25
c       table(i,j)=table(i,j)-tbbias(j-1)
c       enddo
c       table(0,j)=0
c       enddo
c       write(*,*)'tbbias',tbbias
        switch=1
        endif

c        w=wspd
        wc=21
        wd=12

        if( w .le. 16 )then
        iw1=w
                if( iw1 .lt. 1 ) iw1=1
        iw2=iw1+1
        b1=iw2-w
        b2=w-iw1
                do ipol=1,3
                do n=0,2
        d1=table(iw1,jbeam+(ipol-1)*9+n*3)
        d2=table(iw2,jbeam+(ipol-1)*9+n*3)
        an(ipol,n)=b1*d1+b2*d2
                enddo
                enddo
        else if( w .ge. 16 .and. w. lt. wc )then
        iw1=15
        iw2=17
        dw=iw2-iw1
        b1=(iw2-w)/dw
        b2=(w-iw1)/dw
                do ipol=1,3
                do n=0,2
        d1=table(iw1,jbeam+(ipol-1)*9+n*3)
        d2=table(iw2,jbeam+(ipol-1)*9+n*3)
        an(ipol,n)=b1*d1+b2*d2
                enddo
                enddo

c               if( jbeam .eq. 3)then
c               ipol=2
c                n=2
c        d1=table(iw1,jbeam+(ipol-1)*9+n*3)
c        an(ipol,n)=d1
c               endif
        else 
        iw1=15
        iw2=17
c        iw2=19
        dw=iw2-iw1
        b1=(iw2-w)/dw
        b2=(w-iw1)/dw
                do ipol=1,3
                do n=0,0
        d1=table(iw1,jbeam+(ipol-1)*9+n*3)
        d2=table(iw2,jbeam+(ipol-1)*9+n*3)
        an(ipol,n)=b1*d1+b2*d2
                enddo
                enddo

        b1=(iw2-wc)/dw
        b2=(wc-iw1)/dw
                do ipol=1,3
                do n=1,2
        d1=table(iw1,jbeam+(ipol-1)*9+n*3)
        d2=table(iw2,jbeam+(ipol-1)*9+n*3)
        an(ipol,n)=b1*d1+b2*d2
                enddo
                enddo

        do ipol=1,3
        do n=1,2
        if(w .ne. wc)then
        an(ipol,n)=an(ipol,n)*(1-exp(-(wd/abs(w-wc))**1))
        endif
        enddo
        enddo
        endif

c        do ipol=1,3
c        do n=0,2
c        an(ipol,n)=an(ipol,n)*(Ts+273.16)
c        enddo
c        enddo

        do ipol=1,2
        dtb(ipol)=an(ipol,0)+an(ipol,1)*cosd(phi)
     *                  +an(ipol,2)*cosd(2*phi)
        dtb(ipol)=dtb(ipol)*(273.16+Ts)
        enddo
        dtv=dtb(1)
        dth=dtb(2)

        return 
        end
C***********************************************************
        subroutine tbdelta(jbeam,w,kpol,a0,a1,a2)
c This routine provides the An for the radiometer V, H, and third Stokes for three Aquarius antenna beams

c input:
c jbeam: beam number
c w: wind speed in m/s
c ipol: 1 for V, 2 for H and 3 for third Stokes
c output:
c a1: A1 
c a2: A2 

        integer jbeam
        real dtv,dth,w
        real table(0:30,27)
        common /aqdtdwtable/table
        common /aqradan/an

        real dtb(3),an(3,0:2)

        phi=0
        Ts=15
        call aqdtbw(jbeam,w,phi,Ts,dtv,dth)

        a0=an(kpol,0)
        a1=an(kpol,1)
        a2=an(kpol,2)
c       write(*,*)w,kpol,(an(kpol,n),n=0,2)

        return
        end
C***********************************************************
        subroutine deltatbwdir(jbeam,w,phi,ipol,tb)
c This routine provides the Tb for the radiometer V, H, and third Stokes
c at 45 degree incidence angles.

c input:
c jbeam: beam number
c w: wind speed in m/s
c phi: relative angle (phiw-phir) between the wind and look directions in deg
c ipol: 1 for TV, 2 for TH and 3 for U 
c output:
c tb: tb 
        integer ipol
        real w,phi
        real tb,tb0

        call tbdelta(jbeam,w,ipol,a0,a1,a2)
        if( ipol .le. 2) then
        delta=a1*cosd(phi)+a2*cosd(2*phi)
        else
        delta=a1*sind(phi)+a2*sind(2*phi)
        endif
        tb=delta

        return
        end
CC***********************************************************
        subroutine dtbias(jbeam,dtvbias,dthbias)

c input:
c       jbeam: beam number
c output:
c       dtvbias: bias correction for v-pol in K
c       dthbias: bias correction for h-pol in K
C
        integer jbeam
        real dtvbias,dthbias
        real tbbias
        common /aqtbbias/tbbias(6)

c        dtvbias=tbbias(jbeam)
c        dthbias=tbbias(jbeam+3)

        if(jbeam .eq. 1) then
        dtvbias=0.0
        dthbias=0.0
        else if(jbeam .eq. 2) then
        dtvbias=0.0
        dthbias=0.0
        else if(jbeam .eq. 3) then
        dtvbias=0.0
        dthbias=0.0
        endif

        return
        end
CC***********************************************************
        subroutine dielectricmodelbias(model,jbeam,dtvbias,dthbias)

c This routine introduces additional bias correction to reduce the SSS retrieval bias if KS model is uesd. The calibration bias in the suboutine dtbias was tuned for the Meissner-Wentz model.

c input:
c       model: 1 for KS and 4 for MW dielectric constant model
c       jbeam: beam number
c output:
c       dtvbias: bias correction for v-pol in K
c       dthbias: bias correction for h-pol in K
C
        integer jbeam
        real dtvbias,dthbias
        real tbbias

        if( model .eq. 1 )then

        if(jbeam .eq. 1) then
        dtvbias=0.18
        dthbias=0.17
        else if(jbeam .eq. 2) then
        dtvbias=0.15
        dthbias=0.07
        else if(jbeam .eq. 3) then
        dtvbias=0.1
        dthbias=0.0
        endif

        else
        dtvbias=0.0
        dthbias=0.0
        endif

        return
        end
