C--$Header: /mnt/caretr2/scatsim/CVSROOT/L1A_to_L2/stats_module.f,v 1.1 2009/10/05 20:25:53 fore Exp $
c*****************************************************************      
      subroutine stats_module( r_vals,     ! input
     $                         i_nval,     ! input
     $                         r_time,     ! input
     $                         r_sig,      ! input
     $                         r_wght,     ! input
     $                         r_mean,     ! output
     $                         r_stdev,    ! output
     $                         r_max,      ! output
     $                         r_min,      ! output
     $                         r_rate,     ! output
     $                         r_maxstep,  ! output 
     $                         i_outliers )! output
c*****************************************************************      
c**   
c**   FILE NAME: stats_module.f
c**   
c**   DATE WRITTEN:
c**   
c**   PROGRAMMER:
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
      integer stdout, stderr, stdin
      parameter( stdout = 6, stderr = 0, stdin = 5 )
c     INPUT VARIABLES
      real*8  r_vals(*) ! array of values to perform stats on
      integer i_nval    ! array size of r_vals
      real*8  r_time    ! temporal span of data (Assumes data are equally spaced in time!)
      real*8  r_sig     ! sigma to use for outlier calculation, and/or to blend with stdev computed
      real*8  r_wght    ! flag and weight value, indicating uses of r_sig and r_stdev
                        ! r_wght < 0 means that r_sig is invalid, use computed r_stdev instead
                        ! r_wght >= 1 means that r_sig remains unmodified
                        ! 0 < r_wght < 1 means to update r_sig with weighted average of r_stdev, 
                        !    with r_wght weight of r_stdev and (1-r_wght) weight of r_sig.
c     OUTPUT VARIABLES
      real*8 r_mean, r_stdev, r_max, r_min  ! standard statistics
      real*8 r_rate ! best fit slope of value vs time
      real*8 r_maxstep ! maximim amplitude changes (positive or negative)
      integer i_outliers(*) ! integer string of length i_nval indicating valid data
     
c     LOCAL VARIABLES

      real*8 r_t, r_sumt, r_sumtsq, r_sumtval  ! used to estimate slope via least squares

      integer i
      integer j
      integer i_num
      real*8  r_sum
      real*8  r_sumsq
      real*8  r_sigthresh

c     DATA STATEMENTS
c     COMMON BLOCKS
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      
      r_sum     = 0.0d0
      r_sumsq   = 0.0d0
      i_num     = 0
      r_max     = -1.d10
      r_min     = +1.d10
      r_maxstep = 0.0d0
      r_sumt    = 0.0d0
      r_sumtsq  = 0.0d0
      r_sumtval = 0.0d0
      
      ! loop 1
      do i = 1, i_nval

         if (i.gt.1) r_maxstep = max( r_maxstep, abs( r_vals(i)-r_vals(i-1) ))
         
         i_num     = i_num + 1
         r_sum     = r_sum + r_vals(i)
         r_sumsq   = r_sumsq + r_vals(i)*r_vals(i)
         r_max     = max( r_max, r_vals(i) )
         r_min     = min( r_min, r_vals(i) )
         r_t       = float(i-1) * r_time(i) / float(i_nval - 1)
         r_sumt    = r_sumt + r_t
         r_sumtsq  = r_sumtsq + r_t*r_t
         r_sumtval = r_sumtval + r_t*r_vals(i)
      enddo
      
      r_mean = r_sum / float(i_num)
      r_stdev = sqrt((r_sumsq - float(i_num)*r_mean*r_mean) / (float(i_num)-1))
      r_rate = (float(i_num)*r_sumtval - r_sumt*r_sum) / (float(i_num)*r_sumtsq - r_sumt*r_sumt)
      
      if (r_wght.lt.0.0) r_sig = r_stdev ! no apriori estimate of sigma
      
      ! loop 2
      r_sigthresh = 5.0d0
      do i = 1, i_nval
        if (abs(r_vals(i)-r_mean) .gt. r_sigthresh*r_sig) then
           !l_outliers(i) = .true.
           i_outliers(i) = 1
        else
           !_outliers(i) = .false.
           i_outliers(i) = 0
        endif
        ! anything else?
      enddo
      
      ! modify r_sig with currently computed standard deviation
      if ((r_wght.gt.0.0d0).and.(r_wght.lt.1.0d0)) then
         r_sig = (1.0d0-r_wght)*r_sig + r_wght*r_stdev
      endif
      
      return
      end