        subroutine fd_land_refl(itype,xlat,xlon,tht,sm, EV, EH, refl)

        implicit none
        

        integer(4) itype

        real(4) tht, xlat,xlon,sm,refl(2),EV,EH

        real(4) acoef,alpha,beta

        real(4) costht, sinsqtht

        real(4) cosd

        complex(4) bcoef,rv,rh, esqrt, permit, permit_alpha,er_r
 
!        data acoef/1.850842/               !from fd_soil_coefs.f assuming freq= 1.413

!        data bcoef/(16.06828,-0.7877967)/ ! from fd_soil_coefs.f assuming freq= 1.413

!        data alpha/0.65/        ! see Ulaby vol3, page 2103

!        data beta/1.09/         ! sand and clay fraction should appear in this definition

        if(itype.lt.1 .or. itype.gt.3) stop 'itype oob in fd_land_refl, pgm stopped'

 

c     this is the ulaby,moore,fumg model, vol 3 page 2103 that sab used.  also see sab doc on o:\aquarius\orbit_simulator\surface_atmos

        if(itype.eq.1) then     !soil above freezing
c   Complete formulation of the Dobson model
c
c   rho_b                   ! g/cm3
c   sand                    ! %
c   clay                    ! %
c   sm                      ! m3/m3
c
c   rho_ss = 2.65           ! g/cm3   Ulaby, vol3, pag2086
c   epsilon_ss = 4.7        ! dielectric contant of solid soil. Ulaby, vol3, pag2086
c   alpha = 0.65            ! Ulaby, vol3, pag2103
c   beta = 1.09 - 0.11*sand + 0.18*clay     ! Ulaby, pag2103
c   epsilon_fw = 78.5766 - i5.585   ! dielectric constant of fresh water at 23C. Ulaby, pag2103 (with freq=1.413)
c
c   permit^alpha = 1 + rho_b/rho_ss*(epsilon_ss^alpha - 1) + sm^beta*(epsilon_fw^alpha - 1)

c           permit_alpha=acoef + bcoef*sm**beta
c       from Ulaby vol3 p2013:
c
c       bcoef = [(epsilon_fw)^alpha - 1]
c       where
c       epsilon_fw (fresh water) = 78.6 - 5.6*i    (see Ulaby, vol3, E.112)
c       This is the value for sst=23C, sss=0, freq=1.413GHz
c
c       acoef = 1 + rho_b/rho_ss*[(espilon_ss)^alpha -1]
c       where
c       rho_b = bulk density of the soil, set to 1.3 g/cm3
c       rho_ss = density of solid soil = 2.65 g/cm3     ! Ulaby, vol3, pag2086
c       epsilon_ss: real part = (1+0.44*rho_ss)^2       ! Ulaby, vol3, pag2086
c       epsilon_ss: imag part = 0                       ! Ulaby, vol3, pag2086
c

c           permit = permit_alpha**(1./alpha) !soil.   Removes the exponent

           call mironov(sm,xlat,xlon,er_r)
           permit = real(er_r)

c   From the dielectric constant to the reflectivities

           costht=cosd(tht)

           sinsqtht=1.-costht*costht

           esqrt=csqrt(permit-sinsqtht)

           rh=(costht-esqrt)/(costht+esqrt)

           rv=(permit*costht-esqrt)/(permit*costht+esqrt)

           refl(1) =rv*conjg(rv)
           refl(2) =rh*conjg(rh) ! dielectric contants are complex, but reflectivities are real.

        endif



c     page 2026 and 2027 give the following for fresh-water ice at 1.4 ghz and t=-20:   permit_ice=(3.15,-0.0005)

c     fig e.26 on page 2062 gives real part of snow to be about 1.73 for snow density of 0.4

c     fig e.28 on page 2065 gives imag part of snow to be 0.23*(imag part of ice)=0.23*0.0005  for snow density of 0.4
c     real part is indicated with epsilon' and the imag part with epsilon"

        if(itype.eq.2)  then    !frozen
           refl(1) = 1-EV
           refl(2) = 1-EH
        endif


        if(itype.eq.3)  then
           permit=(3.,-0.1)     !sea ice

c   From the dielectric constant to the reflectivities

           costht=cosd(tht)

           sinsqtht=1.-costht*costht

           esqrt=csqrt(permit-sinsqtht)

           rh=(costht-esqrt)/(costht+esqrt)

           rv=(permit*costht-esqrt)/(permit*costht+esqrt)

           refl(1) =rv*conjg(rv)
           refl(2) =rh*conjg(rh) ! dielectric contants are complex, but reflectivities are real.
        endif

        
        return
        
        end



 

