       subroutine find_refl_tot(idayjl,irad,xlat,xlon,tht,phir,
     1                          frac_land,frac_ice,percent_frozen,
     2                          sss,surtep,wind,sm,staticdata,refl_tot,tbsur) 
        use static_data_module

        implicit none

        integer(4), intent(in)  :: idayjl,irad
        real(4),    intent(in)  :: xlat,xlon,tht,phir,frac_land,frac_ice
        real(4),    intent(in)  :: sss,surtep,wind,sm,percent_frozen
        type(static_data_struct), intent(in) :: staticdata
        real(4),    intent(out) :: refl_tot(2),tbsur(2)

        real(4) sst,frac_ice_adj,frac_water,frac_snow,frac_land_nosnow,surtep_k
        real(4) refl_water(2),refl_ice(2),refl_land(2),tausq,sm_adj,EV,EH
        real(4) refl_land_fresnel(2), refl_land_rough(2), refl_land_rough_veg(2)
        real(4) gamma_squared, frozen_land, nonfrozen_land,refl_frozen(2), land_roughness
        real(4) gamma, albedo, omega
        real(4) wt(4)
        REAL(4) B1,B2,C1,C2,BRIEF
        real(8) tsum(0:3)
        INTEGER(4) J1,J2,K1,K2,L1,L2
        integer(4) k, j, n, kk, jj, gammasq_cnt, albedo_cnt
        real(4) xlat0, xlon0, gammasq_sum, landrgh_sum, albedo_sum

        ! surtep (degrees C)
        sst=surtep
        if(sst.lt.-2) sst=-2
        if(sst.gt.35) sst=35

        ! surtep_k (degrees K)
        surtep_k = surtep + 273.15
  
        frac_ice_adj=frac_ice
        if(frac_ice.gt.1-frac_land) frac_ice_adj=1-frac_land

        frac_water=1 - frac_land - frac_ice_adj

        refl_water=0
        refl_ice  =0
        refl_land =0
        refl_land_fresnel =0
        refl_land_rough =0
        refl_land_rough_veg =0
        refl_frozen =0

        if(frac_water  .ne.0) then
           call fd_water_refl_exact(tht,sss,sst,wind,phir,staticdata,refl_water) 
        endif

        if(frac_ice_adj.ne.0) call fd_seaice_refl(tht, refl_ice) 

        if (percent_frozen .ne. -999) then
           frozen_land = frac_land * percent_frozen
           nonfrozen_land = frac_land * (1 - percent_frozen)
        else
           frozen_land = 0.0
           nonfrozen_land = frac_land
        endif

        !write (*,*) xlon-floor(xlon),(xlat+90)-floor(xlat+90)
        brief=1*(xlat+89.5)
        j1=int(1+brief)
        j2=j1+1

        b1=j1-brief
        b2=1.-b1          
        if(j1.eq.  0) j1=  1
        if(j2.eq.361) j2=360
 
        brief=1*(xlon-0.5)
        k1=int(1+brief)
        k2=k1+1
        c1=k1-brief
        c2=1.-c1
 
        wt(1)=c1*b1
        wt(2)=c2*b1
        wt(3)=c1*b2
        wt(4)=c2*b2

        tsum=0; n=0
        do j=j1,j2
           do k=k1,k2
              kk=k
              if(kk.eq.   0) kk=360
              if(kk.eq.361) kk=   1
              n=n+1
              tsum(0)=tsum(0) + wt(n)
              tsum(1)=tsum(1) + wt(n)*staticdata%land_EV_tab(kk,j,irad)

              tsum(2)=tsum(2) + wt(n)
              tsum(3)=tsum(3) + wt(n)*staticdata%land_EH_tab(kk,j,irad)
!              write(*,*) j,kk,irad,staticdata%land_EH_tab(kk,j,irad)
           enddo
        enddo
        if(tsum(0).ne.0) EV=tsum(1)/tsum(0)
        if(tsum(2).ne.0) EH=tsum(3)/tsum(2)

!       EV = staticdata%land_EV_tab(floor(xlon),floor(xlat+90),irad)
!        EH = staticdata%land_EH_tab(floor(xlon),floor(xlat+90),irad)

!        if (xlat.le.-60) then
!           write (*,*) 'frozen: ', irad, xlon, xlat, percent_frozen, EV, EH
!        endif

        ! Normal soil
        if (nonfrozen_land .ne. 0) then

c     using 1.01 rather than 1 accounts for possible roundoff error
           sm_adj=sm
           if(sm_adj.gt.1.01) sm_adj=0.35 !fill in missing values, looking at the sm images 0.35 is a typical value for islands
           if(sm_adj.gt.1.)   sm_adj=1. !fixes any roundoff error

            ! computes reflectivities for land
           call fd_land_refl(1,xlat,xlon,tht,sm_adj, EV, EH, refl_land) !1 denotes soil
        else
           refl_land = 0.0
        endif

        if (frozen_land .ne. 0) then
        !computes reflectivities for snow
           call fd_land_refl(2,xlat,xlon,tht,0, EV, EH, refl_frozen) !2 denotes snow
        else
           refl_frozen = 0.0
        endif

!        call fd_tausq(idayjl,xlat,xlon,tht, staticdata%itau, tausq)
!        refl_land = refl_land*0.74*tausq 

        gammasq_sum = 0.0
        landrgh_sum = 0.0
        albedo_sum = 0.0
        gammasq_cnt = 0
        albedo_cnt = 0

        do jj=-10,10,1
           xlat0 = xlat + jj*0.05
           do kk=-10,10,1
              xlon0 = xlon + kk*0.05
              if (xlon0 .lt. 0)   xlon0 = xlon0 + 360
              if (xlon0 .gt. 360) xlon0 = xlon0 - 360
              call fd_gammasq(idayjl,xlat0,xlon0,tht, gamma_squared, land_roughness, albedo)
              if (gamma_squared.ne.-999.0) then
                 gammasq_sum = gammasq_sum + gamma_squared
                 landrgh_sum = landrgh_sum + land_roughness
                 gammasq_cnt = gammasq_cnt + 1
              endif
              albedo_sum = albedo_sum + albedo
              albedo_cnt = albedo_cnt + 1
           enddo
        enddo

        if (gammasq_cnt .gt. 0) then
           gamma_squared = gammasq_sum / gammasq_cnt
           land_roughness = landrgh_sum / gammasq_cnt
        else
           gamma_squared = 1.0
           land_roughness = 0.0
        endif

        omega = albedo_sum / albedo_cnt
        gamma = sqrt(gamma_squared)

!        refl_land = refl_land*exp(-land_roughness)*gamma_squared 
        refl_land_fresnel = refl_land
        refl_land_rough = refl_land_fresnel*exp(-land_roughness)
        refl_land_rough_veg = 1 - (1-omega)*(1-gamma)*(1+gamma*refl_land_rough) - (1-refl_land_rough)*gamma
        refl_land = refl_land_rough_veg

!        write(*,*) tausq, gamma_squared

           ! Change stop to write JMG  09/20/11
           if(minval(refl_land).lt.0 .or. maxval(refl_land).gt.1) write(*,*) 'land refl oob, pgm stopped'

        refl_tot=frac_water*refl_water + frac_ice_adj*refl_ice + nonfrozen_land*refl_land + frozen_land*refl_frozen

        tbsur=frac_water*(1-refl_water)*(sst+273.15) +
     1        frac_ice_adj*(1-refl_ice)*surtep_k + 
     2        nonfrozen_land*(1-refl_land)*surtep_k +
     3        frozen_land*(1-refl_frozen)*surtep_k

!        write(*,300) xlon, xlat, tbsur
300     format(4f10.5)
        return
        end
 
