        SUBROUTINE mironov(mv,xlat,xlon,er_r)

          IMPLICIT NONE

          REAL(4), INTENT(IN)     :: mv
          real(4)   , intent(in)  :: xlat,xlon
          COMPLEX(4), INTENT(OUT) :: er_r

          REAL(4)                 :: f, C, PI, freq
          REAL(4)                 :: nd, kd, mvt, eps0b, eps0u, taub, tauu, sigb, sigu
          REAL(4)                 :: eps0, epsinf, epsb_real, epsu_real, epsb_imag, epsu_imag
          REAL(4)                 :: nb, kb, nu, ku
          REAL(4)                 :: nm, km
          REAL(4)                 :: er_r_real, er_r_imag
          Integer*2               :: first=1
          Real, Dimension(3600)                     :: slt1d
          Real, Dimension(3600,1800)                :: cly2d
          character*320           :: read_ancillary_dir    ! INPUT. Folder with NDVI products
          Integer*4                                 :: i, j
          real(4) xlon180

          save first, cly2d

          if (first.eq.1) then
             call getenv('OCDATAROOT', read_ancillary_dir)
             read_ancillary_dir = read_ancillary_dir(1:lnblnk(read_ancillary_dir)) // '/aquarius/radiometer/soil_moisture'

             open(1,file=read_ancillary_dir(1:lnblnk(read_ancillary_dir))//'/clay_fraction.dat',status='old', &
                  convert='little_endian',form='unformatted',access='direct',recl=14400)
             do i=1,1800
                read(1,rec=i) slt1d
                do j=1,3600
                   cly2d(j,i) = slt1d(j)
                end do
             end do
             close(1)
          endif
          first = 0

          xlon180 = xlon
          if (xlon.gt.180) xlon180 = xlon - 360

          PI = acos(-1.0)
          freq=1.41
          f = freq*1e9                                                                              ! Section IV
          C = cly2d(int((xlon180+180)/0.1)+1,int((90-xlat)/0.1)+1)*100

       !! Mironov's regression expressions based on Curtis, Dobson, and Hallikainen datasets
          nd = 1.634 - 0.539e-2 * C + 0.2748e-4 * C**2                                              ! Eqn 17
          kd = 0.03952 - 0.04038e-2 * C                                                             ! Eqn 18
          mvt = 0.02863 + 0.30673e-2 * C                                                            ! Eqn 19
          eps0b = 79.8 - 85.4e-2 * C + 32.7e-4 * C**2                                               ! Eqn 20
          taub = 1.062e-11 + 3.450e-12 * 1e-2 * C                                                   ! Eqn 21
          sigb = 0.3112 + 0.467e-2 * C                                                              ! Eqn 22
          sigu = 0.3631 + 1.217e-2 * C                                                              ! Eqn 23
          eps0u = 100                                                                               ! Eqn 24
          tauu = 8.5e-12                                                                            ! Eqn 25

       !! Debye relaxation equations for water as a function of frequency
          eps0 = 8.854e-12                                                                          ! Vacuum permittivity
          epsinf = 4.9                                                                              ! Section IV
          epsb_real = epsinf + ( (eps0b - epsinf)/(1 + (2*PI*f*taub)**2) )                          ! Eqn 16
          epsb_imag = (eps0b - epsinf)/(1 + (2*PI*f*taub)**2) * (2*PI*f*taub) + sigb/(2*PI*eps0*f)  ! Eqn 16
          epsu_real = epsinf + ( (eps0u - epsinf)/(1 + (2*PI*f*tauu)**2) )                          ! Eqn 16
          epsu_imag = (eps0u - epsinf)/(1 + (2*PI*f*tauu)**2) * (2*PI*f*tauu) + sigu/(2*PI*eps0*f)  ! Eqn 16

       !! Refractive indices
          nb = 1/sqrt(2.0) * sqrt( sqrt(epsb_real**2 + epsb_imag**2) + epsb_real )                  ! Eqn 14
          kb = 1/sqrt(2.0) * sqrt( sqrt(epsb_real**2 + epsb_imag**2) - epsb_real )                  ! Eqn 14
          nu = 1/sqrt(2.0) * sqrt( sqrt(epsu_real**2 + epsu_imag**2) + epsu_real )                  ! Eqn 14
          ku = 1/sqrt(2.0) * sqrt( sqrt(epsu_real**2 + epsu_imag**2) - epsu_real )                  ! Eqn 14

       !! n(*) are refractive indices, k(*) are normalized attenuation coefficients
       !!   m: moist soil
       !!   d: dry soil
       !!   b: bound soil water (BSW)
       !!   u: unbound (free) soil water (FSW)
          IF (mv <= mvt) THEN
              nm = nd + (nb - 1) * mv                                                               ! Eqn 12
              km = kd + kb * mv                                                                     ! Eqn 13
              er_r_real = nm**2 - km**2                                                             ! Eqn 11
              er_r_imag = 2 * nm * km                                                               ! Eqn 11
          ELSE
              nm = nd + (nb - 1) * mvt + (nu - 1) * (mv - mvt)                                      ! Eqn 12
              km = kd + kb * mvt + ku * (mv - mvt)                                                  ! Eqn 13
              er_r_real = nm**2 - km**2                                                             ! Eqn 11
              er_r_imag = 2 * nm * km                                                               ! Eqn 11
          ENDIF

          er_r = CMPLX(er_r_real,er_r_imag)

          return
        END SUBROUTINE mironov
