#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "hdf5.h"

herr_t read_tecfld(hid_t id, char *name, void *data, hsize_t rank, 
		   hsize_t s[2], hsize_t e[2])
{
  int i;
  hid_t dataset, dataspace, filespace, datatype;
  hsize_t start[2], edge[2], dims[2];
  herr_t status;

  dataset = H5Dopen1(id, name);
  if (dataset == -1) {
    printf("Dataset: %s not found (%d)\n", name, id);
    exit(1);
  }

  // Get datatype
  datatype = H5Dget_type(dataset);

  // Select hyperslab
  filespace = H5Dget_space(dataset);


  // Copy start and edge
  for (i=0; i<2; i++) {
    start[i] = s[i];
    edge[i]  = e[i];
  }

  // Check for out-of-bounds read
  status = H5Sget_simple_extent_dims(filespace, dims, NULL);
  for (i=0; i<rank; i++) {
    if (start[i]+edge[i] > dims[i]) {
      printf("Read to dim (0-based): %d of \"%s\" out of bounds (%d %d)\n", 
	     i, name, (int) (start[i]+edge[i]), (int) dims[i]);
      exit (110);
    }
  }

  status = H5Sselect_hyperslab(filespace, H5S_SELECT_SET, start, NULL,
			       edge, NULL);  

  dataspace = H5Screate_simple(rank, edge, NULL); 

  /*
   * Read the data from the hyperslab.
   */
  status = H5Dread(dataset, datatype, dataspace, filespace,
		   H5P_DEFAULT, data);

  H5Dclose(dataset);
  H5Sclose(dataspace);
  H5Sclose(filespace);

  return(0);
}


int dump_tec_(char *tec_filename, double *tec_out)
{
  int i, j, offset;
  hid_t h5fid, grp0;
  hsize_t start[2], count[2];
  short int tec_short[71*73];
  char *tag[13] = {"00", "02", "04", "06", "08", "10", "12", 
		   "14", "16", "18", "20", "22", "24"};
  char fld[16];

  h5fid =  H5Fopen(tec_filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if ( h5fid < 0) {
    printf("%s not found\n", tec_filename);
    exit(1);
  }

  grp0 = H5Gopen1(h5fid,"/");

  start[0] = 0;
  start[1] = 0;
  count[0] = 71;
  count[1] = 73;

  for (j=0; j<13; j++) {
    strcpy( fld, "tec_xxh");
    memcpy( &fld[4], tag[j], 2);
    read_tecfld(grp0, fld, tec_short, 2, start, count);

    offset = j*71*73;
    for (i=0; i<71*73; i++) {
      tec_out[offset+i] = (double) tec_short[i] / 10;
    }
  }

  H5Gclose(grp0);
  H5Fclose(h5fid);

  return(0);
}
