c*****************************************************************      
      subroutine find_ant_look_and_ant_azi_angle( sc_position, sc_velocity,
     $ yaw_deg, pitch_deg, roll_deg, jbeam, antlook, antazi )
c*****************************************************************      
c**   
c**   FILE NAME: geolocate.f
c**   
c**   DATE WRITTEN: 1/28/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  returns the antenna look angle and 
c**    azimuth angle.  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: Modularization of existing code.
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 sc_position(3) ! units?
      real*8 sc_velocity(3) ! units?
      real*8 yaw_deg            ! must be in degrees...
      real*8 pitch_deg          ! must be in degrees...
      real*8 roll_deg           ! must be in degrees...
      integer jbeam                     ! which beam
c     OUTPUT VARIABLES
      real*8 antlook
      real*8 antazi
c     LOCAL VARIABLES
      real*8 vector_stu(3)    
      real*8 vector_local(3)
      real*8 ypr_matrix(3,3)
      real*8 inst_rotation_matrix(3,3)
      real*8 beam_rot_VPol(3,3)                                 ! Not used...
      real*8 beam_rot_HPol(3,3)      
      real*8 rot2H(3,3)
      real*8 rotation_matrix(3,3)        
      real*8 PI
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug_level/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      PI = 4.0d0 * atan( 1.0d0 )

      vector_local(1) = 0.0d0
      vector_local(2) = 0.0d0
      vector_local(3) = 1.0d0

C--beam_rot_VPol rotates a vector in the local beam VTX vector basis to the
C--global vector basis.  beam_rot_HPol does the same for the local beam HTX 
C--vector basis.
      call compute_beam_rotations(jbeam, beam_rot_VPol, beam_rot_HPol)
C--inst_rotation_matrix rotates a vector in the global vector basis to the
C--instrument vector basis.
      call compute_instrument_rotation( inst_rotation_matrix )
C--ypr_matrix rotates a vector in the instrument vector basis to a
C--geocentric (s,t,u) vector basis.
      call compute_attitude_rotation_matrix_rhs( roll_deg, pitch_deg, yaw_deg, ypr_matrix )      
      
      call matrix_multiply( inst_rotation_matrix, beam_rot_HPol, rot2H )
      call matrix_multiply( ypr_matrix, rot2H, rotation_matrix )
      
      call matrix_vector_multiply( rotation_matrix, vector_local, vector_stu )
      
      antlook = acos( vector_stu(3) ) * 180.0d0 / PI
      antazi  = atan2( vector_stu(1), vector_stu(2) ) * 180.0d0 / PI
      antazi  = atan( vector_stu(1) / vector_stu(2) ) * 180.0d0 / PI
      end

c*****************************************************************      
      subroutine convert_to_rect_basis( vector_in, sc_position, sc_velocity,
     $ yaw_deg, pitch_deg, roll_deg, jbeam, vector_rect )
c*****************************************************************      
c**   
c**   FILE NAME: geolocate.f
c**   
c**   DATE WRITTEN:  1/28/2009
c**   
c**   PROGRAMMER:  Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  Modularization of existing code.
c**
c**   ROUTINES CALLED: compute_beam_rotations, compute_instrument_rotation,
c**    compute_attitude_rotation_matrix, compute_local_coordinates,
c**    compute_sc_nadir_components, compute_geodetic_to_geocentric_matrix,
c**    matrix_multiply, matrix_vector_multiply, convert_local_to_rect_system
c**   
c**   NOTES:  Takes a vector in the HTX beam local vector
c**    basis and converts it to a geocentric XYZ vector basis. 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 vector_in(3)       ! vector in the local beam vector basis.
      real*8 sc_position(3) ! units?
      real*8 sc_velocity(3) ! units?
      real*8 yaw_deg            ! must be in degrees...
      real*8 pitch_deg          ! must be in degrees...
      real*8 roll_deg           ! must be in degrees...
      integer jbeam                     ! which beam
c     OUTPUT VARIABLES
      real*8 vector_rect(3)
c     LOCAL VARIABLES
      real*8 sc_u(3)
      real*8 sc_t(3)
      real*8 sc_s(3)
      real*8 sc_nadir_vector(3)
      real*8 sc_lon, sc_lat, sc_alt
      real*8 vector_local(3)
      real*8 geo_rotation_matrix(3,3)
      real*8 ypr_matrix(3,3)
      real*8 inst_rotation_matrix(3,3)
      real*8 beam_rot_VPol(3,3)                                 ! Not used...
      real*8 beam_rot_HPol(3,3)
      real*8 rot2H(3,3)
      real*8 rotation_matrix1(3,3)
      real*8 rotation_matrix(3,3)
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug_level/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS

C--beam_rot_VPol rotates a vector in the local beam VTX vector basis to the
C--global vector basis.  beam_rot_HPol does the same for the local beam HTX 
C--vector basis.
      call compute_beam_rotations(jbeam, beam_rot_VPol, beam_rot_HPol)
C--inst_rotation_matrix rotates a vector in the global vector basis to the
C--instrument vector basis.
      call compute_instrument_rotation( inst_rotation_matrix )
C--ypr_matrix rotates a vector in the instrument vector basis to a
C--geocentric (s,t,u) vector basis.
      call compute_attitude_rotation_matrix_rhs( roll_deg, pitch_deg, yaw_deg, ypr_matrix )      
      
      call compute_local_coordinates( sc_position, sc_velocity, sc_s, sc_t, sc_u )      
C--Compute the nadir vector.      
      call compute_sc_nadir_components( sc_position, sc_lon, sc_lat, sc_alt, sc_nadir_vector)

      call compute_geodetic_to_geocentric_matrix( sc_s, sc_t, sc_u, 
     $ sc_position, sc_velocity, sc_nadir_vector, geo_rotation_matrix )

C--Compute the product of all the rotation matricies.          
      call matrix_multiply( inst_rotation_matrix, beam_rot_HPol, rot2H)
      call matrix_multiply( geo_rotation_matrix, ypr_matrix, rotation_matrix1)
      !call matrix_multiply( rotation_matrix1, rot2H, rotation_matrix )
      call matrix_multiply( ypr_matrix, rot2H, rotation_matrix )
C--Apply the matrix to vector_in, producing vector_local.
      call matrix_vector_multiply( rotation_matrix, vector_in, vector_local )
C--Convert to a geocentric XYZ vector basis.    
      call convert_local_to_rect_system(sc_s, sc_t, sc_u, vector_local, vector_rect)
      end

c*****************************************************************      
      subroutine geolocate( vector_in, sc_position, sc_velocity,
     $ yaw_deg, pitch_deg, roll_deg, jbeam, earth_position,
     $ slant_range, good_solution )
c*****************************************************************      
c**   
c**   FILE NAME: geolocate.f
c**   
c**   DATE WRITTEN: 1/28/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  Given an input vector in the local
c**    HTX beam vector basis, we project it onto earth and find the
c**    position on earth in an geocentric XYZ coordinate system.
c**    convert_to_rect_basis does the dirty work.
c**
c**   ROUTINES CALLED: convert_to_rect_basis, locate_cell_on_earth
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 vector_in(3)       ! vector in the local beam vector basis.
      real*8 sc_position(3) ! units?
      real*8 sc_velocity(3) ! units?
      real*8 yaw_deg            ! must be in degrees...
      real*8 pitch_deg          ! must be in degrees...
      real*8 roll_deg           ! must be in degrees...
      integer jbeam                     ! which beam
c     OUTPUT VARIABLES
      real*8 earth_position(3)
      real*8 slant_range
      logical good_solution ! set to false if no real solution exists. (quadratic equation)
c     LOCAL VARIABLES
      real*8 vector_rect(3)
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug_level/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      call convert_to_rect_basis( vector_in, sc_position, sc_velocity,
     $ yaw_deg, pitch_deg, roll_deg, jbeam, vector_rect )
C--Solve for the location of this point on Earth.      
      call locate_cell_on_earth(sc_position, vector_rect, earth_position, slant_range, good_solution)  

      ! Require range to be positive; don't find solutions looking behind the antenna!
      ! AGF 03/30/2011
      if( slant_range .lt. 0 ) good_solution = .false.
    
      end
