c*****************************************************************      
      subroutine get_ice_mask( julian_time, ylat, xlon, nll, imask )
c*****************************************************************      
c**   
c**   FILE NAME: get_ice_mask.f
c**   
c**   DATE WRITTEN: 1/13/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: JD2CAL, read_wgrib_dump_file
c**
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
      integer N_LAT_MAX, N_LON_MAX
      parameter (N_LAT_MAX=12*180)
      parameter (N_LON_MAX=12*360)
      integer, parameter :: DFACC_READ = 1

c     INPUT VARIABLES
      real*8 julian_time
      integer nll
      real*4 ylat(nll), xlon(nll)
c     OUTPUT VARIABLES
      real*4 imask(*)
c     LOCAL VARIABLES
      integer i_year, i_month, i_day, i_hour, i_minute, i_second
      integer i_year_last, i_month_last, i_day_last
      integer*4 n_lat, n_lon
      real*8 r_frac
      character*600 str_system_exec
      integer*1 mask(N_LON_MAX,N_LAT_MAX)
      real*4 lat(N_LAT_MAX), lon(N_LON_MAX)
      character*320 ice_mask_dump_file
      integer i_lat, i_lon, ii, jj, i_lon_closest, i_lat_closest, ierr
!      integer(kind=1) ierr_system
      integer(kind=4) ierr_system ! JMG 11/03/09
      real*4 min_delta
      real*4 lon_inp

C     HDF4 routines
      integer sfstart, sfrdata, sfn2index, sfselect
      integer sd_id_anc, sds_id, sds_index
      integer sfendacc, sfend
      integer start(2), count(2), stride(2)
      integer retn 
      integer leap, i_doy

      integer doy_off0(12)
      integer doy_off1(12)
      data doy_off0 /0,31,59,90,120,151,181,212,243,273,304,334/
      data doy_off1 /0,31,60,91,121,152,182,213,244,274,305,335/

      character*200 ancfile

c     DATA STATEMENTS
      data i_year_last/-1/
      data i_month_last/-1/
      data i_day_last/-1/
c     COMMON BLOCKS
      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, 
     $ AQ_SCAT_SIM_DIR_SCRATCH
      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, 
     $ AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
c     FUNCTION STATEMENTS
      integer lnblnk
c     PROCESSING STEPS
      save
      
C--Convert the julian_time to YYYY MM DD integers.
      call JD2CAL(julian_time,i_year,i_month,i_day,i_hour,i_minute,
     $ i_second,r_frac)

      leap = 0
      if (i_year/4.0.eq.i_year/4)     leap = 1
      if (i_year/100.0.eq.i_year/100) leap = 0
      if (i_year/400.0.eq.i_year/400) leap = 1

      if (leap.eq.0) i_doy = doy_off0(i_month) + i_day
      if (leap.eq.1) i_doy = doy_off1(i_month) + i_day

C--2007 ice mask does not have entries for day 2 and 3 of Jan.
!      if( i_year.eq.2007 .and. i_month .eq.1 .and. (i_day.eq.2 .or. i_day.eq.3))then
!        i_day = 1
!      endif

C--Load new mask file for each new day.      
      if( i_year_last.ne.i_year .or. i_month_last.ne.i_month
     $ .or. i_day_last.ne.i_day )then
        write(6,*) 'Loading new ice mask data...'
C-Call my shell script to extract the desired ice mask from
C--data files assumed to exist in AQ_SCAT_SIM_DIR_DAT/seaice directory.
!900     format('geticemask.sh -y ',I4,' -m ',
!     $ I2,' -d ',I2,' -s ',a,' -p ',a)   
!        write(str_system_exec,900) 
!     $ i_year, i_month, i_day,
!     $ AQ_SCAT_SIM_DIR_SCRATCH(1:lnblnk(AQ_SCAT_SIM_DIR_SCRATCH)),
!     $ AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT))    
!        write(6,*) str_system_exec(1:lnblnk(str_system_exec))

C--Check if script failed.
!        call system(  str_system_exec, ierr_system )
!        write(6,*) 'ierr: ', ierr_system
        
!        if( ierr_system .eq. 256 )then
!          write(6,*) 'Script: geticemask.sh failed'
!          write(6,*) 'you probably are missing some sea ice files'
!          write(6,*) 'or your config is not right...'
!          stop
!        endif
        
!        if( ierr_system .eq. 512)then
!          write(6,*) 'Cannot find a record for this particular day!'        
!          if( i_day_last .ne. -1)then
!            write(6,*) 'Using previously loaded ice mask!'
!            goto 901
!          else
!            write(6,*) 'Quitting!'
!            stop
!           endif
!        endif

C--Script dumps out the ice mask in binary at AQ_SCAT_SIM_DIR_SCRATCH/ice_mask_dump
C--as well as a small text header file at AQ_SCAT_SIM_DIR_SCRATCH/ice_mask_dump.hdrs
!        ice_mask_dump_file = AQ_SCAT_SIM_DIR_SCRATCH(1:
!     $   lnblnk(AQ_SCAT_SIM_DIR_SCRATCH)) // '/ice_mask_dump'

C--Call subroutine to read in the wgrib binary dump file.
!        call read_wgrib_dump_file( ice_mask_dump_file, lat, lon, n_lat, 
!     $   n_lon, mask )



        write(ancfile, 904) i_year, i_doy
        if (i_doy.lt.100) write(ancfile, 903) i_year, i_doy
        if (i_doy.lt.10)  write(ancfile, 902) i_year, i_doy
 902    format('N',I4,'00',I1,'00_SEAICE_NCEP_24h.hdf')
 903    format('N',I4, '0',I2,'00_SEAICE_NCEP_24h.hdf')
 904    format('N',I4,     I3,'00_SEAICE_NCEP_24h.hdf')


        sd_id_anc = sfstart( ancfile, DFACC_READ)

        start(1) = 0
        start(2) = 0
        stride(1) = 1
        stride(2) = 1
        count(1) = N_LON_MAX
        count(2) = N_LAT_MAX

C       sea ice mask is byte from 0-100

        sds_index = sfn2index(sd_id_anc, 'seaice')
        sds_id = sfselect (sd_id_anc, sds_index)
        retn = sfrdata(sds_id, start, stride, count, mask)
        retn = sfendacc(sds_id)
        retn = sfend(sd_id_anc) 

        n_lon = N_LON_MAX
        n_lat = N_LAT_MAX

901     continue
C--End of code to load in the ice mask file.     
C--Keep track of the YYYYMMDD integers so we don't need to reload
C--ice mask file every time we are called.
        i_year_last  = i_year
        i_month_last = i_month
        i_day_last   = i_day
      endif


C--Find closest lat, lon points, and return the values of the mask at the
C--closest point.
      do jj=1,nll
        if( xlon(jj) .lt. 0 )then
          lon_inp = xlon(jj) + 360.0
        else
          lon_inp = xlon(jj)
        endif
        i_lon_closest = 1 + nint( (n_lon-1) * lon_inp / 360.0 ) 
        i_lat_closest = 1 + nint( (n_lat-1) * (90.0-ylat(jj)) / 180.0 ) 
        imask( jj ) = real(mask(i_lon_closest,i_lat_closest))/100.0
        !write(6,*) '(lon,lat) specified:',lon_inp,ylat(jj)
        !write(6,*) 'i_lon_closest: ',i_lon_closest
        !write(6,*) 'i_lat_closest: ',i_lat_closest    
        !write(6,*) '(lon,lat): ',lon(i_lon_closest),lat(i_lat_closest)
      enddo      
      end

c*****************************************************************      
      subroutine read_wgrib_dump_file( ice_mask_file, lat, lon, 
     $ n_lat, n_lon, mask )
c*****************************************************************      
c**   
c**   FILE NAME: get_ice_mask.f
c**   
c**   DATE WRITTEN: 1/13/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED:  fsread_seq
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
      integer N_LAT_MAX, N_LON_MAX
      parameter (N_LAT_MAX=12*180)
      parameter (N_LON_MAX=12*360)
c     INPUT VARIABLES
      character*320 ice_mask_file
c     OUTPUT VARIABLES
      integer*1 mask(N_LON_MAX,N_LAT_MAX)
      real*4 lat(N_LAT_MAX), lon(N_LON_MAX)      
      integer*4 n_lat, n_lon      
c     LOCAL VARIABLES
      character*320 ice_mask_header_file, str_line, str_system_exec
      integer i_lat, i_lon, k, ierr
      real*4 lat_start, lat_end, lon_start, lon_end
      real*4 buff(N_LAT_MAX*N_LON_MAX)
c     DATA STATEMENTS
c     FUNCTION STATEMENTS
      integer lnblnk
c     PROCESSING STEPS

C--We assume the header file has the same name as the ice_mask_file, but with
C-- a ".hdrs" appended to the name.
910   format(a,'.hdrs')    
      write(ice_mask_header_file,910) ice_mask_file(1:lnblnk( 
     $ ice_mask_file))
      
      !write(6,*) ice_mask_header_file
      
C--Read in the header file.      
      open(50,file=ice_mask_header_file)
      read(50,*) str_line
      read(50,*) lat_start, lat_end, lon_start, lon_end, n_lat, n_lon
      close(50)
      
      !write(6,*) 'lat_start, lat_end, lon_start, lon_end, n_lat, n_lon'
      !write(6,*)  lat_start, lat_end, lon_start, lon_end, n_lat, n_lon

C--NCEP has lon starting at 0 increasing to 180, then wrapping to -180
C--and then increasing to 0.  So I add 360 to lon_end when composing
C--the range of values.  Therefore my lon is in [0,360] and is always
C--increasing without a discontinuity in the middle.
      lon_end = lon_end + 360.0

C--Size of wgrib dump file is n_lat * n_lon * 4 bytes (real*4)      
      open(50,file=ice_mask_file,form='unformatted')
      call fsread_seq(50,buff,n_lat*n_lon*4,ierr)
      close(50)

C--Populate the integer*1 mask.      
      do i_lat=1,n_lat
        do i_lon=1,n_lon
          k = i_lon+(i_lat-1)*n_lon
          if( buff(k) .ge. 0 .and. buff(k) .le. 1.0)then
            mask(i_lon,i_lat) = nint( 100.0 * buff(k) )
          else
            mask(i_lon,i_lat) = 0 ! maybe we should flag odd data?
          endif
        enddo
      enddo        

C--Create the lat and lon vector of values.      
      do i_lat=1,n_lat
        lat( i_lat ) = lat_start + (lat_end-lat_start) * 
     $   real(i_lat-1) / real(n_lat-1)
      enddo
      do i_lon=1,n_lon
        lon( i_lon ) = lon_start + (lon_end-lon_start) * 
     $   real(i_lon-1) / real(n_lon-1)
      enddo

C--Remove the temporary files from the scratch directory.      
      str_system_exec = 'rm ' // ice_mask_header_file(1:lnblnk( 
     $ ice_mask_header_file))
      call system( str_system_exec )
      str_system_exec = 'rm ' // ice_mask_file(1:lnblnk( 
     $ ice_mask_file))     
      call system( str_system_exec )
      
      end
      

C      program test_ice_mask
C      implicit none
C      
C      real*8 julian_time
C      
C      real*4 ylat, xlon, imask
C      integer nll, i_lat, i_lon, n_lat, n_lon
C      character*320 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, 
C     $ AQ_SCAT_SIM_DIR_SCRATCH, string
C      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, 
C     $ AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
C
C
Cc      call getenv('AQ_SCAT_SIM_DIR_OUT',AQ_SCAT_SIM_DIR_OUT)
Cc      call getenv('AQ_SCAT_SIM_DIR_DAT',AQ_SCAT_SIM_DIR_DAT)
Cc      call getenv('AQ_SCAT_SIM_DIR_SCRATCH',AQ_SCAT_SIM_DIR_SCRATCH)
C      
C      call getarg(1,AQ_SCAT_SIM_DIR_SCRATCH)
C      call getarg(2,AQ_SCAT_SIM_DIR_DAT)
C      call getarg(3,string)
C      read(string,*) xlon
C      call getarg(4,string)
C      read(string,*) ylat
C      
C      
C      call find_julian_day( 2004, 10, 01, julian_time )
C      
C      write(6,*) julian_time
C      
C      nll=1
C      
C      call get_ice_mask( julian_time, ylat, xlon, nll, imask)
C      write(6,*) xlon, ylat, imask
C      
C      
C      n_lat = 180*4
C      n_lon = 360*4
C      
C      do i_lat=1,n_lat
C        ylat = -90.0 + 180.0 * real(i_lat-1) / real(n_lat-1)
C        do i_lon=1,n_lon
C          xlon = 360.0 * real(i_lon-1) / real(n_lon-1)
C          call get_ice_mask( julian_time, ylat, xlon, nll, imask)
C          write(33,*) xlon, ylat, imask
C        enddo
C      enddo
C      end      
