c*****************************************************************      
      subroutine get_ice_mask_Op( julian_time, ylat, xlon, nll, imask )
c*****************************************************************      
c**   
c**   FILE NAME: get_ice_mask_Op.f
c**   
c**   DATE WRITTEN: 1/13/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: JD2CAL, read_wgrib_dump_file
c**
c**   NOTES: 
c**
c**   UPDATE LOG: Modified for operational simulation
c**    May 2010.
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
      integer N_LAT_MAX, N_LON_MAX
      parameter (N_LAT_MAX=12*180)
      parameter (N_LON_MAX=12*360)
c     INPUT VARIABLES
      real*8 julian_time, jd_tmp
      integer nll
      real*4 ylat(nll), xlon(nll)
c     OUTPUT VARIABLES
      real*4 imask(*)
c     LOCAL VARIABLES
      integer i_year, i_month, i_day, i_hour, i_minute, i_second, i_doy
      integer i_year_last, i_doy_last
      integer counter
      integer*4 n_lat, n_lon
      real*8 r_frac
      integer*1 mask(N_LON_MAX,N_LAT_MAX)
      real*4 lat(N_LAT_MAX), lon(N_LON_MAX)
      character*320 HDF_icemask_file
      integer i_lat, i_lon, ii, jj, i_lon_closest, i_lat_closest, ierr
      integer(kind=1) ierr_system
      real*4 min_delta
      real*4 lon_inp
      logical       file_exists

      character datadir*255
      integer len, lenstr

c     DATA STATEMENTS
      data i_year_last/-1/
      data i_doy_last/-1/
      data counter/1/
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level
      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, 
     $ AQ_SCAT_SIM_DIR_SCRATCH
      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, 
     $ AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
c     FUNCTION STATEMENTS
      integer lnblnk
c     PROCESSING STEPS
      save
      
      jd_tmp = julian_time
      file_exists            = .false.

C--Convert the julian_time to YYYY MM DD integers.
      call JD2CAL(julian_time,i_year,i_month,i_day,i_hour,i_minute,
     $ i_second,r_frac)
      
      call convert_day_and_month_to_dayofyear( i_year, i_day, 
     $ i_month, i_doy )
      
C--Load new mask file for each new day.      
      if( i_year_last.ne.i_year .or. i_doy_last.ne.i_doy )then
        write(6,*) 'Loading new ice mask data...'

C--Keep track of the YYYYDDD integers so we don't need to reload
C--ice mask file every time we are called.
        i_year_last  = i_year
        i_doy_last   = i_doy

        call getenv('L2GEN_AQUARIUS_ANCILLARY_DIR', datadir)
        len = lenstr(datadir)

900     format(a,'/N',I4.4,I3.3,'00_SEAICE_NCEP_24h.hdf')

        do while( .not.file_exists )
           call JD2CAL(jd_tmp,i_year,i_month,i_day,i_hour,i_minute,i_second,r_frac)
           call convert_day_and_month_to_dayofyear( i_year, i_day, i_month, i_doy )

           write(HDF_icemask_file,900) datadir(1:len), i_year, i_doy
           inquire( file=HDF_icemask_file, exist=file_exists )
           jd_tmp = jd_tmp - 1.0d0 ! Check previous day
        enddo

        if( i_dbg_level .ge. 2 )then
           write(6,*) 'year, doy: ', i_year, i_doy
           write(6,*) 'icemask file: ', HDF_icemask_file
        endif
        
C--Call subroutine to read in the wgrib binary dump file.
        write(35,'((a),i1,(a),(a))') 
     $       'SEAICEFILE', counter, '=', 
     $       HDF_icemask_file(scan(HDF_icemask_file, '/', back=.true.)+1:lnblnk(HDF_icemask_file))
        call read_HDF_seaice( HDF_icemask_file, lat, lon, n_lat, n_lon, mask )
        counter = counter + 1
C--End of code to load in the ice mask file.     
      endif


C--Find closest lat, lon points, and return the values of the mask at the
C--closest point.
      do jj=1,nll
        ! shift by +180 degrees since ice map starts at -180 deg not at
        ! prime meridian. 12/10/2010 AGF
        lon_inp = xlon(jj) + 180.0d0
        
        if( lon_inp .ge. 360.0 )then
          lon_inp = lon_inp - 360.0
        endif

        i_lon_closest = 1 + nint( (n_lon-1) * lon_inp / 360.0 ) 
        i_lat_closest = 1 + nint( (n_lat-1) * (90.0-ylat(jj)) / 180.0 )

        if( i_lon_closest.lt.1 .or. i_lon_closest.gt.n_lon .or.
     $      i_lat_closest.lt.1 .or. i_lat_closest.gt.n_lat ) then
          write(6,*) 'WARNING: get_ice_mask_Op: lon, lat out of range: ', xlon(jj), ylat(jj)
          imask( jj ) = 0
        else
          imask( jj ) = real(mask(i_lon_closest,i_lat_closest))/100.0
        endif
        !write(6,*) '(lon,lat) specified:',lon_inp,ylat(jj)
        !write(6,*) 'i_lon_closest: ',i_lon_closest
        !write(6,*) 'i_lat_closest: ',i_lat_closest    
        !write(6,*) '(lon,lat): ',lon(i_lon_closest),lat(i_lat_closest)
      enddo      
      end

c*****************************************************************      
      subroutine read_HDF_seaice( HDF_icemask_file, 
     $                            lat,
     $                            lon, 
     $                            n_lat, 
     $                            n_lon, 
     $                            mask )
c*****************************************************************      
c**   
c**   FILE NAME: get_ice_mask_Op.f
c**   
c**   DATE WRITTEN: 5/17/2010
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED:         stop

c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
      include 'hdf.inc'
c     PARAMETER STATEMENTS
      integer stdout, stderr, stdin
      parameter( stdout = 6, stderr = 0, stdin = 5 )
      integer N_LAT_MAX, N_LON_MAX
      parameter (N_LAT_MAX=12*180)
      parameter (N_LON_MAX=12*360)
c     INPUT VARIABLES
      character*320 HDF_icemask_file
c     OUTPUT VARIABLES
      integer*1 mask(N_LON_MAX,N_LAT_MAX)
      real*4 lat(N_LAT_MAX), lon(N_LON_MAX)      
      integer*4 n_lat, n_lon    
c     LOCAL VARIABLES
      integer*4    sd_id
      integer*4    sds_id
      integer      ierr
      integer      start(2)
      integer      stride(2)
      integer      edge(2)
      integer      num_datasets
      integer      num_global_attrs
      integer      rank
      integer      data_type
      integer      num_attrs
      integer      dimsizes( 32 )
      character*64 sds_name
      integer      sds_index
      
      integer ii
      integer i_len
      
      integer i_lat, i_lon
      real*8 lat_start, lat_end
      real*8 lon_start, lon_end
      
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug/ i_dbg_level
c     FUNCTION STATEMENTS
      integer sfstart
      integer sffinfo
      integer sfselect
      integer sfginfo
      integer sfrdata
      integer sfendacc
      integer sfend
      integer sfn2index
      integer lnblnk
c     PROCESSING STEPS
      
      start(1)  = 0
      start(2)  = 0
      
      stride(1) = 1
      stride(2) = 1
      
      edge(1)   = 4320
      edge(2)   = 2160
      
      sd_id     = sfstart( HDF_icemask_file, DFACC_RDONLY )
      if( sd_id .eq. -1 ) then
        write(6,*) 'ERROR: Unable to open HDF4 file: ', 
     $   HDF_icemask_file(1:lnblnk(HDF_icemask_file))
        call exit(1)
      endif
      
      ierr      = sffinfo( sd_id, num_datasets, num_global_attrs ) 
      sds_index = sfn2index( sd_id, 'seaice' )
      if( sds_index .eq. -1 ) then
        write(6,*) 'ERROR: Unable to find seaice in HDF4 file: ', 
     $   HDF_icemask_file(1:lnblnk(HDF_icemask_file))
        call exit(1)
      endif
      
      sds_id    = sfselect(sd_id, sds_index)
      if( sds_id .eq. -1 ) then
        write(6,*) 'ERROR: Unable to select seaice in HDF4 file: ', 
     $   HDF_icemask_file(1:lnblnk(HDF_icemask_file))
        call exit(1)
      endif
      
      ierr      = sfginfo( sds_id, sds_name, rank, dimsizes, data_type, num_attrs )
      i_len     = lnblnk( sds_name )
      
      !if( i_dbg_level .ge. 2 ) then
        write(6,*) 'sds_name, rank, dimsizes, data_type, num_attrs: ',
     $              sds_name(1:i_len), rank, (dimsizes(ii),ii=1,rank), 
     $              data_type, num_attrs
      !endif
      
      ierr   = sfrdata( sds_id, start, stride, edge, mask )
      if( ierr .eq. -1 ) then
        write(6,*) 'ERROR: Unable to read seaice in HDF4 file: ', 
     $   HDF_icemask_file(1:lnblnk(HDF_icemask_file))
        call exit(1)
      endif
      
      ierr   = sfendacc(sds_id)
      if( ierr .eq. -1 ) then
        write(6,*) 'ERROR: Unable to close sds seaice in HDF4 file: ', 
     $   HDF_icemask_file(1:lnblnk(HDF_icemask_file))
        call exit(1)
      endif 
      
      ierr   = sfend(sd_id)
      if( ierr .eq. -1 ) then
        write(6,*) 'ERROR: Unable to close HDF4 file: ', 
     $   HDF_icemask_file(1:lnblnk(HDF_icemask_file))
        call exit(1)
      endif
      
      n_lon   = edge(1)
      n_lat   = edge(2)
      
      lat_start =   90.0d0 - 1.0d0 / 24.0d0
      lat_end   =  -90.0d0 + 1.0d0 / 24.0d0
      lon_start = -180.0d0 + 1.0d0 / 24.0d0
      lon_end   =  180.0d0 - 1.0d0 / 24.0d0      
      
      do i_lat = 1, n_lat
        lat( i_lat ) = lat_start + ( lat_end - lat_start ) *
     $   real( i_lat - 1 ) / real( n_lat - 1 )
      enddo
      
      do i_lon = 1, n_lon
        lon( i_lon ) = lon_start + ( lon_end - lon_start ) *
     $   real( i_lon - 1 ) / real( n_lon - 1 )
      enddo
      
      end


C      program test_ice_mask
C      implicit none
C      
C      real*8 julian_time
C      
C      real*4 ylat, xlon, imask
C      integer nll, i_lat, i_lon, n_lat, n_lon
C      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, 
C     $ AQ_SCAT_SIM_DIR_SCRATCH, string
C      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, 
C     $ AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
C      
C      call getarg(1,AQ_SCAT_SIM_DIR_DAT)
C      call getarg(2,string)
C      read(string,*) xlon
C      call getarg(3,string)
C      read(string,*) ylat
C      
C      
C      call find_julian_day( 2010, 5, 10, julian_time )
C      
C      write(6,*) julian_time
C      
C      nll=1
C      
C      call get_ice_mask_Op( julian_time, ylat, xlon, nll, imask)
C      write(6,*) 'iii:', xlon, ylat, imask
C      pause
C      
C      n_lat = 180*2
C      n_lon = 360*2
C      
C      do i_lat=1,n_lat
C        ylat = -90.0 + 180.0 * real(i_lat-1) / real(n_lat-1)
C        do i_lon=1,n_lon
C          xlon = 360.0 * real(i_lon-1) / real(n_lon-1)
C          call get_ice_mask_Op( julian_time, ylat, xlon, nll, imask)
C          write(33,*) xlon, ylat, imask
C        enddo
C      enddo
C      end      
