c*****************************************************************      
      subroutine get_ionex_tec( lat, lon, julian_time, tec )
c*****************************************************************      
c**   
c**   FILE NAME: get_ionex_tec.f
c**   
c**   DATE WRITTEN: 5/27/2009
c**   
c**   PROGRAMMER:  Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 julian_time
      real*8 lat
      real*8 lon
c     OUTPUT VARIABLES
      real*8 tec
c     LOCAL VARIABLES
      real*8 MJD
      
      integer*4 i_year  ! Year
      integer*4 i_month ! Monday of year
      real*8    r_day   ! floating-point day-of-month
      
      real*8 MJD_1      /-1.d0/         ! modified julian time of TEC map 1
      real*8 MJD_2      /-1.d0/         ! modified julian time of TEC map 2
      real*8 tec_map_1(71*73)   ! TEC map 1
      real*8 tec_map_2(71*73)  ! TEC map 2
      real*8 lat_start, lat_end, dlat, lon_start, lon_end, dlon ! TEC map coordinates
      
      real*8 lon_minus, lon_plus
      real*8 tec_minus, tec_plus
      
      integer*4 i_lat, i_lon
      integer*4 n_lat, n_lon
      
      
c     DATA STATEMENTS
c     COMMON BLOCKS
      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH

      integer i_dbg_level
      common /debug_level/ i_dbg_level            
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      save MJD_1, MJD_2, tec_map_1, tec_map_2, lat_start, lat_end, dlat, 
     $ lon_start, lon_end, dlon
      
      MJD = julian_time - 2400000.5d0
      
      call JMT( MJD, i_year, i_month, r_day)
     
      if( i_dbg_level.ge.2)then
        write(6,*) ' '
        write(6,*) 'In subroutine get_ionex_tec: '
        write(6,*) 'julian_time, MJD, MJD_1, MJD_2: ',julian_time,MJD, MJD_1, MJD_2            
      endif
C--Only reload from IONEX files if we have passed outside of the 
C--bracketed time interval [MJD_1,MJD_2].
      if( MJD .ge. MJD_2 )then
        write(6,*) 'time to load new IONEX TEC maps!.'
        call get_ionex_maps( MJD, MJD_1, tec_map_1, MJD_2, 
     $ tec_map_2, lat_start, lat_end, dlat, lon_start, lon_end,
     $ dlon )
        if( i_dbg_level.ge.2)then
          write(6,*) 'After reloading'
          write(6,*) 'MJD, MJD_1, MJD_2: ',MJD, MJD_1, MJD_2            
        endif         
      endif
      
  
      
      n_lat  = int((lat_end-lat_start)/dlat) + 1
      n_lon  = int((lon_end-lon_start)/dlon) + 1
      
C--Find (lat,lon) coordinates in TEC maps, corresponding to a rotation
C--of the requested (lat,lon) coordinates by (MJD-MJD_1)*360.0d0
C--degrees.
      lon_minus = lon + (MJD-MJD_1)*360.0d0
      lon_plus  = lon + (MJD-MJD_2)*360.0d0
      
      if( lon_minus.lt.-180.0d0 ) lon_minus = lon_minus + 360.0d0
      if( lon_minus.gt.180.0d0 )  lon_minus = lon_minus - 360.0d0
      if( lon_plus.lt.-180.0d0 )  lon_plus = lon_plus + 360.0d0
      if( lon_plus.gt.180.0d0 )   lon_plus = lon_plus - 360.0d0
      
      if( i_dbg_level.ge.2)then
        write(6,*) ' '
        write(6,*) 'In subroutine get_ionex_tec with lat, lon, MJD, MJD_1, MJD_2: ',lat,lon,MJD, MJD_1, MJD_2
        write(6,*) 'lat_start, lat_end, dlat, lon_start, lon_end, dlon',lat_start, lat_end, dlat, lon_start, lon_end, dlon
        write(6,*) 'lon, lat, lon_minus, lon_plus', lon, lat, lon_minus, lon_plus
      endif
C--Interpolate TEC map from MJD_1 to coordinate (lat_minus,lon)            
      i_lon = int((lon_minus-lon_start)/dlon) + 1
      i_lat = int((lat-lat_start)/dlat) + 1

C--Deal with boundaries of IONEX TEC map
      if(i_lon.lt.1) i_lon=1
      if(i_lat.lt.1) i_lat=1
      if(i_lon.ge.n_lon) i_lon=n_lon-1
      if(i_lat.ge.n_lat) i_lat=n_lat-1    

      call bilinear_interp( lon_start+(i_lon-1)*dlon,
     $ lon_start+i_lon*dlon,lat_start+(i_lat-1)*dlat,
     $ lat_start+i_lat*dlat,
     $ tec_map_1(i_lon   +n_lon*(i_lat-1)),
     $ tec_map_1(i_lon   +n_lon*(i_lat-1+1)),
     $ tec_map_1(i_lon+1 +n_lon*(i_lat-1)),
     $ tec_map_1(i_lon+1 +n_lon*(i_lat-1+1)),
     $ lon_minus, lat, tec_minus )

      if( i_dbg_level.ge.2)then
        write(6,*) 'In subroutine get_ionex_tec'
        write(6,*) 'tec_map_1(i_lon,i_lat): ',tec_map_1(i_lon   +n_lon*(i_lat-1))
        write(6,*) 'tec_map_1(i_lon+1,i_lat): ',tec_map_1(i_lon+1   +n_lon*(i_lat-1))
        write(6,*) 'tec_map_1(i_lon,i_lat+1): ',tec_map_1(i_lon   +n_lon*(i_lat-1+1))
        write(6,*) 'tec_map_1(i_lon+1,i_lat+1): ',tec_map_1(i_lon+1   +n_lon*(i_lat+1-1))
      endif

C--Interpolate TEC map from MJD_2 to coordinate (lat_plus,lon)      
      i_lon = int((lon_plus-lon_start)/dlon) + 1
      i_lat = int((lat-lat_start)/dlat) + 1

C--Deal with boundaries of IONEX TEC map      
      if(i_lon.lt.1) i_lon=1
      if(i_lat.lt.1) i_lat=1
      if(i_lon.ge.n_lon) i_lon=n_lon-1
      if(i_lat.ge.n_lat) i_lat=n_lat-1
      
      call bilinear_interp( lon_start+(i_lon-1)*dlon,
     $ lon_start+i_lon*dlon,lat_start+(i_lat-1)*dlat,
     $ lat_start+i_lat*dlat,
     $ tec_map_2(i_lon   +n_lon*(i_lat-1)),
     $ tec_map_2(i_lon   +n_lon*(i_lat-1+1)),
     $ tec_map_2(i_lon+1 +n_lon*(i_lat-1)),
     $ tec_map_2(i_lon+1 +n_lon*(i_lat-1+1)),
     $ lon_plus, lat, tec_plus ) 
            
      call lininterp1d( MJD_1, MJD_2, tec_minus, tec_plus, MJD, tec )
      
      if( i_dbg_level.ge.2)then
        write(6,*) 'tec_map_2(i_lon,i_lat): ',tec_map_2(i_lon   +n_lon*(i_lat-1))
        write(6,*) 'tec_map_2(i_lon+1,i_lat): ',tec_map_2(i_lon+1   +n_lon*(i_lat-1))
        write(6,*) 'tec_map_2(i_lon,i_lat+1): ',tec_map_2(i_lon   +n_lon*(i_lat-1+1))
        write(6,*) 'tec_map_2(i_lon+1,i_lat+1): ',tec_map_2(i_lon+1   +n_lon*(i_lat+1-1))
        write(6,*) 'tec, tec_minus, tec_plus: ',tec,tec_minus,tec_plus
      endif    
      
      end
      
      
c*****************************************************************      
      subroutine get_ionex_maps( MJD, MJD_1, tec_map_1, MJD_2, 
     $ tec_map_2, lat_start, lat_end, dlat, lon_start, lon_end,
     $ dlon )
c*****************************************************************      
c**   
c**   FILE NAME: get_ionex_tec.f
c**   
c**   DATE WRITTEN:  5/27/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     5/19/2010        Modified to use igrg files, with differnt
c**                      directory structure (i.e. generation of 
c**                      full path to IONEX files).  AGF
c**     3/25/2011        Modified to use CODE files  JMG
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
      integer*4 MAXVAL          ! MAXIMUM NUMBER OF TEC/RMS VALUES    I*4
      integer*4 MAXADT          ! MAXIMUM NUMBER OF AUX RECORDS      
      parameter( MAXVAL = 67379 )
      parameter( MAXADT = 50 )
c     INPUT VARIABLES
      real*8 MJD
c     OUTPUT VARIABLES
      real*8 MJD_1
      real*8 MJD_2
      
      real*8 tec_map_1(MAXVAL), tec_map_2(MAXVAL)
      real*8 lat_start, lat_end, dlat, lon_start, lon_end, dlon
      
c     LOCAL VARIABLES
      integer i
      integer*4 i_year                  ! Year
      integer*4 i_month                 ! Month of year
      integer*4 i_day_of_year   ! integer day of year
      integer*4 i_day
      real*8    r_day                   ! floating-point day-of-month
      real*8    d_JD                    ! step size of tec maps in units of days

      character*320 ionex_file, loaded_ionex_file, ionex_h5file
      character*4   str_year
      character*3   str_day
      character*2   str_year_short
      
      integer*4 n_lat
      integer*4 n_lon
      
      integer*4 i_map
      integer*4 n_maps
      
      integer*4 i_lat
      integer*4 i_lon
      integer*4 map_ind
      
      integer counter
    !Variables needed for call to RDIXFL      
      CHARACTER*80  ADTLST(MAXADT) ! ADTLST(I),I=1,..,MAXADT: LIST OF AUX DATA    CH*80(*)
      CHARACTER*60  OBSTXT ! OBSERVABLES USED                    CH*60
      CHARACTER*60  ADTLBL ! LABEL OF AUX DATA REQUESTED/FOUND   CH*60
      CHARACTER*60  TITTXT ! TITLE LINE / FIRST COMMENT LINE     CH*60
      CHARACTER*20  SATSTR ! SATELLITE SYSTEM                    CH*20
      CHARACTER*20  PGMSTR ! PROGRAM                             CH*20
      CHARACTER*20  AGESTR ! AGENCY                              CH*20
      CHARACTER*20  DATSTR ! DATE AND TIME                       CH*20
      CHARACTER*4   MAPSTR ! MAPPING FUNCTION                    CH*4
      REAL*8        INXSP1(14) ! SPECIFICATIONS 1                    R*8(*)
      REAL*8        TECMAP(MAXVAL,2) ! TECMAP(I,J),I=1,..,MAXVAL,J=1,2: TEC/RMS MAP R*8(*,*)
      INTEGER*4     INXSP2(2) ! SPECIFICATIONS 2                    I*4(*)
      INTEGER*4     INXINF(2) ! INFORMATION FOUND                   I*4(*)
      INTEGER*4         IRC ! RETURN CODE (1 good / 0 bad)              I*4

      REAL*8 DJUL
      REAL*8        TECMAP_sdps(MAXVAL)
      INTEGER :: len, lenstr

c     DATA STATEMENTS
      data counter/1/

c     COMMON BLOCKS
      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH

      integer i_dbg_level
      common /debug_level/ i_dbg_level            
c     FUNCTION STATEMENTS
      integer*4 lnblnk
      LOGICAL THERE  

c     PROCESSING STEPS
      save loaded_ionex_file, INXSP1, TECMAP, counter

      call JMT(MJD, i_year, i_month, r_day)
      
      i_day = int( r_day ) ! int() does typcasting and takes the floor...
      
      call CAL2DAY( i_year, i_month, i_day, i_day_of_year )

      if( i_dbg_level.ge.0)then
        write(6,*) 'In subroutine get_ionex_maps: '
        write(6,*) 'i_year, i_month, i_day, i_day_of_year',i_year, i_month, i_day, i_day_of_year
      endif
      
      write(str_year,'(I4)') i_year
      write(str_day,'(I3.3)') i_day_of_year
      
      str_year_short = str_year(3:4)
            
!      ionex_file = AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) //
!     $ '/ionex/igrg' // str_day // '0.' // str_year_short // 'i'

 !     ionex_file = './igrg' // str_day // '0.' // str_year_short // 'i'

      call getenv('L2GEN_AQUARIUS_ANCILLARY_DIR', ionex_h5file)
      len = lenstr(ionex_h5file)
      ionex_h5file = ionex_h5file(1:len) // '/N' // str_year // str_day // 
     1                '00_TEC_IGS_24h.h5'
      ionex_file = ionex_h5file

      INQUIRE( FILE=ionex_file, EXIST=THERE )  
      if (.not.there) then
         ionex_h5file = ionex_h5file(1:len) // '/N' // str_year // str_day // 
     1                '00_TEC_IGR_24h.h5'
         ionex_file = ionex_h5file
      endif

      INQUIRE( FILE=ionex_file, EXIST=THERE )  
      if (.not.there) then
         ionex_h5file = ionex_h5file(1:len) // '/N' // str_year // str_day // 
     1                '00_TEC_CODE_24h.h5'
         ionex_file = ionex_h5file
      endif

      INQUIRE( FILE=ionex_file, EXIST=THERE )  
      if (.not.there) then
         ionex_file = 'NO_TEC_FILE'
      endif

! Consider NO_TEC_FILE as not loaded  JMG  08/15/11
      if( .not.loaded_ionex_file .eq. ionex_file .or. (ionex_file.eq.'NO_TEC_FILE')) then
         write(6,*) 'Loading IONEX file: ',ionex_file(1:lnblnk(ionex_file))

         write(35,'((a),i1,(a),(a))') 
     $        'TECFILE', counter, '=', 
     $        ionex_file(scan(ionex_file, '/', back=.true.)+1:lnblnk(ionex_file))

         if ( ionex_file .ne. 'NO_TEC_FILE') then
            call dump_tec( ionex_file(1:lnblnk(ionex_file))//CHAR(0), TECMAP_sdps)
         else
            do i=1,MAXVAL
               TECMAP_sdps(i) = -999.0
            enddo
         endif

         inxsp1(1) = djul(i_year, i_month, i_day*1.0D0)
         inxsp1(2) = inxsp1(1) + 1
         inxsp1(3) = 7200
         inxsp1(4) = +87.5
         inxsp1(5) = -87.5
         inxsp1(6) = -2.5
         inxsp1(7) = -180.0
         inxsp1(8) = +180.0
         inxsp1(9) = 5

         counter = counter + 1

!        call RDIXFL(ionex_file,MAXVAL,MAXADT,ADTLBL,SATSTR,PGMSTR,
!     1                  AGESTR,DATSTR,TITTXT,MAPSTR,OBSTXT,INXSP1,
!     2                  INXSP2,ADTLST,INXINF,TECMAP,IRC   )      
      else
        write(6,*) 'Using IONEX file already in memory.'
      endif
      
      
      d_JD   = INXSP1(3) / ( 24.0d0*60.0d0*60.0d0) 

      n_lat  = int((INXSP1(5)-INXSP1(4))/INXSP1(6)) + 1
      n_lon  = int((INXSP1(8)-INXSP1(7))/INXSP1(9)) + 1      
      n_maps = int((INXSP1(2)-INXSP1(1))/d_JD) + 1
      
      lat_start = INXSP1(4)
      lat_end   = INXSP1(5)
      dlat      = INXSP1(6)

      lon_start = INXSP1(7)
      lon_end   = INXSP1(8)
      dlon      = INXSP1(9)
      
      
      if( MJD .lt. INXSP1(1) )then
        MJD_2 = INXSP1(1)
        MJD_1 = -1         ! -1 means that later we will trap this and
                           ! load map from prev day...
        
        do i_lat=1,n_lat
          do i_lon=1,n_lon        
            map_ind = i_lon+(i_lat-1)*n_lon
!            tec_map_2(map_ind) = TECMAP(map_ind,1)
            tec_map_2(map_ind) = TECMAP_sdps(map_ind)
          enddo
        enddo
        
                           
      elseif( MJD .gt. INXSP1(2) )then
        MJD_1 = INXSP1(2)
        MJD_2 = -1         ! -1 means that later we will trap this and
                           ! load map from next day...
        do i_lat=1,n_lat
          do i_lon=1,n_lon        
            map_ind = i_lon+(i_lat-1)*n_lon
!            tec_map_1(map_ind) = TECMAP(map_ind+(n_maps-1)*n_lat*n_lon,1)
            tec_map_1(map_ind) = TECMAP_sdps(map_ind+(n_maps-1)*n_lat*n_lon)
          enddo
        enddo                           
                           
      else
C        write(6,*) 'Using IONEX file already in memory!'
        do i_map=1,n_maps
          if( MJD .ge. INXSP1(1)+(i_map-1)*d_JD .and.
     $   MJD .le. INXSP1(1)+i_map*d_JD)then            
            
            MJD_1 = INXSP1(1)+(i_map-1)*d_JD
            MJD_2 = INXSP1(1)+i_map*d_JD           
            
            do i_lat=1,n_lat
              do i_lon=1,n_lon
                map_ind = i_lon+n_lon*(i_lat-1)
             
!                tec_map_1(map_ind) = TECMAP(map_ind+(i_map-1)*n_lat*n_lon,1)
!                tec_map_2(map_ind) = TECMAP(map_ind+i_map*n_lat*n_lon,1)
                tec_map_1(map_ind) = TECMAP_sdps(map_ind+(i_map-1)*n_lat*n_lon)
                tec_map_2(map_ind) = TECMAP_sdps(map_ind+i_map*n_lat*n_lon)
              enddo
            enddo
          endif
        enddo
      endif
      
      if( MJD_1 .lt. 0 )then            ! load map from previous day...
      
        if( i_day_of_year-1.eq.0)then
          write(str_year,'(I4)') i_year-1
          str_year_short = str_year(3:4)
          if( mod(i_year-1,4).eq.0)then
            write(str_day,'(I3.3)') 366
          else
            write(str_day,'(I3.3)') 365
          endif
        else
          write(str_day,'(I3.3)') i_day_of_year-1
        endif
        
        ionex_file = AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) //
     $   '/ionex/' // str_year // '/jplg' // str_day // '0.' // str_year_short // 'i'
      
        write(6,*) 'Loading IONEX file: ',ionex_file(1:lnblnk(ionex_file))
      
      
        call RDIXFL(ionex_file,MAXVAL,MAXADT,ADTLBL,SATSTR,PGMSTR,
     1                  AGESTR,DATSTR,TITTXT,MAPSTR,OBSTXT,INXSP1,
     2                  INXSP2,ADTLST,INXINF,TECMAP,IRC)     
      
        n_lat  = int((INXSP1(5)-INXSP1(4))/INXSP1(6)) + 1
        n_lon  = int((INXSP1(8)-INXSP1(7))/INXSP1(9)) + 1      
        n_maps = int(24.0d0*60.0d0*60.0d0*(INXSP1(2)-INXSP1(1))/INXSP1(3)) + 1
        
        MJD_1 = INXSP1(2)
        
         do i_lat=1,n_lat
           do i_lon=1,n_lon
             map_ind = i_lon+(i_lat-1)*n_lon
             tec_map_1(map_ind) = TECMAP(map_ind+(n_maps-1)*n_lat*n_lon,1)
           enddo
         enddo         
        
        
      elseif( MJD_2 .lt. 0)then         ! load map from next day...
      
        if( mod(i_year,4).eq.0.and.i_day_of_year.eq.366 .or. 
     $   mod(i_year,4).ne.0.and.i_day_of_year.eq.365 )then ! use next year
          write(str_year,'(I4)') i_year+1
          str_year_short = str_year(3:4)
          write(str_day,'(I3.3)') 1
        else
          write(str_day,'(I3.3)') i_day_of_year+1
        endif
        
        ionex_file = AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) //
     $   '/ionex/' // str_year // '/jplg' // str_day // '0.' // str_year_short // 'i'
      
        write(6,*) 'Loading IONEX file: ',ionex_file(1:lnblnk(ionex_file))
      
        call RDIXFL(ionex_file,MAXVAL,MAXADT,ADTLBL,SATSTR,PGMSTR,
     1                  AGESTR,DATSTR,TITTXT,MAPSTR,OBSTXT,INXSP1,
     2                  INXSP2,ADTLST,INXINF,TECMAP,IRC   )      
        
        n_lat  = int((INXSP1(5)-INXSP1(4))/INXSP1(6)) + 1
        n_lon  = int((INXSP1(8)-INXSP1(7))/INXSP1(9)) + 1      
        n_maps = int(24.0d0*60.0d0*60.0d0*(INXSP1(2)-INXSP1(1))/INXSP1(3)) + 1
        

        
        MJD_2 = INXSP1(1)
        
         do i_lat=1,n_lat
           do i_lon=1,n_lon
             map_ind = i_lon+(i_lat-1)*n_lon
             tec_map_2(map_ind) = TECMAP(map_ind,1)
           enddo
         enddo         
        
      endif
      
C      open(30,file='maps.txt')
C      do i_lat=1,n_lat
C        do i_lon=1,n_lon
C          map_ind = i_lon+(i_lat-1)*n_lon
C          write(30,*) lat_start+(i_lat-1)*dlat, lon_start+(i_lon-1)*dlon,tec_map_1(map_ind), tec_map_2(map_ind)
C        enddo
C      enddo   
C      close(30)
      
      loaded_ionex_file = ionex_file
      
      end      
  
      
c*****************************************************************      
      subroutine CAL2DAY( i_year, i_month, i_day, i_day_of_year )
c*****************************************************************      
c**   
c**   FILE NAME:
c**   
c**   DATE WRITTEN: 5/27/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      integer*4 i_year, i_month, i_day
c     OUTPUT VARIABLES
      integer*4 i_day_of_year
c     LOCAL VARIABLES
      integer im
      INTEGER MTAB(12)
      DATA MTAB / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

c     DATA STATEMENTS
c     COMMON BLOCKS
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      
      i_day_of_year=0
      
      if( mod(i_year,4) .eq. 0)then
        mtab(2) = 29
      else
        mtab(2) = 28
      endif
      
      if( mod(i_year,100).eq.0 .and. mod(i_year,400).ne.0) mtab(2) = 28
      
      do im=1,12
        if( im .lt. i_month) i_day_of_year=i_day_of_year+mtab(im)
      enddo
      
      i_day_of_year = i_day_of_year + i_day
      
      end
      
c*****************************************************************
      subroutine bilinear_interp( x1, x2, y1, y2, f11, f12, f21,
     $ f22, x, y, f )
c*****************************************************************      
C Inputs
      real*8 x1, x2             ! indepentant variable 1
      real*8 y1, y2             ! indepentant variable 2
      real*8 f11                ! dependant variable at x=x1, y=y1
      real*8 f12                ! dependant variable at x=x1, y=y2
      real*8 f21                ! dependant variable at x=x2, y=y1
      real*8 f22                ! dependant variable at x=x2, y=y2
      real*8 x,y        ! where we want to interpolat dependent variable
C Outputs
      real*8 f                  ! interpolated value at (x,y)
C Local vars
      real*8 fx1                ! dependant variable at x=x, y=y1
      real*8 fx2                ! dependant variable at x=x, y=y2
      
      call lininterp1d(x1,x2,f11,f21,x,fx1)
      call lininterp1d(x1,x2,f12,f22,x,fx2)
      call lininterp1d(y1,y2,fx1,fx2,y,f)
      end
      
c***************************************************************** 
      subroutine lininterp1d(x1,x2,f1,f2,x,f)
c*****************************************************************
c**   Performs a simple linear interpolation.
c*****************************************************************      
      implicit none
C Inputs      
      real*8 x1 ! independant variable x1
      real*8 x2 ! independant variable x2
      real*8 f1 ! dependant variable at x=x1
      real*8 f2 ! dependant variable at x=x2
      real*8 x  ! where we want to interpolate dependant variable to.
C outputs
      real*8 f
C Local vars
      real*8 eps /10.0d-20/
      
      f = f1 + (f2-f1) * (x-x1) / (x2-x1+eps)
      end
      
      
C      program test_iri
C      implicit none
C      
C      character*320 str_arg
C      integer*4 i_year
C      integer*4 i_month
C      real*8 r_day
C      real*8 lat
C      real*8 lon
C      real*8 tec
C      
C      real*8 MJD
C      real*8 julian_time
C      
C      integer*4 i_lat, i_lon
C      
CC Function Declarations      
C      real*8 djul
C
C      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
C      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
C
C      integer i_dbg_level
C      common /debug_level/ i_dbg_level
C      
C      i_dbg_level = 0
C      
C      call getenv('AQ_SCAT_SIM_DIR_OUT',AQ_SCAT_SIM_DIR_OUT)
C      call getenv('AQ_SCAT_SIM_DIR_DAT',AQ_SCAT_SIM_DIR_DAT)
C      call getenv('AQ_SCAT_SIM_DIR_SCRATCH',AQ_SCAT_SIM_DIR_SCRATCH)      
C      
C      call getarg(1,str_arg)
C      read(str_arg,*) i_year
C      call getarg(2,str_arg)
C      read(str_arg,*) i_month
C      call getarg(3,str_arg)
C      read(str_arg,*) r_day     
C      call getarg(4,str_arg)
C      read(str_arg,*) lat
C      call getarg(5,str_arg)
C      read(str_arg,*) lon 
C      
C      MJD = djul( i_year, i_month, r_day )
C      
C      julian_time = MJD + 2400000.5d0
C      
CC      write(6,*) 'Modified Juian Day: ', MJD
CC      
CC      call get_ionex_tec( lat, lon, julian_time, tec )
C      
CC      open(40,file='map_test.txt')
CC      
CC      do i_lat=-90,90,2
CC        do i_lon=-180,180,2
CC          
CC          lat = i_lat
CC          lon = i_lon
CC          
CC          call get_ionex_tec( lat, lon, julian_time, tec )
CC          write(40,*) lat, lon, tec
CC        enddo
CC      enddo
CC      
CC      close(40)
C      
C      
C      do i_lat=1,100
C         julian_time = julian_time + 0.011
C         call get_ionex_tec( lat, lon, julian_time, tec )
C         write(6,*) 'julian_time, tec:', julian_time, tec
C      enddo
C      end
      
      
