      subroutine get_mask(ylat,xlon,nll,lmask)


c  This subroutine generates land mask values for an array of input
c  locations.  It reads a packed mask file (hardcoded as mask128.dat) and 
c  unpacks the mask bits for the specified latitudes and longitudes.  It 
c  returns the mask values in a byte array (0=ocean, 1=land).  


c  Calling Arguments


c  Name         Type    I/O     Description
c
c  ylat(nll)    R*4      I      Input array of latitudes 
c  xlon(nll)    R*4      I      Input array of longitudes
c  nll          I*4      I      Number of latitudes and longitudes for which
c                                mask values are requested (maximum of 1285)
c  lmask(nll)   I*1      O      Mask values at locations (0=ocean, 1=land) 


c       Subprograms Called:

c       REC2MASK        Unpack mask record to byte array



c       Program written by:     Frederick S. Patt
c                               General Sciences Corporation
c                               May 17, 1992
c
c       Modification History:


      implicit none
      integer nll, i,j,nirec,k, lat,lon,lat2,lon2
      real*4 ylat(nll),xlon(nll)
      integer*4 lons(2),lats(2),lonc,latc
      integer*2 itmp(1024),iptr(360,180),itmp2(1024,64),ir
      integer*4 npd
      byte lmask(*),rmask(2048),bmask(128,128)
      character*32 mafile
      character*128 filnm, fpath
      integer*4 ilen, lenstr
      integer lnblnk

      logical first

C**Include AQ SCAT simulator dirs *************************************
      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
c**********************************************************************

      data first/.true./,latc/-999./,lonc/-999./
c      data mafile/'landmask.dat'/
c      data mafile/'landmask_little_endian'/
      data mafile/'landmask_big_endian'/
c      data mafile/'MASK2'/

      equivalence (iptr(1,1),itmp2(1,1))


      save

c        write(*,*) 'Adentro: ',ylat(nll),xlon(nll)
      if (first) then
        first = .false.

c        call getenv('', fpath)
        fpath = AQ_SCAT_SIM_DIR_DAT
        ilen = lnblnk(fpath)
        if (fpath(ilen:ilen) .eq. '/') then
          filnm = fpath(1:ilen) // '' // mafile 
        else
          filnm = fpath(1:ilen) // '/' //  mafile
        endif
        write(6,'(a,a)') ' Reading land mask file: ', filnm(1:lnblnk(filnm)) 

        open(16,err=999,file=filnm,
     *       status='old',access='direct',recl=2048)
        read(16,rec=1) itmp
        npd = itmp(1)
        lons(1) = itmp(4)
        lons(2) = itmp(5)
        lats(1) = itmp(6)
        lats(2) = itmp(7)
        nirec = ((lats(2)-lats(1))*(lons(2)-lons(1))+1023)/1024
        do i=2,nirec+1
          read(16,rec=i) (itmp2(j,i-1),j=1,1024)
        end do
      
      end if
c      write(*,*)itmp(1),itmp(4),itmp(5),itmp(6)

      do i=1,nll
        lon = xlon(i) + 181.
c        lon = xlon(i) + 1.
        lat = ylat(i) + 91.
        if ((lon.lt.1) .or. (lon.gt.360) .or.
     .      (lat.lt.1) .or. (lat.gt.180)) then
            lmask(i) = 0
            write(*,*) 'Lon/Lat out of range: ',xlon(i),ylat(i),lon,lat
        elseif ((lon.ne.lonc).or.(lat.ne.latc)) then
c          write(*,*)'IR: ',ir,lonc, lon, latc , lat
          lonc = lon 
          latc = lat
          ir = iptr(lon,lat)
c          write(*,*)'IR: ',ir,lonc, lon, latc , lat
          
          if ((ir.eq.0).or.(ir.eq.1)) then
            do j=1,npd
              do k=1,npd
                bmask(k,j) = ir
              end do
            end do 
          else
            read(16,rec=ir+1) rmask
            call rec2mask(rmask,bmask,npd)
          end if
        end if
        lon2 = npd*mod((xlon(i)+180.),1.0) + 1
        lat2 = npd*mod((ylat(i)+180.),1.0) + 1
c        lon2 = npd*mod((xlon(i)),1.0) + 1
c        lat2 = npd*mod((ylat(i)),1.0) + 1
        lmask(i) = bmask(lon2,lat2)
c        write(*,*)'Here: ', lat,lon,lat2,lon2,lmask(i),i
      end do

      return
c
 999  write(*,*) 'Error opening land mask file'
      return
      end

      subroutine rec2mask(rmask,bmask,npd)


c  Converts a 1-D byte array of packed mask bits to a 2-D array of mask
c  bytes.  Dimensions of arrays are assumed for 128x128 mask array.  


c  Calling Arguments


c  Name           Type  I/O     Description


c  rmask(2048)    I*1    I      1-D array of mask bits packed into bytes
c  bmask(128,128) I*1    O      2-D array of mask bytes
c  npd            I*4    I      Number of mask values per degree


      implicit none
      integer nll,  j,i, npd
      integer*2 ir, no8,  ic, ib
      byte rmask(2048),bmask(128,128)
      byte bitm(8)
      data bitm/128,64,32,16,8,4,2,1/
      
      no8 = (npd+7)/8
      do i=1,npd
        ir = no8*(i-1) 
        do j=1,npd 
          ic = (j-1)/8 + 1
          ib = mod((j-1),8) + 1
          bmask(j,i) = iand(rmask(ir+ic),bitm(ib))/bitm(ib)
        end do
      end do
      return
      end    


C      subroutine rec2mask(rmask,bmask,npd)
C
C  Attempt to make one for gfortran....
Cc  Converts a 1-D byte array of packed mask bits to a 2-D array of mask
Cc  bytes.  Dimensions of arrays are assumed for 128x128 mask array.  
C
C
Cc  Calling Arguments
C
C
Cc  Name           Type  I/O     Description
C
C
Cc  rmask(2048)    I*1    I      1-D array of mask bits packed into bytes
Cc  bmask(128,128) I*1    O      2-D array of mask bytes
Cc  npd            I*4    I      Number of mask values per degree
C
C
C      implicit none
C      integer nll,  j,i, npd
C      integer*2 ir, no8,  ic, ib
C      byte rmask(2048),bmask(128,128)
C      integer*2 bitm(8)
C      data bitm/128,64,32,16,8,4,2,1/
C      
C      no8 = (npd+7)/8
C      do i=1,npd
C        ir = no8*(i-1) 
C        do j=1,npd 
C          ic = (j-1)/8 + 1
C          ib = mod((j-1),8) + 1
C          bmask(j,i) = iand(rmask(ir+ic),bitm(ib))/bitm(ib)
C        end do
C      end do
C      return
C      end    
      
