C*
      FUNCTION DJUL(J1,M1,T)
CC
CC NAME       :  DJUL
CC
CC    XX = DJUL(J1,M1,T)
CC
CC PURPOSE    :  COMPUTES THE MODIFIED JULIAN DATE (MJD)
CC               FROM YEAR,MONTH AND DAY
CC               MJD = JULIAN DATE - 2400000.5
CC
CC PARAMETERS :
CC         IN :  J1     : YEAR (E.G. 1984)                    I*4
CC               M1     : MONTH(E.G. 2 FOR FEBRUARY)          I*4
CC               T      : DAY OF MONTH                        R*8
CC        OUT :  DJUL   : MODIFIED JULIAN DATE                R*8
CC
CC SR CALLED  :  ---
CC
CC REMARKS    :  ---
CC
CC AUTHOR     :  G.BEUTLER
CC
CC VERSION    :  3.4  (JAN 93)
CC
CC CREATED    :  87/10/30 17:23        LAST MODIFIED :  88/11/21 16:42
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1987      UNIVERSITY OF BERNE
CC                    SWITZERLAND
CC
C*
      REAL*8 DJUL,T
      J=J1
      M=M1
      IF(M.GT.2)GO TO 1
      J=J-1
      M=M+12
1     I=J/100
      K=2-I+I/4
      DJUL=(365.25D0*J-DMOD(365.25D0*J,1.D0))-679006.D0
      DJUL=DJUL+AINT(30.6001*(M+1))+T+K
      RETURN
      END
C*
      SUBROUTINE GTIXPS(INXSP1,IGP,INPARG,OUTARG,IPOS)
CC
CC NAME       :  GTIXPS
CC
CC PURPOSE    :  GET POSITION OF (1) NEAREST OR (2) LOWER LEFT HAND
CC               GRID POINT - OR ARGUMENTS BASED ON POSITION
CC
CC PARAMETERS :
CC         IN :  INXSP1 : SPECIFICATIONS 1                    R*8(*)
CC                        ( 1): EPOCH OF FIRST MAP (IN MJD)
CC                        ( 2): EPOCH OF LAST MAP (IN MJD)
CC                        ( 3): INTERVAL (IN SEC)
CC                        ( 4): FROM LATITUDE
CC                        ( 5): TO LATITUDE
CC                        ( 6): WITH INCREMENT (IN DEG)
CC                        ( 7): FROM LONGITUDE
CC                        ( 8): TO LONGITUDE
CC                        ( 9): WITH INCREMENT (IN DEG)
CC                        (10): ELEVATION CUTOFF (IN DEG)
CC                        (11): BASE RADIUS (IN KM)
CC                        (12): FROM HEIGHT
CC                        (13): TO HEIGHT
CC                        (14): WITH INCREMENT (IN KM)
CC               IGP    : GRID POINT TO BE RETURNED           I*4
CC                        =0: RETURN ARGUMENTS "OUTARG" BASED
CC                            ON POSITION "IPOS"
CC                        =1: NEAREST
CC                        =2: LOWER LEFT HAND
CC               INPARG : INPUT ARGUMENTS                     R*8(*)
CC                        (1): EPOCH (IN MJD)
CC                        (2): LATITUDE (IN DEG)
CC                        (3): LONGITUDE (IN DEG)
CC                        (4): HEIGHT (IN KM)
CC        OUT :  OUTARG : OUTPUT ARGUMENTS                    R*8(*)
CC                        (1): EPOCH (IN MJD)
CC                        (2): LATITUDE (IN DEG)
CC                        (3): LONGITUDE (IN DEG)
CC                        (4): HEIGHT (IN KM)
CC     IN/OUT :  IPOS   : POSITION                            I*4
CC                        =0: UNDEFINED
CC
CC SR CALLED  :  ---
CC
CC REMARKS    :  IONEX VERSION 1.0
CC
CC AUTHOR     :  S.SCHAER
CC
CC VERSION    :  4.1
CC
CC CREATED    :  01-MAR-98             LAST MODIFIED :  08-JUN-98
CC
CC CHANGES    :  08-APR-98 : SS: CONSIDER "IGP"
CC               08-JUN-98 : SS: RETURN ARGUMENTS, IF "IGP=0"
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1998      UNIVERSITY OF BERNE
CC                    SWITZERLAND
CC
C*
      IMPLICIT REAL*8 (A-H,O-Z)
C
      REAL*8        INXSP1(*),INPARG(*),OUTARG(*)
C
      INCLUDE 'IONEX.inc'
C
C SET DEFAULT RETURN CODE
C -----------------------
      IRC=1
C
C GET NUMBER OF MAPS AND GRID DIMENSIONS
C --------------------------------------
      XINT=INXSP1(3)/86400.D0
      NMAP=IDNINT((INXSP1(2)-INXSP1(1))/XINT)+1
C
      NLAT=IDNINT((INXSP1(5)-INXSP1(4))/INXSP1(6))+1
      NLON=IDNINT((INXSP1(8)-INXSP1(7))/INXSP1(9))+1
C
      IF (INXSP1(14).EQ.0.D0) THEN
        NHGT=1
      ELSE
        NHGT=IDNINT((INXSP1(13)-INXSP1(12))/INXSP1(14))+1
      ENDIF
C
C GET EPOCH, LATITUDE, LONGITUDE, AND HEIGHT BASED ON POSITION
C ------------------------------------------------------------
      IF (IGP.EQ.0) THEN
        IVAL=IPOS
        IMAP=(IVAL-1)/(NHGT*NLAT*NLON)+1
        IVAL=IVAL-(IMAP-1)*NHGT*NLAT*NLON
        IHGT=(IVAL-1)/(NLAT*NLON)+1
        IVAL=IVAL-(IHGT-1)*NLAT*NLON
        ILAT=(IVAL-1)/NLON+1
        IVAL=IVAL-(ILAT-1)*NLON
        ILON=IVAL
C
        OUTARG(1)=INXSP1( 1)+(IMAP-1)*XINT
        OUTARG(2)=INXSP1( 4)+(ILAT-1)*INXSP1( 6)
        OUTARG(3)=INXSP1( 7)+(ILON-1)*INXSP1( 9)
        OUTARG(4)=INXSP1(12)+(IHGT-1)*INXSP1(14)
C
        RETURN
      ENDIF
C
C GET POSITION OF NEAREST OR LOWER LEFT HAND GRID POINT
C -----------------------------------------------------
C
C - EPOCH
      XMAP=(INPARG(1)-INXSP1(1))/XINT+1.D0
      IF (IGP.EQ.1) THEN
        IMAP=IDNINT(XMAP)
      ELSE
        IMAP=IDINT(XMAP)
      ENDIF
      IF (IMAP.GE.1 .AND. IMAP.LE.NMAP) THEN
        OUTARG(1)=INXSP1(1)+(IMAP-1)*XINT
      ELSE
        WRITE(LFNERR,1901) INPARG(1)
1901    FORMAT(/,' ### SR GTIXPS: IRREGULAR EPOCH',
     1    /,16X,'EPOCH: ',F12.5,' MJD',/)
        IRC=0
      ENDIF
C
C - LATITUDE
      XLAT=(INPARG(2)-INXSP1(4))/INXSP1(6)+1.D0
      IF (IGP.EQ.1) THEN
        ILAT=IDNINT(XLAT)
      ELSE
        ILAT=IDINT(XLAT)
      ENDIF
      IF (ILAT.GE.1 .AND. ILAT.LE.NLAT) THEN
        OUTARG(2)=INXSP1(4)+(ILAT-1)*INXSP1(6)
      ELSE
        WRITE(LFNERR,1902) INPARG(2)
1902    FORMAT(/,' ### SR GTIXPS: IRREGULAR LATITUDE',
     1    /,16X,'LATITUDE: ',F8.3,' DEG',/)
        IRC=0
      ENDIF
C
C - LONGITUDE
      XLON=(INPARG(3)-INXSP1(7))/INXSP1(9)+1.D0
      IF (IGP.EQ.1) THEN
        ILON=IDNINT(XLON)
      ELSE
        ILON=IDINT(XLON)
      ENDIF
C
      NCYC=IDNINT(360.D0/DABS(INXSP1(9)))
      IF (ILON.LT.1) THEN
        ILON=ILON+NCYC
      ELSEIF (ILON.GT.NLON) THEN
        ILON=ILON-NCYC
      ENDIF
C
      IF (ILON.GE.1 .AND. ILON.LE.NLON) THEN
        OUTARG(3)=INXSP1(7)+(ILON-1)*INXSP1(9)
      ELSE
        WRITE(LFNERR,1903) INPARG(3)
1903    FORMAT(/,' ### SR GTIXPS: IRREGULAR LONGITUDE',
     1    /,16X,'LONGITUDE: ',F8.3,' DEG',/)
        IRC=0
      ENDIF
C
C - HEIGHT
      IF (INXSP1(14).EQ.0.D0) THEN
        IHGT=1
      ELSE
        XHGT=(INPARG(4)-INXSP1(12))/INXSP1(14)+1.D0
        IF (IGP.EQ.1) THEN
          IHGT=IDNINT(XHGT)
        ELSE
          IHGT=IDINT(XHGT)
        ENDIF
        IF (IHGT.GE.1 .AND. IHGT.LE.NHGT) THEN
          OUTARG(4)=INXSP1(12)+(IHGT-1)*INXSP1(14)
        ELSE
          WRITE(LFNERR,1904) INPARG(4)
1904      FORMAT(/,' ### SR GTIXPS: IRREGULAR HEIGHT',
     1      /,16X,'HEIGHT: ',F8.3,' KM',/)
          IRC=0
        ENDIF
      ENDIF
C
C RETURN POSITION OF GRID POINT
C -----------------------------
      IF (IRC.EQ.1) THEN
        IPOS=(IMAP-1)*NHGT*NLAT*NLON+
     1    (IHGT-1)*NLAT*NLON+
     2    (ILAT-1)*NLON+ILON
      ELSE
        IPOS=0
      ENDIF
C
      RETURN
      END
C*
      SUBROUTINE GTIXVL(MAXVAL,INXSP1,INXINF,TECMAP,TECARG,IIS   ,
     1                  TECVAL,IRC   )
CC
CC NAME       :  GTIXVL
CC
CC PURPOSE    :  GET IONEX TEC AND RMS VALUES AS A FUNCTION OF
CC               EPOCH, LATITUDE, LONGITUDE, AND NOMINAL HEIGHT.
CC               FOUR INTERPOLATION STRATEGIES ARE SUPPORTED (SEE
CC               ALSO IONEX MANUAL):
CC                 (1) TAKE NEAREST MAP,
CC                 (2) INTERPOLATE BETWEEN TWO CONSECUTIVE MAPS,
CC                 (3) INTERPOLATE BETWEEN TWO CONSECUTIVE ROTATED
CC                     MAPS, OR
CC                 (4) TAKE NEAREST ROTATED MAP.
CC               A SIMPLE 4-POINT FORMULA IS APPLIED TO INTERPOLATE
CC               BETWEEN THE GRID POINTS.
CC
CC PARAMETERS :
CC         IN :  MAXVAL : MAXIMUM NUMBER OF TEC/RMS VALUES    I*4
CC               INXSP1 : SPECIFICATIONS 1                    R*8(*)
CC                        ( 1): EPOCH OF FIRST MAP (IN MJD)
CC                        ( 2): EPOCH OF LAST MAP (IN MJD)
CC                        ( 3): INTERVAL (IN SEC)
CC                        ( 4): FROM LATITUDE
CC                        ( 5): TO LATITUDE
CC                        ( 6): WITH INCREMENT (IN DEG)
CC                        ( 7): FROM LONGITUDE
CC                        ( 8): TO LONGITUDE
CC                        ( 9): WITH INCREMENT (IN DEG)
CC                        (10): ELEVATION CUTOFF (IN DEG)
CC                        (11): BASE RADIUS (IN KM)
CC                        (12): FROM HEIGHT
CC                        (13): TO HEIGHT
CC                        (14): WITH INCREMENT (IN KM)
CC               INXINF : INFORMATION AVAILABLE               I*4(*)
CC                        =0/1: NO/YES
CC                        (1): TEC MAPS
CC                        (2): RMS MAPS
CC               TECMAP(I,J),I=1,..,MAXVAL,J=1,2: TEC/RMS MAP R*8(*,*)
CC                        (IN TECU)
CC                        =999.9: UNDEFINED
CC               TECARG : ARGUMENTS                           R*8(*)
CC                        (1): EPOCH (IN MJD)
CC                        (2): LATITUDE (IN DEG)
CC                        (3): LONGITUDE (IN DEG)
CC                        (4): HEIGHT (IN KM)
CC               IIS    : INTERPOLATION STRATEGY              I*4
CC                        =1: TAKE NEAREST MAP
CC                        =2: INTERPOLATE BETWEEN CONSECUTIVE
CC                            MAPS
CC                        =3: INTERPOLATE BETWEEN CONSECUTIVE
CC                            ROTATED MAPS
CC                        =4: TAKE NEAREST ROTATED MAP
CC        OUT :  TECVAL : TEC AND RMS VALUES (IN TECU)        R*8(*)
CC                        (1): TEC
CC                        (2): RMS
CC               IRC    : RETURN CODE                         I*4
CC                        =0: ERROR OCCURRED
CC                        =1: OK
CC
CC SR CALLED  :  GTIXPS
CC
CC REMARKS    :  IONEX VERSION 1.0
CC
CC AUTHOR     :  S.SCHAER
CC
CC VERSION    :  4.1
CC
CC CREATED    :  20-NOV-97             LAST MODIFIED :  15-APR-98
CC
CC CHANGES    :  ---
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1997      UNIVERSITY OF BERNE
CC                    SWITZERLAND
CC
C*
      IMPLICIT REAL*8 (A-H,O-Z)
C
      REAL*8        INXSP1(*),TECMAP(MAXVAL,*)
      REAL*8        TECARG(*),TECVAL(*)
      REAL*8        INPARG(4),OUTARG(4),ORIARG(4)
      REAL*8        TWOEPO(2),TWOFAC(2),PNTVAL(4,2)
C
      INTEGER*4     INXINF(*)
      INTEGER*4     PNTPOS(4)
C
      INCLUDE 'IONEX.inc'
C
C SET DEFAULT RETURN CODE
C -----------------------
      IRC=1
C
C DEFINE NUMBER OF MAPS TO BE CONSIDERED
C --------------------------------------
      IF (IIS.EQ.1) THEN
        NMAP=1
      ELSEIF (IIS.EQ.2) THEN
        NMAP=2
      ELSEIF (IIS.EQ.3) THEN
        NMAP=2
      ELSEIF (IIS.EQ.4) THEN
        NMAP=1
      ELSE
        WRITE(LFNERR,1901)
1901    FORMAT(/,' ### SR GTIXVL: INVALID INTERPOLATION STRATEGY',/)
        GOTO 900
      ENDIF
C
C LOOK FOR VALID IONEX MAPS
C -------------------------
      XINT=INXSP1(3)/86400.D0
      IF (NMAP.EQ.1) THEN
        CALL GTIXPS(INXSP1,1,TECARG,OUTARG,IPOS)
        IF (IPOS.EQ.0) GOTO 900
C
        TWOEPO(1)=OUTARG(1)
        TWOFAC(1)=1.D0
      ELSE
        CALL GTIXPS(INXSP1,2,TECARG,OUTARG,IPOS)
        IF (IPOS.EQ.0) GOTO 900
C
        TWOEPO(1)=OUTARG(1)
        TWOEPO(2)=OUTARG(1)+XINT
        TWOFAC(1)=(TWOEPO(2)-TECARG(1))/XINT
        TWOFAC(2)=(TECARG(1)-TWOEPO(1))/XINT
      ENDIF
C
      TECVAL(1)=0.D0
      TECVAL(2)=0.D0
      DO IMAP=1,NMAP
C
C GET POSITION OF LOWER LEFT HAND GRID POINT E00
C ----------------------------------------------
        INPARG(1)=TWOEPO(IMAP)+XINT/2.D0
        INPARG(2)=TECARG(2)
        IF (IIS.EQ.1 .OR. IIS.EQ.2) THEN
          INPARG(3)=TECARG(3)
          INPARG(4)=TECARG(4)
        ELSE
          INPARG(3)=TECARG(3)+360.D0*(TECARG(1)-TWOEPO(IMAP))
          INPARG(4)=TECARG(4)
        ENDIF
        CALL GTIXPS(INXSP1,2,INPARG,ORIARG,IPOS)
        IF (IPOS.EQ.0) GOTO 900
        PNTPOS(1)=IPOS
C
C CHECK HEIGHT ARGUMENT
C ---------------------
        IF (INXSP1(14).NE.0.D0) THEN
          XDIF=TECARG(4)-ORIARG(4)
          IF (XDIF.NE.0.D0) THEN
            WRITE(LFNERR,1902) TECARG(4)
1902        FORMAT(/,' ### SR GTIXVL: IRREGULAR HEIGHT',
     1        /,16X,'HEIGHT: ',F8.3,' KM',/)
            GOTO 900
          ENDIF
        ENDIF
C
C COMPUTE FACTORS P AND Q
C -----------------------
        XP=(INPARG(3)-ORIARG(3))/INXSP1(9)
        XP=DMOD(XP+360.D0/DABS(INXSP1(9)),1.D0)
        XQ=(INPARG(2)-ORIARG(2))/INXSP1(6)
C
        INPARG(1)=TWOEPO(IMAP)
        INPARG(4)=ORIARG(4)
C
C GET E01'S POSITION
        INPARG(2)=ORIARG(2)
        INPARG(3)=ORIARG(3)+INXSP1(9)
        CALL GTIXPS(INXSP1,1,INPARG,OUTARG,IPOS)
        IF (IPOS.EQ.0) GOTO 900
        PNTPOS(2)=IPOS
C
C GET E10'S POSITION
        INPARG(2)=ORIARG(2)+INXSP1(6)
        INPARG(3)=ORIARG(3)
        CALL GTIXPS(INXSP1,1,INPARG,OUTARG,IPOS)
        IF (IPOS.EQ.0) GOTO 900
        PNTPOS(3)=IPOS
C
C GET E11'S POSITION
        INPARG(2)=ORIARG(2)+INXSP1(6)
        INPARG(3)=ORIARG(3)+INXSP1(9)
        CALL GTIXPS(INXSP1,1,INPARG,OUTARG,IPOS)
        IF (IPOS.EQ.0) GOTO 900
        PNTPOS(4)=IPOS
C
        DO ITYP=1,2
          IF (INXINF(ITYP).EQ.1) THEN
            DO IPNT=1,4
              XVAL=TECMAP(PNTPOS(IPNT),ITYP)
              IF (XVAL.NE.999.9D0) THEN
                PNTVAL(IPNT,ITYP)=XVAL
              ELSE
                WRITE(LFNERR,1903)
1903            FORMAT(/,' ### SR GTIXVL: UNDEFINED TEC/RMS VALUE(S)',/)
                GOTO 900
              ENDIF
            ENDDO
C
            XSUM=(1.D0-XP)*(1.D0-XQ)*PNTVAL(1,ITYP)+
     1        XP*(1.D0-XQ)*PNTVAL(2,ITYP)+
     2        XQ*(1.D0-XP)*PNTVAL(3,ITYP)+
     3        XP*XQ*PNTVAL(4,ITYP)
            TECVAL(ITYP)=TECVAL(ITYP)+TWOFAC(IMAP)*XSUM
          ENDIF
        ENDDO
      ENDDO
C
C OK
C --
      RETURN
C
C ERROR OCCURRED
C --------------
900   CONTINUE
      IRC=0
C
      RETURN
      END
C*
      SUBROUTINE JMT(DJUL,J,M,D)
CC
CC NAME       :  JMT
CC
CC    CALL JMT(DJUL,J,M,D)
CC
CC PURPOSE    :  COMPUTE YEAR,MONTH,DAY OF MONTH FROM
CC               MODIFIED JULIAN DATE (MJD=JUL. DATE-2400000.5)
CC
CC PARAMETERS :
CC         IN :  DJUL   : MODIFIED JULIAN DATE                R*8
CC        OUT :  J      : YEAR                                I*4
CC               M      : MONTH                               I*4
CC               D      : DAY OF MONTH                        R*8
CC
CC SR CALLED  :  ---
CC
CC REMARKS    :  ---
CC
CC AUTHOR     :  G.BEUTLER
CC
CC VERSION    :  3.4  (JAN 93)
CC
CC CREATED    :  87/11/02 15:22        LAST MODIFIED :  14-JAN-92
CC
CC CHANGES    :  14-JAN-92 : CHANGE OF HEAD (IN/OUT PARAMETERS)
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1987      UNIVERSITY OF BERNE
CC                    SWITZERLAND
CC
C*
      REAL*8 DJUL,D,T1,T2,T3,T4
      T1=1.D0+DJUL-DMOD(DJUL,1.D0)+2400000.D0
      T4=DMOD(DJUL,1.D0)
      IH=(T1-1867216.25D0)/36524.25D0
      T2=T1+1+IH-IH/4
      T3=T2-1720995.D0
      IH1=(T3-122.1D0)/365.25D0
      T1=365.25D0*IH1-DMOD(365.25D0*IH1,1.D0)
      IH2=(T3-T1)/30.6001D0
      D=T3-T1-INT(30.6001*IH2)+T4
      M=IH2-1
      IF(IH2.GT.13)M=IH2-13
      J=IH1
      IF(M.LE.2)J=J+1
      RETURN
      END
C*
      SUBROUTINE RADGMS(I,ARC,VORZ,IG,M,S)
CC
CC NAME       :  RADGMS
CC
CC PURPOSE    :  TRANSFORMS AN ANGLE IN RADIANS INTO
CC                (A) SIGN,HOURS,MINUTES,SECONDS     FOR I=0
CC                (B) SIGN,DEG,MIN,SEC (SEXAGESIMAL) FOR I=1
CC                (C) SIGN,DEG,MIN,SEC (CENTESIMAL)  FOR I=2
CC               RADGMS MAY ALSO BE USED WITH            I=3
CC               TO COMPUTE HOURS,MINUTES,SECONDS
CC               OF A DAY-FRACTION. THE SIGN-VARIABLE
CC               VORZ IS MEANINGLESS IN THIS CASE
CC               THE SIGN IS EDITED IN THE VARIABLE VORZ
CC
CC PARAMETERS :  I      : TRANSFORMATION TYPE (SEE ABOVE)     I*4
CC               ARC    : ANGLE IN RADIANS                    R*8
CC               VORZ   : SIGN OF ANGLE:  " " FOR POSITIVE,   CH*1
CC                        "-" FOR NEGATIVE ANGLES
CC               IG,M   : DEGREES(HOURS),MINUTES              I*4
CC               S      : SECONDS                             R*8
CC
CC SR CALLED  :  ---
CC
CC REMARKS    :  ---
CC
CC AUTHOR     :  W.GURTNER
CC
CC VERSION    :  3.4  (JAN 93)
CC
CC CREATED    :  87/11/03 08:45        LAST MODIFIED :  01-MAR-98
CC
CC CHANGES    :  10-AUG-94 : MR: CALL EXITRC
CC               01-MAR-98 : SS: CALL EXITRC REPLACED BY STOP COMMAND
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1987      UNIVERSITY OF BERNE
CC                    SWITZERLAND
CC
C*
      IMPLICIT REAL*8 (A-H,O-Z)
      CHARACTER*1 VORZ
      INCLUDE 'IONEX.inc'
      PI=3.1415926535897932D0
      DELTA=1.D-6
      GOTO (1,2,3,4),I+1
      WRITE(LFNERR,11) I
11    FORMAT(/,' *** SR RADGMS: ILLEGAL TRANSFORMATION TYPE',/,
     1                     16X,'TRANS. TYPE:',I2,/)
      STOP
C  H,M,S
1     SEC=DABS(ARC)*4.32D4/PI+DELTA
      SPM=60.D0
      GOTO 5
C  G,M,S
2     SEC=DABS(ARC)*6.48D5/PI+DELTA
      SPM=60.D0
      GOTO 5
C  GRADS
3     SEC=DABS(ARC)*2.D6/PI+DELTA
      SPM=100.D0
      GOTO 5
C  FRACTIONAL PART OF DAY IN H,M,S
4     SEC=DMOD(ARC,1.D0)*86400.D0+DELTA
      SPM=60.D0
C
5     IG=SEC/(SPM*SPM)
      SECNOG=SEC-IG*SPM*SPM
      M=SECNOG/SPM
      S=SECNOG-M*SPM-DELTA
      IF(S.LT.0.D0) S=0.D0
      IF(I.NE.3) THEN
        VORZ=' '
        IF(ARC.LT.0)VORZ='-'
      END IF
      RETURN
      END
C*
      SUBROUTINE RDIXDT(NUMVAL,IEXP,TECVAL,IRC)
CC
CC NAME       :  RDIXDT
CC
CC PURPOSE    :  READ IONEX DATA
CC
CC PARAMETERS :
CC         IN :  NUMVAL : NUMBER OF TEC/RMS VALUES            I*4
CC               IEXP   : ACTIVE EXPONENT                     I*4
CC        OUT :  TECVAL : TEC/RMS VALUES (IN TECU)            R*8(*)
CC                        =999.9: UNDEFINED
CC               IRC    : RETURN CODE                         I*4
CC                        =0: ERROR OCCURRED
CC                        =1: OK
CC
CC SR CALLED  :  ---
CC
CC REMARKS    :  IONEX VERSION 1.0
CC
CC AUTHOR     :  S.SCHAER
CC
CC VERSION    :  4.1
CC
CC CREATED    :  11-SEP-97             LAST MODIFIED :  05-JAN-98
CC
CC CHANGES    :  05-JAN-98 : SS: "AUXVAL" REDIMENSIONED
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1997      UNIVERSITY OF BERNE
CC                    SWITZERLAND
CC
C*
      IMPLICIT REAL*8 (A-H,O-Z)
C
      CHARACTER*80  LINE
C
      REAL*8        TECVAL(*)
C
      INTEGER*4     AUXVAL(16)
C
      INCLUDE 'IONEX.inc'
C
C COMPUTE NUMBER OF DATA LINES TO BE READ
C ---------------------------------------
      NLIN=(NUMVAL-1)/16+1
C
C READ SINGLE DATA BLOCK
C ----------------------
      DO ILIN=1,NLIN
        IVAL1=16*ILIN-15
        IVAL2=16*ILIN
        IF (IVAL2.GT.NUMVAL) IVAL2=NUMVAL
C
        READ(LFNLOC,910,END=100) LINE
910     FORMAT(A80)
C
        NAUX=IVAL2-IVAL1+1
        READ(LINE,920,ERR=200) (AUXVAL(IAUX),IAUX=1,NAUX)
920     FORMAT(16I5)
C
C CONVERT TEC/RMS VALUES FROM ACTIVE UNIT INTO TECU
C -------------------------------------------------
        DO IAUX=1,NAUX
          IVAL=IVAL1+IAUX-1
          IF (AUXVAL(IAUX).NE.9999) THEN
            TECVAL(IVAL)=DBLE(10.D0**IEXP*AUXVAL(IAUX))
          ELSE
            TECVAL(IVAL)=999.9D0
          ENDIF
        ENDDO
      ENDDO
C
      IRC=1
      GOTO 300
C
C END OF FILE REACHED
C -------------------
100   CONTINUE
      WRITE(LFNERR,1910)
1910  FORMAT(/,' *** SR RDIXDT: END OF FILE REACHED',/)
C
      IRC=0
      GOTO 300
C
C ERROR READING IONEX DATA
C ------------------------
200   CONTINUE
      WRITE(LFNERR,1920) LINE
1920  FORMAT(/,' *** SR RDIXDT: ERROR READING IONEX DATA',
     1  /,A80,/)
C
      IRC=0
      GOTO 300
C
C RETURN
C ------
300   CONTINUE
C
      RETURN
      END
C*
      SUBROUTINE RDIXFL(INXFIL,MAXVAL,MAXADT,ADTLBL,SATSTR,PGMSTR,
     1                  AGESTR,DATSTR,TITTXT,MAPSTR,OBSTXT,INXSP1,
     2                  INXSP2,ADTLST,INXINF,TECMAP,IRC   )
CC
CC NAME       :  RDIXFL
CC
CC PURPOSE    :  READ IONEX FILE
CC
CC PARAMETERS :
CC         IN :  INXFIL : EXTERNAL IONEX INPUT FILE NAME      CH*32
CC               MAXVAL : MAXIMUM NUMBER OF TEC/RMS VALUES    I*4
CC                        =0: READ HEADER ONLY
CC               MAXADT : MAXIMUM NUMBER OF AUX RECORDS       I*4
CC     IN/OUT :  ADTLBL : LABEL OF AUX DATA REQUESTED/FOUND   CH*60
CC                        =' ': NOT REQUESTED/FOUND
CC        OUT :  SATSTR : SATELLITE SYSTEM                    CH*20
CC               PGMSTR : PROGRAM                             CH*20
CC               AGESTR : AGENCY                              CH*20
CC               DATSTR : DATE AND TIME                       CH*20
CC               TITTXT : TITLE LINE / FIRST COMMENT LINE     CH*60
CC                        =' ': UNDEFINED
CC               MAPSTR : MAPPING FUNCTION                    CH*4
CC               OBSTXT : OBSERVABLES USED                    CH*60
CC               INXSP1 : SPECIFICATIONS 1                    R*8(*)
CC                        ( 1): EPOCH OF FIRST MAP (IN MJD)
CC                        ( 2): EPOCH OF LAST MAP (IN MJD)
CC                        ( 3): INTERVAL (IN SEC)
CC                        ( 4): FROM LATITUDE
CC                        ( 5): TO LATITUDE
CC                        ( 6): WITH INCREMENT (IN DEG)
CC                        ( 7): FROM LONGITUDE
CC                        ( 8): TO LONGITUDE
CC                        ( 9): WITH INCREMENT (IN DEG)
CC                        (10): ELEVATION CUTOFF (IN DEG)
CC                        (11): BASE RADIUS (IN KM)
CC                        (12): FROM HEIGHT
CC                        (13): TO HEIGHT
CC                        (14): WITH INCREMENT (IN KM)
CC               INXSP2 : SPECIFICATIONS 2                    I*4(*)
CC                        =0: NOT FOUND
CC                        (1): NUMBER OF STATIONS
CC                        (2): NUMBER OF SATELLITES
CC               ADTLST(I),I=1,..,MAXADT: LIST OF AUX DATA    CH*80(*)
CC               INXINF : INFORMATION FOUND                   I*4(*)
CC                        =0/1: NO/YES
CC                        (1): TEC MAPS
CC                        (2): RMS MAPS
CC               TECMAP(I,J),I=1,..,MAXVAL,J=1,2: TEC/RMS MAP R*8(*,*)
CC                        (IN TECU)
CC                        =999.9: UNDEFINED
CC               IRC    : RETURN CODE                         I*4
CC                        =0: ERROR OCCURRED
CC                        =1: OK
CC
CC SR CALLED  :  DJUL  , GTIXPS, OPNERR, OPNFIL, RDIXHD, RDIXDT
CC
CC REMARKS    :  IONEX VERSION 1.0
CC
CC AUTHOR     :  S.SCHAER
CC
CC VERSION    :  4.1
CC
CC CREATED    :  25-SEP-97             LAST MODIFIED :  07-OCT-98
CC
CC CHANGES    :  13-NOV-97 : SS: RETURN FIRST COMMENT LINE
CC               20-NOV-97 : SS: READ HEADER ONLY
CC               09-MAR-98 : SS: USE SR GTIXPS
CC               08-APR-98 : SS: SR GTIXPS REFINED
CC               07-OCT-98 : SS: CHECK WHETHER LAST DATA BLOCK CLOSED
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1997      UNIVERSITY OF BERNE
CC                    SWITZERLAND
CC
C*
      IMPLICIT REAL*8 (A-H,O-Z)
C
      CHARACTER*80  ADTLST(*),LINE
      CHARACTER*60  OBSTXT,ADTLBL,TITTXT
      CHARACTER*320  INXFIL
      CHARACTER*20  SATSTR,PGMSTR,AGESTR,DATSTR,INXREC(11),AUXREC
      CHARACTER*4   MAPSTR
C
      REAL*8        INXSP1(*),TECMAP(MAXVAL,*),INPARG(4),OUTARG(4)
C
      INTEGER*4     INXSP2(*),INXINF(*)
      INTEGER*4     NUMMAP(3),NUMNEG(2)
C
      INCLUDE 'IONEX.inc'
C
C SET IONEX RECORDS
C -----------------
      DATA INXREC/'START OF TEC MAP    ','START OF RMS MAP    ',
     1            'START OF HEIGHT MAP ','EPOCH OF CURRENT MAP',
     2            'EXPONENT            ','COMMENT             ',
     3            'LAT/LON1/LON2/DLON/H','END OF TEC MAP      ',
     4            'END OF RMS MAP      ','END OF HEIGHT MAP   ',
     5            'END OF FILE         '/
C
C OPEN IONEX INPUT FILE
C ---------------------
C      CALL OPNFIL(LFNLOC,INXFIL,'OLD','FORMATTED',
C     1  'READONLY',' ',IOSTAT)
C      CALL OPNERR(LFNERR,LFNLOC,IOSTAT,INXFIL,'RDIXFL')
      
      open(LFNLOC,file=INXFIL,status='old',form='formatted')
      LFNERR = 6
C
C READ IONEX HEADER
C -----------------
      CALL RDIXHD(MAXADT,ADTLBL,SATSTR,PGMSTR,AGESTR,DATSTR,
     1            TITTXT,MAPSTR,OBSTXT,INXSP1,INXSP2,ADTLST,
     2            IEXP  ,IRC   )
C
      IF (IRC.EQ.0 .OR. MAXVAL.EQ.0) GOTO 300
C
      XINT=INXSP1(3)/86400.D0
      NMAP=IDNINT((INXSP1(2)-INXSP1(1))/XINT)+1
C
      NLAT=IDNINT((INXSP1(5)-INXSP1(4))/INXSP1(6))+1
      NLON=IDNINT((INXSP1(8)-INXSP1(7))/INXSP1(9))+1
C
      IF (INXSP1(14).EQ.0.D0) THEN
        NHGT=1
      ELSE
        NHGT=IDNINT((INXSP1(13)-INXSP1(12))/INXSP1(14))+1
      ENDIF
C
      NVAL=NMAP*NHGT*NLAT*NLON
C
      IF (NVAL.GT.MAXVAL) THEN
        WRITE(LFNERR,1901) NVAL,MAXVAL
1901    FORMAT(/,' *** SR RDIXFL: TOO MANY TEC/RMS VALUES',
     1    /,16X,'NUMBER OF VALUES : ',I6,
     2    /,16X,'MAXIMUM NUMBER   : ',I6,/)
C
        IRC=0
        GOTO 300
      ENDIF
C
C INITIALIZE TEC/RMS MAPS
C -----------------------
      DO ITYP=1,2
        DO IVAL=1,NVAL
          TECMAP(IVAL,ITYP)=-999.9D0
        ENDDO
        NUMMAP(ITYP)=0
      ENDDO
      NUMMAP(3)=0
C
      ITYP=0
C
C LOOP OVER ALL DATA LINES
C ------------------------
400   CONTINUE
      READ(LFNLOC,901,END=100) LINE
901   FORMAT(A80)
C
C CURRENT KEYWORD
      AUXREC=LINE(61:80)
C
C LOOK FOR CURRENT KEYWORD AND STORE CORRESPONDING INFORMATION
C ------------------------------------------------------------
C
C "START OF TEC MAP"
      IF (AUXREC.EQ.INXREC(1)) THEN
        IF (ITYP.NE.0) GOTO 201
        READ(LINE,911,ERR=200) IMAP
911     FORMAT(I6)
C
        ITYP=1
        NUMMAP(ITYP)=NUMMAP(ITYP)+1
C
C "START OF RMS MAP"
      ELSEIF (AUXREC.EQ.INXREC(2)) THEN
        IF (ITYP.NE.0) GOTO 201
        READ(LINE,911,ERR=200) IMAP
C
        ITYP=2
        NUMMAP(ITYP)=NUMMAP(ITYP)+1
C
C "START OF HEIGHT MAP"
      ELSEIF (AUXREC.EQ.INXREC(3)) THEN
        IF (ITYP.NE.0) GOTO 201
        READ(LINE,911,ERR=200) IMAP
C
        ITYP=3
        NUMMAP(ITYP)=NUMMAP(ITYP)+1
C
C SKIP HEIGHT MAP
401     CONTINUE
        READ(LFNLOC,901,END=100) LINE
        AUXREC=LINE(61:80)
        IF (AUXREC.EQ.INXREC(10)) GOTO 400
        GOTO 401
C
C "EPOCH OF CURRENT MAP"
      ELSEIF (AUXREC.EQ.INXREC(4)) THEN
        READ(LINE,921,ERR=200) IYEAR,IMONTH,IDAY,IHOUR,IMIN,ISEC
921     FORMAT(6I6)
C
        XEPO=DJUL(IYEAR,IMONTH,IDAY+IHOUR/24.D0+
     1    IMIN/1440.D0+ISEC/86400.D0)
        XEPO0=INXSP1(1)+(IMAP-1)*INXSP1(3)/86400.D0
        IDIF=IDNINT(DABS((XEPO-XEPO0))*86400.D0)
C
        IF (IDIF.NE.0) THEN
          WRITE(LFNERR,1902) IMAP,ITYP
1902      FORMAT(/,' *** SR RDIXFL: "EPOCH OF CURRENT MAP" WRONG',
     1      /,16X,'MAP NUMBER : ',I6,
     2      /,16X,'MAP TYPE   : ',I6,/)
          IRC=0
          GOTO 300
        ENDIF
C
C "EXPONENT"
      ELSEIF (AUXREC.EQ.INXREC(5)) THEN
        READ(LINE,911,ERR=200) IEXP
C
C "COMMENT"
      ELSEIF (AUXREC.EQ.INXREC(6)) THEN
        GOTO 400
C
C "LAT/LON1/LON2/DLON/H"
      ELSEIF (AUXREC.EQ.INXREC(7)) THEN
        IF (ITYP.EQ.0) THEN
          WRITE(LFNERR,1903) LINE
1903      FORMAT(/,' *** SR RDIXFL: MAP TYPE UNDEFINED',
     1      /,A80,/)
          IRC=0
          GOTO 300
        ENDIF
C
        READ(LINE,931,ERR=200) XLAT,XLON1,XLON2,XDLON,XHGT
931     FORMAT(2X,5F6.1)
C
C GET INITIAL POSITION
        INPARG(1)=XEPO
        INPARG(2)=XLAT
        INPARG(3)=XLON1
        INPARG(4)=XHGT
        CALL GTIXPS(INXSP1,1,INPARG,OUTARG,IVAL)
C
C READ TEC/RMS DATA BLOCK
        CALL RDIXDT(NLON,IEXP,TECMAP(IVAL,ITYP),IRC)
C
        IF (IRC.EQ.0) GOTO 300
C
C "END OF TEC MAP"
      ELSEIF (AUXREC.EQ.INXREC(8)) THEN
        ITYP=0
        GOTO 400
C
C "END OF RMS MAP"
      ELSEIF (AUXREC.EQ.INXREC(9)) THEN
        ITYP=0
        GOTO 400
C
C "END OF FILE"
      ELSEIF (AUXREC.EQ.INXREC(11)) THEN
        GOTO 500
C
C SKIP BLANK LINES
      ELSEIF (LINE.EQ.' ') THEN
        GOTO 400
C
C UNKNOWN DATA RECORD FOUND
      ELSE
        WRITE(LFNERR,1904) LINE
1904    FORMAT(/,' ### SR RDIXFL: UNIDENTIFIED IONEX DATA RECORD',
     1    /,A80,/)
        GOTO 400
      ENDIF
C
C GOTO NEXT DATA LINE
C -------------------
      GOTO 400
C
C SOME FINAL CHECKS
C -----------------
500   CONTINUE
      DO ITYP=1,2
        IF (NUMMAP(ITYP).GT.0) THEN
          INXINF(ITYP)=1
        ELSE
          INXINF(ITYP)=0
        ENDIF
      ENDDO
C
C CHECK WHETHER ALL TEC/RMS VALUES ARE AVAILABLE
      DO ITYP=1,2
        NUMNEG(ITYP)=0
        DO IVAL=1,NVAL
          IF (TECMAP(IVAL,ITYP).EQ.-999.9D0) THEN
            NUMNEG(ITYP)=NUMNEG(ITYP)+1
            TECMAP(IVAL,ITYP)=999.9D0
          ENDIF
        ENDDO
C
        IF (INXINF(ITYP).EQ.1 .AND. NUMNEG(ITYP).GT.0) THEN
          WRITE(LFNERR,1911) NUMNEG(ITYP),ITYP
1911      FORMAT(/,' *** SR RDIXFL: TEC/RMS VALUES MISSING',
     1      /,16X,'NUMBER OF VALUES : ',I6,
     2      /,16X,'MAP TYPE         : ',I6,/)
          IRC=0
        ENDIF
      ENDDO
C
C HEIGHT MAPS SKIPPED
      IF (NUMMAP(3).GT.0) THEN
        WRITE(LFNERR,1912) NUMMAP(3)
1912    FORMAT(/,' ### SR RDIXFL: HEIGHT MAPS SKIPPED',
     1    /,16X,'NUMBER OF MAPS: ',I6,/)
      ENDIF
C
      GOTO 300
C
C END OF FILE REACHED
C -------------------
100   CONTINUE
      WRITE(LFNERR,1921)
1921  FORMAT(/,' *** SR RDIXFL: END OF FILE REACHED',/)
C
      IRC=0
      GOTO 300
C
C ERROR READING IONEX FILE
C ------------------------
200   CONTINUE
      WRITE(LFNERR,1922) LINE
1922  FORMAT(/,' *** SR RDIXFL: ERROR READING IONEX FILE',
     1  /,A80,/)
C
      IRC=0
      GOTO 300
C
C DATA BLOCK NOT CLOSED
C ---------------------
201   CONTINUE
      WRITE(LFNERR,1923) 
1923  FORMAT(/,' *** SR RDIXFL: DATA BLOCK NOT CLOSED',/)
C
      IRC=0
      GOTO 300
C
C CLOSE IONEX INPUT FILE
C ----------------------
300   CONTINUE
      CLOSE (UNIT=LFNLOC)
C
      RETURN
      END
C*
      SUBROUTINE RDIXHD(MAXADT,ADTLBL,SATSTR,PGMSTR,AGESTR,DATSTR,
     1                  TITTXT,MAPSTR,OBSTXT,INXSP1,INXSP2,ADTLST,
     2                  IEXP  ,IRC   )
CC
CC NAME       :  RDIXHD
CC
CC PURPOSE    :  READ IONEX HEADER
CC
CC PARAMETERS :
CC         IN :  MAXADT : MAXIMUM NUMBER OF AUX RECORDS       I*4
CC     IN/OUT :  ADTLBL : LABEL OF AUX DATA REQUESTED/FOUND   CH*60
CC                        =' ': NOT REQUESTED/FOUND
CC        OUT :  SATSTR : SATELLITE SYSTEM                    CH*20
CC               PGMSTR : PROGRAM                             CH*20
CC               AGESTR : AGENCY                              CH*20
CC               DATSTR : DATE AND TIME                       CH*20
CC               TITTXT : TITLE LINE / FIRST COMMENT LINE     CH*60
CC                        =' ': UNDEFINED
CC               MAPSTR : MAPPING FUNCTION                    CH*4
CC               OBSTXT : OBSERVABLES USED                    CH*60
CC               INXSP1 : SPECIFICATIONS 1                    R*8(*)
CC                        ( 1): EPOCH OF FIRST MAP (IN MJD)
CC                        ( 2): EPOCH OF LAST MAP (IN MJD)
CC                        ( 3): INTERVAL (IN SEC)
CC                        ( 4): FROM LATITUDE
CC                        ( 5): TO LATITUDE
CC                        ( 6): WITH INCREMENT (IN DEG)
CC                        ( 7): FROM LONGITUDE
CC                        ( 8): TO LONGITUDE
CC                        ( 9): WITH INCREMENT (IN DEG)
CC                        (10): ELEVATION CUTOFF (IN DEG)
CC                        (11): BASE RADIUS (IN KM)
CC                        (12): FROM HEIGHT
CC                        (13): TO HEIGHT
CC                        (14): WITH INCREMENT (IN KM)
CC               INXSP2 : SPECIFICATIONS 2                    I*4(*)
CC                        =0: NOT FOUND
CC                        (1): NUMBER OF STATIONS
CC                        (2): NUMBER OF SATELLITES
CC               ADTLST(I),I=1,..,MAXADT: LIST OF AUX DATA    CH*80(*)
CC               IEXP   : DEFAULT EXPONENT                    I*4
CC               IRC    : RETURN CODE                         I*4
CC                        =0: ERROR OCCURRED
CC                        =1: OK
CC
CC SR CALLED  :  DJUL
CC
CC REMARKS    :  IONEX VERSION 1.0
CC
CC AUTHOR     :  S.SCHAER
CC
CC VERSION    :  4.1
CC
CC CREATED    :  16-SEP-97             LAST MODIFIED :  09-JUN-98
CC
CC CHANGES    :  13-NOV-97 : SS: RETURN FIRST COMMENT LINE
CC               09-JUN-98 : SS: CHECK IONEX DATA GRID CORRECTLY
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1997      UNIVERSITY OF BERNE
CC                    SWITZERLAND
CC
C*
      IMPLICIT REAL*8 (A-H,O-Z)
C
      PARAMETER (NUMREC=22)
C
      CHARACTER*80  ADTLST(*),LINE
      CHARACTER*60  OBSTXT,ADTLBL,ADTLB0,TITTXT
      CHARACTER*20  SATSTR,PGMSTR,AGESTR,DATSTR,INXREC(NUMREC),AUXREC
      CHARACTER*20  INXLB0,INXLBL
      CHARACTER*4   MAPSTR
C
      REAL*8        INXSP1(*),GRDFAC(4)
C
      INTEGER*4     INXSP2(*),RECFLG(NUMREC)
C
      INCLUDE 'IONEX.inc'
C
      DATA XVER0/1.0D0/
C
C SET IONEX RECORDS
C -----------------
      DATA INXREC/'IONEX VERSION / TYPE','PGM / RUN BY / DATE ',
     1            'COMMENT             ','DESCRIPTION         ',
     2            'EPOCH OF FIRST MAP  ','EPOCH OF LAST MAP   ',
     3            'INTERVAL            ','# OF MAPS IN FILE   ',
     4            'MAPPING FUNCTION    ','ELEVATION CUTOFF    ',
     5            'OBSERVABLES USED    ','# OF STATIONS       ',
     6            '# OF SATELLITES     ','BASE RADIUS         ',
     7            'MAP DIMENSION       ','HGT1 / HGT2 / DHGT  ',
     8            'LAT1 / LAT2 / DLAT  ','LON1 / LON2 / DLON  ',
     9            'EXPONENT            ','START OF AUX DATA   ',
     1            'END OF AUX DATA     ','END OF HEADER       '/,
     2     INXLB0/'IONOSPHERE MAPS     '/
C
C INITIALIZE VARIABLES
C --------------------
      SATSTR=' '
      PGMSTR=' '
      AGESTR=' '
      DATSTR=' '
      TITTXT=' '
      MAPSTR=' '
      OBSTXT=' '
      DO I=1,12
        INXSP1(I)=0.D0
      ENDDO
      DO I=1,2
        INXSP2(I)=0
      ENDDO
      DO I=1,MAXADT
        ADTLST(I)=' '
      ENDDO
      ADTLB0=' '
      IEXP=-1
C
C SET DEFAULT RETURN CODE
      IRC=1
C
C INITIALIZE FLAGS
C ----------------
      DO I=1,NUMREC
        RECFLG(I)=0
      ENDDO
C
C SET DEFAULT FLAGS FOR OPTIONAL IONEX RECORDS
      RECFLG( 3)=-1
      RECFLG( 4)=-1
      RECFLG(12)=-1
      RECFLG(13)=-1
      RECFLG(19)=-1
      RECFLG(20)=-1
      RECFLG(21)=-1
C
C READ AND CHECK FIRST IONEX RECORD
C ---------------------------------
      READ(LFNLOC,901,END=100) LINE
901   FORMAT(A80)
C
C FIRST KEYWORD
      AUXREC=LINE(61:80)
      IF (AUXREC.NE.INXREC(1)) THEN
        WRITE(LFNERR,1901) LINE
1901    FORMAT(/,' *** SR RDIXHD: INVALID FIRST IONEX RECORD',
     1    /,A80,/)
        IRC=0
        GOTO 300
      ENDIF
C
      READ(LINE,902,ERR=200) XVER,INXLBL,SATSTR
902   FORMAT(F8.1,12X,A20,A20)
C
C IONEX VERSION NUMBER
      IF (XVER.NE.XVER0) THEN
        WRITE(LFNERR,1902) XVER,XVER0
1902    FORMAT(/,' *** SR RDIXHD: INVALID IONEX VERSION NUMBER',
     1    /,16X,'VERSION NUMBER FOUND   : ',F8.1,
     2    /,16X,'VERSION NUMBER ALLOWED : ',F8.1,/)
        IRC=0
        GOTO 300
      ENDIF
C
C IONEX LABEL
      IF (INXLBL(1:1).NE.INXLB0(1:1)) THEN
        WRITE(LFNERR,1903) INXLBL(1:1),INXLBL(2:20),INXLB0(1:1),
     1    INXLB0(2:2)
1903    FORMAT(/,' *** SR RDIXHD: INVALID IONEX LABEL',
     1    /,16X,'LABEL FOUND   : ',A1,'(',A19,')',
     2    /,16X,'LABEL ALLOWED : ',A1,'(',A19,')',/)
        IRC=0
        GOTO 300
      ENDIF
C
      RECFLG(1)=1
C
C LOOP OVER ALL HEADER LINES
C --------------------------
400   CONTINUE
      READ(LFNLOC,901,END=100) LINE
C
C CURRENT KEYWORD
      AUXREC=LINE(61:80)
C
C LOOK FOR CURRENT KEYWORD AND STORE CORRESPONDING INFORMATION
C ------------------------------------------------------------
C
C PROGRAM, AGENCY, DATE
      IF (AUXREC.EQ.INXREC(2)) THEN
        READ(LINE,903) PGMSTR,AGESTR,DATSTR
903     FORMAT(A20,A20,A20)
        RECFLG(2)=1
C
C SKIP COMMENT LINES
      ELSEIF (AUXREC.EQ.INXREC(3)) THEN
        IF (RECFLG(3).NE.1) READ(LINE,904) TITTXT
904     FORMAT(A60)
        RECFLG(3)=1
        GOTO 400
C
C SKIP DESCRIPTION LINES
      ELSEIF (AUXREC.EQ.INXREC(4)) THEN
        RECFLG(4)=1
        GOTO 400
C
C "EPOCH OF FIRST MAP"
      ELSEIF (AUXREC.EQ.INXREC(5)) THEN
        READ(LINE,911,ERR=200) IYEAR,IMONTH,IDAY,IHOUR,IMIN,ISEC
911     FORMAT(6I6)
        RECFLG(5)=1
C
        INXSP1(1)=DJUL(IYEAR,IMONTH,IDAY+IHOUR/24.D0+
     1    IMIN/1440.D0+ISEC/86400.D0)
C
C "EPOCH OF LAST MAP"
      ELSEIF (AUXREC.EQ.INXREC(6)) THEN
        READ(LINE,911,ERR=200) IYEAR,IMONTH,IDAY,IHOUR,IMIN,ISEC
        RECFLG(6)=1
C
        INXSP1(2)=DJUL(IYEAR,IMONTH,IDAY+IHOUR/24.D0+
     1    IMIN/1440.D0+ISEC/86400.D0)
C
C "INTERVAL"
      ELSEIF (AUXREC.EQ.INXREC(7)) THEN
        READ(LINE,912,ERR=200) IINT
912     FORMAT(I6)
        RECFLG(7)=1
C
        INXSP1(3)=DBLE(IINT)
C
C "# OF MAPS IN FILE"
      ELSEIF (AUXREC.EQ.INXREC(8)) THEN
        READ(LINE,912,ERR=200) NMAP
        RECFLG(8)=1
C
C "MAPPING FUNCTION"
      ELSEIF (AUXREC.EQ.INXREC(9)) THEN
        READ(LINE,921) MAPSTR
921     FORMAT(2X,A4)
        RECFLG(9)=1
C
C "ELEVATION CUTOFF"
      ELSEIF (AUXREC.EQ.INXREC(10)) THEN
        READ(LINE,922,ERR=200) INXSP1(10)
922     FORMAT(F8.1)
        RECFLG(10)=1
C
C "OBSERVABES USED"
      ELSEIF (AUXREC.EQ.INXREC(11)) THEN
        READ(LINE,931) OBSTXT
931     FORMAT(A60)
        RECFLG(11)=1
C
C "# OF STATIONS"
      ELSEIF (AUXREC.EQ.INXREC(12)) THEN
        READ(LINE,912,ERR=200) INXSP2(1)
        RECFLG(12)=1
C
C "# OF SATELLITES"
      ELSEIF (AUXREC.EQ.INXREC(13)) THEN
        READ(LINE,912,ERR=200) INXSP2(2)
        RECFLG(13)=1
C
C "BASE RADIUS"
      ELSEIF (AUXREC.EQ.INXREC(14)) THEN
        READ(LINE,922,ERR=200) INXSP1(11)
        RECFLG(14)=1
C
C "MAP DIMENSION"
      ELSEIF (AUXREC.EQ.INXREC(15)) THEN
        READ(LINE,912,ERR=200) IDIM
        RECFLG(15)=1
C
C "HGT1 / HGT2 / DHGT"
      ELSEIF (AUXREC.EQ.INXREC(16)) THEN
        READ(LINE,941,ERR=200) (INXSP1(I),I=12,14)
941     FORMAT(2X,3F6.1)
        RECFLG(16)=1
C
C "LAT1 / LAT2 / DLAT"
      ELSEIF (AUXREC.EQ.INXREC(17)) THEN
        READ(LINE,941,ERR=200) (INXSP1(I),I=4,6)
        RECFLG(17)=1
C
C "LON1 / LON2 / DLON"
      ELSEIF (AUXREC.EQ.INXREC(18)) THEN
        READ(LINE,941,ERR=200) (INXSP1(I),I=7,9)
        RECFLG(18)=1
C
C "EXPONENT"
      ELSEIF (AUXREC.EQ.INXREC(19)) THEN
        READ(LINE,912,ERR=200) IEXP
        RECFLG(19)=1
C
C AUXILIARY DATA BLOCK FOUND
      ELSEIF (AUXREC.EQ.INXREC(20)) THEN
        READ(LINE,931) ADTLB0
        RECFLG(20)=1
C
        NADT=0
401     CONTINUE
        READ(LFNLOC,901,END=100) LINE
        AUXREC=LINE(61:80)
C
C SAVE OR SKIP AUX DATA LINES
        IF (AUXREC.EQ.INXREC(21)) THEN
          RECFLG(21)=1
          GOTO 400
        ELSEIF (AUXREC.EQ.INXREC(3)) THEN
          GOTO 401
        ELSEIF (LINE.EQ.' ') THEN
          GOTO 401
        ELSEIF (ADTLBL.EQ.ADTLB0) THEN
          NADT=NADT+1
          IF (NADT.GT.MAXADT) THEN
            WRITE(LFNERR,951)
951         FORMAT(/,' *** SR RDIXHD: TOO MANY AUX DATA LINES',/)
            IRC=0
            GOTO 300
          ENDIF
          ADTLST(NADT)=LINE
        ENDIF
C
C READ NEXT AUX DATA LINE
          GOTO 401
C
C END OF HEADER FOUND
      ELSEIF (AUXREC.EQ.INXREC(22)) THEN
        RECFLG(22)=1
        GOTO 500
C
C SKIP BLANK LINES
      ELSEIF (LINE.EQ.' ') THEN
        GOTO 400
C
C SKIP UNKNOWN HEADER RECORDS
      ELSE
        WRITE(LFNERR,1911) LINE
1911    FORMAT(/,' ### SR RDIXHD: UNIDENTIFIED IONEX HEADER RECORD',
     1    /,A80,/)
        GOTO 400
      ENDIF
C
C READ NEXT HEADER LINE
C ---------------------
      GOTO 400
C
C CHECK WHETHER ALL NECESSARY IONEX HEADER RECORDS DEFINED
C --------------------------------------------------------
500   CONTINUE
      DO I=1,NUMREC
        IF (RECFLG(I).EQ.0) THEN
          WRITE(LFNERR,1921) INXREC(I)
1921      FORMAT(/,' *** SR RDIXHD: "',A20,'" RECORD MISSING',/)
          IRC=0
        ENDIF
      ENDDO
C
      IF (IRC.EQ.0) GOTO 300
C
C CHECK SOME IONEX SPECIFICATIONS
C -------------------------------
C
C TIME ARGUMENTS
      IF (IINT.EQ.0) THEN
        WRITE(LFNERR,1931)
1931    FORMAT(/,' *** SR RDIXHD: INTERVAL "0" NOT SUPPORTED',/)
        IRC=0
      ELSE
        IF (NMAP.GT.1) THEN
          IINT0=IDNINT((INXSP1(2)-INXSP1(1))/(NMAP-1)*86400.D0)
          IF (IINT.NE.IINT0) THEN
            WRITE(LFNERR,1932)
1932        FORMAT(/,' *** SR RDIXHD: INCONSISTENT TIME ARGUMENTS',/)
            IRC=0
          ENDIF
        ENDIF
      ENDIF
C
C SATELLITE SYSTEM
      IF (SATSTR(1:3).NE.'BEN' .AND.
     1    SATSTR(1:3).NE.'ENV' .AND.
     2    SATSTR(1:3).NE.'ERS' .AND.
     3    SATSTR(1:3).NE.'GEO' .AND.
     4    SATSTR(1:3).NE.'GLO' .AND.
     5    SATSTR(1:3).NE.'GNS' .AND.
     6    SATSTR(1:3).NE.'GPS' .AND.
     7    SATSTR(1:3).NE.'IRI' .AND.
     8    SATSTR(1:3).NE.'MIX' .AND.
     9    SATSTR(1:3).NE.'NNS' .AND.
     1    SATSTR(1:3).NE.'TOP') THEN
        WRITE(LFNERR,1933) SATSTR(1:3)
1933    FORMAT(/,' *** SR RDIXHD: SATELLITE SYSTEM "',A3,'" UNKNOWN',/)
        IRC=0
      ENDIF
C
C MAPPING FUNCTION
      IF (MAPSTR.NE.'NONE' .AND.
     1    MAPSTR.NE.'COSZ' .AND.
     2    MAPSTR.NE.'QFAC') THEN
        WRITE(LFNERR,1934) MAPSTR
1934    FORMAT(/,' *** SR RDIXHD: MAPPING FUNCTION "',A4,'" UNKNOWN',/)
        IRC=0
      ENDIF
C
C MAP DIMENSION
      IF (IDIM.EQ.2) THEN
        IF (INXSP1(12).NE.INXSP1(13) .OR. INXSP1(14).NE.0.D0) THEN
          WRITE(LFNERR,1935)
1935      FORMAT(/,' *** SR RDIXHD: ERROR CONCERNING MAP DIMENSION',/)
          IRC=0
        ENDIF
      ELSE
        IF (INXSP1(12).EQ.INXSP1(13) .OR. INXSP1(13).EQ.0.D0) THEN
          WRITE(LFNERR,1935)
          IRC=0
        ENDIF
      ENDIF
C
C CHECK WHETHER "LAT1", "LAT2", ... ARE MULTIPLES OF "DLAT" AND "DLON"
      GRDFAC(1)=INXSP1(4)/INXSP1(6)
      GRDFAC(2)=INXSP1(5)/INXSP1(6)
      GRDFAC(3)=INXSP1(7)/INXSP1(9)
      GRDFAC(4)=INXSP1(8)/INXSP1(9)
      DO I=1,4
        XDIF=DABS(GRDFAC(I)-DNINT(GRDFAC(I)))
        IF (XDIF.GT.1.D-4) THEN
1936      FORMAT(/,' *** SR RDIXHD: IRREGULAR IONEX DATA GRID',/)
          IRC=0
        ENDIF
      ENDDO
C
C SAVE LABEL OF AUX DATA FOUND
C ----------------------------
      IF (ADTLBL.NE.' ' .AND. ADTLBL.NE.ADTLB0) THEN
        WRITE(LFNERR,1941)
1941    FORMAT(/,' ### SR RDIXHD: REQUESTED AUX DATA NOT FOUND',/)
      ENDIF
C
      ADTLBL=ADTLB0
C
      GOTO 300
C
C END OF FILE REACHED
C -------------------
100   CONTINUE
      WRITE(LFNERR,1951)
1951  FORMAT(/,' *** SR RDIXHD: END OF FILE REACHED',/)
C
      IRC=0
      GOTO 300
C
C ERROR READING IONEX FILE
C ------------------------
200   CONTINUE
      WRITE(LFNERR,1952) LINE
1952  FORMAT(/,' *** SR RDIXHD: ERROR READING IONEX FILE',
     1  /,A80,/)
C
      IRC=0
      GOTO 300
C
C RETURN
C ------
300   CONTINUE
C
      RETURN
      END
C*
      SUBROUTINE WTIXDT(NUMVAL,IEXP,TECVAL)
CC
CC NAME       :  WTIXDT
CC
CC PURPOSE    :  WRITE IONEX DATA
CC
CC PARAMETERS :
CC         IN :  NUMVAL : NUMBER OF TEC/RMS VALUES            I*4
CC               IEXP   : ACTIVE EXPONENT                     I*4
CC               TECVAL : TEC/RMS VALUES (IN TECU)            R*8(*)
CC                        =999.9: UNDEFINED
CC
CC SR CALLED  :  ---
CC
CC REMARKS    :  IONEX VERSION 1.0
CC
CC AUTHOR     :  S.SCHAER
CC
CC VERSION    :  4.1
CC
CC CREATED    :  10-SEP-97             LAST MODIFIED :  05-JAN-98
CC
CC CHANGES    :  05-JAN-98 : SS: "AUXVAL" REDIMENSIONED
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1997      UNIVERSITY OF BERNE
CC                    SWITZERLAND
CC
C*
      IMPLICIT REAL*8 (A-H,O-Z)
C
      REAL*8        TECVAL(*)
C
      INTEGER*4     AUXVAL(16)
C
      INCLUDE 'IONEX.inc'
C
C COMPUTE NUMBER OF DATA LINES TO BE WRITTEN
C ------------------------------------------
      NLIN=(NUMVAL-1)/16+1
C
C WRITE SINGLE TEC/RMS DATA BLOCK
C -------------------------------
      DO ILIN=1,NLIN
        IVAL1=16*ILIN-15
        IVAL2=16*ILIN
        IF (IVAL2.GT.NUMVAL) IVAL2=NUMVAL
C
C CONVERT TEC/RMS VALUES FROM TECU INTO ACTIVE UNIT
C -------------------------------------------------
        NAUX=IVAL2-IVAL1+1
        DO IAUX=1,NAUX
          IVAL=IVAL1+IAUX-1
          IF (TECVAL(IVAL).NE.999.9D0) THEN
            AUXVAL(IAUX)=IDNINT(10.D0**(-IEXP)*TECVAL(IVAL))
          ELSE
            AUXVAL(IAUX)=9999
          ENDIF
        ENDDO
C
        WRITE(LFNLOC,900) (AUXVAL(IAUX),IAUX=1,NAUX)
900     FORMAT(16I5)
      ENDDO
C
      RETURN
      END
C*
      SUBROUTINE WTIXFL(INXFIL,MAXVAL,MAXLIN,MAXADT,SATSTR,PGMSTR,
     1                  AGESTR,DATSTR,TITTXT,DESTXT,MAPSTR,OBSTXT,
     2                  COMTXT,INXSP1,INXSP2,ADTLBL,ADTLST,INXINF,
     3                  TECMAP,IEXP0 )
CC
CC NAME       :  WTIXFL
CC
CC PURPOSE    :  WRITE IONEX FILE
CC
CC PARAMETERS :
CC         IN :  INXFIL : EXTERNAL IONEX OUTPUT FILE NAME     CH*32
CC               MAXVAL : MAXIMUM NUMBER OF TEC/RMS VALUES    I*4
CC               MAXLIN : MAXIMUM NUMBER OF DES/COM LINES     I*4
CC               MAXADT : MAXIMUM NUMBER OF AUX RECORDS       I*4
CC               SATSTR : SATELLITE SYSTEM                    CH*20
CC               PGMSTR : PROGRAM                             CH*20
CC               AGESTR : AGENCY                              CH*20
CC               DATSTR : DATE AND TIME                       CH*20
CC               TITTXT : TITLE LINE                          CH*60
CC                        =' ': UNDEFINED
CC               DESTXT(I),I=1,..,MAXLIN: MULTI-LINE          CH*60(*)
CC                        DESCRIPTION
CC                        =' ': UNDEFINED
CC               MAPSTR : MAPPING FUNCTION                    CH*4
CC               OBSTXT : OBSERVABLES USED                    CH*60
CC               COMTXT(I),I=1,..,MAXLIN: MULTI-LINE COMMENT  CH*60(*)
CC                        =' ': UNDEFINED
CC               INXSP1 : SPECIFICATIONS 1                    R*8(*)
CC                        ( 1): EPOCH OF FIRST MAP (IN MJD)
CC                        ( 2): EPOCH OF LAST MAP (IN MJD)
CC                        ( 3): INTERVAL (IN SEC)
CC                        ( 4): FROM LATITUDE
CC                        ( 5): TO LATITUDE
CC                        ( 6): WITH INCREMENT (IN DEG)
CC                        ( 7): FROM LONGITUDE
CC                        ( 8): TO LONGITUDE
CC                        ( 9): WITH INCREMENT (IN DEG)
CC                        (10): ELEVATION CUTOFF (IN DEG)
CC                        (11): BASE RADIUS (IN KM)
CC                        (12): FROM HEIGHT
CC                        (13): TO HEIGHT
CC                        (14): WITH INCREMENT (IN KM)
CC               INXSP2 : SPECIFICATIONS 2                    I*4(*)
CC                        =0: UNDEFINED
CC                        (1): NUMBER OF STATIONS
CC                        (2): NUMBER OF SATELLITES
CC               ADTLBL : LABEL OF AUX DATA                   CH*60
CC                        =' ': NO AUX DATA
CC               ADTLST(I),I=1,..,MAXADT: LIST OF AUX DATA    CH*80(*)
CC                        =' ': UNDEFINED
CC               INXINF : INFORMATION TO BE SAVED             I*4(*)
CC                        =0/1: NO/YES
CC                        (1): TEC MAPS
CC                        (2): RMS MAPS
CC               TECMAP(I,J),I=1,..,MAXVAL,J=1,2: TEC/RMS MAP R*8(*,*)
CC                        (IN TECU)
CC                        =999.9: UNDEFINED
CC               IEXP0  : DEFAULT EXPONENT                    I*4
CC                        =-1: RECOMMENDED VALUE
CC                        =99: EXPONENT AUTOMATICALLY ADAPTED
CC
CC SR CALLED  :  OPNERR, OPNFIL, JMT   , RADGMS, WTIXDT, WTIXHD
CC
CC REMARKS    :  IONEX VERSION 1.0
CC
CC AUTHOR     :  S.SCHAER
CC
CC VERSION    :  4.1
CC
CC CREATED    :  16-SEP-97             LAST MODIFIED :  23-AUG-99
CC
CC CHANGES    :  01-OCT-97 : SS: "IEXP0=99" AS SPECIAL OPTION
CC               23-AUG-99 : SS: DECLARE "VORZ" AS CH*1
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1997      UNIVERSITY OF BERNE
CC                    SWITZERLAND
CC
C*
      IMPLICIT REAL*8 (A-H,O-Z)
C
      CHARACTER*80  ADTLST(*)
      CHARACTER*60  TITTXT,DESTXT(*),OBSTXT,COMTXT(*),ADTLBL
      CHARACTER*320  INXFIL
      CHARACTER*20  SATSTR,PGMSTR,AGESTR,DATSTR,INXREC(6),AUXREC
      CHARACTER*4   MAPSTR
      CHARACTER*3   MAPTYP(2)
      CHARACTER*1   VORZ
C
      REAL*8        INXSP1(*),TECMAP(MAXVAL,*)
C
      INTEGER*4     INXSP2(*),INXINF(*)
C
      INCLUDE 'IONEX.inc'
C
      DATA MAPTYP/'TEC','RMS'/
C
C SET IONEX RECORDS
C -----------------
      DATA INXREC/'START OF     MAP    ','EPOCH OF CURRENT MAP',
     1            'EXPONENT            ','LAT/LON1/LON2/DLON/H',
     2            'END OF     MAP      ','END OF FILE         '/
C
C OPEN IONEX OUTPUT FILE
C ----------------------
c      CALL OPNFIL(LFNLOC,INXFIL,'UNKNOWN','FORMATTED',
c     1  ' ',' ',IOSTAT)
c      CALL OPNERR(LFNERR,LFNLOC,IOSTAT,INXFIL,'WTIXFL')
      
      open(LFNLOC,file=INXFIL,status='UNKNOWN',form='formatted')
C
C WRITE IONEX HEADER
C ------------------
      CALL WTIXHD(MAXLIN,MAXADT,SATSTR,PGMSTR,AGESTR,DATSTR,
     1            TITTXT,DESTXT,MAPSTR,OBSTXT,COMTXT,INXSP1,
     2            INXSP2,ADTLBL,ADTLST,IEXP0 )
C
C WRITE TEX/RMS MAPS
C ------------------
      XINT=INXSP1(3)/86400.D0
      NMAP=IDNINT((INXSP1(2)-INXSP1(1))/XINT)+1
C
      NLAT=IDNINT((INXSP1(5)-INXSP1(4))/INXSP1(6))+1
      NLON=IDNINT((INXSP1(8)-INXSP1(7))/INXSP1(9))+1
C
      IF (INXSP1(14).EQ.0.D0) THEN
        NHGT=1
      ELSE
        NHGT=IDNINT((INXSP1(13)-INXSP1(12))/INXSP1(14))+1
      ENDIF
C
      DO 110 ITYP=1,2
        IF (INXINF(ITYP).EQ.0) GOTO 110
        IVAL=1
C
        DO 120 IMAP=1,NMAP
          XEPO=INXSP1(1)+(IMAP-1)*XINT
C
          CALL JMT(XEPO,IYEAR,IMONTH,XDAY)
          IDAY=IDINT(XDAY)
          CALL RADGMS(3,XDAY,VORZ,IHOUR,IMIN,XSEC)
          ISEC=IDNINT(XSEC)
C
C WRITE RECORD OPENING TEC/RMS MAP
          AUXREC=INXREC(1)
          WRITE(AUXREC(10:12),'(A3)') MAPTYP(ITYP)
C
          WRITE(LFNLOC,910) IMAP,AUXREC
910       FORMAT(I6,54X,A20)
C
C WRITE EPOCH OF CURRENT TEC/RMS MAP
          WRITE(LFNLOC,920) IYEAR,IMONTH,IDAY,IHOUR,IMIN,ISEC,
     1      INXREC(2)
920       FORMAT(6I6,24X,A20)
C
          DO 130 IHGT=1,NHGT
            XHGT=INXSP1(12)+(IHGT-1)*INXSP1(14)
C
C AUTOMATICALLY ADAPT EXPONENT
            IF (IEXP0.EQ.99) THEN
              IEXP=-99
              DO 135 ITEC=IVAL,IVAL+NLAT*NLON-1
                XTEC=TECMAP(ITEC,ITYP)
                IF (XTEC.NE.0.D0 .AND. XTEC.NE.999.9D0)
     1            IEXP=MAX0(IDINT(DLOG10(DABS(XTEC)/9998.D0)-
     2            100)+100,IEXP)
135           CONTINUE
              IF (IEXP.EQ.-99) IEXP=-1
C
C WRITE ACTIVE EXPONENT
              WRITE(LFNLOC,910) IEXP,INXREC(3)
            ELSE
              IEXP=IEXP0
            ENDIF
C
            DO 140 ILAT=1,NLAT
              XLAT=INXSP1(4)+(ILAT-1)*INXSP1(6)
C
C WRITE TEC/RMS DATA SEQUENCE
              WRITE(LFNLOC,930) XLAT,(INXSP1(I),I=7,9),XHGT,INXREC(4)
930           FORMAT(2X,5F6.1,28X,A20)
C
              CALL WTIXDT(NLON,IEXP,TECMAP(IVAL,ITYP))
              IVAL=IVAL+NLON
140         CONTINUE
130       CONTINUE
C
C WRITE RECORD CLOSING TEC/RMS MAP
          AUXREC=INXREC(5)
          WRITE(AUXREC(8:10),'(A3)') MAPTYP(ITYP)
C
          WRITE(LFNLOC,940) IMAP,AUXREC
940       FORMAT(I6,54X,A20)
120     CONTINUE
110   CONTINUE
C
C WRITE LAST RECORD CLOSING IONEX OUTPUT FILE
C -------------------------------------------
      WRITE(LFNLOC,900) INXREC(6)
900   FORMAT(60X,A20)
C
C CLOSE IONEX OUTPUT FILE
C -----------------------
      CLOSE (UNIT=LFNLOC)
C
      RETURN
      END
C*
      SUBROUTINE WTIXHD(MAXLIN,MAXADT,SATSTR,PGMSTR,AGESTR,DATSTR,
     1                  TITTXT,DESTXT,MAPSTR,OBSTXT,COMTXT,INXSP1,
     2                  INXSP2,ADTLBL,ADTLST,IEXP0 )
CC
CC NAME       :  WTIXHD
CC
CC PURPOSE    :  WRITE IONEX HEADER
CC
CC PARAMETERS :
CC         IN :  MAXLIN : MAXIMUM NUMBER OF DES/COM LINES     I*4
CC               MAXADT : MAXIMUM NUMBER OF AUX RECORDS       I*4
CC               SATSTR : SATELLITE SYSTEM                    CH*20
CC               PGMSTR : PROGRAM                             CH*20
CC               AGESTR : AGENCY                              CH*20
CC               DATSTR : DATE AND TIME                       CH*20
CC               TITTXT : TITLE LINE                          CH*60
CC                        =' ': UNDEFINED
CC               DESTXT(I),I=1,..,MAXLIN: MULTI-LINE          CH*60(*)
CC                        DESCRIPTION
CC                        =' ': UNDEFINED
CC               MAPSTR : MAPPING FUNCTION                    CH*4
CC               OBSTXT : OBSERVABLES USED                    CH*60
CC               COMTXT(I),I=1,..,MAXLIN: MULTI-LINE COMMENT  CH*60(*)
CC                        =' ': UNDEFINED
CC               INXSP1 : SPECIFICATIONS 1                    R*8(*)
CC                        ( 1): EPOCH OF FIRST MAP (IN MJD)
CC                        ( 2): EPOCH OF LAST MAP (IN MJD)
CC                        ( 3): INTERVAL (IN SEC)
CC                        ( 4): FROM LATITUDE
CC                        ( 5): TO LATITUDE
CC                        ( 6): WITH INCREMENT (IN DEG)
CC                        ( 7): FROM LONGITUDE
CC                        ( 8): TO LONGITUDE
CC                        ( 9): WITH INCREMENT (IN DEG)
CC                        (10): ELEVATION CUTOFF (IN DEG)
CC                        (11): BASE RADIUS (IN KM)
CC                        (12): FROM HEIGHT
CC                        (13): TO HEIGHT
CC                        (14): WITH INCREMENT (IN KM)
CC               INXSP2 : SPECIFICATIONS 2                    I*4(*)
CC                        =0: UNDEFINED
CC                        (1): NUMBER OF STATIONS
CC                        (2): NUMBER OF SATELLITES
CC               ADTLBL : LABEL OF AUX DATA                   CH*60
CC                        =' ': NO AUX DATA
CC               ADTLST(I),I=1,..,MAXADT: LIST OF AUX DATA    CH*80(*)
CC                        =' ': UNDEFINED
CC               IEXP0  : DEFAULT EXPONENT                    I*4
CC                        =-1: RECOMMENDED VALUE
CC                        =99: EXPONENT AUTOMATICALLY ADAPTED
CC
CC SR CALLED  :  JMT   , RADGMS
CC
CC REMARKS    :  IONEX VERSION 1.0
CC
CC AUTHOR     :  S.SCHAER
CC
CC VERSION    :  4.1
CC
CC CREATED    :  11-SEP-97             LAST MODIFIED :  23-AUG-99
CC
CC CHANGES    :  01-OCT-97 : SS: "IEXP0=99" AS SPECIAL OPTION
CC               23-AUG-99 : SS: DECLARE "VORZ" AS CH*1
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1997      UNIVERSITY OF BERNE
CC                    SWITZERLAND
CC
C*
      IMPLICIT REAL*8 (A-H,O-Z)
C
      CHARACTER*80  ADTLST(*)
      CHARACTER*60  TITTXT,DESTXT(*),OBSTXT,COMTXT(*),ADTLBL
      CHARACTER*20  SATSTR,PGMSTR,AGESTR,DATSTR,INXLBL,INXREC(22)
      CHARACTER*4   MAPSTR
      CHARACTER*1   VORZ
C
      REAL*8        INXSP1(*)
C
      INTEGER*4     INXSP2(*)
C
      INCLUDE 'IONEX.inc'
C
      DATA XVER/1.0D0/
C
C SET IONEX RECORDS
C -----------------
      DATA INXREC/'IONEX VERSION / TYPE','PGM / RUN BY / DATE ',
     1            'COMMENT             ','DESCRIPTION         ',
     2            'EPOCH OF FIRST MAP  ','EPOCH OF LAST MAP   ',
     3            'INTERVAL            ','# OF MAPS IN FILE   ',
     4            'MAPPING FUNCTION    ','ELEVATION CUTOFF    ',
     5            'OBSERVABLES USED    ','# OF STATIONS       ',
     6            '# OF SATELLITES     ','BASE RADIUS         ',
     7            'MAP DIMENSION       ','HGT1 / HGT2 / DHGT  ',
     8            'LAT1 / LAT2 / DLAT  ','LON1 / LON2 / DLON  ',
     9            'EXPONENT            ','START OF AUX DATA   ',
     1            'END OF AUX DATA     ','END OF HEADER       '/,
     2     INXLBL/'IONOSPHERE MAPS     '/
C
C WRITE FIRST IONEX RECORD
C ------------------------
      WRITE(LFNLOC,911) XVER,INXLBL,SATSTR,INXREC(1)
911   FORMAT(F8.1,12X,A20,A20,A20)
C
      WRITE(LFNLOC,912) PGMSTR,AGESTR,DATSTR,INXREC(2)
912   FORMAT(3A20,A20)
C
C WRITE TITLE
C -----------
      IF (TITTXT.NE.' ') WRITE(LFNLOC,921) TITTXT,INXREC(3)
921   FORMAT(A60,A20)
C
C WRITE MULTI-LINE DESCRIPTION
C ----------------------------
      DO ILIN=1,MAXLIN
        IF (DESTXT(ILIN).EQ.' ') GOTO 110
        WRITE(LFNLOC,922) DESTXT(ILIN),INXREC(4)
922     FORMAT(A60,A20)
      ENDDO
110   CONTINUE
C
C WRITE EPOCH OF FIRST/LAST MAP
C -----------------------------
      CALL JMT(INXSP1(1),IYEAR,IMONTH,XDAY)
      IDAY=IDINT(XDAY)
      CALL RADGMS(3,XDAY,VORZ,IHOUR,IMIN,XSEC)
      ISEC=IDNINT(XSEC)
      WRITE(LFNLOC,931) IYEAR,IMONTH,IDAY,IHOUR,IMIN,ISEC,INXREC(5)
931   FORMAT(6I6,24X,A20)
C
      CALL JMT(INXSP1(2),IYEAR,IMONTH,XDAY)
      IDAY=IDINT(XDAY)
      CALL RADGMS(3,XDAY,VORZ,IHOUR,IMIN,XSEC)
      ISEC=IDNINT(XSEC)
      WRITE(LFNLOC,932) IYEAR,IMONTH,IDAY,IHOUR,IMIN,ISEC,INXREC(6)
932   FORMAT(6I6,24X,A20)
C
      IINT=IDNINT(INXSP1(3))
      WRITE(LFNLOC,933) IINT,INXREC(7)
933   FORMAT(I6,54X,A20)
C
      XINT=INXSP1(3)/86400.D0
      NMAP=IDNINT((INXSP1(2)-INXSP1(1))/XINT)+1
      WRITE(LFNLOC,934) NMAP,INXREC(8)
934   FORMAT(I6,54X,A20)
C
C WRITE MAPPING FUNCTION, ELEVATION CUTOFF, OBSERVABES USED
C ---------------------------------------------------------
      WRITE(LFNLOC,941) MAPSTR,INXREC(9)
941   FORMAT(2X,A4,54X,A20)
C
      WRITE(LFNLOC,942) INXSP1(10),INXREC(10)
942   FORMAT(F8.1,52X,A20)
C
      WRITE(LFNLOC,943) OBSTXT,INXREC(11)
943   FORMAT(A60,A20)
C
C WRITE NUMBER OF STATIONS/SATELLITES
C -----------------------------------
      IF (INXSP2(1).GT.0) WRITE(LFNLOC,951) INXSP2(1),INXREC(12)
951   FORMAT(I6,54X,A20)
C
      IF (INXSP2(2).GT.0) WRITE(LFNLOC,952) INXSP2(2),INXREC(13)
952   FORMAT(I6,54X,A20)
C
C WRITE BASE RADIUS, MAP DIMENSION
C --------------------------------
      WRITE(LFNLOC,961) INXSP1(11),INXREC(14)
961   FORMAT(F8.1,52X,A20)
C
      IF (INXSP1(14).EQ.0.D0) THEN
        IDIM=2
      ELSE
        IDIM=3
      ENDIF
      WRITE(LFNLOC,962) IDIM,INXREC(15)
962   FORMAT(I6,54X,A20)
C
C WRITE GRID SPECIFICATIONS
C -------------------------
      WRITE(LFNLOC,971) (INXSP1(I),I=12,14),INXREC(16)
971   FORMAT(2X,3F6.1,40X,A20)
C
      WRITE(LFNLOC,972) (INXSP1(I),I=4,6),INXREC(17)
972   FORMAT(2X,3F6.1,40X,A20)
C
      WRITE(LFNLOC,973) (INXSP1(I),I=7,9),INXREC(18)
973   FORMAT(2X,3F6.1,40X,A20)
C
C WRITE DEFAULT EXPONENT AND MULTI-LINE COMMENT
C ---------------------------------------------
      IF (IEXP0.NE.99) WRITE(LFNLOC,981) IEXP0,INXREC(19)
981   FORMAT(I6,54X,A20)
C
      DO ILIN=1,MAXLIN
        IF (COMTXT(ILIN).EQ.' ') GOTO 120
        WRITE(LFNLOC,982) COMTXT(ILIN),INXREC(3)
982     FORMAT(A60,A20)
      ENDDO
120   CONTINUE
C
C WRITE AUX DATA
C --------------
      IF (ADTLBL.NE.' ' .AND. ADTLST(1).NE.' ') THEN
        WRITE(LFNLOC,983) ADTLBL,INXREC(20)
983     FORMAT(A60,A20)
C
        DO IADT=1,MAXADT
          IF (ADTLST(IADT).EQ.' ') GOTO 130
          WRITE(LFNLOC,984) ADTLST(IADT)
984       FORMAT(A80)
        ENDDO
130     CONTINUE
C
        WRITE(LFNLOC,985) ADTLBL,INXREC(21)
985     FORMAT(A60,A20)
      ENDIF
C
C WRITE RECORD CLOSING IONEX HEADER
C ---------------------------------
      WRITE(LFNLOC,991) INXREC(22)
991   FORMAT(60X,A20)
C
      RETURN
      END
