c*****************************************************************      
      subroutine get_kfactor( lat, incidence_angle_deg, i_beam, 
     $ i_dir, kfactor )
c*****************************************************************      
c**   
c**   FILE NAME: kfactor_subs.f
c**   
c**   DATE WRITTEN:  2/4/2009
c**   
c**   PROGRAMMER:  Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  modularization of existing code:
c**    removed code from main program and made a subroutine out
c**    of existing code.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES:
c**
c**   UPDATE LOG:
c**    Changed Kfactor to -999 if out of range.
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**   2/26/2009           Add comments           
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 lat
      real*8 incidence_angle_deg
      integer i_beam            ! 1, 2, or 3
      integer i_dir                     ! 0 ascending, 1 descending
c     OUTPUT VARIABLES
      real*8 kfactor(3)         ! order HH VH VV
c     LOCAL VARIABLES
      integer bia, tia, blat, tlat     
      real*8 ia, lati
      real*8 iabottom, iatop
c     DATA STATEMENTS
c     COMMON BLOCKS
      real*8 kfactorb1TableA(164,27,5), kfactorb2TableA(164,28,5), kfactorb3TableA(164,30,5) !Arrays containing the K factor tables; two fro beam: Ascending and Descending pass
      real*8 kfactorb1TableD(163,27,5), kfactorb2TableD(163,28,5), kfactorb3TableD(163,30,5)
      common /kfactor_block/ kfactorb1TableA, kfactorb2TableA, 
     $ kfactorb3TableA, kfactorb1TableD, kfactorb2TableD, kfactorb3TableD
c     FUNCTION STATEMENTS
      integer r8_floor
c     PROCESSING STEPS
      
      if( i_beam.eq.1 .and. i_dir.eq.0)then ! Asceding orbit....
         lati = kfactorb1TableA(1,1,1) !first latitude in table
         ia  = kfactorb1TableA(1,1,2) !first incidence angle in table
         !bia = int(incidence_angle_deg-ia - mod(incidence_angle_deg-ia,1.0d0)) + 1 !Locate incidence angle cell in table
         bia = r8_floor(incidence_angle_deg-ia) + 1 !Locate incidence angle cell in table
         tia = bia + 1
         !blat = int(lat-lati - mod(lat-lati,1.0)) + 1 !Locate latitude cell in table
         blat = r8_floor(lat-lati) + 1 !Locate latitude cell in table
         tlat = blat + 1
         
         ! check that bia and tia are in range AGF 3/2/2011
         if( bia.lt.1 .or. bia.gt.26 .or. 
     $       tia.lt.2 .or. tia.gt.27 ) then
           write(6,*) 'WARNING: get_kfactor: incidence angle out of range: ', incidence_angle_deg, bia, tia
           call flush(6)
           kfactor(1) = -999
           kfactor(2) = -999
           kfactor(3) = -999
           return
         endif
         
c     Performing bilinear interpolation in latitude and incidence angle for three polarizations
         
         if(tlat.le.164) then   !If latitude is outside boundary I set its value to the value of the maximun latitude in table
            iatop=(kfactorb1TableA(tlat,tia,1)- lat)
     &  /(kfactorb1TableA(tlat,tia,1)-kfactorb1TableA(blat,tia,1))
     &           *(kfactorb1TableA(blat,tia,3)-kfactorb1TableA(tlat,tia,3))+kfactorb1TableA(tlat,tia,3)
            
            iabottom=(kfactorb1TableA(tlat,bia,1)- lat)
     &  /(kfactorb1TableA(tlat,bia,1)-kfactorb1TableA(blat,bia,1))
     &           *(kfactorb1TableA(blat,bia,3)-kfactorb1TableA(tlat,bia,3))+kfactorb1TableA(tlat,bia,3)
         else 
            tlat = blat
            iatop=kfactorb1TableA(tlat,tia,3)
            iabottom=kfactorb1TableA(tlat,bia,3)
         endif
         
         kfactor(1) = (kfactorb1TableA(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb1TableA(tlat,tia,2)-kfactorb1TableA(tlat,bia,2))*(iabottom-iatop)+iatop ! HH pol
         
         
         tlat = blat + 1
         if(tlat.le.164) then
            iatop=(kfactorb1TableA(tlat,tia,1)- lat)
     &  /(kfactorb1TableA(tlat,tia,1)-kfactorb1TableA(blat,tia,1))
     &           *(kfactorb1TableA(blat,tia,4)-kfactorb1TableA(tlat,tia,4))+kfactorb1TableA(tlat,tia,4)
            
            iabottom=(kfactorb1TableA(tlat,bia,1)- lat)
     &  /(kfactorb1TableA(tlat,bia,1)-kfactorb1TableA(blat,bia,1))
     &           *(kfactorb1TableA(blat,bia,4)-kfactorb1TableA(tlat,bia,4))+kfactorb1TableA(tlat,bia,4)
         else
            tlat = blat
            iatop=kfactorb1TableA(tlat,tia,4)
            iabottom=kfactorb1TableA(tlat,bia,4)
         endif
         kfactor(2) = (kfactorb1TableA(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb1TableA(tlat,tia,2)-kfactorb1TableA(tlat,bia,2))*(iabottom-iatop)+iatop  ! VH pol
         
         tlat = blat + 1
         if(tlat.le.164) then
            iatop=(kfactorb1TableA(tlat,tia,1)- lat)
     &  /(kfactorb1TableA(tlat,tia,1)-kfactorb1TableA(blat,tia,1))
     &           *(kfactorb1TableA(blat,tia,5)-kfactorb1TableA(tlat,tia,5))+kfactorb1TableA(tlat,tia,5)
            
            iabottom=(kfactorb1TableA(tlat,bia,1)- lat)
     &  /(kfactorb1TableA(tlat,bia,1)-kfactorb1TableA(blat,bia,1))
     &           *(kfactorb1TableA(blat,bia,5)-kfactorb1TableA(tlat,bia,5))+kfactorb1TableA(tlat,bia,5)
         else
            tlat = blat
            iatop=kfactorb1TableA(tlat,tia,5)
            iabottom=kfactorb1TableA(tlat,bia,5)
         endif
         kfactor(3) = (kfactorb1TableA(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb1TableA(tlat,tia,2)-kfactorb1TableA(tlat,bia,2))*(iabottom-iatop)+iatop  ! VV pol
         
      elseif( i_beam.eq.2 .and. i_dir.eq.0)then ! Asceding orbit....
                 lati = kfactorb2TableA(1,1,1)
                 ia  = kfactorb2TableA(1,1,2)
                 
                 !bia = int(incidence_angle_deg-ia - mod(incidence_angle_deg-ia,1.0)) + 1
                 bia = r8_floor(incidence_angle_deg-ia) + 1
                 tia = bia + 1
                 !blat = int(lat-lati - mod(lat-lati,1.0)) + 1
                 blat = r8_floor(lat-lati) + 1
                 tlat = blat + 1

         ! check that bia and tia are in range AGF 3/2/2011
         if( bia.lt.1 .or. bia.gt.27 .or. 
     $       tia.lt.2 .or. tia.gt.28 ) then
           write(6,*) 'WARNING: get_kfactor: incidence angle out of range: ', incidence_angle_deg, bia, tia
           call flush(6)
           kfactor(1) = -999
           kfactor(2) = -999
           kfactor(3) = -999
           return
         endif

                 
                 if(tlat.le.164) then
                    iatop=(kfactorb2TableA(tlat,tia,1)- lat)
     &  /(kfactorb2TableA(tlat,tia,1)-kfactorb2TableA(blat,tia,1))
     &                   *(kfactorb2TableA(blat,tia,3)-kfactorb2TableA(tlat,tia,3))+kfactorb2TableA(tlat,tia,3)
                    
                    iabottom=(kfactorb2TableA(tlat,bia,1)- lat)
     &  /(kfactorb2TableA(tlat,bia,1)-kfactorb2TableA(blat,bia,1))
     &                   *(kfactorb2TableA(blat,bia,3)-kfactorb2TableA(tlat,bia,3))+kfactorb2TableA(tlat,bia,3)
                 else
                    tlat = blat
                    iatop=kfactorb2TableA(tlat,tia,3)
                    iabottom=kfactorb2TableA(tlat,bia,3)
                 endif
                 
                 kfactor(1) = (kfactorb2TableA(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb2TableA(tlat,tia,2)-kfactorb2TableA(tlat,bia,2))*(iabottom-iatop)+iatop
                 
                 
                 tlat = blat + 1
                 if(tlat.le.164) then
                    iatop=(kfactorb2TableA(tlat,tia,1)- lat)
     &  /(kfactorb2TableA(tlat,tia,1)-kfactorb2TableA(blat,tia,1))
     &                   *(kfactorb2TableA(blat,tia,4)-kfactorb2TableA(tlat,tia,4))+kfactorb2TableA(tlat,tia,4)
                    
                    iabottom=(kfactorb2TableA(tlat,bia,1)- lat)
     &  /(kfactorb2TableA(tlat,bia,1)-kfactorb2TableA(blat,bia,1))
     &                   *(kfactorb2TableA(blat,bia,4)-kfactorb2TableA(tlat,bia,4))+kfactorb2TableA(tlat,bia,4)
                 else
                    tlat = blat
                    iatop=kfactorb2TableA(tlat,tia,4)
                    iabottom=kfactorb2TableA(tlat,bia,4)
                 endif
                 kfactor(2) = (kfactorb2TableA(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb2TableA(tlat,tia,2)-kfactorb2TableA(tlat,bia,2))*(iabottom-iatop)+iatop
                 
                 tlat = blat + 1
                 if(tlat.le.164) then
                    iatop=(kfactorb2TableA(tlat,tia,1)- lat)
     &  /(kfactorb2TableA(tlat,tia,1)-kfactorb2TableA(blat,tia,1))
     &                   *(kfactorb2TableA(blat,tia,5)-kfactorb2TableA(tlat,tia,5))+kfactorb2TableA(tlat,tia,5)
                    
                    iabottom=(kfactorb2TableA(tlat,bia,1)- lat)
     &  /(kfactorb2TableA(tlat,bia,1)-kfactorb2TableA(blat,bia,1))
     &                   *(kfactorb2TableA(blat,bia,5)-kfactorb2TableA(tlat,bia,5))+kfactorb2TableA(tlat,bia,5)
                    
                 else
                    tlat = blat
                    iatop=kfactorb2TableA(tlat,tia,5)
                    iabottom=kfactorb2TableA(tlat,bia,5)
                 endif
                    kfactor(3) = (kfactorb2TableA(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb2TableA(tlat,tia,2)-kfactorb2TableA(tlat,bia,2))*(iabottom-iatop)+iatop            
      
      elseif( i_beam.eq.3 .and. i_dir.eq.0)then ! Asceding orbit....      
                 lati = kfactorb3TableA(1,1,1)
                 ia  = kfactorb3TableA(1,1,2)
                 
                 !bia = int(incidence_angle_deg-ia - mod(incidence_angle_deg-ia,1.0)) + 1
                 bia = r8_floor(incidence_angle_deg-ia) + 1
                 tia = bia + 1
                 !blat = int(lat-lati - mod(lat-lati,1.0)) + 1
                 blat = r8_floor(lat-lati) + 1
                 tlat = blat + 1

         ! check that bia and tia are in range AGF 3/2/2011
         if( bia.lt.1 .or. bia.gt.29 .or. 
     $       tia.lt.2 .or. tia.gt.30 ) then
           write(6,*) 'WARNING: get_kfactor: incidence angle out of range: ', incidence_angle_deg, bia, tia
           call flush(6)
           kfactor(1) = -999
           kfactor(2) = -999
           kfactor(3) = -999
           return
         endif

                 
                 if(tlat.le.164) then
                    iatop=(kfactorb3TableA(tlat,tia,1)- lat)
     &  /(kfactorb3TableA(tlat,tia,1)-kfactorb3TableA(blat,tia,1))
     &                   *(kfactorb3TableA(blat,tia,3)-kfactorb3TableA(tlat,tia,3))+kfactorb3TableA(tlat,tia,3)
                    
                    iabottom=(kfactorb3TableA(tlat,bia,1)- lat)
     &  /(kfactorb3TableA(tlat,bia,1)-kfactorb3TableA(blat,bia,1))
     &                   *(kfactorb3TableA(blat,bia,3)-kfactorb3TableA(tlat,bia,3))+kfactorb3TableA(tlat,bia,3)
                 else
                    tlat = blat
                    iatop=kfactorb3TableA(tlat,tia,3)
                    iabottom=kfactorb3TableA(tlat,bia,3)
                 endif
                 
                 kfactor(1) = (kfactorb3TableA(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb3TableA(tlat,tia,2)-kfactorb3TableA(tlat,bia,2))*(iabottom-iatop)+iatop
c     write(6,*) 'Asc: ', kfactor(1), incidence_angle_deg,tlat,tia,ia ,mod(incidence_angle_deg-ia,1)
c     pause
                 
                 tlat = blat + 1
                 if(tlat.le.164) then
                    iatop=(kfactorb3TableA(tlat,tia,1)- lat)
     &  /(kfactorb3TableA(tlat,tia,1)-kfactorb3TableA(blat,tia,1))
     &                   *(kfactorb3TableA(blat,tia,4)-kfactorb3TableA(tlat,tia,4))+kfactorb3TableA(tlat,tia,4)
                    
                    iabottom=(kfactorb3TableA(tlat,bia,1)- lat)
     &  /(kfactorb3TableA(tlat,bia,1)-kfactorb3TableA(blat,bia,1))
     &                   *(kfactorb3TableA(blat,bia,4)-kfactorb3TableA(tlat,bia,4))+kfactorb3TableA(tlat,bia,4)
                 else
                    tlat = blat
                    iatop=kfactorb3TableA(tlat,tia,4)
                    iabottom=kfactorb3TableA(tlat,bia,4)
                 endif
                 kfactor(2) = (kfactorb3TableA(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb3TableA(tlat,tia,2)-kfactorb3TableA(tlat,bia,2))*(iabottom-iatop)+iatop
                 
                 tlat = blat + 1
                 if(tlat.le.164) then
                    iatop=(kfactorb3TableA(tlat,tia,1)- lat)
     &  /(kfactorb3TableA(tlat,tia,1)-kfactorb3TableA(blat,tia,1))
     &                   *(kfactorb3TableA(blat,tia,5)-kfactorb3TableA(tlat,tia,5))+kfactorb3TableA(tlat,tia,5)
                    
                    iabottom=(kfactorb3TableA(tlat,bia,1)- lat)
     &  /(kfactorb3TableA(tlat,bia,1)-kfactorb3TableA(blat,bia,1))
     &                   *(kfactorb3TableA(blat,bia,5)-kfactorb3TableA(tlat,bia,5))+kfactorb3TableA(tlat,bia,5)
                 else
                    tlat = blat
                    iatop=kfactorb3TableA(tlat,tia,5)
                    iabottom=kfactorb3TableA(tlat,bia,5)
                 endif
                 kfactor(3) = (kfactorb3TableA(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb3TableA(tlat,tia,2)-kfactorb3TableA(tlat,bia,2))*(iabottom-iatop)+iatop
      
      elseif( i_beam.eq.1 .and. i_dir.eq.1)then ! Desceding orbit....       
                 lati = kfactorb1TableD(1,1,1)
                 ia  = kfactorb1TableD(1,1,2)
                 
                 !bia = int(incidence_angle_deg-ia - mod(incidence_angle_deg-ia,1.0)) + 1
                 bia = r8_floor(incidence_angle_deg-ia) + 1
                 tia = bia + 1
                 !blat = int(lat-lati - mod(lat-lati,1.0)) + 1
                 blat = r8_floor(lat-lati) + 1
                 tlat = blat + 1
                 
         ! check that bia and tia are in range AGF 3/2/2011
         if( bia.lt.1 .or. bia.gt.26 .or. 
     $       tia.lt.2 .or. tia.gt.27 ) then
           write(6,*) 'WARNING: get_kfactor: incidence angle out of range: ', incidence_angle_deg, bia, tia
           call flush(6)
           kfactor(1) = -999
           kfactor(2) = -999
           kfactor(3) = -999
           return
         endif
                 
                 
                 
                 
                 if(tlat.le.163) then
                    iatop=(kfactorb1TableD(tlat,tia,1)- lat)
     &  /(kfactorb1TableD(tlat,tia,1)-kfactorb1TableD(blat,tia,1))
     &                   *(kfactorb1TableD(blat,tia,3)-kfactorb1TableD(tlat,tia,3))+kfactorb1TableD(tlat,tia,3)
                    
                    iabottom=(kfactorb1TableD(tlat,bia,1)- lat)
     &  /(kfactorb1TableD(tlat,bia,1)-kfactorb1TableD(blat,bia,1))
     &                   *(kfactorb1TableD(blat,bia,3)-kfactorb1TableD(tlat,bia,3))+kfactorb1TableD(tlat,bia,3)
                 else
                    tlat = blat
                    iatop=kfactorb1TableD(tlat,tia,3)
                    iabottom=kfactorb1TableD(tlat,bia,3)
                 endif
                 
                 kfactor(1) = (kfactorb1TableD(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb1TableD(tlat,tia,2)-kfactorb1TableD(tlat,bia,2))*(iabottom-iatop)+iatop
                 
                 
                 tlat = blat + 1
                 if(tlat.le.163) then
                    iatop=(kfactorb1TableD(tlat,tia,1)- lat)
     &  /(kfactorb1TableD(tlat,tia,1)-kfactorb1TableD(blat,tia,1))
     &                   *(kfactorb1TableD(blat,tia,4)-kfactorb1TableD(tlat,tia,4))+kfactorb1TableD(tlat,tia,4)
                    
                    iabottom=(kfactorb1TableD(tlat,bia,1)- lat)
     &  /(kfactorb1TableD(tlat,bia,1)-kfactorb1TableD(blat,bia,1))
     &                   *(kfactorb1TableD(blat,bia,4)-kfactorb1TableD(tlat,bia,4))+kfactorb1TableD(tlat,bia,4)
                 else
                    tlat = blat
                    iatop=kfactorb1TableD(tlat,tia,4)
                    iabottom=kfactorb1TableD(tlat,bia,4)
                 endif
                 
                 kfactor(2) = (kfactorb1TableD(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb1TableD(tlat,tia,2)-kfactorb1TableD(tlat,bia,2))*(iabottom-iatop)+iatop
                 
                 tlat = blat + 1
                 if(tlat.le.163) then
                    iatop=(kfactorb1TableD(tlat,tia,1)- lat)
     &  /(kfactorb1TableD(tlat,tia,1)-kfactorb1TableD(blat,tia,1))
     &                   *(kfactorb1TableD(blat,tia,5)-kfactorb1TableD(tlat,tia,5))+kfactorb1TableD(tlat,tia,5)
                    
                    iabottom=(kfactorb1TableD(tlat,bia,1)- lat)
     &  /(kfactorb1TableD(tlat,bia,1)-kfactorb1TableD(blat,bia,1))
     &                   *(kfactorb1TableD(blat,bia,5)-kfactorb1TableD(tlat,bia,5))+kfactorb1TableD(tlat,bia,5)
                 else
                    tlat = blat
                    iatop=kfactorb1TableD(tlat,tia,5)
                    iabottom=kfactorb1TableD(tlat,bia,5)
                 endif
                 
                 kfactor(3) = (kfactorb1TableD(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb1TableD(tlat,tia,2)-kfactorb1TableD(tlat,bia,2))*(iabottom-iatop)+iatop
                 
      elseif( i_beam.eq.2 .and. i_dir.eq.1)then ! Desceding orbit....        

                 lati = kfactorb2TableD(1,1,1)
                 ia  = kfactorb2TableD(1,1,2)
                 !bia = int(incidence_angle_deg-ia - mod(incidence_angle_deg-ia,1.0)) + 1
                 bia = r8_floor(incidence_angle_deg-ia) + 1
                 tia = bia + 1
                 !blat = int(lat-lati - mod(lat-lati,1.0)) + 1
                 blat = r8_floor(lat-lati) + 1
                 tlat = blat + 1
                 
         ! check that bia and tia are in range AGF 3/2/2011
         if( bia.lt.1 .or. bia.gt.27 .or. 
     $       tia.lt.2 .or. tia.gt.28 ) then
           write(6,*) 'WARNING: get_kfactor: incidence angle out of range: ', incidence_angle_deg, bia, tia
           call flush(6)
           kfactor(1) = -999
           kfactor(2) = -999
           kfactor(3) = -999
           return
         endif
                 
                 
                 
                 
c     write(86,*) lat,kfactorb1TableD(tlat,tia,1),kfactorb1TableD(blat,tia,1),tlat,blat,icount,dir
c     write(86,*) lat,kfactorb1TableD(tlat,bia,1),kfactorb1TableD(blat,bia,1)
c     write(86,*) incidence_angle_deg,kfactorb1TableD(tlat,tia,2),kfactorb1TableD(tlat,bia,2)
c                 write(6,*) incidence_angle_deg,kfactorb1TableD(blat,tia,2),kfactorb1TableD(blat,bia,2),tia,bia,icount
c     pause
                 
                 
                 
                 if(tlat.le.163) then
                    iatop=(kfactorb2TableD(tlat,tia,1)- lat)
     &  /(kfactorb2TableD(tlat,tia,1)-kfactorb2TableD(blat,tia,1))
     &                   *(kfactorb2TableD(blat,tia,3)-kfactorb2TableD(tlat,tia,3))+kfactorb2TableD(tlat,tia,3)
                    
                    iabottom=(kfactorb2TableD(tlat,bia,1)- lat)
     &  /(kfactorb2TableD(tlat,bia,1)-kfactorb2TableD(blat,bia,1))
     &                   *(kfactorb2TableD(blat,bia,3)-kfactorb2TableD(tlat,bia,3))+kfactorb2TableD(tlat,bia,3)
                 else
                    tlat = blat
                    iatop=kfactorb2TableD(tlat,tia,3)
                    iabottom=kfactorb2TableD(tlat,bia,3)
                 endif
                 
                 kfactor(1) = (kfactorb2TableD(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb2TableD(tlat,tia,2)-kfactorb2TableD(tlat,bia,2))*(iabottom-iatop)+iatop
c     write(6,*) 'Desc: ', kb2HH, incidence_angle_deg,tlat,tia,ia ,mod(incidence_angle_deg-ia,1)
                 
                 tlat = blat + 1
                 if(tlat.le.163) then
                    iatop=(kfactorb2TableD(tlat,tia,1)- lat)
     &  /(kfactorb2TableD(tlat,tia,1)-kfactorb2TableD(blat,tia,1))
     &                   *(kfactorb2TableD(blat,tia,4)-kfactorb2TableD(tlat,tia,4))+kfactorb2TableD(tlat,tia,4)
                    
                    iabottom=(kfactorb2TableD(tlat,bia,1)- lat)
     &  /(kfactorb2TableD(tlat,bia,1)-kfactorb2TableD(blat,bia,1))
     &                   *(kfactorb2TableD(blat,bia,4)-kfactorb2TableD(tlat,bia,4))+kfactorb2TableD(tlat,bia,4)
                 else
                    tlat = blat
                    iatop=kfactorb2TableD(tlat,tia,4)
                    iabottom=kfactorb2TableD(tlat,bia,4)
                 endif
                 kfactor(2) = (kfactorb2TableD(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb2TableD(tlat,tia,2)-kfactorb2TableD(tlat,bia,2))*(iabottom-iatop)+iatop
                 
                 tlat = blat + 1
                 if(tlat.le.163) then
                    iatop=(kfactorb2TableD(tlat,tia,1)- lat)
     &  /(kfactorb2TableD(tlat,tia,1)-kfactorb2TableD(blat,tia,1))
     &                   *(kfactorb2TableD(blat,tia,5)-kfactorb2TableD(tlat,tia,5))+kfactorb2TableD(tlat,tia,5)
                    
                    iabottom=(kfactorb2TableD(tlat,bia,1)- lat)
     &  /(kfactorb2TableD(tlat,bia,1)-kfactorb2TableD(blat,bia,1))
     &                   *(kfactorb2TableD(blat,bia,5)-kfactorb2TableD(tlat,bia,5))+kfactorb2TableD(tlat,bia,5)
                 else
                    tlat = blat
                    iatop=kfactorb2TableD(tlat,tia,5)
                    iabottom=kfactorb2TableD(tlat,bia,5)
                 endif
                 kfactor(3) = (kfactorb2TableD(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb2TableD(tlat,tia,2)-kfactorb2TableD(tlat,bia,2))*(iabottom-iatop)+iatop

      elseif( i_beam.eq.3 .and. i_dir.eq.1)then ! Desceding orbit....              
                 lati = kfactorb3TableD(1,1,1)
                 ia  = kfactorb3TableD(1,1,2)
                 
                 !bia = int(incidence_angle_deg-ia - mod(incidence_angle_deg-ia,1.0)) + 1
                 bia = r8_floor(incidence_angle_deg-ia) + 1
                 tia = bia + 1
                 !blat = int(lat-lati - mod(lat-lati,1.0)) + 1
                 blat = r8_floor(lat-lati) + 1
                 tlat = blat + 1

         ! check that bia and tia are in range AGF 3/2/2011
         if( bia.lt.1 .or. bia.gt.29 .or. 
     $       tia.lt.2 .or. tia.gt.30 ) then
           write(6,*) 'WARNING: get_kfactor: incidence angle out of range: ', incidence_angle_deg, bia, tia
           call flush(6)
           kfactor(1) = -999
           kfactor(2) = -999
           kfactor(3) = -999
           return
         endif

                 
                 if(tlat.le.163) then
                    iatop=(kfactorb3TableD(tlat,tia,1)- lat)
     &  /(kfactorb3TableD(tlat,tia,1)-kfactorb3TableD(blat,tia,1))
     &                   *(kfactorb3TableD(blat,tia,3)-kfactorb3TableD(tlat,tia,3))+kfactorb3TableD(tlat,tia,3)
                    
                    iabottom=(kfactorb3TableD(tlat,bia,1)- lat)
     &  /(kfactorb3TableD(tlat,bia,1)-kfactorb3TableD(blat,bia,1))
     &                   *(kfactorb3TableD(blat,bia,3)-kfactorb3TableD(tlat,bia,3))+kfactorb3TableD(tlat,bia,3)
                 else
                    tlat = blat
                    iatop=kfactorb3TableD(tlat,tia,3)
                    iabottom=kfactorb3TableD(tlat,bia,3)
                 endif
                 
                 kfactor(1) = (kfactorb3TableD(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb3TableD(tlat,tia,2)-kfactorb3TableD(tlat,bia,2))*(iabottom-iatop)+iatop
c     write(6,*) 'Desc: ', kfactor(1), incidence_angle_deg,tlat,tia,ia ,mod(incidence_angle_deg-ia,1)
c     pause
                 tlat = blat + 1
                 
                 
                 if(tlat.le.163) then
                    iatop=(kfactorb3TableD(tlat,tia,1)- lat)
     &  /(kfactorb3TableD(tlat,tia,1)-kfactorb3TableD(blat,tia,1))
     &                   *(kfactorb3TableD(blat,tia,4)-kfactorb3TableD(tlat,tia,4))+kfactorb3TableD(tlat,tia,4)
                    
                    iabottom=(kfactorb3TableD(tlat,bia,1)- lat)
     &  /(kfactorb3TableD(tlat,bia,1)-kfactorb3TableD(blat,bia,1))
     &                   *(kfactorb3TableD(blat,bia,4)-kfactorb3TableD(tlat,bia,4))+kfactorb3TableD(tlat,bia,4)
                    
                 else
                    tlat = blat
                    iatop=kfactorb3TableD(tlat,tia,4)
                    iabottom=kfactorb3TableD(tlat,bia,4)
                 endif
                 kfactor(2) = (kfactorb3TableD(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb3TableD(tlat,tia,2)-kfactorb3TableD(tlat,bia,2))*(iabottom-iatop)+iatop
                 tlat = blat + 1
                 
                 if(tlat.le.163) then
                    iatop=(kfactorb3TableD(tlat,tia,1)- lat)
     &  /(kfactorb3TableD(tlat,tia,1)-kfactorb3TableD(blat,tia,1))
     &                   *(kfactorb3TableD(blat,tia,5)-kfactorb3TableD(tlat,tia,5))+kfactorb3TableD(tlat,tia,5)
                    
                    iabottom=(kfactorb3TableD(tlat,bia,1)- lat)
     &  /(kfactorb3TableD(tlat,bia,1)-kfactorb3TableD(blat,bia,1))
     &                   *(kfactorb3TableD(blat,bia,5)-kfactorb3TableD(tlat,bia,5))+kfactorb3TableD(tlat,bia,5)
                 else
                    tlat = blat
                    iatop=kfactorb3TableD(tlat,tia,5)
                    iabottom=kfactorb3TableD(tlat,bia,5)
                 endif
                 kfactor(3) = (kfactorb3TableD(tlat,tia,2)- incidence_angle_deg)
     &  /(kfactorb3TableD(tlat,tia,2)-kfactorb3TableD(tlat,bia,2))*(iabottom-iatop)+iatop

      endif
      end
      
      
c*****************************************************************      
      subroutine read_k_factors( flag_use_scale_model_patts )
c*****************************************************************      
c**   
c**   FILE NAME:
c**   
c**   DATE WRITTEN:
c**   
c**   PROGRAMMER:
c**   
c**   FUNCTIONAL DESCRIPTION: Read Kfactor tables
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: 
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     3/24/2008        Added ability to load scale-model kfactors. AGF
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      logical flag_use_scale_model_patts        ! if true, we load kfactors for scale-model patterns. 
c     OUTPUT VARIABLES
c     LOCAL VARIABLES
      character*320 file_name_in
      integer i_ia, i_k
      integer icount
      real*8 ia, lat, khh, kvh, kvv
      integer lnblnk

c     DATA STATEMENTS
c     COMMON BLOCKS
      character*200 AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT, AQ_SCAT_SIM_DIR_SCRATCH
      common /AQ_SCAT_SIM_DIRS/ AQ_SCAT_SIM_DIR_OUT, AQ_SCAT_SIM_DIR_DAT,
     $ AQ_SCAT_SIM_DIR_SCRATCH
     
      real*8 kfactorb1TableA(164,27,5), kfactorb2TableA(164,28,5), kfactorb3TableA(164,30,5)
      real*8 kfactorb1TableD(163,27,5), kfactorb2TableD(163,28,5), kfactorb3TableD(163,30,5)
      common /kfactor_block/ kfactorb1TableA, kfactorb2TableA, 
     $ kfactorb3TableA, kfactorb1TableD, kfactorb2TableD, kfactorb3TableD

c     FUNCTION STATEMENTS
c     PROCESSING STEPS


C--Read in KFactor tables.  Modified 3/24/2009 by AGF to read in scale-model KFactors
C--if the command line argument "-sm_patts" was passed to the executable.  The default
C--is to read the KFactors corresponding to the theory antenna patterns.
      if( .not.flag_use_scale_model_patts )then
        write(6,*) 'Reading KFactor tables from ', 
     $   AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // '/Kfactor/theory'
        file_name_in=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // 
     $   '/Kfactor/theory/kfactor_beam1_asc.dat'
        open(90,file=file_name_in,status='unknown')
        file_name_in=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // 
     $   '/Kfactor/theory/kfactor_beam1_dec.dat'
        open(91,file=file_name_in,status='unknown')
        file_name_in=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // 
     $   '/Kfactor/theory/kfactor_beam2_asc.dat'
        open(92,file=file_name_in,status='unknown')
        file_name_in=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // 
     $   '/Kfactor/theory/kfactor_beam2_dec.dat'
        open(93,file=file_name_in,status='unknown')
        file_name_in=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // 
     $   '/Kfactor/theory/kfactor_beam3_asc.dat'
        open(94,file=file_name_in,status='unknown')
        file_name_in=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // 
     $   '/Kfactor/theory/kfactor_beam3_dec.dat'
        open(95,file=file_name_in,status='unknown')
      else
        write(6,*) 'Reading KFactor tables from ', 
     $   AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // '/Kfactor/scale_model'      
        file_name_in=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // 
     $   '/Kfactor/scale_model/kfactor_beam1_asc.dat'
        open(90,file=file_name_in,status='unknown')
        file_name_in=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // 
     $   '/Kfactor/scale_model/kfactor_beam1_dec.dat'
        open(91,file=file_name_in,status='unknown')
        file_name_in=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // 
     $   '/Kfactor/scale_model/kfactor_beam2_asc.dat'
        open(92,file=file_name_in,status='unknown')
        file_name_in=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // 
     $   '/Kfactor/scale_model/kfactor_beam2_dec.dat'
        open(93,file=file_name_in,status='unknown')
        file_name_in=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // 
     $   '/Kfactor/scale_model/kfactor_beam3_asc.dat'
        open(94,file=file_name_in,status='unknown')
        file_name_in=AQ_SCAT_SIM_DIR_DAT(1:lnblnk(AQ_SCAT_SIM_DIR_DAT)) // 
     $   '/Kfactor/scale_model/kfactor_beam3_dec.dat'
        open(95,file=file_name_in,status='unknown')
      endif
      
      icount = 1
      i_ia = 1
      do i_k=1,4428
        read(90,*) lat, ia, khh, kvh, kvv
        kfactorb1TableA(icount,i_ia,1) = lat
        kfactorb1TableA(icount,i_ia,2) = ia
        kfactorb1TableA(icount,i_ia,3) = khh
        kfactorb1TableA(icount,i_ia,4) = kvh
        kfactorb1TableA(icount,i_ia,5) = kvv
        i_ia = i_ia + 1
        if( i_ia .gt. 27 )then
          i_ia = 1
          icount = icount + 1
        endif
      enddo

      icount = 1
      i_ia = 1
      do i_k=1,4592
        read(92,*) lat, ia, khh, kvh, kvv
        kfactorb2TableA(icount,i_ia,1) = lat
        kfactorb2TableA(icount,i_ia,2) = ia
        kfactorb2TableA(icount,i_ia,3) = khh
        kfactorb2TableA(icount,i_ia,4) = kvh
        kfactorb2TableA(icount,i_ia,5) = kvv
        i_ia = i_ia + 1
        if( i_ia .gt. 28 ) then
          i_ia = 1
          icount = icount + 1
        endif
      enddo
      
      icount = 1
      i_ia = 1
      do i_k=1,4401
        read(91,*) lat, ia, khh, kvh, kvv
        kfactorb1TableD(icount,i_ia,1) = lat
        kfactorb1TableD(icount,i_ia,2) = ia
        kfactorb1TableD(icount,i_ia,3) = khh
        kfactorb1TableD(icount,i_ia,4) = kvh
        kfactorb1TableD(icount,i_ia,5) = kvv
        i_ia = i_ia + 1
        if(i_ia .gt. 27) then
          i_ia = 1
          icount = icount + 1
        endif
      enddo



      icount = 1
      i_ia = 1
      do i_k=1,4564
        read(93,*) lat, ia, khh, kvh, kvv
        kfactorb2TableD(icount,i_ia,1) = lat
        kfactorb2TableD(icount,i_ia,2) = ia
        kfactorb2TableD(icount,i_ia,3) = khh
        kfactorb2TableD(icount,i_ia,4) = kvh
        kfactorb2TableD(icount,i_ia,5) = kvv
        i_ia = i_ia + 1
        if( i_ia .gt. 28) then
          i_ia = 1
          icount = icount + 1
        endif
      enddo


      icount = 1
      i_ia = 1
      do i_k=1,4920
        read(94,*) lat, ia, khh, kvh, kvv
        kfactorb3TableA(icount,i_ia,1) = lat
        kfactorb3TableA(icount,i_ia,2) = ia
        kfactorb3TableA(icount,i_ia,3) = khh
        kfactorb3TableA(icount,i_ia,4) = kvh
        kfactorb3TableA(icount,i_ia,5) = kvv
        i_ia = i_ia+1
        if(i_ia.gt.30) then
          i_ia = 1
          icount = icount+1
        endif
      enddo



      icount = 1
      i_ia = 1
      do i_k=1,4890
        read(95,*) lat, ia, khh, kvh, kvv
        kfactorb3TableD(icount,i_ia,1) = lat
        kfactorb3TableD(icount,i_ia,2) = ia
        kfactorb3TableD(icount,i_ia,3) = khh
        kfactorb3TableD(icount,i_ia,4) = kvh
        kfactorb3TableD(icount,i_ia,5) = kvv
        i_ia = i_ia+1
        if(i_ia.gt.30) then
          i_ia = 1
          icount = icount+1
        endif
      enddo
      
      close(90)
      close(91)
      close(92)
      close(93)
      close(94)
      close(95)     
      end      
