c*****************************************************************      
      subroutine ecf_to_mj2k( sc_position_ecf, sc_velocity_ecf,
     $ julian_time, sc_position_mj2k, sc_velocity_mj2k )
c*****************************************************************      
c**   
c**   FILE NAME: mj2k_subs.f
c**   
c**   DATE WRITTEN: 1/30/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  Converts from ecf to mj2k
c**     coordinate systems.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES: Modularization of existing code.
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 sc_position_ecf(3)
      real*8 sc_velocity_ecf(3)
      real*8 julian_time
c     OUTPUT VARIABLES
      real*8 sc_position_mj2k(3)
      real*8 sc_velocity_mj2k(3)
c     LOCAL VARIABLES
      integer i_year
      integer i_month
      integer i_day
      integer i_hour
      integer i_minute
      integer i_second
      real*8 r_frac
      real*8 sc_position_tmp(3)
      real*8 sc_velocity_tmp(3)
      real*8 utcmjd
      real*8 greenwich_apparent_hour_angle_rad
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug_level/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      
      call JD2CAL( julian_time, i_year, i_month, i_day, i_hour, 
     $ i_minute, i_second, r_frac )
     
      utcmjd = julian_time - 2400000.5d0 
      call gha (utcmjd,greenwich_apparent_hour_angle_rad)

C--inverse rotation.      
      greenwich_apparent_hour_angle_rad = -1.0d0 * greenwich_apparent_hour_angle_rad
      
      call rotate_ephemeris( sc_position_ecf, sc_velocity_ecf, 
     $ sc_position_tmp, sc_velocity_tmp, greenwich_apparent_hour_angle_rad )
     
      if( i_dbg_level.ge.2 )then
        write(6,*) " jjjday ", i_year, i_month, i_day, utcmjd, greenwich_apparent_hour_angle_rad
      endif

      if( i_dbg_level.ge.2 )then
        write(6,*) 'after rotate_ephemeris (inv):'
        write(6,*) sc_position_tmp
        write(6,*) sc_velocity_tmp
      endif  
      
      call rotate2_system_due_to_precession( i_year, i_month, i_day,
     $ sc_position_tmp, sc_velocity_tmp, sc_position_mj2k,
     $ sc_velocity_mj2k, -1 )
      
      if( i_dbg_level.ge.2)then
        write(6,*) 'after rotate2_system_due_to_precession (inv):'
        write(6,*) sc_position_mj2k
        write(6,*) sc_velocity_mj2k
      endif

     
      end

c*****************************************************************      
      subroutine mj2k_to_ecf( sc_position_mj2k, sc_velocity_mj2k,
     $ julian_time, sc_position_ecf, sc_velocity_ecf )
c*****************************************************************      
c**   
c**   FILE NAME: mj2k_subs.f
c**   
c**   DATE WRITTEN: 1/30/2009
c**   
c**   PROGRAMMER: Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:   Converts from mj2k to ecf
c**     coordinate systems.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES:  Modularization of existing code.
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      real*8 sc_position_mj2k(3)
      real*8 sc_velocity_mj2k(3)
      real*8 julian_time
c     OUTPUT VARIABLES
      real*8 sc_position_ecf(3)
      real*8 sc_velocity_ecf(3)
c     LOCAL VARIABLES
      integer i_year
      integer i_month
      integer i_day
      integer i_hour
      integer i_minute
      integer i_second
      real*8 r_frac
      real*8 sc_position_tmp(3)
      real*8 sc_velocity_tmp(3)
      real*8 utcmjd
      real*8 greenwich_apparent_hour_angle_rad
c     DATA STATEMENTS
c     COMMON BLOCKS
      integer i_dbg_level
      common /debug_level/ i_dbg_level
c     FUNCTION STATEMENTS
c     PROCESSING STEPS
      
      if( i_dbg_level.ge.2)then
        write(6,*) 'before rotate2_system_due_to_precession:'
        write(6,*) sc_position_mj2k
        write(6,*) sc_velocity_mj2k
      endif      
      
      call JD2CAL( julian_time, i_year, i_month, i_day, i_hour, 
     $ i_minute, i_second, r_frac )
      
      utcmjd = julian_time - 2400000.5d0 
      call gha (utcmjd,greenwich_apparent_hour_angle_rad)

      if( i_dbg_level.ge.2 )then
        write(6,*) " jjjday ", i_year, i_month, i_day, utcmjd, greenwich_apparent_hour_angle_rad
      endif
      
      call rotate2_system_due_to_precession( i_year, i_month, i_day,
     $ sc_position_mj2k, sc_velocity_mj2k, sc_position_tmp,
     $ sc_velocity_tmp, 1 )
      
      if( i_dbg_level.ge.2)then
        write(6,*) 'after rotate2_system_due_to_precession:'
        write(6,*) sc_position_tmp
        write(6,*) sc_velocity_tmp
      endif
      
      call rotate_ephemeris( sc_position_tmp, sc_velocity_tmp, 
     $ sc_position_ecf, sc_velocity_ecf, greenwich_apparent_hour_angle_rad )
     
      end