c*****************************************************************      
      subroutine read_alex_groupD_antenna_pattern_relative(
     $ str_file_name, d_antenna_pattern_relative, d_max_gain) 
c*****************************************************************      
c**   
c**   FILE NAME:  pattern_subroutines_alex_groupD.f
c**   
c**   DATE WRITTEN:  11/18/2008
c**   
c**   PROGRAMMER:  Alex Fore
c**   
c**   FUNCTIONAL DESCRIPTION:  Reads in my "groupD" scale-model 
c**   antenna pattern files.  These files have:
c**      
c**     theta \in [ -180, -179.5, -179, ..., 179, 179.5, 180]
c**     phi   \in [0, 0.5, 1, ..., 178.5, 179, 179.5]
c**
c**   on a uniform in (theta,phi)-space grid.  See my antenna memo
c**   for more info on the antenna pattern files.
c**
c**   ROUTINES CALLED: 
c**   
c**   NOTES:  Programmed to be a drop-in replacement for 
c**   the subroutine "read_jpl_cut_antenna_pattern_relative", which
c**   reads in the theory patterns.
c**
c**   UPDATE LOG:
c**
c**   Date Changed        Reason Changed        CR # and Version #
c**   ------------       ----------------       ------------------
c**     
c*****************************************************************   
      implicit none
c     PARAMETER STATEMENTS
c     INPUT VARIABLES
      character*200 str_file_name   ! input pattern file name
      
c     OUTPUT VARIABLES
      real*8 d_antenna_pattern_relative(360,721,6) ! relative ant patt
      real*8 d_max_gain ! max gain in dB
      
c     LOCAL VARIABLES
      real*8 d_antenna_pattern_raw(360,721,4)  ! raw ant patt
      real*8 d_theta(360,721)                  ! beam theta coordinate
      real*8 d_phi(360,721)                    ! beam phi coordinate
      real*8 d_max_amp_co                      ! max co-pol amplitude
      real*8 d_amp_co                          ! co-pol amplitude
      real*8 d_amp_cx                          ! cx-pol amplitude
      integer i_theta, i_phi, ii               ! loop integers
      character*320 str_header_line            ! junk bin for header lines
      character*320 str_line
      
c     DATA STATEMENTS
c     FUNCTION STATEMENTS
      integer lnblnk
c     PROCESSING STEPS
      
      write(6,*) 'Reading in  patt file: ', str_file_name(1:lnblnk(str_file_name))
      
255   format(a320)
      
      open(25,file=str_file_name,status='old')
      
C     There are seven header lines.
      do ii=1,7
        read(25,255) str_header_line
c        write(6,*) str_header_line
      enddo

C Read in the antenna pattern and find max co-pol gain.
      d_max_amp_co = 0.0d0
      
C Order of the loops is important here since we read from a file!
C  This order is right.
      do i_phi=1,360
        do i_theta=1,721
          read(25,'(a)') str_line
          
          if( index(str_line,'NaN') .eq. 0)then
            read(str_line,*) d_theta(i_phi,i_theta), d_phi(i_phi,i_theta), 
     $ d_antenna_pattern_raw(i_phi,i_theta,1), 
     $ d_antenna_pattern_raw(i_phi,i_theta,2),
     $ d_antenna_pattern_raw(i_phi,i_theta,3),
     $ d_antenna_pattern_raw(i_phi,i_theta,4)
          
          d_amp_co = cabs( cmplx( 
     $ d_antenna_pattern_raw(i_phi,i_theta,1), 
     $ d_antenna_pattern_raw(i_phi,i_theta,2)))
          
          d_max_amp_co = max( d_amp_co, d_max_amp_co )
          else
            read(str_line,*) d_theta(i_phi,i_theta), d_phi(i_phi,i_theta)
            d_antenna_pattern_raw(i_phi,i_theta,1) = 0.0d0
            d_antenna_pattern_raw(i_phi,i_theta,2) = 0.0d0
            d_antenna_pattern_raw(i_phi,i_theta,3) = 0.0d0
            d_antenna_pattern_raw(i_phi,i_theta,4) = 0.0d0
            d_amp_co = 0.0d0
          endif
        enddo
      enddo
      close(25)

C Here we can use the other order to improve cache reuse.
      do i_theta=1,721
        do i_phi=1,360
          d_amp_co = cabs( cmplx( 
     $ d_antenna_pattern_raw(i_phi,i_theta,1), 
     $ d_antenna_pattern_raw(i_phi,i_theta,2)))
          d_amp_cx = cabs( cmplx( 
     $ d_antenna_pattern_raw(i_phi,i_theta,3), 
     $ d_antenna_pattern_raw(i_phi,i_theta,4)))
          
          d_antenna_pattern_relative(i_phi,i_theta,1) = 10*log10( 
     $     d_amp_co**2 / d_max_amp_co**2 )
     
          d_antenna_pattern_relative(i_phi,i_theta,2) = 10*log10( 
     $     d_amp_cx**2 / d_max_amp_co**2 )
          
          do ii=1,4
            d_antenna_pattern_relative(i_phi,i_theta,ii+2) = 
     $ d_antenna_pattern_raw(i_phi,i_theta,ii) / d_max_amp_co
          enddo
        enddo
      enddo
      
      d_max_gain = 10*log10( d_max_amp_co**2 ) 
      end
      
C**********************************************************************
C  Ignore this stuff, it is just some testing code I used to test the
C  subroutine.
C
C      program test_alex_groupD
C      
C      character*320 th_file_name, sm_file_name
C      real*8 antenna_pattern_relative(360,721,6)
C      real*8 max_gain
C      integer i_theta, i_phi, ii
C      
C      call getarg(1,th_file_name)
C      call getarg(2,sm_file_name)
C      
C      call read_jpl_cut_antenna_pattern_relative(
C     $ th_file_name, antenna_pattern_relative, max_gain) 
C     
C     
C      open(20,file='th_patt_test.txt')
C      
C      do i_theta=1,721
C        do i_phi=1,360
C          write(20,*) (antenna_pattern_relative(i_phi,i_theta,ii)
C     $     ,ii=1,6)
C        enddo
C      enddo
C      close(20)
C      
C      call read_alex_groupD_antenna_pattern_relative(
C     $ sm_file_name, antenna_pattern_relative, max_gain)       
C
C      open(20,file='sm_patt_test.txt')
C      
C      do i_theta=1,721
C        do i_phi=1,360
C          write(20,*) (antenna_pattern_relative(i_phi,i_theta,ii)
C     $     ,ii=1,6)
C        enddo
C      enddo
C
C      close(20)
C
C      end
C
C       Do this in matlab to test the code:
C       clear;
C       base='/Users/fore/Documents/Aquarius/scat_science/Current.Photon';
C 
C       A = load([base,'/th_patt_test.txt']);
C       B = load([base,'/sm_patt_test.txt']);
C 
C       A = reshape(A,360,721,6);
C       B = reshape(B,360,721,6);
C 
C       figure;
C       subaxis(2,1,1)
C       surf(A(:,:,2),'linestyle','none');view(2);caxis([-85 0]);colorbar
C 
C       subaxis(2,1,2)
C       surf(B(:,:,2),'linestyle','none');view(2);caxis([-85 0]);colorbar
C
C**********************************************************************

      