//--$Header: /u/caretr2/scatsim/CVSROOT/L1A_to_L2/rfi_stats.c,v 1.2 2012/05/07 16:44:12 fore Exp $
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

/* Some sample C code for the quickselect algorithm, 
   taken from Numerical Recipes in C. */

#define SWAP(a,b) temp=(a); (a)=(b); (b)=temp;

double quickselect(double* arr, int n, int k) {
  unsigned long i,ir,j,l,mid;
  double a,temp;

  l=0;
  ir=n-1;
  for(;;) {
    if (ir <= l+1) { 
      if (ir == l+1 && arr[ir] < arr[l]) {
            SWAP(arr[l],arr[ir]);
      }
      return arr[k];
    }
    else {
      mid=(l+ir) >> 1; 
      SWAP(arr[mid],arr[l+1]);
      if (arr[l] > arr[ir]) {
            SWAP(arr[l],arr[ir]);
      }
      if (arr[l+1] > arr[ir]) {
            SWAP(arr[l+1],arr[ir]);
      }
      if (arr[l] > arr[l+1]) {
            SWAP(arr[l],arr[l+1]);
      }
      i=l+1; 
      j=ir;
      a=arr[l+1]; 
      for (;;) { 
            do i++; while (arr[i] < a); 
            do j--; while (arr[j] > a); 
            if (j < i) break; 
            SWAP(arr[i],arr[j]);
      } 
      arr[l+1]=arr[j]; 
      arr[j]=a;
      if (j >= k) ir=j-1; 
      if (j <= k) l=i;
    }
  }
}
// END NR code------

void rfi_stats__( double* data, int* num_elements, 
                  double* mean, double* median, double* std ) {
  // verified to give exactly the same results as MATLAB (for odd n)
  // For even n, it does not average the two middle data points, and matlab does.
  int    n = *num_elements;
  int    ii;
  int    median_idx = (n-1)/2; 
  double sum        = 0;
  double sum_sq     = 0;
  double tmp;
  
  for( ii = 0; ii < n; ++ii ) {
    sum    += data[ii];
    sum_sq += data[ii] * data[ii];
  }
  
  double local_data[n];
  memcpy( &local_data[0], data, n * sizeof( double ) );
  
  tmp = sum_sq / (double)(n-1) - pow(sum,2) / ( (double)n*(double)(n-1) );
  if( tmp < 0 ) tmp = 0;
  
  *mean   = sum / (double)n;
  *std    = sqrt( tmp );
  *median = quickselect( &local_data[0], n, median_idx );
  return;
}
