c       ---------------------------------------------------------------
        subroutine nswinterpol(clon, clat, wspd, wdir, iflag, windfd)
c       Input variables: clat and clon 
c       clat    : latitude of the wind cell location
c       clon    : longitude of the wind cell location
c       Output arguments: 
c       wspd    : interpolated wind speed
c       wdir    : interpolated wind direction from the North
c       iflag   : 0 indicates good quality, i. e., the wind cell is on the ocean : 1 indicates bad quality, i. e., the wind cell is not on the ocean
c       ---------------------------------------------------------------
        real*8 windfd(720,301,2)
        real*8 clon, clat, wspd, wdir
        integer iflag
        real*8 uu, vv
        integer i,j
        
        iflag = 0
        
        if( clat .gt. 75 .or. clat .lt. -75) then
           iflag=1
           wspd=-9999
           wdir=0
           return
        endif
        
        if( clon .le. 0.0 ) clon =clon+360.0
        if( clon .gt. 360.0 ) clon =clon-360.0
        
        j=((clat+75)/0.5+1.0)
        i=(clon/0.5)
        
c       uu=windfd(i*720+j)
c       vv=windfd(i*720+j+720*301+2)
        uu=windfd(i,j,1)
        vv=windfd(i,j,2)
        
        if ( uu .eq. -9999 .or. vv .eq. -9999 ) iflag=1
c       ---------------------------------------------------------------
c       Convert the wind vector in (u,v) components into wind speed 
c       and wind direction from North.
        
c       write(*,*) 'NSW1: ',uu, vv
        call  unpol(uu, vv, wspd, wdir)

        return
        end





c       ---------------------------------------------------------------
        subroutine nswinterpol2(clon, clat, wspd, wdir, iflag, windfd1, windfd2,initial_time, time)
c       Input variables: clat and clon 
c       clat    : latitude of the wind cell location
c       clon    : longitude of the wind cell location
c       Output arguments: 
c       wspd    : interpolated wind speed
c       wdir    : interpolated wind direction from the North
c       iflag   : 0 indicates good quality, i. e., the wind cell is on the ocean : 1 indicates bad quality, i. e., the wind cell is not on the ocean
c       ---------------------------------------------------------------
        real*8 windfd1(720,301,2) ,windfd2(720,301,2)
        real*8 clon, clat, wspd, wdir
        integer iflag
        real*8 uu1, vv1,uu2, vv2,uu, vv,initial_time, step_time, time
        integer i,j
        
        iflag = 0
        step_time =.25d0
        if( clat .gt. 75 .or. clat .lt. -75) then
           iflag=1
           wspd=-9999
           wdir=0
           return
        endif
        
        if( clon .le. 0.0 ) clon =clon+360.0
        if( clon .gt. 360.0 ) clon =clon-360.0
        
        j=((clat+75)/0.5+1.0)
        i=(clon/0.5)
        
c       uu=windfd(i*720+j)
c       vv=windfd(i*720+j+720*301+2)
        uu1=windfd1(i,j,1)
        vv1=windfd1(i,j,2)
        uu2=windfd2(i,j,1)
        vv2=windfd2(i,j,2)
        
        if ( uu1 .eq. -9999 .or. vv1 .eq. -9999 ) iflag=1
c       ---------------------------------------------------------------
c       Convert the wind vector in (u,v) components into wind speed 
c       and wind direction from North.
c$$$    
c$$$    if(( uu1 .eq. -9999 .and. vv1 .gt. -9999 ).or.( uu1 .gt. -9999 .and. vv1 .eq. -9999 )) then
c$$$       write(*,*) 'NSW2: ',time, initial_time, uu1, vv1
c$$$       write(*,*) 'NSW2: ',time, initial_time, uu2, vv2
c$$$       pause
c$$$    endif
        uu= (uu2-uu1) * (time-initial_time)/step_time + uu1
        vv= (vv2-vv1) * (time-initial_time)/step_time + vv1

c       write(*,*) 'NSW2: ',time, initial_time, uu,(time-initial_time)/step_time
c       write(*,*) 'NSW2: ',time, initial_time, vv, (time-initial_time)/step_time

        call  unpol(uu, vv, wspd, wdir)
        return
        end
        


c       ---------------------------------------------------------------
        subroutine nswinterpol3(clon, clat, wspd, wdir, iflag, windfd1, windfd2,initial_time, time)
c       Input variables: clat and clon 
c       clat    : latitude of the wind cell location
c       clon    : longitude of the wind cell location
c       Output arguments: 
c       wspd    : interpolated wind speed
c       wdir    : interpolated wind direction from the North
c       iflag   : 0 indicates good quality, i. e., the wind cell is on the ocean : 1 indicates bad quality, i. e., the wind cell is not on the ocean
c       ---------------------------------------------------------------
        real*8 windfd1(720,361,2) ,windfd2(720,361,2)
        real*8 clon, clat, wspd, wdir
        integer iflag
        real*8 uu1, vv1, uu2, vv2,uu, vv,initial_time, step_time, time
        real*8 tmp_lon, tmp_lat
        integer i,j
        
        iflag = 0
        step_time =.25d0
        if( clat .gt. 90 .or. clat .lt. -90) then
           iflag=1
           wspd=-9999
           wdir=0
           return
        endif
        
        tmp_lon = clon
        tmp_lat = clat

        if( tmp_lon .lt. 0.0 )   tmp_lon = tmp_lon + 360.0
        if( tmp_lon .ge. 360.0 ) tmp_lon = tmp_lon - 360.0

        j = nint( ( tmp_lat + 90) / 0.5) + 1
        i = nint(   tmp_lon       / 0.5) + 1
        
        uu1=windfd1(i,j,1)
        vv1=windfd1(i,j,2)
        uu2=windfd2(i,j,1)
        vv2=windfd2(i,j,2)
        
        if ( uu1 .eq. -9999 .or. vv1 .eq. -9999 ) iflag=1
c       ---------------------------------------------------------------
c       Convert the wind vector in (u,v) components into wind speed 
c       and wind direction from North.
c$$$    
c$$$    if(( uu1 .eq. -9999 .and. vv1 .gt. -9999 ).or.( uu1 .gt. -9999 .and. vv1 .eq. -9999 )) then
c$$$       write(*,*) 'NSW2: ',time, initial_time, uu1, vv1
c$$$       write(*,*) 'NSW2: ',time, initial_time, uu2, vv2
c$$$       pause
c$$$    endif
        uu= (uu2-uu1) * (time-initial_time)/step_time + uu1
        vv= (vv2-vv1) * (time-initial_time)/step_time + vv1

c       write(*,*) 'NSW2: ',time, initial_time, uu,(time-initial_time)/step_time
c       write(*,*) 'NSW2: ',time, initial_time, vv, (time-initial_time)/step_time

        call  unpol(uu, vv, wspd, wdir)
        return
        end
        







c---------------------------------------------------------
        subroutine readwf(wffile, windfd)
c       This routine is used to gridded wind fields.
c       Each file has two (720,301) arrays, for zonal and meridional components respectively, in unit of m/sec.
c       Data domain covers global region from 75S to 75N.  Resolution is 0.5x0.5 degrees.
c       Values over land are -9999.0, and over the ocean each grid point is initialized to NSCAT monthly average.
        
c       Input: file name of wind field
c       Output:
c       windfd(*,*,1): u, zonal component (m/s)
c       windfd(*,*,2): v, meridional component
        
        parameter(imt=720, jmt=301)
        real*8 windfd(imt,jmt,2)
        real*8 u(imt,jmt)
        real*8 v(imt,jmt)
        character*100 wffile
        !!integer iosn
        !!integer*2 itmp(2,3)/0,0,0,0,0,0/
        !!integer itmp2/0/
        !!real*4 rtmp(3)
        !!equivalence(itmp,rtmp)
        !!character*20 text(3)/'aa','bb','cc'/

        !!inquire(file=wffile,form=text(1),access=text(2),direct=text(3),recl=itmp2)
        !!write(*,*)'W0', text 
        !!write(*,*)'W0b', itmp2
  !!!      open(11,file=wffile, form='unformatted',status='old',iostat= iosn)
        open(11,file=wffile, form='formatted',status='old',iostat= iosn)
        !!inquire(unit=11,form=text(1),access=text(2),direct=text(3),recl=itmp2)
        !!inquire(file=wffile
        !!write(*,*)'W0c', text 
        !!write(*,*)'W0d', itmp2

        !!write(*,*) 'iosn = ', iosn
              !!read(11,iostat=iosn) itmp(1,1),itmp(2,1) !****
              !!write(*,*) 'W1 ',iosn,itmp
              !!write(*,*) 'W2', rtmp
  !!!      read(11,iostat= iosn) ((u(i,j),i=1,imt),j=1,jmt)
        read(11,101,iostat= iosn) ((u(i,j),i=1,imt),j=1,jmt)
 101    format (E16.4)
          !!write(*,*) 'iosn = ', iosn, u(1,1)
   !!!     read(11,iostat= iosn) ((v(i,j),i=1,imt),j=1,jmt)
          read(11,101,iostat= iosn) ((v(i,j),i=1,imt),j=1,jmt)
           !!write(*,*) 'iosn = ', iosn,v(imt,jmt)
        !!write(*,*) ((u(i,j),i=1,10),j=1,10)
        !!write(*,*) ((v(i,j),i=imt-10,imt),j=jmt-10,jmt)

        do i=1,imt
           do j=1,jmt
              windfd(i,j,1)=u(i,j)
              windfd(i,j,2)=v(i,j)
           enddo
        enddo
        
        close(11)
        return
        end

c---------------------------------------------------------
        subroutine readwf2(wffile1,wffile2, windfd1, windfd2)
c       This routine is used to gridded wind fields.
c       Each file has two (720,301) arrays, for zonal and meridional components respectively, in unit of m/sec.
c       Data domain covers global region from 75S to 75N.  Resolution is 0.5x0.5 degrees.
c       Values over land are -9999.0, and over the ocean each grid point is initialized to NSCAT monthly average.
        
c       Input: file name of wind field
c       Output:
c       windfd(*,*,1): u, zonal component (m/s)
c       windfd(*,*,2): v, meridional component
        
        parameter(imt=720, jmt=301)
        real*8 windfd1(imt,jmt,2)
        real*8 windfd2(imt,jmt,2)
        real*8 u1(imt,jmt), u2(imt,jmt)
        real*8 v1(imt,jmt), v2(imt,jmt)
        character*100 wffile1, wffile2

        open(11,file=wffile1, form='formatted',status='old',iostat= iosn)
        open(12,file=wffile2, form='formatted',status='old',iostat= iosn)
        read(11,101,iostat= iosn) ((u1(i,j),i=1,imt),j=1,jmt)
        read(12,101,iostat= iosn) ((u2(i,j),i=1,imt),j=1,jmt)
 101    format (E16.4)

        read(11,101,iostat= iosn) ((v1(i,j),i=1,imt),j=1,jmt)
        read(12,101,iostat= iosn) ((v2(i,j),i=1,imt),j=1,jmt)


        do i=1,imt
           do j=1,jmt
              windfd1(i,j,1)=u1(i,j)
              windfd1(i,j,2)=v1(i,j)
              windfd2(i,j,1)=u2(i,j)
              windfd2(i,j,2)=v2(i,j)

           enddo
        enddo
        
        close(11)
        close(12)
        return
        end



c---------------------------------------------------------
        subroutine readwf3(wffile1,wffile2, windfd1, windfd2)
c       This routine is used to gridded wind fields.
c       Each file has two (720,361) arrays, for zonal and meridional components respectively, in unit of m/sec.
c       Data domain covers global region from 90S to 90N.  Resolution is 0.5x0.5 degrees.
        
c       Input: file name of wind field
c       Output:
c       windfd(*,*,1): u, zonal component (m/s)
c       windfd(*,*,2): v, meridional component
        
        parameter(imt=720, jmt=361)
        real*8 windfd1(imt,jmt,2)
        real*8 windfd2(imt,jmt,2)
        real*8 u1(imt,jmt), u2(imt,jmt)
        real*8 v1(imt,jmt), v2(imt,jmt)
        character*100 wffile1, wffile2

        open(11,file=wffile1, form='formatted',status='old',iostat= iosn)
        open(12,file=wffile2, form='formatted',status='old',iostat= iosn)
        read(11,101,iostat= iosn) ((u1(i,j),i=1,imt),j=1,jmt)
        read(12,101,iostat= iosn) ((u2(i,j),i=1,imt),j=1,jmt)
 101    format (E16.4)

        read(11,101,iostat= iosn) ((v1(i,j),i=1,imt),j=1,jmt)
        read(12,101,iostat= iosn) ((v2(i,j),i=1,imt),j=1,jmt)


        do i=1,imt
           do j=1,jmt
              windfd1(i,j,1)=u1(i,j)
              windfd1(i,j,2)=v1(i,j)
              windfd2(i,j,1)=u2(i,j)
              windfd2(i,j,2)=v2(i,j)

           enddo
        enddo
        
        close(11)
        close(12)
        return
        end









c       This subroutine convert a wind vector in x(+E)-y(+N) 
c       cartesian coordinate system into North/South clockwise 
c       polar coordinate system.
        
c       ** Input variables: uu and vv
c       uu      : wind component in x axis
c       vv      : wind component in y axis
        
c       ** Output variables: wspd and wdir
c       wspd    : wind speed (m/s)
c       wdir    : wind direction clockwise from the North
c       ---------------------------------------------------------------
        subroutine unpol(uu, vv, wspd, wdir)
        real*8 RTD
        parameter (RTD=57.2958279)
        real*8 uu,vv,wspd,wdir
        real*8 arg2

        arg2 = uu*uu + vv*vv
c       RTD=180/3.14159
        if (arg2 .eq. 0.0) then
           wspd=0
           wdir=0
        else
           wspd = sqrt(arg2)
           wdir = 450.0 - atan2(vv,uu)*RTD
           if (wdir .ge. 360.0) wdir = wdir - 360.0
        endif
        return
        end
        




        subroutine readEcco_data(eccofile1,eccofile2, WSTfd1, WSTfd2)
        integer imt, jmt, flat, flon, clat,clon
        parameter(imt=1440, jmt=720)
        real*8 WSTfd1(imt,jmt,5)
        real*8 WSTfd2(imt,jmt,5)
        character*100 eccofile1,eccofile2


        open(11,file=eccofile1, form='formatted',status='old',iostat= iosn)
        open(12,file=eccofile2, form='formatted',status='old',iostat= iosn)
        read(11,*)
        read(12,*)

 101    format(4f12.4, i5)

        do j=1,jmt
           do  i = 1,imt
              read(11,101,iostat= iosn) WSTfd1(i,j,1)
     &        ,WSTfd1(i,j,2),WSTfd1(i,j,3),WSTfd1(i,j,4),WSTfd1(i,j,5)
              read(12,101,iostat= iosn) WSTfd2(i,j,1)
     &        ,WSTfd2(i,j,2),WSTfd2(i,j,3),WSTfd2(i,j,4),WSTfd2(i,j,5)

           enddo
        enddo


        close(11)
        close(12)
        return
        end



        subroutine readHycom_data(hycomfile1,hycomfile2, WSTfd1, WSTfd2)
        integer imt, jmt, flat, flon, clat,clon
        parameter(imt=1440, jmt=720)
        real*8 WSTfd1(imt,jmt,2), sal, temp
        real*8 WSTfd2(imt,jmt,2)
        character*100 hycomfile1,hycomfile2


        open(11,file=hycomfile1, form='formatted',status='old',iostat= iosn)
        open(12,file=hycomfile2, form='formatted',status='old',iostat= iosn)

 101    format (F17.4,F12.4,F10.4,F1.0)
        

        do j=1,jmt
           do  i = 1,imt
              read(11,101,iostat= iosn) temp
     &        ,sal,WSTfd1(i,j,1),WSTfd1(i,j,2)
              read(12,101,iostat= iosn) temp
     &        ,sal,WSTfd2(i,j,1),WSTfd2(i,j,2)

           enddo
        enddo

        close(11)
        close(12)
        return
        end






        subroutine TSinterpol(latitude, longitude,wind_speed,
     &      WSTfd1, WSTfd2, initial_time, time)
        integer imt, jmt
        integer flon, flat, clon,clat
        integer fun_floor, fun_ceil
        parameter(imt=1440, jmt=720)
        real*8 WSTfd1(imt,jmt,2)
        real*8 WSTfd2(imt,jmt,2)
        real*8 temp_flon1, temp_clon1, temp_flat1, temp_clat1
        real*8 temp_flon2, temp_clon2, temp_flat2, temp_clat2
        real*8 temp_flon3, temp_clon3, temp_flat3, temp_clat3
        real*8 interpolate_value1, interpolate_value2, interpolate_value3
        real*8 initial_time, step_time, latitude, longitude, step, time
        real*8 temperature, salinity, wind_speed
        step = .25
        step_time =.25d0

        flat = fun_floor((90.-latitude)/step)+1
        flon = fun_floor(longitude/step)+1

        clat = fun_ceil((90.-latitude)/step)+1
        clon = fun_ceil(longitude/step)+1



        temp_flon =  (WSTfd1(flon,flat,1) -  WSTfd1(flon,clat,1))/step 
     &  * ((90.-step*dfloat((flat-1))) - latitude) 
     &    +  WSTfd1(flon,clat,1)

        temp_clon =  (WSTfd1(clon,flat,1) -  WSTfd1(clon,clat,1))/step 
     &  * ((90.-step*dfloat((flat-1))) - latitude) 
     &   +  WSTfd1(clon,clat,1)


        interpolate_value1 = (temp_flon - temp_clon)/step 
     &  * (longitude - step*dfloat((flon-1))) +  temp_clon


        temp_flon =  (WSTfd2(flon,flat,1) -  WSTfd2(flon,clat,1))/step 
     &  * ((90.-step*dfloat((flat-1))) - latitude) 
     &    +  WSTfd2(flon,clat,1)

        temp_clon =  (WSTfd2(clon,flat,1) -  WSTfd2(clon,clat,1))/step 
     &  * ((90.-step*dfloat((flat-1))) - latitude) 
     &   +  WSTfd2(clon,clat,1)



        interpolate_value2 = (temp_flon - temp_clon)/step 
     &  * (longitude - step*dfloat((flon-1))) +  temp_clon


        wind_speed = (interpolate_value2 - interpolate_value1) 
     &   * (time-initial_time)/step_time + interpolate_value1





        return

        end




        subroutine TS_ecco_interpol(latitude, longitude, temperature, salinity,wind_speed,
     &      wind_dir,WSTfd1, WSTfd2, initial_time, time)
        integer imt, jmt
        integer flon, flat, clon,clat
        integer fun_floor, fun_ceil
        parameter(imt=1440, jmt=720)
        real*8 WSTfd1(imt,jmt,5)
        real*8 WSTfd2(imt,jmt,5)
        real*8 temp_flon1, temp_clon1, temp_flat1, temp_clat1
        real*8 temp_flon2, temp_clon2, temp_flat2, temp_clat2
        real*8 temp_flon3, temp_clon3, temp_flat3, temp_clat3
        real*8 interpolate_value1, interpolate_value2, interpolate_value3
        real*8 initial_time, step_time, latitude, longitude, step, time
        real*8 temperature, salinity, wind_speed, wind_dir,u,v
        

        real*8 ini_lat, ini_lon, temp_longitude, temp_latitude

        step = .25d0
        step_time =.25d0

        ini_lat = -89.875d0
        ini_lon = .125d0


        flat = fun_floor((latitude - ini_lat)/step)+1
        flon = fun_floor((longitude - ini_lon)/step)+1

        clat = flat + 1
        clon = flon + 1

        temp_longitude = longitude
        temp_latitude = latitude

        if (longitude<ini_lon .or. longitude>359.875d0) then
           flon = 1440
           clon = 1
           if (longitude<ini_lon) then 
              longitude = longitude + 360
           endif
        endif


        if (latitude<ini_lat .or. latitude>89.875) then
           flat = 720
           clat = 1
           if (latitude<ini_lat) then 
              latitude = latitude + 180
           endif
        endif

        temp_flon =  (WSTfd1(flon,clat,1) -  WSTfd1(flon,flat,1))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &    +  WSTfd1(flon,flat,1)

        temp_clon =  (WSTfd1(clon,clat,1) -  WSTfd1(clon,flat,1))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &   +  WSTfd1(clon,flat,1)



        interpolate_value1 = (temp_clon - temp_flon)/step 
     &  * (longitude - (ini_lon+step*dfloat(flon-1))) +  temp_flon

c$$$
c$$$
c$$$    write(*,*) latitude,ini_lat+step*dfloat(flat-1),ini_lat + step*dfloat(clat-1) 
c$$$    write(*,*) longitude,ini_lon +step*dfloat(flon-1) 
c$$$    write(*,*) flon, clat,WSTfd1(flon,clat,1)
c$$$    write(*,*) flon, flat,WSTfd1(flon,flat,1)
c$$$    write(*,*) clon, clat,WSTfd1(clon,clat,1)
c$$$    write(*,*) clon, flat,WSTfd1(clon,flat,1)
c$$$    write(*,*) temp_flon, temp_clon
c$$$    write(*,*) interpolate_value1,WSTfd1(flon,flat,1)
c$$$    pause
c$$$
c$$$



        temp_flon =  (WSTfd2(flon,clat,1) -  WSTfd2(flon,flat,1))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &    +  WSTfd2(flon,flat,1)

        temp_clon =  (WSTfd2(clon,clat,1) -  WSTfd2(clon,flat,1))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &   +  WSTfd2(clon,flat,1)



        interpolate_value2 = (temp_clon - temp_flon)/step 
     &  * (longitude - (ini_lon+step*dfloat(flon-1))) +  temp_flon


        temperature = (interpolate_value2 - interpolate_value1) 
     &   * (time-initial_time)/step_time + interpolate_value1


c ----------


        temp_flon =  (WSTfd1(flon,clat,2) -  WSTfd1(flon,flat,2))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &    +  WSTfd1(flon,flat,2)

        temp_clon =  (WSTfd1(clon,clat,2) -  WSTfd1(clon,flat,2))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &   +  WSTfd1(clon,flat,2)



        interpolate_value1 = (temp_clon - temp_flon)/step 
     &  * (longitude - (ini_lon+step*dfloat(flon-1))) +  temp_flon



        temp_flon =  (WSTfd2(flon,clat,2) -  WSTfd2(flon,flat,2))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &    +  WSTfd2(flon,flat,2)

        temp_clon =  (WSTfd2(clon,clat,2) -  WSTfd2(clon,flat,2))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &   +  WSTfd2(clon,flat,2)



        interpolate_value2 = (temp_clon - temp_flon)/step 
     &  * (longitude - (ini_lon+step*dfloat(flon-1))) +  temp_flon


        salinity = (interpolate_value2 - interpolate_value1) 
     &   * (time-initial_time)/step_time + interpolate_value1


c -------------------



        temp_flon =  (WSTfd1(flon,clat,3) -  WSTfd1(flon,flat,3))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &    +  WSTfd1(flon,flat,3)

        temp_clon =  (WSTfd1(clon,clat,3) -  WSTfd1(clon,flat,3))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &   +  WSTfd1(clon,flat,3)



        interpolate_value1 = (temp_clon - temp_flon)/step 
     &  * (longitude - (ini_lon+step*dfloat(flon-1))) +  temp_flon



        temp_flon =  (WSTfd2(flon,clat,3) -  WSTfd2(flon,flat,3))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &    +  WSTfd2(flon,flat,3)

        temp_clon =  (WSTfd2(clon,clat,3) -  WSTfd2(clon,flat,3))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &   +  WSTfd2(clon,flat,3)



        interpolate_value2 = (temp_clon - temp_flon)/step 
     &  * (longitude - (ini_lon+step*dfloat(flon-1))) +  temp_flon


        u = (interpolate_value2 - interpolate_value1) 
     &   * (time-initial_time)/step_time + interpolate_value1




c -------------------


        temp_flon =  (WSTfd1(flon,clat,4) -  WSTfd1(flon,flat,4))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &    +  WSTfd1(flon,flat,4)

        temp_clon =  (WSTfd1(clon,clat,4) -  WSTfd1(clon,flat,4))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &   +  WSTfd1(clon,flat,4)



        interpolate_value1 = (temp_clon - temp_flon)/step 
     &  * (longitude - (ini_lon+step*dfloat(flon-1))) +  temp_flon



        temp_flon =  (WSTfd2(flon,clat,4) -  WSTfd2(flon,flat,4))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &    +  WSTfd2(flon,flat,4)

        temp_clon =  (WSTfd2(clon,clat,4) -  WSTfd2(clon,flat,4))/step 
     &  * (latitude - (ini_lat+step*dfloat((flat-1)))) 
     &   +  WSTfd2(clon,flat,4)



        interpolate_value2 = (temp_clon - temp_flon)/step 
     &  * (longitude - (ini_lon+step*dfloat(flon-1))) +  temp_flon


        v = (interpolate_value2 - interpolate_value1) 
     &   * (time-initial_time)/step_time + interpolate_value1




c -------------------



        call  unpol(u, v, wind_speed, wind_dir)

        longitude = temp_longitude
        latitude = temp_latitude

        return

        end


        function fun_floor(x)
        real*8 x
        integer fun_floor
        if(int(x)>x) then
           fun_floor= int(x) - 1
        else
           fun_floor= int(x)
        endif
        return
        end

        function fun_ceil(x)
        real*8 x
        integer fun_ceil
        if(int(x)<x) then
           fun_ceil= int(x) + 1
        else
           fun_ceil= int(x)
        endif
        return
        end
