#include <stdlib.h>
#include <libgen.h>
#include <string.h>
#include "hdf5.h"
#include "hdf5_hl.h"

int main (int argc, char* argv[])
{
    float surtemp[720][1440];
    float soilmoisture[181][360];
    
    float tsurf[361][540];
    float tsoil1[361][540];
    float sfmc[361][540];

    int i,j;
    
    // Open the yfile and get surface temperatures
    hid_t fid = H5Fopen(argv[1], H5F_ACC_RDWR, H5P_DEFAULT);
    herr_t status =
      H5LTread_dataset_float( fid, "Surface Temp", (float *) surtemp);

    hsize_t  offset[3];
    hsize_t  count[3];

    // Get hr from yfile name for GEOS file time dimension
    char *cptr = basename(argv[1]);
    offset[0] = atoi(cptr+9);
    offset[1] = 0;
    offset[2] = 0;

    // Access a single GEOS file time slice
    count[0] = 1;
    count[1]  = 361;
    count[2]  = 540;

    // Memory space for GEOS fields
    hsize_t dimsm[2]; 
    dimsm[0] = 361;
    dimsm[1] = 540;
    hid_t memspace = H5Screate_simple(2, dimsm, NULL);

    hid_t dataset, dataspace;

    // Open GEOS file
    hid_t fidgeos = H5Fopen(argv[2], H5F_ACC_RDONLY, H5P_DEFAULT);

    // Read GEOS TSURF
    dataset = H5Dopen2(fidgeos, "TSURF", H5P_DEFAULT);
    dataspace = H5Dget_space(dataset);    /* dataspace handle */

    status = H5Sselect_hyperslab(dataspace, H5S_SELECT_SET, offset, NULL,
				 count, NULL);

    status = H5Dread(dataset, H5T_NATIVE_FLOAT, memspace, dataspace,
		     H5P_DEFAULT, tsurf);

    H5Dclose(dataset);
    H5Sclose(dataspace);

    // Read GEOS TSOIL1
    dataset = H5Dopen2(fidgeos, "TSOIL1", H5P_DEFAULT);
    dataspace = H5Dget_space(dataset);    /* dataspace handle */

    status = H5Sselect_hyperslab(dataspace, H5S_SELECT_SET, offset, NULL,
				 count, NULL);

    status = H5Dread(dataset, H5T_NATIVE_FLOAT, memspace, dataspace,
		     H5P_DEFAULT, tsoil1);

    H5Dclose(dataset);
    H5Sclose(dataspace);


    // Read GEOS SFMC 
    dataset = H5Dopen2(fidgeos, "SFMC", H5P_DEFAULT);
    dataspace = H5Dget_space(dataset);    /* dataspace handle */

    status = H5Sselect_hyperslab(dataspace, H5S_SELECT_SET, offset, NULL,
				 count, NULL);

    status = H5Dread(dataset, H5T_NATIVE_FLOAT, memspace, dataspace,
		     H5P_DEFAULT, sfmc);

    H5Dclose(dataset);
    H5Sclose(dataspace);


    // Put GEOS land surface temperature into yfile field 
    int gi, gj; // GEOS map indicies
    for (j=0; j<720; j++) {
      float y = j + 0.125;
      gj = (int) (361 - (361.0/720)*y + 0.5);
	
      for (i=0; i<1440; i++) {
	float x = i + 0.125;
	if (i >= 0 && i < 720) {
	  gi = (int) ((540.0/1440)*x + (540/2) + 0.5);
	} else {
	  gi = (int) ((x - 720) * (540.0/1440) + 0.5);
	}

	// Skip if not land pixel
	if (tsurf[gj][gi] > 0.9999e+15)
	  continue;

	// Land surface temp is average of tsurf and tsoil1
	surtemp[j][i] = 0.5 * (tsurf[gj][gi] + tsoil1[gj][gi]);
      }
    }

    dataset = H5Dopen2(fid, "Surface Temp", H5P_DEFAULT);
    status = H5Dwrite(dataset, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL,
		      H5P_DEFAULT, surtemp);
    H5Dclose(dataset);



    // Put GEOS soil moisture into yfile field 
    for (j=0; j<181; j++) {
      float y = j + 0.0;

      gj = (int) (361 - (361.0/181)*y + 0.5);

	
      for (i=0; i<360; i++) {
	float x = i + 0.5;
	if (i >= 0 && i < 180) {
	  gi = (int) ((540.0/360)*x + 270 + 0.5);
	} else {
	  gi = (int) ((x - 181) * (540.0/360) + 0.5);
	}

	soilmoisture[j][i] = sfmc[gj][gi];
      }
    }

    dataset = H5Dopen2(fid, "Soil Moisture", H5P_DEFAULT);
    status = H5Dwrite(dataset, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL,
		      H5P_DEFAULT, soilmoisture);
    H5Dclose(dataset);

    H5Sclose(memspace);
    H5Fclose(fidgeos);

    // Read in input ancillary files attribute
    char rad_anc_name[16][80];
    hid_t attr = H5Aopen_name(fid, "Input Ancillary Files");
    hid_t type = H5Tcopy(H5T_C_S1);
    hid_t space_id = H5Aget_space(attr);
    hsize_t adim;
    H5Sget_simple_extent_dims(space_id, &adim, NULL);
    H5Tset_size(type, 80);
    H5Aread(attr, type, rad_anc_name);
    H5Aclose(attr);

    // Delete attribute
    status = H5Adelete( fid, "Input Ancillary Files");

    // Add geos ancillary filename
    strcpy(rad_anc_name[adim], basename(argv[2]));
    adim++;

    // Create and write new attribute
    hid_t aid = H5Screate_simple(1, &adim, NULL);
    attr = H5Acreate1(fid, "Input Ancillary Files", type, aid, H5P_DEFAULT);
    status = H5Awrite(attr, type, rad_anc_name);

    H5Aclose(attr);
    H5Tclose(type);
    H5Fclose(fid);
    
    return 0;
}

