      subroutine equin(iy1,id1,s1,iy2,id2,s2,v1,v2)

c Rotate vector from mean equator and equinox 
c  of year, day, time iy1, id1, s1 to iy2, id2, s2.
c  Uses first-order approximation accurate to about 0.01 degree

c  Calling Arguments:
c
c  Name         Type    I/O     Description
c  --------     ----    ---     -----------
c  iy1          I*4      I      Year of epoch 1
c  id1          I*4      I      Day of epoch 1
c  s1           I*4      I      Seconds of epoch 1
c  iy2          I*4      I      Year of epoch 2
c  id2          I*4      I      Day of epoch 2
c  s2           I*4      I      Seconds of epoch 2
c  v1(3)        R*8      I      Vector at epoch 1   
c  v1(3)        R*8      O      Vector at epoch 2   

c Borrowed from FORTRAN code developed by M. Shear
c  F. S. Patt, SAIC, July 19, 2011
c
c Added terms to improve accuracy
c  F. S. Patt, SAIC, July 26, 2011

      real*8 s1,s2
      real*8 dt,phi2,phi3
      real*8 v1(3),v2(3)
      integer*4 iy1,id1,iy2,id2
      dt = jd(iy2,1,id2) - jd(iy1,1,id1) + (s2 - s1)/864.d2

      phi2 = 2.6604171d-7*dt
      phi3 = -6.1223057d-7*dt
      phi23 = phi2*phi3/2.d0
      phi23sq = phi2*phi2 + phi3*phi3

      v2(1) = v1(1)*(1.d0 - phi23sq/2.d0) + phi3*v1(2) - phi2*v1(3)
      v2(2) = v1(2)*(1.d0 - phi3*phi3/2.d0) - phi3*v1(1) + phi23*v1(3)
      v2(3) = v1(3)*(1.d0 - phi2*phi2/2.d0) + phi2*v1(1) + phi23*v1(2)
      
      return
      end
