      subroutine get_nut(iyr,iday,xnut)

c  Subroutine to get nutation matrix for TOD-to-MOD conversion

c  Calling Arguments:
c
c  Name         Type    I/O     Description
c  --------     ----    ---     -----------
c  iyr          I*4      I      Year
c  iday         I*4      I      Day of year
c  xnut(3,3)    R*8      O      Nutation matrix

c  By: Frederick S. Patt, SAIC, July 19, 2011
c
c  Notes:  Reference GTDS Mathematical Theory, Revision 1, July 1989
c
c  Modification History:
c


      implicit none
      common /gconst/pi,radeg,re,rem,f,omf2,omegae

      real*8 xnut(3,3)
      real*8 t,xls,gs,xlm,omega,dpsi,eps,epsm
      real*8 pi,radeg,re,rem,f,omf2,omegae
      integer*4 iyr,iday,jd,imon/1/

c  Get nutation in longitude and obliquity
      t = jd(iyr,imon,iday) - 2451545.5d0
      call ephparms(t,xls,gs,xlm,omega)
      call nutate(t,xls,gs,xlm,omega,dpsi,eps,epsm)
      
c  Compute matrix
      xnut(1,1) = cos(dpsi/radeg)
      xnut(2,1) = -sin(dpsi/radeg)*cos(epsm/radeg)
      xnut(3,1) = -sin(dpsi/radeg)*sin(epsm/radeg)
      xnut(1,2) = sin(dpsi/radeg)*cos(eps/radeg)
      xnut(2,2) = cos(dpsi/radeg)*cos(eps/radeg)*cos(epsm/radeg) 
     *     + sin(eps/radeg)*sin(epsm/radeg)
      xnut(3,2) = cos(dpsi/radeg)*cos(eps/radeg)*sin(epsm/radeg) 
     *     - sin(eps/radeg)*cos(epsm/radeg)
      xnut(1,3) = sin(dpsi/radeg)*sin(eps/radeg)
      xnut(2,3) = cos(dpsi/radeg)*sin(eps/radeg)*cos(epsm/radeg) 
     *     - cos(eps/radeg)*sin(epsm/radeg)
      xnut(3,3) = cos(dpsi/radeg)*sin(eps/radeg)*sin(epsm/radeg) 
     *     + cos(eps/radeg)*cos(epsm/radeg)

      return
      end

      
