      subroutine orb2lla(orb,xlon,xlat,alt)

c Compute geodetic longitude, latitude and altitude 
c  from Cartesian orbit vector in ECR coordinates

c  Calling Arguments:
c
c     Name         Type    I/O     Description
c     --------     ----    ---     -----------
c     orb(3)       R*8      I      Orbit position vector in ECEF frame
c     xlon         R*8      O      Longitude
c     xlat         R*8      O      Geodetic latitude
c     alt          R*8      O      Altitude above WGS84 elipsoid

c  By: Frederick S. Patt, SAIC, July 19, 2011
c
c  Notes:  Uses geodetic approximation from Patt and Gregg, IJRS, 1993.
c
c  Modification History:
c

      implicit none

c Define constants
#include "nav_cnst.fin"

      real*8 omf2p
      real*8 orb(3),xlon,xlat,alt
      real*4 rad,pxy,temp,rl

c Compute longitude
      xlon = radeg*atan2(orb(2),orb(1))

c Compute geodetic latitude 
      rad = sqrt(orb(1)*orb(1)+orb(2)*orb(2)+orb(3)*orb(3))
      omf2p = (omf2*rem + rad - rem)/rad
      pxy = orb(1)*orb(1)+orb(2)*orb(2)
      temp = sqrt(orb(3)*orb(3) + omf2p*omf2p*pxy)
      xlat = radeg*asin(orb(3)/temp)

c Compute altitude

      rl = re*(1.-f)/sqrt(1.-(2.-f)*f*pxy/rad**2)
      alt = rad - rl

      return
      end
