      subroutine read_gps_from_hkt(hlun,iyr,iday,sec,gvec,ms,
     *     csec,hdata)

c     Read GPS data from one SAC-D HKT record

c     Calling Arguments:
c
c     Name         Type    I/O     Description
c     --------     ----    ---     -----------
c     hlun         I*4      I      FORTRAN logical unit for HKT file
c     iyr          I*4      O      Year of HKT file
c     iday         I*4      O      Day of HKT file
c     sec          R*8      O      Seconds of GPS time tag
c     gvec(6)      R*4      O      GPS vector
c     ms           I*4      O      Number of GPS satellites tracked
c     csec         R*8      O      Seconds of CDH time tag
c     hdata        L*4      O      End of data flag

c  By: Frederick S. Patt, SAIC, July 19, 2011
c
c  Notes: .
c
c  Modification History:
c
c  Changed time offset to constant value of 0.02 based on input from CONAE
c  F. S. Patt, SAIC, July 25, 2011

      implicit none
      real*8 sec
      real*4 gvec(6)
      real*4 gpscal/128000./
      integer*4 hlun,igyr,igday,ms,ns,csec
      integer*4 gptag,gpns,isec,ioff,ivec,cdt,iyr,iday
      integer irec/1/, loff/580/, i
      byte hrec(4000),aocs(1024)
      logical hdata 
      
c  Read HKT record from file
      read (hlun,err=990,rec=irec) hrec

c  Extract CDH time
      call read_long(cdt,101,hrec)
      call gps2yds(cdt,iyr,iday,csec)

c  Byte-swap and transfer AOCS data
      do i = 1,512
         aocs(2*i-1) = hrec(loff + 2*i)
         aocs(2*i) = hrec(loff + 2*i-1)
      end do

c  Extract GPS time tag
      call read_long(gptag,239,aocs)
      call read_long(gpns,243,aocs)
      call gps2yds(gptag,igyr,igday,isec)
      sec = isec + 0.02d0 ! Sum of ns and clock bias is constant
c      sec = isec + gpns/1.d9

c      print *,irec,gptag,gpns,igyr,igday,sec

c  Extract GPS vector
      do i=1,6
         ioff = 207 + 4*(i-1)
         call read_long(ivec,ioff,aocs)
         gvec(i) = ivec/gpscal
      end do

c  Extract number of GPS satellites tracked
      call read_long(ns,197,aocs)
      ms = mod(ns,16)
c      print *,ns,ms,gvec

      irec = irec + 1

      return

 990  hdata = .false.
      print *,'End of SAC-D HKT data at record',irec
      return

      end

      
