      subroutine write_cods_files(orbitr, orbitt, ier)

c  Subroutine to write Aquarius orbit ephemeris data in CODS format

c  Calling Arguments:
c
c  Name         Type    I/O     Description
c  --------     ----    ---     -----------
c  orbitr	struct	 I	Output orbit data structure in ECEF frame
c  orbitt	struct	 I	Output orbit data structure in TOD frame
c  ier		I*4	 I	Error code:	=0, success

c  By: Frederick S. Patt, SAIC, July 19, 2011
c
c  Notes:  
c
c  Modification History:
c
c  Modified to not write the 00:00 record at the start of the next day
c  to the definitive files.
c  F. S. Patt, SAIC, August 4, 2011

      implicit none
c
#include "nav_cnst.fin"
#include "orbit_s.fin"
c
c
      type(orbit_struct) :: orbitr     
      type(orbit_struct) :: orbitt     

      real*8 sec,pos2k(3),vel2k(3),xlat,xlon,alt
      real*8 posmd(3),velmd(3),xnutate(3,3)
c      real*4 xlat,xlon,alt
      integer*4 iyr,mon,idm,ihr,min,isc,numpred
      integer ier,lun,i,j,id,nd,numdef/1440/
      character*80 prej2k,pretod,pregeo,orbj2k,orbtod,orbgeo
      character*20 cname,headstr1/'*HEADER SAC-D       '/
      character*34 headj2k/' XYZ        UTC        MJ2K       '/
      character*34 headtod/' XYZ        UTC        TOD        '/
      character*34 headgeo/' LLH        UTC        WGS84      '/
      character*40 headstr2/'PRO        26620 00075B      CONAE      '/

c  Get file string from calling arguments and generate CODS file names
      call getarg(2, cname)
      prej2k = cname//'_SACD_PREDEPHEM_MJ2K_XYZ_O.TXT'      
      orbj2k = cname//'_SACD_ORBEPHEM_MJ2K_XYZ_O.TXT'      
      pretod = cname//'_SACD_PREDEPHEM_TOD_XYZ_O.TXT'      
      orbtod = cname//'_SACD_ORBEPHEM_TOD_XYZ_O.TXT'   
      pregeo = cname//'_SACD_PREDEPHEM_GEOD_LLH_O.TXT'      
      orbgeo = cname//'_SACD_ORBEPHEM_GEOD_LLH_O.TXT'
c      print *,orbj2k

c  Open CODS files
      open (41, file=orbj2k, status='new')
      open (42, file=prej2k, status='new')
      open (43, file=orbtod, status='new')
      open (44, file=pretod, status='new')
      open (45, file=orbgeo, status='new')
      open (46, file=pregeo, status='new')

c  First definitive files
c  Get time tag from first record
      call yds2ymdhms(orbitt%iyr,orbitt%iday,orbitt%torb(1),
     *     iyr,mon,idm,ihr,min,sec)
      isc = sec
c  Write header records
      write (41,1201) headstr1,numdef,headj2k,headstr2,
     *     iyr,mon,idm,ihr,min,isc
      write (43,1201) headstr1,numdef,headtod,headstr2,
     *     iyr,mon,idm,ihr,min,isc
      write (45,1201) headstr1,numdef,headgeo,headstr2,
     *     iyr,mon,idm,ihr,min,isc
 1201 format(a20,i5,a34,a40,i4,2('/',i2.2),1x,i2.2,2(':',i2.2))

c  Get nutation matrix
      call get_nut(orbitt%iyr,orbitt%iday,xnutate)

c  Write data records
      do i=1,numdef
         call yds2ymdhms(orbitt%iyr,orbitt%iday,orbitt%torb(i),
     *        iyr,mon,idm,ihr,min,sec)
         call matvec2(xnutate,3,orbitt%pos(1,i),posmd)
         call matvec2(xnutate,3,orbitt%vel(1,i),velmd)
         call equin(orbitt%iyr,orbitt%iday,orbitt%torb(i),2000,1,0.d0,
     *        posmd,pos2k)
         call equin(orbitt%iyr,orbitt%iday,orbitt%torb(i),2000,1,0.d0,
     *        velmd,vel2k)
         write(41,1202) iyr,mon,idm,ihr,min,sec,pos2k,vel2k
         write(43,1202) iyr,mon,idm,ihr,min,sec,
     *        (orbitt%pos(j,i),j=1,3),(orbitt%vel(j,i),j=1,3)
 1202    format(i4,2('/',i2.2),1x,2(i2.2,':'),'0',f10.8,3f15.6,3f15.10)
         call orb2lla(orbitr%pos(1,i),xlon,xlat,alt)
         write (45,1203)  iyr,mon,idm,ihr,min,sec,xlat,xlon,alt
 1203 format(i4,2('/',i2.2),1x,2(i2.2,':'),'0',f10.8,2f15.5,f15.3)
      end do
     
c  Next predictive files
c  Get time tag from first record
      call yds2ymdhms(orbitt%iyr,orbitt%iday,orbitt%torb(numdef),
     *     iyr,mon,idm,ihr,min,sec)
      isc = sec
      numpred = orbitt%nvec - numdef

c  Write header records
      write (42,1201) headstr1,numpred,headj2k,headstr2,
     *     iyr,mon,idm,ihr,min,isc
      write (44,1201) headstr1,numpred,headtod,headstr2,
     *     iyr,mon,idm,ihr,min,isc
      write (46,1201) headstr1,numpred,headgeo,headstr2,
     *     iyr,mon,idm,ihr,min,isc

c  Write data records
      do i=numdef+1,orbitt%nvec
          
         call yds2ymdhms(orbitt%iyr,orbitt%iday,orbitt%torb(i),
     *        iyr,mon,idm,ihr,min,sec)
         call matvec2(xnutate,3,orbitt%pos(1,i),posmd)
         call matvec2(xnutate,3,orbitt%vel(1,i),velmd)
         call equin(orbitt%iyr,orbitt%iday,orbitt%torb(i),2000,1,0.d0,
     *        posmd,pos2k)
         call equin(orbitt%iyr,orbitt%iday,orbitt%torb(i),2000,1,0.d0,
     *        velmd,vel2k)
         write(42,1202) iyr,mon,idm,ihr,min,sec,pos2k,vel2k
         write(44,1202) iyr,mon,idm,ihr,min,sec,
     *        (orbitt%pos(j,i),j=1,3),(orbitt%vel(j,i),j=1,3)
         call orb2lla(orbitr%pos(1,i),xlon,xlat,alt)
         write(46,1203)  iyr,mon,idm,ihr,min,sec,xlat,xlon,alt

      end do
     
      do lun = 41,46
         close(lun)
      end do

      return
      end
