#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <iomanip>
#include <math.h>
#include <iostream>
#include <sstream>
#include <fstream>
//#include <stack>
//#include <list>
#include "calibrate_smap.h"
//#include <clo.h>
#include <getopt.h>

#define VERSION "0.24"

//    Modification history:
//  Programmer     Organization   Date     Ver   Description of change
//  ----------     ------------   ----     ---   ---------------------
//  Joel Gales     FutureTech     09/01/16 0.10  Original development
//                                               based on MATLAB routines
//                                               developed by P. Mohammed-Tano
//  Joel Gales     FutureTech     09/20/16 0.11  Set kurtosis RFIflags to 1
//                                               if kurt = -9999
//  Joel Gales     FutureTech     10/20/16 0.12  Add nedt, bitflags, history
//                                               attribute
//  Joel Gales     FutureTech     12/16/16 0.20  Add support for low-rate scans
//  Joel Gales     FutureTech     12/22/16 0.22  Add fields/attributes to
//                                               support mk_rss_l1a_smap
//  Joel Gales     FutureTech     01/03/17 0.23  Refactor code to fix remaining
//                                               problems with low-rate scans
//  Joel Gales     FutureTech     01/04/17 0.24  Add support for tb qual flags

/*
g++ -g -o TAgen_smap TAgen_smap.cpp -I/disk02/joel/ocssw/build/lib3/include/ /disk02/joel/ocssw/build/cbuild/src/libtimeutils/libtimeutils.a -L/disk02/joel/ocssw/build/lib3/lib/ -lhdf5_hl -lhdf5 -lz

 */

// (nPRI_ANT/4)-last_PKTs = nPRI_REF

uint32_t nPRI_ANT; // 7820 = 4 * 1955 = 4 * 1944 + 44
uint32_t nPRI_REF; // 1944 = 4 *  486

void insertion_sort(float a[], int length);
extern "C" double ymds2unix(int16_t year, int16_t month, int16_t day, 
                            double secs);
extern "C" void unix2yds(double usec, int16_t *year, int16_t *day, 
                         double *secs);

int main (int argc, char* argv[])
{

  int c;
  int subbandTA_flag=0;

  // Handle keyword options
  string outlist = "";
  while (1) {
    static struct option long_options[] =
      {
        {"subbandTA", no_argument,  &subbandTA_flag, 1},
        {0, 0, 0, 0}
      };
    
      /* getopt_long stores the option index here. */
      int option_index = 0;

      c = getopt_long (argc, argv, "",
                       long_options, &option_index);

      /* Detect the end of the options. */
      if (c == -1)
        break;
  }

  string parameters[8];
  string *L1Afile=&parameters[0];
  string *TAfile=&parameters[1];
  string *RFIKurtosisParameters=&parameters[2];
  string *RFIParameters=&parameters[3];
  string *ReflectorParameters=&parameters[4];
  string *FullBandCoeffs=&parameters[5];
  string *SubBandCoeffs=&parameters[6];
  string *LinearCoeffs=&parameters[7];

  cout << "calibrate_smap " << VERSION << " (" 
       <<  __DATE__ << " " << __TIME__ << ")" << endl;

  string history="";
  if ((argc-optind) == 0) {
    cout << "calibrate_smap parfile" << endl;
    cout << "parfile includes L1Afile TAfile RFIKurtosisParameters RFIParameters ReflectorParameters FullBandCoeffs SubBandCoeffs LinearCoeffs" << endl;
    return 0;
  } else if ((argc-optind) == 1) {
    ifstream parfile;
    parfile.open ( argv[optind], ifstream::in);
    if ( parfile.fail() == true) {
      cout << "Cannot open: " << argv[optind] << endl;
      exit(1);
    }

    int i=0;
    string sLine;
    while( !parfile.eof()) {
      getline( parfile, sLine);
      if ( sLine.size() != 0) {
        cout << sLine.c_str() << endl;
        history.append(basename(sLine.c_str()));
        history.append(",");
        parameters[i].assign(sLine);
        i++;
      }
    }
    parfile.close();
  }
  //  cout << history.c_str() << endl;

  // Open SMAP L1A radiometer file and create SMAP TA radiometer file
  l1aSMAPFile l1aFile;
  l1aFile.openl1( L1Afile->c_str(), TAfile->c_str(), history);
  
  //  uint32_t nscans = l1aFile.nscans;

  // Load RFI threshold Kurtosis parameters for Cal Counts
  cout << "Loading RFI Kurtosis Cal Parameters" << endl;
  rfiThresh_k_struct rfiKurtosisParam_Cal;
  loadRFIKurtosisParam( RFIKurtosisParameters->c_str(), "RFIthresh_k_cal",
                        &rfiKurtosisParam_Cal);


  // Allocate RFI flags and Kurtosis output arrays for reference counts
  rfiFlagsKurtosis_struct RFIflagsKurtosis;
  l1aFile.allocateRFIFlagsKurtosis( nPRI_REF, &RFIflagsKurtosis);

  double lat1 = -90;
  double lon1 = 0;
  //  double dlat = 1;
  //double dlon = 1;

  uint32_t highresolution_scan_index[MAXSCANS];
  for (size_t i=0; i<MAXSCANS; i++) highresolution_scan_index[i] = 0;
  readU32( l1aFile.gid[HIRESMOM], "highresolution_scan_index", 
           (uint32_t *) highresolution_scan_index);

  // Generate Kurtosis/RFI flags output for reference counts
  cout << "Generating Kurtosis/RFI flags for reference counts" << endl;
  l1aFile.kurtosisCalData( "Ref", lat1, lon1, highresolution_scan_index,
                           &rfiKurtosisParam_Cal, &RFIflagsKurtosis);


  // Allocate RFI flags and Kurtosis output arrays for reference+ND counts
  rfiFlagsKurtosis_struct RFIflagsKurtosisND;
  l1aFile.allocateRFIFlagsKurtosis( nPRI_REF, &RFIflagsKurtosisND);

  // Generate Kurtosis/RFI flags output for reference+ND counts
  cout << "Generating Kurtosis/RFI flags for reference+ND counts" << endl;
  l1aFile.kurtosisCalData( "RefND", lat1, lon1, highresolution_scan_index,
                           &rfiKurtosisParam_Cal, &RFIflagsKurtosisND);

  delete rfiKurtosisParam_Cal.RFIthresh;
  delete rfiKurtosisParam_Cal.RFIthresh16;


  uint32_t nPRI_fp = 32;
  // footprintsPerScan has "nscans" entries 
  uint16_t footprintsPerScan[MAXSCANS];
  readU16( l1aFile.gid[SCDATA], "footprints_per_scan", 
           (uint16_t *) footprintsPerScan);

  uint32_t szREF = l1aFile.nscans*nPRI_REF*4;
  uint32_t szREF16 = l1aFile.nscans*(nPRI_REF/4)*16*4;
  
  rfiFlagsCalData_struct RFIflagsCalData;
  RFIflagsCalData.RFIflag_ref_td1 = new uint8_t[szREF];
  RFIflagsCalData.RFIflag_refnd_td1 = new uint8_t[szREF];
  RFIflagsCalData.RFIflag16_ref_cf1 = new uint8_t[szREF16];
  RFIflagsCalData.RFIflag16_refnd_cf1 = new uint8_t[szREF16];
  memset(RFIflagsCalData.RFIflag_ref_td1, 0, szREF);
  memset(RFIflagsCalData.RFIflag_refnd_td1, 0, szREF);
  memset(RFIflagsCalData.RFIflag16_ref_cf1, 0, szREF16);
  memset(RFIflagsCalData.RFIflag16_refnd_cf1, 0, szREF16);

  cout << "Generating RFI flags for Cal Data" << endl;
  l1aFile.rfiDetectCalData( RFIParameters->c_str(), highresolution_scan_index, 
                            footprintsPerScan, lat1, lon1, nPRI_fp, 
                            &RFIflagsCalData);

  calDataNoRFI_struct calDataNoRFI;
  calDataNoRFI.m2_ref = new float[szREF];
  calDataNoRFI.m2_ref_nd = new float[szREF];
  calDataNoRFI.m2_16_ref = new float[szREF16];
  calDataNoRFI.m2_16_ref_nd = new float[szREF16];
  
  calDataNoRFI.t3_ref = new float[szREF];
  calDataNoRFI.t3_ref_nd = new float[szREF];
  calDataNoRFI.t3_16_ref = new float[szREF16/4];
  calDataNoRFI.t3_16_ref_nd = new float[szREF16/4];
  
  calDataNoRFI.t4_ref = new float[szREF];
  calDataNoRFI.t4_ref_nd = new float[szREF];
  calDataNoRFI.t4_16_ref = new float[szREF16/4];
  calDataNoRFI.t4_16_ref_nd = new float[szREF16/4];

  cout << "Removing RFI flagged Cal Data" << endl;
  l1aFile.rfiRemovalCalData( footprintsPerScan, highresolution_scan_index, 
                             &RFIflagsKurtosis, &RFIflagsKurtosisND, 
                             &RFIflagsCalData, &calDataNoRFI);

  delete( RFIflagsCalData.RFIflag_ref_td1); 
  delete( RFIflagsCalData.RFIflag_refnd_td1); 
  delete( RFIflagsCalData.RFIflag16_ref_cf1); 
  delete( RFIflagsCalData.RFIflag16_refnd_cf1); 


  ofstream noRFIOut;

  string surface_temp_selection("center");
  double cal_loss12_radome[MAXSCANS][2];
  double cal_temp12_radome[MAXSCANS];
  cout << "Computing Radome Characteristics" << endl;
  l1aFile.radomeCharacteristics( ReflectorParameters->c_str(),
                                 surface_temp_selection,
                                 cal_loss12_radome, cal_temp12_radome);

  fullCalGroup_struct fullCalGroup;
  subCalGroup_struct subCalGroup;

  for (size_t i=0; i<l1aFile.nscans; i++) {
    fullCalGroup.cal_loss12_radome[i][0] = cal_loss12_radome[i][0];
    fullCalGroup.cal_loss12_radome[i][1] = cal_loss12_radome[i][1];
    fullCalGroup.cal_temp12_radome[i] = cal_temp12_radome[i];
  }

  for (size_t i=0; i<l1aFile.nhires; i++) {
    subCalGroup.cal_loss12_radome16[i][0] =
      cal_loss12_radome[highresolution_scan_index[i]][0];
    subCalGroup.cal_loss12_radome16[i][1] =
      cal_loss12_radome[highresolution_scan_index[i]][1];
    subCalGroup.cal_temp12_radome16[i] =
      cal_temp12_radome[highresolution_scan_index[i]];
  }  
  
  deleteRFIFlagsKurtosis( &RFIflagsKurtosis);
  deleteRFIFlagsKurtosis( &RFIflagsKurtosisND);

  // Load Full/SubBand calibration coefficients
  fullBandCoeffs_struct fullBandCoeffs;
  subBandCoeffs_struct subBandCoeffs;

  cout << "Loading Band Coefficients" << endl;
  loadCoeffs( FullBandCoeffs->c_str(), SubBandCoeffs->c_str(),
              &fullBandCoeffs, &subBandCoeffs);

  cout << "Computing Calibration Coefficients" << endl;
  l1aFile.calCoefficient( &fullBandCoeffs, &subBandCoeffs,
                          &fullCalGroup, &subCalGroup,
                          highresolution_scan_index);

  linearCoeffs_struct linearCoeffs;
  loadLinearCoeffs( LinearCoeffs->c_str(), &linearCoeffs);

  cout << "Generating Antenna Temperatures" << endl;
  TAs_struct TAs;
  l1aFile.calibrationNew_FPS( &linearCoeffs, &fullBandCoeffs, &subBandCoeffs,
                              &fullCalGroup, &subCalGroup, &calDataNoRFI,
                              highresolution_scan_index, footprintsPerScan,
                              &TAs);

  delete( calDataNoRFI.m2_ref);
  delete( calDataNoRFI.m2_ref_nd);
  delete( calDataNoRFI.m2_16_ref);
  delete( calDataNoRFI.m2_16_ref_nd);
  delete( calDataNoRFI.t3_ref);
  delete( calDataNoRFI.t3_ref_nd);
  delete( calDataNoRFI.t3_16_ref);
  delete( calDataNoRFI.t3_16_ref_nd);
  delete( calDataNoRFI.t4_ref);
  delete( calDataNoRFI.t4_ref_nd);
  delete( calDataNoRFI.t4_16_ref);
  delete( calDataNoRFI.t4_16_ref_nd);

  ofstream tempOut;
  //tempOut.open ("TA_h.bin", ios::out | ios::trunc | ios::binary);
  //tempOut.write((char *) TAs.TA_h, nscans*(nPRI_ANT-last_PRIs)*sizeof(float));
  //tempOut.close();

  tempOut.open ("TA16_h.bin", ios::out | ios::trunc | ios::binary);
  tempOut.write((char *) TAs.TA16_h,
                l1aFile.nhires*((nPRI_ANT/4)-last_PKTs)*16*sizeof(float));
  tempOut.close();

    
  if ( subbandTA_flag) {
    cout << "Writing subband TA values to output file" << endl;
    
    l1aFile.gid_ta = H5Gopen1( l1aFile.h5fid_ta, "/Brightness_Temperature");
    hsize_t dims[3];
    dims[0] = l1aFile.nscans;
    dims[1] = (nPRI_ANT/4)-last_PKTs;
    dims[2] = 16;

    H5LTmake_dataset( l1aFile.gid_ta, "TA16_v", 3, dims, 
                      H5T_NATIVE_FLOAT, TAs.TA16_v);
    H5LTmake_dataset( l1aFile.gid_ta, "TA16_h", 3, dims, 
                      H5T_NATIVE_FLOAT, TAs.TA16_h);
    H5LTmake_dataset( l1aFile.gid_ta, "T3_16", 3, dims, 
                      H5T_NATIVE_FLOAT, TAs.T3_16);
    H5LTmake_dataset( l1aFile.gid_ta, "T4_16", 3, dims, 
                      H5T_NATIVE_FLOAT, TAs.T4_16);
    H5Gclose( l1aFile.gid_ta); 
  }


  uint32_t szANT = l1aFile.nscans*(nPRI_ANT*4);  
  float *m1_ant = new float[szANT];
  float *m2_ant = new float[szANT];
  float *m3_ant = new float[szANT];
  float *m4_ant = new float[szANT];

  readFloat( l1aFile.gid[MOMDATA], "m1_ant", m1_ant);
  readFloat( l1aFile.gid[MOMDATA], "m2_ant", m2_ant);
  readFloat( l1aFile.gid[MOMDATA], "m3_ant", m3_ant);
  readFloat( l1aFile.gid[MOMDATA], "m4_ant", m4_ant);

  uint32_t szANT16 = l1aFile.nscans*(nPRI_ANT/4)*16*4;
  float *m1_16_ant = new float[szANT16];
  float *m2_16_ant = new float[szANT16];
  float *m3_16_ant = new float[szANT16];
  float *m4_16_ant = new float[szANT16];

  readFloat( l1aFile.gid[HIRESMOM], "m1_16_ant", m1_16_ant);
  readFloat( l1aFile.gid[HIRESMOM], "m2_16_ant", m2_16_ant);
  readFloat( l1aFile.gid[HIRESMOM], "m3_16_ant", m3_16_ant);
  readFloat( l1aFile.gid[HIRESMOM], "m4_16_ant", m4_16_ant);

  // Load RFI threshold Kurtosis parameters for Data Counts
  rfiThresh_k_struct rfiKurtosisParam_Data;
  cout << "Loading RFI Kurtosis Data Parameters" << endl;
  loadRFIKurtosisParam( RFIKurtosisParameters->c_str(),
                        "RFIthresh_k_data", &rfiKurtosisParam_Data);

  rfiFlagsKurtosis_struct RFIflagsKurtosisData;
  l1aFile.allocateRFIFlagsKurtosis( nPRI_ANT, &RFIflagsKurtosisData);

  cout << "RFI Kurtosis Detection on Antenna Data" << endl;
  l1aFile.kurtosis_RFI_Detection( "ANT", &lat1, &lon1,
                                  highresolution_scan_index,
                                  m1_ant, m2_ant, m3_ant, m4_ant,
                                  m1_16_ant, m2_16_ant, m3_16_ant, m4_16_ant,
                                  &rfiKurtosisParam_Data, 
                                  &RFIflagsKurtosisData);

  delete( m1_ant);
  delete( m2_ant);
  delete( m3_ant);
  delete( m4_ant);
  delete( m1_16_ant);
  delete( m2_16_ant);
  delete( m3_16_ant);
  delete( m4_16_ant);

  rfiFlags_struct RFIflagsAnt;
  uint32_t szFLAG = l1aFile.nscans*(nPRI_ANT-last_PRIs);
  uint32_t szFLAG16 = l1aFile.nscans*nPRI_REF*16;

  RFIflagsAnt.RFIflag_h_td1 = new uint8_t[szFLAG];
  RFIflagsAnt.RFIflag_v_td1 = new uint8_t[szFLAG];
  RFIflagsAnt.RFIflag_t3 = new uint8_t[szFLAG];
  RFIflagsAnt.RFIflag_t4 = new uint8_t[szFLAG];

  RFIflagsAnt.RFIflag16_h_cf1 = new uint8_t[szFLAG16];
  RFIflagsAnt.RFIflag16_v_cf1 = new uint8_t[szFLAG16];
  RFIflagsAnt.RFIflag16_h_cf11 = new uint8_t[szFLAG16/8];
  RFIflagsAnt.RFIflag16_v_cf11 = new uint8_t[szFLAG16/8];
  RFIflagsAnt.RFIflag16_t3 = new uint8_t[szFLAG16];
  RFIflagsAnt.RFIflag16_t4 = new uint8_t[szFLAG16];

  memset(RFIflagsAnt.RFIflag_h_td1, 0, szFLAG);
  memset(RFIflagsAnt.RFIflag_v_td1, 0, szFLAG);
  memset(RFIflagsAnt.RFIflag_t3, 0, szFLAG);
  memset(RFIflagsAnt.RFIflag_t4, 0, szFLAG);

  memset(RFIflagsAnt.RFIflag16_h_cf1, 0, szFLAG16);
  memset(RFIflagsAnt.RFIflag16_v_cf1, 0, szFLAG16);
  memset(RFIflagsAnt.RFIflag16_h_cf11, 0, szFLAG16/8);
  memset(RFIflagsAnt.RFIflag16_v_cf11, 0, szFLAG16/8);
  memset(RFIflagsAnt.RFIflag16_t3, 0, szFLAG16);
  memset(RFIflagsAnt.RFIflag16_t4, 0, szFLAG16);

  cout << "Generating RFI flags for Antenna Data" << endl;
  l1aFile.rfiDetect( RFIParameters->c_str(), highresolution_scan_index,
                     footprintsPerScan, nPRI_fp, &TAs, &RFIflagsAnt);


  //  tempOut.open ("RFIflag_t3.bin", ios::out | ios::trunc | ios::binary);
  //tempOut.write((char *) RFIflagsAnt.RFIflag_t3, 
  //            nscans*(nPRI_ANT-last_PRIs)*sizeof(uint8_t));
  //tempOut.close();

  //tempOut.open ("RFIflag_t4.bin", ios::out | ios::trunc | ios::binary);
  //tempOut.write((char *) RFIflagsAnt.RFIflag_t4, 
  //            nscans*(nPRI_ANT-last_PRIs)*sizeof(uint8_t));
  //tempOut.close();

  cout << "Generating footprint TA values" << endl;
  // Note: Using TAs structure to hold no-filtered/filtered TA values
  l1aFile.MAXPD_Algorithm( &TAs, &RFIflagsAnt, &RFIflagsKurtosisData,
                           highresolution_scan_index);

  double cal_loss1_reflector[MAXSCANS][2];
  double cal_TB1_reflector[MAXSCANS][2];
  double cal_temp1_reflector[MAXSCANS][2];
  cout << "Computing Reflector Characteristics" << endl;
  l1aFile.reflectorCharacteristics( ReflectorParameters->c_str(),
                                    cal_loss1_reflector, cal_temp1_reflector, 
                                    cal_TB1_reflector);

  l1aFile.closel1( &TAs, cal_loss1_reflector, cal_temp1_reflector);

  deleteRFIFlagsKurtosis( &RFIflagsKurtosisData);

  delete( RFIflagsAnt.RFIflag_h_td1);
  delete( RFIflagsAnt.RFIflag_v_td1);
  delete( RFIflagsAnt.RFIflag_t3);
  delete( RFIflagsAnt.RFIflag_t4);

  delete( RFIflagsAnt.RFIflag16_h_cf1);
  delete( RFIflagsAnt.RFIflag16_v_cf1);
  delete( RFIflagsAnt.RFIflag16_h_cf11);
  delete( RFIflagsAnt.RFIflag16_v_cf11);
  delete( RFIflagsAnt.RFIflag16_t3);
  delete( RFIflagsAnt.RFIflag16_t4);

  delete( TAs.TA_v);
  delete( TAs.TA_h);
  delete( TAs.Gainv);
  delete( TAs.Gainh);
  delete( TAs.Offsetv);
  delete( TAs.Offseth);
  delete( TAs.T3);
  delete( TAs.T4);

  delete( TAs.TA16_v);
  delete( TAs.TA16_h);
  delete( TAs.Gainv16);
  delete( TAs.Gainh16);
  delete( TAs.Offsetv16);
  delete( TAs.Offseth16);
  delete( TAs.T3_16);
  delete( TAs.T4_16);

  delete( l1aFile.rfiThreshidx);
  delete( l1aFile.rfiThreshidx16);
  delete( l1aFile.rfiThreshidxFP);

  return 0;
}


l1aSMAPFile::l1aSMAPFile() {
}


l1aSMAPFile::~l1aSMAPFile() {
}

int deleteRFIFlagsKurtosis( rfiFlagsKurtosis_struct *RFIflagsKurtosis) {

  delete( RFIflagsKurtosis->RFIflag_kurt_h);
  delete( RFIflagsKurtosis->RFIflag_kurt_v);
  delete( RFIflagsKurtosis->RFIflag16_kurt_h);
  delete( RFIflagsKurtosis->RFIflag16_kurt_v);
  delete( RFIflagsKurtosis->kurtosis_h);
  delete( RFIflagsKurtosis->kurtosis_v);
  delete( RFIflagsKurtosis->kurtosis16_h);
  delete( RFIflagsKurtosis->kurtosis16_v);

  return 0;
}

int loadCoeffs( const char* fullBand_filename, const char* subBand_filename,
                fullBandCoeffs_struct *fullBandCoeffs,
                subBandCoeffs_struct *subBandCoeffs) {

  hid_t file_id, gid;
  herr_t status;

  file_id = H5Fopen ( fullBand_filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  gid = H5Gopen1( file_id, "FullBandCoeffs");

  readDouble( gid, "C1", (double *) &fullBandCoeffs->C1);
  readDouble( gid, "C2", (double *) &fullBandCoeffs->C2);
  readDouble( gid, "C3", (double *) &fullBandCoeffs->C3);
  readDouble( gid, "C4", (double *) &fullBandCoeffs->C4);
  readDouble( gid, "C5", (double *) &fullBandCoeffs->C5);
  readDouble( gid, "L1", (double *) &fullBandCoeffs->L1);
  readDouble( gid, "L2", (double *) &fullBandCoeffs->L2);
  readDouble( gid, "L3", (double *) &fullBandCoeffs->L3);
  readDouble( gid, "L4", (double *) &fullBandCoeffs->L4);
  readDouble( gid, "L5", (double *) &fullBandCoeffs->L5);
  readDouble( gid, "T1", (double *) &fullBandCoeffs->T1);
  readDouble( gid, "T2", (double *) &fullBandCoeffs->T2);
  readDouble( gid, "T3", (double *) &fullBandCoeffs->T3);
  readDouble( gid, "T4", (double *) &fullBandCoeffs->T4);
  readDouble( gid, "T5", (double *) &fullBandCoeffs->T5);
  readDouble( gid, "TDL0", (double *) &fullBandCoeffs->TDL0);
  readDouble( gid, "TDLoffset", (double *) &fullBandCoeffs->TDLoffset);
  readDouble( gid, "Tcoupler0", (double *) &fullBandCoeffs->Tcoupler0);
  readDouble( gid, "Tdiplexer0", (double *) &fullBandCoeffs->Tdiplexer0);
  readDouble( gid, "Tnd", (double *) &fullBandCoeffs->Tnd);
  readDouble( gid, "Tnd0", (double *) &fullBandCoeffs->Tnd0);
  readDouble( gid, "Tomt0", (double *) &fullBandCoeffs->Tomt0);
  readDouble( gid, "Txnd", (double *) &fullBandCoeffs->Txnd);
  readDouble( gid, "Txnd0", (double *) &fullBandCoeffs->Txnd0);
  readDouble( gid, "c1_DLoffset", (double *) &fullBandCoeffs->c1_DLoffset);
  readDouble( gid, "c1_nd", (double *) &fullBandCoeffs->c1_nd);
  readDouble( gid, "c1_xnd", (double *) &fullBandCoeffs->c1_xnd);
  readDouble( gid, "c2_DLoffset", (double *) &fullBandCoeffs->c2_DLoffset);
  readDouble( gid, "c2_nd", (double *) &fullBandCoeffs->c2_nd);
  readDouble( gid, "c2_xnd", (double *) &fullBandCoeffs->c2_xnd);
  readDouble( gid, "c3_DLoffset", (double *) &fullBandCoeffs->c3_DLoffset);
  readDouble( gid, "c3_nd", (double *) &fullBandCoeffs->c3_nd);
  readDouble( gid, "c3_xnd", (double *) &fullBandCoeffs->c3_xnd);
  readDouble( gid, "c4_DLoffset", (double *) &fullBandCoeffs->c4_DLoffset);
  readDouble( gid, "c4_nd", (double *) &fullBandCoeffs->c4_nd);
  readDouble( gid, "c4_xnd", (double *) &fullBandCoeffs->c4_xnd);
  readDouble( gid, "channel_phase",(double *) &fullBandCoeffs->channel_phase);
  readDouble( gid, "dgamma", (double *) &fullBandCoeffs->dgamma);
  readDouble( gid, "dpsi", (double *) &fullBandCoeffs->dpsi);
  readDouble( gid, "lambda", (double *) &fullBandCoeffs->lambda);
  readDouble( gid, "nd_phase", (double *) &fullBandCoeffs->nd_phase);
  readDouble( gid, "rx_phase", (double *) &fullBandCoeffs->rx_phase);
  readDouble( gid, "tcor_im", (double *) &fullBandCoeffs->tcor_im);
  readDouble( gid, "tcor_re", (double *) &fullBandCoeffs->tcor_re);
  readDouble( gid, "xnd_phase", (double *) &fullBandCoeffs->xnd_phase);

  H5Gclose( gid);
  H5Fclose( file_id);

  file_id = H5Fopen ( subBand_filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  gid = H5Gopen1( file_id, "SubBandCoeffs");

  readDouble( gid, "C1_16", (double *) &subBandCoeffs->C1_16);
  readDouble( gid, "C2_16", (double *) &subBandCoeffs->C2_16);
  readDouble( gid, "C3_16", (double *) &subBandCoeffs->C3_16);
  readDouble( gid, "C4_16", (double *) &subBandCoeffs->C4_16);
  readDouble( gid, "C5_16", (double *) &subBandCoeffs->C5_16);
  readDouble( gid, "L1_16", (double *) &subBandCoeffs->L1_16);
  readDouble( gid, "L2_16", (double *) &subBandCoeffs->L2_16);
  readDouble( gid, "L3_16", (double *) &subBandCoeffs->L3_16);
  readDouble( gid, "L4_16", (double *) &subBandCoeffs->L4_16);
  readDouble( gid, "L5_16", (double *) &subBandCoeffs->L5_16);
  readDouble( gid, "T1_16", (double *) &subBandCoeffs->T1_16);
  readDouble( gid, "T2_16", (double *) &subBandCoeffs->T2_16);
  readDouble( gid, "T3_16", (double *) &subBandCoeffs->T3_16);
  readDouble( gid, "T4_16", (double *) &subBandCoeffs->T4_16);
  readDouble( gid, "T5_16", (double *) &subBandCoeffs->T5_16);
  readDouble( gid, "TDL0_16", (double *) &subBandCoeffs->TDL0_16);
  readDouble( gid, "TDLoffset16", (double *) &subBandCoeffs->TDLoffset16);
  readDouble( gid, "Tcoupler0_16", (double *) &subBandCoeffs->Tcoupler0_16);
  readDouble( gid, "Tdiplexer0_16", (double *) &subBandCoeffs->Tdiplexer0_16);
  readDouble( gid, "Tnd0_16", (double *) &subBandCoeffs->Tnd0_16);
  readDouble( gid, "Tnd16", (double *) &subBandCoeffs->Tnd16);
  readDouble( gid, "Tomt0_16", (double *) &subBandCoeffs->Tomt0_16);
  readDouble( gid, "Txnd0_16", (double *) &subBandCoeffs->Txnd0_16);
  readDouble( gid, "Txnd16", (double *) &subBandCoeffs->Txnd16);
  readDouble( gid, "c1_DLoffset16", (double *) &subBandCoeffs->c1_DLoffset16);
  readDouble( gid, "c1_nd16", (double *) &subBandCoeffs->c1_nd16);
  readDouble( gid, "c1_xnd16", (double *) &subBandCoeffs->c1_xnd16);
  readDouble( gid, "c2_DLoffset16", (double *) &subBandCoeffs->c2_DLoffset16);
  readDouble( gid, "c2_nd16", (double *) &subBandCoeffs->c2_nd16);
  readDouble( gid, "c2_xnd16", (double *) &subBandCoeffs->c2_xnd16);
  readDouble( gid, "c3_DLoffset16", (double *) &subBandCoeffs->c3_DLoffset16);
  readDouble( gid, "c3_nd16", (double *) &subBandCoeffs->c3_nd16);
  readDouble( gid, "c3_xnd16", (double *) &subBandCoeffs->c3_xnd16);
  readDouble( gid, "c4_DLoffset16", (double *) &subBandCoeffs->c4_DLoffset16);
  readDouble( gid, "c4_nd16", (double *) &subBandCoeffs->c4_nd16);
  readDouble( gid, "c4_xnd16", (double *) &subBandCoeffs->c4_xnd16);
  readDouble( gid, "channel_phase16", 
              (double *) &subBandCoeffs->channel_phase16);
  readDouble( gid, "dgamma16", (double *) &subBandCoeffs->dgamma16);
  readDouble( gid, "lambda16", (double *) &subBandCoeffs->lambda16);
  readDouble( gid, "nd_phase16", (double *) &subBandCoeffs->nd_phase16);
  readDouble( gid, "psi16", (double *) &subBandCoeffs->psi16);
  readDouble( gid, "rx_phase16", (double *) &subBandCoeffs->rx_phase16);
  readDouble( gid, "tcor16_im", (double *) &subBandCoeffs->tcor16_im);
  readDouble( gid, "tcor16_re", (double *) &subBandCoeffs->tcor16_re);
  readDouble( gid, "xnd_phase16", (double *) &subBandCoeffs->xnd_phase16);

  H5Gclose( gid);
  H5Fclose( file_id);

  return 0;
}

int loadLinearCoeffs( const char* linearCoeffs_filename, 
                      linearCoeffs_struct *linearCoeffs) {

  hid_t file_id, gid;
  herr_t status;

  file_id = H5Fopen( linearCoeffs_filename, H5F_ACC_RDONLY, H5P_DEFAULT);

  gid = H5Gopen1( file_id, "C2");
  readDouble( gid, "c2_0", (double *) &linearCoeffs->C2.c2_0);
  readDouble( gid, "c2_1", (double *) &linearCoeffs->C2.c2_1);
  readDouble( gid, "c2_2", (double *) &linearCoeffs->C2.c2_2);
  H5Gclose(gid);

  gid = H5Gopen1( file_id, "C2_16");
  readDouble( gid, "c2_0", (double *) &linearCoeffs->C2_16.c2_0);
  readDouble( gid, "c2_1", (double *) &linearCoeffs->C2_16.c2_1);
  readDouble( gid, "c2_2", (double *) &linearCoeffs->C2_16.c2_2);
  H5Gclose(gid);

  gid = H5Gopen1( file_id, "C3");
  readDouble( gid, "c3_0", (double *) &linearCoeffs->C3.c3_0);
  readDouble( gid, "c3_1", (double *) &linearCoeffs->C3.c3_1);
  readDouble( gid, "c3_2", (double *) &linearCoeffs->C3.c3_2);
  H5Gclose(gid);

  gid = H5Gopen1( file_id, "C3_16");
  readDouble( gid, "c3_0", (double *) &linearCoeffs->C3_16.c3_0);
  readDouble( gid, "c3_1", (double *) &linearCoeffs->C3_16.c3_1);
  readDouble( gid, "c3_2", (double *) &linearCoeffs->C3_16.c3_2);
  H5Gclose(gid);

  readDouble( file_id, "Tref", (double *) &linearCoeffs->Tref);
  readDouble( file_id, "Tref16", (double *) &linearCoeffs->Tref16);

  readDouble( file_id, "h", (double *) &linearCoeffs->h);
  readDouble( file_id, "time_gap", (double *) &linearCoeffs->time_gap);

  return 0;
}

int loadRFIKurtosisParam( const char* rfiKurtosisParameters_filename,
                          const char *groupName,
                          rfiThresh_k_struct *rfiKurtosisParameters) {

  hid_t file_id, gid;
  herr_t status;

  file_id = H5Fopen ( rfiKurtosisParameters_filename,
                      H5F_ACC_RDONLY, H5P_DEFAULT);

  gid = H5Gopen1( file_id, groupName);

  uint32_t RFIthresh_sz;
  if ( strcmp( groupName, "RFIthresh_k_cal") == 0) 
    RFIthresh_sz = 180*360; 
  else if ( strcmp( groupName, "RFIthresh_k_data") == 0)
     RFIthresh_sz = 1800*3600*4;
 
  rfiKurtosisParameters->RFIthresh = new double[RFIthresh_sz];
  readDouble( gid, "RFIthresh", rfiKurtosisParameters->RFIthresh);

  rfiKurtosisParameters->RFIthresh16 = new double[RFIthresh_sz];
  readDouble( gid, "RFIthresh", rfiKurtosisParameters->RFIthresh16);

  readDouble( gid, "knom16_h", rfiKurtosisParameters->knom16_h);
  readDouble( gid, "knom16_v", rfiKurtosisParameters->knom16_v);
  readDouble( gid, "knom_h", &rfiKurtosisParameters->knom_h);
  readDouble( gid, "knom_v", &rfiKurtosisParameters->knom_v);

  readDouble( gid, "kstd16_h", rfiKurtosisParameters->kstd16_h);
  readDouble( gid, "kstd16_v", rfiKurtosisParameters->kstd16_v);
  readDouble( gid, "kstd_h", &rfiKurtosisParameters->kstd_h);
  readDouble( gid, "kstd_v", &rfiKurtosisParameters->kstd_v);

  H5Gclose( gid);
  H5Fclose( file_id);

  return 0;
}

int gainFilter( hsize_t nscans, size_t i, uint16_t *footprintsPerScan,  
                uint32_t nh, double *h, 
                double *gainv, double *gainh, double *gain, 
                double *offsetv, double *offseth,
                double *T3offset, double *T4offset, 
                double *Gainv, double *Gainh, double *Gain,
                double *Offsetv, double *Offseth,
                double *T3Offset, double *T4Offset) {

  int const bufsize = 100;
  int const pkt_fp = 8;

  int32_t scan1[bufsize];
  int32_t scan_idx1[bufsize*2];
  int32_t scan2[bufsize];
  int32_t scan_idx2[bufsize*2];
  int32_t scan[bufsize*2];
  int32_t scan_idx[bufsize*2*2];
  
  int Ncal = 0;

  double *gain_v = new double[nh];
  double *gain_h = new double[nh];
  double *gain_34 = new double[nh];
  double *offset_v = new double[nh];
  double *offset_h = new double[nh];
  double *t3offset = new double[nh];
  double *t4offset = new double[nh];

  for (size_t j=0; j<footprintsPerScan[i]; j++) {

    for (size_t k=0; k<bufsize; k++) {
      scan1[k] = -1;
      scan_idx1[2*k+0] = -1;
      scan_idx1[2*k+1] = -1;
      scan2[k] = -1;
      scan_idx2[2*k+0] = -1;
      scan_idx2[2*k+1] = -1;
    }

    int prv_samp1 = Ncal - (nh/2);
    int nxt_samp1 = Ncal + (nh/2);
    
    int A1, A2;
    int k;
    if ( prv_samp1 < 0 && i > 0) {
      A1 = prv_samp1;
      k = 0;

      while ( i-k > 0 && A1 < 0) {
        scan1[k] = i - k - 1;
        if ( (A1 + 2*footprintsPerScan[i-k-1]) < 0) {
          scan_idx1[2*k+0] = 0;
          scan_idx1[2*k+1] = 2*footprintsPerScan[i-k-1] - 1;
        } else {
          scan_idx1[2*k+0] = A1 + 2*footprintsPerScan[i-k-1];
          scan_idx1[2*k+1] = 2*footprintsPerScan[i-k-1] - 1;
        }
        A1 += 2*footprintsPerScan[i-k-1];
        k++;
      } // endwhile
    } else {
      A1 = prv_samp1;
    }

    if ( prv_samp1 < 0) prv_samp1 = 0;

    if ( nxt_samp1 > 2*footprintsPerScan[i]-1 && i == (nscans-1)) {
      scan2[0] = i;
      scan_idx2[0] = prv_samp1;
      scan_idx2[1] = 2*footprintsPerScan[i] - 1;
    } else if (nxt_samp1 > 2*footprintsPerScan[i]-1 && i < (nscans-1)) {
      A2 = nxt_samp1 - 2*footprintsPerScan[i] + 1;
      k = 0;
      scan2[k] = i;
      scan_idx2[2*k+0] = prv_samp1;
      scan_idx2[2*k+1] = 2*footprintsPerScan[i] - 1;

      while ( i+k < (nscans-1) && A2 > 0) {
        scan2[k+1] = i + k + 1;
        if ( A2 - 2*footprintsPerScan[i+k+1] > 0) {
          scan_idx2[2*(k+1)+0] = 0;
          scan_idx2[2*(k+1)+1] = 2*footprintsPerScan[i+k+1] - 1;
        } else {
          scan_idx2[2*(k+1)+0] = 0;
          scan_idx2[2*(k+1)+1] = A2-1;
        }
        A2 -= 2*footprintsPerScan[i+k+1];
        k++;
      } // endwhile
    } else {
      scan2[0] = i;
      scan_idx2[0] = prv_samp1;
      scan_idx2[1] = nxt_samp1;
    }
      
    // Build scan/scan_idx from scan1,scan2/scan_idx1,scan_idx2
    uint l = 0;
    for (size_t k=0; k<bufsize; k++) {
      if ( scan1[bufsize-1-k] == -1) {
        continue;
      } else {
        scan[l] = scan1[bufsize-1-k];
        scan_idx[2*l] = scan_idx1[2*(bufsize-1-k)];
        scan_idx[2*l+1] = scan_idx1[2*(bufsize-1-k)+1];
        l++;
      }
    }

    for (size_t k=0; k<bufsize; k++) {
      if ( scan2[k] != -1) {
        scan[l] = scan2[k];
        scan_idx[2*l] = scan_idx2[2*k];
        scan_idx[2*l+1] = scan_idx2[2*k+1];
        l++;
      } else {
          break;
      }
    }

    //   cout << "==================" << endl;
    //    cout << "scan: " << i << "  pix: " << j << endl;
    //    for (size_t k=0; k<l; k++) {
    // cout << scan[k] << " " << 
    //  scan_idx[2*k] << " " << scan_idx[2*k+1] << endl;
    //}
    

    uint n = 0;
    for (size_t k=0; k<l; k++) {
      int len = scan_idx[2*k+1]-scan_idx[2*k]+1;
      int nbytes = len * sizeof(double);
      int start = scan[k]*(nPRI_REF/4) + scan_idx[2*k];
      memcpy( &gain_v[n], &gainv[start], nbytes); 
      memcpy( &gain_h[n], &gainh[start], nbytes);
      memcpy( &gain_34[n], &gain[start], nbytes); 
      memcpy( &offset_v[n], &offsetv[start], nbytes); 
      memcpy( &offset_h[n], &offseth[start], nbytes); 
      memcpy( &t3offset[n], &T3offset[start], nbytes); 
      memcpy( &t4offset[n], &T4offset[start], nbytes); 
      n += len;
      if (n > nh) {
        cout << k << " " << n << endl;
        exit(1);
      }

    }

    int starth;
    if ( n < nh && A1 < 0) starth = nh - n; else starth = 0;

    double sumh = 0;
    for (size_t k=0; k<n; k++) sumh += h[starth+k];

    double GainvSum= 0;
    double GainhSum= 0;
    double GainSum= 0;
    double OffsetvSum= 0;
    double OffsethSum= 0;
    double T3OffsetSum= 0;
    double T4OffsetSum= 0;
    for (size_t k=0; k<n; k++) {
      GainvSum += gain_v[k]*h[starth+k];
      GainhSum += gain_h[k]*h[starth+k];
      GainSum += gain_34[k]*h[starth+k];
      OffsetvSum += offset_v[k]*h[starth+k];
      OffsethSum += offset_h[k]*h[starth+k];
      T3OffsetSum += t3offset[k]*h[starth+k];
      T4OffsetSum += t4offset[k]*h[starth+k];
    }

    if ( isnan( GainvSum)) {
      cout << "GainvSum is NaN: " << j << endl;
      exit(1);
    }

    if ( isnan( GainhSum)) {
      cout << "GainhSum is NaN: " << j << endl;
      exit(1);
    }

    if ( isnan( GainSum)) {
      cout << "GainSum is NaN: " << j << endl;
      exit(1);
    }
    
    for (size_t k=0; k<pkt_fp*4; k++) {
      Gainv[j*pkt_fp*4+k] = GainvSum / sumh;
      Gainh[j*pkt_fp*4+k] = GainhSum / sumh;
      Gain[j*pkt_fp*4+k] = GainSum / sumh;
      Offsetv[j*pkt_fp*4+k] = OffsetvSum / sumh;
      Offseth[j*pkt_fp*4+k] = OffsethSum / sumh;
      T3Offset[j*pkt_fp*4+k] = T3OffsetSum / sumh;
      T4Offset[j*pkt_fp*4+k] = T4OffsetSum / sumh;
    }


    Ncal += 2;
    
  } // j loop

  delete( gain_v);
  delete( gain_h);
  delete( gain_34);
  delete( offset_v);
  delete( offset_h);
  delete( t3offset);
  delete( t4offset);

  return 0;
}


int gainFilter16( hsize_t nscans, size_t i,
                  uint32_t *highresolution_scan_index,
                  uint16_t *footprintsPerScan,  
                  uint32_t nh, double *h, 
                  double time_gap, double *scan_diff1, double *scan_diff2, 
                  double *gainv16, double *gainh16, double *gain16, 
                  double *offsetv16, double *offseth16,
                  double *T3offset16, double *T4offset16, 
                  double *Gainv16, double *Gainh16, double *Gain16,
                  double *Offsetv16, double *Offseth16,
                  double *T3Offset16, double *T4Offset16) {

  int const bufsize = 100;

  int32_t scan1[bufsize];
  int32_t scan_idx1[bufsize*2];
  int32_t scan2[bufsize];
  int32_t scan_idx2[bufsize*2];
  int32_t scan[bufsize*2];
  int32_t scan_idx[bufsize*2*2];
  
  int Ncal = 0;

  double *gain_v = new double[nh*16];
  double *gain_h = new double[nh*16];
  double *gain_34 = new double[nh*16];
  double *offset_v = new double[nh*16];
  double *offset_h = new double[nh*16];
  double *t3offset = new double[nh*16];
  double *t4offset = new double[nh*16];

  uint16_t fpi = footprintsPerScan[highresolution_scan_index[i]];
    
  for (size_t j=0; j<fpi; j++) {

    for (size_t k=0; k<bufsize; k++) {
      scan1[k] = -1;
      scan_idx1[2*k+0] = -1;
      scan_idx1[2*k+1] = -1;
      scan2[k] = -1;
      scan_idx2[2*k+0] = -1;
      scan_idx2[2*k+1] = -1;
    }

    int prv_samp1 = Ncal - (nh/2);
    int nxt_samp1 = Ncal + (nh/2);
  // ant_pkt = footprints_per_scan(highresolution_scan_index)*8;
  //ref_pkt16 = footprints_per_scan(highresolution_scan_index)*2;
    
    int A1s, A2s;
    int k;

    if ( prv_samp1 < 0 && scan_diff1[i] < time_gap) {
      A1s = prv_samp1;
      k = 0;
      
      while ( i-k > 0 && A1s < 0 && scan_diff1[i-k] < time_gap) {
        scan1[k] = i-k-1;
        
        uint16_t fpm = footprintsPerScan[highresolution_scan_index[i-k-1]];

        if ( (A1s + 2*fpm) < 0) {
          scan_idx1[2*k+0] = 0;
          scan_idx1[2*k+1] = 2*fpm-1;
        } else {
          scan_idx1[2*k+0] = A1s + 2*fpm;
          scan_idx1[2*k+1] = 2*fpm-1;
        }
        A1s += 2*fpm;
        k++;
      } // endwhile
    } else {
      A1s = prv_samp1;
    }

    if ( prv_samp1 < 0) prv_samp1 = 0;

    if ( nxt_samp1 > 2*fpi-1 && scan_diff2[i] >= time_gap) {
      A2s = nxt_samp1 - 2*fpi+1;
      scan2[0] = i;
      scan_idx2[0] = prv_samp1;
      scan_idx2[1] = 2*fpi-1;
    } else if (nxt_samp1 > 2*fpi-1 && i < (nscans-1)) {
      A2s = nxt_samp1 - 2*fpi+1;
      k = 0;
 
      scan2[k] = i;
      scan_idx2[2*k+0] = prv_samp1;
      scan_idx2[2*k+1] = 2*fpi-1;

      while ( scan_diff2[i+k] < time_gap && A2s > 0) {
        scan2[k+1] = i+k+1;

        uint16_t fpp = footprintsPerScan[highresolution_scan_index[i+k+1]];

        if ( A2s - 2*fpp > 0) {
          scan_idx2[2*(k+1)+0] = 0;
          scan_idx2[2*(k+1)+1] = 2*fpp-1;
        } else {
          scan_idx2[2*(k+1)+0] = 0;
          scan_idx2[2*(k+1)+1] = A2s-1;
        }
        A2s -= 2*fpp;
        k++;
      } // endwhile
    } else {
      A2s = nxt_samp1 - 2*fpi+1;
      scan2[0] = i;
      scan_idx2[0] = prv_samp1;
      scan_idx2[1] = nxt_samp1;
    }
      
    // Build scan/scan_idx from scan1,scan2/scan_idx1,scan_idx2
    uint l = 0;
    for (size_t k=0; k<bufsize; k++) {
      if ( scan1[bufsize-1-k] == -1) {
        continue;
      } else {
        scan[l] = scan1[bufsize-1-k];
        scan_idx[2*l] = scan_idx1[2*(bufsize-1-k)];
        scan_idx[2*l+1] = scan_idx1[2*(bufsize-1-k)+1];
        l++;
      }
    }

    for (size_t k=0; k<bufsize; k++) {
      if ( scan2[k] != -1) {
        scan[l] = scan2[k];
        scan_idx[2*l] = scan_idx2[2*k];
        scan_idx[2*l+1] = scan_idx2[2*k+1];
        l++;
      } else {
          break;
      }
    }

    //   cout << "==================" << endl;
    //    cout << "scan: " << i << "  pix: " << j << endl;
    //    for (size_t k=0; k<l; k++) {
    // cout << scan[k] << " " << 
    //  scan_idx[2*k] << " " << scan_idx[2*k+1] << endl;
    //}
    

    uint n = 0;
    for (size_t k=0; k<l; k++) {
      int len = (scan_idx[2*k+1]-scan_idx[2*k]+1)*16;
      int nbytes = len * sizeof(double);
      int start = scan[k]*(nPRI_REF/4)*16 + scan_idx[2*k]*16;

      if (sizeof(double)*n + nbytes > sizeof(double)*nh*16) {
        cout << k << " " << n << " " << nbytes << endl;
        exit(1);
      }

      memcpy( &gain_v[n], &gainv16[start], nbytes); 
      memcpy( &gain_h[n], &gainh16[start], nbytes);
      memcpy( &gain_34[n], &gain16[start], nbytes); 
      memcpy( &offset_v[n], &offsetv16[start], nbytes); 
      memcpy( &offset_h[n], &offseth16[start], nbytes); 
      memcpy( &t3offset[n], &T3offset16[start], nbytes); 
      memcpy( &t4offset[n], &T4offset16[start], nbytes); 
      n += len;
      if (n > nh*16) {
        cout << k << " " << n << endl;
        exit(1);
      }
    }

    int starth;
    if ( (n/16) < nh && A1s < 0) starth = nh - (n/16); else starth = 0;

    double sumh = 0;
    for (size_t k=0; k<n/16; k++) sumh += h[starth+k];

    for (size_t l=0; l<16; l++) { 
      double GainvSum= 0;
      double GainhSum= 0;
      double GainSum= 0;
      double OffsetvSum= 0;
      double OffsethSum= 0;
      double T3OffsetSum= 0;
      double T4OffsetSum= 0;
      for (size_t k=0; k<n/16; k++) {
        GainvSum += gain_v[k*16+l]*h[starth+k];
        GainhSum += gain_h[k*16+l]*h[starth+k];
        GainSum += gain_34[k*16+l]*h[starth+k];
        OffsetvSum += offset_v[k*16+l]*h[starth+k];
        OffsethSum += offset_h[k*16+l]*h[starth+k];
        T3OffsetSum += t3offset[k*16+l]*h[starth+k];
        T4OffsetSum += t4offset[k*16+l]*h[starth+k];
      }

      for (size_t k=0; k<nANT_PKTs_fp; k++) {
        Gainv16[j*nANT_PKTs_fp*16+k*16+l] = GainvSum / sumh;
        Gainh16[j*nANT_PKTs_fp*16+k*16+l] = GainhSum / sumh;
        Gain16[j*nANT_PKTs_fp*16+k*16+l] = GainSum / sumh;
        Offsetv16[j*nANT_PKTs_fp*16+k*16+l] = OffsetvSum / sumh;
        Offseth16[j*nANT_PKTs_fp*16+k*16+l] = OffsethSum / sumh;
        T3Offset16[j*nANT_PKTs_fp*16+k*16+l] = T3OffsetSum / sumh;
        T4Offset16[j*nANT_PKTs_fp*16+k*16+l] = T4OffsetSum / sumh;
      }
    } // l loop

    Ncal += 2;
    
  } // j loop

  delete( gain_v);
  delete( gain_h);
  delete( gain_34);
  delete( offset_v);
  delete( offset_h);
  delete( t3offset);
  delete( t4offset);

  return 0;
}


int l1aSMAPFile::openl1( const char* l1a_filename, const char* ta_filename,
                         string history) {

  h5fid =  H5Fopen(l1a_filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if ( h5fid < 0) {
    cout << l1a_filename << " not found.";
    exit(1);
  }

    gid[ROOT] = H5Gopen1(h5fid,"/");
    if ( gid[ROOT] < 0) {
      cout << "Root group not found.";
      exit(1);
    }

    gid[HIRESMOM] = H5Gopen1(h5fid, "/HighResolution_Moments_Data");
    if ( gid[HIRESMOM] < 0) {
      cout << "'HighResolution_Moments_Data' group not found.";
      exit(1);
    }

    gid[HOUSEKEEP] = H5Gopen1(h5fid, "/House_Keeping_Data");
    if ( gid[HOUSEKEEP] < 0) {
      cout << "'House_Keeping_Data' group not found.";
      exit(1);
    }

    gid[METADATA] = H5Gopen1(h5fid, "/Metadata");
    if ( gid[METADATA] < 0) {
      cout << "'/Metadata' group not found.";
      exit(1);
    }

    gid[MOMDATA] = H5Gopen1(h5fid, "/Moments_Data");
    if ( gid[MOMDATA] < 0) {
      cout << "'/Moments_Data' group not found.";
      exit(1);
    }

    hsize_t dims[2];
    H5LTget_dataset_info( gid[MOMDATA], "ant_time_seconds", 
                          dims, NULL, NULL);  

    nPRI_ANT = dims[1];
    nPRI_REF = (nPRI_ANT/4)-last_PKTs;
    
    gid[SCDATA] = H5Gopen1(h5fid, "/Spacecraft_Data");
    if ( gid[SCDATA] < 0) {
      cout << "'/Spacecraft_Data' group not found.";
      exit(1);
    }

    H5LTget_dataset_info( gid[SCDATA], "x_pos", &nscans, NULL, NULL);
 

    char *string_attr[2];
    char zulu[17];
    hid_t attr, ftype, type;
    herr_t ret;

    hid_t gidOrbit = H5Gopen1(h5fid, "/Metadata/OrbitMeasuredLocation");

    attr = H5Aopen_name(gidOrbit, "halfOrbitStartDateTime");
    ftype = H5Aget_type(attr);
    type = H5Tget_native_type(ftype, H5T_DIR_ASCEND);
    ret = H5Aread(attr, type, &string_attr);
    halfOrbitStartDateTime.assign( string_attr[0]);

    attr = H5Aopen_name(gidOrbit, "halfOrbitStopDateTime");
    ret = H5Aread(attr, type, &string_attr);
    halfOrbitStopDateTime.assign( string_attr[0]);

    attr = H5Aopen_name(gidOrbit, "orbitDirection");
    ret = H5Aread(attr, type, &string_attr);
    orbitDirection.assign( string_attr[0]);

    float orbitPeriodFloat;
    attr = H5Aopen_name(gidOrbit, "orbitPeriod");
    ret = H5Aread(attr, H5T_NATIVE_FLOAT, &orbitPeriodFloat);
    orbitPeriod = floor( orbitPeriodFloat); 

    int32_t revNumber;
    attr = H5Aopen_name(gidOrbit, "revNumber");
    ret = H5Aread(attr, H5T_NATIVE_LONG, &revNumber);
    
    H5Aclose( attr);
    H5Tclose( type);
    H5Gclose( gidOrbit);

    // Geolocation FullBand
    float *lat = new float[nscans*nPRI_ANT];
    float *lon = new float[nscans*nPRI_ANT];
    double *sec = new double[nscans*nPRI_ANT];
    readFloat( this->gid[MOMDATA], "moments_lat", lat);
    readFloat( this->gid[MOMDATA], "moments_lon", lon);
    readDouble( this->gid[MOMDATA], "ant_time_seconds", sec);

    int OrbitD = orbitDirection.compare("Ascending");

    uint8_t *FB = new uint8_t[nscans*nPRI_ANT];
    float *scan_angle = new float[nscans*(nPRI_ANT/32)];
    memset( FB, 0, nscans*nPRI_ANT);
    lookFB( nscans, nPRI_ANT, sec, FB, scan_angle);
    
    ofstream tempOut;
    //    tempOut.open ("RFI_idx.bin", ios::out | ios::trunc | ios::binary);
    //tempOut.write((char *) FB, nscans*nPRI_ANT*sizeof(uint8_t));
    //tempOut.close();


    rfiThreshidx = new int32_t[nscans*nPRI_ANT];
    for (size_t i=0; i<nscans*nPRI_ANT; i++) rfiThreshidx[i] = -1;
    for (size_t i=0; i<nscans*nPRI_ANT; i++) {
      if ( lat[i] >= -90 && lat[i] <= +90) {
        rfiThreshidx[i] = 
          floor((lat[i]+90)*10) * 3600 * 4 + floor((lon[i]+180)*10) * 4;
        if (rfiThreshidx[i] >= 4*3600*1800) {
          cout << i << " " << rfiThreshidx[i] << endl;
          exit(1);
        }
        if (rfiThreshidx[i] < 0) {
          cout << i << " " << rfiThreshidx[i] << endl;
          exit(1);
        }
        if ( OrbitD == 0) rfiThreshidx[i] += 2;
        if ( FB[i] == 0) rfiThreshidx[i] += 1;
      } else if ( lat[i] < -9e20) {
        rfiThreshidx[i] = 0;
        if ( OrbitD == 0) rfiThreshidx[i] += 2;
        if ( FB[i] == 0) rfiThreshidx[i] += 1;
      }
    }

    rfiThreshidxFP = new int32_t[nscans*nPRI_ANT/32];
    for (size_t i=0; i<nscans; i++) { 
      for (size_t j=0; j<nPRI_ANT/32; j++) { 
        rfiThreshidxFP[i*(nPRI_ANT/32)+j] = rfiThreshidx[nPRI_ANT*i+32*j+15];
      }
    }

    // Create TA output file and copy S/C from L1A
    h5fid_ta = 
      H5Fcreate(ta_filename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
    gid_ta = H5Gcreate1(h5fid_ta, "/Spacecraft_Data", 0);

    size_t found = history.find_last_of(",");
    Hdf::SetScalarH5A( h5fid_ta, "history", H5T_STRING, 
                       (VOIDP) history.substr(0,found).c_str());
    
    float fltarr[MAXSCANS];
    double dblarr[MAXSCANS];
    dims[0] = nscans;
    dims[1] = nPRI_ANT/32;

    uint16_t footprintsPerScan[MAXSCANS];
    readU16( this->gid[SCDATA], "footprints_per_scan", 
             (uint16_t *) footprintsPerScan);

    readDouble( this->gid[SCDATA], "antenna_scan_time", dblarr);
    H5LTmake_dataset( gid_ta, "antenna_scan_time", 1, dims, 
                      H5T_NATIVE_DOUBLE, dblarr);

    H5LTmake_dataset( gid_ta, "footprints_per_scan", 1, dims, 
                      H5T_NATIVE_USHORT, footprintsPerScan);

    readFloat( this->gid[SCDATA], "roll", fltarr);
    H5LTmake_dataset( gid_ta, "roll", 1, dims, H5T_NATIVE_FLOAT, fltarr);
    readFloat( this->gid[SCDATA], "pitch", fltarr);
    H5LTmake_dataset( gid_ta, "pitch", 1, dims, H5T_NATIVE_FLOAT, fltarr);
    readFloat( this->gid[SCDATA], "yaw", fltarr);
    H5LTmake_dataset( gid_ta, "yaw", 1, dims, H5T_NATIVE_FLOAT, fltarr);

    readFloat( this->gid[SCDATA], "sc_alongtrack_velocity", fltarr);
    H5LTmake_dataset( gid_ta, "sc_alongtrack_velocity", 1, dims, 
                      H5T_NATIVE_FLOAT, fltarr);
    readFloat( this->gid[SCDATA], "sc_geodetic_alt_ellipsoid", fltarr);
    H5LTmake_dataset( gid_ta, "sc_geodetic_alt_ellipsoid", 1, dims, 
                      H5T_NATIVE_FLOAT, fltarr);
    readFloat( this->gid[SCDATA], "sc_nadir_angle", fltarr);
    H5LTmake_dataset( gid_ta, "sc_nadir_angle", 1, dims, 
                      H5T_NATIVE_FLOAT, fltarr);
    readFloat( this->gid[SCDATA], "sc_nadir_lat", fltarr);
    H5LTmake_dataset( gid_ta, "sc_nadir_lat", 1, dims, 
                      H5T_NATIVE_FLOAT, fltarr);
    readFloat( this->gid[SCDATA], "sc_nadir_lon", fltarr);
    H5LTmake_dataset( gid_ta, "sc_nadir_lon", 1, dims, 
                      H5T_NATIVE_FLOAT, fltarr);
    readFloat( this->gid[SCDATA], "sc_radial_velocity", fltarr);
    H5LTmake_dataset( gid_ta, "sc_radial_velocity", 1, dims, 
                      H5T_NATIVE_FLOAT, fltarr);

    readFloat( this->gid[SCDATA], "x_pos", fltarr);
    H5LTmake_dataset( gid_ta, "x_pos", 1, dims, H5T_NATIVE_FLOAT, fltarr);
    readFloat( this->gid[SCDATA], "x_vel", fltarr);
    H5LTmake_dataset( gid_ta, "x_vel", 1, dims, H5T_NATIVE_FLOAT, fltarr);
    readFloat( this->gid[SCDATA], "y_pos", fltarr);
    H5LTmake_dataset( gid_ta, "y_pos", 1, dims, H5T_NATIVE_FLOAT, fltarr);
    readFloat( this->gid[SCDATA], "y_vel", fltarr);
    H5LTmake_dataset( gid_ta, "y_vel", 1, dims, H5T_NATIVE_FLOAT, fltarr);
    readFloat( this->gid[SCDATA], "z_pos", fltarr);
    H5LTmake_dataset( gid_ta, "z_pos", 1, dims, H5T_NATIVE_FLOAT, fltarr);
    readFloat( this->gid[SCDATA], "z_vel", fltarr);
    H5LTmake_dataset( gid_ta, "z_vel", 1, dims, H5T_NATIVE_FLOAT, fltarr);

    H5Gclose( gid_ta);

    gid_ta = H5Gcreate1(h5fid_ta, "/Brightness_Temperature", 0);
    double *tb_time_seconds = new double[nscans*(nPRI_ANT/32)];
    float *tb_lat = new float[nscans*(nPRI_ANT/32)];
    float *tb_lon = new float[nscans*(nPRI_ANT/32)];
    for (size_t j=0; j<nscans*(nPRI_ANT/32); j++) {
      tb_time_seconds[j] = -9999.0;
      tb_lat[j] = -9999.0;
      tb_lon[j] = -9999.0;
    }

    for (size_t i=0; i<nscans; i++) {

      int k=0;
      for (size_t j=0; j<nPRI_ANT; j++) {
        uint32_t idx = i*nPRI_ANT+j;

        if ( k >= footprintsPerScan[i]) continue;

        if ( j % 32 == 15) {
          tb_time_seconds[i*(nPRI_ANT/32)+k] = sec[idx];
          tb_lat[i*(nPRI_ANT/32)+k] = lat[idx];
          tb_lon[i*(nPRI_ANT/32)+k] = lon[idx];
        }

        if ( j % 32 == 16) {
          tb_time_seconds[i*(nPRI_ANT/32)+k] += sec[idx];
          tb_time_seconds[i*(nPRI_ANT/32)+k] /= 2;

          tb_lat[i*(nPRI_ANT/32)+k] += lat[idx];
          tb_lat[i*(nPRI_ANT/32)+k] /= 2;

          tb_lon[i*(nPRI_ANT/32)+k] += lon[idx];
          tb_lon[i*(nPRI_ANT/32)+k] /= 2;

          if ( tb_time_seconds[i*(nPRI_ANT/32)+k] < -9999) 
            tb_time_seconds[i*(nPRI_ANT/32)+k] = -9999;

          k++;
        }
      } // j loop
    } // i loop

    H5LTmake_dataset( gid_ta, "antenna_scan_angle", 2, dims, 
                      H5T_NATIVE_FLOAT, scan_angle);

    readFloat( this->gid[SCDATA], "antenna_look_angle", fltarr);
    H5LTmake_dataset( gid_ta, "antenna_look_angle", 1, dims, 
                      H5T_NATIVE_FLOAT, fltarr);

    H5LTmake_dataset( gid_ta, "tb_lat", 2, dims, H5T_NATIVE_FLOAT, tb_lat);
    H5LTmake_dataset( gid_ta, "tb_lon", 2, dims, H5T_NATIVE_FLOAT, tb_lon);
    H5LTmake_dataset( gid_ta, "tb_time_seconds", 2, dims, 
                      H5T_NATIVE_DOUBLE, tb_time_seconds);

    H5Gclose( gid_ta);
    
    gid_ta = H5Gcreate1(h5fid_ta, "/Metadata", 0);
    int gid_ta1 = H5Gcreate1(gid_ta, "OrbitMeasuredLocation", 0);
    Hdf::SetScalarH5A( gid_ta1, "revNumber", H5T_NATIVE_LONG, &revNumber);
    H5Gclose( gid_ta1);

    H5Gclose( gid_ta);

    delete( tb_lat);
    delete( tb_lon);
    delete( tb_time_seconds);

    delete( lat);
    delete( lon);
    delete( sec);
    delete( FB);
    delete( scan_angle);

    // Geolocation SubBand
    H5LTget_dataset_info( gid[HIRESMOM], "highresolution_scan_index", &nhires,
                          NULL, NULL);
    
    float *lat16 = new float[nhires*(nPRI_ANT/4)];
    float *lon16 = new float[nhires*(nPRI_ANT/4)];
    double *sec16 = new double[nhires*(nPRI_ANT/4)];
    readFloat( this->gid[HIRESMOM], "moments16_lat", lat16);
    readFloat( this->gid[HIRESMOM], "moments16_lon", lon16);
    readDouble( this->gid[HIRESMOM], "ant_16_time_seconds", sec16);

    //  tempOut.open ("moments16_lat.bin", ios::out | ios::trunc | ios::binary);
    //tempOut.write((char *) lat16, nscans*nPRI_ANT*sizeof(float));
    //tempOut.close();

    uint8_t *FB16 = new uint8_t[nhires*(nPRI_ANT/4)];
    memset( FB16, 0, nhires*(nPRI_ANT/4));
    //  float *scan_angle = new float[nscans*(nPRI_ANT/4)];
    lookFB( nhires, nPRI_ANT/4, sec16, FB16, NULL);

    //  tempOut.open ("scan_angle.bin", ios::out | ios::trunc | ios::binary);
    //tempOut.write((char *) scan_angle, nscans*nPRI_ANT*sizeof(float));
    //tempOut.close();

    //    tempOut.open ("RFI_idx_sub.bin", ios::out | ios::trunc | ios::binary);
    //tempOut.write((char *) FB16, nscans*(nPRI_ANT/4)*sizeof(uint8_t));
    //tempOut.close();

    rfiThreshidx16 = new int32_t[nhires*(nPRI_ANT/4)];
    for (size_t i=0; i<nhires*(nPRI_ANT/4); i++) rfiThreshidx16[i] = -1;
    for (size_t i=0; i<nhires*(nPRI_ANT/4); i++) {
      if ( lat16[i] >= -90  && lat16[i] <= +90) {
        rfiThreshidx16[i] = 
          floor((lat16[i]+90)*10) * 3600 * 4 + floor((lon16[i]+180)*10) * 4;
        if (rfiThreshidx16[i] >= 4*3600*1800) {
          cout << i << " " << rfiThreshidx16[i] << endl;
          exit(1);
        }
        if ( OrbitD == 0) rfiThreshidx16[i] += 2;
        if ( FB16[i] == 0) rfiThreshidx16[i] += 1;
      } else if ( lat16[i] < -9e20) {
        rfiThreshidx16[i] = 0;
        if ( OrbitD == 0) rfiThreshidx16[i] += 2;
        if ( FB16[i] == 0) rfiThreshidx16[i] += 1;
      }
    }

    for (size_t i=0; i<nhires*(nPRI_ANT/4); i++) {
      if ( rfiThreshidx16[i] < -1) {
        cout << i << " " << rfiThreshidx16[i] << endl;
      }
    }
    
    delete( lat16);
    delete( lon16);
    delete( sec16);
    delete( FB16);

    return 0;
}

int l1aSMAPFile::lookFB( hsize_t n, uint32_t nPRI, double *sec, uint8_t *FB, 
                         float *scan_angle) {

  // time at which scan angle is 0
  double scan_time[MAXSCANS];
  readDouble( this->gid[SCDATA], "antenna_scan_time", scan_time);

  // rotation rate in revolutions/min
  float rot_rate[MAXSCANS];
  readFloat( this->gid[SCDATA], "antenna_rotation_rate", rot_rate);

  for (size_t i=0; i<n; i++) {

    int k=0;
    for (size_t j=0; j<nPRI; j++) {
      uint32_t idx = i*nPRI+j;

      if (sec[idx] < 0) continue;

      // Factor of 6 to convert from rev/min to deg/sec
      float sa = (sec[idx] - scan_time[i]) * rot_rate[i] * 6;
      if ( sa < 0) sa += 360;

      FB[idx] = 0;
      if ( sa > 90 && sa < 270) FB[idx] = 1;

      if ( scan_angle != NULL) {
        if ( j % 32 == 15) scan_angle[i*(nPRI/32)+k] = sa;
        if ( j % 32 == 16) {
          scan_angle[i*(nPRI/32)+k] += sa;
          scan_angle[i*(nPRI/32)+k] /= 2;
          k++;
        }
      }

    } // j loop
  } // i loop

  return 0;
}

int l1aSMAPFile::allocateRFIFlagsKurtosis( uint32_t nPRI, 
                                           rfiFlagsKurtosis_struct
                                           *RFIflagsKurtosis) {

  uint32_t szkurt = nscans*nPRI;
  uint32_t szkurt16 = nhires*(nPRI/4)*16;
  
  RFIflagsKurtosis->RFIflag_kurt_h = new uint8_t[szkurt];
  RFIflagsKurtosis->RFIflag_kurt_v = new uint8_t[szkurt];
  RFIflagsKurtosis->RFIflag16_kurt_h = new uint8_t[szkurt16];
  RFIflagsKurtosis->RFIflag16_kurt_v = new uint8_t[szkurt16];
  RFIflagsKurtosis->kurtosis_h = new double[szkurt];
  RFIflagsKurtosis->kurtosis_v = new double[szkurt];
  RFIflagsKurtosis->kurtosis16_h = new double[szkurt16];
  RFIflagsKurtosis->kurtosis16_v = new double[szkurt16];

  memset(RFIflagsKurtosis->RFIflag_kurt_h, 0, szkurt);
  memset(RFIflagsKurtosis->RFIflag_kurt_v, 0, szkurt);
  memset(RFIflagsKurtosis->RFIflag16_kurt_h, 0, szkurt16);
  memset(RFIflagsKurtosis->RFIflag16_kurt_v, 0, szkurt16);

  return 0;
}


int l1aSMAPFile::calCoefficient( fullBandCoeffs_struct *fullBandCoeff,
                                 subBandCoeffs_struct *subBandCoeff,
                                 fullCalGroup_struct *fullCalGroup,
                                 subCalGroup_struct *subCalGroup,
                                 uint32_t *subband_scan_counter) {

  float *analog_eu = new float[MAXSCANS*160];
  readFloat( gid[HOUSEKEEP], "analog_eu", analog_eu);

  for (size_t i=0; i<nscans; i++) {
    fullCalGroup->cal_temp5_diplexer[i][0] = analog_eu[i*160+50] + CELOFF;
    fullCalGroup->cal_temp5_diplexer[i][1] = analog_eu[i*160+47] + CELOFF;
    fullCalGroup->cal_temp4_coupler[i][0] = analog_eu[i*160+56] + CELOFF;
    fullCalGroup->cal_temp4_coupler[i][1] = analog_eu[i*160+53] + CELOFF;
    fullCalGroup->cal_temp3_omt[i][0] = analog_eu[i*160+45] + CELOFF;
    fullCalGroup->cal_temp3_omt[i][1] = analog_eu[i*160+44] + CELOFF;
    fullCalGroup->cal_temp2_feed[i] = analog_eu[i*160+42] + CELOFF;

    fullCalGroup->cal_temp_ref[i][0] = analog_eu[i*160+2] + CELOFF;
    fullCalGroup->cal_temp_ref[i][1] = analog_eu[i*160+1] + CELOFF; 
    fullCalGroup->cal_temp_nd[i] = analog_eu[i*160+5] + CELOFF;
    fullCalGroup->cal_temp_xnd[i] = analog_eu[i*160+0] + CELOFF;
    fullCalGroup->cal_temp_det[i][0] = analog_eu[i*160+60] + CELOFF;
    fullCalGroup->cal_temp_det[i][1] = analog_eu[i*160+59] + CELOFF;

    for (size_t j=0; j<2; j++) {
      fullCalGroup->cal_loss5_diplexer[i][j] = fullBandCoeff->L5[j] + 
        fullBandCoeff->C5[j] *
        (fullCalGroup->cal_temp5_diplexer[i][j] - fullBandCoeff->T5[j]);
      fullCalGroup->cal_loss4_coupler[i][j] = fullBandCoeff->L4[j] + 
        fullBandCoeff->C4[j] * 
        (fullCalGroup->cal_temp4_coupler[i][j] - fullBandCoeff->T4[j]);

      fullCalGroup->cal_loss3_omt[i][j] = fullBandCoeff->L3[j] + 
        fullBandCoeff->C3[j] * 
        (fullCalGroup->cal_temp3_omt[i][j]- fullBandCoeff->T3[j]);

      fullCalGroup->cal_tempref_offset[i][j] = fullBandCoeff->TDLoffset[j] + 
        fullBandCoeff->c1_DLoffset[j] * 
        (fullCalGroup->cal_temp_ref[i][j] - fullBandCoeff->TDL0[j]) +
        fullBandCoeff->c2_DLoffset[j] * 
        (fullCalGroup->cal_temp3_omt[i][j] - fullBandCoeff->Tomt0[j]) + 
        fullBandCoeff->c3_DLoffset[j] * 
        (fullCalGroup->cal_temp4_coupler[i][j] - fullBandCoeff->Tcoupler0[j]) +
        fullBandCoeff->c4_DLoffset[j] *
        (fullCalGroup->cal_temp5_diplexer[i][j]-fullBandCoeff->Tdiplexer0[j]);

      fullCalGroup->cal_tref[i][j] = 
        fullCalGroup->cal_temp_ref[i][j] + 
        fullCalGroup->cal_tempref_offset[i][j];

      fullCalGroup->cal_tnd[i][j] = 
        fullBandCoeff->Tnd[j] + fullBandCoeff->c1_nd[j] *
        (fullCalGroup->cal_temp_nd[i] - fullBandCoeff->Tnd0[j]) + 
        fullBandCoeff->c2_nd[j] * 
        (fullCalGroup->cal_temp3_omt[i][j] - fullBandCoeff->Tomt0[j]) + 
        fullBandCoeff->c3_nd[j] *
        (fullCalGroup->cal_temp4_coupler[i][j] - fullBandCoeff->Tcoupler0[j]) + 
        fullBandCoeff->c4_nd[j] *
        (fullCalGroup->cal_temp5_diplexer[i][j]-fullBandCoeff->Tdiplexer0[j]);

      fullCalGroup->cal_txnd[i][j] = fullBandCoeff->Txnd[j] + 
        fullBandCoeff->c1_xnd[j] * 
        (fullCalGroup->cal_temp_xnd[i] - fullBandCoeff->Txnd0[j]) + 
        fullBandCoeff->c2_xnd[j] *
        (fullCalGroup->cal_temp3_omt[i][j]-fullBandCoeff->Tomt0[j]) + 
        fullBandCoeff->c3_xnd[j] * 
        (fullCalGroup->cal_temp4_coupler[i][j] - fullBandCoeff->Tcoupler0[j]) + 
        fullBandCoeff->c4_xnd[j] * 
        (fullCalGroup->cal_temp5_diplexer[i][j] - fullBandCoeff->Tdiplexer0[j]);

      fullCalGroup->cal_lambda[i][j] = fullBandCoeff->lambda[j];
      fullCalGroup->cal_gamma[i][j] = fullBandCoeff->dgamma[j];
      fullCalGroup->cal_tcor_re[i][j] = fullBandCoeff->tcor_re[j];
      fullCalGroup->cal_tcor_im[i][j] = fullBandCoeff->tcor_im[j];
    } // j loop

    fullCalGroup->cal_loss2_feed[i] = 
      fullBandCoeff->L2 + 
      fullBandCoeff->C2 * 
      (fullCalGroup->cal_temp2_feed[i]-fullBandCoeff->T2);

    fullCalGroup->cal_tnd34[i] = 
      2 * sqrt(fullBandCoeff->Tnd[0]*fullBandCoeff->Tnd[1]);

    fullCalGroup->cal_nd_phase[i] = fullBandCoeff->nd_phase;
    fullCalGroup->cal_xnd_phase[i] = fullBandCoeff->xnd_phase;
    fullCalGroup->cal_rx_phase[i] = fullBandCoeff->rx_phase;
    fullCalGroup->cal_phase[i] = fullBandCoeff->channel_phase;
    fullCalGroup->cal_psi[i] = fullBandCoeff->dpsi;

    bool found = false;
    for (size_t k=0; k<MAXSCANS; k++) {
      if (subband_scan_counter[k] == i) {
        found = true;
        break;
      }
    }

    if ( found) {
      for (size_t k=0; k<16; k++) {
        subCalGroup->cal_temp5_diplexer16[i][k][0] = 
          analog_eu[i*160+50] + CELOFF;
        subCalGroup->cal_temp5_diplexer16[i][k][1] = 
          analog_eu[i*160+47] + CELOFF;
        subCalGroup->cal_temp4_coupler16[i][k][0] = 
          analog_eu[i*160+56] + CELOFF;
        subCalGroup->cal_temp4_coupler16[i][k][1] = 
          analog_eu[i*160+53] + CELOFF;
        subCalGroup->cal_temp3_omt16[i][k][0] = analog_eu[i*160+45] + CELOFF;
        subCalGroup->cal_temp3_omt16[i][k][1] = analog_eu[i*160+44] + CELOFF;
        subCalGroup->cal_temp2_feed16[i][k] = analog_eu[i*160+42] + CELOFF;

        subCalGroup->cal_temp_ref16[i][k][0] = analog_eu[i*160+2] + CELOFF;
        subCalGroup->cal_temp_ref16[i][k][1] = analog_eu[i*160+1] + CELOFF; 
        subCalGroup->cal_temp_nd16[i][k][0] = analog_eu[i*160+5] + CELOFF;
        subCalGroup->cal_temp_nd16[i][k][1] = analog_eu[i*160+5] + CELOFF;
        subCalGroup->cal_temp_xnd16[i][k][0] = analog_eu[i*160+0] + CELOFF;
        subCalGroup->cal_temp_xnd16[i][k][1] = analog_eu[i*160+0] + CELOFF;
        subCalGroup->cal_temp_det16[i][k][0] = analog_eu[i*160+60] + CELOFF;
        subCalGroup->cal_temp_det16[i][k][1] = analog_eu[i*160+59] + CELOFF;

        for (size_t j=0; j<2; j++) {
          subCalGroup->cal_loss5_diplexer16[i][k][j] = 
            subBandCoeff->L5_16[k][j] + subBandCoeff->C5_16[k][j] * 
            (subCalGroup->cal_temp5_diplexer16[i][k][j] - 
             subBandCoeff->T5_16[k][j]);

          subCalGroup->cal_loss4_coupler16[i][k][j] = 
            subBandCoeff->L4_16[k][j] + subBandCoeff->C4_16[k][j] *
            (subCalGroup->cal_temp4_coupler16[i][k][j] - 
             subBandCoeff->T4_16[k][j]);

          subCalGroup->cal_loss3_omt16[i][k][j] = subBandCoeff->L3_16[k][j] + 
            subBandCoeff->C3_16[k][j] * 
            (subCalGroup->cal_temp3_omt16[i][k][j] - subBandCoeff->T3_16[k][j]);
        } // j loop

        subCalGroup->cal_loss2_feed16[i][k] = subBandCoeff->L2_16[k] + 
          subBandCoeff->C2_16[k] * 
          (subCalGroup->cal_temp2_feed16[i][k] - subBandCoeff->T2_16[k]);     

        for (size_t j=0; j<2; j++) {
          subCalGroup->cal_tempref_offset16[i][k][j] = 
            subBandCoeff->TDLoffset16[k][j] + 
            subBandCoeff->c1_DLoffset16[k][j] *
            (subCalGroup->cal_temp_ref16[i][k][j] - 
             subBandCoeff->TDL0_16[k][j]) + 
            subBandCoeff->c2_DLoffset16[k][j] *
            (subCalGroup->cal_temp3_omt16[i][k][j] - 
             subBandCoeff->Tomt0_16[k][j]) + 
            subBandCoeff->c3_DLoffset16[k][j] *
            (subCalGroup->cal_temp4_coupler16[i][k][j] -
             subBandCoeff->Tcoupler0_16[k][j]) + 
            subBandCoeff->c4_DLoffset16[k][j] *
            (subCalGroup->cal_temp5_diplexer16[i][k][j] - 
             subBandCoeff->Tdiplexer0_16[k][j]);

          subCalGroup->cal_tref16[i][k][j] = 
            subCalGroup->cal_temp_ref16[i][k][j] + 
            subCalGroup->cal_tempref_offset16[i][k][j];

          subCalGroup->cal_tnd16[i][k][j] = subBandCoeff->Tnd16[k][j] + 
            subBandCoeff->c1_nd16[k][j] *
            (subCalGroup->cal_temp_nd16[i][k][j] - 
             subBandCoeff->Tnd0_16[k][j]) + 
            subBandCoeff->c2_nd16[k][j] *
            (subCalGroup->cal_temp3_omt16[i][k][j] -
             subBandCoeff->Tomt0_16[k][j]) + 
            subBandCoeff->c3_nd16[k][j] *
            (subCalGroup->cal_temp4_coupler16[i][k][j] -
             subBandCoeff->Tcoupler0_16[k][j]) + 
            subBandCoeff->c4_nd16[k][j] *
            (subCalGroup->cal_temp5_diplexer16[i][k][j] -
             subBandCoeff->Tdiplexer0_16[k][j]);

          subCalGroup->cal_txnd16[i][k][j] = subBandCoeff->Txnd16[k][j] + 
            subBandCoeff->c1_xnd16[k][j] *
            (subCalGroup->cal_temp_xnd16[i][k][j] -
             subBandCoeff->Txnd0_16[k][j]) + 
            subBandCoeff->c2_xnd16[k][j] *
            (subCalGroup->cal_temp3_omt16[i][k][j] -
             subBandCoeff->Tomt0_16[k][j]) + 
            subBandCoeff->c3_xnd16[k][j] *
            (subCalGroup->cal_temp4_coupler16[i][k][j] -
             subBandCoeff->Tcoupler0_16[k][j]) + 
            subBandCoeff->c4_xnd16[k][j] *
            (subCalGroup->cal_temp5_diplexer16[i][k][j] -
             subBandCoeff->Tdiplexer0_16[k][j]);
        }
        subCalGroup->cal_tnd34_16[i][k] = 
          2 * sqrt(subBandCoeff->Tnd16[k][0] * subBandCoeff->Tnd16[k][1]);

        subCalGroup->cal_nd_phase16[i][k] = subBandCoeff->nd_phase16[k];
        subCalGroup->cal_xnd_phase16[i][k] = subBandCoeff->xnd_phase16[k];
        subCalGroup->cal_rx_phase16[i][k] = subBandCoeff->rx_phase16[k];
        subCalGroup->cal_phase16[i][k] = subBandCoeff->channel_phase16[k];
        subCalGroup->cal_psi16[i][k] = subBandCoeff->psi16[k];
        subCalGroup->cal_lambda16[i][k][0] = subBandCoeff->lambda16[k][0];
        subCalGroup->cal_lambda16[i][k][1] = subBandCoeff->lambda16[k][1];
        subCalGroup->cal_gamma16[i][k][0] = subBandCoeff->dgamma16[k][0];
        subCalGroup->cal_gamma16[i][k][1] = subBandCoeff->dgamma16[k][1];
        subCalGroup->cal_tcor16_re[i][k][0] = subBandCoeff->tcor16_re[k][0];
        subCalGroup->cal_tcor16_re[i][k][1] = subBandCoeff->tcor16_re[k][1];
        subCalGroup->cal_tcor16_im[i][k][0] = subBandCoeff->tcor16_im[k][0];
        subCalGroup->cal_tcor16_im[i][k][1] = subBandCoeff->tcor16_im[k][1];
      } // (k) 16 loop
    } // if highrate data

  } // scan loop

  delete analog_eu;

  return 0;
}


int l1aSMAPFile::kurtosisCalData( const char *refType,
                                  double lat1, double lon1,
                                  uint32_t *highresolution_scan_index,
                                  rfiThresh_k_struct *rfiKurtosisParameters,
                                  rfiFlagsKurtosis_struct *rfiFlagsKurt) {

  /*
The first raw moment in each packet of the fullband radiometer data in the 
antenna state, which have been integrated to ~300 μs or one pulse repetition 
interval (PRI). 

The first raw moment in each packet of subband radiometer data in the antenna 
state, which have been integrated to 4 pulse repetition intervals (PRIs) or 
~1.2 ms.
   */

  float *m1_ref = new float[this->nscans*nPRI_REF*4];
  float *m2_ref = new float[this->nscans*nPRI_REF*4];
  float *m3_ref = new float[this->nscans*nPRI_REF*4];
  float *m4_ref = new float[this->nscans*nPRI_REF*4];

  float *m1_16_ref = new float[this->nhires*(nPRI_REF/4)*16*4];
  float *m2_16_ref = new float[this->nhires*(nPRI_REF/4)*16*4];
  float *m3_16_ref = new float[this->nhires*(nPRI_REF/4)*16*4];
  float *m4_16_ref = new float[this->nhires*(nPRI_REF/4)*16*4];

  if ( strcmp( refType, "Ref") == 0) {
    readFloat( gid[MOMDATA], "m1_ref", m1_ref);
    readFloat( gid[MOMDATA], "m2_ref", m2_ref);
    readFloat( gid[MOMDATA], "m3_ref", m3_ref);
    readFloat( gid[MOMDATA], "m4_ref", m4_ref);

    readFloat( gid[HIRESMOM], "m1_16_ref", m1_16_ref);
    readFloat( gid[HIRESMOM], "m2_16_ref", m2_16_ref);
    readFloat( gid[HIRESMOM], "m3_16_ref", m3_16_ref);
    readFloat( gid[HIRESMOM], "m4_16_ref", m4_16_ref);
  }

  if ( strcmp( refType, "RefND") == 0) {
    readFloat( gid[MOMDATA], "m1_ref_nd", m1_ref);
    readFloat( gid[MOMDATA], "m2_ref_nd", m2_ref);
    readFloat( gid[MOMDATA], "m3_ref_nd", m3_ref);
    readFloat( gid[MOMDATA], "m4_ref_nd", m4_ref);

    readFloat( gid[HIRESMOM], "m1_16_ref_nd", m1_16_ref);
    readFloat( gid[HIRESMOM], "m2_16_ref_nd", m2_16_ref);
    readFloat( gid[HIRESMOM], "m3_16_ref_nd", m3_16_ref);
    readFloat( gid[HIRESMOM], "m4_16_ref_nd", m4_16_ref);
  }

  kurtosis_RFI_Detection( "REF", &lat1, &lon1,
                          highresolution_scan_index,
                          m1_ref, m2_ref, m3_ref, m4_ref,
                          m1_16_ref, m2_16_ref, m3_16_ref, m4_16_ref,
                          rfiKurtosisParameters, rfiFlagsKurt);

  delete( m1_ref); 
  delete( m2_ref); 
  delete( m3_ref); 
  delete( m4_ref); 
  delete( m1_16_ref);
  delete( m2_16_ref);
  delete( m3_16_ref);
  delete( m4_16_ref);

  return 0;
}

int l1aSMAPFile::kurtosis_RFI_Detection( const char *ref_ant,
                                         double *lat, double *lon,
                                         uint32_t *highresolution_scan_index,
                                         float *m1, float *m2, 
                                         float *m3, float *m4,
                                         float *m1_16, float *m2_16, 
                                         float *m3_16, float *m4_16,
                                         rfiThresh_k_struct 
                                         *rfiKurtosisParameters,
                                         rfiFlagsKurtosis_struct 
                                         *rfiFlagsKurt) {

  uint16_t *digital_dn = new uint16_t[this->nscans*51];
  readU16( this->gid[HOUSEKEEP], "digital_dn", digital_dn);

  int lost = 61;
  double Tau[MAXSCANS];
  for (size_t i=0; i<nscans; i++) 
    Tau[i] = ((((double) digital_dn[i*51+6]) / 640) + 280 - lost) * 1e-6; 
  for (size_t i=1; i<nscans; i++) Tau[nscans-i] = Tau[nscans-i-1];

  double tau0 = 293.4e-6;
  delete( digital_dn);

  double kstd_h[MAXSCANS], kstd_v[MAXSCANS];
  for (size_t i=0; i<nscans; i++) { 
    kstd_h[i] = rfiKurtosisParameters->kstd_h * sqrt(tau0/Tau[i]);
    kstd_v[i] = rfiKurtosisParameters->kstd_v * sqrt(tau0/Tau[i]);
  }

  int ipol;
  uint32_t idx;
  double kurt_h, kurt_v;

  uint32_t nPRI;
  if ( strcmp( ref_ant, "REF") == 0) nPRI = nPRI_REF;
  if ( strcmp( ref_ant, "ANT") == 0) nPRI = nPRI_ANT;

  // FullBand
  for (size_t i=0; i<nscans; i++) {

    double kurt_num[4], kurt_den[4]; 
    for (size_t j=0; j<nPRI; j++) {

      for (size_t ipol=0; ipol<4; ipol++) {
        idx = i*nPRI*4 + j*4 + ipol;

        kurt_num[ipol] = m4[idx] - 4*m1[idx] * m3[idx] + 
          6*pow(m1[idx],2) * m2[idx] - 3*pow(m1[idx],4);

        kurt_den[ipol] = pow(m2[idx],2) - 2*m2[idx] * pow(m1[idx],2) + 
          pow(m1[idx],4);
      } // k loop

      kurt_h = 0.5*((kurt_num[0]/kurt_den[0]) + (kurt_num[1]/kurt_den[1]));
      kurt_v = 0.5*((kurt_num[2]/kurt_den[2]) + (kurt_num[3]/kurt_den[3]));

      if ( isinf(kurt_h)) kurt_h = -9999.0;
      if ( isinf(kurt_v)) kurt_v = -9999.0;

      rfiFlagsKurt->kurtosis_h[i*nPRI+j] = kurt_h;
      if ( fabs(kurt_h - rfiKurtosisParameters->knom_h) > 
           rfiKurtosisParameters->RFIthresh[0] * kstd_h[i])
        rfiFlagsKurt->RFIflag_kurt_h[i*nPRI+j] = 1;

      rfiFlagsKurt->kurtosis_v[i*nPRI+j] = kurt_v;
      if ( fabs(kurt_v - rfiKurtosisParameters->knom_v) > 
           rfiKurtosisParameters->RFIthresh[0] * kstd_v[i])
        rfiFlagsKurt->RFIflag_kurt_v[i*nPRI+j] = 1;

      if ( kurt_h == -9999.0) rfiFlagsKurt->RFIflag_kurt_h[i*nPRI+j] = 1;
      if ( kurt_v == -9999.0) rfiFlagsKurt->RFIflag_kurt_v[i*nPRI+j] = 1;

    } // j loop
  } // i loop


  double kurt16_h_num, kurt16_h_den, kurt16_h_real, kurt16_h_imag; 
  double kurt16_v_num, kurt16_v_den, kurt16_v_real, kurt16_v_imag;
  double kurt16_h, kurt16_v;

  uint8_t RFIflag16_kurt_h[16];
  uint8_t RFIflag16_kurt_v[16];

  double kstd16_h[MAXSCANS][16], kstd16_v[MAXSCANS][16];
  for (size_t i=0; i<nhires; i++) {
    double Tau16 = Tau[highresolution_scan_index[i]];
    for (size_t k=0; k<16; k++) {
      kstd16_h[i][k] = rfiKurtosisParameters->kstd16_h[k] * sqrt(tau0/Tau16);
      kstd16_v[i][k] = rfiKurtosisParameters->kstd16_v[k] * sqrt(tau0/Tau16);
    }
  }

  // SubBand
  for (size_t i=0; i<nhires; i++) {
    for (size_t j=0; j<nPRI/4; j++) {
      memset( RFIflag16_kurt_h, 0, 16);
      memset( RFIflag16_kurt_v, 0, 16);
      for (size_t k=0; k<16; k++) {

        // H real 16
        ipol = 0;
        idx = i*(nPRI/4)*16*4 + j*16*4 + k*4 + ipol;
        
        kurt16_h_num = m4_16[idx] - 4*m1_16[idx] * m3_16[idx] + 
          6*pow(m1_16[idx],2) * m2_16[idx] - 3*pow(m1_16[idx],4);

        kurt16_h_den = pow(m2_16[idx],2) - 2*m2_16[idx] * pow(m1_16[idx],2) + 
          pow(m1_16[idx],4);

        kurt16_h_real = kurt16_h_num / kurt16_h_den;

        // H imag 16
        ipol = 1;
        idx = i*(nPRI/4)*16*4 + j*16*4 + k*4 + ipol;

        kurt16_h_num = m4_16[idx] - 4*m1_16[idx]*m3_16[idx] + 
          6*pow(m1_16[idx],2) * m2_16[idx] - 3*pow(m1_16[idx],4);

        kurt16_h_den = pow(m2_16[idx],2) - 2*m2_16[idx] * pow(m1_16[idx],2) + 
          pow(m1_16[idx],4);

        kurt16_h_imag = kurt16_h_num / kurt16_h_den;
        kurt16_h = 0.5*(kurt16_h_real + kurt16_h_imag);
        if ( isinf(kurt16_h)) kurt16_h = -9999.0;

        rfiFlagsKurt->kurtosis16_h[i*(nPRI/4)*16 + j*16 + k] = kurt16_h;

        if ( fabs(kurt16_h - rfiKurtosisParameters->knom16_h[k]) > 
             rfiKurtosisParameters->RFIthresh[0] * kstd16_h[i][k])
          RFIflag16_kurt_h[k] = 1;
        if ( kurt16_h == -9999.0) RFIflag16_kurt_h[k] = 1;

        // V real 16
        ipol = 2;
        idx = i*(nPRI/4)*16*4 + j*16*4 + k*4 + ipol;
        
        kurt16_v_num = m4_16[idx] - 4*m1_16[idx] * m3_16[idx] + 
          6*pow(m1_16[idx],2) * m2_16[idx] - 3*pow(m1_16[idx],4);

        kurt16_v_den = pow(m2_16[idx],2) - 2*m2_16[idx] * pow(m1_16[idx],2) + 
          pow(m1_16[idx],4);

        kurt16_v_real = kurt16_v_num / kurt16_v_den;

        // V imag 16
        ipol = 3;
        idx = i*(nPRI/4)*16*4 + j*16*4 + k*4 + ipol;

        kurt16_v_num = m4_16[idx] - 4*m1_16[idx]*m3_16[idx] + 
          6*pow(m1_16[idx],2) * m2_16[idx] - 3*pow(m1_16[idx],4);

        kurt16_v_den = pow(m2_16[idx],2) - 2*m2_16[idx] * pow(m1_16[idx],2) + 
          pow(m1_16[idx],4);

        kurt16_v_imag = kurt16_v_num / kurt16_v_den;
        kurt16_v = 0.5*(kurt16_v_real + kurt16_v_imag);
        if ( isinf(kurt16_v)) kurt16_v = -9999.0;

        rfiFlagsKurt->kurtosis16_v[i*(nPRI/4)*16 + j*16 + k] = kurt16_v;

          if ( fabs(kurt16_v - rfiKurtosisParameters->knom16_v[k]) > 
               rfiKurtosisParameters->RFIthresh[0] * kstd16_v[i][k])
            RFIflag16_kurt_v[k] = 1;
          if ( kurt16_v == -9999.0) RFIflag16_kurt_v[k] = 1;

      } // k loop

      uint8_t RFIflag16_kurt_temp[16];

      // H-pol 16
      RFIflag16_kurt_temp[0] = 
        RFIflag16_kurt_h[0] | RFIflag16_kurt_h[1] | RFIflag16_kurt_h[15];
      for (size_t k=1; k<15; k++) {
        RFIflag16_kurt_temp[k] = 
          RFIflag16_kurt_h[k-1] | RFIflag16_kurt_h[k] | RFIflag16_kurt_h[k+1];
      } // k loop
      RFIflag16_kurt_temp[15] = 
        RFIflag16_kurt_h[15] | RFIflag16_kurt_h[14] | RFIflag16_kurt_h[0];

      memcpy( &rfiFlagsKurt->RFIflag16_kurt_h[i*(nPRI/4)*16 + j*16],
              RFIflag16_kurt_temp, 16);

      // V-pol 16
      RFIflag16_kurt_temp[0] = 
        RFIflag16_kurt_v[0] | RFIflag16_kurt_v[1] | RFIflag16_kurt_v[15];
      for (size_t k=1; k<15; k++) {
        RFIflag16_kurt_temp[k] = 
          RFIflag16_kurt_v[k-1] | RFIflag16_kurt_v[k] | RFIflag16_kurt_v[k+1];
      } // k loop
      RFIflag16_kurt_temp[15] = 
        RFIflag16_kurt_v[15] | RFIflag16_kurt_v[14] | RFIflag16_kurt_v[0];

      memcpy( &rfiFlagsKurt->RFIflag16_kurt_v[i*(nPRI/4)*16 + j*16],
              RFIflag16_kurt_temp, 16);

    } // j loop
  } // i loop

  return 0;
}

int l1aSMAPFile::rfiDetectCalData( const char *rfiParameters_filename,
                                   uint32_t *highresolution_scan_index,
                                   uint16_t *footprintsPerScan,
                                   double lat1, double lon1,
                                   uint32_t nPRI_fp,
                                   rfiFlagsCalData_struct *RFIflagsCalData) {

  hid_t file_id, gid;
  herr_t status;

  file_id = H5Fopen( rfiParameters_filename, H5F_ACC_RDONLY, H5P_DEFAULT);

  gid = H5Gopen1( file_id, "RFIthresh_cal");

  double *RFIthresh_td = new double[180*360];
  double *RFIthresh_cf = new double[180*360];
  readDouble( gid, "RFIthresh_td", RFIthresh_td);
  readDouble( gid, "RFIthresh_cf", RFIthresh_cf);

  float *m2_ref = new float[this->nscans*nPRI_REF*4];
  float *m2_16_ref = new float[this->nhires*(nPRI_REF/4)*16*4];
  readFloat( this->gid[MOMDATA], "m2_ref", m2_ref);
  readFloat( this->gid[HIRESMOM], "m2_16_ref", m2_16_ref);

  float *m2_ref_nd = new float[this->nscans*nPRI_REF*4];
  float *m2_16_ref_nd = new float[this->nhires*(nPRI_REF/4)*16*4];
  readFloat( this->gid[MOMDATA], "m2_ref_nd", m2_ref_nd);
  readFloat( this->gid[HIRESMOM], "m2_16_ref_nd", m2_16_ref_nd);

  uint32_t ref_PRIs = 8; //Number of reference PRIs per footprint
  uint32_t refnd_PRIs = 8; //Number of reference+noise diode PRIs per footprint

  uint32_t ref_PKTs = 2; //Number of reference packets per footprint
  uint32_t refnd_PKTs = 2; //Number of reference+noise diode pkts per footprint

  float ref_counts[nPRI_REF*4];
  float refnd_counts[nPRI_REF*4];
  float TAtd[nPRI_REF];

  //  remove ( "meanstd.bin");
  //remove( "meanstd16.bin");

  // Full Band
  float M = 0.9;
  uint32_t int_time = 1;
  int n = 0;
  for (size_t i=0; i<nscans; i++) {
    uint16_t fp = footprintsPerScan[i];

    // Ref
    memcpy( ref_counts, &m2_ref[i*nPRI_REF*4], fp*ref_PRIs*4*sizeof(float));  
    for (size_t ipol=0; ipol<4; ipol++) {
      for (size_t j=0; j<fp*ref_PRIs; j++) 
        TAtd[j] = ref_counts[j*4+ipol];

      int idx = i*nPRI_REF*4 + ipol*nPRI_REF;

      rfiDetectTimeDomain( "REF", nPRI_fp, fp*ref_PRIs, M, TAtd, NULL,
                           0.0, 0.0,
                           RFIthresh_td, NULL, 
                           &RFIflagsCalData->RFIflag_ref_td1[idx]);

      //      for (size_t j=0; j<fp*ref_PRIs; j++) {
      //if (RFIflagsCalData->RFIflag_ref_td1[idx+j] == 1) {
      //  cout << "Ref " << n << " " << i << " " << ipol << " " << j << " "
      //       << idx+j << endl;
      //  n++;
      //}
      //}

    } // ipol loop

    // Ref ND
    memcpy( refnd_counts, &m2_ref_nd[i*nPRI_REF*4], 
            fp*ref_PRIs*4*sizeof(float));  
    for (size_t ipol=0; ipol<4; ipol++) {
      for (size_t j=0; j<fp*ref_PRIs; j++) 
        TAtd[j] = refnd_counts[j*4+ipol];

      int idx = i*nPRI_REF*4 + ipol*nPRI_REF;

      rfiDetectTimeDomain( "REF", nPRI_fp, fp*ref_PRIs, M, TAtd, NULL, 
                           0.0, 0.0,
                           RFIthresh_td, NULL,
                           &RFIflagsCalData->RFIflag_refnd_td1[idx]);

    } // ipol loop

  } // i loop


  // Sub Band
  double ref_pb[16][4];
  double refnd_pb[16][4];
  readDouble( file_id, "ref_pb", &ref_pb[0][0]);
  readDouble( file_id, "refnd_pb", &refnd_pb[0][0]);

  float ref_counts16[(nPRI_REF/4)*16*4];
  float refnd_counts16[(nPRI_REF/4)*16*4];
  float TAcf[(nPRI_REF/4)*16];

  uint32_t M16 = 4;
  for (size_t i=0; i<nhires; i++) {

    uint16_t fp = footprintsPerScan[highresolution_scan_index[i]];

    // Ref
    memcpy( ref_counts16, &m2_16_ref[i*(nPRI_REF/4)*16*4], 
            fp*ref_PKTs*16*4*sizeof(float));  
    for (size_t ipol=0; ipol<4; ipol++) {
      for (size_t j=0; j<fp*ref_PKTs*16; j++) 
        TAcf[j] = ref_counts16[j*4+ipol] / ref_pb[j % 16][ipol];

      int idx = i*(nPRI_REF/4)*16*4 + ipol*(nPRI_REF/4)*16;

      rfiDetectCrossFreq( "REF", fp*ref_PKTs, M16, TAcf, NULL,
                          0.0, 0.0, 0.0, RFIthresh_cf, NULL,
                          &RFIflagsCalData->RFIflag16_ref_cf1[idx]);
    } // ipol loop

    // Ref ND
    memcpy( refnd_counts16, &m2_16_ref_nd[i*(nPRI_REF/4)*16*4], 
            fp*ref_PKTs*16*4*sizeof(float));  
    for (size_t ipol=0; ipol<4; ipol++) {
      for (size_t j=0; j<fp*ref_PKTs*16; j++) 
        TAcf[j] = refnd_counts16[j*4+ipol] /  refnd_pb[j % 16][ipol];;

      int idx = i*(nPRI_REF/4)*16*4 + ipol*(nPRI_REF/4)*16;

      rfiDetectCrossFreq( "REF", fp*ref_PKTs, M16, TAcf, NULL,
                          0.0, 0.0, 0.0, RFIthresh_cf, NULL,
                          &RFIflagsCalData->RFIflag16_refnd_cf1[idx]);
    } // ipol loop

  } // i loop

  delete( m2_ref); 
  delete( m2_16_ref);
  delete( m2_ref_nd); 
  delete( m2_16_ref_nd);

  delete( RFIthresh_td);
  delete( RFIthresh_cf);

  H5Gclose( gid);
  H5Fclose( file_id);

  return 0;
}


int l1aSMAPFile::rfiDetect( const char *rfiParameters_filename,
                            uint32_t *highresolution_scan_index,
                            uint16_t *footprintsPerScan,
                            uint32_t nPRI_fp,
                            TAs_struct *TAs,
                            rfiFlags_struct *RFIflagsAnt) {

  uint16_t *digital_dn = new uint16_t[this->nscans*51];
  readU16( this->gid[HOUSEKEEP], "digital_dn", digital_dn);

  int lost = 61;
  double Tau[MAXSCANS];
  for (size_t i=0; i<nscans; i++) 
    Tau[i] = ((((double) digital_dn[i*51+6]) / 640) + 280 - lost) * 1e-6; 
  for (size_t i=1; i<nscans; i++) Tau[nscans-i] = Tau[nscans-i-1];

  delete( digital_dn);

  hid_t file_id = 
    H5Fopen( rfiParameters_filename, H5F_ACC_RDONLY, H5P_DEFAULT);

  double AntMax_td;
  double AntMax_cf;
  double BW;
  readDouble( file_id, "AntMax_td", &AntMax_td);
  readDouble( file_id, "AntMax_cf", &AntMax_cf);
  readDouble( file_id, "BW", &BW);

  hid_t gid = H5Gopen1( file_id, "RFIthresh_data");

  double *RFIthresh_td = new double[1800*3600*4];
  double *RFIthresh_t3 = new double[1800*3600*4];
  double *RFIthresh_t4 = new double[1800*3600*4];
  readDouble( gid, "RFIthresh_td", RFIthresh_td);
  readDouble( gid, "RFIthresh_t3", RFIthresh_t3);
  readDouble( gid, "RFIthresh_t4", RFIthresh_t4);

  float M = 0.9;
  float TAtd[nPRI_ANT];
  float Trec[nPRI_ANT];
  double tau;
  uint32_t int_time = 1;
  for (size_t i=0; i<nscans; i++) {
    tau = Tau[i];
    uint16_t fp = footprintsPerScan[i];
    uint32_t idx = i*(nPRI_ANT-last_PRIs);

    // H pol
    for (size_t j=0; j<fp*nPRI_fp; j++) {
      uint32_t go_idx = idx+j;

      TAtd[j] = TAs->TA_h[go_idx];
      Trec[j] = TAs->Offseth[go_idx] / TAs->Gainh[go_idx];
    } // j loop

    rfiDetectTimeDomain( "ANT", nPRI_fp, fp*nPRI_fp, M, TAtd, Trec, 
                         AntMax_td, BW*tau,
                         RFIthresh_td,
                         &this->rfiThreshidx[i*nPRI_ANT],
                         &RFIflagsAnt->RFIflag_h_td1[idx]);

    // V pol
    for (size_t j=0; j<fp*nPRI_fp; j++) {
      uint32_t go_idx = idx+j;

      TAtd[j] = TAs->TA_v[go_idx];
      Trec[j] = TAs->Offsetv[go_idx] / TAs->Gainv[go_idx];
    } // j loop

    rfiDetectTimeDomain( "ANT", nPRI_fp, fp*nPRI_fp, M, TAtd, Trec, 
                         AntMax_td, BW*tau,
                         RFIthresh_td,
                         &this->rfiThreshidx[i*nPRI_ANT],
                         &RFIflagsAnt->RFIflag_v_td1[idx]);

    // T3 T4
    for (size_t j=0; j<footprintsPerScan[i]*nPRI_fp; j++) {
      uint32_t go_idx = idx+j;

      RFIflagsAnt->RFIflag_t3[go_idx] = 0;
      RFIflagsAnt->RFIflag_t4[go_idx] = 0;

      if ( rfiThreshidx[i*nPRI_ANT+j] == -1) continue;

      if (fabs(TAs->T3[go_idx]) > RFIthresh_t3[rfiThreshidx[i*nPRI_ANT+j]]) 
          RFIflagsAnt->RFIflag_t3[go_idx] = 1;

      if (fabs(TAs->T4[go_idx]) > RFIthresh_t4[rfiThreshidx[i*nPRI_ANT+j]]) 
          RFIflagsAnt->RFIflag_t4[go_idx] = 1;
    } // j loop

  } // i loop

  delete( RFIthresh_td);
  delete( RFIthresh_t3);
  delete( RFIthresh_t4);


  // SubBand
  double *RFIthresh_cf = new double[1800*3600*4];
  double *RFIthresh_t3sub = new double[1800*3600*4];
  double *RFIthresh_t4sub = new double[1800*3600*4];
  readDouble( gid, "RFIthresh_cf", RFIthresh_cf);
  readDouble( gid, "RFIthresh_t3sub", RFIthresh_t3sub);
  readDouble( gid, "RFIthresh_t4sub", RFIthresh_t4sub);

  uint32_t M16 = 4;
  float TAcf[(nPRI_ANT/4)*16];
  float Trec16[(nPRI_ANT/4)*16];
  float accm[16];
  for (size_t i=0; i<nhires; i++) {

    size_t hr_idx = highresolution_scan_index[i];
    uint16_t fp = footprintsPerScan[hr_idx];
    
    uint32_t idx = i*(nPRI_ANT-last_PRIs)/4;

    // H-pol
    for (size_t j=0; j<fp*nANT_PKTs_fp*16; j++) {
      uint32_t go_idx = idx*16+j;

      TAcf[j] = TAs->TA16_h[go_idx];
      Trec16[j] = TAs->Offseth16[go_idx] / TAs->Gainh16[go_idx];
    } // j loop

    rfiDetectCrossFreq( "ANT", fp*nANT_PKTs_fp, M16, TAcf, Trec16,
                        AntMax_cf, BW, Tau[hr_idx], RFIthresh_cf, 
                        &this->rfiThreshidx16[i*(nPRI_ANT/4)],
                        &RFIflagsAnt->RFIflag16_h_cf1[idx*16]);      

    // H-pol FP
    for (size_t j=0; j<fp; j++) {
      for (size_t k=0; k<16; k++) {
        accm[k] = 0.0;
        for (size_t l=0; l<nANT_PKTs_fp; l++) 
          accm[k] += TAcf[j*nANT_PKTs_fp*16+l*16+k];
        TAcf[j*16+k] = accm[k] / nANT_PKTs_fp;
        Trec[j*16+k] = Trec[j*nANT_PKTs_fp*16+k];
      } // k loop
    } // j loop

    rfiDetectCrossFreq( "ANT", fp, M16, TAcf, Trec16,
                        AntMax_cf, BW, Tau[hr_idx]*nANT_PKTs_fp, RFIthresh_cf, 
                        &this->rfiThreshidxFP[i*(nPRI_ANT/4/nANT_PKTs_fp)],
                        &RFIflagsAnt->RFIflag16_h_cf11[(idx/nANT_PKTs_fp)*16]);


    // V-pol
    for (size_t j=0; j<fp*nANT_PKTs_fp*16; j++) {
      uint32_t go_idx = idx*16+j;

      TAcf[j] = TAs->TA16_v[go_idx];
      Trec16[j] = TAs->Offsetv16[go_idx] / TAs->Gainv16[go_idx];
    } // j loop

    rfiDetectCrossFreq( "ANT", fp*nANT_PKTs_fp, M16, TAcf, Trec16,
                        AntMax_cf, BW, Tau[hr_idx], RFIthresh_cf, 
                        &this->rfiThreshidx16[i*(nPRI_ANT/4)],
                        &RFIflagsAnt->RFIflag16_v_cf1[idx*16]);      

    // V-pol FP
    for (size_t j=0; j<fp; j++) {
      for (size_t k=0; k<16; k++) {
        accm[k] = 0.0;
        for (size_t l=0; l<nANT_PKTs_fp; l++) 
          accm[k] += TAcf[j*nANT_PKTs_fp*16+l*16+k];
        TAcf[j*16+k] = accm[k] / nANT_PKTs_fp;
        Trec[j*16+k] = Trec[j*nANT_PKTs_fp*16+k];
      } // k loop
    } // j loop

    rfiDetectCrossFreq( "ANT", fp, M16, TAcf, Trec16,
                        AntMax_cf, BW, Tau[hr_idx]*nANT_PKTs_fp, RFIthresh_cf, 
                        &this->rfiThreshidxFP[i*(nPRI_ANT/4/nANT_PKTs_fp)],
                        &RFIflagsAnt->RFIflag16_v_cf11[(idx/nANT_PKTs_fp)*16]);

    // T3_16 T4_16
    for (size_t j=0; j<fp*nANT_PKTs_fp*16; j++) {
      uint32_t go_idx = idx*16+j;

      RFIflagsAnt->RFIflag16_t3[go_idx] = 0;
      RFIflagsAnt->RFIflag16_t4[go_idx] = 0;

      int32_t threshIdx = rfiThreshidx16[i*(nPRI_ANT/4)+(j/16)];
      if ( threshIdx == -1) continue;

      if (fabs(TAs->T3_16[go_idx]) > RFIthresh_t3sub[threshIdx]) 
        RFIflagsAnt->RFIflag16_t3[go_idx] = 1;

      if (fabs(TAs->T4_16[go_idx]) > RFIthresh_t4sub[threshIdx]) 
          RFIflagsAnt->RFIflag16_t4[go_idx] = 1;
    } // j loop

  } // i loop

  ofstream tempOut;
  //  tempOut.open ("RFIflag16_h_cf1.bin", ios::out | ios::trunc | ios::binary);
  //tempOut.write((char *) RFIflagsAnt->RFIflag16_h_cf1, 
  //             nscans*nPRI_REF*16*sizeof(uint8_t));
  //tempOut.close();

  //tempOut.open ("RFIflag16_h_cf11.bin", ios::out | ios::trunc | ios::binary);
  //tempOut.write((char *) RFIflagsAnt->RFIflag16_h_cf11, 
  //            nscans*(nPRI_REF/8)*16*sizeof(uint8_t));
  //tempOut.close();

  //tempOut.open ("RFIflag16_v_cf1.bin", ios::out | ios::trunc | ios::binary);
  //tempOut.write((char *) RFIflagsAnt->RFIflag16_v_cf1, 
  //            nscans*nPRI_REF*16*sizeof(uint8_t));
  //tempOut.close();

  //tempOut.open ("RFIflag16_v_cf11.bin", ios::out | ios::trunc | ios::binary);
  //tempOut.write((char *) RFIflagsAnt->RFIflag16_v_cf11, 
  //            nscans*(nPRI_REF/8)*16*sizeof(uint8_t));
  //tempOut.close();

  //  tempOut.open ("RFIflag16_t3.bin", ios::out | ios::trunc | ios::binary);
  //tempOut.write((char *) RFIflagsAnt->RFIflag16_t3, 
  //            nscans*nPRI_REF*16*sizeof(uint8_t));
  //tempOut.close();

  //tempOut.open ("RFIflag16_t4.bin", ios::out | ios::trunc | ios::binary);
  //tempOut.write((char *) RFIflagsAnt->RFIflag16_t4, 
  //              nscans*nPRI_REF*16*sizeof(uint8_t));
  //tempOut.close();

  delete( RFIthresh_cf);
  delete( RFIthresh_t3sub);
  delete( RFIthresh_t4sub);

  H5Gclose( gid);
  H5Fclose( file_id);

  return 0;
}


int l1aSMAPFile::rfiDetectTimeDomain( const char *ref_ant,
                                      uint32_t nPRI_fp, uint32_t nCounts,
                                      float M, float *TAtd, float *Trec,
                                      double AntMax_td, double BWtau,
                                      double *RFIthresh_td,
                                      int32_t *rfiThreshidx,
                                      uint8_t *RFIflag_td) {

  uint32_t fp = nCounts / nPRI_fp;

  float M_remove_td = (1-M)/2;

  ofstream meanstdOut;

  float ANT[2*nPRI_fp];
  double mean, std;
  for (uint i=0; i<fp; i++) {
    int len = 0, minj, maxj;
    if (i == 0) {
      minj = 0;
      maxj = 2 * nPRI_fp;
    } else if (i == (fp-1)) {
      minj = (i-1) * nPRI_fp;
      maxj = fp * nPRI_fp;
    } else {
      minj = i * nPRI_fp - (nPRI_fp / 2);
      maxj = minj + 2 * nPRI_fp;
    }
    for (int j=minj; j<maxj; j++) {
      //      cout << j << endl;
      ANT[len] = TAtd[j];
      len++;
    } // j loop
    insertion_sort( ANT, len);

    // Compute mean/std
    mean=0.0;
    std=0.0;
    uint N;
    if ( strcmp( ref_ant, "REF") == 0) {
      N = (int) (M*len + 0.5);
      for (size_t k=0; k<N; k++) { 
        mean += ANT[k];
        std += ANT[k]*ANT[k];
      } // k loop
      mean /= N;
      std = sqrt((std - N*mean*mean) / (N-1));

      // Check if TAtd value exceeds RFIthreshold
      for (size_t k=i*nPRI_fp; k<(i+1)*nPRI_fp; k++) {
        RFIflag_td[k] = 0;
        if (TAtd[k] > (mean + RFIthresh_td[0]*std)) 
          RFIflag_td[k] = 1;
      }

    } else if ( strcmp( ref_ant, "ANT") == 0) {
      N = (int) (M_remove_td*len + 0.5);
      for (size_t k=N; k<len-N; k++) {
        mean += ANT[k];
      } // k loop
      mean /= len-2*N;
      if ( mean > AntMax_td) mean = AntMax_td;

      std = (Trec[i*nPRI_fp]+mean) / sqrt(BWtau);

      // Check if TAtd value exceeds RFIthreshold
      for (size_t k=i*nPRI_fp; k<(i+1)*nPRI_fp; k++) {
        RFIflag_td[k] = 0;
        if ( rfiThreshidx[k] == -1) continue;
        if (fabs(TAtd[k] - ((float) mean)) > 
            RFIthresh_td[rfiThreshidx[k]] * ((float) std)) 
          RFIflag_td[k] = 1;
      }
    } else {
      cout << "Bad ref_ant: " << ref_ant << endl;
      exit(1);
    }
  } // i loop

  if ( strcmp( ref_ant, "REF") == 0) {
    for (size_t k=fp*nPRI_fp; k<nCounts; k++) {
      RFIflag_td[k] = 0;
      if (TAtd[k] > (mean + RFIthresh_td[0]*std)) 
        RFIflag_td[k] = 1;
    }
  } else if ( strcmp( ref_ant, "ANT") == 0) {
    for (size_t k=fp*nPRI_fp; k<nCounts; k++) {
      RFIflag_td[k] = 0;
      if ( rfiThreshidx[k] == -1) continue;
      if (fabs(TAtd[k] - ((float) mean)) > 
          RFIthresh_td[rfiThreshidx[k]] * ((float) std)) 
        RFIflag_td[k] = 1;
    }
  }
  //  meanstdOut.close();

  return 0;
}

int l1aSMAPFile::rfiDetectCrossFreq( const char *ref_ant,
                                     uint32_t nCounts, uint32_t M, 
                                     float *TAcf, float *Trec16,
                                     double AntMax_cf, double BW, double tau16,
                                     double *RFIthresh_cf, 
                                     int32_t *rfiThreshidx,
                                     uint8_t *RFIflag_cf) {

  float ANT[16];
  float trec[16];
  double mean, std;
  if ( strcmp( ref_ant, "REF") == 0) {
    for (uint i=0; i<nCounts; i++) {
      for (int j=0; j<16; j++) ANT[j] = TAcf[i*16+j];
      insertion_sort( ANT, 16);
      mean=0.0;
      std=0.0;
      uint N = 16 - M;
      for (size_t k=0; k<N; k++) { 
        mean += ANT[k];
        std += ANT[k]*ANT[k];
      } // k loop
      mean /= N;
      std = sqrt((std - N*mean*mean) / (N-1));

      RFIflag_cf[i*16+0] = 0;
      if ((TAcf[i*16+0] > (mean + RFIthresh_cf[0]*std)) ||
          (TAcf[i*16+1] > (mean + RFIthresh_cf[0]*std)))
        RFIflag_cf[i*16+0] = 1;

      if ((RFIflag_cf[i*16+0] == 1) ||
          (TAcf[i*16+15] > (mean + RFIthresh_cf[0]*std)))
        RFIflag_cf[i*16+0] = 1;

      for (int j=1; j<15; j++) {
        RFIflag_cf[i*16+j] = 0;
        if ((TAcf[i*16+j-1] > (mean + RFIthresh_cf[0]*std)) ||
            (TAcf[i*16+j] > (mean + RFIthresh_cf[0]*std)))
          RFIflag_cf[i*16+j] = 1;
        
        if ((RFIflag_cf[i*16+j] == 1) || 
            (TAcf[i*16+j+1] > (mean + RFIthresh_cf[0]*std)))
          RFIflag_cf[i*16+j] = 1;
      }

      RFIflag_cf[i*16+15] = 0;
      if ((TAcf[i*16+14] > (mean + RFIthresh_cf[0]*std)) ||
          (TAcf[i*16+15] > (mean + RFIthresh_cf[0]*std)))
        RFIflag_cf[i*16+15] = 1;
      
      if ((RFIflag_cf[i*16+15] == 1) ||
          (TAcf[i*16+0] > (mean + RFIthresh_cf[0]*std)))
      RFIflag_cf[i*16+15] = 1;
    } // i loop
  } else {
    // ANT
    uint32_t Mch_remove = M/2;
    uint8_t detidx[16];
    for (uint i=0; i<nCounts; i++) {
      if ( rfiThreshidx[i] == -1) {
        //        cout << "Threshold index equal to -1" << endl;
        //exit(1);
      }

      double rfiThresh = 0.0;
      if ( rfiThreshidx[i] != -1) rfiThresh = RFIthresh_cf[rfiThreshidx[i]];
      //if ( rfiThresh != 3.0) exit(1); // JMG
      
      for (int j=0; j<16; j++) ANT[j] = TAcf[i*16+j];
      insertion_sort( ANT, 16);
      mean=0.0;
      float std16;
      uint N = 16 - Mch_remove;
      for (size_t k=Mch_remove; k<N; k++) { 
        mean += ANT[k];
      } // k loop
      mean /= N - (Mch_remove);
      if (mean > AntMax_cf) mean = AntMax_cf;  

      for (int j=0; j<16; j++) {
        trec[j] = Trec16[i*16+j];
        std16 = (trec[j] + ((float) mean)) / sqrt((BW/16)*tau16*4);
        detidx[j] = fabs(TAcf[i*16+j] - ((float) mean)) > rfiThresh*std16;
      } // j loop

     RFIflag_cf[i*16+0] = 0;
     if ( (detidx[0] == 1) || (detidx[1] == 1)) RFIflag_cf[i*16+0] = 1;

     if ( (RFIflag_cf[i*16+0] == 1) || (detidx[15] == 1)) 
       RFIflag_cf[i*16+0] = 1;

      for (int j=1; j<15; j++) {
        RFIflag_cf[i*16+j] = 0;
        if ( (detidx[j-1] == 1) || (detidx[j] == 1)) RFIflag_cf[i*16+j] = 1;
        
        if ( (RFIflag_cf[i*16+j] == 1) || (detidx[j+1] == 1)) 
          RFIflag_cf[i*16+j] = 1;
      }

      RFIflag_cf[i*16+15] = 0;
      if ( detidx[14] || detidx[15]) RFIflag_cf[i*16+15] = 1;
      
      if ( (RFIflag_cf[i*16+15] == 1) || (detidx[0] == 1)) 
        RFIflag_cf[i*16+15] = 1;
    } // i loop

  } // if ( strcmp( ref_ant, "REF")

  return 0;
}


int l1aSMAPFile::rfiRemovalCalData( uint16_t *footprintsPerScan,
                                    uint32_t *highresolution_scan_index,
                                    rfiFlagsKurtosis_struct *RFIflagsKurt,
                                    rfiFlagsKurtosis_struct *RFIflagsKurtND,
                                    rfiFlagsCalData_struct *RFIflagsCalData,
                                    calDataNoRFI_struct *calDataNoRFI) {

  // Read m2, t3, t4 reference (& ND) data
  readFloat( this->gid[MOMDATA], "m2_ref", calDataNoRFI->m2_ref);
  readFloat( this->gid[MOMDATA], "m2_ref_nd", calDataNoRFI->m2_ref_nd);
  readFloat( this->gid[MOMDATA], "t3_ref", calDataNoRFI->t3_ref);
  readFloat( this->gid[MOMDATA], "t3_ref_nd", calDataNoRFI->t3_ref_nd);
  readFloat( this->gid[MOMDATA], "t4_ref", calDataNoRFI->t4_ref);
  readFloat( this->gid[MOMDATA], "t4_ref_nd", calDataNoRFI->t4_ref_nd);

  //Note: For these arrays: pxl# changes fastest, then pol, then scan
  uint8_t *RFIflag_MAXPD_ref = new uint8_t[nscans*nPRI_REF*4];
  uint8_t *RFIflag_MAXPD_refnd = new uint8_t[nscans*nPRI_REF*4];

  memset(RFIflag_MAXPD_ref, 0, nscans*nPRI_REF*4);
  memset(RFIflag_MAXPD_refnd, 0, nscans*nPRI_REF*4);

  uint32_t ref_PRIs = 8; //Number of reference PRIs per footprint
  uint32_t refnd_PRIs = 8; //Number of reference+noise diode PRIs per footprint

  uint8_t iflagHref[nPRI_REF];
  uint8_t iflagVref[nPRI_REF];
  uint32_t sum;

  // FullBand
  for (size_t i=0; i<nscans; i++) {
    //    cout << i << endl;
    uint16_t fp = footprintsPerScan[i];

    uint32_t idx = i*nPRI_REF*4;

    // Ref H-pol
    sum = 0;
    for (size_t j=0; j<fp*ref_PRIs; j++) {
      iflagHref[j] = 
        RFIflagsCalData->RFIflag_ref_td1[idx+0*nPRI_REF+j] | 
        RFIflagsCalData->RFIflag_ref_td1[idx+1*nPRI_REF+j] | 
        RFIflagsKurt->RFIflag_kurt_h[i*nPRI_REF+j];
    }
    for (size_t j=0; j<fp*ref_PRIs; j++) 
      sum += iflagHref[j];
    if (sum == fp*ref_PRIs) 
      for (size_t j=0; j<fp*ref_PRIs; j++) iflagHref[j] = 0;
    memcpy( &RFIflag_MAXPD_ref[i*nPRI_REF*4+0*nPRI_REF], 
            iflagHref, fp*ref_PRIs);

    // Ref V-pol
    sum = 0;
    for (size_t j=0; j<fp*ref_PRIs; j++) {
      iflagVref[j] = 
        RFIflagsCalData->RFIflag_ref_td1[idx+2*nPRI_REF+j] | 
        RFIflagsCalData->RFIflag_ref_td1[idx+3*nPRI_REF+j] | 
        RFIflagsKurt->RFIflag_kurt_v[i*nPRI_REF+j];
    }
    for (size_t j=0; j<fp*ref_PRIs; j++) 
      sum += iflagVref[j];
    if (sum == fp*ref_PRIs) 
      for (size_t j=0; j<fp*ref_PRIs; j++) iflagVref[j] = 0;
    memcpy( &RFIflag_MAXPD_ref[i*nPRI_REF*4+2*nPRI_REF], 
            iflagVref, fp*ref_PRIs);


    // RefND H-pol
    sum = 0;
    for (size_t j=0; j<fp*refnd_PRIs; j++) {
      iflagHref[j] = 
        RFIflagsCalData->RFIflag_refnd_td1[idx+0*nPRI_REF+j] | 
        RFIflagsCalData->RFIflag_refnd_td1[idx+1*nPRI_REF+j] | 
        RFIflagsKurtND->RFIflag_kurt_h[i*nPRI_REF+j];
    }
    for (size_t j=0; j<fp*refnd_PRIs; j++) 
      sum += iflagHref[j];
    if (sum == fp*refnd_PRIs) 
      for (size_t j=0; j<fp*refnd_PRIs; j++) iflagHref[j] = 0;
    memcpy( &RFIflag_MAXPD_refnd[i*nPRI_REF*4+0*nPRI_REF], 
            iflagHref, fp*refnd_PRIs);

    // RefND V-pol
    sum = 0;
    for (size_t j=0; j<fp*refnd_PRIs; j++) {
      iflagVref[j] = 
        RFIflagsCalData->RFIflag_refnd_td1[idx+2*nPRI_REF+j] | 
        RFIflagsCalData->RFIflag_refnd_td1[idx+3*nPRI_REF+j] | 
        RFIflagsKurtND->RFIflag_kurt_v[i*nPRI_REF+j];
    }
    for (size_t j=0; j<fp*refnd_PRIs; j++) 
      sum += iflagVref[j];
    if (sum == fp*refnd_PRIs) 
      for (size_t j=0; j<fp*refnd_PRIs; j++) iflagVref[j] = 0;
    memcpy( &RFIflag_MAXPD_refnd[i*nPRI_REF*4+2*nPRI_REF], 
            iflagVref, fp*refnd_PRIs);

    //Note: For these arrays: pxl# changes fastest, then pol, then scan
    memcpy( &RFIflag_MAXPD_ref[i*nPRI_REF*4+nPRI_REF*1],
            &RFIflag_MAXPD_ref[i*nPRI_REF*4+nPRI_REF*0],
            nPRI_REF);

    memcpy( &RFIflag_MAXPD_ref[i*nPRI_REF*4+nPRI_REF*3],
            &RFIflag_MAXPD_ref[i*nPRI_REF*4+nPRI_REF*2],
            nPRI_REF);

    memcpy( &RFIflag_MAXPD_refnd[i*nPRI_REF*4+nPRI_REF*1],
            &RFIflag_MAXPD_refnd[i*nPRI_REF*4+nPRI_REF*0],
            nPRI_REF);

    memcpy( &RFIflag_MAXPD_refnd[i*nPRI_REF*4+nPRI_REF*3],
            &RFIflag_MAXPD_refnd[i*nPRI_REF*4+nPRI_REF*2],
            nPRI_REF);
  } // i loop


  for (size_t j=0; j<nscans*nPRI_REF*4; j++) {
    if ( RFIflag_MAXPD_ref[j] == 1) {
      uint32_t iscan = j / (nPRI_REF*4);
      uint32_t ipol = (j - iscan*(nPRI_REF*4)) / nPRI_REF;
      uint32_t ipxl = j - iscan*(nPRI_REF*4) - ipol*nPRI_REF;
      calDataNoRFI->m2_ref[iscan*(nPRI_REF*4)+ipxl*4+ipol] = 0.0;

      if ( ipol == 0 || ipol == 2) {
        calDataNoRFI->t3_ref[iscan*nPRI_REF+ipxl] = nanf("");
        calDataNoRFI->t4_ref[iscan*nPRI_REF+ipxl] = nanf("");
      }
    }

    if ( RFIflag_MAXPD_refnd[j] == 1) {
      uint32_t iscan = j / (nPRI_REF*4);
      uint32_t ipol = (j - iscan*(nPRI_REF*4)) / nPRI_REF;
      uint32_t ipxl = j - iscan*(nPRI_REF*4) - ipol*nPRI_REF;
      calDataNoRFI->m2_ref_nd[iscan*(nPRI_REF*4)+ipxl*4+ipol] = 0.0;

      if ( ipol == 0 || ipol == 2) {
        calDataNoRFI->t3_ref_nd[iscan*nPRI_REF+ipxl] = nanf("");
        calDataNoRFI->t4_ref_nd[iscan*nPRI_REF+ipxl] = nanf("");
      }
    }
  }
  delete( RFIflag_MAXPD_ref);
  delete( RFIflag_MAXPD_refnd);


  /// SubBand Section ////
  readFloat( this->gid[HIRESMOM], "m2_16_ref", calDataNoRFI->m2_16_ref);
  readFloat( this->gid[HIRESMOM], "m2_16_ref_nd", calDataNoRFI->m2_16_ref_nd);
  readFloat( this->gid[HIRESMOM], "t3_16_ref", calDataNoRFI->t3_16_ref);
  readFloat( this->gid[HIRESMOM], "t3_16_ref_nd", calDataNoRFI->t3_16_ref_nd);
  readFloat( this->gid[HIRESMOM], "t4_16_ref", calDataNoRFI->t4_16_ref);
  readFloat( this->gid[HIRESMOM], "t4_16_ref_nd", calDataNoRFI->t4_16_ref_nd);

  uint8_t *RFIflag_MAXPD_ref16 = new uint8_t[nhires*(nPRI_REF/4)*16*4];
  uint8_t *RFIflag_MAXPD_refnd16 = new uint8_t[nhires*(nPRI_REF/4)*16*4];

  memset(RFIflag_MAXPD_ref16, 0, nhires*(nPRI_REF/4)*16*4);
  memset(RFIflag_MAXPD_refnd16, 0, nhires*(nPRI_REF/4)*16*4);

  uint8_t iflag16Href[nPRI_REF/4][16];
  uint8_t iflag16Vref[nPRI_REF/4][16];

  uint32_t ref_PKTs = 2;
  
  for (size_t i=0; i<nhires; i++) {
    //    cout << i << endl;

    uint32_t idx = i * (nPRI_REF/4)*16*4;

    uint16_t fp = footprintsPerScan[highresolution_scan_index[i]];

    // H-pol 16
    sum = 0;
    for (size_t j=0; j<fp*ref_PKTs; j++) {

      for (size_t k=0; k<16; k++) {

        iflag16Href[j][k] =
          RFIflagsCalData->RFIflag16_ref_cf1[idx+0*(nPRI_REF/4)*16+j*16+k] | 
          RFIflagsCalData->RFIflag16_ref_cf1[idx+1*(nPRI_REF/4)*16+j*16+k] | 
          RFIflagsKurt->RFIflag16_kurt_h[i*(nPRI_REF/4)*16+j*16+k];

        sum += iflag16Href[j][k];
      } // k loop
    } // j loop

    if (sum == (fp*ref_PKTs)*16) {
      for (size_t j=0; j<(nPRI_REF/4); j++)
        for (size_t k=0; k<16; k++)
          iflag16Href[j][k] = 0;
    }

    for (size_t j=0; j<fp*ref_PKTs; j++) {
      for (size_t k=0; k<16; k++) {
        RFIflag_MAXPD_ref16[idx+0*(nPRI_REF/4)*16+j*16+k] = iflag16Href[j][k];
        RFIflag_MAXPD_ref16[idx+1*(nPRI_REF/4)*16+j*16+k] = iflag16Href[j][k];
      }
    }

    // V-pol 16
    sum = 0;
    for (size_t j=0; j<fp*ref_PKTs; j++) {
      for (size_t k=0; k<16; k++) {

        iflag16Vref[j][k] =
          RFIflagsCalData->RFIflag16_ref_cf1[idx+2*(nPRI_REF/4)*16+j*16+k] | 
          RFIflagsCalData->RFIflag16_ref_cf1[idx+3*(nPRI_REF/4)*16+j*16+k] | 
          RFIflagsKurt->RFIflag16_kurt_v[i*(nPRI_REF/4)*16+j*16+k];

        sum += iflag16Vref[j][k];
      } // k loop
    } // j loop

    if (sum == (fp*ref_PKTs)*16) {
      for (size_t j=0; j<(nPRI_REF/4); j++)
        for (size_t k=0; k<16; k++)
          iflag16Vref[j][k] = 0;
    }

    for (size_t j=0; j<fp*ref_PKTs; j++) {
      for (size_t k=0; k<16; k++) {
        RFIflag_MAXPD_ref16[idx+2*(nPRI_REF/4)*16+j*16+k] = iflag16Vref[j][k];
        RFIflag_MAXPD_ref16[idx+3*(nPRI_REF/4)*16+j*16+k] = iflag16Vref[j][k];
      }
    }

    // H-pol ND 16
    sum = 0;
    for (size_t j=0; j<fp*ref_PKTs; j++) {
      for (size_t k=0; k<16; k++) {

        iflag16Href[j][k] =
          RFIflagsCalData->RFIflag16_refnd_cf1[idx+0*(nPRI_REF/4)*16+j*16+k] | 
          RFIflagsCalData->RFIflag16_refnd_cf1[idx+1*(nPRI_REF/4)*16+j*16+k] | 
          RFIflagsKurtND->RFIflag16_kurt_h[i*(nPRI_REF/4)*16+j*16+k];

        sum += iflag16Href[j][k];
      } // k loop
    } // j loop

    if (sum == (fp*ref_PKTs)*16) {
      for (size_t j=0; j<(nPRI_REF/4); j++)
        for (size_t k=0; k<16; k++)
          iflag16Href[j][k] = 0;
    }

    for (size_t j=0; j<fp*ref_PKTs; j++) {
      for (size_t k=0; k<16; k++) {
        RFIflag_MAXPD_refnd16[idx+0*(nPRI_REF/4)*16+j*16+k] = iflag16Href[j][k];
        RFIflag_MAXPD_refnd16[idx+1*(nPRI_REF/4)*16+j*16+k] = iflag16Href[j][k];
      }
    }

    // V-pol ND 16
    sum = 0;
    for (size_t j=0; j<fp*ref_PKTs; j++) {
      for (size_t k=0; k<16; k++) {

        iflag16Vref[j][k] =
          RFIflagsCalData->RFIflag16_refnd_cf1[idx+2*(nPRI_REF/4)*16+j*16+k] | 
          RFIflagsCalData->RFIflag16_refnd_cf1[idx+3*(nPRI_REF/4)*16+j*16+k] | 
          RFIflagsKurtND->RFIflag16_kurt_v[i*(nPRI_REF/4)*16+j*16+k];

        sum += iflag16Vref[j][k];
      } // k loop
    } // j loop

    if (sum == (fp*ref_PKTs)*16) {
      for (size_t j=0; j<(nPRI_REF/4); j++)
        for (size_t k=0; k<16; k++)
          iflag16Vref[j][k] = 0;
    }

    for (size_t j=0; j<fp*ref_PKTs; j++) {
      for (size_t k=0; k<16; k++) {
        RFIflag_MAXPD_refnd16[idx+2*(nPRI_REF/4)*16+j*16+k] = iflag16Vref[j][k];
        RFIflag_MAXPD_refnd16[idx+3*(nPRI_REF/4)*16+j*16+k] = iflag16Vref[j][k];
      }
    }

  } // i loop

  uint32_t pixPerScan = (nPRI_REF/4)*16*4;
  for (size_t j=0; j<nhires*pixPerScan; j++) {
    // Ref
    if ( RFIflag_MAXPD_ref16[j] == 1) {
      uint32_t iscan = j / pixPerScan;
      uint32_t ipol = (j - iscan*pixPerScan) / ((nPRI_REF/4)*16);
      uint32_t ipxl = (j - iscan*pixPerScan - ipol*(nPRI_REF/4)*16) / 16;
      uint32_t i16 = j - iscan*pixPerScan - ipol*(nPRI_REF/4)*16 - ipxl*16;
                                                     
      uint32_t idx = iscan*pixPerScan + ipxl*(16*4) + i16*4;

      calDataNoRFI->m2_16_ref[idx+ipol] = 0.0;

      if ( ipol == 0 || ipol == 2) {
        idx = iscan*(nPRI_REF/4)*16 + ipxl*16 + i16;
        calDataNoRFI->t3_16_ref[idx] = nanf("");
        calDataNoRFI->t4_16_ref[idx] = nanf("");
      }
    } // j loop

    // Ref ND
    if ( RFIflag_MAXPD_refnd16[j] == 1) {
      uint32_t iscan = j / pixPerScan;
      uint32_t ipol = (j - iscan*pixPerScan) / ((nPRI_REF/4)*16);
      uint32_t ipxl = (j - iscan*pixPerScan - ipol*(nPRI_REF/4)*16) / 16;
      uint32_t i16 = j - iscan*pixPerScan - ipol*(nPRI_REF/4)*16 - ipxl*16;
                                                     
      uint32_t idx = iscan*pixPerScan + ipxl*(16*4) + i16*4;
      calDataNoRFI->m2_16_ref_nd[idx+ipol] = 0.0;

      if ( ipol == 0 || ipol == 2) {
        idx = iscan*(nPRI_REF/4)*16 + ipxl*16 + i16;
        calDataNoRFI->t3_16_ref_nd[idx] = nanf("");
        calDataNoRFI->t4_16_ref_nd[idx] = nanf("");
      }
    } // j loop
  } // i loop
  delete( RFIflag_MAXPD_ref16);
  delete( RFIflag_MAXPD_refnd16);

  return 0;
}

int l1aSMAPFile::radomeCharacteristics( const char *reflectorFilename,
                                        string surface_temp_selection,
                                        double cal_loss12_radome[MAXSCANS][2],
                                        double cal_temp12_radome[MAXSCANS]) {

  int temp_selection_array;
  if (surface_temp_selection.compare("center") == 0)
    temp_selection_array = 0;
  else
    temp_selection_array = 1;

  int OrbitD = orbitDirection.compare("Ascending");


  char timestr[MAXSCANS*24];
  double unixtime[MAXSCANS];
  H5LTread_dataset_string ( this->gid[SCDATA], "antenna_scan_time_utc", 
                            timestr);

  char zulu[24];
  int16_t yy=0, mm=0, dd=0;
  int hh=0, mn=0, sc=0, cc=0;
  double sec;
  int status;

  for (size_t i=0; i<nscans; i++) {
    memcpy( zulu, &timestr[i*24], 24);

    status = sscanf( zulu, "%4hd-%2hd-%2hdT%2d:%2d:%2d.%3dZ", 
                     &yy, &mm, &dd, &hh, &mn, &sc, &cc);

    sec = hh*3600.0 + mn*60.0 + sc + ((double) cc) / 1000.0;
    unixtime[i] = ymds2unix( yy, mm, dd, sec);
  }

  memcpy( zulu, halfOrbitStartDateTime.c_str(), 24);
  status = sscanf( zulu, "%4hd-%2hd-%2hdT%2d:%2d:%2d.%3dZ", 
                   &yy, &mm, &dd, &hh, &mn, &sc, &cc);
  sec = hh*3600.0 + mn*60.0 + sc + ((double) cc) / 1000.0;
  double OrbitStartDateNum = ymds2unix( yy, mm, dd, sec);

  memcpy( zulu, halfOrbitStopDateTime.c_str(), 24);
  status = sscanf( zulu, "%4hd-%2hd-%2hdT%2d:%2d:%2d.%3dZ", 
                   &yy, &mm, &dd, &hh, &mn, &sc, &cc);
  sec = hh*3600.0 + mn*60.0 + sc + ((double) cc) / 1000.0;
  double OrbitStopDateNum = ymds2unix( yy, mm, dd, sec);

  double secondsinceNorth[MAXSCANS];
  if (OrbitD != 0) {
    for (size_t i=0; i<nscans; i++) {
      secondsinceNorth[i] = unixtime[i] - OrbitStartDateNum;
      if ( secondsinceNorth[i] < 0) secondsinceNorth[i] += orbitPeriod;
    }
  } else {
    for (size_t i=0; i<nscans; i++) {
      secondsinceNorth[i] = unixtime[i] - (OrbitStopDateNum - orbitPeriod);
      if ( secondsinceNorth[i] < 0) secondsinceNorth[i] -= orbitPeriod;
    }
  }

  double dLdT[2];
  double emissivity_loss[2];

  hid_t file_id = H5Fopen( reflectorFilename, H5F_ACC_RDONLY, H5P_DEFAULT);
  hid_t gid = H5Gopen1( file_id, "Radome");

  readDouble( gid, "dLdT", dLdT);
  readDouble( gid, "emissivity_loss", emissivity_loss);

  double *radomeTemp = new double[366*1000*2];
  readDouble( gid, "temperature", radomeTemp);

  double *radomeTime = new double[366*1000];
  readDouble( gid, "time", radomeTime);

  H5Gclose( gid);
  H5Fclose( file_id);

  double Loss_V = emissivity_loss[1];
  double Loss_H = emissivity_loss[0];

  double Loss_senst_V = dLdT[1];
  double Loss_senst_H = dLdT[0];

  for (size_t i=0; i<nscans; i++) {
    int16_t yr, doy;
    double secs;
    unix2yds( unixtime[i], &yr, &doy, &secs);
    int Column_selection;
    for (size_t j=1000-1; j>=0; j--) {
      int Column_ratio = 
        floor(radomeTime[(doy-1)*1000+j] / secondsinceNorth[i]);
      if ( Column_ratio == 0) {
        Column_selection = j;
        break;
      }
    } // j loop
    //cout << i << " " << Column_selection << endl;

    double Current_Radome12_Temp = 
      radomeTemp[(doy-1)*1000*2+Column_selection*2+temp_selection_array];

    cal_loss12_radome[i][0] =
      Loss_V + Loss_senst_V*(Current_Radome12_Temp - 20);

    cal_loss12_radome[i][1] = 
      Loss_H + Loss_senst_H*(Current_Radome12_Temp - 20);

    cal_temp12_radome[i] = Current_Radome12_Temp + 273.15;

  } // i loop

  delete( radomeTemp);
  delete( radomeTime);

  return 0;
}


int l1aSMAPFile::reflectorCharacteristics( const char *reflectorFilename,
                                           double cal_loss1_refl[MAXSCANS][2],
                                           double cal_temp1_refl[MAXSCANS][2],
                                           double cal_TB1_refl[MAXSCANS][2]) {

  int OrbitD = orbitDirection.compare("Ascending");

  char timestr[MAXSCANS*24];
  double unixtime[MAXSCANS];
  H5LTread_dataset_string ( this->gid[SCDATA], "antenna_scan_time_utc", 
                            timestr);

  char zulu[24];
  int16_t yy=0, mm=0, dd=0;
  int hh=0, mn=0, sc=0, cc=0;
  double sec;
  int status;

  for (size_t i=0; i<nscans; i++) {
    memcpy( zulu, &timestr[i*24], 24);

    status = sscanf( zulu, "%4hd-%2hd-%2hdT%2d:%2d:%2d.%3dZ", 
                     &yy, &mm, &dd, &hh, &mn, &sc, &cc);

    sec = hh*3600.0 + mn*60.0 + sc + ((double) cc) / 1000.0;
    unixtime[i] = ymds2unix( yy, mm, dd, sec);
  }

  memcpy( zulu, halfOrbitStartDateTime.c_str(), 24);
  status = sscanf( zulu, "%4hd-%2hd-%2hdT%2d:%2d:%2d.%3dZ", 
                   &yy, &mm, &dd, &hh, &mn, &sc, &cc);
  sec = hh*3600.0 + mn*60.0 + sc + ((double) cc) / 1000.0;
  double OrbitStartDateNum = ymds2unix( yy, mm, dd, sec);

  memcpy( zulu, halfOrbitStopDateTime.c_str(), 24);
  status = sscanf( zulu, "%4hd-%2hd-%2hdT%2d:%2d:%2d.%3dZ", 
                   &yy, &mm, &dd, &hh, &mn, &sc, &cc);
  sec = hh*3600.0 + mn*60.0 + sc + ((double) cc) / 1000.0;
  double OrbitStopDateNum = ymds2unix( yy, mm, dd, sec);

  double secondsinceNorth[MAXSCANS];
  if (OrbitD != 0) {
    for (size_t i=0; i<nscans; i++) {
      secondsinceNorth[i] = unixtime[i] - OrbitStartDateNum;
      if ( secondsinceNorth[i] < 0) secondsinceNorth[i] += orbitPeriod;
    }
  } else {
    for (size_t i=0; i<nscans; i++) {
      secondsinceNorth[i] = unixtime[i] - (OrbitStopDateNum - orbitPeriod);
      if ( secondsinceNorth[i] < 0) secondsinceNorth[i] -= orbitPeriod;
    }
  }

  hid_t file_id = H5Fopen( reflectorFilename, H5F_ACC_RDONLY, H5P_DEFAULT);
  hid_t gid = H5Gopen1( file_id, "Reflector");

  double *reflEmiss = new double[96*2];
  readDouble( gid, "emissivity", reflEmiss);

  double *reflTemp = new double[366*1000*96];
  readDouble( gid, "temperature", reflTemp);

  double *reflTime = new double[366*1000];
  readDouble( gid, "time", reflTime);

  H5Gclose( gid);
  H5Fclose( file_id);

  for (size_t i=0; i<nscans; i++) {
    int16_t yr, doy;
    double secs;
    unix2yds( unixtime[i], &yr, &doy, &secs);
    int Column_selection;
    for (size_t j=1000-1; j>=0; j--) {
      int Column_ratio = 
        floor(reflTime[(doy-1)*1000+j] / secondsinceNorth[i]);
      if ( Column_ratio == 0) {
        Column_selection = j;
        break;
      }
    } // j loop
    //cout << i << " " << Column_selection << endl;

    double Current_Refl_Temp[96];
    double Current_Emis_H[96];
    double Current_Emis_V[96];
    double Total_Emis_H = 0.0;
    double Total_Emis_V = 0.0;
    for (size_t j=0; j<96; j++) {
      Current_Refl_Temp[j] = reflTemp[(doy-1)*1000*96+Column_selection*96+j];
      Current_Emis_H[j] = reflEmiss[j*2+1] * 
        (0.00000000332359075944 * pow(Current_Refl_Temp[j], 3) -
         0.00000277123107850697 * pow(Current_Refl_Temp[j], 2) + 
         0.00249949890189042 * Current_Refl_Temp[j] + 0.950924464199886);
      Current_Emis_V[j] = reflEmiss[j*2+0] * 
        (0.00000000332359075944 * pow(Current_Refl_Temp[j], 3) -
         0.00000277123107850697 * pow(Current_Refl_Temp[j], 2) + 
         0.00249949890189042 * Current_Refl_Temp[j] + 0.950924464199886);

      Total_Emis_H += Current_Emis_H[j];
      Total_Emis_V += Current_Emis_V[j];
    } // j loop

    cal_loss1_refl[i][0] = 1 / (1-Total_Emis_V);
    cal_loss1_refl[i][1] = 1 / (1-Total_Emis_H);

    cal_TB1_refl[i][0] = 0.0;
    cal_TB1_refl[i][1] = 0.0;
    for (size_t j=0; j<96; j++) {
      cal_TB1_refl[i][0] += (Current_Refl_Temp[j] + 273.15) * Current_Emis_V[j];
      cal_TB1_refl[i][1] += (Current_Refl_Temp[j] + 273.15) * Current_Emis_H[j];
    } // j loop

    cal_temp1_refl[i][0] = cal_TB1_refl[i][0] / Total_Emis_V;
    cal_temp1_refl[i][1] = cal_TB1_refl[i][1] / Total_Emis_H;

  } // i loop

  delete( reflTemp);
  delete( reflTime);

  return 0;
}


int l1aSMAPFile::calibrationNew_FPS( linearCoeffs_struct *linearCoeffs, 
                                     fullBandCoeffs_struct *fullBandCoeffs, 
                                     subBandCoeffs_struct *subBandCoeffs,
                                     fullCalGroup_struct *fullCalGroup, 
                                     subCalGroup_struct *subCalGroup, 
                                     calDataNoRFI_struct *calDataNoRFI,
                                     uint32_t *highresolution_scan_index, 
                                     uint16_t *footprintsPerScan,
                                     TAs_struct *TAs) {
 
  uint32_t nPRI_fp = 32; // Number of antenna PRIs per footprint
  uint32_t ant_PRIs = 8; // Number of antenna packets per footprint
  uint32_t ref_PKTs = 2; // Number of reference packets per footprint
  uint32_t nPRI_PKT = 4; // Number of PRIs in a packet

  uint32_t npix;
  
  // FullBand
  cout << "   FullBand Calibration" << endl;

  float *m1_ant = new float[this->nscans*(nPRI_ANT*4)];
  float *m2_ant = new float[this->nscans*(nPRI_ANT*4)];
  float *m1_ref = new float[this->nscans*(nPRI_REF*4)];
  float *m1_ref_nd = new float[this->nscans*(nPRI_REF*4)];

  readFloat( this->gid[MOMDATA], "m1_ant", m1_ant);
  readFloat( this->gid[MOMDATA], "m2_ant", m2_ant);
  readFloat( this->gid[MOMDATA], "m1_ref", m1_ref);
  readFloat( this->gid[MOMDATA], "m1_ref_nd", m1_ref_nd);

  double *gainv = new double[this->nscans*(nPRI_REF/4)];
  double *gainh = new double[this->nscans*(nPRI_REF/4)];
  double *offsetv = new double[this->nscans*(nPRI_REF/4)];
  double *offseth = new double[this->nscans*(nPRI_REF/4)];

  double *gain = new double[this->nscans*(nPRI_REF/4)];
  double *T3offset = new double[this->nscans*(nPRI_REF/4)];
  double *T4offset = new double[this->nscans*(nPRI_REF/4)];

  double *m2_antv = new double[this->nscans*nPRI_ANT];
  double *m2_anth = new double[this->nscans*nPRI_ANT];

  for (size_t i=0; i<nscans; i++) {

    double cal_tref[2] =
      {fullCalGroup->cal_tref[i][0],fullCalGroup->cal_tref[i][1]};
    double cal_tnd[2] =
      {fullCalGroup->cal_tnd[i][0],fullCalGroup->cal_tnd[i][1]};
    double cal_temp_det[2] =
      {fullCalGroup->cal_temp_det[i][0],fullCalGroup->cal_temp_det[i][1]};

    uint16_t fp = footprintsPerScan[i];

    // for first scan:
    //  m2_ref      1x1912x4            61184  double              
    //  m1_ref      1x1912x4            61184  double              
    //  m1_ref_nd   1x1912x4            61184  double      

    double m1_ant_dbl[nPRI_ANT*4];
    double m2_ant_dbl[nPRI_ANT*4];
    double m1_ref_dbl[nPRI_REF*4];
    double m1_ref_nd_dbl[nPRI_REF*4];
    double m2_ref_dbl[nPRI_REF*4];
    double m2_ref_nd_dbl[nPRI_REF*4];

    // Set zeros in m2_ref to NaN in m1_ref, Convert to double
    for (size_t j=0; j<fp*ref_PKTs*nPRI_PKT; j++) {
      for (size_t k=0; k<2; k++) {
        uint32_t idx0 = j*4+2*k;
        uint32_t idx1 = j*4+2*k+1;

        if ( calDataNoRFI->m2_ref[i*(nPRI_REF*4)+idx0] == 0) {
          m1_ref_dbl[idx0] = nan("");
          m1_ref_dbl[idx1] = nan("");
        } else {
          m1_ref_dbl[idx0] = (double) m1_ref[i*(nPRI_REF*4)+idx0];
          m1_ref_dbl[idx1] = (double) m1_ref[i*(nPRI_REF*4)+idx1];
        }

        if ( calDataNoRFI->m2_ref_nd[i*(nPRI_REF*4)+idx0] == 0) {
          m1_ref_nd_dbl[idx0] = nan("");
          m1_ref_nd_dbl[idx1] = nan("");
        } else {
          m1_ref_nd_dbl[idx0] = (double) m1_ref_nd[i*(nPRI_REF*4)+idx0];
          m1_ref_nd_dbl[idx1] = (double) m1_ref_nd[i*(nPRI_REF*4)+idx1];
        }
      }
    }

    //  m2_ant      1x7692x4            246144  double
    // Correct to variance
    for (size_t j=0; j<fp*nPRI_fp+last_PRIs; j++) {
      for (size_t k=0; k<4; k++) {
        uint32_t idx = i*(nPRI_ANT*4)+j*4+k;
        m1_ant_dbl[j*4+k] = (double) m1_ant[idx];
        m2_ant_dbl[j*4+k] = (double) m2_ant[idx] - pow(m1_ant_dbl[j*4+k],2);    
      }
    } // j loop

    for (size_t j=0; j<fp*ref_PKTs*nPRI_PKT; j++) {
      for (size_t k=0; k<4; k++) {
        m2_ref_dbl[j*4+k] = 
          (double) calDataNoRFI->m2_ref[i*(nPRI_REF*4)+j*4+k] - 
          pow(m1_ref_dbl[j*4+k],2);

        m2_ref_nd_dbl[j*4+k] = 
          (double) calDataNoRFI->m2_ref_nd[i*(nPRI_REF*4)+j*4+k] - 
          pow(m1_ref_nd_dbl[j*4+k],2);
      }
    } // j loop

    // Linearization Parameters
    double deltaTv = cal_temp_det[0] - linearCoeffs->Tref[0];
    double deltaTh = cal_temp_det[1] - linearCoeffs->Tref[1];
    double c2v = linearCoeffs->C2.c2_0[0] + 
      linearCoeffs->C2.c2_1[0]*deltaTv + 
      linearCoeffs->C2.c2_2[0]*deltaTv*deltaTv;
    double c2h = linearCoeffs->C2.c2_0[1] + 
      linearCoeffs->C2.c2_1[1]*deltaTh + 
      linearCoeffs->C2.c2_2[1]*deltaTh*deltaTh;
    double c3v = linearCoeffs->C3.c3_0[0] + 
      linearCoeffs->C3.c3_1[0]*deltaTv + 
      linearCoeffs->C3.c3_2[0]*deltaTv*deltaTv;
    double c3h = linearCoeffs->C3.c3_0[1] + 
      linearCoeffs->C3.c3_1[1]*deltaTh + 
      linearCoeffs->C3.c3_2[1]*deltaTh*deltaTh;

    // Linearize
    double m2_refh[nPRI_REF];
    double m2_refv[nPRI_REF];
    double m2_ref_ndh[nPRI_REF];
    double m2_ref_ndv[nPRI_REF];
    for (size_t j=0; j<fp*ref_PKTs*nPRI_PKT; j++) {
      m2_refh[j] = m2_ref_dbl[j*4+0] + m2_ref_dbl[j*4+1];
      m2_refv[j] = m2_ref_dbl[j*4+2] + m2_ref_dbl[j*4+3];
      m2_refh[j] += c2h*pow(m2_refh[j],2) + c3h*pow(m2_refh[j],3); 
      m2_refv[j] += c2v*pow(m2_refv[j],2) + c3v*pow(m2_refv[j],3); 

      m2_ref_ndh[j] = m2_ref_nd_dbl[j*4+0] + m2_ref_nd_dbl[j*4+1];
      m2_ref_ndv[j] = m2_ref_nd_dbl[j*4+2] + m2_ref_nd_dbl[j*4+3];
      m2_ref_ndh[j] += c2h*pow(m2_ref_ndh[j],2) + c3h*pow(m2_ref_ndh[j],3); 
      m2_ref_ndv[j] += c2v*pow(m2_ref_ndv[j],2) + c3v*pow(m2_ref_ndv[j],3); 
    } // j loop

    for (size_t j=0; j<fp*nPRI_fp+last_PRIs; j++) {
      uint32_t idx = i*nPRI_ANT+j;
      m2_anth[idx] = m2_ant_dbl[j*4+0] + m2_ant_dbl[j*4+1];
      m2_antv[idx] = m2_ant_dbl[j*4+2] + m2_ant_dbl[j*4+3];
      m2_anth[idx] += c2h*pow(m2_anth[idx],2) + c3h*pow(m2_anth[idx],3); 
      m2_antv[idx] += c2v*pow(m2_antv[idx],2) + c3v*pow(m2_antv[idx],3); 
    } // j loop

    // Average every 4 PRIs for Ref
    for (size_t j=0; j<fp*ref_PKTs*(nPRI_PKT/4); j++) {
      int cnt;
      double sum;
      double *dptr[4] = {m2_refh, m2_refv, m2_ref_ndh, m2_ref_ndv};

      for (size_t l=0; l<4; l++) {  
        cnt = 0;
        sum = 0.0;
        for (size_t k=0; k<4; k++) {
          if ( !isnan(*(dptr[l]+j*4+k))) { 
            cnt++;
            sum = sum + *(dptr[l]+j*4+k);
          }
        } // k loop
        *(dptr[l]+j) = sum / cnt;
      } // l loop
    } // j loop
 
    // Code for replacement of NaN with nearest neighbor
    npix = fp*ref_PKTs*nPRI_PKT;
    replaceNaN( m2_refv, m2_refh, m2_ref_ndv, m2_ref_ndh, npix, 1);
    
    // Compute gains & offsets
    for (size_t j=0; j<(nPRI_REF/4); j++) {
      uint32_t idx = i*(nPRI_REF/4)+j;

      if ( j < fp*ref_PKTs) {
        gainv[idx] = (m2_ref_ndv[j] - m2_refv[j]) / cal_tnd[0];
        gainh[idx] = (m2_ref_ndh[j] - m2_refh[j]) / cal_tnd[1];

        if ( isnan( gainv[idx])) {
          cout << "gainv[] is NaN: " << idx << " " <<
            cal_tnd[0] << " " << cal_tnd[1] << " " <<
            m2_ref_ndv[j] << " " << m2_refv[j] << endl;
          exit(1);
        }

        offsetv[idx] = m2_refv[j] - gainv[idx] * cal_tref[0];
        offseth[idx] = m2_refh[j] - gainh[idx] * cal_tref[1];
      } else {
        gainv[idx] = -9999.0;
        gainh[idx] = -9999.0;
        offsetv[idx] = -9999.0;
        offseth[idx] = -9999.0;
      }
    } // j loop


    // Fullband T3T4
    double t3_ref[nPRI_REF];
    double t4_ref[nPRI_REF];
    double t3_ref_nd[nPRI_REF];
    double t4_ref_nd[nPRI_REF];
    for (size_t j=0; j<fp*ref_PKTs*nPRI_PKT; j++) {
      t3_ref[j] = (double) calDataNoRFI->t3_ref[i*nPRI_REF+j];
      t4_ref[j] = (double) calDataNoRFI->t4_ref[i*nPRI_REF+j];
      t3_ref_nd[j] = (double) calDataNoRFI->t3_ref_nd[i*nPRI_REF+j];
      t4_ref_nd[j] = (double) calDataNoRFI->t4_ref_nd[i*nPRI_REF+j];
    } // j loop

    // Average every 4 PRIs for Ref
    for (size_t j=0; j<fp*ref_PKTs*(nPRI_PKT/4); j++) {
      int cnt;
      double sum;
      double *dptr[4] = {t3_ref, t4_ref, t3_ref_nd, t4_ref_nd};

      for (size_t l=0; l<4; l++) {  
        cnt = 0;
        sum = 0.0;
        for (size_t k=0; k<4; k++) {
          if ( !isnan(*(dptr[l]+j*4+k))) { 
            cnt++;
            sum = sum + *(dptr[l]+j*4+k);
          }
        } // k loop
        *(dptr[l]+j) = sum / cnt;
      } // l loop
    } // j loop

    // Code for replacement of NaN with nearest neighbor
    replaceNaN( t3_ref, t4_ref, t3_ref_nd, t4_ref_nd, npix, 1);
    
    double cal_tnd34 = fullCalGroup->cal_tnd34[i];
    double cal_nd_phase = fullCalGroup->cal_nd_phase[i];
    double cal_phase = fullCalGroup->cal_phase[i];
    for (size_t j=0; j<(nPRI_REF/4); j++) {

      if ( j < fp*ref_PKTs) {
        gain[i*(nPRI_REF/4)+j] = 
          ((t3_ref_nd[j] - t3_ref[j]) * cos(cal_phase - cal_nd_phase) -
           (t4_ref_nd[j] - t4_ref[j]) * sin(cal_phase - cal_nd_phase)) /
          cal_tnd34;

        T3offset[i*(nPRI_REF/4)+j] = 
          (t3_ref[j] * cos(cal_phase) - t4_ref[j] * sin(cal_phase)) /
          gain[i*(nPRI_REF/4)+j];

        T4offset[i*(nPRI_REF/4)+j] = 
          (t3_ref[j] * sin(cal_phase) + t4_ref[j] * cos(cal_phase)) /
          gain[i*(nPRI_REF/4)+j];
      } else {
        gain[i*(nPRI_REF/4)+j] = -9999.0;
        T3offset[i*(nPRI_REF/4)+j] = -9999.0;
        T4offset[i*(nPRI_REF/4)+j] = -9999.0;
      }

    } // j loop

  } // i loop (FullBand)

  delete( m1_ant);
  delete( m2_ant);
  delete( m1_ref);
  delete( m1_ref_nd);


  // Filtered Gains/Offsets
  cout << "   Filtering FullBand Gains/Offsets" << endl;

  double *Gainv = new double[nscans*(nPRI_ANT-last_PRIs)];
  double *Gainh = new double[nscans*(nPRI_ANT-last_PRIs)];
  double *Gain = new double[nscans*(nPRI_ANT-last_PRIs)];
  double *Offsetv = new double[nscans*(nPRI_ANT-last_PRIs)];
  double *Offseth = new double[nscans*(nPRI_ANT-last_PRIs)];
  double *T3Offset = new double[nscans*(nPRI_ANT-last_PRIs)];
  double *T4Offset = new double[nscans*(nPRI_ANT-last_PRIs)];

  for (size_t i=0; i<nscans; i++) {

    // Set fill value
    for (size_t j=0; j<(nPRI_ANT-last_PRIs); j++) {
      Gainv[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      Gainh[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      Gain[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      Offsetv[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      Offseth[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      T3Offset[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      T4Offset[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
    }

    gainFilter( nscans, i, footprintsPerScan, 5295, linearCoeffs->h,
                gainv, gainh, gain, offsetv, offseth, T3offset, T4offset,
                &Gainv[i*(nPRI_ANT-last_PRIs)], 
                &Gainh[i*(nPRI_ANT-last_PRIs)],
                &Gain[i*(nPRI_ANT-last_PRIs)],
                &Offsetv[i*(nPRI_ANT-last_PRIs)], 
                &Offseth[i*(nPRI_ANT-last_PRIs)],
                &T3Offset[i*(nPRI_ANT-last_PRIs)], 
                &T4Offset[i*(nPRI_ANT-last_PRIs)]);

  } // i loop

  delete( gainv);
  delete( gainh);
  delete( offsetv);
  delete( offseth);
  delete( gain);
  delete( T3offset);
  delete( T4offset);

  // Compute FullBand TAs
  cout << "   Compute FullBand TAs" << endl;

  TAs->TA_v = new float[this->nscans*(nPRI_ANT-last_PRIs)];
  TAs->TA_h = new float[this->nscans*(nPRI_ANT-last_PRIs)];
  TAs->Gainv = new float[this->nscans*(nPRI_ANT-last_PRIs)];
  TAs->Gainh = new float[this->nscans*(nPRI_ANT-last_PRIs)];
  TAs->Offsetv = new float[this->nscans*(nPRI_ANT-last_PRIs)];
  TAs->Offseth = new float[this->nscans*(nPRI_ANT-last_PRIs)];
  TAs->T3 = new float[this->nscans*(nPRI_ANT-last_PRIs)];
  TAs->T4 = new float[this->nscans*(nPRI_ANT-last_PRIs)];

  double *t3_ant = new double[this->nscans*nPRI_ANT];
  double *t4_ant = new double[this->nscans*nPRI_ANT];
  readDouble( this->gid[MOMDATA], "t3_ant", t3_ant);
  readDouble( this->gid[MOMDATA], "t4_ant", t4_ant);

  for (size_t i=0; i<nscans; i++) {
    for (size_t j=0; j<nPRI_ANT-last_PRIs; j++) {
      TAs->TA_v[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      TAs->TA_h[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      TAs->T3[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      TAs->T4[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      TAs->Gainv[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      TAs->Gainh[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      TAs->Offsetv[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
      TAs->Offseth[i*(nPRI_ANT-last_PRIs)+j] = -9999.0;
    }

    for (size_t j=0; j<footprintsPerScan[i]*nPRI_fp; j++) {
      uint32_t ant_idx = i*nPRI_ANT+j;
      uint32_t go_idx = i*(nPRI_ANT-last_PRIs)+j;

      float cal_loss4_coupler[2];
      float cal_loss5_diplexer[2];
      float cal_temp4_coupler[2];
      float cal_temp_ref[2];
      float cal_tcor_re[2];
      float cal_tcor_im[2];
      float cal_lambda[2];
      float cal_gamma[2];
      float cal_loss12_radome[2];
      float cal_loss3_omt[2];
      float cal_temp3_omt[2];
      float cal_loss2_feed;
      float cal_temp12_radome;
      float cal_temp2_feed;
      float cal_phase;
      float cal_psi;
      for (size_t k=0; k<2; k++) {
        cal_loss4_coupler[k] = fullCalGroup->cal_loss4_coupler[i][k];
        cal_loss5_diplexer[k] = fullCalGroup->cal_loss5_diplexer[i][k];
        cal_temp4_coupler[k] = fullCalGroup->cal_temp4_coupler[i][k];
        cal_temp_ref[k] = fullCalGroup->cal_temp_ref[i][k];
        cal_tcor_re[k] = fullCalGroup->cal_tcor_re[i][k];
        cal_tcor_im[k] = fullCalGroup->cal_tcor_im[i][k];
        cal_lambda[k] = fullCalGroup->cal_lambda[i][k];
        cal_gamma[k] = fullCalGroup->cal_gamma[i][k];
        cal_loss12_radome[k] = fullCalGroup->cal_loss12_radome[i][k];
        cal_loss3_omt[k] = fullCalGroup->cal_loss3_omt[i][k];
        cal_temp3_omt[k] = fullCalGroup->cal_temp3_omt[i][k];
      } // k loop
      cal_loss2_feed = fullCalGroup->cal_loss2_feed[i];
      cal_temp12_radome = fullCalGroup->cal_temp12_radome[i];
      cal_temp2_feed = fullCalGroup->cal_temp2_feed[i];
      cal_phase  = fullCalGroup->cal_phase[i];
      cal_psi  = fullCalGroup->cal_psi[i];

      // TA_v

      // compute TA up to the RFE
      double TA_RFEv = (m2_antv[ant_idx] - Offsetv[go_idx]) / Gainv[go_idx];

      // include losses
      double TA_CPv = 
        cal_loss4_coupler[0] * cal_loss5_diplexer[0] * TA_RFEv -
        (cal_loss4_coupler[0]-1) * cal_temp4_coupler[0];

      // Equation 4.18 in ATBD
      double Tcor_v_re =  -cal_temp_ref[0] * cal_tcor_re[0];
      double Tcor_v_im =  -cal_temp_ref[0] * cal_tcor_im[0];

      // New equation, eq. 5.10 in ATBD, add gamma, tcor to IPF
      // these along with cal_lambda are complex values
      double TA_CPvhat = TA_CPv - 
        2 * (cal_lambda[0] * cal_gamma[0] * Tcor_v_re) / 
        powf(cal_lambda[0],2) - cal_temp_ref[0] * powf(cal_gamma[0],2);

      // Modified to include radome
      double TAv = cal_loss12_radome[0] * 
        cal_loss2_feed * cal_loss3_omt[0] * TA_CPvhat - 
        (cal_loss12_radome[0]-1) * cal_temp12_radome - 
        cal_loss12_radome[0] * (cal_loss2_feed-1) * cal_temp2_feed - 
        cal_loss12_radome[0] * cal_loss2_feed * 
        (cal_loss3_omt[0]-1) * cal_temp3_omt[0];

      TAs->TA_v[go_idx] = (float) TAv;
      TAs->Gainv[go_idx] = (float) Gainv[go_idx];
      TAs->Offsetv[go_idx] = (float) Offsetv[go_idx];


      // TA_h
      double TA_RFEh = (m2_anth[ant_idx] - Offseth[go_idx]) / Gainh[go_idx];

      double TA_CPh = 
        cal_loss4_coupler[1] * cal_loss5_diplexer[1] * TA_RFEh -
        (cal_loss4_coupler[1]-1) * cal_temp4_coupler[1];

      double Tcor_h_re =  -cal_temp_ref[1] * cal_tcor_re[1];
      double Tcor_h_im =  -cal_temp_ref[1] * cal_tcor_im[1];

      double TA_CPhhat = TA_CPh - 
        2 * (cal_lambda[1] * cal_gamma[1] * Tcor_h_re) / 
        powf(cal_lambda[1],2) - cal_temp_ref[1] * powf(cal_gamma[1],2);

      double TAh = cal_loss12_radome[1] * 
        cal_loss2_feed * cal_loss3_omt[1] * TA_CPhhat - 
        (cal_loss12_radome[1]-1) * cal_temp12_radome - 
        cal_loss12_radome[1] * (cal_loss2_feed-1) * cal_temp2_feed - 
        cal_loss12_radome[1] * cal_loss2_feed * 
        (cal_loss3_omt[1]-1) * cal_temp3_omt[1];

      TAs->TA_h[i*(nPRI_ANT-last_PRIs)+j] = (float) TAh;
      TAs->Gainh[i*(nPRI_ANT-last_PRIs)+j] = (float) Gainh[go_idx];
      TAs->Offseth[i*(nPRI_ANT-last_PRIs)+j] = (float) Offseth[go_idx];

      // T3T4
      double T3_RFE = 
        (t3_ant[ant_idx] * cos(cal_phase) - t4_ant[ant_idx] * sin(cal_phase)) /
        Gain[go_idx] - T3Offset[go_idx];
      double T4_RFE = 
        (t3_ant[ant_idx] * sin(cal_phase) + t4_ant[ant_idx] * cos(cal_phase)) /
        Gain[go_idx] - T4Offset[go_idx];
    
      // Added cal_loss12_radome
      double T34_losses = 
        sqrt(cal_loss12_radome[0] * cal_loss2_feed * cal_loss3_omt[0] * 
             cal_loss4_coupler[0] * cal_loss5_diplexer[0] *
             cal_loss12_radome[1] * cal_loss2_feed * cal_loss3_omt[1] *
             cal_loss4_coupler[1] * cal_loss5_diplexer[1]) /
        abs(cal_lambda[0] * powf(cal_lambda[1],2.0));
    
      double T3_hat1 =  cos(cal_psi) * cal_lambda[0] * cal_lambda[1] * T3_RFE;
      double T3_hat2 = -sin(cal_psi) * cal_lambda[0] * cal_lambda[1] * T4_RFE;
      double T3hat = T34_losses * (T3_hat1 + T3_hat2);
      TAs->T3[go_idx] = (float) T3hat;
    
      double T4_hat1 = sin(cal_psi) * cal_lambda[0] * cal_lambda[1] * T3_RFE;
      double T4_hat2 = cos(cal_psi) * cal_lambda[0] * cal_lambda[1] * T4_RFE;
      double T4hat = T34_losses * (T4_hat1 + T4_hat2);
      TAs->T4[go_idx] = (float) T4hat;

    } // j loop

  } // i loop

  delete( Gainv);
  delete( Gainh);
  delete( Gain);
  delete( Offsetv);
  delete( Offseth);
  delete( T3Offset);
  delete( T4Offset);
  delete( m2_antv);
  delete( m2_anth);
  delete( t3_ant);
  delete( t4_ant);

  // SubBand
  cout << "   SubBand Calibration" << endl;

  float *m1_16_ant = new float[this->nhires*(nPRI_ANT/4)*16*4];
  float *m2_16_ant = new float[this->nhires*(nPRI_ANT/4)*16*4];
  float *m1_16_ref = new float[this->nhires*(nPRI_REF/4)*16*4];
  float *m1_16_ref_nd = new float[this->nhires*(nPRI_REF/4)*16*4];

  readFloat( this->gid[HIRESMOM], "m1_16_ant", m1_16_ant);
  readFloat( this->gid[HIRESMOM], "m2_16_ant", m2_16_ant);
  readFloat( this->gid[HIRESMOM], "m1_16_ref", m1_16_ref);
  readFloat( this->gid[HIRESMOM], "m1_16_ref_nd", m1_16_ref_nd);

  double *gainv_16 = new double[this->nhires*(nPRI_REF/4)*16];
  double *gainh_16 = new double[this->nhires*(nPRI_REF/4)*16];
  double *offsetv_16 = new double[this->nhires*(nPRI_REF/4)*16];
  double *offseth_16 = new double[this->nhires*(nPRI_REF/4)*16];

  double *gain_16 = new double[this->nhires*(nPRI_REF/4)*16];
  double *T3offset_16 = new double[this->nhires*(nPRI_REF/4)*16];
  double *T4offset_16 = new double[this->nhires*(nPRI_REF/4)*16];

  double *m2_16_antv = new double[this->nhires*(nPRI_ANT/4)*16];
  double *m2_16_anth = new double[this->nhires*(nPRI_ANT/4)*16];

  for (size_t i=0; i<nhires; i++) {

    size_t hr_idx = highresolution_scan_index[i];

    double cal_tref16[16][2];
    double cal_tnd16[16][2];
    //    double cal_temp_det16[16][2];
    for (size_t j=0; j<16; j++) {
      for (size_t k=0; k<2; k++) {
        cal_tref16[j][k] = subCalGroup->cal_tref16[hr_idx][j][k];
        cal_tnd16[j][k] = subCalGroup->cal_tnd16[hr_idx][j][k];
        //  cal_temp_det16[j][k] = subCalGroup->cal_temp_det16[hr_idx][j][k];
      } // k loop
    } // j loop

    uint16_t fp = footprintsPerScan[highresolution_scan_index[i]];

    double m1_16_ant_dbl[(nPRI_ANT/4)*16*4];
    double m2_16_ant_dbl[(nPRI_ANT/4)*16*4];
    double m1_16_ref_dbl[(nPRI_REF/4)*16*4];
    double m1_16_ref_nd_dbl[(nPRI_REF/4)*16*4];
    double m2_16_ref_dbl[(nPRI_REF/4)*16*4];
    double m2_16_ref_nd_dbl[(nPRI_REF/4)*16*4];

    // Set zeros in m2_16_ref to NaN in m1_16_ref, Convert to double
    for (size_t j=0; j<fp*ref_PKTs; j++) {
      for (size_t l=0; l<16; l++) {
        for (size_t k=0; k<2; k++) {
          uint32_t idx0 = j*16*4+l*4+2*k;
          uint32_t idx1 = j*16*4+l*4+2*k+1;

          if ( calDataNoRFI->m2_16_ref[i*(nPRI_REF/4)*16*4+idx0] == 0) {
            m1_16_ref_dbl[idx0] = nan("");
            m1_16_ref_dbl[idx1] = nan("");
          } else {
            m1_16_ref_dbl[idx0] = (double) m1_16_ref[i*(nPRI_REF/4)*16*4+idx0];
            m1_16_ref_dbl[idx1] = (double) m1_16_ref[i*(nPRI_REF/4)*16*4+idx1];
          }

          if ( calDataNoRFI->m2_16_ref_nd[i*(nPRI_REF/4)*16*4+idx0] == 0) {
            m1_16_ref_nd_dbl[idx0] = nan("");
            m1_16_ref_nd_dbl[idx1] = nan("");
          } else {
            m1_16_ref_nd_dbl[idx0] = 
              (double) m1_16_ref_nd[i*(nPRI_REF/4)*16*4+idx0];
            m1_16_ref_nd_dbl[idx1] = 
              (double) m1_16_ref_nd[i*(nPRI_REF/4)*16*4+idx1];
          }
        } // k loop
      } // l loop
    } // j loop

    // Correct to variance
    for (size_t j=0; j<fp*ant_PRIs+11; j++) {
      for (size_t l=0; l<16; l++) {
        for (size_t k=0; k<4; k++) {
          uint32_t idx = i*(nPRI_ANT/4)*16*4+j*16*4+l*4+k;
          m1_16_ant_dbl[j*16*4+l*4+k] = (double) m1_16_ant[idx];
          m2_16_ant_dbl[j*16*4+l*4+k] = 
            (double) m2_16_ant[idx] - pow(m1_16_ant_dbl[j*16*4+l*4+k],2);
        } // k loop
      } // l loop
    } // j loop

    for (size_t j=0; j<fp*ref_PKTs; j++) {
      for (size_t l=0; l<16; l++) {
        for (size_t k=0; k<4; k++) {
          uint32_t idx = j*16*4+l*4+k;
          m2_16_ref_dbl[idx] = 
            (double) calDataNoRFI->m2_16_ref[i*(nPRI_REF/4)*16*4+idx] - 
            pow(m1_16_ref_dbl[idx],2);

          m2_16_ref_nd_dbl[idx] = 
            (double) calDataNoRFI->m2_16_ref_nd[i*(nPRI_REF/4)*16*4+idx] - 
            pow(m1_16_ref_nd_dbl[idx],2);
        } // k loop
      } // l loop
    } // j loop

    // Linearization Parameters
    double deltaTv16[16];
    double deltaTh16[16];
    double c2v16[16], c2h16[16], c3v16[16], c3h16[16];
    for (size_t l=0; l<16; l++) {
     deltaTv16[l] = 
       subCalGroup->cal_temp_det16[hr_idx][l][0] - linearCoeffs->Tref16[l][0];
     deltaTh16[l] = 
       subCalGroup->cal_temp_det16[hr_idx][l][1] - linearCoeffs->Tref16[l][1];

     c2v16[l] = linearCoeffs->C2_16.c2_0[0] + 
       linearCoeffs->C2_16.c2_1[0]*deltaTv16[l] + 
       linearCoeffs->C2_16.c2_2[0]*deltaTv16[l]*deltaTv16[l];

     c2h16[l] = linearCoeffs->C2_16.c2_0[1] + 
       linearCoeffs->C2_16.c2_1[1]*deltaTh16[l] + 
       linearCoeffs->C2_16.c2_2[1]*deltaTh16[l]*deltaTh16[l];

     c3v16[l] = linearCoeffs->C3_16.c3_0[0] + 
       linearCoeffs->C3_16.c3_1[0]*deltaTv16[l] + 
       linearCoeffs->C3_16.c3_2[0]*deltaTv16[l]*deltaTv16[l];

     c3h16[l] = linearCoeffs->C3_16.c3_0[1] + 
       linearCoeffs->C3_16.c3_1[1]*deltaTh16[l] + 
       linearCoeffs->C3_16.c3_2[1]*deltaTh16[l]*deltaTh16[l];
    } // l loop

    // Linearize
    double m2_16_refh[(nPRI_REF/4)*16];
    double m2_16_refv[(nPRI_REF/4)*16];
    double m2_16_ref_ndh[(nPRI_REF/4)*16];
    double m2_16_ref_ndv[(nPRI_REF/4)*16];
    double temp;
    for (size_t j=0; j<fp*ref_PKTs; j++) {
      for (size_t l=0; l<16; l++) {
        uint32_t idx = j*16*4+l*4;
        m2_16_refh[j*16+l] = m2_16_ref_dbl[idx+0] + m2_16_ref_dbl[idx+1];
        m2_16_refv[j*16+l] = m2_16_ref_dbl[idx+2] + m2_16_ref_dbl[idx+3];

        temp = m2_16_refh[j*16+l];
        m2_16_refh[j*16+l] += c2h16[l]*pow(temp,2) + c3h16[l]*pow(temp,3);
        temp = m2_16_refv[j*16+l];
        m2_16_refv[j*16+l] += c2v16[l]*pow(temp,2) + c3v16[l]*pow(temp,3);

        m2_16_ref_ndh[j*16+l] = 
          m2_16_ref_nd_dbl[idx+0] + m2_16_ref_nd_dbl[idx+1];
        m2_16_ref_ndv[j*16+l] = 
          m2_16_ref_nd_dbl[idx+2] + m2_16_ref_nd_dbl[idx+3];

        temp = m2_16_ref_ndh[j*16+l];
        m2_16_ref_ndh[j*16+l] += c2h16[l]*pow(temp,2) + c3h16[l]*pow(temp,3);
        temp = m2_16_ref_ndv[j*16+l];
        m2_16_ref_ndv[j*16+l] += c2v16[l]*pow(temp,2) + c3v16[l]*pow(temp,3);
      } // l loop
    } // j loop

    for (size_t j=0; j<fp*ant_PRIs+11; j++) {
      for (size_t l=0; l<16; l++) {
        uint32_t idx0 = i*(nPRI_ANT/4)*16+j*16+l;
        uint32_t idx = j*16*4+l*4;
        m2_16_anth[idx0] = m2_16_ant_dbl[idx+0] + m2_16_ant_dbl[idx+1];
        m2_16_antv[idx0] = m2_16_ant_dbl[idx+2] + m2_16_ant_dbl[idx+3];

        temp = m2_16_anth[idx0];
        m2_16_anth[idx0] += c2h16[l]*pow(temp,2) + c3h16[l]*pow(temp,3);
        temp = m2_16_antv[idx0];
        m2_16_antv[idx0] += c2v16[l]*pow(temp,2) + c3v16[l]*pow(temp,3);
      } // l loop
    } // j loop


    npix = fp*ref_PKTs;
    replaceNaN( m2_16_refv, m2_16_refh, m2_16_ref_ndv, m2_16_ref_ndh, npix, 16);

    // x /4fg &m1_16_ref_dbl[6*16*4+1*4] = m1_16_ref(1,7,2,:)
    // x /16fg &m2_16_refv[6*16] = m2_16.m2_16_refv(1,7,:)

    // Compute gains & offsets
    for (size_t j=0; j<nPRI_REF/4; j++) {
      for (size_t l=0; l<16; l++) {
        uint32_t idx = i*(nPRI_REF/4)*16+j*16+l;

        if (idx >= this->nhires*(nPRI_REF/4)*16) {
          cout << i << " " << j << " " << l << endl;
          exit(1);
        }
            
        if ( j < fp*ref_PKTs) {
          gainv_16[idx] = 
            (m2_16_ref_ndv[j*16+l] - m2_16_refv[j*16+l]) / cal_tnd16[l][0];
          gainh_16[idx] = 
            (m2_16_ref_ndh[j*16+l] - m2_16_refh[j*16+l]) / cal_tnd16[l][1];

          offsetv_16[idx] = 
            m2_16_refv[j*16+l] - gainv_16[idx] * cal_tref16[l][0];
          offseth_16[idx] = 
            m2_16_refh[j*16+l] - gainh_16[idx] * cal_tref16[l][1];
        } else {
          gainv_16[idx] = -9999.0;
          gainh_16[idx] = -9999.0;
          offsetv_16[idx] = -9999.0;
          offseth_16[idx] = -9999.0;
        }

      } // l loop
    } // j loop

 
    // Subband T3T4
    double t3_16_ref[(nPRI_REF/4)*16];
    double t4_16_ref[(nPRI_REF/4)*16];
    double t3_16_ref_nd[(nPRI_REF/4)*16];
    double t4_16_ref_nd[(nPRI_REF/4)*16];
    for (size_t j=0; j<fp*ref_PKTs; j++) {
      for (size_t l=0; l<16; l++) {
        uint32_t idx = i*(nPRI_REF/4)*16+j*16+l;

        t3_16_ref[j*16+l] = (double) calDataNoRFI->t3_16_ref[idx];
        t4_16_ref[j*16+l] = (double) calDataNoRFI->t4_16_ref[idx];
        t3_16_ref_nd[j*16+l] = (double) calDataNoRFI->t3_16_ref_nd[idx];
        t4_16_ref_nd[j*16+l] = (double) calDataNoRFI->t4_16_ref_nd[idx];
      } // l loop
    } // j loop

    replaceNaN( t3_16_ref, t4_16_ref, t3_16_ref_nd, t4_16_ref_nd, npix, 16);

    double *dptr[4] = {t3_16_ref, t4_16_ref, t3_16_ref_nd, t4_16_ref_nd};
    for (size_t k=0; k<4; k++) {
      for (size_t j=0; j<fp*ref_PKTs; j++) {
        for (size_t l=0; l<16; l++) {
          if ( isnan(dptr[k][j*16+l])) {
            cout << "here" << endl;
          }
        }
      }
    }

    double cal_tnd34_16[(nPRI_REF/4)*16];
    double cal_nd_phase16[(nPRI_REF/4)*16];
    double cal_phase16[(nPRI_REF/4)*16];
    for (size_t j=0; j<nPRI_REF/4; j++) {
      for (size_t l=0; l<16; l++) {
        uint32_t idx = i*(nPRI_REF/4)*16+j*16+l;

        cal_tnd34_16[j*16+l] = subCalGroup->cal_tnd34_16[hr_idx][l];
        cal_nd_phase16[j*16+l] = subCalGroup->cal_nd_phase16[hr_idx][l];
        cal_phase16[j*16+l] = subCalGroup->cal_phase16[hr_idx][l];

        if ( j < fp*ref_PKTs) {
          gain_16[idx] = ((t3_16_ref_nd[j*16+l] - t3_16_ref[j*16+l]) * 
                          cos(cal_phase16[j*16+l] - cal_nd_phase16[j*16+l]) -
                          (t4_16_ref_nd[j*16+l] - t4_16_ref[j*16+l]) * 
                          sin(cal_phase16[j*16+l] - cal_nd_phase16[j*16+l])) /
            cal_tnd34_16[j*16+l];

          T3offset_16[idx] = (t3_16_ref[j*16+l] * cos(cal_phase16[j*16+l]) - 
                              t4_16_ref[j*16+l] * sin(cal_phase16[j*16+l])) /
            gain_16[idx];

          T4offset_16[idx] = (t3_16_ref[j*16+l] * sin(cal_phase16[j*16+l]) + 
                              t4_16_ref[j*16+l] * cos(cal_phase16[j*16+l])) /
            gain_16[idx];
        } else {
          gain_16[idx] = -9999.0;
          T3offset_16[idx] = -9999.0;
          T4offset_16[idx] = -9999.0;
        }

      } // l loop
    } // j loop

  } // i loop

  /*
  ofstream gainOut;
  gainOut.open ("gain.bin", ios::out | ios::trunc | ios::binary);
  gainOut.write((char *) gainv, nscans*(nPRI_REF/4)*sizeof(double));
  gainOut.write((char *) gainh, nscans*(nPRI_REF/4)*sizeof(double));
  gainOut.write((char *) offsetv, nscans*(nPRI_REF/4)*sizeof(double));
  gainOut.write((char *) offseth, nscans*(nPRI_REF/4)*sizeof(double));
  gainOut.write((char *) gainv_16, nscans*(nPRI_REF/4)*16*sizeof(double));
  gainOut.write((char *) gainh_16, nscans*(nPRI_REF/4)*16*sizeof(double));
  gainOut.write((char *) offsetv_16, nscans*(nPRI_REF/4)*16*sizeof(double));
  gainOut.write((char *) offseth_16, nscans*(nPRI_REF/4)*16*sizeof(double));
  gainOut.write((char *) gain, nscans*(nPRI_REF/4)*sizeof(double));
  gainOut.write((char *) gain_16, nscans*(nPRI_REF/4)*16*sizeof(double));
  gainOut.write((char *) T3offset, nscans*(nPRI_REF/4)*sizeof(double));
  gainOut.write((char *) T4offset, nscans*(nPRI_REF/4)*sizeof(double));
  gainOut.write((char *) T3offset_16, nscans*(nPRI_REF/4)*16*sizeof(double));
  gainOut.write((char *) T4offset_16, nscans*(nPRI_REF/4)*16*sizeof(double));
  gainOut.close();
  */

  delete( m1_16_ant);
  delete( m2_16_ant);
  delete( m1_16_ref);
  delete( m1_16_ref_nd);

  // Filtered Gains/Offsets
  cout << "   Filtering SubBand Gains/Offsets" << endl;

  double *Gainv16 = new double[nhires*nPRI_REF*16];
  double *Gainh16 = new double[nhires*nPRI_REF*16];
  double *Gain16 = new double[nhires*nPRI_REF*16];
  double *Offsetv16 = new double[nhires*nPRI_REF*16];
  double *Offseth16 = new double[nhires*nPRI_REF*16];
  double *T3Offset16 = new double[nhires*nPRI_REF*16];
  double *T4Offset16 = new double[nhires*nPRI_REF*16];

  double scan_diff1[MAXSCANS];
  double scan_diff2[MAXSCANS];

  scan_diff1[0] = linearCoeffs->time_gap;
  for (size_t i=1; i<nhires; i++) 
    scan_diff1[i] = 
      highresolution_scan_index[i] - highresolution_scan_index[i-1]; 

  for (size_t i=0; i<nhires-1; i++) 
    scan_diff2[i] = 
      highresolution_scan_index[i+1] - highresolution_scan_index[i]; 
  scan_diff2[nhires-1] = linearCoeffs->time_gap;

  // This loop takes a long time
  for (size_t i=0; i<nhires; i++) {
    if ((i % 100) == 0) cout << "        Scan " << i << endl;

    // Set fill value
    for (size_t j=0; j<nPRI_REF*16; j++) {
      Gainv16[i*nPRI_REF*16+j] = -9999.0;
      Gainh16[i*nPRI_REF*16+j] = -9999.0;
      Gain16[i*nPRI_REF*16+j] = -9999.0;
      Offsetv16[i*nPRI_REF*16+j] = -9999.0;
      Offseth16[i*nPRI_REF*16+j] = -9999.0;
      T3Offset16[i*nPRI_REF*16+j] = -9999.0;
      T4Offset16[i*nPRI_REF*16+j] = -9999.0;
    }

    gainFilter16( nhires, i, highresolution_scan_index, footprintsPerScan,
                  5295, linearCoeffs->h,
                  linearCoeffs->time_gap, scan_diff1, scan_diff2,
                  gainv_16, gainh_16, gain_16, offsetv_16, 
                  offseth_16, T3offset_16, T4offset_16,
                  &Gainv16[i*nPRI_REF*16], &Gainh16[i*nPRI_REF*16],
                  &Gain16[i*nPRI_REF*16],
                  &Offsetv16[i*nPRI_REF*16], &Offseth16[i*nPRI_REF*16],
                  &T3Offset16[i*nPRI_REF*16], &T4Offset16[i*nPRI_REF*16]);

  } // i loop

  delete( gainv_16);
  delete( gainh_16);
  delete( offsetv_16);
  delete( offseth_16);
  delete( gain_16);
  delete( T3offset_16);
  delete( T4offset_16);

  // Compute SubBand TAs
  cout << "   Compute SubBand TAs" << endl;

  TAs->TA16_v = new float[this->nhires*((nPRI_ANT/4)-last_PKTs)*16];
  TAs->TA16_h = new float[this->nhires*((nPRI_ANT/4)-last_PKTs)*16];
  TAs->Gainv16 = new float[this->nhires*((nPRI_ANT/4)-last_PKTs)*16];
  TAs->Gainh16 = new float[this->nhires*((nPRI_ANT/4)-last_PKTs)*16];
  TAs->Offsetv16 = new float[this->nhires*((nPRI_ANT/4)-last_PKTs)*16];
  TAs->Offseth16 = new float[this->nhires*((nPRI_ANT/4)-last_PKTs)*16];
  TAs->T3_16 = new float[this->nhires*((nPRI_ANT/4)-last_PKTs)*16];
  TAs->T4_16 = new float[this->nhires*((nPRI_ANT/4)-last_PKTs)*16];

  for (size_t i=0; i<nhires; i++) {
    for (size_t j=0; j<nPRI_REF; j++) {
      for (size_t l=0; l<16; l++) {
      TAs->TA16_v[i*nPRI_REF*16+j*16+l] = -9999.0;
      TAs->TA16_h[i*nPRI_REF*16+j*16+l] = -9999.0;
      TAs->T3_16[i*nPRI_REF*16+j*16+l] = -9999.0;
      TAs->T4_16[i*nPRI_REF*16+j*16+l] = -9999.0;
      TAs->Gainv16[i*nPRI_REF*16+j*16+l] = -9999.0;
      TAs->Gainh16[i*nPRI_REF*16+j*16+l] = -9999.0;
      TAs->Offsetv16[i*nPRI_REF*16+j*16+l] = -9999.0;
      TAs->Offseth16[i*nPRI_REF*16+j*16+l] = -9999.0;
      }
    }
  }

  double *t3_16_ant = new double[this->nhires*(nPRI_ANT/4)*16];
  double *t4_16_ant = new double[this->nhires*(nPRI_ANT/4)*16];

  readDouble( this->gid[HIRESMOM], "t3_16_ant", t3_16_ant);
  readDouble( this->gid[HIRESMOM], "t4_16_ant", t4_16_ant);

  for (size_t i=0; i<nhires; i++) {
    
    size_t hr_idx = highresolution_scan_index[i];

    uint16_t fp = footprintsPerScan[hr_idx];

    for (size_t j=0; j<fp*ant_PRIs; j++) {
      for (size_t l=0; l<16; l++) {
        uint32_t ant_idx = i*(nPRI_ANT/4)*16+j*16+l;
        uint32_t go_idx = i*((nPRI_ANT/4)-last_PKTs)*16+j*16+l;

        float cal_loss4_coupler16[2];
        float cal_loss5_diplexer16[2];
        float cal_temp4_coupler16[2];
        float cal_temp_ref16[2];
        float cal_tcor16_re[2];
        float cal_tcor16_im[2];
        float cal_lambda16[2];
        float cal_gamma16[2];
        float cal_loss12_radome16[2];
        float cal_loss3_omt16[2];
        float cal_temp3_omt16[2];
        float cal_loss2_feed16;
        float cal_temp12_radome16;
        float cal_temp2_feed16;
        float cal_phase16;
        float cal_psi16;
        for (size_t k=0; k<2; k++) {
          cal_loss4_coupler16[k] =
            subCalGroup->cal_loss4_coupler16[hr_idx][l][k];
          cal_loss5_diplexer16[k] =
            subCalGroup->cal_loss5_diplexer16[hr_idx][l][k];
          cal_temp4_coupler16[k] =
            subCalGroup->cal_temp4_coupler16[hr_idx][l][k];
          cal_temp_ref16[k] = subCalGroup->cal_temp_ref16[hr_idx][l][k];
          cal_tcor16_re[k] = subCalGroup->cal_tcor16_re[hr_idx][l][k];
          cal_tcor16_im[k] = subCalGroup->cal_tcor16_im[hr_idx][l][k];
          cal_lambda16[k] = subCalGroup->cal_lambda16[hr_idx][l][k];
          cal_gamma16[k] = subCalGroup->cal_gamma16[hr_idx][l][k];
          cal_loss12_radome16[k] = subCalGroup->cal_loss12_radome16[i][k];
          cal_loss3_omt16[k] = subCalGroup->cal_loss3_omt16[hr_idx][l][k];
          cal_temp3_omt16[k] = subCalGroup->cal_temp3_omt16[hr_idx][l][k];
        } // k loop
        cal_loss2_feed16 = subCalGroup->cal_loss2_feed16[hr_idx][l];
        cal_temp12_radome16 = subCalGroup->cal_temp12_radome16[i];
        cal_temp2_feed16 = subCalGroup->cal_temp2_feed16[hr_idx][l];
        cal_phase16  = subCalGroup->cal_phase16[hr_idx][l];
        cal_psi16  = subCalGroup->cal_psi16[hr_idx][l];

        // TA_v
        
        // compute TA up to the RFE
        double TA16_RFEv = 
          (m2_16_antv[ant_idx] - Offsetv16[go_idx]) / Gainv16[go_idx];

        // include losses
        double TA16_CPv = 
          cal_loss4_coupler16[0] * cal_loss5_diplexer16[0] * TA16_RFEv -
          (cal_loss4_coupler16[0]-1) * cal_temp4_coupler16[0];

        // Equation 4.18 in ATBD
        double Tcor16_v_re =  -cal_temp_ref16[0] * cal_tcor16_re[0];
        double Tcor16_v_im =  -cal_temp_ref16[0] * cal_tcor16_im[0];

        // New equation, eq. 5.10 in ATBD, add gamma, tcor to IPF
        // these along with cal_lambda are complex values
        double TA16_CPvhat = TA16_CPv - 
          2 * (cal_lambda16[0] * cal_gamma16[0] * Tcor16_v_re) / 
          powf(cal_lambda16[0],2) - cal_temp_ref16[0] * powf(cal_gamma16[0],2);
        
        // Modified to include radome
        double TA16v = cal_loss12_radome16[0] * 
          cal_loss2_feed16 * cal_loss3_omt16[0] * TA16_CPvhat - 
          (cal_loss12_radome16[0]-1) * cal_temp12_radome16 - 
          cal_loss12_radome16[0] * (cal_loss2_feed16-1) * cal_temp2_feed16 - 
          cal_loss12_radome16[0] * cal_loss2_feed16 * 
          (cal_loss3_omt16[0]-1) * cal_temp3_omt16[0];

        TAs->TA16_v[go_idx] = (float) TA16v;
        TAs->Gainv16[go_idx] = (float) Gainv16[go_idx];
        TAs->Offsetv16[go_idx] = (float) Offsetv16[go_idx];

        // TA_h
        // compute TA up to the RFE
        double TA16_RFEh = 
          (m2_16_anth[ant_idx] - Offseth16[go_idx]) / Gainh16[go_idx];

        double TA16_CPh = 
          cal_loss4_coupler16[1] * cal_loss5_diplexer16[1] * TA16_RFEh -
          (cal_loss4_coupler16[1]-1) * cal_temp4_coupler16[1];

        double Tcor16_h_re =  -cal_temp_ref16[1] * cal_tcor16_re[1];
        double Tcor16_h_im =  -cal_temp_ref16[1] * cal_tcor16_im[1];

        double TA16_CPhhat = TA16_CPh - 
          2 * (cal_lambda16[1] * cal_gamma16[1] * Tcor16_h_re) / 
          powf(cal_lambda16[1],2) - cal_temp_ref16[1] * powf(cal_gamma16[1],2);
        
        double TA16h = cal_loss12_radome16[1] * 
          cal_loss2_feed16 * cal_loss3_omt16[1] * TA16_CPhhat - 
          (cal_loss12_radome16[1]-1) * cal_temp12_radome16 - 
          cal_loss12_radome16[1] * (cal_loss2_feed16-1) * cal_temp2_feed16 - 
          cal_loss12_radome16[1] * cal_loss2_feed16 * 
          (cal_loss3_omt16[1]-1) * cal_temp3_omt16[1];

        TAs->TA16_h[go_idx] = (float) TA16h;
        TAs->Gainh16[go_idx] = (float) Gainh16[go_idx];
        TAs->Offseth16[go_idx] = (float) Offseth16[go_idx];

        // T3T4
        double T3_RFE16 = 
          (t3_16_ant[ant_idx] * cos(cal_phase16) - 
           t4_16_ant[ant_idx] * sin(cal_phase16)) /
          Gain16[go_idx] - T3Offset16[go_idx];
        double T4_RFE16 = 
          (t3_16_ant[ant_idx] * sin(cal_phase16) + 
           t4_16_ant[ant_idx] * cos(cal_phase16)) /
          Gain16[go_idx] - T4Offset16[go_idx];

        // Added cal_loss12_radome16
        double T34_losses16 = 
        sqrt(cal_loss12_radome16[0] * cal_loss2_feed16 * cal_loss3_omt16[0] * 
             cal_loss4_coupler16[0] * cal_loss5_diplexer16[0] *
             cal_loss12_radome16[1] * cal_loss2_feed16 * cal_loss3_omt16[1] *
             cal_loss4_coupler16[1] * cal_loss5_diplexer16[1]) /
        abs(cal_lambda16[0] * powf(cal_lambda16[1],2.0));

        double T3_16_hat1 =  
           cos(cal_psi16) * cal_lambda16[0] * cal_lambda16[1] * T3_RFE16;
        double T3_16_hat2 = 
          -sin(cal_psi16) * cal_lambda16[0] * cal_lambda16[1] * T4_RFE16;
        double T3hat16 = T34_losses16 * (T3_16_hat1 + T3_16_hat2);
        TAs->T3_16[go_idx] = (float) T3hat16;

        double T4_16_hat1 = 
          sin(cal_psi16) * cal_lambda16[0] * cal_lambda16[1] * T3_RFE16;
        double T4_16_hat2 = 
          cos(cal_psi16) * cal_lambda16[0] * cal_lambda16[1] * T4_RFE16;
        double T4hat16 = T34_losses16 * (T4_16_hat1 + T4_16_hat2);
        TAs->T4_16[go_idx] = (float) T4hat16;

      } // l loop
    } // j loop
  } // i loop

  delete( Gainv16);
  delete( Gainh16);
  delete( Gain16);
  delete( Offsetv16);
  delete( Offseth16);
  delete( T3Offset16);
  delete( T4Offset16);
  delete( m2_16_antv);
  delete( m2_16_anth);
  delete( t3_16_ant);
  delete( t4_16_ant);

  return 0;
}


int l1aSMAPFile::MAXPD_Algorithm( TAs_struct *TAs, 
                                  rfiFlags_struct *RFIflagsAnt, 
                                  rfiFlagsKurtosis_struct *rfiFlagsKurt,
                                  uint32_t *highresolution_scan_index) {

  uint8_t *RFIflag_MPD_h = new uint8_t[nscans*(nPRI_ANT-last_PRIs)];
  uint8_t *RFIflag_MPD_v = new uint8_t[nscans*(nPRI_ANT-last_PRIs)];
  uint8_t *RFIflag16_MPD_h = new uint8_t[nhires*((nPRI_ANT/4)-last_PKTs)*16];
  uint8_t *RFIflag16_MPD_v = new uint8_t[nhires*((nPRI_ANT/4)-last_PKTs)*16];
  memset( RFIflag_MPD_h, 0, nscans*(nPRI_ANT-last_PRIs));
  memset( RFIflag_MPD_v, 0, nscans*(nPRI_ANT-last_PRIs));
  memset( RFIflag16_MPD_h, 0, nhires*((nPRI_ANT/4)-last_PKTs)*16);
  memset( RFIflag16_MPD_v, 0, nhires*((nPRI_ANT/4)-last_PKTs)*16);

  int k = 0;
  
  // Flag loop
  uint8_t iflagh, iflagv, sum;
  for (size_t i=0; i<nscans; i++) {

    // Check whether scan is high-res
    bool found = false;
    if ( highresolution_scan_index[k] == i) found = true;

    if ( found == true) {
      // Sub Band
      for (size_t j=0; j<((nPRI_ANT/4)-last_PKTs); j++) {

        uint32_t idx16 = k*((nPRI_ANT/4)-last_PKTs)*16+j*16;

        uint32_t idx_flag = i*((nPRI_ANT/4)-last_PKTs)+j;
        uint32_t idx_kurt = i*(nPRI_ANT/4)+j;

        uint32_t idx16_flag = k*((nPRI_ANT/4)-last_PKTs)+j;
        uint32_t idx16_kurt = k*(nPRI_ANT/4)+j;
 
        // H-pol
        iflagh = 0;

        sum = 0;
        for (size_t l=0; l<4; l++) 
          sum += RFIflagsAnt->RFIflag_h_td1[idx_flag*4+l];
        if ( sum > 0) iflagh = 1;

        sum = 0;
        for (size_t l=0; l<4; l++) 
          sum += rfiFlagsKurt->RFIflag_kurt_h[idx_kurt*4+l];
        if ( sum > 0) iflagh = 1;

        sum = 0;
        for (size_t l=0; l<4; l++) 
          sum += RFIflagsAnt->RFIflag_t3[idx_flag*4+l];
        if ( sum > 0) iflagh = 1;

        sum = 0;
        for (size_t l=0; l<4; l++) 
          sum += RFIflagsAnt->RFIflag_t4[idx_flag*4+l];
        if ( sum > 0) iflagh = 1;

        for (size_t l=0; l<16; l++) RFIflag16_MPD_h[idx16+l] = iflagh;
        if ( iflagh == 0) {
          for (size_t l=0; l<16; l++) {
            RFIflag16_MPD_h[idx16+l] = 
              RFIflagsAnt->RFIflag16_h_cf1[idx16_flag*16+l] |
              rfiFlagsKurt->RFIflag16_kurt_h[idx16_kurt*16+l] |
              RFIflagsAnt->RFIflag16_t3[idx16_flag*16+l] |
              RFIflagsAnt->RFIflag16_t4[idx16_flag*16+l] |
              RFIflagsAnt->RFIflag16_h_cf11[(idx16_flag/8)*16+l];
          } // l loop
        }

        // V-pol
        iflagv = 0;

        sum = 0;
        for (size_t l=0; l<4; l++) 
          sum += RFIflagsAnt->RFIflag_v_td1[idx_flag*4+l];
        if ( sum > 0) iflagv = 1;

        sum = 0;
        for (size_t l=0; l<4; l++) 
          sum += rfiFlagsKurt->RFIflag_kurt_v[idx_kurt*4+l];
        if ( sum > 0) iflagv = 1;

        sum = 0;
        for (size_t l=0; l<4; l++) 
          sum += RFIflagsAnt->RFIflag_t3[idx_flag*4+l];
        if ( sum > 0) iflagv = 1;

        sum = 0;
        for (size_t l=0; l<4; l++) 
          sum += RFIflagsAnt->RFIflag_t4[idx_flag*4+l];
        if ( sum > 0) iflagv = 1;

        for (size_t l=0; l<16; l++) RFIflag16_MPD_v[idx16+l] = iflagv;
        if ( iflagv == 0) {
          for (size_t l=0; l<16; l++) {
            RFIflag16_MPD_v[idx16+l] = 
              RFIflagsAnt->RFIflag16_v_cf1[idx16_flag*16+l] |
              rfiFlagsKurt->RFIflag16_kurt_v[idx16_kurt*16+l] |
              RFIflagsAnt->RFIflag16_t3[idx16_flag*16+l] |
              RFIflagsAnt->RFIflag16_t4[idx16_flag*16+l] |
              RFIflagsAnt->RFIflag16_v_cf11[(idx16_flag/8)*16+l];
          } // l loop
        }
        
      } // j loop

      k++;  // Increment high res scan index
      
    } else {
      //      cout << "subband not found" << endl;
      // Full Band
      for (size_t j=0; j<(nPRI_ANT-last_PRIs); j++) {
        uint32_t idx_flag = i*(nPRI_ANT-last_PRIs)+j;
        uint32_t idx_kurt = i*(nPRI_ANT)+j;
        RFIflag_MPD_h[idx_flag] =
             RFIflagsAnt->RFIflag_h_td1[idx_flag] |
             rfiFlagsKurt->RFIflag_kurt_h[idx_kurt] |
             RFIflagsAnt->RFIflag_t3[idx_flag] |
             RFIflagsAnt->RFIflag_t4[idx_flag];

        RFIflag_MPD_v[idx_flag] =
          RFIflagsAnt->RFIflag_v_td1[idx_flag] |
          rfiFlagsKurt->RFIflag_kurt_v[idx_kurt] |
          RFIflagsAnt->RFIflag_t3[idx_flag] |
          RFIflagsAnt->RFIflag_t4[idx_flag];
      }
    }
  } // i loop


  // TA loop
  double mean;
  double mean_clean;
  double nedt;
  uint32_t clean_cnt = 0;
  uint32_t n_filtered = ((nPRI_ANT/4)-last_PKTs)/8;

  TAs->ta_v = new float[this->nscans*n_filtered];
  TAs->ta_h = new float[this->nscans*n_filtered];
  TAs->ta_3 = new float[this->nscans*n_filtered];
  TAs->ta_4 = new float[this->nscans*n_filtered];

  TAs->ta_filtered_v = new float[this->nscans*n_filtered];
  TAs->ta_filtered_h = new float[this->nscans*n_filtered];
  TAs->ta_filtered_3 = new float[this->nscans*n_filtered];
  TAs->ta_filtered_4 = new float[this->nscans*n_filtered];

  TAs->nedt_v = new float[this->nscans*n_filtered];
  TAs->nedt_h = new float[this->nscans*n_filtered];
  TAs->nedt_3 = new float[this->nscans*n_filtered];
  TAs->nedt_4 = new float[this->nscans*n_filtered];

  for (size_t i=0; i<nscans*n_filtered; i++) {
    TAs->ta_v[i] = -9999.0;
    TAs->ta_h[i] = -9999.0;
    TAs->ta_3[i] = -9999.0;
    TAs->ta_4[i] = -9999.0;
    TAs->ta_filtered_v[i] = -9999.0;
    TAs->ta_filtered_h[i] = -9999.0;
    TAs->ta_filtered_3[i] = -9999.0;
    TAs->ta_filtered_4[i] = -9999.0;
    TAs->nedt_v[i] = -9999.0;
    TAs->nedt_h[i] = -9999.0;
    TAs->nedt_3[i] = -9999.0;
    TAs->nedt_4[i] = -9999.0;
  }
  
  k = 0;
  
  for (size_t i=0; i<nscans; i++) {

     // Check whether scan is high-res
    bool found = false;
    if ( highresolution_scan_index[k] == i) found = true;

    if ( found == true) {
      
      uint32_t idx16 = k*((nPRI_ANT/4)-last_PKTs)*16;

      for (size_t j=0; j<n_filtered; j++) {
        // Sub Band Data Used
        
        // H-pol
        mean = 0.0;
        mean_clean = 0.0;
        nedt = 0.0;
        clean_cnt = 0;
        for (size_t l=0; l<8*16; l++) {
          uint32_t idxl = idx16+j*8*16+l;
          mean += TAs->TA16_h[idxl];
  
          if ( RFIflag16_MPD_h[idxl] == 0) {
            clean_cnt++;
            mean_clean += TAs->TA16_h[idxl];
            nedt += TAs->TA16_h[idxl]*TAs->TA16_h[idxl];
          }
        } // l loop

        mean /= 8*16;
        mean_clean /= clean_cnt;
        nedt = sqrt((nedt - clean_cnt*mean_clean*mean_clean) / (clean_cnt-1));
        nedt /= sqrt(clean_cnt);
      
        TAs->ta_h[i*n_filtered+j] = mean;
        TAs->ta_filtered_h[i*n_filtered+j] = mean_clean;
        TAs->nedt_h[i*n_filtered+j] = nedt;
      
        // V-pol
        mean = 0.0;
        mean_clean = 0.0;
        nedt = 0.0;
        clean_cnt = 0;
        for (size_t l=0; l<8*16; l++) {
          uint32_t idxl = idx16+j*8*16+l;
          mean += TAs->TA16_v[idxl];
    
          if ( RFIflag16_MPD_v[idxl] == 0) {
            clean_cnt++;
            mean_clean += TAs->TA16_v[idxl];
            nedt += TAs->TA16_v[idxl]*TAs->TA16_v[idxl];
          }
        } // l loop

        mean /= 8*16;
        mean_clean /= clean_cnt;
        nedt = sqrt((nedt - clean_cnt*mean_clean*mean_clean) / (clean_cnt-1));
        nedt /= sqrt(clean_cnt);
      
        TAs->ta_v[i*n_filtered+j] = mean;
        TAs->ta_filtered_v[i*n_filtered+j] = mean_clean;
        TAs->nedt_v[i*n_filtered+j] = nedt;

        // T3
        mean = 0.0;
        mean_clean = 0.0;
        nedt = 0.0;
        clean_cnt = 0;
        for (size_t l=0; l<8*16; l++) {
          uint32_t idxl = idx16+j*8*16+l;
          mean += TAs->T3_16[idxl];
    
          if ( RFIflag16_MPD_h[idxl] == 0 && RFIflag16_MPD_v[idxl] == 0) {
            clean_cnt++;
            mean_clean += TAs->T3_16[idxl];
            nedt += TAs->T3_16[idxl]*TAs->T3_16[idxl];
          }
        } // l loop

        mean /= 8*16;
        mean_clean /= clean_cnt;
        nedt = sqrt((nedt - clean_cnt*mean_clean*mean_clean) / (clean_cnt-1));
        nedt /= sqrt(clean_cnt);
      
        TAs->ta_3[i*n_filtered+j] = mean;
        TAs->ta_filtered_3[i*n_filtered+j] = mean_clean;
        TAs->nedt_3[i*n_filtered+j] = nedt;

        // T4
        mean = 0.0;
        mean_clean = 0.0;
        nedt = 0.0;
        clean_cnt = 0;
        for (size_t l=0; l<8*16; l++) {
          uint32_t idxl = idx16+j*8*16+l;
          mean += TAs->T4_16[idxl];
    
          if ( RFIflag16_MPD_h[idxl] == 0 && RFIflag16_MPD_v[idxl] == 0) {
            clean_cnt++;
            mean_clean += TAs->T4_16[idxl];
            nedt += TAs->T4_16[idxl]*TAs->T4_16[idxl];
          }
        } // l loop

        mean /= 8*16;
        mean_clean /= clean_cnt;
        nedt = sqrt((nedt - clean_cnt*mean_clean*mean_clean) / (clean_cnt-1));
        nedt /= sqrt(clean_cnt);

        TAs->ta_4[i*n_filtered+j] = mean;
        TAs->ta_filtered_4[i*n_filtered+j] = mean_clean;
        TAs->nedt_4[i*n_filtered+j] = nedt;
      
      } // j loop

      k++;  // Increment high res scan index
            
    } else {

      // Full Band Data Used
      for (size_t j=0; j<n_filtered; j++) {

        uint32_t idx = i*(nPRI_ANT-last_PRIs);
        
        // H-pol
        mean = 0.0;
        mean_clean = 0.0;
        nedt = 0.0;
        clean_cnt = 0;
        for (size_t l=0; l<32; l++) {
          uint32_t idxl = idx+j*32+l;
          mean += TAs->TA_h[idxl];
    
          if ( RFIflag_MPD_h[idxl] == 0) {
            clean_cnt++;
            mean_clean += TAs->TA_h[idxl];
            nedt += TAs->TA_h[idxl]*TAs->TA_h[idxl];
          }
        } // l loop

        mean /= 32;
        mean_clean /= clean_cnt;
        nedt = sqrt((nedt - clean_cnt*mean_clean*mean_clean) / (clean_cnt-1));
        nedt /= sqrt(clean_cnt);

        TAs->ta_h[i*n_filtered+j] = mean;
        TAs->ta_filtered_h[i*n_filtered+j] = mean_clean;
        TAs->nedt_h[i*n_filtered+j] = nedt;

        // V-pol
        mean = 0.0;
        mean_clean = 0.0;
        nedt = 0.0;
        clean_cnt = 0;
        for (size_t l=0; l<32; l++) {
          uint32_t idxl = idx+j*32+l;
          mean += TAs->TA_v[idxl];
    
          if ( RFIflag_MPD_v[idxl] == 0) {
            clean_cnt++;
            mean_clean += TAs->TA_v[idxl];
            nedt += TAs->TA_v[idxl]*TAs->TA_v[idxl];
          }
        } // l loop

        mean /= 32;
        mean_clean /= clean_cnt;
        nedt = sqrt((nedt - clean_cnt*mean_clean*mean_clean) / (clean_cnt-1));
        nedt /= sqrt(clean_cnt);

        TAs->ta_v[i*n_filtered+j] = mean;
        TAs->ta_filtered_v[i*n_filtered+j] = mean_clean;
        TAs->nedt_v[i*n_filtered+j] = nedt;
        
        // T3
        mean = 0.0;
        mean_clean = 0.0;
        nedt = 0.0;
        clean_cnt = 0;
        for (size_t l=0; l<32; l++) {
          uint32_t idxl = idx+j*32+l;
          mean += TAs->T3[idxl];
    
          if ( RFIflag_MPD_h[idxl] == 0 && RFIflag_MPD_v[idxl] == 0) {
            clean_cnt++;
            mean_clean += TAs->T3[idxl];
            nedt += TAs->T3[idxl]*TAs->T3[idxl];
          }
        } // l loop

        mean /= 32;
        mean_clean /= clean_cnt;
        nedt = sqrt((nedt - clean_cnt*mean_clean*mean_clean) / (clean_cnt-1));
        nedt /= sqrt(clean_cnt);

        TAs->ta_3[i*n_filtered+j] = mean;
        TAs->ta_filtered_3[i*n_filtered+j] = mean_clean;
        TAs->nedt_3[i*n_filtered+j] = nedt;
        
        // T4
        mean = 0.0;
        mean_clean = 0.0;
        nedt = 0.0;
        clean_cnt = 0;
        for (size_t l=0; l<32; l++) {
          uint32_t idxl = idx+j*32+l;
          mean += TAs->T4[idxl];
    
          if ( RFIflag_MPD_h[idxl] == 0 && RFIflag_MPD_v[idxl] == 0) {
            clean_cnt++;
            mean_clean += TAs->T4[idxl];
            nedt += TAs->T4[idxl]*TAs->T4[idxl];
          }
        } // l loop

        mean /= 32;
        mean_clean /= clean_cnt;
        nedt = sqrt((nedt - clean_cnt*mean_clean*mean_clean) / (clean_cnt-1));
        nedt /= sqrt(clean_cnt);

        TAs->ta_4[i*n_filtered+j] = mean;
        TAs->ta_filtered_4[i*n_filtered+j] = mean_clean;
        TAs->nedt_4[i*n_filtered+j] = nedt;
      }
    }
  } // i loop


  ofstream tempOut;
  tempOut.open ("flagMPD_h.bin", ios::out | ios::trunc | ios::binary);
  //  tempOut.write((char *) RFIflag_MPD_h,
  //            nscans*(nPRI_ANT-last_PRIs)*sizeof(float));
  tempOut.write((char *) RFIflag16_MPD_h,
                nscans*((nPRI_ANT/4)-last_PKTs)*16*sizeof(float));
  tempOut.close();
  
  delete( RFIflag_MPD_h);
  delete( RFIflag_MPD_v);
  delete( RFIflag16_MPD_h);
  delete( RFIflag16_MPD_v);
  
  return 0;
}

int l1aSMAPFile:: L1B_bit_flags( float *tah, float *tah_filtered, float *nedt_h,
                                 float *tav, float *tav_filtered, float *nedt_v,
                                 float *ta3, float *ta3_filtered, float *nedt_3,
                                 float *ta4, float *ta4_filtered, float *nedt_4,
                                 uint16_t *tb_qual_flag_h,
                                 uint16_t *tb_qual_flag_v,
                                 uint16_t *tb_qual_flag_3,
                                 uint16_t *tb_qual_flag_4) {

  float threshold_bit2 = 0.2;
  float threshold_bit14 = 100.0;
  float NEDTthreshold = 1.5;

  // bit 3
  for (size_t j=0; j<nscans*(nPRI_ANT/32); j++) {
    if ( tah[j] > -9990 && tah_filtered[j] < -9990)
      tb_qual_flag_h[j] |= 1 << 3;
    if ( tav[j] > -9990 && tav_filtered[j] < -9990)
      tb_qual_flag_v[j] |= 1 << 3;
    if ( ta3[j] > -9990 && ta3_filtered[j] < -9990)
      tb_qual_flag_3[j] |= 1 << 3;
    if ( ta4[j] > -9990 && ta4_filtered[j] < -9990)
      tb_qual_flag_4[j] |= 1 << 3;
  }


  // bit 2
  for (size_t j=0; j<nscans*(nPRI_ANT/32); j++) {
    if ( (tb_qual_flag_h[j] & (1 << 3)) != 0)
      tb_qual_flag_h[j] |= 1 << 2;

    if ( (tah[j] > -9990 && tah_filtered[j] > -9990) &&
         (tah[j] - tah_filtered[j] > threshold_bit2))
      tb_qual_flag_h[j] |= 1 << 2;

    
    if ( (tb_qual_flag_v[j] & (1 << 3)) != 0)
      tb_qual_flag_v[j] |= 1 << 2;

    if ( (tav[j] > -9990 && tav_filtered[j] > -9990) &&
         (tav[j] - tav_filtered[j] > threshold_bit2))
      tb_qual_flag_v[j] |= 1 << 2;


    if ( (tb_qual_flag_3[j] & (1 << 3)) != 0)
      tb_qual_flag_3[j] |= 1 << 2;

    if ( (ta3[j] > -9990 && ta3_filtered[j] > -9990) &&
         (ta3[j] - ta3_filtered[j] > threshold_bit2))
      tb_qual_flag_3[j] |= 1 << 2;

         
    if ( (tb_qual_flag_4[j] & (1 << 3)) != 0)
      tb_qual_flag_4[j] |= 1 << 2;

         if ( (ta4[j] > -9990 && ta4_filtered[j] > -9990) &&
         (ta4[j] - ta4_filtered[j] > threshold_bit2))
      tb_qual_flag_4[j] |= 1 << 2;
  }

  
  // bit 14
  for (size_t j=0; j<nscans*(nPRI_ANT/32); j++) {
    if ( tah[j] > -9990 && tah_filtered[j] > -9990 &&
         tah[j] - tah_filtered[j] > threshold_bit14)
      tb_qual_flag_h[j] |= 1 << 14;

    if ( tav[j] > -9990 && tav_filtered[j] > -9990 &&
         tav[j] - tav_filtered[j] > threshold_bit14)
      tb_qual_flag_v[j] |= 1 << 14;

    if ( ta3[j] > -9990 && ta3_filtered[j] > -9990 &&
         ta3[j] - ta3_filtered[j] > threshold_bit14)
      tb_qual_flag_3[j] |= 1 << 14;

    if ( ta4[j] > -9990 && ta4_filtered[j] > -9990 &&
         ta4[j] - ta4_filtered[j] > threshold_bit14)
      tb_qual_flag_4[j] |= 1 << 14;
  }

  
  // bit 4
  for (size_t j=0; j<nscans*(nPRI_ANT/32); j++) {
    if ( nedt_h[j] > NEDTthreshold)
      tb_qual_flag_h[j] |= 1 << 4;

    if ( nedt_v[j] > NEDTthreshold)
      tb_qual_flag_v[j] |= 1 << 4;

    if ( nedt_3[j] > NEDTthreshold)
      tb_qual_flag_3[j] |= 1 << 4;

    if ( nedt_4[j] > NEDTthreshold)
      tb_qual_flag_4[j] |= 1 << 4;
  }

  return 0;
}

int l1aSMAPFile::closel1( TAs_struct *TAs, double cal_loss1_refl[MAXSCANS][2],
                          double cal_temp1_refl[MAXSCANS][2]) {

  float *tah = new float[nscans*(nPRI_ANT/32)];
  float *tah_filtered = new float[nscans*(nPRI_ANT/32)];
  float *nedt_h = new float[nscans*(nPRI_ANT/32)];
  
  float *tav = new float[nscans*(nPRI_ANT/32)];
  float *tav_filtered = new float[nscans*(nPRI_ANT/32)];
  float *nedt_v = new float[nscans*(nPRI_ANT/32)];

  float *ta3 = new float[nscans*(nPRI_ANT/32)];
  float *ta3_filtered = new float[nscans*(nPRI_ANT/32)];
  float *nedt_3 = new float[nscans*(nPRI_ANT/32)];
  
  float *ta4 = new float[nscans*(nPRI_ANT/32)];
  float *ta4_filtered = new float[nscans*(nPRI_ANT/32)];
  float *nedt_4 = new float[nscans*(nPRI_ANT/32)];
  
  for (size_t j=0; j<nscans*(nPRI_ANT/32); j++) {
    tah[j] = -9999.0;
    tav[j] = -9999.0;
    ta3[j] = -9999.0;
    ta4[j] = -9999.0;
    tah_filtered[j] = -9999.0;
    tav_filtered[j] = -9999.0;
    ta3_filtered[j] = -9999.0;
    ta4_filtered[j] = -9999.0;
    nedt_h[j] = -9999.0;
    nedt_v[j] = -9999.0;
    nedt_3[j] = -9999.0;
    nedt_4[j] = -9999.0;
  }
    
  gid_ta = H5Gopen1( h5fid_ta, "/Brightness_Temperature");

  hsize_t dims[2];
  dims[0] = nscans;
  dims[1] = nPRI_ANT/32;

  uint32_t n_filtered = ((nPRI_ANT/4)-last_PKTs)/8;
  for (size_t i=0; i<nscans; i++) {
    for (size_t j=0; j<n_filtered; j++) {
      uint32_t ta_idx = i*(nPRI_ANT/32)+j;
      //      uint32_t TAs_idx = i*n_filtered*3+3*j;
      uint32_t TAs_idx = i*n_filtered+j;
        
      tah[ta_idx]          = TAs->ta_h[TAs_idx];
      tah_filtered[ta_idx] = TAs->ta_filtered_h[TAs_idx];
      nedt_h[ta_idx]       = TAs->nedt_h[TAs_idx];

      tav[ta_idx]          = TAs->ta_v[TAs_idx];
      tav_filtered[ta_idx] = TAs->ta_filtered_v[TAs_idx];
      nedt_v[ta_idx]       = TAs->nedt_v[TAs_idx];

      ta3[ta_idx]          = TAs->ta_3[TAs_idx];
      ta3_filtered[ta_idx] = TAs->ta_filtered_3[TAs_idx];
      nedt_3[ta_idx]       = TAs->nedt_3[TAs_idx];

      ta4[ta_idx]          = TAs->ta_4[TAs_idx];
      ta4_filtered[ta_idx] = TAs->ta_filtered_4[TAs_idx];
      nedt_4[ta_idx]       = TAs->nedt_4[TAs_idx];  
    }
  }

  H5LTmake_dataset( gid_ta, "ta_h", 2, dims, H5T_NATIVE_FLOAT, tah);
  H5LTmake_dataset( gid_ta, "ta_filtered_h", 2, dims, H5T_NATIVE_FLOAT, 
                    tah_filtered);
  H5LTmake_dataset( gid_ta, "nedt_h", 2, dims, H5T_NATIVE_FLOAT, nedt_h);
  
  H5LTmake_dataset( gid_ta, "ta_v", 2, dims, H5T_NATIVE_FLOAT, tav);
  H5LTmake_dataset( gid_ta, "ta_filtered_v", 2, dims, H5T_NATIVE_FLOAT, 
                    tav_filtered);
  H5LTmake_dataset( gid_ta, "nedt_v", 2, dims, H5T_NATIVE_FLOAT, nedt_v);
  
  H5LTmake_dataset( gid_ta, "ta_3", 2, dims, H5T_NATIVE_FLOAT, ta3);
  H5LTmake_dataset( gid_ta, "ta_filtered_3", 2, dims, H5T_NATIVE_FLOAT, 
                    ta3_filtered);
  H5LTmake_dataset( gid_ta, "nedt_3", 2, dims, H5T_NATIVE_FLOAT, nedt_3);
  
  H5LTmake_dataset( gid_ta, "ta_4", 2, dims, H5T_NATIVE_FLOAT, ta4);
  H5LTmake_dataset( gid_ta, "ta_filtered_4", 2, dims, H5T_NATIVE_FLOAT, 
                    ta4_filtered);
  H5LTmake_dataset( gid_ta, "nedt_4", 2, dims, H5T_NATIVE_FLOAT, nedt_4);
  
  // Compute bit flags
  uint16_t *tb_qual_flag_h = new uint16_t[nscans*(nPRI_ANT/32)];
  uint16_t *tb_qual_flag_v = new uint16_t[nscans*(nPRI_ANT/32)];
  uint16_t *tb_qual_flag_3 = new uint16_t[nscans*(nPRI_ANT/32)];
  uint16_t *tb_qual_flag_4 = new uint16_t[nscans*(nPRI_ANT/32)];
  for (size_t j=0; j<nscans*(nPRI_ANT/32); j++) {
    tb_qual_flag_h[j] = 0;
    tb_qual_flag_v[j] = 0;
    tb_qual_flag_3[j] = 0;
    tb_qual_flag_4[j] = 0;
  }
 
  L1B_bit_flags( tah, tah_filtered, nedt_h, tav, tav_filtered, nedt_v,
                 ta3, ta3_filtered, nedt_3, ta4, ta4_filtered, nedt_4,
                 tb_qual_flag_h, tb_qual_flag_v,
                 tb_qual_flag_3, tb_qual_flag_4);

  H5LTmake_dataset( gid_ta, "tb_qual_flag_h", 2, dims, H5T_NATIVE_USHORT,
                    tb_qual_flag_h);

  H5LTmake_dataset( gid_ta, "tb_qual_flag_v", 2, dims, H5T_NATIVE_USHORT,
                    tb_qual_flag_v);

  H5LTmake_dataset( gid_ta, "tb_qual_flag_3", 2, dims, H5T_NATIVE_USHORT,
                    tb_qual_flag_3);

  H5LTmake_dataset( gid_ta, "tb_qual_flag_4", 2, dims, H5T_NATIVE_USHORT,
                    tb_qual_flag_4);

  H5Gclose( gid_ta);

  gid_ta = H5Gcreate1(h5fid_ta, "/Calibration_Data", 0);
  
  dims[1] = 2;
  H5LTmake_dataset( gid_ta, "cal_loss1_reflector", 2, dims, H5T_NATIVE_DOUBLE, 
                    cal_loss1_refl);
  H5LTmake_dataset( gid_ta, "cal_temp1_reflector", 2, dims, H5T_NATIVE_DOUBLE, 
                    cal_temp1_refl);

  H5Gclose( gid_ta);
  H5Fclose( h5fid_ta);

  delete( tah);
  delete( tah_filtered);
  delete( nedt_h);
  delete( tav);
  delete( tav_filtered);
  delete( nedt_v);
  delete( ta3);
  delete( ta3_filtered);
  delete( nedt_3);
  delete( ta4);
  delete( ta4_filtered);
  delete( nedt_4);

  delete( TAs->ta_h);
  delete( TAs->ta_filtered_h);
  delete( TAs->nedt_h);
  delete( TAs->ta_v);
  delete( TAs->ta_filtered_v);
  delete( TAs->nedt_v);
  delete( TAs->ta_3);
  delete( TAs->ta_filtered_3);
  delete( TAs->nedt_3);
  delete( TAs->ta_4);
  delete( TAs->ta_filtered_4);
  delete( TAs->nedt_4);
  
  delete( tb_qual_flag_h);
  delete( tb_qual_flag_v);
  delete( tb_qual_flag_3);
  delete( tb_qual_flag_4);
  
  return 0;
}

void readShort( hid_t fid, const char* field, short* data) {

  herr_t status = H5LTread_dataset_short( fid, field, data);
  if (status == -1) {
    cout << "Error reading " << field << endl;
  }
}

void readU16( hid_t fid, const char* field, uint16_t* data) {

  herr_t status = H5LTread_dataset( fid, field, H5T_STD_U16LE, 
                                    (uint32_t *) data);
  if (status == -1) {
    cout << "Error reading " << field << endl;
  }
}

void readU32( hid_t fid, const char* field, uint32_t* data) {

  herr_t status = H5LTread_dataset( fid, field, H5T_STD_U32LE, 
                                    (uint32_t *) data);
  if (status == -1) {
    cout << "Error reading " << field << endl;
  }
}

void readFloat( hid_t fid, const char* field, float* data) {

  herr_t status = H5LTread_dataset_float( fid, field, data);
  if (status == -1) {
    cout << "Error reading " << field << endl;
  }
}

void readDouble( hid_t fid, const char* field, double* data) {

  herr_t status = H5LTread_dataset_double( fid, field, data);
  if (status == -1) {
    cout << "Error reading " << field << endl;
  }
}


int replaceNaN( double *refv, double *refh, double *ref_ndv, double *ref_ndh,
                uint32_t npix, uint nelem) {

  // Replace NaN with nearest neighbor
  double *dptr[4] = {refv, refh, ref_ndv, ref_ndh};
  for (size_t k=0; k<4; k++) {
    for (size_t j=0; j<npix; j++) {
      for (size_t l=0; l<nelem; l++) {
        if ( isnan(dptr[k][j*nelem+l])) {
          int m=1;
          while (1) {
            double left  = dptr[k][(j-m)*nelem+l];
            double right = dptr[k][(j+m)*nelem+l];
            if (j == 0) {
              if ( !isnan(right)) {
                dptr[k][j*16+l] = right;
                break;
              } 
            } else if (j == (npix-1)) {
              if ( !isnan(left)) {
                dptr[k][j*nelem+l] = left;
                break;
              }
            } else {
              if ( !isnan(right)) {
                dptr[k][j*nelem+l] = right;
                break;
              }
              if ( !isnan(left)) {
                dptr[k][j*nelem+l] = left;
                break;
              }
            }
            m++;
            if (k > 5) {
              cout << "Can't find replacement for NaN" << endl;
              exit(1);
            }
          } // while loop
        } // nan found
      } // l loop
    } // j loop
  } // k loop

  return 0;
}


/* Copyright (c) 2009 the authors listed at the following URL, and/or
the authors of referenced articles or incorporated external code:
http://en.literateprograms.org/Insertion_sort_(C)?action=history&offset=20081205204844

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Retrieved from: http://en.literateprograms.org/Insertion_sort_(C)?oldid=15530
*/

/* Sort an array of integers */
void insertion_sort(float a[], int length)
{
  int i;
  for (i=0; i < length; i++)
  {
      /* Insert a[i] into the sorted sublist */
    int j;
    float v = a[i];

      for (j = i - 1; j >= 0; j--)
      {
          if (a[j] <= v) break;
          a[j + 1] = a[j];
      }
      a[j + 1] = v;

  }
}


